#include "bend.h"
















#define I_INPUT "data/sp1/angle_bin/1VA2.4I.first_4.txt"
#define V_INPUT "data/sp1/angle_bin/1VA2.4V.first_4.txt"
#define M_INPUT "data/sp1/angle_bin/1VA2.NoSubs.first_4.txt"

#define OFFSET_REGION { 11, 16 }
#define SUSTAINED_REGION { 22, 28 }
#define MY_REGION { 17, 28 }
#define START_STOP_BUFFER 2

#define OUTPUT_FNAME "data/sp1/angle_bin/bend.txt"















void do_bend( void )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( OUTPUT_FNAME, "" );
  
  do_bend_w_fh( I_INPUT, V_INPUT, "4I", "4V", out );
  do_bend_w_fh( M_INPUT, V_INPUT, "NoSubs", "4V", out );
  do_bend_w_fh( M_INPUT, I_INPUT, "NoSubs", "4I", out );
  
  close_file( out );

}













void do_bend_w_fh( char a_fname[], char b_fname[], char a_label[], char b_label[],
  FILE *out )
{

  fprintf( out, "%s\n", a_label );
  fprintf( out, "%s\n", b_label );
  fprintf( out, "\n" );
  
  do_my_bend_w_fh( a_fname, b_fname, out );
  
  fprintf( out, "\n\n" );
  
  print_sep( out );
  
  fprintf( out, "\n\n" );

}












void do_my_bend_w_fh( char a_fname[], char b_fname[], FILE *out )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  
  MOL_LIST *a = NULL, *b = NULL;  
  /*  In ../parse/mol_list.c:  */
  a = (MOL_LIST *)get_mol_list_w_fname( a, a_fname );
  b = (MOL_LIST *)get_mol_list_w_fname( b, b_fname );
  
  /*  In this file, below:  */
  do_bend_mol_list_pair( a, b, out );
  
  kill_mol_list( a );
  kill_mol_list( b );

}













void do_bend_mol_list_pair( MOL_LIST *a, MOL_LIST *b, FILE *out )
{

  int offset_reg[ START_STOP_BUFFER ] = OFFSET_REGION;
  int sus_reg[ START_STOP_BUFFER ] = SUSTAINED_REGION;
  int my_reg[ START_STOP_BUFFER ] = MY_REGION;
  
  do_bend_single_region( a, b, offset_reg, "Offset Region", out );
  do_bend_single_region( a, b, sus_reg, "Sustained Region", out );
  do_bend_single_region( a, b, my_reg, "My Region", out );

}










void do_bend_single_region( MOL_LIST *a, MOL_LIST *b, int reg[], char label[], 
  FILE *out )
{

  XYZ_CELL *create_xyz_cell( XYZ_CELL *xx, int nn );
  
  XYZ_CELL *a_cell = NULL, *b_cell = NULL;
  a_cell = (XYZ_CELL *)create_xyz_cell( a_cell, reg[ 1 ] - reg[ 0 ] + 1 );
  b_cell = (XYZ_CELL *)create_xyz_cell( b_cell, reg[ 1 ] - reg[ 0 ] + 1 );
  
  do_bend_single_region_w_cells( a, b, a_cell, b_cell, reg, label, out );
  
  kill_xyz_cell( a_cell );
  kill_xyz_cell( b_cell );

}











void do_bend_single_region_w_cells( MOL_LIST *a, MOL_LIST *b, XYZ_CELL *a_cell,
  XYZ_CELL *b_cell, int reg[], char label[], FILE *out )
{

  STAT get_rmsd_xyz_cell_w_params( ROT_PARAMS *my_params, XYZ_CELL *major_cell,
    XYZ_CELL *minor_cell );
    
  ROT_PARAMS my_params;

  /*  In comp.c:  */
  set_xyz_cell_carbon_alpha_atoms_index( a_cell, a, reg[ 0 ] - 1 );
  set_xyz_cell_carbon_alpha_atoms_index( b_cell, b, reg[ 0 ] - 1 );
  
  /*  In ../peptide/rmsd.c:  */
  STAT rmsd = get_rmsd_xyz_cell_w_params( &my_params, a_cell, b_cell );
  
  printf( "\t\tRMSD :  %.8f\n", rmsd );
  
  /*  In rmsd.c:  */
  rotate_mol_lister_w_rot_params( b, &my_params );
  
  /*  In comp.c:  */
  print_mol_compare_seq( a, b, out );
  
  fprintf( out, "\n\n" );
  
  fprintf( out, "%s\n", label );
  fprintf( out, "Region Aligned :  %d-%d\n\n", reg[ 0 ], reg[ 1 ] );
  
  /*  In comp.c:  */
  print_compare_rotated_mol_lister_dist( a, b, out );
  
  fflush( out );

}





















