#include "bin_change.h"










#define INPUT_FNAME "data/sp1/1VA2.txt"

#define OUTPUT_FNAME_BASE "data/sp1/bin_sp1/1VA2.txt.sub"


#define AA_NUMB { 4, 6, 8, 9 }
#define AA_TYPE { 'V', 'S', 'V', 'F' }
#define AA_SUBS { 0, 0, 0, 0 }
#define CHANGED_BUFFER 4

#define INIT_ANGLE 2.0
#define ANGLE_THRESH 0.125
#define ANGLE_STEP 0.7

#define FIRST_SITE 3
//#define FIRST_SITE 1
#define FIRST_FIRST_RESID 4

#define ANGLE_BUFFER 2
#define BIG 100000000.0




#define PHI_PSI_WEIGHT 0.556




#define RADIANS_TO_DEGREES(a) (180.0 * (a) / PI)
























void do_evol_change_bin( void )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  
  MOL_LIST *mol = NULL;  
  /*  In ../parse/mol_list.c:  */
  mol = (MOL_LIST *)get_mol_list_w_fname( mol, INPUT_FNAME ); 
  
  int aa_numb[ CHANGED_BUFFER ] = AA_NUMB;
  char aa_type[ CHANGED_BUFFER ] = AA_TYPE;
  int aa_subs[ CHANGED_BUFFER ] = AA_SUBS;
  
  /*  In this file, below:  */
  do_my_evol_change_bin( mol, aa_numb, aa_type, aa_subs, INPUT_FNAME );
  
}















void do_my_evol_change_bin( MOL_LIST *mol, int aa_numb[], char aa_type[],
  int aa_subs[], char input_fname[] )
{

  MOL_LIST *copy_mol_lister( MOL_LIST *major, MOL_LIST *minor );
  
  MOL_LIST *init_mol = NULL;

  /*  In ../mm/mol.c:  */
  init_mol = (MOL_LIST *)copy_mol_lister( init_mol, mol );
  
  /*  In this file, below:  */
  do_evol_change_bin_sub_mol_list( mol, aa_numb, aa_type, aa_subs );

  /*  In ../peptide/phi_psi.c:  */
  set_phi_psi_angles_w_mol_list( init_mol );
  set_phi_psi_angles_w_mol_list( mol );
  
  /*  In this file, below:  */
  output_changed_mol( mol, init_mol, OUTPUT_FNAME_BASE, aa_numb, aa_type, aa_subs, 
    FIRST_SITE, FIRST_FIRST_RESID );

  /*  In ../peptide/peptide.c:  */
  print_two_mol_lists_diff_phi_psi( mol, init_mol );
  
}
















void do_evol_change_bin_sub_mol_list( MOL_LIST *mol, int aa_numb[], char aa_type[],
  int aa_subs[] )
{

  /*  In change.c:  */
  do_evol_change_point_substitutions( mol, aa_numb, aa_type, aa_subs );
  
  /*  In this file, below:  */
  minimize_free_energy_evol_point_subs_bin( mol );

}
















void minimize_free_energy_evol_point_subs_bin( MOL_LIST *mol )
{

  int changed_array[ SHORT_BUFFER ];
  changed_array[ 0 ] = 0;

  STAT angle = INIT_ANGLE;
  
  while ( angle > ANGLE_THRESH )
    minimize_free_energy_single_angle_iteration_bin( mol, changed_array, &angle );
    
}













void minimize_free_energy_single_angle_iteration_bin( MOL_LIST *mol,
  int changed_array[], STAT *angle )
{

  RTYPE_CELL *make_and_set_rtype_cell_bin( RTYPE_CELL *my_rtype );
  TYPE_CELL *make_and_set_type_cell( TYPE_CELL *my_type );
  
  TYPE_CELL *my_type = NULL;
  RTYPE_CELL *my_rtype = NULL;

  /*  In ../peptide/atype.c:  */
  my_type = (TYPE_CELL *)make_and_set_type_cell( my_type );
  my_rtype = (RTYPE_CELL *)make_and_set_rtype_cell_bin( my_rtype );
  
  /*  In this file, below:  */
  minimize_my_free_energy_single_angle_iteration_bin( mol, my_type, my_rtype,
    changed_array, angle );
    
  kill_type_cell( my_type );
  kill_rtype_cell( my_rtype );
  
}
  
  
  









void minimize_my_free_energy_single_angle_iteration_bin( MOL_LIST *mol,
  TYPE_CELL *my_type, RTYPE_CELL *my_rtype, int changed_array[], STAT *angle )
{  

  RESID_LIST *resid = mol->my_mol->resid_header;
  RESID_LIST *end_resid = mol->my_mol->resid_header;
  
  int i = 0;
  
  while ( ++i < FIRST_FIRST_RESID )
    resid = resid->next;
    
  while ( end_resid->next != NULL )
    end_resid = end_resid->next;
  
  min_my_free_energy_single_angle_iteration_bin( mol, resid, end_resid,
    my_type, my_rtype, changed_array, angle );
    
  resid = mol->my_mol->resid_header;
  end_resid = mol->my_mol->resid_header;
  
  i = 0;
  while ( ++i < FIRST_SITE )
    resid = resid->next;
    
  i = 0;
  while ( ++i < FIRST_FIRST_RESID )
    end_resid = end_resid->next;
    
  min_my_free_energy_single_angle_iteration_bin( mol, resid, end_resid,
    my_type, my_rtype, changed_array, angle );

}















void min_my_free_energy_single_angle_iteration_bin( MOL_LIST *mol,
  RESID_LIST *resid, RESID_LIST *end_resid, TYPE_CELL *my_type, RTYPE_CELL *my_rtype,
  int changed_array[], STAT *angle )
{

  int arrays_are_same( int a[], int b[] );

  int curr_changed_array[ SHORT_BUFFER ];
  int changed_numb = 0;
  int index = 0;

  curr_changed_array[ changed_numb ] = 0;
  
  RESID_LIST *r = resid;
  
  while ( r != end_resid )
    /*  In this file, below:  */
    minimize_free_single_angle_single_resid_bin_and_step( mol, &r, 
      my_type, my_rtype, curr_changed_array, &changed_numb, ++index, *angle );
      
  printf( "\n\n" );
  
  if ( ! changed_numb )
    *angle *= ANGLE_STEP;
    
  /*  In change.c:  */  
  else if ( arrays_are_same( changed_array, curr_changed_array ))
    *angle /= ANGLE_STEP;
    
  int i = 0;
  
  while ( curr_changed_array[ i ] != 0 ) {
    changed_array[ i ] = curr_changed_array[ i ];
    ++i;
  }
  
  changed_array[ i ] = 0;

}












void minimize_free_single_angle_single_resid_bin_and_step( MOL_LIST *mol,
  RESID_LIST **resid, TYPE_CELL *my_type, RTYPE_CELL *my_rtype,
  int curr_changed_array[], int *changed_numb, int index, STAT angle )
{

  minimize_free_single_angle_single_resid_bin( mol, *resid, my_type, my_rtype, 
    curr_changed_array, changed_numb, index, angle );
    
  *resid = (*resid)->next;

}













void minimize_free_single_angle_single_resid_bin( MOL_LIST *mol, RESID_LIST *resid,
  TYPE_CELL *my_type, RTYPE_CELL *my_rtype, int curr_changed_array[], 
  int *changed_numb, int index, STAT angle )
{

  if ( resid->my_resid->mol_type == MOL_TYPE_AA )
    minimize_free_single_angle_bin_aa_resid( mol, resid, my_type, my_rtype, 
      curr_changed_array, changed_numb, index, angle );

}















void minimize_free_single_angle_bin_aa_resid( MOL_LIST *mol, RESID_LIST *resid,
  TYPE_CELL *my_type, RTYPE_CELL *my_rtype, int curr_changed_array[], 
  int *changed_numb, int index, STAT angle )
{

  STAT angle_array[ ANGLE_BUFFER ] = { angle, -1.0 * angle };
  
  int i;
  STAT holder;
  STAT phi_best = 0.0, psi_best = 0.0;
  
  STAT g = BIG;
  
  /*  In this file, below:  */
  do_min_g_rotate_single_phi_psi_pair_bin( mol, resid, my_type, my_rtype, 0.0, 0.0, 
    &phi_best, &psi_best, &g );
    
  for ( i = 0; i < ANGLE_BUFFER; i++ ) {

    /*  In this file, below:  */
    do_min_g_rotate_single_phi_psi_pair_bin( mol, resid, my_type, my_rtype,
      angle_array[ i ], 0.0, &phi_best, &holder, &g );      

    /*  In this file, below:  */
    do_min_g_rotate_single_phi_psi_pair_bin( mol, resid, my_type, my_rtype,
      0.0, angle_array[ i ], &holder, &psi_best, &g );

  }
  
  if (( phi_best != 0.0 ) && ( psi_best != 0.0 )) {
  
    STAT my_phi_best = phi_best;
    STAT my_psi_best = psi_best;
  
    /*  In this file, below:  */
    do_min_g_rotate_single_phi_psi_pair_bin( mol, resid, my_type, my_rtype,
      my_phi_best, 0.0, &phi_best, &psi_best, &g );
      
    /*  In this file, below:  */
    do_min_g_rotate_single_phi_psi_pair_bin( mol, resid, my_type, my_rtype,
      0.0, my_psi_best, &phi_best, &psi_best, &g );
      
    /*  In this file, below:  */
    do_min_g_rotate_single_phi_psi_pair_bin( mol, resid, my_type, my_rtype, 
      my_phi_best, my_psi_best, &phi_best, &psi_best, &g );
  
  }
  
  int curr_index = 0;
  
  if ( phi_best != 0.0 ) {
    /*  In phi_psi.c:  */
    rotate_phi_angle( mol, resid, phi_best );
    curr_index = index;
  }
  
  if ( psi_best != 0.0 ) {
    /*  In phi_psi.c:  */
    rotate_psi_angle( mol, resid, psi_best );
    curr_index = index;
  }
  
  if ( curr_index ) {

    curr_changed_array[ (*changed_numb)++ ] = curr_index;
    curr_changed_array[ *changed_numb ] = 0;
    
    printf( "%.3f   %s :    %8.3f  %8.3f\n", angle, resid->my_resid->resid, 
      phi_best, psi_best );
    
  }
  
}

















void do_min_g_rotate_single_phi_psi_pair_bin( MOL_LIST *mol, RESID_LIST *resid, 
  TYPE_CELL *my_type, RTYPE_CELL *my_rtype, STAT phi, STAT psi, STAT *phi_best, 
  STAT *psi_best, STAT *g )
{

  MOL_LIST *copy_mol_lister( MOL_LIST *mm, MOL_LIST *mmm );
  
  MOL_LIST *new_mol = NULL;
  /*  In ../mm/mol.c:  */
  new_mol = (MOL_LIST *)copy_mol_lister( new_mol, mol );
  
  /*  In this file, below:  */
  do_my_min_g_rotate_single_phi_psi_pair_bin( new_mol, resid, my_type, my_rtype,
    phi, psi, phi_best, psi_best, g );

  kill_mol_list( new_mol );

}














void do_my_min_g_rotate_single_phi_psi_pair_bin( MOL_LIST *mol, RESID_LIST *resid,
  TYPE_CELL *my_type, RTYPE_CELL *my_rtype, STAT phi, STAT psi, STAT *phi_best, 
  STAT *psi_best, STAT *g )
{

  RESID_LIST *get_resid_lister_w_numb( MOL_LIST *mol, int numb );

  RESID_LIST *my_resid = NULL;
  /*  In ../peptide/peptide.c:  */
  my_resid = (RESID_LIST *)get_resid_lister_w_numb( mol, resid->my_resid->numb );
  
  if ( phi != 0.0 )
    /*  In phi_psi.c:  */
    rotate_phi_angle( mol, my_resid, phi );
    
  if ( psi != 0.0 )
    /*  In phi_psi.c:  */
    rotate_psi_angle( mol, my_resid, psi );
  
  /*  In this file, below:  */
  STAT gg = do_free_energy_mol_list_bin( mol, my_type, my_rtype );
  
  if ( gg < *g ) {
    *phi_best = phi;
    *psi_best = psi;
    *g = gg;
  }
  
}


















/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/

















STAT do_free_energy_mol_list_bin( MOL_LIST *mol, TYPE_CELL *my_type, 
  RTYPE_CELL *my_rtype )
{

  /*  In ../peptide/atype.c:  */
  set_atom_types_mol_list( mol, my_type );
  set_resid_types_mol_list( mol, my_rtype );
  
  /*  In ../peptide/free_energy.c:  */
  set_mol_list_neighbors( mol );
  
  /*  In this file, below:  */
  return get_free_energy_mol_list_bin( mol );

}















STAT get_free_energy_mol_list_bin( MOL_LIST *mol_header )
{

  /*  In ../peptide/phi_psi.c:  */
  set_phi_psi_angles_w_mol_list( mol_header );
  
  STAT g = 0.0;
  
  MOL_LIST *mol = mol_header;
  
  while ( mol != NULL )
    /*  In this file, below:  */
    g += get_free_energy_single_mol_lister_bin_and_step( &mol );
    
  return g;

}













STAT get_free_energy_single_mol_lister_bin_and_step( MOL_LIST **mol )
{
  STAT g = get_free_energy_single_mol_lister_bin( *mol );
  *mol = (*mol)->next;
  return g;
}












STAT get_free_energy_single_mol_lister_bin( MOL_LIST *mol )
{

  STAT g = 0.0;
  
  RESID_LIST *resid = mol->my_mol->resid_header;
  
  while ( resid != NULL )
    g += get_free_energy_single_resid_lister_bin_and_step( &resid );
    
  return g;

}













STAT get_free_energy_single_resid_lister_bin_and_step( RESID_LIST **resid )
{
  STAT g = get_free_energy_single_resid_lister_bin( *resid );
  *resid = (*resid)->next;
  return g;
}













STAT get_free_energy_single_resid_lister_bin( RESID_LIST *resid )
{

  STAT get_free_energy_zinc_ion( RESID_LIST *resid );
  int get_phi_psi_bin_index( STAT angle );
  STAT get_free_energy_single_resid_pair_and_step( RESID_LIST *resid,
    RESID_LIST **neigh );
    
  if ( ! strcmp( resid->my_resid->resid, "ZN" )) 
    /*  In ../peptide/free_energy.c:  */
    return get_free_energy_zinc_ion( resid );

  /*  In ../peptide/pep_params.c:  */
  int phi_index = get_phi_psi_bin_index( RADIANS_TO_DEGREES(resid->my_resid->phi ));
  int psi_index = get_phi_psi_bin_index( RADIANS_TO_DEGREES(resid->my_resid->psi ));

  RTYPE *r = resid->my_resid->type; 
    
  STAT g = 0.0;
  STAT a = PHI_PSI_WEIGHT * (r->phi_array[ phi_index ] + r->psi_array[ psi_index ]);
  
  RESID_LIST *neigh = resid->my_resid->neigh_header;
  
  while ( neigh != NULL )
    /*  In ../peptide/free_energy.c:  */
    g += get_free_energy_single_resid_pair_and_step( resid, &neigh );

  return a + g;

}















/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/

















void do_mol_compare_bin( void )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  
  char major_file[ BUFFER ], minor_file[ BUFFER ], ext[ BUFFER ];
  
  sprintf( major_file, 
    "%s.sub.first_3.thresh_0.125.txt.fromCenter.firstFirst_4.NoSubs.txt",
    OUTPUT_FNAME_BASE ); 

  sprintf( minor_file, 
    "%s.sub.first_3.thresh_0.125.4V.txt.fromCenter.firstFirst_4.txt",
    OUTPUT_FNAME_BASE ); 

//  sprintf( minor_file, "data/sp1/1VA2.txt" );
    
  strcpy( ext, "4V" );
  
  FILE *out = NULL;
  out = (FILE *)fopen( "hello", "w" );
  
  MOL_LIST *major_mol = NULL, *minor_mol = NULL;  
  /*  In ../parse/mol_list.c:  */
  major_mol = (MOL_LIST *)get_mol_list_w_fname( major_mol, major_file );
  minor_mol = (MOL_LIST *)get_mol_list_w_fname( minor_mol, minor_file );
  
  printf( "%s\n%s\n", major_file, minor_file );
  
  /*  In this file, below:  */
  do_mol_compare_single_mol_pair( major_mol, minor_mol, major_file, ext, out );
  
  kill_mol_list( major_mol );
  kill_mol_list( minor_mol );

  fclose( out );

}



    
    
    
    
    




