#include "comp.h"













#define INPUT_FNAME "data/sp1/1VA2.txt"
#define MID_FNAME "sub.first_3.thresh_0.125"
#define EXT ".fromCenter.firstFirst_5.txt"
#define OUTPUT_FNAME "data/sp1/compare_results.Xorg"

#define FNAME_BUFFER 8
#define VERSION_BUFFER 3

#define VERSIONS { "txt.fromCenter.firstFirst_4.txt", "txt.fromCenter.firstFirst_5.txt", "txt.fromCenter.firstFirst_6.txt" }



#define SEQUENCE "RPFMCTWSYCGKRFTRSDELQRHKRTHTGEK"
//#define SEQUENCE "PFMCTWSYCGKRFTRSDELQRHKRTHTGEK"




#define REGION { 11, 19 }
#define START_STOP_BUFFER 2

#define SMALL_ANGLE 0.001
#define ABS(a) (((a) > 0.0) ? (a) : (-1.0 * (a)))
#define RADIANS_TO_DEGREES(a) (180 * (a) / PI)

#define BIG 1000000.0






#define SP1_DIR "data/sp1"
#define OUTPUT_ZF_FNAME "data/sp1/compare_zinc_fingers.txt"

#define HELIX_ZF1 "TSHLRA"
#define HELIX_ZF2 "SDELQR"
#define HELIX_ZF3 "SDHLSK"















void do_mol_compare( void )
{

  char *ext[ FNAME_BUFFER ] = 
    {
    "NoSubs",
    "4V",
    "4I",
    "6S.8S.9F",
    "4I.8M",
    "6S",
    "8V",
    "9F" };
    
  char *version[ VERSION_BUFFER ] = VERSIONS;
    
  do_my_mol_compare( INPUT_FNAME, MID_FNAME, ext, version, OUTPUT_FNAME );

}













void do_my_mol_compare( char input_fname[], char mid_fname[], char *ext[],
  char *version[], char output_fname[] )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  char fname[ BUFFER ];
  sprintf( fname, "%s.%s", output_fname, mid_fname );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( fname, "txt" );
  
  print_sep( out );
  
  do_mol_compare_w_output_fh( input_fname, mid_fname, ext, version, out );
  
  close_file( out );

}















void do_mol_compare_w_output_fh( char input_fname[], char mid_fname[],
  char *ext[], char *version[], FILE *out )
{

  char major[ BUFFER ];

  /*
  get_best_compare_version_fname( major, input_fname, mid_fname, ext[ 0 ], version );
    
  do_mol_compare_single_major_file( major, input_fname, mid_fname, ext, version,
    out );
  
  
  get_best_compare_version_fname( major, input_fname, mid_fname, ext[ 1 ], version );
    
  do_mol_compare_single_major_file( major, input_fname, mid_fname, ext, version,
    out );
  
  /*  For compare_results_orig...  */
  
  do_mol_compare_single_major_file( input_fname, input_fname, mid_fname, 
    ext, version, out );
  
  
}














void do_mol_compare_single_major_file( char major_file[], char input_fname[],
  char mid_fname[], char *ext[], char *version[], FILE *out )
{

  int i;
  
  for ( i = 0; i < FNAME_BUFFER; i++ )
    do_mol_compare_single_file_pair( major_file, input_fname, mid_fname, ext[ i ], 
      version, out );

}













void get_best_compare_version_fname( char fname[], char input[], char mid_fname[],
  char *ext, char *version[] )
{

  DATA *make_data_struct( DATA *my_data );
  RTYPE_CELL *make_and_set_rtype_cell( RTYPE_CELL *my_rtype );
  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  STAT get_free_energy_mol_list( MOL_LIST *mol_header );

  int i, best = -1;
  STAT g_best = BIG;
  
  char ff[ BUFFER ];
  
  for ( i = 0; i < VERSION_BUFFER; i++ ) {
  
    MOL_LIST *mol = NULL;
    DATA *my_data = NULL;
    RTYPE_CELL *my_rtype = NULL;
    
    sprintf( ff, "%s.%s.%s.%s", input, mid_fname, ext, version[ i ] );
    
    /*  In ../parse/mol_list.c:  */
    mol = (MOL_LIST *)get_mol_list_w_fname( mol, ff );
    
    /*  In atype.c:  */
    my_data = (DATA *)make_data_struct( my_data );
    my_rtype = (RTYPE_CELL *)make_and_set_rtype_cell( my_rtype );
    
    /*  In atype.c:  */
    set_atom_types_mol_list( mol, my_data->type );
  
    /*  In ../peptide/free_energy.c:  */
    set_mol_list_neighbors( mol );
  
    /*  In ../peptide/free_energy.c:  */
    STAT g = get_free_energy_mol_list( mol );
    
    if ( g < g_best ) {
      best = i;
      g_best = g;
    }
    
    printf( "g = %.3f\n", g );
    
    kill_mol_list( mol );  
  
  }

  sprintf( fname, "%s.%s.%s.%s", input, mid_fname, ext, version[ best ] );
    
}












void do_mol_compare_single_file_pair( char major_file[], char input_fname[],
  char mid_fname[], char *ext, char *version[], FILE *out )
{

  char minor_file[ BUFFER ];

  /*  In this file, above:  */
  get_best_compare_version_fname( minor_file, input_fname, mid_fname, ext,
    version );
  
  if ( ! strcmp( minor_file, major_file ))
    return;
  
  do_mol_compare_single_file_fname_pair( major_file, minor_file, ext, out );

}














void do_mol_compare_single_file_fname_pair( char major_file[], char minor_file[],
  char *ext, FILE *out )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  
  MOL_LIST *major_mol = NULL, *minor_mol = NULL;  
  /*  In ../parse/mol_list.c:  */
  major_mol = (MOL_LIST *)get_mol_list_w_fname( major_mol, major_file );
  minor_mol = (MOL_LIST *)get_mol_list_w_fname( minor_mol, minor_file );
  
  printf( "%s\n%s\n", major_file, minor_file );
  
  /*  In this file, below:  */
  do_mol_compare_single_mol_pair( major_mol, minor_mol, major_file, ext, out );
  
  kill_mol_list( major_mol );
  kill_mol_list( minor_mol );

}














void do_mol_compare_single_mol_pair( MOL_LIST *major_mol, MOL_LIST *minor_mol,
  char major_file[], char *ext, FILE *out )
{

  int get_seq_index_substr( MOL_LIST *m, char substr[] );
  
  /*  In ../peptide/rmsd.c:  */
  int index = get_seq_index_substr( major_mol, SEQUENCE );
  int length = strlen( SEQUENCE );
  
  printf( "index :  %d\nlength :  %d\n", index, length );
  
  /*  In this file, below:  */
  do_mol_compare_single_mol_pair_substr_index( major_mol, minor_mol, index, 
    length, major_file, ext, out );
  
}













void do_mol_compare_single_mol_pair_substr_index( MOL_LIST *major_mol,
  MOL_LIST *minor_mol, int index, int length, char major_file[], char *ext, 
  FILE *out )
{

  XYZ_CELL *create_xyz_cell( XYZ_CELL *xx, int nn );
  
  XYZ_CELL *major_cell = NULL, *minor_cell = NULL;
  major_cell = (XYZ_CELL *)create_xyz_cell( major_cell, length );
  minor_cell = (XYZ_CELL *)create_xyz_cell( minor_cell, length );
  
  do_my_mol_compare_single_mol_pair_substr_index( major_mol, minor_mol, 
    major_cell, minor_cell, index, major_file, ext, out );
    
  kill_xyz_cell( major_cell );
  kill_xyz_cell( minor_cell );

}














void do_my_mol_compare_single_mol_pair_substr_index( MOL_LIST *major_mol,
  MOL_LIST *minor_mol, XYZ_CELL *major_cell, XYZ_CELL *minor_cell, int index,
  char major_file[], char *ext, FILE *out )
{

  STAT get_rmsd_xyz_cell_w_params( ROT_PARAMS *my_params, XYZ_CELL *major_cell,
    XYZ_CELL *minor_cell );
    
  ROT_PARAMS my_params;

  /*  In this file, below:  */
  set_xyz_cell_carbon_alpha_atoms_index( major_cell, major_mol, index );
  set_xyz_cell_carbon_alpha_atoms_index( minor_cell, minor_mol, index );
  
  /*  In ../peptide/rmsd.c:  */
  STAT rmsd = get_rmsd_xyz_cell_w_params( &my_params, major_cell, minor_cell );
  
  printf( "\t\tRMSD :  %.8f\n", rmsd );
  
  /*  In this file, below:  */
  do_mol_compare_w_rotation_params( &my_params, major_mol, minor_mol, major_cell,
    minor_cell, rmsd, index, major_file, ext, out );

}













void set_xyz_cell_carbon_alpha_atoms_index( XYZ_CELL *my_cell, MOL_LIST *my_mol, 
  int index )
{

  RESID_LIST *resid = my_mol->my_mol->resid_header;

  int i = 0;
  
  while ( i++ < index )
    resid = resid->next;
    
  XYZ *ptr = my_cell->array;
  
  i = 0;
  
  while ( i++ < my_cell->n ) {
  
    ATOM_LIST *atom = NULL;
    atom = (ATOM_LIST *)get_atom_lister_w_name( resid, "CA" );
    
    if ( atom != NULL )
      *ptr++ = *(atom->my_atom->xyz);
    
    resid = resid->next;
  
  }

}












ATOM_LIST *get_atom_lister_w_name( RESID_LIST *resid, char str[] )
{

  ATOM_LIST *atom = resid->my_resid->atom_header;
  
  while ( atom != NULL ) {
  
    if ( ! strcmp( atom->my_atom->atom, str ))
      return atom;
      
    atom = atom->next;
  
  }
  
  return NULL;

}













void do_mol_compare_w_rotation_params( ROT_PARAMS *my_params, MOL_LIST *major_mol, 
  MOL_LIST *minor_mol, XYZ_CELL *major_cell, XYZ_CELL *minor_cell, STAT rmsd,
  int index, char major_file[], char *ext, FILE *out )
{

  /*  In rmsd.c:  */
  rotate_mol_lister_w_rot_params( minor_mol, my_params );
  
  /*  In this file, below:  */
  do_mol_compare_w_rotated_mol_lister( major_mol, minor_mol, major_cell,
    minor_cell, rmsd, index, major_file, ext, out );

}











void do_mol_compare_w_rotated_mol_lister( MOL_LIST *major_mol, MOL_LIST *minor_mol,
  XYZ_CELL *major_cell, XYZ_CELL *minor_cell, STAT rmsd, int index, 
  char major_file[], char *ext, FILE *out )
{

  print_mol_compare_header( major_file, ext, major_mol, minor_mol, out );
  
  fprintf( out, "\n" );
  fprintf( out, "Total RMSD :  %.3f\n", rmsd );
  fprintf( out, "\n" );
  
  get_and_print_compare_rotated_mol_lister( major_mol, minor_mol, out );
  
  print_sep( out );
  
  fflush( out );

}












void print_mol_compare_header( char major_file[], char *ext, MOL_LIST *major_mol,
  MOL_LIST *minor_mol, FILE *out )
{

  print_mol_compare_file_names( major_file, ext, out );
  
  print_mol_compare_seq( major_mol, minor_mol, out );

}












void print_mol_compare_file_names( char major_file[], char *ext, FILE *out )
{

  fprintf( out, "MajorFile:   %s\n", major_file );
  fprintf( out, "%s\n\n", ext );

}












void print_mol_compare_seq( MOL_LIST *major_mol, MOL_LIST *minor_mol, FILE *out )
{

  print_mol_compare_single_seq( major_mol, minor_mol, out );
  print_mol_compare_single_seq( minor_mol, major_mol, out );
  
  int numb = get_numb_of_resids_mol( major_mol );
  
  print_numbs( numb, out );

}












void print_mol_compare_single_seq( MOL_LIST *major_mol, MOL_LIST *minor_mol,
  FILE *out )
{

  RESID_LIST *major_resid = major_mol->my_mol->resid_header;
  RESID_LIST *minor_resid = minor_mol->my_mol->resid_header;
  
  while ( major_resid != NULL )
    print_resid_compare_single_resid_and_step( &major_resid, &minor_resid, out );
    
  fprintf( out, "\n" );

}












void print_resid_compare_single_resid_and_step( RESID_LIST **major_resid,
  RESID_LIST **minor_resid, FILE *out )
{
  print_resid_compare_single_resid( *major_resid, *minor_resid, out );
  *major_resid = (*major_resid)->next;
  *minor_resid = (*minor_resid)->next;
}












void print_resid_compare_single_resid( RESID_LIST *major_resid,
  RESID_LIST *minor_resid, FILE *out )
{

  char major_aa = major_resid->my_resid->aa;
  char minor_aa = minor_resid->my_resid->aa;
  
  if ( major_aa == minor_aa )
    major_aa = tolower( major_aa );
    
  fprintf( out, "%c", major_aa );

}












int get_numb_of_resids_mol( MOL_LIST *mol )
{

  int numb = 0;
  
  RESID_LIST *resid = mol->my_mol->resid_header;
  
  while ( resid != NULL ) {
  
    ++numb;
    
    resid = resid->next;
  
  }
  
  return numb;

}









void print_numbs( int numb, FILE *out )
{

  int i = 0;
  
  while ( i++ < numb )
    fprintf( out, "%d", i % 10 );
    
  fprintf( out, "\n" );

}













void get_and_print_compare_rotated_mol_lister( MOL_LIST *major_mol, 
  MOL_LIST *minor_mol, FILE *out )
{

  /*  In this file, below:  */
  print_compare_rotated_mol_lister_dist( major_mol, minor_mol, out );
  
  fprintf( out, "\n\n" );
  
  int region[ START_STOP_BUFFER ] = REGION;
  
  /*  In this file, below:  */
  get_and_print_region_rmsd( major_mol, minor_mol, region, "My Region", out );
  
  /*  In ../peptide/phi_psi.c:  */
  set_phi_psi_angles_w_mol_list( major_mol );
  set_phi_psi_angles_w_mol_list( minor_mol );

  /*  In this file, below:  */
  print_compare_rotated_mol_lister_angles( major_mol, minor_mol, out );
  
}











void print_compare_rotated_mol_lister_dist( MOL_LIST *major_mol,
  MOL_LIST *minor_mol, FILE *out )
{

  fprintf( out, "    AA  AA\t%8s\t%8s\n", "Dist", "RMSD" );

  int i = 0;

  RESID_LIST *major_resid = major_mol->my_mol->resid_header;
  RESID_LIST *minor_resid = minor_mol->my_mol->resid_header;
  
  while ( major_resid != NULL )
    print_compare_rotated_single_resid_lister_and_step( &major_resid, &minor_resid, 
      ++i, out );

}













void print_compare_rotated_single_resid_lister_and_step( RESID_LIST **major_resid,
  RESID_LIST **minor_resid, int i, FILE *out )
{
  print_compare_rotated_single_resid_lister( *major_resid, *minor_resid, i, out );
  *major_resid = (*major_resid)->next;
  *minor_resid = (*minor_resid)->next;
}











void print_compare_rotated_single_resid_lister( RESID_LIST *major_resid,
  RESID_LIST *minor_resid, int i, FILE *out )
{

  ATOM_LIST *get_atom_from_resid( RESID_LIST *resid, char a[] );
  STAT get_dist_xyz( XYZ *a, XYZ *b );
  
  RESID_LIST *major = major_resid;
  RESID_LIST *minor = minor_resid;
  
  if (( major->my_resid->mol_type != MOL_TYPE_AA ) ||
    ( minor->my_resid->mol_type != MOL_TYPE_AA ))
  {
    return;
  }

  ATOM_LIST *major_atom = NULL, *minor_atom = NULL;
  
  /*  In ../peptide/sub.c:  */
  major_atom = (ATOM_LIST *)get_atom_from_resid( major_resid, "CA" );
  minor_atom = (ATOM_LIST *)get_atom_from_resid( minor_resid, "CA" );
  
  if (( major_atom == NULL ) || ( minor_atom == NULL )) {
    printf( "%d :  %s\n", i, major_resid->my_resid->resid );
    error( 4100, "Did not find alpha carbon in residue.\n" );
  }
  
  /*  In ../main/xyz.c:  */
  STAT dist = get_dist_xyz( major_atom->my_atom->xyz, minor_atom->my_atom->xyz );
  
  fprintf( out, "%2d   %c   %c\t%8.3f\t%.5f\n",
    i,
    major_resid->my_resid->aa,
    minor_resid->my_resid->aa,
    dist,
    dist * dist );

}














void get_and_print_region_rmsd( MOL_LIST *major_mol, MOL_LIST *minor_mol, 
  int region[], char label[], FILE *out )
{

  STAT sum = 0.0;
  
  RESID_LIST *major_resid = major_mol->my_mol->resid_header;
  RESID_LIST *minor_resid = minor_mol->my_mol->resid_header;
  
  int i = 0;
  
  while ( i++ < region[ 1 ] )
    sum += get_region_rmsd_single_resid_lister_and_step( &major_resid, &minor_resid, 
      region[ 0 ], i );
    
  STAT rmsd = sqrt( sum / (STAT)(region[ 1 ] - region[ 0 ] + 1) );
  
  fprintf( out, "\n" );
  fprintf( out, "%20s\tRegion (%d,%d) RMSD :  %.3f\n", label, region[ 0 ], 
    region[ 1 ], (float)rmsd );
  fprintf( out, "\n" );

}











STAT get_region_rmsd_single_resid_lister_and_step( RESID_LIST **major_resid,
  RESID_LIST **minor_resid, int start, int i )
{
  
  STAT x = get_region_rmsd_single_resid_lister( *major_resid, *minor_resid, 
    start, i );
  
  *major_resid = (*major_resid)->next;
  *minor_resid = (*minor_resid)->next;
  
  return x;
  
}













STAT get_region_rmsd_single_resid_lister( RESID_LIST *major_resid,
  RESID_LIST *minor_resid, int start, int i )
{

  ATOM_LIST *get_atom_from_resid( RESID_LIST *resid, char a[] );
  STAT get_dist_xyz( XYZ *a, XYZ *b );
  
  if ( i < start )
    return 0.0;

  RESID_LIST *major = major_resid;
  RESID_LIST *minor = minor_resid;
  
  if (( major->my_resid->mol_type != MOL_TYPE_AA ) ||
    ( minor->my_resid->mol_type != MOL_TYPE_AA ))
  {
    return 0.0;
  }

  ATOM_LIST *major_atom = NULL, *minor_atom = NULL;
  
  /*  In ../peptide/sub.c:  */
  major_atom = (ATOM_LIST *)get_atom_from_resid( major_resid, "CA" );
  minor_atom = (ATOM_LIST *)get_atom_from_resid( minor_resid, "CA" );
  
  /*  In ../main/xyz.c:  */
  STAT dist = get_dist_xyz( major_atom->my_atom->xyz, minor_atom->my_atom->xyz );
  
  return dist * dist;
  
}









void print_compare_rotated_mol_lister_angles( MOL_LIST *major_mol, 
  MOL_LIST *minor_mol, FILE *out )
{

  fprintf( out, "    AA  AA\t%8s  %8s\t%8s  %8s\n", 
    "PHI", "PSI",
    "PHI", "PSI" );

  int i = 0;

  RESID_LIST *major_resid = major_mol->my_mol->resid_header;
  RESID_LIST *minor_resid = minor_mol->my_mol->resid_header;
  
  while ( major_resid != NULL )
    print_compare_resid_lister_angles_and_step( &major_resid, &minor_resid, 
      ++i, out );

}












void print_compare_resid_lister_angles_and_step( RESID_LIST **major_resid,
  RESID_LIST **minor_resid, int i, FILE *out )
{
  print_compare_resid_lister_angles( *major_resid, *minor_resid, i, out );
  *major_resid = (*major_resid)->next;
  *minor_resid = (*minor_resid)->next;
}












void print_compare_resid_lister_angles( RESID_LIST *major_resid,
  RESID_LIST *minor_resid, int i, FILE *out )
{

  STAT major_phi = major_resid->my_resid->phi;
  STAT major_psi = major_resid->my_resid->psi;
  STAT minor_phi = minor_resid->my_resid->phi;
  STAT minor_psi = minor_resid->my_resid->psi;

  fprintf( out, "%2d   %c  %c\t%8.2f  %8.2f\t%8.2f  %8.2f",
    i,
    major_resid->my_resid->aa, minor_resid->my_resid->aa,
    RADIANS_TO_DEGREES(major_phi), RADIANS_TO_DEGREES(major_psi),
    RADIANS_TO_DEGREES(minor_phi), RADIANS_TO_DEGREES(minor_psi) );
    
  if (( ABS( major_phi - minor_phi ) > SMALL_ANGLE ) ||
    ( ABS( major_psi - minor_psi ) > SMALL_ANGLE ))
  {
    fprintf( out, "\t%8.3f  %8.3f",
      RADIANS_TO_DEGREES(major_phi - minor_phi),
      RADIANS_TO_DEGREES(major_psi - minor_psi) );
  }
  
  fprintf( out, "\n" );

}







/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
















void do_zinc_finger_compare( void )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  
  MOL_LIST *zf1 = NULL, *zf2 = NULL, *zf3 = NULL;
  
  char zf1_fname[ BUFFER ], zf2_fname[ BUFFER ], zf3_fname[ BUFFER ];
  
  sprintf( zf1_fname, "%s/1VA1.txt", SP1_DIR );
  sprintf( zf2_fname, "%s/1VA2.txt", SP1_DIR );
  sprintf( zf3_fname, "%s/1VA3.txt", SP1_DIR );
  
  /*  In ../parse/mol_list.c:  */
  zf1 = (MOL_LIST *)get_mol_list_w_fname( zf1, zf1_fname );
  zf2 = (MOL_LIST *)get_mol_list_w_fname( zf2, zf2_fname );
  zf3 = (MOL_LIST *)get_mol_list_w_fname( zf3, zf3_fname );
  
  /*  In this file, below:  */
  do_zinc_finger_compare_mol_listers( zf1, zf2, zf3, OUTPUT_ZF_FNAME );

}















void do_zinc_finger_compare_mol_listers( MOL_LIST *zf1, MOL_LIST *zf2,
  MOL_LIST *zf3, char output_fname[] )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( output_fname, "" );
  
  do_zinc_finger_compare_mol_listers_w_fh( zf1, zf2, zf3, out );
  
  close_file( out );

}












void do_zinc_finger_compare_mol_listers_w_fh( MOL_LIST *zf1, MOL_LIST *zf2,
  MOL_LIST *zf3, FILE *out )
{

  print_sep( out );

  do_zinc_finger_pair_compare( zf2, zf1, HELIX_ZF2, HELIX_ZF1, "ZF2", "ZF1", out );
  do_zinc_finger_pair_compare( zf2, zf3, HELIX_ZF2, HELIX_ZF3, "ZF2", "ZF3", out );
  do_zinc_finger_pair_compare( zf1, zf3, HELIX_ZF1, HELIX_ZF3, "ZF1", "ZF3", out );

}














void do_zinc_finger_pair_compare( MOL_LIST *a, MOL_LIST *b, char helix_a[],
  char helix_b[], char zf_a[], char zf_b[], FILE *out )
{

  fprintf( out, "%s vs %s\n", zf_a, zf_b );
  fprintf( out, "\n" );
  fprintf( out, "%s\n", helix_a );
  fprintf( out, "%s\n", helix_b );
  fprintf( out, "\n" );
  
  do_my_zinc_finger_pair_compare( a, b, helix_a, helix_b, out );
  
  print_sep( out );

}














void do_my_zinc_finger_pair_compare( MOL_LIST *a, MOL_LIST *b, char helix_a[],
  char helix_b[], FILE *out )
{

  int get_seq_index_substr( MOL_LIST *m, char substr[] );
  
  /*  In ../peptide/rmsd.c:  */
  int index_a = get_seq_index_substr( a, helix_a );
  int index_b = get_seq_index_substr( b, helix_b );
  
  int length = strlen( helix_a );
  
  /*  In this file, below:  */
  do_zinc_finger_pair_compare_indices( a, b, index_a, index_b, length, out );

}















void do_zinc_finger_pair_compare_indices( MOL_LIST *a, MOL_LIST *b,
  int index_a, int index_b, int length, FILE *out )
{

  XYZ_CELL *create_xyz_cell( XYZ_CELL *xx, int nn );
  
  XYZ_CELL *a_cell = NULL, *b_cell = NULL;
  a_cell = (XYZ_CELL *)create_xyz_cell( a_cell, length );
  b_cell = (XYZ_CELL *)create_xyz_cell( b_cell, length );
  
  do_my_zinc_finger_pair_compare_indices( a, b, a_cell, b_cell, index_a, index_b, 
    length, out );
    
  kill_xyz_cell( a_cell );
  kill_xyz_cell( b_cell );

}
















void do_my_zinc_finger_pair_compare_indices( MOL_LIST *a, MOL_LIST *b, 
  XYZ_CELL *a_cell, XYZ_CELL *b_cell, int index_a, int index_b, int length,
  FILE *out )
{

  STAT get_rmsd_xyz_cell_w_params( ROT_PARAMS *my_params, XYZ_CELL *major_cell,
    XYZ_CELL *minor_cell );
    
  ROT_PARAMS my_params;

  /*  In this file, below:  */
  set_xyz_cell_carbon_alpha_atoms_index( a_cell, a, index_a );
  set_xyz_cell_carbon_alpha_atoms_index( b_cell, b, index_b );
  
  /*  In ../peptide/rmsd.c:  */
  STAT rmsd = get_rmsd_xyz_cell_w_params( &my_params, a_cell, b_cell );
  
  printf( "\t\tRMSD :  %.3f\n", rmsd );
  
  fprintf( out, "\n" );
  fprintf( out, "Total RMSD :  %.3f\n", rmsd );
  fprintf( out, "\n" );
  
  /*  In rmsd.c:  */
  rotate_mol_lister_w_rot_params( b, &my_params );
  
  /*  In this file, below:  */
  print_zinc_finger_compare( a, b, index_a, index_b, length, out );
  
  print_sep( out );
  
  fflush( out );


}














void print_zinc_finger_compare( MOL_LIST *a, MOL_LIST *b, int index_a, int index_b,
  int length, FILE *out )
{

  RESID_LIST *a_resid = a->my_mol->resid_header;
  RESID_LIST *b_resid = b->my_mol->resid_header;
  
  int i = 0;
  
  while ( i++ < index_a )
    a_resid = a_resid->next;
    
  i = 0;  
    
  while ( i++ < index_b )
    b_resid = b_resid->next;
    
  print_zinc_finger_compare_resid_start( a_resid, b_resid, length, out );

}














void print_zinc_finger_compare_resid_start( RESID_LIST *a, RESID_LIST *b,
  int length, FILE *out )
{

  fprintf( out, "    AA  AA\t%8s\t%8s\n", "Dist", "RMSD" );

  RESID_LIST *resid_a = a, *resid_b = b;

  int i = 0;
  
  while ( i++ < length )
    print_zinc_finger_compare_single_resid_lister_and_step( &resid_a, &resid_b,
      i, out );

}













void print_zinc_finger_compare_single_resid_lister_and_step( RESID_LIST **resid_a,
  RESID_LIST **resid_b, int i, FILE *out )
{
  print_zinc_finger_compare_single_resid_lister( *resid_a, *resid_b, i, out );
  *resid_a = (*resid_a)->next;
  *resid_b = (*resid_b)->next;
}












void print_zinc_finger_compare_single_resid_lister( RESID_LIST *resid_a,
  RESID_LIST *resid_b, int i, FILE *out )
{

  /*  In this file, above:  */
  print_compare_rotated_single_resid_lister( resid_a, resid_b, i, out );
  
}













