#include "replace.h"


















#define INPUT_FNAME "data/sp1/1VA2.txt"



#define AA_NUMB { 4, 6, 8, 9 }
#define AA_TYPE { 'L', 'S', 'M', 'F' }
#define AA_SUBS { 1, 0, 0, 0 }
#define CHANGED_BUFFER 4

#define INIT_ANGLE 2.0
#define ANGLE_THRESH 0.125
#define ANGLE_STEP 0.7

#define ZINC_INIT 0.100
#define ZINC_THRESH 0.01
#define ZINC_STEP 0.7
#define ZINC_ARRAY_BUFFER 3

#define ZINC_K 640.0
#define ZINC_SG_DIST 2.298
#define ZINC_NE_DIST 2.053


#define FIRST_SITE 3
#define FIRST_FIRST_RESID 4


#define ORIG_REPLACE_TYPE 0
#define ANGLE_BIN_TYPE 1
#define BIN_ZINC_TYPE 2

#define REPLACE_TYPE (ANGLE_BIN_TYPE)


#define MAIN_DIR "data/sp1/main"
#define ZINC_BIN_DIR "data/sp1/zinc_and_bin"
#define ANGLE_BIN_DIR "data/sp1/angle_bin"

#define ANGLE_BUFFER 2
#define BIG 100000000.0






















void do_replace( void )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  
  MOL_LIST *mol = NULL;  
  /*  In ../parse/mol_list.c:  */
  mol = (MOL_LIST *)get_mol_list_w_fname( mol, INPUT_FNAME ); 
  
  int aa_numb[ CHANGED_BUFFER ] = AA_NUMB;
  char aa_type[ CHANGED_BUFFER ] = AA_TYPE;
  int aa_subs[ CHANGED_BUFFER ] = AA_SUBS;
  
  /*  In this file, below:  */
  do_my_replace( mol, aa_numb, aa_type, aa_subs, INPUT_FNAME, REPLACE_TYPE );

}















void do_my_replace( MOL_LIST *mol, int aa_numb[], char aa_type[], int aa_subs[],
  char input_fname[], int replace_type )
{

  MOL_LIST *copy_mol_lister( MOL_LIST *major, MOL_LIST *minor );
  
  MOL_LIST *init_mol = NULL;

  /*  In ../mm/mol.c:  */
  init_mol = (MOL_LIST *)copy_mol_lister( init_mol, mol );
  
  /*  In this file, below:  */
  do_replace_mol_list( mol, aa_numb, aa_type, aa_subs, replace_type );
  
  /*  In ../peptide/phi_psi.c:  */
  set_phi_psi_angles_w_mol_list( init_mol );
  set_phi_psi_angles_w_mol_list( mol );
  
  /*  In this file, below:  */
  output_replacement_mol( mol, aa_numb, aa_type, aa_subs, FIRST_FIRST_RESID,
    replace_type );

  /*  In ../peptide/peptide.c:  */
  print_two_mol_lists_diff_phi_psi( mol, init_mol );
  
}















void do_replace_mol_list( MOL_LIST *mol, int aa_numb[], char aa_type[],
  int aa_subs[], int replace_type )
{

  /*  In change.c:  */
  do_evol_change_point_substitutions( mol, aa_numb, aa_type, aa_subs );
  
  /*  In this file, below:  */
  minimize_free_energy_replacement( mol, replace_type );

}














void minimize_free_energy_replacement( MOL_LIST *mol, int replace_type )
{

  int changed_array[ SHORT_BUFFER ];
  changed_array[ 0 ] = 0;
  
  STAT angle = INIT_ANGLE;
  
  while ( angle > ANGLE_THRESH )
    minimize_free_energy_replacement_single_angle( mol, changed_array, &angle,
      replace_type );

}














void minimize_free_energy_replacement_single_angle( MOL_LIST *mol,
  int changed_array[], STAT *angle, int replace_type )
{

  RTYPE_CELL *make_and_set_rtype_cell( RTYPE_CELL *my_rtype );
  TYPE_CELL *make_and_set_type_cell( TYPE_CELL *my_type );
  
  TYPE_CELL *my_type = NULL;
  RTYPE_CELL *my_rtype = NULL;

  /*  In ../peptide/atype.c:  */
  my_type = (TYPE_CELL *)make_and_set_type_cell( my_type );
  my_rtype = (RTYPE_CELL *)make_and_set_rtype_cell( my_rtype );
  
  /*  In this file, below:  */
  minimize_free_energy_replacement_angle_iter( mol, my_type, my_rtype, 
    changed_array, angle, replace_type );
    
  printf( "\n\n" );
    
  kill_type_cell( my_type );
  kill_rtype_cell( my_rtype );
  
}















void minimize_free_energy_replacement_angle_iter( MOL_LIST *mol, TYPE_CELL *my_type,
  RTYPE_CELL *my_rtype, int changed_array[], STAT *angle, int replace_type )
{

  int curr_changed_array[ SHORT_BUFFER ];
  int changed_numb = 0;
  
  curr_changed_array[ changed_numb ] = 0;

  do_minimize_free_energy_replace_three_prime( mol, my_type, my_rtype,
    curr_changed_array, angle, &changed_numb, replace_type );
    
  do_minimize_free_energy_replace_five_prime( mol, my_type, my_rtype,
    curr_changed_array, angle, &changed_numb, replace_type );

  update_min_free_energy_single_iter( changed_array, curr_changed_array,
    changed_numb, angle );

}













void do_minimize_free_energy_replace_three_prime( MOL_LIST *mol, TYPE_CELL *my_type,
  RTYPE_CELL *my_rtype, int changed_array[], STAT *angle, int *changed_numb,
  int replace_type )
{

  RESID_LIST *resid = mol->my_mol->resid_header;
  RESID_LIST *end_resid = NULL;
  
  int i = 0;
  
  while ( ++i < FIRST_FIRST_RESID )
    resid = resid->next;
    
  do_minimize_free_energy_single_angle_iteration( mol, resid, end_resid, my_type, 
    my_rtype, changed_array, angle, changed_numb, replace_type );

}














void do_minimize_free_energy_replace_five_prime( MOL_LIST *mol, TYPE_CELL *my_type, 
  RTYPE_CELL *my_rtype, int changed_array[], STAT *angle, int *changed_numb, 
  int replace_type )
{

  RESID_LIST *resid = mol->my_mol->resid_header;
  RESID_LIST *end_resid = mol->my_mol->resid_header;

  int i = 0;
  
  while ( ++i < FIRST_SITE )
    resid = resid->next;
    
  i = 0;
  
  while ( ++i < FIRST_FIRST_RESID )
    end_resid = end_resid->next;
    
  do_minimize_free_energy_single_angle_iteration( mol, resid, end_resid,
    my_type, my_rtype, changed_array, angle, changed_numb, replace_type );

}













void do_minimize_free_energy_single_angle_iteration( MOL_LIST *mol,
  RESID_LIST *resid, RESID_LIST *end_resid, TYPE_CELL *my_type, RTYPE_CELL *my_rtype,
  int changed_array[], STAT *angle, int *changed_numb, int replace_type )
{

  minimize_free_energy_each_resid( mol, resid, end_resid, my_type, my_rtype, *angle, 
    changed_array, changed_numb, replace_type );
    
}












void update_min_free_energy_single_iter( int changed_array[], 
  int curr_changed_array[], int curr_changed_numb, STAT *angle )
{

  int arrays_are_same( int a[], int b[] );

  if ( ! curr_changed_numb )
    *angle *= ANGLE_STEP;
    
  /*  In change.c:  */  
  else if ( arrays_are_same( changed_array, curr_changed_array ))
    *angle /= ANGLE_STEP;
    
  int i = 0;
  
  while ( curr_changed_array[ i ] != 0 ) {
    changed_array[ i ] = curr_changed_array[ i ];
    ++i;
  }
  
  changed_array[ i ] = 0;

}
















/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/










void minimize_free_energy_each_resid( MOL_LIST *mol, RESID_LIST *start_resid,
  RESID_LIST *end_resid, TYPE_CELL *my_type, RTYPE_CELL *my_rtype,
  STAT angle, int curr_changed_array[], int *curr_changed_numb, int replace_type )
{

  int index = 0;
  
  RESID_LIST *resid = start_resid;
  
  while (( resid != NULL ) && ( resid != end_resid ))
    minimize_free_energy_single_resid_lister_and_step( mol, &resid, my_type,
      my_rtype, curr_changed_array, curr_changed_numb, ++index, angle, 
      replace_type );

}












void minimize_free_energy_single_resid_lister_and_step( MOL_LIST *mol,
  RESID_LIST **resid, TYPE_CELL *my_type, RTYPE_CELL *my_rtype,
  int curr_changed_array[], int *curr_changed_numb, int index, STAT angle, 
  int replace_type )
{
  minimize_free_energy_single_resid_lister( mol, *resid, my_type, my_rtype, 
    curr_changed_array, curr_changed_numb, index, angle, replace_type );
  *resid = (*resid)->next;
}











void minimize_free_energy_single_resid_lister( MOL_LIST *mol, RESID_LIST *resid,
  TYPE_CELL *my_type, RTYPE_CELL *my_rtype, int curr_changed_array[], 
  int *curr_changed_numb, int index, STAT angle, int replace_type )
{

  switch ( resid->my_resid->mol_type ) {
  
  case MOL_TYPE_AA :
    
    minimize_free_energy_single_aa_resid_lister( mol, resid, my_type, my_rtype, 
      curr_changed_array, curr_changed_numb, index, angle, replace_type );
    
    break;
    
  case MOL_TYPE_ION :
  
    minimize_free_energy_zinc_ion_lister( mol, resid, my_type, my_rtype,
      curr_changed_array, curr_changed_numb, index, replace_type );
      
    break;
    
  case MOL_TYPE_NUCL :
    break;
    
  default :
    error( 4600, "Unknown molecule type.\n" );
    break;
  
  }

}













void minimize_free_energy_single_aa_resid_lister( MOL_LIST *mol, RESID_LIST *resid,
  TYPE_CELL *my_type, RTYPE_CELL *my_rtype, int curr_changed_array[], 
  int *curr_changed_numb, int index, STAT angle, int replace_type )
{

  STAT do_free_energy_mol_list( MOL_LIST *mol_header );

  STAT (*get_g)( MOL_LIST *mm );
  
  switch ( replace_type ) {
  
  case ORIG_REPLACE_TYPE :
    /*  In ../peptide/free_energy.c:  */
    get_g = do_free_energy_mol_list;
    break;
    
  case ANGLE_BIN_TYPE :
  case BIN_ZINC_TYPE :
    /*  In this file, below:  */
    get_g = do_free_energy_angle_bin;
    break;
    
  default :
    error( 4610, "Unknown replacement type mode.\n" );
    break;
  
  }
  
  /*  In this file, below:  */
  minimize_my_free_energy_single_aa_resid_lister( mol, resid, my_type, my_rtype,
    get_g, curr_changed_array, curr_changed_numb, index, angle );

}












void minimize_my_free_energy_single_aa_resid_lister( MOL_LIST *mol,
  RESID_LIST *resid, TYPE_CELL *my_type, RTYPE_CELL *my_rtype,
  STAT (*get_g)( MOL_LIST *mm ), int curr_changed_array[], int *curr_changed_numb, 
  int index, STAT angle )
{

  STAT phi_best = 0.0, psi_best = 0.0;
  STAT g = BIG;
  
  /*  In this file, down below:  */
  minimize_free_energy_aa_resid_single_comp( mol, resid, my_type, my_rtype, get_g, 
    &phi_best, &psi_best, 0.0, 0.0, &g );
    
  /*  In this file, below:  */
  minimize_free_energy_change_phi_psi( mol, resid, my_type, my_rtype, get_g,
    &phi_best, &psi_best, angle, &g );
    
  /*  In this file, below:  */  
  minimize_free_energy_double_phi_psi( mol, resid, my_type, my_rtype, get_g,
    &phi_best, &psi_best, angle, &g );
    
  /*  In this file, below:  */
  set_min_free_energy_single_aa_resid_lister( mol, resid, phi_best, psi_best,
    curr_changed_array, curr_changed_numb, index, angle );
    
}















void minimize_free_energy_change_phi_psi( MOL_LIST *mol, RESID_LIST *resid,
  TYPE_CELL *my_type, RTYPE_CELL *my_rtype, STAT (*get_g)( MOL_LIST *mm ),
  STAT *phi_best, STAT *psi_best, STAT angle, STAT *g )
{

  STAT angle_array[ ANGLE_BUFFER ] = { angle, -1.0 * angle };
  STAT holder;
  
  int i;
  
  for ( i = 0; i < ANGLE_BUFFER; i++ ) {
  
    minimize_free_energy_aa_resid_single_comp( mol, resid, my_type, my_rtype,
      get_g, phi_best, &holder, angle_array[ i ], 0.0, g );
      
    minimize_free_energy_aa_resid_single_comp( mol, resid, my_type, my_rtype,
      get_g, &holder, psi_best, 0.0, angle_array[ i ], g );
  
  }

}














void minimize_free_energy_double_phi_psi( MOL_LIST *mol, RESID_LIST *resid, 
  TYPE_CELL *my_type, RTYPE_CELL *my_rtype, STAT (*get_g)( MOL_LIST *mm ),
  STAT *phi_best, STAT *psi_best, STAT angle, STAT *g )
{

  if (( *phi_best == 0.0 ) || ( *psi_best == 0.0 ))
    return;
    
  minimize_my_free_energy_double_phi_psi( mol, resid, my_type, my_rtype, get_g,
    phi_best, psi_best, angle, g );

}













void minimize_my_free_energy_double_phi_psi( MOL_LIST *mol, RESID_LIST *resid,
  TYPE_CELL *my_type, RTYPE_CELL *my_rtype, STAT (*get_g)( MOL_LIST *mm ),
  STAT *phi_best, STAT *psi_best, STAT angle, STAT *g )
{

  STAT my_phi = *phi_best;
  STAT my_psi = *psi_best;

  minimize_free_energy_aa_resid_single_comp( mol, resid, my_type, my_rtype,
    get_g, phi_best, psi_best, my_phi, 0.0, g );
    
  minimize_free_energy_aa_resid_single_comp( mol, resid, my_type, my_rtype,
    get_g, phi_best, psi_best, 0.0, my_psi, g );
    
  minimize_free_energy_aa_resid_single_comp( mol, resid, my_type, my_rtype,
    get_g, phi_best, psi_best, my_phi, my_psi, g );

}















void set_min_free_energy_single_aa_resid_lister( MOL_LIST *mol, RESID_LIST *resid, 
  STAT phi_best, STAT psi_best, int curr_changed_array[], int *curr_changed_numb, 
  int index, STAT angle )
{

  int curr_index = 0;
  
  if ( phi_best != 0.0 ) {
    /*  In ../peptide/phi_psi.c:  */
    rotate_phi_angle( mol, resid, phi_best );
    curr_index = index;
  }
  
  if ( psi_best != 0.0 ) {
    /*  In ../peptide/phi_psi.c:  */
    rotate_psi_angle( mol, resid, psi_best );
    curr_index = index;
  }
  
  if ( curr_index ) {
  
    curr_changed_array[ (*curr_changed_numb)++ ] = curr_index;
    curr_changed_array[ *curr_changed_numb ] = 0;
    
    printf( "%.3f  (%d)  %s :    %8.3f  %8.3f\n", angle, 
      resid->my_resid->numb,
      resid->my_resid->resid,
      phi_best,
      psi_best );
  
  }

}











/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
















void minimize_free_energy_zinc_ion_lister( MOL_LIST *mol, RESID_LIST *resid, 
  TYPE_CELL *my_type, RTYPE_CELL *my_rtype, int curr_changed_array[], 
  int *curr_changed_numb, int index, int replace_type )
{

  if ( replace_type == BIN_ZINC_TYPE )
    minimize_my_free_energy_zinc_ion_lister( mol, resid );

}














void minimize_my_free_energy_zinc_ion_lister( MOL_LIST *mol, RESID_LIST *resid )
{

  STAT get_free_energy_zinc_ion( RESID_LIST *resid );

  /*  In ../peptide/free_energy.c:  */
  STAT curr_g = get_free_energy_zinc_ion( resid );
  
  /*  In this file, below:  */
  minimize_free_energy_move_zinc_ion( mol, resid, curr_g, ZINC_INIT );

}












void minimize_free_energy_move_zinc_ion( MOL_LIST *mol, RESID_LIST *zinc,
  STAT curr_g, STAT a )
{

  STAT array[ ZINC_ARRAY_BUFFER ] = { a, 0, -1.0 * a };
  
  if ( a < ZINC_THRESH )
    return;
   
  int changed = 0; 
    
  int xx, yy, zz;
  
  for ( xx = 0; xx < ZINC_ARRAY_BUFFER; xx++ ) {
  
    for ( yy = 0; yy < ZINC_ARRAY_BUFFER; yy++ ) {
    
      for ( zz = 0; zz < ZINC_ARRAY_BUFFER; zz++ ) {
      
        changed = minimize_free_energy_move_zinc( zinc, array[ xx ], array[ yy ],
          array[ zz ], &curr_g );
      
      }
    
    }
  
  }
  
  STAT my_a = a;
  
  if ( ! changed )
    my_a *= ZINC_STEP;
    
  minimize_free_energy_move_zinc_ion( mol, zinc, curr_g, my_a );

}










int minimize_free_energy_move_zinc( RESID_LIST *zinc, STAT x, STAT y, STAT z,
  STAT *g )
{

  XYZ pos = *(zinc->my_resid->atom_header->my_atom->xyz);
  
  pos.x += x;
  pos.y += y;
  pos.z += z;
  
  return minimize_my_free_energy_move_zinc( zinc, &pos, g );

}












int minimize_my_free_energy_move_zinc( RESID_LIST *zinc, XYZ *pos, STAT *g )
{

  STAT new_g = get_zinc_free_energy_xyz( zinc, pos );
  
  if ( new_g >= *g )
    return 0;
    
  *g = new_g;  
  *(zinc->my_resid->atom_header->my_atom->xyz) = *pos;
  
  printf( "New zinc position :  %.3f\n", *g );
  
  return 1;

}











STAT get_zinc_free_energy_xyz( RESID_LIST *zinc, XYZ *pos )
{

  ATOM_LIST *atom = zinc->my_resid->atom_header->my_atom->con_header;
  
  STAT g = 0.0;
  
  while ( atom != NULL )
    g += get_zinc_free_energy_single_atom_and_step( pos, &atom );
    
  return g;

}











STAT get_zinc_free_energy_single_atom_and_step( XYZ *pos, ATOM_LIST **atom )
{
  STAT g = get_zinc_free_energy_single_atom( pos, *atom );
  *atom = (*atom)->next;
  return g;
}












STAT get_zinc_free_energy_single_atom( XYZ *pos, ATOM_LIST *atom )
{

  STAT get_dist_xyz( XYZ *a, XYZ *b );
  
  /*  In ../main/xyz.c:  */
  STAT d = get_dist_xyz( pos, atom->my_atom->xyz );
  
  if ( ! strcmp( atom->my_atom->atom, "NE2" ))
    return (ZINC_K) * (d - ZINC_NE_DIST) * (d - ZINC_NE_DIST);
  
  if ( ! strcmp( atom->my_atom->atom, "SG" ))  
    return (ZINC_K) * (d - ZINC_SG_DIST) * (d - ZINC_SG_DIST);
    
  error( 3100, "Unrecognized atom type bound to zinc.\n" );

}













/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/













void minimize_free_energy_aa_resid_single_comp( MOL_LIST *mol, RESID_LIST *resid,
  TYPE_CELL *my_type, RTYPE_CELL *my_rtype, STAT (*get_g)( MOL_LIST *mm ), 
  STAT *phi_best, STAT *psi_best, STAT phi, STAT psi, STAT *g )
{

  MOL_LIST *copy_mol_lister( MOL_LIST *mm, MOL_LIST *mmm );
  
  MOL_LIST *new_mol = NULL;
  /*  In ../mm/mol.c:  */
  new_mol = (MOL_LIST *)copy_mol_lister( new_mol, mol );
  
  /*  In this file, below:  */
  minimize_my_free_energy_aa_resid_single_comp( new_mol, resid, my_type, my_rtype,
    get_g, phi_best, psi_best, phi, psi, g );
    
  kill_mol_list( new_mol );
  
}












void minimize_my_free_energy_aa_resid_single_comp( MOL_LIST *mol, RESID_LIST *resid,
  TYPE_CELL *my_type, RTYPE_CELL *my_rtype, STAT (*get_g)( MOL_LIST *mm ),
  STAT *phi_best, STAT *psi_best, STAT phi, STAT psi, STAT *g )
{

  /*  In ../peptide/atype.c:  */
  set_atom_types_mol_list( mol, my_type );
  set_resid_types_mol_list( mol, my_rtype );
  
  /*  In ../peptide/free_energy.c:  */
  set_mol_list_neighbors( mol );

  /*  In this file, below:  */
  minimize_free_energy_aa_resid_w_types( mol, resid, get_g, phi_best, psi_best,
    phi, psi, g );
    
}












void minimize_free_energy_aa_resid_w_types( MOL_LIST *mol, RESID_LIST *resid,
  STAT (*get_g)( MOL_LIST *mm ), STAT *phi_best, STAT *psi_best, STAT phi,
  STAT psi, STAT *g )
{

  RESID_LIST *get_resid_lister_w_numb( MOL_LIST *mol, int numb );

  RESID_LIST *my_resid = NULL;
  /*  In ../peptide/peptide.c:  */
  my_resid = (RESID_LIST *)get_resid_lister_w_numb( mol, resid->my_resid->numb );
  
  if ( phi != 0.0 )
    /*  In ../peptide/phi_psi.c:  */
    rotate_phi_angle( mol, my_resid, phi );
    
  if ( psi != 0.0 )
    /*  In ../peptide/phi_psi.c:  */
    rotate_psi_angle( mol, my_resid, psi );
    
  /*  Function pointer, in this file, above:  */  
  STAT gg = get_g( mol );
  
  if ( gg < *g ) {
    *phi_best = phi;
    *psi_best = psi;
    *g = gg;
  }

}














/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/















STAT do_free_energy_angle_bin( MOL_LIST *mol )
{

  STAT get_free_energy_mol_list_bin( MOL_LIST *mol_header );
  
  /*  In bin_change.c:  */
  return get_free_energy_mol_list_bin( mol );

}















/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/















void output_replacement_mol( MOL_LIST *mol, int aa_numb[], char aa_type[], 
  int aa_subs[], int first_first_resid, int replace_type )
{

  char output_fname[ BUFFER ];
  
  get_output_fname_replacement_mol( output_fname, aa_numb, aa_type, aa_subs,
    first_first_resid, replace_type );
    
  output_replacement_mol_fname( mol, output_fname );

}













void get_output_fname_replacement_mol( char output_fname[], int aa_numb[],
  char aa_type[], int aa_subs[], int first_first_resid, int replace_type )
{

  char dir[ BUFFER ], res[ BUFFER ], first[ BUFFER ];
  
  get_output_fname_dir_replacement_mol( dir, replace_type );
  
  get_output_fname_res_changes_mol( res, aa_numb, aa_type, aa_subs );
  
  get_output_fname_replacement_first( first, first_first_resid );
  
  sprintf( output_fname, "%s/1VA2.%s%s.txt", dir, res, first );
  
}












  
void get_output_fname_dir_replacement_mol( char dir[], int replace_type )
{
  
  switch ( replace_type ) {
  
  case ORIG_REPLACE_TYPE :

    strcpy( dir, MAIN_DIR );
    break;
    
  case ANGLE_BIN_TYPE :
    
    strcpy( dir, ANGLE_BIN_DIR );
    break;
    
  case BIN_ZINC_TYPE :
    
    strcpy( dir, ZINC_BIN_DIR );
    break;
    
  default :
    error( 4620, "Unknown replacement type mode.\n" );
    break;
  
  }
  
}













void get_output_fname_res_changes_mol( char res[], int aa_numb[], char aa_type[], 
  int aa_subs[] )
{

  if ( blank_array( aa_subs, CHANGED_BUFFER )) {
    strcpy( res, "NoSubs." );
    return;
  }
  
  res[ 0 ] = '\0';
  
  int i;
  
  for ( i = 0; i < CHANGED_BUFFER; i++ ) {
    if ( aa_subs[ i ] )
      sprintf( res, "%s%d%c.", res, aa_numb[ i ], aa_type[ i ] );
  }

}













int blank_array( int array[], int n )
{

  int i;
  
  for ( i = 0; i < n; i++ ) {
    if ( array[ i ] )
      return 0;
  }
  
  return 1;

}














void get_output_fname_replacement_first( char first[], int first_first_resid )
{
  sprintf( first, "first_%d", first_first_resid );
}















void output_replacement_mol_fname( MOL_LIST *mol, char output_fname[] )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( output_fname, "" );
  
  int atom_numb = 1;

  /*  In ../parse/pdb_print.c:  */
  print_mol_lister_pdb_fh( mol, 'A', 1, &atom_numb, out );

  close_file( out );

}















