#include "rmsd.h"















#define HG_INPUT "data/sp1/angle_bin/1VA2.NoSubs.rotated.txt"
#define BIRD_INPUT "data/sp1/angle_bin/1VA2.4I.rotated.txt"
#define ANC_INPUT "data/sp1/angle_bin/1VA2.4V.first_4.txt"

#define OUTPUT_FNAME "data/sp1/angle_bin/dists_rotated_mol.txt"













void do_rotated_mol_dists( void )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  
  MOL_LIST *hg_mol = NULL, *bird_mol = NULL, *anc_mol = NULL;
  /*  In ../parse/mol_list.c:  */
  hg_mol = (MOL_LIST *)get_mol_list_w_fname( hg_mol, HG_INPUT );
  bird_mol = (MOL_LIST *)get_mol_list_w_fname( bird_mol, BIRD_INPUT );
  anc_mol = (MOL_LIST *)get_mol_list_w_fname( anc_mol, ANC_INPUT );
  
  /*  In this file, below:  */
  do_my_rotate_mol_dists( hg_mol, bird_mol, anc_mol, OUTPUT_FNAME );

}













void do_my_rotate_mol_dists( MOL_LIST *hg_mol, MOL_LIST *bird_mol, 
  MOL_LIST *anc_mol, char fname[] )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( fname, "" );
  
  do_rotate_mol_dists_output_fh( hg_mol, bird_mol, anc_mol, out );
  
  close_file( out );

}














void do_rotate_mol_dists_output_fh( MOL_LIST *hg_mol, MOL_LIST *bird_mol,
  MOL_LIST *anc_mol, FILE *out )
{

  fprintf( out, "\"NUMB\"\t\"RES\"\t\"INDEX\"\t\"MI\"\t\"MV\"\t\"IV\"\n" );
  
  do_my_rotate_mol_dists_output_fh( hg_mol, bird_mol, anc_mol, out );
  
}











void do_my_rotate_mol_dists_output_fh( MOL_LIST *hg_mol, MOL_LIST *bird_mol,
  MOL_LIST *anc_mol, FILE *out )
{

  XYZ_CELL *create_xyz_cell( XYZ_CELL *xx, int nn );
  
  int nn = get_numb_aa_mol_list( hg_mol );
  
  XYZ_CELL *hg_cell = NULL, *bird_cell = NULL, *anc_cell = NULL;
  hg_cell = (XYZ_CELL *)create_xyz_cell( hg_cell, nn );
  bird_cell = (XYZ_CELL *)create_xyz_cell( bird_cell, nn );
  anc_cell = (XYZ_CELL *)create_xyz_cell( anc_cell, nn );
  
  do_rotate_mol_dists_w_cells( hg_mol, bird_mol, anc_mol, hg_cell, bird_cell,
    anc_cell, out );
    
  kill_xyz_cell( hg_cell );
  kill_xyz_cell( bird_cell );
  kill_xyz_cell( anc_cell );
  
}










int get_numb_aa_mol_list( MOL_LIST *mol )
{

  int i = 0;
  
  RESID_LIST *resid = mol->my_mol->resid_header;
  
  while ( resid != NULL ) {
  
    if ( resid->my_resid->mol_type == MOL_TYPE_AA )
      ++i;
      
    resid = resid->next;
  
  }
  
  return i;

}
















void do_rotate_mol_dists_w_cells( MOL_LIST *hg_mol, MOL_LIST *bird_mol, 
  MOL_LIST *anc_mol, XYZ_CELL *hg_cell, XYZ_CELL *bird_cell, XYZ_CELL *anc_cell, 
  FILE *out )
{

  /*  In comp.c:  */
  set_xyz_cell_carbon_alpha_atoms_index( hg_cell, hg_mol, 0 );
  set_xyz_cell_carbon_alpha_atoms_index( bird_cell, bird_mol, 0 );
  set_xyz_cell_carbon_alpha_atoms_index( anc_cell, anc_mol, 0 );
  
  /*  In this file, below:  */
  do_my_rotate_mol_dists_w_cells( hg_mol, hg_cell, bird_cell, anc_cell, out );

}














void do_my_rotate_mol_dists_w_cells( MOL_LIST *hg_mol, XYZ_CELL *hg_cell, 
  XYZ_CELL *bird_cell, XYZ_CELL *anc_cell, FILE *out )
{

  STAT get_dist_xyz( XYZ *a, XYZ *b );

  XYZ *hg = hg_cell->array, *bird = bird_cell->array, *anc = anc_cell->array;
  
  RESID_LIST *resid = hg_mol->my_mol->resid_header;
  
  int i = 0;
  
  while ( i++ < hg_cell->n ) {
  
    fprintf( out, "%d\t%c\t%d\t", i, resid->my_resid->aa, resid->my_resid->numb );
    
    /*  In ../main/xyz.c:  */
    fprintf( out, "%.3f\t%.3f\t%.3f\n",
      get_dist_xyz( hg, bird ),
      get_dist_xyz( hg, anc ),
      get_dist_xyz( bird, anc ) );
      
    resid = resid->next;
  
    ++hg;
    ++bird;
    ++anc;
  
  }

}











