#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include <stdarg.h>

















#define BUFFER 256
#define LINE_BUFFER 7000
#define SHORT_BUFFER 25
#define LABEL_BUFFER 7
#define SEQ_BUFFER 1000




#define X_INDEX 0
#define Y_INDEX 1
#define Z_INDEX 2
#define COORD_BUFFER 3
#define PARAM_TYPE_BUFFER 2



#define PI 3.14159265
#define PI_STEP 0.25

#define LARGE_NUMB 1.0e20


#define MOL_TYPE_UNSET 0
#define MOL_TYPE_AA 1
#define MOL_TYPE_NUCL 2
#define MOL_TYPE_ION 3


#define CO_BUFFER 3

#define NUMB_OF_AA 20
#define NUMB_OF_ANGLES 36






#define NUMB_OF_TYPES 25

#define T_C_NYL 0
#define T_C_OXL 1
#define T_C_1_H 2
#define T_C_2_H 3
#define T_C_3_H 4
#define T_C_ARO 5
#define T_N_PTN 6
#define T_N_DEP 7
#define T_N_NEU 8
#define T_N_LYS 9
#define T_N_ARG 10
#define T_N_PRO 11
#define T_O_HYD 12
#define T_O_NYL 13
#define T_O_XYL 14
#define T_S_SUL 15
#define T_N_BAC 16
#define T_C_CAL 17
#define T_C_BBC 18
#define T_O_BBO 19
#define T_Z_ZIN 20
#define T_H_POL 21
#define T_H_NON 22
#define T_H_ARO 23
#define T_H_BBH 24






typedef float COORD;
typedef double STAT;







/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/










typedef struct xyz_struct {
  COORD x;
  COORD y;
  COORD z;
} XYZ;







typedef struct xyz_cell_struct {
  XYZ *array;
  int n;
} XYZ_CELL;








typedef struct type_struct {
  STAT r;
  STAT e;
  char description[ BUFFER ];
  STAT r_pow[ SHORT_BUFFER ];
  unsigned acc : 1;
  unsigned don : 1;
} TYPE;








typedef struct type_cell_struct {
  TYPE *x;
  int n;
} TYPE_CELL;







typedef struct rtype_struct {
  char resid[ LABEL_BUFFER ];
  STAT phi_coeff[ CO_BUFFER ];
  STAT psi_coeff[ CO_BUFFER ];
  STAT phi_gamma;
  STAT psi_gamma;
  STAT phi_array[ NUMB_OF_ANGLES ];
  STAT psi_array[ NUMB_OF_ANGLES ];
} RTYPE;








typedef struct rtype_cell_struct {
  RTYPE *x;
  int n;
} RTYPE_CELL;





typedef struct atom_struct {
  char atom[ LABEL_BUFFER ];
  char type[ LABEL_BUFFER ];
  int numb;
  XYZ *xyz;
  TYPE *t;
  char line_end[ BUFFER ];
  char line_first[ LABEL_BUFFER ];
  char name[ LABEL_BUFFER ];
  struct atom_list_struct *con_header;
} ATOM;








typedef struct atom_list_struct {
  ATOM *my_atom;
  struct atom_list_struct *next;
} ATOM_LIST;









typedef struct resid_struct {
  char resid[ LABEL_BUFFER ];
  char aa;
  int numb;
  int mol_type;
  STAT phi;
  STAT psi;
  unsigned use : 1;
  ATOM_LIST *atom_header;
  RTYPE *type;
  struct resid_list_struct *neigh_header;
} RESID;









typedef struct resid_list_struct {
  RESID *my_resid;
  struct resid_list_struct *next;
} RESID_LIST;









typedef struct mol_struct {
  char mol[ LABEL_BUFFER ];
  char label_char;
  char seq[ SEQ_BUFFER ];
  RESID_LIST *resid_header;
} MOL;








typedef struct mol_list_struct {
  MOL *my_mol;
  struct mol_list_struct *next;
} MOL_LIST;







/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/










typedef struct slist_struct {
  char x[ BUFFER ];
  STAT f;
  int n;
  unsigned use :  1;
  struct slist_struct *next;
} SLIST;








typedef struct matrix_struct {
  STAT *x;
  int i;
  int j;
} MATRIX;








typedef struct mx_struct_struct {
  MATRIX *p;
  MATRIX *f;
  MATRIX *j;
  MATRIX *b;
  MATRIX *d;
  MATRIX *z;
} MX_STRUCT;







typedef struct rot_params_struct {
  STAT angle[ COORD_BUFFER ];
  STAT bef_angle[ COORD_BUFFER ];
  STAT aft_angle[ COORD_BUFFER ];
  STAT c[ COORD_BUFFER ];
  STAT major_trans[ COORD_BUFFER ];
  STAT minor_trans[ COORD_BUFFER ];
  STAT vector[ COORD_BUFFER ];
  STAT theta;
  STAT sum_of_squares;
} ROT_PARAMS;








typedef struct mm_struct {
  MOL_LIST *mol_header;
  int iter;
  STAT g;
  STAT rmsd;
} MM;






typedef struct mm_list_struct {
  MM *my_mm;
  struct mm_list_struct *next;
} MM_LIST;







typedef struct data_struct {
  MM_LIST *mm_header;
  TYPE_CELL *type;
} DATA;







/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/










char mode, minor_mode;
char input_file[ BUFFER ], minor_input_file[ BUFFER ];
char output_file[ BUFFER ];
char seq_param[ BUFFER ], minor_seq_param[ BUFFER ];

char major_mol, minor_mol;




















