#include "dyn_alloc.h"
























MOL_LIST *create_mol_lister( MOL_LIST *mm )
{

  MOL *mol = NULL;
  
  mol = (MOL *)malloc( sizeof(MOL) );
  mm = (MOL_LIST *)malloc( sizeof(MOL_LIST) );
  
  check_null( 2, mm, mol );
  
  return initialize_mol_lister( mm, mol );
  
}














RESID_LIST *create_resid_lister( RESID_LIST *rr )
{

  RESID *res = NULL;
  res = (RESID *)malloc( sizeof(RESID) );
  rr = (RESID_LIST *)malloc( sizeof(RESID_LIST) );
  
  check_null( 2, rr, res );
  
  return initialize_resid_lister( rr, res );

}













ATOM_LIST *create_atom_lister( ATOM_LIST *aa )
{

  ATOM *atom = NULL;
  XYZ *xyz = NULL;
  
  xyz = (XYZ *)malloc( sizeof(XYZ) );
  atom = (ATOM *)malloc( sizeof(ATOM) );
  aa = (ATOM_LIST *)malloc( sizeof(ATOM_LIST) );
  
  check_null( 2, aa, atom );
  
  return initialize_atom_lister( aa, atom, xyz );

}













XYZ_CELL *create_xyz_cell( XYZ_CELL *xx, int nn )
{

  XYZ *a = NULL;
  a = (XYZ *)malloc( sizeof(XYZ) * nn );
  xx = (XYZ_CELL *)malloc( sizeof(XYZ_CELL) );
  
  check_null( 2, xx, a );
  
  return initialize_xyz_cell( xx, a, nn );

}












MATRIX *create_matrix( MATRIX *xx, int ii, int jj )
{

  STAT *mx = NULL;
  mx = (STAT *)malloc( sizeof(STAT) * ii * jj );
  xx = (MATRIX *)malloc( sizeof(MATRIX) );
  
  check_null( 2, xx, mx );
  
  return initialize_matrix( xx, mx, ii, jj );

}











MM_LIST *create_mm_lister( MM_LIST *xx )
{

  MM *x = NULL;
  x = (MM *)malloc( sizeof(MM) );
  xx = (MM_LIST *)malloc( sizeof(MM_LIST) );
  
  check_null( 2, x, xx );
  
  return initialize_mm_lister( xx, x );

}












TYPE_CELL *create_type_cell( TYPE_CELL *x, int nn )
{

  TYPE *t = NULL;
  t = (TYPE *)malloc( sizeof(TYPE) * nn );
  x = (TYPE_CELL *)malloc( sizeof(TYPE_CELL) );
  
  check_null( 2, t, x );
  
  return initialize_type_cell( x, t, nn );

}












DATA *create_data( DATA *x )
{

  x = (DATA *)malloc( sizeof(DATA) );
  
  check_null( 1, x );
  
  return initialize_data( x );

}













SLIST *create_slister( SLIST *x )
{

  x = (SLIST *)malloc( sizeof(SLIST) );
  
  check_null( 1, x );
  
  return initialize_slister( x );

}













RTYPE_CELL *create_rtype_cell( RTYPE_CELL *x, int nn )
{

  RTYPE *xx = NULL;
  xx = (RTYPE *)malloc( sizeof(RTYPE) * nn );
  x = (RTYPE_CELL *)malloc( sizeof(RTYPE_CELL) );
  
  check_null( 2, x, xx );
  
  return initialize_rtype_cell( x, xx, nn );

}








/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/











MOL_LIST *initialize_mol_lister( MOL_LIST *mm, MOL *mol )
{

  mol->mol[ 0 ] = '\0';
  mol->seq[ 0 ] = '\0';
  mol->label_char = '\0';
  mol->resid_header = NULL;
  
  mm->my_mol = mol;
  mm->next = NULL;
  
  return mm;

}














RESID_LIST *initialize_resid_lister( RESID_LIST *rr, RESID *res )
{

  res->resid[ 0 ] = '\0';
  res->aa = '\0';
  res->numb = 0;
  res->mol_type = MOL_TYPE_UNSET;
  res->phi = 0.0;
  res->psi = 0.0;
  res->atom_header = NULL;
  res->neigh_header = NULL;
  res->type = NULL;
  
  rr->my_resid = res;
  rr->next = NULL;
  
  return rr;

}















ATOM_LIST *initialize_atom_lister( ATOM_LIST *aa, ATOM *atom, XYZ *xyz )
{

  xyz->x = 0.0;
  xyz->y = 0.0;
  xyz->z = 0.0;
  
  atom->atom[ 0 ] = '\0';
  atom->type[ 0 ] = '\0';
  atom->xyz = xyz;
  atom->line_end[ 0 ] = '\0';
  atom->line_first[ 0 ] = '\0';
  atom->con_header = NULL;
  
  aa->my_atom = atom;
  aa->next = NULL;
  
  return aa;

}












XYZ_CELL *initialize_xyz_cell( XYZ_CELL *xx, XYZ *array, int nn )
{

  XYZ *ptr = array;
  
  int i = 0;
  
  while ( i++ < nn ) {
  
    ptr->x = 0.0;
    ptr->y = 0.0;
    ptr->z = 0.0;
    
    ++ptr;
    
  }
  
  xx->array = array;
  xx->n = nn;
  
  return xx;

}













MATRIX *initialize_matrix( MATRIX *mx, STAT *x, int ii, int jj )
{

  mx->x = x;
  mx->i = ii;
  mx->j = jj;
  
  return mx;

}













MM_LIST *initialize_mm_lister( MM_LIST *xx, MM *x )
{

  x->mol_header = NULL;
  xx->my_mm = x;
  xx->next = NULL;
  
  return xx;

}












TYPE_CELL *initialize_type_cell( TYPE_CELL *x, TYPE *t, int nn )
{

  x->x = t;
  x->n = nn;
  
  char i = 0;
  TYPE *ptr = t;
  
  while ( i++ < x->n )
    initialize_type( ptr++ );
    
  return x;

}













void initialize_type( TYPE *t )
{
  t->r = 0.0;
  t->e = 0.0;
  t->acc = 0;
  t->don = 0;
}










DATA *initialize_data( DATA *x )
{

  x->mm_header = NULL;
  x->type = NULL;
  
  return x;

}














SLIST *initialize_slister( SLIST *x )
{

  x->x[ 0 ] = '\0';
  x->use = 0;
  x->f = 0.0;
  x->next = NULL;
  
  return x;

}












RTYPE_CELL *initialize_rtype_cell( RTYPE_CELL *x, RTYPE *xx, int nn )
{

  RTYPE *ptr = xx;

  int i;
  
  for ( i = 0; i < nn; i++ )
    initialize_rtype( ptr++ );
    
  x->x = xx;
  x->n = nn;
  
  return x;

}











void initialize_rtype( RTYPE *r )
{

  r->resid[ 0 ] = '\0';
  
  int i;
  
  for ( i = 0; i < CO_BUFFER; i++ ) {
    r->phi_coeff[ i ] = 0.0;
    r->psi_coeff[ i ] = 0.0;
  }
  
  r->phi_gamma = 0.0;
  r->psi_gamma = 0.0;

}









/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/















void kill_xyz_cell( XYZ_CELL *xx )
{

  if ( xx == NULL )
    return;
    
  free( xx->array );
  free( xx );

}











void kill_matrix( MATRIX *mx )
{

  if ( mx == NULL )
    return;
    
  free( mx->x );
  free( mx );

}












void kill_mol_list( MOL_LIST *header )
{

  if ( header == NULL )
    return;
    
  MOL_LIST *mol = header;
  
  while ( mol != NULL ) {
  
    MOL_LIST *next = mol->next;
    
    kill_mol_lister( mol );
    
    mol = next;
  
  }

}













void kill_mol_lister( MOL_LIST *mol )
{

  if ( mol == NULL )
    return;
    
  kill_resid_list( mol->my_mol->resid_header );
  
  free( mol->my_mol );
  free( mol );

}












void kill_resid_list( RESID_LIST *header )
{

  if ( header == NULL )
    return;
    
  RESID_LIST *resid = header;
  
  while ( resid != NULL ) {
  
    RESID_LIST *next = resid->next;
    
    kill_resid_lister( resid );
    
    resid = next;
  
  }

}












void kill_resid_lister( RESID_LIST *resid )
{

  if ( resid == NULL )
    return;
    
  kill_atom_list( resid->my_resid->atom_header );
  
  kill_resid_list_only( resid->my_resid->neigh_header );
  
  free( resid->my_resid );
  free( resid );

}












void kill_resid_list_only( RESID_LIST *header )
{

  if ( header == NULL )
    return;
    
  RESID_LIST *resid = header;
  
  while ( resid != NULL ) {
  
    RESID_LIST *next = resid->next;
    
    free( resid );
    
    resid = next;
    
  }

}











void kill_atom_list( ATOM_LIST *header )
{

  if ( header == NULL )
    return;
    
  ATOM_LIST *atom = header;
  
  while ( atom != NULL ) {
  
    ATOM_LIST *next = atom->next;
    
    kill_atom_lister( atom );
    
    atom = next;
  
  }

}












void kill_atom_lister( ATOM_LIST *atom )
{

  kill_atom_list_only( atom->my_atom->con_header );

  free( atom->my_atom->xyz );
  free( atom->my_atom );
  free( atom );

}











void kill_atom_list_only( ATOM_LIST *header )
{

  if ( header == NULL )
    return;

  ATOM_LIST *atom = header;
  
  while ( atom != NULL ) {
  
    ATOM_LIST *next = atom->next;
  
    free( atom );
    
    atom = next;
  
  }

}













void kill_slist( SLIST *header )
{

  if ( header == NULL )
    return;
    
  SLIST *x = header;
  
  while ( x != NULL ) {
  
    SLIST *next = x->next;
    
    free( x );
    
    x = next;
  
  }

}














void kill_type_cell( TYPE_CELL *t )
{

  if ( t == NULL )
    return;
    
  free( t->x );
  free( t );

}














void kill_rtype_cell( RTYPE_CELL *t )
{

  if ( t == NULL )
    return;
    
  free( t->x );
  free( t );

}











void kill_data( DATA *x )
{

  if ( x == NULL )
    return;
    
  free( x->type );
  free( x );

}







/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
















void check_null( int numb, ... )
{

  int i;
  
  va_list arg_ptr;
  
  va_start( arg_ptr, numb );
  
  for ( i = 0; i < numb; i++ ) 
    check_single_null( (void *)va_arg( arg_ptr, void * ) );
    
  va_end( arg_ptr );

}












void check_single_null( void *x )
{
  if ( x == NULL )
    error( 300, "Memory allocation exceeds buffer.\n" );
}
































