#include "deriv.h"
















#define DIFF_MX_DIM 20

#define H_VAL 0.01
#define THRESHOLD 1.0e-5




#define ABS(a) (((a) > 0.0) ? (a) : (-1.0 * (a)))


























STAT get_numerical_double_deriv( MATRIX *p, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell, int a, int b )
{

  MATRIX *create_matrix( MATRIX *mm, int ii, int jj );
  
  MATRIX *deriv_mx = NULL, *p_copy = NULL;
  deriv_mx = (MATRIX *)create_matrix( deriv_mx, DIFF_MX_DIM, DIFF_MX_DIM );
  p_copy = (MATRIX *)create_matrix( p_copy, p->i, p->j );
  
  STAT x = get_my_numerical_double_deriv( deriv_mx, p, p_copy, major_cell, 
    minor_cell, a, b );
    
  kill_matrix( deriv_mx );
  kill_matrix( p_copy );
  
  return x;

}













STAT get_my_numerical_double_deriv( MATRIX *mx, MATRIX *p, MATRIX *p_copy,
  XYZ_CELL *major_cell, XYZ_CELL *minor_cell, int a, int b )
{

  STAT h = H_VAL;
  
  *(mx->x) = get_numerical_double_deriv_single_elem( p, p_copy, major_cell, 
    minor_cell, a, b, h );

  int i, j;
  
  for ( i = 0; i < mx->i; i++ ) {
  
    h /= 2.0;

    *(mx->x + (i * mx->j)) = get_numerical_double_deriv_single_elem( p, 
      p_copy, major_cell, minor_cell, a, b, h );
      
    for ( j = 1; j <= i; j++ ) 
      *(mx->x + (i * mx->j) + j) = get_numerical_dd_single_mx_elem( mx, i, j );
      
    STAT diff = *(mx->x + (i * mx->j) + i) - *(mx->x + ((i-1) * mx->j) + (i-1)); 
    
    if ( ABS(diff) < THRESHOLD ) 
      return *(mx->x + (i * mx->j) + i);
      
  }
  
  error( 1700, "Unable to obtain derivative in maximum steps.\n" );
  
  return 0.0;

}














STAT get_numerical_double_deriv_single_elem( MATRIX *p, MATRIX *p_copy, 
  XYZ_CELL *major_cell, XYZ_CELL *minor_cell, int a, int b, STAT h )
{

  STAT get_single_deriv_rotation_xyz_cells( MATRIX *p, XYZ_CELL *major_cell,
    XYZ_CELL *minor_cell, int j );

  /*  In matrix.c:  */
  copy_matrix( p_copy, p );
  
  /*  In this file, below:  */
  update_matrix_single_deriv_step( p_copy, a, h );
  
  /*  In newton.c:  */
  STAT ff = get_single_deriv_rotation_xyz_cells( p_copy, major_cell, minor_cell, b );
  
  /*  In this file, below:  */
  update_matrix_single_deriv_step( p_copy, a, -2.0 * h );
  
  /*  In newton.c:  */
  STAT gg = get_single_deriv_rotation_xyz_cells( p_copy, major_cell, minor_cell, b );  
  
  return (ff - gg) / (2.0 * h);

}















void update_matrix_single_deriv_step( MATRIX *mx, int a, STAT h )
{
  *(mx->x + a) += h;
}












STAT get_numerical_dd_single_mx_elem( MATRIX *mx, int i, int j )
{

  STAT aa = *(mx->x + (i * mx->j) + (j-1));
  STAT bb = *(mx->x + ((i-1) * mx->j) + (j-1));
  
  float x = 1.0;
  register char y = 0;
  
  while ( y++ < j )
    x *= 4.0;
  
  return aa + ((aa - bb) / (x - 1.0));

}












STAT get_numerical_euclidean_double_deriv( MATRIX *p, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell, int a, int b )
{

  MATRIX *create_matrix( MATRIX *mm, int ii, int jj );
  
  MATRIX *deriv_mx = NULL, *p_copy = NULL;
  deriv_mx = (MATRIX *)create_matrix( deriv_mx, DIFF_MX_DIM, DIFF_MX_DIM );
  p_copy = (MATRIX *)create_matrix( p_copy, p->i, p->j );
  
  STAT x = get_my_numerical_euclidean_double_deriv( deriv_mx, p, p_copy, major_cell, 
    minor_cell, a, b );
    
  kill_matrix( deriv_mx );
  kill_matrix( p_copy );
  
  return x;

}













STAT get_my_numerical_euclidean_double_deriv( MATRIX *mx, MATRIX *p, MATRIX *p_copy,
  XYZ_CELL *major_cell, XYZ_CELL *minor_cell, int a, int b )
{

  STAT h = H_VAL;

  int i, j;
  
  /*  In this file, below:  */
  *(mx->x) = get_numerical_euclidean_double_deriv_single_elem( p, p_copy, 
    major_cell, minor_cell, a, b, h );
  
  for ( i = 1; i < mx->i; i++ ) {
  
    h /= 2.0;

    /*  In this file, below:  */
    *(mx->x + (i * mx->j)) = get_numerical_euclidean_double_deriv_single_elem( p, 
      p_copy, major_cell, minor_cell, a, b, h );
      
    for ( j = 1; j <= i; j++ ) 
      /*  In this file, above:  */
      *(mx->x + (i * mx->j) + j) = get_numerical_dd_single_mx_elem( mx, i, j );
      
    STAT diff = *(mx->x + (i * mx->j) + i) - *(mx->x + ((i-1) * mx->j) + (i-1)); 
    
    if ( ABS(diff) < THRESHOLD ) 
      return *(mx->x + (i * mx->j) + i);
      
  }
  
  error( 1700, "Unable to obtain derivative in maximum steps.\n" );
  
  return 0.0;

}














STAT get_numerical_euclidean_double_deriv_single_elem( MATRIX *p, MATRIX *p_copy, 
  XYZ_CELL *major_cell, XYZ_CELL *minor_cell, int a, int b, STAT h )
{

  STAT get_single_deriv_euclidean_rotation_xyz_cells( MATRIX *p, 
    XYZ_CELL *major_cell, XYZ_CELL *minor_cell, int j );

  /*  In matrix.c:  */
  copy_matrix( p_copy, p );
  
  /*  In this file, above:  */
  update_matrix_single_deriv_step( p_copy, a, h );
  
  /*  In newton.c:  */
  STAT ff = get_single_deriv_euclidean_rotation_xyz_cells( p_copy, major_cell, 
    minor_cell, b );
  
  /*  In this file, above:  */
  update_matrix_single_deriv_step( p_copy, a, -2.0 * h );
  
  /*  In newton.c:  */
  STAT gg = get_single_deriv_euclidean_rotation_xyz_cells( p_copy, major_cell, 
    minor_cell, b );  
  
  return (ff - gg) / (2.0 * h);

}










/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/















void get_gradient_mx( MATRIX *f, MATRIX *p, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell )
{

  STAT *fptr = f->x;
  
  int j = 0;
  
  while ( j < f->i )
    *fptr++ = get_gradient_mx_single_elem( p, major_cell, minor_cell, j++ );

}













STAT get_gradient_mx_single_elem( MATRIX *p, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell, int j )
{

  MATRIX *create_matrix( MATRIX *mm, int ii, int jj );
  
  MATRIX *mx = NULL, *p_copy = NULL;
  mx = (MATRIX *)create_matrix( mx, DIFF_MX_DIM, DIFF_MX_DIM );
  p_copy = (MATRIX *)create_matrix( p_copy, p->i, p->j );
  
  STAT x = get_gradient_mx_single_elem_w_mx( mx, p, p_copy, major_cell, minor_cell,
    j );
    
  kill_matrix( mx );
  kill_matrix( p_copy );
  
  return x;

}














STAT get_gradient_mx_single_elem_w_mx( MATRIX *mx, MATRIX *p, MATRIX *p_copy,
  XYZ_CELL *major_cell, XYZ_CELL *minor_cell, int j )
{

  STAT h = H_VAL;
  
  int a, b;
  
  for ( a = 0; a < mx->i; a++ ) {
  
    *(mx->x + (a * mx->j)) = get_gradient_deriv_elem( p, p_copy, major_cell,
      minor_cell, j, h );
      
    if ( ! a )
      continue;
      
    for ( b = 1; b <= a; b++ ) 
      /*  In this file, above:  */
      *(mx->x + (a * mx->j) + b) = get_numerical_dd_single_mx_elem( mx, a, b );
      
    STAT diff = *(mx->x + (a * mx->j) + a) - *(mx->x + ((a-1) * mx->j) + (a-1));  
      
    if ( ABS(diff) < THRESHOLD ) 
      return *(mx->x + (a * mx->j) + a);
      
    h /= 2.0;
  
  
  }
  
  return *(mx->x + ((mx->i - 1) * mx->i) + (mx->i - 1));

}














STAT get_gradient_deriv_elem( MATRIX *p, MATRIX *p_copy, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell, int j, STAT h )
{

  STAT get_sum_squares_param_mx( MATRIX *p, XYZ_CELL *major_pt, 
    XYZ_CELL *minor_pt );

  /*  In matrix.c:  */
  copy_matrix( p_copy, p );
  
  /*  In this file, below:  */
  update_matrix_single_deriv_step( p_copy, j, h );
  
  /*  In rotate.c:  */
  STAT ff = get_sum_squares_param_mx( p_copy, major_cell, minor_cell );
  
  /*  In this file, below:  */
  update_matrix_single_deriv_step( p_copy, j, -2.0 * h );
  
  /*  In rotate.c:  */
  STAT gg = get_sum_squares_param_mx( p_copy, major_cell, minor_cell );  
  
  return (ff - gg) / (2.0 * h);

}


























