#include "matrix.h"






















void get_matrix_inv( MATRIX *b, MATRIX *a )
{

  MATRIX *create_matrix( MATRIX *xx, int ii, int jj );
  
  MATRIX *vector = NULL, *sol = NULL;
  vector = (MATRIX *)create_matrix( vector, a->i, 1 );
  sol = (MATRIX *)create_matrix( sol, a->i, 1 );
  
  calc_inverse_matrix( b, a, vector, sol );
  
  kill_matrix( vector );
  kill_matrix( sol );

}












void calc_inverse_matrix( MATRIX *b, MATRIX *a, MATRIX *vector, MATRIX *sol )
{

  int col;
  
  for ( col = 0; col < a->i; col++ )
    compute_inv_column( b, a, vector, sol, col );

}













void compute_inv_column( MATRIX *b, MATRIX *a, MATRIX *vector, MATRIX *sol,
  int col )
{

  get_ident_vector_for_inv( vector, col );
  system_of_eq( sol, vector, a );
  put_column_into_matrix( b, sol, col );

}









void get_ident_vector_for_inv( MATRIX *vector, int col )
{

  STAT *ptr = vector->x;
  
  register int i = 0;
  
  while ( i++ < vector->i )
    *ptr++ = 0.0;
    
  *(vector->x + col) = 1.0;

}














void put_column_into_matrix( MATRIX *b, MATRIX *sol, int col )
{

  STAT *b_ptr = b->x + col;
  STAT *sol_ptr = sol->x;
  
  register int i = 0;
  
  while ( i++ < b->i ) {
    *b_ptr = *sol_ptr++;
    b_ptr += b->j;
  }

}












void system_of_eq( MATRIX *sol, MATRIX *vector, MATRIX *a )
{

  MATRIX *create_matrix( MATRIX *xx, int ii, int jj );
  
  MATRIX *l = NULL, *u = NULL;
  l = (MATRIX *)create_matrix( l, a->i, a->j );
  u = (MATRIX *)create_matrix( u, a->i, a->j );
  
  do_system_of_eq( sol, vector, a, l, u );
  
  kill_matrix( l );
  kill_matrix( u );

}












void do_system_of_eq( MATRIX *sol, MATRIX *vector, MATRIX *a, MATRIX *l, 
  MATRIX *u )
{
  set_diag_l( l );
  do_lu_decomp( l, u, a );
  do_my_system_of_eq( sol, l, u, vector );
}











void set_diag_l( MATRIX *l )
{

  int i, j;
  
  for ( i = 0; i < l->i; i++ ) {
    for ( j = 0; j < l->j; j++ ) {
      *(l->x + (i * l->j) + j) = ( i == j ) ? (1.0) : (0.0);
    }
  }

}












void do_lu_decomp( MATRIX *l, MATRIX *u, MATRIX *a )
{

  register int i, k;
  
  for ( k = 0; k < a->i; k++ ) {
  
    for ( i = 0; i <= k; i++ )
      set_u_element( l, u, a, i, k );
      
    for ( i = k + 1; i < a->i; i++ )
      set_l_element( l, u, a, i, k );
  
  }

}












void set_u_element( MATRIX *l, MATRIX *u, MATRIX *a, int i, int k )
{

  int index = (i * a->j) + k;
  
  *(u->x + index) = *(a->x + index);
  
  register int kk = 0;
  
  while ( kk < i )
    *(u->x + index) -= get_single_u_element_term( l, u, i, k, kk++ );

}












STAT get_single_u_element_term( MATRIX *l, MATRIX *u, int i, int k, int kk )
{
  return (*(l->x + (i * l->j) + kk)) * (*(u->x + (kk * u->j) + k));
}












void set_l_element( MATRIX *l, MATRIX *u, MATRIX *a, int i, int k )
{

  int index = (i * a->j) + k;
  
  *(l->x + index) = *(a->x + index);
  
  register int kk = 0;
  
  while ( kk < k )
    *(l->x + index) -= get_single_l_element_term( l, u, i, k, kk++ );
    
  *(l->x + index) = *(l->x + index) / *(u->x + (k * u->j) + k);

}













STAT get_single_l_element_term( MATRIX *l, MATRIX *u, int i, int k, int kk )
{
  return (*(l->x + (i * l->j) + kk)) * (*(u->x + (kk * u->j) + k));
}












void do_my_system_of_eq( MATRIX *sol, MATRIX *l, MATRIX *u, MATRIX *vector )
{

  MATRIX *create_matrix( MATRIX *mm, int ii, int jj );
  
  MATRIX *y = NULL;
  y = (MATRIX *)create_matrix( y, vector->i, 1 );
  
  do_system_of_eq_wyvec( sol, l, u, vector, y );
  
  kill_matrix( y );

}













void do_system_of_eq_wyvec( MATRIX *sol, MATRIX *l, MATRIX *u, MATRIX *vector,
  MATRIX *y )
{
  find_y_system_of_eq( y, l, vector );
  find_sol_vector_system_of_eq( sol, u, y );
}












void find_y_system_of_eq( MATRIX *y, MATRIX *l, MATRIX *vector )
{

  STAT *y_ptr = y->x, *vec_ptr = vector->x;
  
  *y_ptr++ = *vec_ptr++ / *(l->x);
  
  register int j, k = 1;
  
  do {
  
    *y_ptr = *vec_ptr++;
    
    STAT *y_ptr2 = y->x;
    
    for ( j = 0; j < k; j++ )
      *y_ptr -= *(l->x + (k * l->j) + j) * (*y_ptr2++);
      
    *y_ptr /= *(l->x + (k * l->j) + k);
    
    ++y_ptr;
  
  } while ( ++k < y->i );

}














void find_sol_vector_system_of_eq( MATRIX *sol, MATRIX *u, MATRIX *y )
{

  STAT *y_ptr = y->x + y->i - 1;
  STAT *sol_ptr = sol->x + sol->i - 1;
  
  *sol_ptr-- = (*y_ptr--) / (*(u->x + ((u->i - 1) * (u->i + 1))));
  
  register int j, k = sol->i - 2;
  
  do {
  
    *sol_ptr = *y_ptr--;
    
    STAT *sol_ptr2 = sol->x + sol->i - 1;
    
    for ( j = u->i - 1; j > k; j-- )
      *sol_ptr -= (*(u->x + (k * u->j) + j)) * (*sol_ptr2--);
      
    *sol_ptr /= *(u->x + (k * u->j) + k);
    
    --sol_ptr;
  
  } while ( --k >= 0 );

}










/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/












void multiply_matrices( MATRIX *c, MATRIX *a, MATRIX *b )
{

  if (( c->i != a->i ) || ( a->j != b->i ) || ( c->j != b->j ))
    error( 3330, "Matrix dimensions do not match.\n" );

  register int i, j, k;
  
  for ( i = 0; i < a->i; i++ ) {
  
    for ( j = 0; j < b->j; j++ ) {
    
      STAT x = 0.0;
      
      for ( k = 0; k < a->j; k++ )
        x += (*(a->x + (i * a->j) + k)) * (*(b->x + (k * b->j) + j));
        
      *(c->x + (i * c->j) + j) = x;
    
    }
  
  }

}













void multiply_scalar_matrix( MATRIX *a, STAT x )
{

  STAT *aptr = a->x;
  
  int end = a->i * a->j;
  
  register int i = 0;
  
  while ( i++ < end ) {
    *aptr *= x;
    ++aptr;
  }

}














void copy_matrix( MATRIX *b, MATRIX *a )
{

  STAT *a_ptr = a->x;
  STAT *b_ptr = b->x;
  
  int end = a->i * a->j;
  
  register int i = 0;
  
  while ( i++ < end )
    *b_ptr++ = *a_ptr++;

}













STAT dot_prod_vector_matrices( MATRIX *a, MATRIX *b )
{

  STAT x = 0.0;

  STAT *a_ptr = a->x;
  STAT *b_ptr = b->x;
  
  int end = a->i;
  
  register int i = 0;
  
  while ( i++ < end )
    x += (*a_ptr++) * (*b_ptr++);
    
  return x;

}












void add_matrices( MATRIX *c, MATRIX *a, MATRIX *b )
{

  STAT *a_ptr = a->x;
  STAT *b_ptr = b->x;
  STAT *c_ptr = c->x;
  
  int end = a->i * a->j;
  
  register int i = 0;
  
  while ( i++ < end )
    *c_ptr++ = (*a_ptr++) + (*b_ptr++);

}










void print_matrix( MATRIX *mx )
{

  print_sep( stdout );
  
  STAT *ptr = mx->x;
  
  register int i, j;
  
  for ( i = 0; i < mx->i; i++ ) {
  
    for ( j = 0; j < mx->j; j++ )
      printf( "%8.4f  ", *(mx->x + (i * mx->j) + j) );
      
    printf( "\n" );
  
  }
  
  print_sep( stdout );

}














