#include "newton.h"


















#define LAMBDA_STEP 0.5
#define LAMBDA_MIN 0.02

#define ALPHA 1.0e-4















void rotate_mol_single_iter_newton( MX_STRUCT *my_mx, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell )
{
  get_step_matrix_newton( my_mx, major_cell, minor_cell );
  update_vectors_newton( my_mx );
}











void get_step_matrix_newton( MX_STRUCT *my_mx, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell )
{

  void multiply_scalar_matrix( MATRIX *a, STAT x );
  
  /*  In this file, below:  */
  set_f_matrix_newton( my_mx, major_cell, minor_cell );
  
  /*  In this file, below:  */
  set_j_matrix_newton( my_mx, major_cell, minor_cell );
  
  /*  In matrix.c:  */
  get_matrix_inv( my_mx->b, my_mx->j );
  
  /*  In matrix.c:  */
  multiply_scalar_matrix( my_mx->f, -1.0 );
  
  /*  In matrix.c:  */
  multiply_matrices( my_mx->d, my_mx->b, my_mx->f );
  
}


















void update_vectors_newton( MX_STRUCT *my_mx )
{

  update_param_vectors_newton( my_mx->p, my_mx->d );

}











void update_param_vectors_newton( MATRIX *p, MATRIX *d )
{

  STAT *p_ptr = p->x, *d_ptr = d->x;

  register int i = 0;
  
  while ( i++ < p->i ) {
    *p_ptr += *d_ptr++;
    ++p_ptr;
  }

}














void rotate_mol_final_single_iter_newton( MX_STRUCT *my_mx, 
  XYZ_CELL *major_cell, XYZ_CELL *minor_cell )
{
  /*  In this file, below:  */
  get_step_matrix_final_newton( my_mx, major_cell, minor_cell );
  /*  In this file, above:  */
  update_vectors_newton( my_mx );
}












void get_step_matrix_final_newton( MX_STRUCT *my_mx, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell )
{

  void multiply_scalar_matrix( MATRIX *a, STAT x );

  /*  In this file, below:  */
  set_f_matrix_final_newton( my_mx, major_cell, minor_cell );
  
  /*  In this file, below:  */
  set_j_matrix_final_newton( my_mx, major_cell, minor_cell );
  
  /*  In matrix.c:  */
  get_matrix_inv( my_mx->b, my_mx->j );
  
  /*  In matrix.c:  */
  multiply_scalar_matrix( my_mx->f, -1.0 );
  
  /*  In matrix.c:  */
  multiply_matrices( my_mx->d, my_mx->b, my_mx->f );
  
}








/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/













void set_f_matrix_newton( MX_STRUCT *my_mx, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell )
{

  MATRIX *f = my_mx->f, *p = my_mx->p;

  STAT *fptr = f->x;

  register int j = 0;
  
  while ( j < f->i )
    set_f_matrix_newton_single_elem( fptr++, j++, p, major_cell, minor_cell );

}












void set_f_matrix_newton_single_elem( STAT *fptr, int j, MATRIX *p,
  XYZ_CELL *major_cell, XYZ_CELL *minor_cell )
{

  *fptr = 0.0;
  
  XYZ *major_pt = major_cell->array;
  XYZ *minor_pt = minor_cell->array;
  
  register int i = 0;
  
  while ( i++ < major_cell->n )
    *fptr += get_single_deriv_rotation( p, major_pt++, minor_pt++, j );

}












STAT get_single_deriv_rotation( MATRIX *p, XYZ *major_pt, XYZ *minor_pt, int j )
{

  STAT rotate( XYZ *point, MATRIX *p, char index );
  STAT get_squares_xyz_coord_and_mx( MATRIX *p, XYZ *major_coord, XYZ *minor_coord );
  
  /*  In rotate.c:  */
  STAT x_term = major_pt->x - rotate( minor_pt, p, X_INDEX );
  STAT y_term = major_pt->y - rotate( minor_pt, p, Y_INDEX );
  STAT z_term = major_pt->z - rotate( minor_pt, p, Z_INDEX );
  
  /*  In this file, below:  */
  STAT x_der = get_rotation_deriv( minor_pt, p, X_INDEX, j );
  STAT y_der = get_rotation_deriv( minor_pt, p, Y_INDEX, j );
  STAT z_der = get_rotation_deriv( minor_pt, p, Z_INDEX, j );
  
  /*  In rotate.c:  */
  return -2.0 * (( x_term * x_der ) + ( y_term * y_der ) + ( z_term * z_der ));

}















STAT get_single_deriv_rotation_xyz_cells( MATRIX *p, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell, int j )
{

  STAT x = 0.0;
  
  XYZ *major_pt = major_cell->array, *minor_pt = minor_cell->array;
  
  register int i = 0;
  
  while ( i++ < major_cell->n )
    /*  In this file, above:  */
    x += get_single_deriv_rotation( p, major_pt++, minor_pt++, j );
    
  return x;

}














void set_f_matrix_final_newton( MX_STRUCT *my_mx, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell )
{

  MATRIX *f = my_mx->f, *p = my_mx->p;

  STAT *fptr = f->x;

  register int j = 0;
  
  while ( j < f->i )
    set_f_matrix_final_newton_single_elem( fptr++, j++, p, major_cell, minor_cell );

}












void set_f_matrix_final_newton_single_elem( STAT *fptr, int j, MATRIX *p,
  XYZ_CELL *major_cell, XYZ_CELL *minor_cell )
{

  *fptr = 0.0;
  
  XYZ *major_pt = major_cell->array;
  XYZ *minor_pt = minor_cell->array;
  
  register int i = 0;
  
  while ( i++ < major_cell->n )
    *fptr += get_single_deriv_euclidean_rotation( p, major_pt++, minor_pt++, j );

}












STAT get_single_deriv_euclidean_rotation( MATRIX *p, XYZ *major_pt, XYZ *minor_pt, 
  int j )
{

  STAT rotate( XYZ *point, MATRIX *p, char index );
  STAT get_squares_xyz_coord_and_mx( MATRIX *p, XYZ *major_coord, XYZ *minor_coord );
  
  /*  In rotate.c:  */
  STAT x_term = major_pt->x - rotate( minor_pt, p, X_INDEX );
  STAT y_term = major_pt->y - rotate( minor_pt, p, Y_INDEX );
  STAT z_term = major_pt->z - rotate( minor_pt, p, Z_INDEX );
  
  /*  In this file, below:  */
  STAT x_der = get_rotation_deriv( minor_pt, p, X_INDEX, j );
  STAT y_der = get_rotation_deriv( minor_pt, p, Y_INDEX, j );
  STAT z_der = get_rotation_deriv( minor_pt, p, Z_INDEX, j );
  
  /*  In rotate.c:  */
  STAT dd = get_squares_xyz_coord_and_mx( p, major_pt, minor_pt );

  return -1.0 * (( x_term * x_der ) + ( y_term * y_der ) + ( z_term * z_der )) / dd;

}










STAT get_single_deriv_euclidean_rotation_xyz_cells( MATRIX *p, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell, int j )
{

  STAT x = 0.0;
  
  XYZ *major_pt = major_cell->array, *minor_pt = minor_cell->array;
  
  register int i = 0;
  
  while ( i++ < major_cell->n )
    /*  In this file, above:  */
    x += get_single_deriv_euclidean_rotation( p, major_pt++, minor_pt++, j );
    
  return x;

}












void set_j_matrix_newton( MX_STRUCT *my_mx, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell )
{

  MATRIX *j = my_mx->j;

  register int a, b;
  
  for ( a = 0; a < j->i; a++ ) {
  
    for ( b = 0; b < j->j; b++ ) {
    
      *(j->x + (a * j->j) + b) = get_j_matrix_newton_single_elem( my_mx->p, 
        major_cell, minor_cell, a, b );
    
    }
  
  }  

}















STAT get_j_matrix_newton_single_elem( MATRIX *p, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell, int a, int b )
{

  STAT get_numerical_double_deriv( MATRIX *p, XYZ_CELL *major_cell, 
    XYZ_CELL *minor_cell, int a, int b );

  /*  In deriv.c:  */
  return get_numerical_double_deriv( p, major_cell, minor_cell, a, b );

}












void set_j_matrix_final_newton( MX_STRUCT *my_mx, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell )
{

  MATRIX *j = my_mx->j;

  register int a, b;
  
  for ( a = 0; a < j->i; a++ ) {
  
    for ( b = 0; b < j->j; b++ ) {
    
      *(j->x + (a * j->j) + b) = get_j_matrix_final_newton_single_elem( my_mx->p, 
        major_cell, minor_cell, a, b );
    
    }
  
  }  

}















STAT get_j_matrix_final_newton_single_elem( MATRIX *p, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell, int a, int b )
{

  STAT get_numerical_double_deriv( MATRIX *p, XYZ_CELL *major_cell, 
    XYZ_CELL *minor_cell, int a, int b );

  /*  In deriv.c:  */
  return get_numerical_euclidean_double_deriv( p, major_cell, minor_cell, a, b );

}










STAT get_double_deriv_notation( MATRIX *p, XYZ *major_pt, XYZ *minor_pt,
  int a, int b )
{

  STAT x_term = get_rotation_deriv( minor_pt, p, X_INDEX, a );
  x_term *= get_rotation_deriv( minor_pt, p, X_INDEX, b );
  
  STAT y_term = get_rotation_deriv( minor_pt, p, Y_INDEX, a );
  y_term *= get_rotation_deriv( minor_pt, p, Y_INDEX, b );
  
  STAT z_term = get_rotation_deriv( minor_pt, p, Z_INDEX, a );
  z_term *= get_rotation_deriv( minor_pt, p, Z_INDEX, b );
  
  return 2.0 * (x_term + y_term + z_term);

}











STAT get_rotation_deriv( XYZ *pt, MATRIX *p, char index, int j )
{

  switch ( j ) {
  
  case X_INDEX :
    return get_rotation_deriv_x( pt, p, index );
    
  case Y_INDEX :
    return get_rotation_deriv_y( pt, p, index );
    
  case Z_INDEX :
    return get_rotation_deriv_z( pt, p, index );
    
  case X_INDEX + COORD_BUFFER :
    return ( index == X_INDEX ) ? (1.0) : (0.0);
  
  case Y_INDEX + COORD_BUFFER :
    return ( index == Y_INDEX ) ? (1.0) : (0.0);
  
  case Z_INDEX + COORD_BUFFER :
    return ( index == Z_INDEX ) ? (1.0) : (0.0);
  
  default :
    error( 3320, "Unrecognized parameter index value.\n" );
    return 0.0;
  
  }

}










STAT get_rotation_deriv_x( XYZ *pt, MATRIX *p, char index )
{

  STAT t_x = *(p->x + X_INDEX);
  STAT t_y = *(p->x + Y_INDEX);
  STAT t_z = *(p->x + Z_INDEX);
  
  STAT xx = pt->x;
  STAT yy = pt->y;
  STAT zz = pt->z;
  
  STAT x_coeff = 0.0;
  STAT y_coeff = 0.0;
  STAT z_coeff = 0.0;

  switch ( index ) {
  
  case X_INDEX :
    y_coeff = ( cos(t_x) * sin(t_y) * cos(t_z)) + ( sin(t_x) * sin(t_z) );
    z_coeff = ( -1.0 * sin(t_x) * sin(t_y) * cos(t_z)) + ( cos(t_x) * sin(t_z));    
    break;
    
  case Y_INDEX :
    y_coeff = (cos(t_x) * sin(t_y) * sin(t_z)) - (sin(t_x) * cos(t_z));
    z_coeff = (-1.0 * sin(t_x) * sin(t_y) * sin(t_z)) - (cos(t_x) * cos(t_z));
    break;
    
  case Z_INDEX :
    y_coeff = cos(t_x) * cos(t_y);
    z_coeff = -1.0 * sin(t_x) * cos(t_y);
    break;
    
  default :
    error( 3500, "Unexpected coordinate index value.\n" );
    break;
  
  }
  
  return (x_coeff * xx) + (y_coeff * yy) + (z_coeff * zz);

}












STAT get_rotation_deriv_y( XYZ *pt, MATRIX *p, char index )
{

  STAT t_x = *(p->x + X_INDEX);
  STAT t_y = *(p->x + Y_INDEX);
  STAT t_z = *(p->x + Z_INDEX);
  
  STAT xx = pt->x;
  STAT yy = pt->y;
  STAT zz = pt->z;
  
  STAT x_coeff = 0.0;
  STAT y_coeff = 0.0;
  STAT z_coeff = 0.0;

  switch ( index ) {
  
  case X_INDEX :
    x_coeff = -1.0 * sin(t_y) * cos(t_z);
    y_coeff = cos(t_y) * sin(t_x) * cos(t_z);
    z_coeff = cos(t_y) * cos(t_x) * cos(t_z);
    break;
    
  case Y_INDEX :
    x_coeff = -1.0 * sin(t_y) * sin(t_z);
    y_coeff = cos(t_y) * sin(t_x) * sin(t_z);
    z_coeff = cos(t_y) * cos(t_x) * sin(t_z);
    break;
    
  case Z_INDEX :
    x_coeff = -1.0 * cos(t_y);
    y_coeff = -1.0 * sin(t_y) * sin(t_x);
    z_coeff = -1.0 * sin(t_y) * cos(t_x);
    break;
    
  default :
    error( 3500, "Unexpected coordinate index value.\n" );
    break;
  
  }
  
  return (x_coeff * xx) + (y_coeff * yy) + (z_coeff * zz);

}












STAT get_rotation_deriv_z( XYZ *pt, MATRIX *p, char index )
{

  STAT t_x = *(p->x + X_INDEX);
  STAT t_y = *(p->x + Y_INDEX);
  STAT t_z = *(p->x + Z_INDEX);
  
  STAT xx = pt->x;
  STAT yy = pt->y;
  STAT zz = pt->z;
  
  STAT x_coeff = 0.0;
  STAT y_coeff = 0.0;
  STAT z_coeff = 0.0;

  switch ( index ) {
  
  case X_INDEX :
    x_coeff = -1.0 * sin(t_z) * cos(t_y);
    y_coeff = (-1.0 * sin(t_z) * sin(t_x) * sin(t_y)) - (cos(t_z) * cos(t_x));
    z_coeff = (-1.0 * sin(t_z) * cos(t_x) * sin(t_y)) + (cos(t_z) * sin(t_x));
    break;
    
  case Y_INDEX :
    x_coeff = cos(t_z) * cos(t_y);
    y_coeff = (cos(t_z) * sin(t_x) * sin(t_y)) - (sin(t_z) * cos(t_x));
    z_coeff = (cos(t_z) * cos(t_x) * sin(t_y)) + (sin(t_z) * sin(t_x));
    break;
    
  case Z_INDEX :
    break;
    
  default :
    error( 3500, "Unexpected coordinate index value.\n" );
    break;
  
  }
  
  return (x_coeff * xx) + (y_coeff * yy) + (z_coeff * zz);

}































