#include "rotate.h"












#define ABS(a) (((a) > 0.0) ? (a) : (-1.0 * (a)))



#define MAX_ITERATIONS 25

#define SUM_OF_SQ_THRESH 1.0e-4
#define DOT_PROD_THRESH 1.0e-4















void rotate_mol( ROT_PARAMS *my_param, XYZ_CELL *major_cell, XYZ_CELL *minor_cell )
{

  get_and_set_centered_cells( my_param, major_cell, minor_cell );
  
  rotate_my_mol( my_param, major_cell, minor_cell );

}











void get_and_set_centered_cells( ROT_PARAMS *my_param, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell )
{

  get_centered_xyz_coords( my_param->major_trans, major_cell );
  get_centered_xyz_coords( my_param->minor_trans, minor_cell );
  
  set_centered_xyz_coords( major_cell, my_param->major_trans );
  set_centered_xyz_coords( minor_cell, my_param->minor_trans );

}












void get_centered_xyz_coords( STAT trans[], XYZ_CELL *my_cell )
{
  trans[ X_INDEX ] = get_expected_val_coord_index( my_cell, X_INDEX );
  trans[ Y_INDEX ] = get_expected_val_coord_index( my_cell, Y_INDEX );
  trans[ Z_INDEX ] = get_expected_val_coord_index( my_cell, Z_INDEX );
}













STAT get_expected_val_coord_index( XYZ_CELL *my_cell, char index )
{

  STAT x = 0.0;
  
  XYZ *ptr = my_cell->array;
  
  register int i = 0;
  
  while ( i++ < my_cell->n )
    x += get_val_xyz_coord_index( ptr++, index );
    
  return (x) / (STAT)(my_cell->n);

}












STAT get_val_xyz_coord_index( XYZ *x, char index )
{

  switch ( index ) {
  
  case X_INDEX :
    return x->x;
    
  case Y_INDEX :
    return x->y;
    
  case Z_INDEX :
    return x->z;
    
  default :
    error( 3300, "Unrecognized coordinate index.\n" );
    break;
  
  }

}













void set_centered_xyz_coords( XYZ_CELL *my_cell, STAT trans[] )
{

  XYZ *ptr = my_cell->array;

  register int i = 0;
  
  while ( i++ < my_cell->n )
    center_single_xyz_coord_trans_array( ptr++, trans );

}












void center_single_xyz_coord_trans_array( XYZ *x, STAT trans[] )
{
  x->x -= trans[ X_INDEX ];
  x->y -= trans[ Y_INDEX ];
  x->z -= trans[ Z_INDEX ];
}














/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/














void rotate_my_mol( ROT_PARAMS *my_param, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell )
{

  MATRIX *create_matrix( MATRIX *xx, int ii, int jj );
  
  MX_STRUCT my_mx = { .p = NULL, .f = NULL, .j = NULL, .b = NULL, .d = NULL,
    .z = NULL };
  
  int nn = COORD_BUFFER * PARAM_TYPE_BUFFER;
  
  my_mx.p = (MATRIX *)create_matrix( my_mx.p, nn, 1 );
  my_mx.f = (MATRIX *)create_matrix( my_mx.f, nn, 1 );
  my_mx.j = (MATRIX *)create_matrix( my_mx.j, nn, nn );
  my_mx.b = (MATRIX *)create_matrix( my_mx.b, nn, nn );
  my_mx.d = (MATRIX *)create_matrix( my_mx.d, nn, 1 );
  my_mx.z = (MATRIX *)create_matrix( my_mx.z, nn, 1 );
  
  rotate_mol_w_matrices( my_param, &my_mx, major_cell, minor_cell );
  
  kill_matrix( my_mx.p );
  kill_matrix( my_mx.f );
  kill_matrix( my_mx.j );
  kill_matrix( my_mx.b );
  kill_matrix( my_mx.d );
  kill_matrix( my_mx.z );

}












void rotate_mol_w_matrices( ROT_PARAMS *my_param, MX_STRUCT *my_mx, 
  XYZ_CELL *major_cell, XYZ_CELL *minor_cell )
{

  rotate_mol_matrix_guess( my_mx, major_cell, minor_cell );
  
  my_param->sum_of_squares = rotate_mol_matrix_final( my_mx, major_cell, 
    minor_cell );
    
  set_rot_params_from_matrix( my_param, my_mx->z );
  
}












void set_rot_params_from_matrix( ROT_PARAMS *my_param, MATRIX *mx )
{

  char i;
  
  for ( i = 0; i < COORD_BUFFER; i++ ) {
    my_param->angle[ i ] = *(mx->x + i);
    my_param->c[ i ] = *(mx->x + i + COORD_BUFFER);
  }

}










void rotate_mol_matrix_guess( MX_STRUCT *my_mx, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell )
{

  STAT step = PI * 0.2;

  STAT theta_x, theta_y, theta_z;
  
  for ( theta_x = -1.0 * PI; theta_x < PI; theta_x += step ) {
  
    for ( theta_y = -1.0 * PI; theta_y < PI; theta_y += step ) {
    
      for ( theta_z = -1.0 * PI; theta_z < PI; theta_z += step ) {
    
        rotate_mol_single_initial_params( my_mx, major_cell, minor_cell, theta_x, 
          theta_y, theta_z );
    
      }
    
    }
  
  }
  
}












void rotate_mol_single_initial_params( MX_STRUCT *my_mx, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell, STAT theta_x, STAT theta_y, STAT theta_z )
{

  set_initial_mx_theta_vals( my_mx->p, theta_x, theta_y, theta_z );  
  
  STAT sum_of_squares = rotate_my_mol_single_initial_params( my_mx, major_cell, 
    minor_cell );  
    
}











void set_initial_mx_theta_vals( MATRIX *mx, STAT theta_x, STAT theta_y,
  STAT theta_z )
{

  *(mx->x + X_INDEX) = theta_x;
  *(mx->x + Y_INDEX) = theta_y;
  *(mx->x + Z_INDEX) = theta_z;
  
  *(mx->x + X_INDEX + COORD_BUFFER) = 0.0;
  *(mx->x + Y_INDEX + COORD_BUFFER) = 0.0;
  *(mx->x + Z_INDEX + COORD_BUFFER) = 0.0;

}












STAT rotate_my_mol_single_initial_params( MX_STRUCT *my_mx, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell )
{

  STAT sum_of_squares = get_sum_of_squares_matrix( my_mx->z, major_cell, 
    minor_cell );
  
  int converged = 0;

  int iter = 0;
  
  while (( ! converged ) && ( iter++ < MAX_ITERATIONS ))
    converged = rotate_mol_single_iteration( my_mx, major_cell, minor_cell, 
      &sum_of_squares );
      
  return sum_of_squares;
  
}  
      












int rotate_mol_single_iteration( MX_STRUCT *my_mx, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell, STAT *sum_of_squares )
{

  STAT new_ss = rotate_my_mol_single_iteration( my_mx, major_cell, minor_cell );
  
  return set_new_params_rotation_iteration( my_mx, new_ss, sum_of_squares );
  
}











STAT rotate_my_mol_single_iteration( MX_STRUCT *my_mx, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell )
{

  /*  In newton.c:  */
  rotate_mol_single_iter_newton( my_mx, major_cell, minor_cell );
  
  /*  In this file, below:  */
  return get_sum_of_squares_mx_and_cell( my_mx, major_cell, minor_cell );

}













int set_new_params_rotation_iteration( MX_STRUCT *my_mx, STAT new_ss, 
  STAT *sum_of_squares )
{

  if ( non_number_mx_struct( my_mx ))
    return 1;  
    
  int conv = iteration_has_converged( my_mx, new_ss, *sum_of_squares );
  
  if ( new_ss < *sum_of_squares ) {
    set_my_new_params_rotation_iteration( my_mx );
    *sum_of_squares = new_ss;
  }
  
  return conv;
  
}














int non_number_mx_struct( MX_STRUCT *my_mx )
{

  if ( non_number_mx( my_mx->p ))
    return 1;
   
  return 0;

}














int non_number_mx( MATRIX *mx )
{

  STAT *ptr = mx->x;
  
  int end = mx->i * mx->j;
  
  register int i = 0;
  
  while ( i++ < end ) {
  
    if ( isnan( *ptr ) || isinf( *ptr ))
      return 1;
      
    ++ptr;
  
  }
  
  return 0;

}













int iteration_has_converged( MX_STRUCT *my_mx, STAT new_ss, STAT sum_of_squares )
{

  STAT dot_prod_vector_matrices( MATRIX *a, MATRIX *b );

  if ( ABS( sum_of_squares - new_ss ) < SUM_OF_SQ_THRESH ) 
    return 1;
    
  /*  In matrix.c:  */  
  STAT f_dot = dot_prod_vector_matrices( my_mx->f, my_mx->f );
  STAT d_dot = dot_prod_vector_matrices( my_mx->d, my_mx->d );

  /*  In matrix.c:  */
  if (( f_dot < DOT_PROD_THRESH ) || ( d_dot < DOT_PROD_THRESH )) 
    return 1;
        
  return 0;
  
}













void set_my_new_params_rotation_iteration( MX_STRUCT *my_mx )
{

  STAT *z_ptr = my_mx->z->x, *p_ptr = my_mx->p->x;
  
  register int i = 0;
  
  while ( i++ < my_mx->p->i )
    *z_ptr++ = *p_ptr++;

}













/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/















STAT rotate_mol_matrix_final( MX_STRUCT *my_mx, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell )
{

  /*  In matrix.c:  */
  copy_matrix( my_mx->p, my_mx->z );
  
  print_matrix( my_mx->p );
  
  /*  In this file, below:  */
  return rotate_my_mol_matrix_final( my_mx, major_cell, minor_cell );

}













STAT rotate_my_mol_matrix_final( MX_STRUCT *my_mx, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell )
{

  STAT sum_of_squares = get_sum_of_squares_mx_and_cell_euclidean( my_mx, major_cell, 
    minor_cell );
  
  int converged = 0;

  int iter = 0;
  
  while (( ! converged ) && ( iter++ < MAX_ITERATIONS ))
    converged = rotate_mol_final_single_iteration( my_mx, major_cell, minor_cell, 
      &sum_of_squares );
      
  printf( "%.4f\n\n", sum_of_squares );
  
  return sum_of_squares;
    
}










int rotate_mol_final_single_iteration( MX_STRUCT *my_mx, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell, STAT *sum_of_squares )
{

  /*  In this file, below:  */
  STAT new_ss = rotate_my_mol_final_single_iteration( my_mx, major_cell, 
    minor_cell );
    
  /*  In this file, above:  */
  return set_new_params_rotation_iteration( my_mx, new_ss, sum_of_squares );

}










STAT rotate_my_mol_final_single_iteration( MX_STRUCT *my_mx, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell )
{

  /*  In newton.c:  */
  rotate_mol_final_single_iter_newton( my_mx, major_cell, minor_cell );
  
  /*  In this file, below:  */
  return get_sum_of_squares_mx_and_cell_euclidean( my_mx, major_cell, minor_cell );

}








/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/










STAT get_sum_of_squares_mx_and_cell( MX_STRUCT *my_mx, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell )
{

  STAT sum = 0.0;
  
  register int i = 0;
  
  XYZ *major_ptr = major_cell->array, *minor_ptr = minor_cell->array;
  
  while ( i++ < major_cell->n )
    sum += get_squares_xyz_coord_and_mx( my_mx->p, major_ptr++, minor_ptr++ );
    
  return sum;

}













STAT get_sum_of_squares_matrix( MATRIX *mx, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell )
{

  STAT sum = 0.0;
  
  register int i = 0;
  
  XYZ *major_pt = major_cell->array, *minor_pt = minor_cell->array;
  
  while ( i++ < major_cell->n )
    sum += get_squares_xyz_coord_and_mx( mx, major_pt++ , minor_pt++ );
    
  return sum;

}










STAT get_squares_xyz_coord_and_mx( MATRIX *p, XYZ *major_coord, XYZ *minor_coord )
{

  STAT x_diff = major_coord->x - rotate( minor_coord, p, X_INDEX );
  STAT y_diff = major_coord->y - rotate( minor_coord, p, Y_INDEX );
  STAT z_diff = major_coord->z - rotate( minor_coord, p, Z_INDEX );

  return ( x_diff * x_diff ) + ( y_diff * y_diff ) + ( z_diff * z_diff );
  
}













STAT get_sum_squares_param_mx( MATRIX *p, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell )
{

  STAT sum = 0.0;
  
  register int i = 0;
  
  XYZ *major_ptr = major_cell->array, *minor_ptr = minor_cell->array;
  
  while ( i++ < major_cell->n )
    sum += get_squares_xyz_coord_and_mx( p, major_ptr++, minor_ptr++ );
    
  return sum;

}











STAT get_sum_of_squares_mx_and_cell_euclidean( MX_STRUCT *my_mx, 
  XYZ_CELL *major_cell, XYZ_CELL *minor_cell )
{

  STAT sum = 0.0;
  
  register int i = 0;
  
  XYZ *major_ptr = major_cell->array, *minor_ptr = minor_cell->array;
  
  while ( i++ < major_cell->n )
    /*  In this file, above:  */
    sum += sqrt( get_squares_xyz_coord_and_mx( my_mx->p, major_ptr++, 
      minor_ptr++ ));
    
  return sum;

}

















/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/












STAT rotate( XYZ *point, MATRIX *p, char index )
{

  switch ( index ) {
  
  case X_INDEX :
    return rotate_x( point, p );
    
  case Y_INDEX :
    return rotate_y( point, p );
    
  case Z_INDEX :
    return rotate_z( point, p );
    
  default :
    error( 3310, "Unrecognized coordinate index.\n" );
    break;
  }

}











STAT rotate_x( XYZ *point, MATRIX *p )
{

  STAT t_x = *(p->x + X_INDEX);
  STAT t_y = *(p->x + Y_INDEX);
  STAT t_z = *(p->x + Z_INDEX);

  STAT xx = point->x * cos( t_y ) * cos( t_z );
  STAT yy = point->y * 
    (( sin( t_x ) * sin( t_y ) * cos( t_z )) - ( cos( t_x ) * sin( t_z )));
  STAT zz = point->z *
    (( cos( t_x ) * sin( t_y ) * cos( t_z )) + ( sin( t_x ) * sin( t_z )));
    
  return xx + yy + zz + *(p->x + X_INDEX + COORD_BUFFER);

}










STAT rotate_y( XYZ *point, MATRIX *p )
{

  STAT t_x = *(p->x + X_INDEX);
  STAT t_y = *(p->x + Y_INDEX);
  STAT t_z = *(p->x + Z_INDEX);

  STAT xx = point->x * cos( t_y ) * sin( t_z );
  STAT yy = point->y * 
    (( sin( t_x ) * sin( t_y ) * sin( t_z )) + ( cos( t_x ) * cos( t_z )));
  STAT zz = point->z *
    (( cos( t_x ) * sin( t_y ) * sin( t_z )) - ( sin( t_x ) * cos( t_z )));
    
  return xx + yy + zz + *(p->x + Y_INDEX + COORD_BUFFER);

}












STAT rotate_z( XYZ *point, MATRIX *p )
{

  STAT t_x = *(p->x + X_INDEX);
  STAT t_y = *(p->x + Y_INDEX);
  STAT t_z = *(p->x + Z_INDEX);

  STAT xx = point->x * ( -1.0 * sin( t_y ));
  STAT yy = point->y * ( sin( t_x ) * cos( t_y ));
  STAT zz = point->z * ( cos( t_x ) * cos( t_y ));
  
  return xx + yy + zz + *(p->x + Z_INDEX + COORD_BUFFER);
  
}


































