#include "sub_nucl.h"



















#define NUMB_BACKBONE_ATOMS 6
#define NUMB_BACKBONE_ATOMS_TOTAL 11
















MOL_LIST *substitute_nucl_in_mol_list( MOL_LIST *dna_header, char for_major_label,
  char rev_major_label, MOL_LIST *dna_minor_header, char for_minor_label,
  char rev_minor_label, int change_nucl, int from_nucl, int nucl_offset )
{

  ROT_PARAMS my_param;
  
  get_nucl_substitution_rotation_params( &my_param, dna_header, for_major_label,
    rev_major_label, dna_minor_header, for_minor_label, rev_minor_label,
    change_nucl, from_nucl, nucl_offset );

  return rotate_nucl_substitution( dna_header, &my_param, for_major_label,
    rev_major_label, dna_minor_header, for_minor_label, rev_minor_label, change_nucl,
    from_nucl, nucl_offset );

}















void get_nucl_substitution_rotation_params( ROT_PARAMS *my_param, 
  MOL_LIST *dna_header, char for_major_label, char rev_major_label,
  MOL_LIST *dna_minor_header, char for_minor_label, char rev_minor_label,
  int change_nucl, int from_nucl, int nucl_offset )
{

  XYZ_CELL *create_xyz_cell( XYZ_CELL *xx, int nn );
  
  XYZ_CELL *major_cell = NULL, *minor_cell = NULL;
  major_cell = (XYZ_CELL *)create_xyz_cell( major_cell, NUMB_BACKBONE_ATOMS * 2 );
  minor_cell = (XYZ_CELL *)create_xyz_cell( minor_cell, NUMB_BACKBONE_ATOMS * 2 );
    
  /*  In this file, below:  */  
  get_xyz_cell_nucl_backbone( major_cell, dna_header, for_major_label, 
    rev_major_label, change_nucl, nucl_offset );
    
  /*  In this file, below:  */  
  get_xyz_cell_nucl_backbone( minor_cell, dna_minor_header, for_minor_label,
    rev_minor_label, from_nucl, nucl_offset );
    
  /*  In rotate.c:  */
  rotate_mol( my_param, major_cell, minor_cell );

  kill_xyz_cell( major_cell );
  kill_xyz_cell( minor_cell );

}













void get_xyz_cell_nucl_backbone( XYZ_CELL *my_cell, MOL_LIST *dna_header,
  char for_major_label, char rev_major_label, int nucl_numb, int nucl_offset )
{

  int total_major_resid = get_total_residues_mol_list( dna_header, for_major_label );
  int total_minor_resid = get_total_residues_mol_list( dna_header, rev_major_label );
  
  if ( total_major_resid != total_minor_resid )
    error( 2301, "Number of residues in DNA do not match.\n" );
    
  RESID_LIST *for_resid = NULL, *rev_resid = NULL;
    
  for_resid = (RESID_LIST *)get_residue_mol_list_by_numb( dna_header, 
    for_major_label, nucl_numb );
    
  rev_resid = (RESID_LIST *)get_residue_mol_list_by_numb( dna_header,
    rev_major_label, total_minor_resid - nucl_numb + nucl_offset - 1 );
    
  get_xyz_cell_nucl_backbone_resid_pair( my_cell, for_resid, rev_resid );
  
}














int get_total_residues_mol_list( MOL_LIST *header, char label_char )
{

  MOL_LIST *mol = get_mol_lister_label_char( header, label_char );
  
  return total_residues_mol_lister( mol );

}













MOL_LIST *get_mol_lister_label_char( MOL_LIST *header, char label_char )
{

  MOL_LIST *mol = header;
  
  while ( mol != NULL ) {
  
    if ( mol->my_mol->label_char == label_char )
      return mol;
      
    mol = mol->next;
  
  }
  
  error( 2300, "Could not find molecule label character.\n" );

}













int total_residues_mol_lister( MOL_LIST *mol )
{

  int total = 0;
  
  RESID_LIST *resid = mol->my_mol->resid_header;
  
  while ( resid != NULL ) {
  
    ++total;
    
    resid = resid->next;
  
  }
  
  return total;

}
















RESID_LIST *get_residue_mol_list_by_numb( MOL_LIST *dna_header, char label_char,
  int numb )
{

  /*  In this file, above:  */
  MOL_LIST *mol = get_mol_lister_label_char( dna_header, label_char );
  
  /*  In this file, below:  */
  return get_residue_mol_single_lister_by_numb( mol, numb );

}











RESID_LIST *get_residue_mol_single_lister_by_numb( MOL_LIST *dna_header, int numb )
{

  RESID_LIST *resid = dna_header->my_mol->resid_header;
  
  int i = 0;
  
  while ( resid != NULL ) {
  
    if ( numb == i++ )
      return resid;
      
    resid = resid->next;
  
  }

}














void get_xyz_cell_nucl_backbone_resid_pair( XYZ_CELL *my_cell, RESID_LIST *for_resid, 
  RESID_LIST *rev_resid )
{

  char *atom_array[ NUMB_BACKBONE_ATOMS ] =
    { "C4'", "O4'", "C3'", "O3'", "C2'", "C1'" };
    
  set_xyz_cell_array_nucl_backbone_atoms( my_cell->array, for_resid, atom_array );
  set_xyz_cell_array_nucl_backbone_atoms( my_cell->array + NUMB_BACKBONE_ATOMS,
    rev_resid, atom_array );
    
  XYZ *a = my_cell->array + 1, *b = my_cell->array + NUMB_BACKBONE_ATOMS + 1;
  
  STAT x = a->x - b->x, y = a->y - b->y, z = a->z - b->z;
  STAT dist = sqrt( x*x + y*y + z*z );
  
  printf( "Distance :  %.3f  (%s, %s)\n", dist, for_resid->my_resid->resid,
    rev_resid->my_resid->resid );
  
}














void set_xyz_cell_array_nucl_backbone_atoms( XYZ *array, RESID_LIST *resid,
  char *atom_array[] )
{

  XYZ *xyz = array;

  int index;
  
  for ( index = 0; index < NUMB_BACKBONE_ATOMS; index++ )
    set_xyz_array_elem_nucl_backbone( xyz++, resid, atom_array[ index ] );

}














void set_xyz_array_elem_nucl_backbone( XYZ *xyz, RESID_LIST *resid, char *x )
{

  ATOM_LIST *atom = resid->my_resid->atom_header;
  
  while ( atom != NULL ) {
  
    if ( ! strcmp( atom->my_atom->atom, x )) 
      *xyz = *(atom->my_atom->xyz);
    
    atom = atom->next;
  
  }

}















/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/















MOL_LIST *rotate_nucl_substitution( MOL_LIST *dna_header, ROT_PARAMS *my_param, 
  char for_major_label, char rev_major_label, MOL_LIST *dna_minor_header, 
  char for_minor_label, char rev_minor_label, int change_nucl, int from_nucl, 
  int nucl_offset )
{

  /*  In this file, above:  */
  int total_resid = get_total_residues_mol_list( dna_header, for_major_label );

  /*  In this file, below:  */
  dna_header = (MOL_LIST *)rotate_single_nucl_substitution( dna_header, 
    dna_minor_header, my_param, for_major_label, for_minor_label, change_nucl,
    from_nucl );
  
  /*  In this file, below:  */  
  dna_header = (MOL_LIST *)rotate_single_nucl_substitution( dna_header,
    dna_minor_header, my_param, rev_major_label, rev_minor_label,
    total_resid - change_nucl + nucl_offset - 1,
    total_resid - from_nucl + nucl_offset - 1 );

  /*  In this file, below:  */
  return set_resid_list_order_mol_list( dna_header );

}













MOL_LIST *rotate_single_nucl_substitution( MOL_LIST *dna_header,
  MOL_LIST *dna_minor_header, ROT_PARAMS *my_param, char major_label,
  char minor_label, int change_nucl, int from_nucl )
{

  /*  In this file, above:  */
  MOL_LIST *major_dna = get_mol_lister_label_char( dna_header, major_label );  
  MOL_LIST *minor_dna = get_mol_lister_label_char( dna_minor_header, minor_label );
  
  /*  In this file, below:  */
  return rotate_single_nucl_sub_mol_lister( dna_header, major_dna, minor_dna, 
    my_param, change_nucl, from_nucl );

}













MOL_LIST *rotate_single_nucl_sub_mol_lister( MOL_LIST *dna_header,
  MOL_LIST *major_dna, MOL_LIST *minor_dna, ROT_PARAMS *my_param, int change_nucl,
  int from_nucl )
{

  RESID_LIST *to_resid = NULL, *from_resid = NULL, *new_resid = NULL;
  
  /*  In this file, above:  */  
  to_resid = (RESID_LIST *)get_residue_mol_single_lister_by_numb( minor_dna, 
    change_nucl );

  /*  In this file, above:  */
  from_resid = (RESID_LIST *)get_residue_mol_single_lister_by_numb( minor_dna,
    from_nucl );
      
  /*  In this file, below:  */    
  new_resid = (RESID_LIST *)get_substitution_dna_resid_lister( new_resid, to_resid, 
    from_resid, my_param );
      
  /*  In this file, below:  */    
  substitute_new_resid_into_mol_list( major_dna, new_resid, change_nucl );
  
  return dna_header;

}












RESID_LIST *get_substitution_dna_resid_lister( RESID_LIST *new_resid, 
  RESID_LIST *to_resid, RESID_LIST *from_resid, ROT_PARAMS *my_param )
{

  RESID_LIST *copy_resid_lister( RESID_LIST *new_resid, RESID_LIST *resid );
  
  /*  In mol.c:  */
  new_resid = (RESID_LIST *)copy_resid_lister( new_resid, from_resid );
  
  printf( "\t\t\t\t\t%s\n", new_resid->my_resid->resid );
  
  /*  In this file, below:  */
  return rotate_substitution_dna_resid_lister( new_resid, to_resid, my_param );
  
}













RESID_LIST *rotate_substitution_dna_resid_lister( RESID_LIST *new_resid,
  RESID_LIST *to_resid, ROT_PARAMS *my_param )
{

  ATOM_LIST *atom = new_resid->my_resid->atom_header;
  
  while ( atom != NULL )
    new_resid = (RESID_LIST *)rotate_sub_dna_single_atom_lister_and_step( new_resid, 
      &atom, to_resid, my_param );
    
  return new_resid;

}














RESID_LIST *rotate_sub_dna_single_atom_lister_and_step( RESID_LIST *new_resid, 
  ATOM_LIST **atom, RESID_LIST *to_resid, ROT_PARAMS *my_param )
{

  new_resid = (RESID_LIST *)rotate_substitution_dna_single_atom_lister( new_resid,
    *atom, to_resid, my_param );

  *atom = (*atom)->next;
  
  return new_resid;

}













RESID_LIST *rotate_substitution_dna_single_atom_lister( RESID_LIST *new_resid, 
  ATOM_LIST *atom, RESID_LIST *to_resid, ROT_PARAMS *my_param )
{

  if ( atom_is_in_dna_backbone( atom )) 
    return set_atom_to_same_atom_in_atom_list( new_resid, atom, 
      to_resid->my_resid->atom_header );

  new_resid = (RESID_LIST *)rotate_my_sub_dna_single_atom_lister( new_resid, atom, 
    my_param );
    
  ATOM_LIST *old_atom = 
    (ATOM_LIST *)get_atom_lister_atom_str( to_resid->my_resid->atom_header, 
    atom->my_atom->atom );
  
  if ( old_atom == NULL )
    return new_resid;
      
  STAT x = atom->my_atom->xyz->x - old_atom->my_atom->xyz->x;
  STAT y = atom->my_atom->xyz->y - old_atom->my_atom->xyz->y;
  STAT z = atom->my_atom->xyz->z - old_atom->my_atom->xyz->z;

  if ( ! strcmp( atom->my_atom->atom, "C2'" ))
    printf( "dist :  %.3f\n", sqrt( x*x + y*y + z*z ) );
    
  return new_resid;

}













int atom_is_in_dna_backbone( ATOM_LIST *atom )
{

  char *array[ NUMB_BACKBONE_ATOMS_TOTAL ] =
    { "P", "OP1", "OP2", "O5'", "C5'", "C4'", "O4'", "C3'", "O3'", "C2'", "C1'" };
  
  int i;
  
  for ( i = 0; i < NUMB_BACKBONE_ATOMS_TOTAL; i++ ) {
    
    if ( ! strcmp( array[ i ], atom->my_atom->atom ))
      return 1;
    
  }  
  
  return 0;

}











RESID_LIST *set_atom_to_same_atom_in_atom_list( RESID_LIST *resid, ATOM_LIST *atom, 
  ATOM_LIST *atom_header )
{

  ATOM_LIST *old_atom = NULL;
  old_atom = (ATOM_LIST *)get_atom_lister_atom_str( atom_header, 
    atom->my_atom->atom );
  
  if ( old_atom == NULL )
    return delete_atom_from_resid_lister_by_str( resid, atom->my_atom->atom );

  *(atom->my_atom->xyz) = *(old_atom->my_atom->xyz);
  
  return resid;
  
}  
  
  












ATOM_LIST *get_atom_lister_atom_str( ATOM_LIST *atom_header, char atom_type[] )
{

  ATOM_LIST *atom = atom_header;
  
  while ( atom != NULL ) {
  
    if ( ! strcmp( atom->my_atom->atom, atom_type ))
      return atom;
      
    atom = atom->next;
  
  }
  
  return NULL;

}











RESID_LIST *delete_atom_from_resid_lister_by_str( RESID_LIST *resid, 
  char atom_type[] )
{

  ATOM_LIST *atom = resid->my_resid->atom_header, *prev = NULL;
  
  if ( ! strcmp( atom->my_atom->atom, atom_type )) {
    resid->my_resid->atom_header = atom->next;
    kill_atom_lister( atom );
    return resid;
  }
  
  while (( atom != NULL ) && ( strcmp( atom->my_atom->atom, atom_type ))) {
    prev = atom;
    atom = atom->next;
  }
  
  prev->next = atom->next;

  kill_atom_lister( atom );
  
  return resid;

}
















RESID_LIST *rotate_my_sub_dna_single_atom_lister( RESID_LIST *new_resid, 
  ATOM_LIST *atom, ROT_PARAMS *my_param )
{

  XYZ xyz = { .x = atom->my_atom->xyz->x, .y = atom->my_atom->xyz->y,
    .z = atom->my_atom->xyz->z };
    
  /*  In mol.c:  */
  rotate_xyz_rot_params( atom->my_atom->xyz, &xyz, my_param );
  
  return new_resid;

}














void substitute_new_resid_into_mol_list( MOL_LIST *dna_mol, RESID_LIST *new_resid, 
  int numb )
{

  if ( ! numb ) {
    substitute_new_resid_into_head_mol_list( dna_mol, new_resid );
    return;
  }
  
  RESID_LIST *curr = dna_mol->my_mol->resid_header, *prev = NULL;
  
  int i = 0;
  
  while ( i++ < numb ) {
  
    if ( curr == NULL )
      error( 2302, "Residue number surpasses length of list.\n" );

    prev = curr;
    curr = curr->next;

  }
  
  prev->next = new_resid;
  new_resid->next = curr->next;
  
  kill_resid_lister( curr );
  
}










void substitute_new_resid_into_head_mol_list( MOL_LIST *dna_mol, 
  RESID_LIST *new_resid )
{  

  RESID_LIST *to_delete = dna_mol->my_mol->resid_header;

  dna_mol->my_mol->resid_header = new_resid;

  new_resid->next = to_delete->next;

  kill_resid_lister( to_delete );

}















MOL_LIST *set_resid_list_order_mol_list( MOL_LIST *mol_header )
{

  MOL_LIST *mol = mol_header;
  
  while ( mol != NULL )
    set_resid_list_order_single_mol_lister_and_step( &mol );
    
  return mol_header;

}













void set_resid_list_order_single_mol_lister_and_step( MOL_LIST **mol )
{
  set_resid_list_order_single_mol_lister( *mol );
  *mol = (*mol)->next;
}














void set_resid_list_order_single_mol_lister( MOL_LIST *mol )
{

  RESID_LIST *resid = mol->my_mol->resid_header;
  
  int numb = resid->my_resid->numb;
  
  while ( resid != NULL ) {
  
    resid->my_resid->numb = numb++;
    
    resid = resid->next;
  
  }

}



















