#include "zf.h"
















#define ZF1_INPUT_FNAME "data/sp1/1VA1.txt"
#define ZF2_INPUT_FNAME "data/sp1/1VA2.txt"
#define ZF3_INPUT_FNAME "data/sp1/1VA3.txt"
#define ZIF268_INPUT_FNAME "data/zif268/pdb1p47.txt"

#define OUTPUT_FNAME "data/sp1/sp1_dna_complex.changedNucl.txt"







#define AA_ZIF268_1_SUBSTRING "RSDELTR"
#define AA_ZF1_SUBSTRING "KTSHLRA"

#define AA_ZIF268_2_SUBSTRING "RSDHLTT"
#define AA_ZF2_SUBSTRING "RSDELQR"

#define AA_ZIF268_3_SUBSTRING "RSDERKR"
#define AA_ZF3_SUBSTRING "RSDHLSK"

#define ZIF268_MOL_INDEX 'B'
#define ZF1_MOL_INDEX 'A'
#define ZF2_MOL_INDEX 'A'
#define ZF3_MOL_INDEX 'A'



#define NUCL_1_INDEX 19
#define NUCL_1_FROM_INDEX 18
#define NUCL_2_INDEX 16
#define NUCL_2_FROM_INDEX 19
#define NUCL_3_INDEX 15
#define NUCL_3_FROM_INDEX 14
#define NUCL_4_INDEX 13
#define NUCL_4_FROM_INDEX 12

#define DNA_SEQ_OFFSET 1  //  The reverse strand DNA is shifted over 1 nucl bp















void do_zinc_finger_struct_align( void )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  
  MOL_LIST *zif268_header = NULL;
  MOL_LIST *zf_header = NULL, *dna_header = NULL;
  
  /*  In ../parse/mol_list.c:  */
  zif268_header = (MOL_LIST *)get_mol_list_w_fname( zif268_header, 
    ZIF268_INPUT_FNAME );
  
  /*  In this file, below:  */  
  zf_header = (MOL_LIST *)get_zinc_finger_mol_list( zf_header, zif268_header );
  dna_header = (MOL_LIST *)get_zinc_finger_dna_mol_list( dna_header, zif268_header );
  
  /*  In this file, below:  */
  print_zinc_finger_to_fname( zf_header, dna_header, OUTPUT_FNAME );
  
}


















MOL_LIST *get_zinc_finger_mol_list( MOL_LIST *zf_header, MOL_LIST *zif268_header )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  MOL_LIST *get_aligned_struct_w_mol_lists( MOL_LIST *new_mol, MOL_LIST *mol_header,
    MOL_LIST *minor_header, char major_substr[], char minor_substr[], char major_mol,
    char minor_mol );
  
  MOL_LIST *zf1_minor = NULL, *zf2_minor = NULL, *zf3_minor = NULL;
  MOL_LIST *zf1_major = NULL, *zf2_major = NULL, *zf3_major = NULL;
  
  /*  In ../parse/mol_list.c:  */
  zf1_minor = (MOL_LIST *)get_mol_list_w_fname( zf1_minor, ZF1_INPUT_FNAME );
  zf2_minor = (MOL_LIST *)get_mol_list_w_fname( zf2_minor, ZF2_INPUT_FNAME );
  zf3_minor = (MOL_LIST *)get_mol_list_w_fname( zf3_minor, ZF3_INPUT_FNAME );
  
  /*  In mol.c:  */
  zf1_major = (MOL_LIST *)get_aligned_struct_w_mol_lists( zf1_major, zif268_header,
    zf1_minor, AA_ZIF268_1_SUBSTRING, AA_ZF1_SUBSTRING, ZIF268_MOL_INDEX, 
    ZF1_MOL_INDEX );
    
  /*  In mol.c:  */
  zf2_major = (MOL_LIST *)get_aligned_struct_w_mol_lists( zf2_major, zif268_header,
    zf2_minor, AA_ZIF268_2_SUBSTRING, AA_ZF2_SUBSTRING, ZIF268_MOL_INDEX, 
    ZF2_MOL_INDEX );
    
  /*  In mol.c:  */
  zf3_major = (MOL_LIST *)get_aligned_struct_w_mol_lists( zf3_major, zif268_header,
    zf3_minor, AA_ZIF268_3_SUBSTRING, AA_ZF3_SUBSTRING, ZIF268_MOL_INDEX, 
    ZF3_MOL_INDEX );
    
  kill_mol_list( zf1_minor );
  kill_mol_list( zf2_minor );
  kill_mol_list( zf3_minor );
    
  zf1_major->next = zf2_major;
  zf2_major->next = zf3_major;
  
  return zf1_major;
    
}















MOL_LIST *get_zinc_finger_dna_mol_list( MOL_LIST *dna_header, 
  MOL_LIST *zif268_header )
{

  MOL_LIST *copy_mol_lister_w_label_char( MOL_LIST *major, MOL_LIST *minor_header, 
    char label_char );

  /*  In mol.c:  */
  dna_header = (MOL_LIST *)copy_mol_lister_w_label_char( dna_header, zif268_header,  
    'C' );
    
  /*  In mol.c:  */  
  dna_header->next = (MOL_LIST *)copy_mol_lister_w_label_char( dna_header->next,
    zif268_header, 'D' );

  /*  In this file, below:  */
  return get_my_zinc_finger_dna_mol_list( dna_header, zif268_header );  
  
}














MOL_LIST *get_my_zinc_finger_dna_mol_list( MOL_LIST *dna_header,
  MOL_LIST *zif268_header )
{

  MOL_LIST *substitute_nucl_in_mol_list( MOL_LIST *dna_header, char for_major_label,
    char rev_major_label, MOL_LIST *dna_minor_header, char for_minor_label,
    char rev_minor_label, int change_nucl, int from_nucl, int nucl_offset );

  /*  In sub_nucl.c:  */
  dna_header = (MOL_LIST *)substitute_nucl_in_mol_list( dna_header, 'C', 'D', 
    zif268_header, 'C', 'D', NUCL_1_INDEX, NUCL_1_FROM_INDEX, DNA_SEQ_OFFSET );

  /*  In sub_nucl.c:  */
  dna_header = (MOL_LIST *)substitute_nucl_in_mol_list( dna_header, 'C', 'D', 
    zif268_header, 'C', 'D', NUCL_2_INDEX, NUCL_2_FROM_INDEX, DNA_SEQ_OFFSET );

  /*  In sub_nucl.c:  */
  dna_header = (MOL_LIST *)substitute_nucl_in_mol_list( dna_header, 'C', 'D', 
    zif268_header, 'C', 'D', NUCL_3_INDEX, NUCL_3_FROM_INDEX, DNA_SEQ_OFFSET );

  /*  In sub_nucl.c:  */
  dna_header = (MOL_LIST *)substitute_nucl_in_mol_list( dna_header, 'C', 'D', 
    zif268_header, 'C', 'D', NUCL_4_INDEX, NUCL_4_FROM_INDEX, DNA_SEQ_OFFSET );

  return dna_header;

}














/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/













void print_zinc_finger_to_fname( MOL_LIST *zf_header, MOL_LIST *dna_header, 
  char output_fname[] )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( output_fname, "" );
  
  print_zinc_finger_to_fh( zf_header, dna_header, out );
    
  close_file( out );

}













void print_zinc_finger_to_fh( MOL_LIST *zf_header, MOL_LIST *dna_header,
  FILE *out )
{

  int atom_numb = 0;

  /*  In ../parse/pdb_print.c:  */
  print_mol_lister_pdb_fh( zf_header, 'A', 1, &atom_numb, out );
  print_mol_lister_pdb_fh( zf_header->next, 'B', 2, &atom_numb, out );
  print_mol_lister_pdb_fh( zf_header->next->next, 'C', 3, &atom_numb, out );
  print_mol_lister_pdb_fh( dna_header, 'D', 4, &atom_numb, out );
  print_mol_lister_pdb_fh( dna_header->next, 'E', 5, &atom_numb, out );

}






















