#include "mol_list.h"
















#define NUMB_ION_BONDS 4














MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( fname, "" );
  
  mol_header = (MOL_LIST *)get_mol_list_w_fh( mol_header, in );
  
  close_file( in );
  
  return mol_header;

}














MOL_LIST *get_mol_list_w_fh( MOL_LIST *mol_header, FILE *in )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL )
    mol_header = (MOL_LIST *)get_mol_list_single_line( mol_header, line );
    
  set_mol_list_seqs( mol_header );
    
//  print_mol_list( mol_header );
    
  return mol_header;

}













MOL_LIST *get_mol_list_single_line( MOL_LIST *mol_header, char line[] )
{

  char keyword[ BUFFER ], atom[ BUFFER ];
  
  sscanf( line, "%s", keyword );
  
  if ( ! strcmp( keyword, "CONECT" ))
    return connect_atoms( mol_header, line );

  if (( strcmp( keyword, "ATOM" )) && ( strcmp( keyword, "HETATM" )))
    return mol_header;
    
  sscanf( line, "%*s%*d%*s%s", atom );
  
  if ( ! strcmp( atom, "HOH" ))
    return mol_header;
    
  return get_my_mol_list_single_line( mol_header, line );

}













MOL_LIST *get_my_mol_list_single_line( MOL_LIST *mol_header, char line[] )
{

  MOL_LIST *mol = NULL;
  mol = (MOL_LIST *)get_mol_lister_from_line( mol_header, line );

  if ( mol == NULL )
    return make_and_put_new_mol_lister( mol_header, line );
    
  return get_mol_list_existing_mol_single_line( mol_header, mol, line );

}













MOL_LIST *get_mol_list_existing_mol_single_line( MOL_LIST *mol_header, 
  MOL_LIST *mol, char line[] )
{

  int resid_numb;
  
  sscanf( line, "%*s%*d%*s%*s%*s%d", &resid_numb );
  
  RESID_LIST *resid = NULL;
  resid = (RESID_LIST *)get_resid_lister_correct_mol( mol, resid_numb );
    
  if ( resid == NULL ) 
    return get_mol_list_new_resid_lister( mol_header, mol, line );
    
  return get_mol_list_single_atom_line( mol_header, resid, line );
  
}












MOL_LIST *get_mol_list_new_resid_lister( MOL_LIST *mol_header, MOL_LIST *mol, 
  char line[] )
{
    
  RESID_LIST *rheader = mol->my_mol->resid_header;

  mol->my_mol->resid_header = (RESID_LIST *)make_and_put_new_resid_lister( rheader, 
    line );
    
  return mol_header;
  
}















MOL_LIST *get_mol_list_single_atom_line( MOL_LIST *mol_header, RESID_LIST *resid, 
  char line[] )
{

  if ( atom_in_list_in_line( resid->my_resid->atom_header, line ))
    return mol_header;

  ATOM_LIST *atom = NULL;
  atom = (ATOM_LIST *)make_and_set_atom_lister( atom, line );
  
  put_atom_end_of_list( resid->my_resid->atom_header, atom );
  
  return mol_header;

}














int atom_in_list_in_line( ATOM_LIST *atom_header, char line[] )
{

  int numb;
  
  sscanf( line, "%*s%d", &numb );

  ATOM_LIST *atom = atom_header;
  
  while ( atom != NULL ) {
  
    if ( atom->my_atom->numb == numb )
      return 1;
      
    atom = atom->next;
  
  }
  
  return 0;

}










ATOM_LIST *make_and_set_atom_lister( ATOM_LIST *atom, char line[] )
{

  ATOM_LIST *create_atom_lister( ATOM_LIST *xx );
  
  atom = (ATOM_LIST *)create_atom_lister( atom );
  
  sscanf( line, "%s%d%s%s%*s%*d%f%f%f", 
    atom->my_atom->line_first,
    &(atom->my_atom->numb),
    atom->my_atom->atom,
    atom->my_atom->name,
    &(atom->my_atom->xyz->x), 
    &(atom->my_atom->xyz->y), 
    &(atom->my_atom->xyz->z) );
    
  strcpy( atom->my_atom->line_end, line + 56 );
  atom->my_atom->line_end[ strlen( atom->my_atom->line_end ) - 1 ] = '\0';
    
  return atom; 

}

















void put_atom_end_of_list( ATOM_LIST *atom_header, ATOM_LIST *atom )
{

  if ( atom_header == NULL )
    error( 3010, "Unexpected null value for atom header.\n" );
    
  ATOM_LIST *curr = atom_header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = atom;

}











/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/












MOL_LIST *get_mol_lister_from_line( MOL_LIST *mol_header, char line[] )
{

  char mol_label[ BUFFER ];
  
  sscanf( line, "%*s%*d%*s%*s%s", mol_label );
  
  return get_mol_lister_from_mol_char( mol_header, mol_label[ 0 ] );

}














MOL_LIST *get_mol_lister_from_mol_char( MOL_LIST *mol_header, char mol_char )
{

  MOL_LIST *mol = mol_header;
  
  while ( mol != NULL ) {
  
    if ( mol->my_mol->label_char == mol_char )
      return mol;
      
    mol = mol->next;
  
  }
  
  return NULL;

}













RESID_LIST *get_resid_lister_correct_mol( MOL_LIST *mol, int resid_numb )
{

  if ( mol == NULL )
    return NULL;
    
  RESID_LIST *resid = mol->my_mol->resid_header;
  
  while ( resid != NULL ) {
  
    if ( resid->my_resid->numb == resid_numb )
      return resid;
      
    resid = resid->next;
  
  }
  
  return NULL;

}













/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/












MOL_LIST *make_and_put_new_mol_lister( MOL_LIST *mol_header, char line[] )
{

  MOL_LIST *create_mol_lister( MOL_LIST *mm );
  
  MOL_LIST *mol = NULL;
  RESID_LIST *resid = NULL;
  
  mol = (MOL_LIST *)create_mol_lister( mol );
  
  char mol_char[ BUFFER ];
  
  sscanf( line, "%*s%*d%*s%*s%s", mol_char );
  
  mol->my_mol->label_char = mol_char[ 0 ];
  
  resid = (RESID_LIST *)make_and_put_new_resid_lister( resid, line );
  
  mol->my_mol->resid_header = resid;
  
  return put_new_mol_lister_end_of_list( mol_header, mol );

}
















RESID_LIST *make_and_put_new_resid_lister( RESID_LIST *resid_header, char line[] )
{

  RESID_LIST *create_resid_lister( RESID_LIST *xx );
  
  RESID_LIST *resid = NULL;
  ATOM_LIST *atom = NULL;
  
  resid = (RESID_LIST *)create_resid_lister( resid );
  
  sscanf( line, "%*s%*d%*s%s%*s%d", resid->my_resid->resid,
    &(resid->my_resid->numb) );
    
  atom = (ATOM_LIST *)make_and_set_atom_lister( atom, line );
  
  resid->my_resid->atom_header = atom;
    
  return put_new_resid_lister( resid_header, resid );

}















RESID_LIST *put_new_resid_lister( RESID_LIST *resid_header, RESID_LIST *resid )
{

  if ( resid_header == NULL )
    return resid;
    
  RESID_LIST *curr = resid_header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = resid;
  
  return resid_header;

}











MOL_LIST *put_new_mol_lister_end_of_list( MOL_LIST *mol_header, MOL_LIST *mol )
{

  if ( mol_header == NULL )
    return mol;

  MOL_LIST *curr = mol_header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = mol;
  
  return mol_header;

}

















/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/

















MOL_LIST *connect_atoms( MOL_LIST *mol_header, char line[] )
{

  int atom_numb;
  
  sscanf( line, "%*s%d", &atom_numb );
  
  ATOM_LIST *ion = NULL;
  ion = (ATOM_LIST *)get_ion_atom_lister_numb_mol_list( mol_header, atom_numb );

  if ( ion != NULL )
    return connect_atom_ion( mol_header, ion, line );

  return mol_header;

}














ATOM_LIST *get_ion_atom_lister_numb_mol_list( MOL_LIST *mol_header, int atom_numb )
{

  ATOM_LIST *atom = (ATOM_LIST *)get_atom_lister_numb_mol_list( mol_header,
    atom_numb );
    
  if ( atom == NULL )
    return NULL;
    
  return ( ! strcmp( atom->my_atom->atom, "ZN" )) ? (atom) : (NULL);

}












ATOM_LIST *get_atom_lister_numb_mol_list( MOL_LIST *mol_header, int atom_numb )
{  

  MOL_LIST *mol = mol_header;
  
  ATOM_LIST *atom = NULL;
  
  while (( atom == NULL ) && ( mol != NULL ))
    atom = (ATOM_LIST *)get_atom_lister_numb_mlister_and_step( &mol, 
      atom_numb );
      
  return atom;

}











ATOM_LIST *get_atom_lister_numb_mlister_and_step( MOL_LIST **mol, int atom_numb )
{
  ATOM_LIST *atom = (ATOM_LIST *)get_atom_lister_numb_mlister( *mol, atom_numb );   
  *mol = (*mol)->next;
  return atom;
}











ATOM_LIST *get_atom_lister_numb_mlister( MOL_LIST *mol, int atom_numb )
{

  RESID_LIST *resid = mol->my_mol->resid_header;
  
  ATOM_LIST *atom = NULL;
  
  while (( atom == NULL ) && ( resid != NULL ))
    atom = (ATOM_LIST *)get_atom_lister_numb_rlister_and_step( &resid, atom_numb );

  return atom;

}















ATOM_LIST *get_atom_lister_numb_rlister_and_step( RESID_LIST **resid,
  int atom_numb )
{

  ATOM_LIST *atom = (ATOM_LIST *)get_atom_lister_numb_rlister( *resid,
    atom_numb );
    
  *resid = (*resid)->next;
  
  return atom;

}













ATOM_LIST *get_atom_lister_numb_rlister( RESID_LIST *resid, int atom_numb )
{

  ATOM_LIST *atom = resid->my_resid->atom_header;
  
  while ( atom != NULL ) {
  
    if ( atom->my_atom->numb == atom_numb ) 
      return atom;
    
    atom = atom->next;
  
  }
  
  return NULL;

}















MOL_LIST *connect_atom_ion( MOL_LIST *mol_header, ATOM_LIST *atom, char line[] )
{

  if ( atom == NULL )
    return mol_header;

  int numb[ NUMB_ION_BONDS ];
  
  sscanf( line, "%*s%*d%d%d%d%d", numb, numb + 1, numb + 2, numb + 3 );
  
  int i;
  
  for ( i = 0; i < NUMB_ION_BONDS; i++ )
    connect_atom_ion_single_bond( mol_header, atom, numb[ i ] );
    
  return mol_header;

}














void connect_atom_ion_single_bond( MOL_LIST *mol_header, ATOM_LIST *atom,
  int numb )
{

  ATOM_LIST *connect_atom = NULL;
  connect_atom = (ATOM_LIST *)get_atom_lister_numb_mol_list( mol_header, numb ); 
  
  if ( connect_atom == NULL )
    return;
    
  connect_my_atom_ion_single_bonds( atom, connect_atom );

}














void connect_my_atom_ion_single_bonds( ATOM_LIST *atom, ATOM_LIST *connect_atom )
{

  void check_null( int numb, ... );
  
  ATOM_LIST *new_lister = NULL;
  new_lister = (ATOM_LIST *)malloc( sizeof(ATOM_LIST) );
  
  /*  In ../main/dyn_alloc.c:  */
  check_null( 1, new_lister );
  
  new_lister->my_atom = connect_atom->my_atom;
  new_lister->next = NULL;
  
  if ( atom->my_atom->con_header == NULL ) {
    atom->my_atom->con_header = new_lister;
    return;
  }
  
  ATOM_LIST *curr = atom->my_atom->con_header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = new_lister;
  
}








/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
















void print_mol_list( MOL_LIST *mol_header )
{

  MOL_LIST *mol = mol_header;
  
  while ( mol != NULL )
    print_single_mol_lister_and_step( &mol );

}













void print_single_mol_lister_and_step( MOL_LIST **mol )
{
  print_single_mol_lister( *mol );
  *mol = (*mol)->next;
}















void print_single_mol_lister( MOL_LIST *mol )
{

  RESID_LIST *resid = mol->my_mol->resid_header;
  
  while ( resid != NULL )
    print_single_resid_lister_and_step( mol, &resid );

}














void print_single_resid_lister_and_step( MOL_LIST *mol, RESID_LIST **resid )
{
  print_single_resid_lister( mol, *resid );
  *resid = (*resid)->next;
}












void print_single_resid_lister( MOL_LIST *mol, RESID_LIST *resid )
{

  ATOM_LIST *atom = resid->my_resid->atom_header;
  
  while ( atom != NULL )
    print_single_atom_lister_and_step( mol, resid, &atom );

}











void print_single_atom_lister_and_step( MOL_LIST *mol, RESID_LIST *resid,
  ATOM_LIST **atom )
{
  print_single_atom_lister( mol, resid, *atom );
  *atom = (*atom)->next;
}













void print_single_atom_lister( MOL_LIST *mol, RESID_LIST *resid, ATOM_LIST *atom )
{

  printf( "%c   %3s (%d)  %s  %f  %f  %f\n",
    mol->my_mol->label_char,
    resid->my_resid->resid,
    resid->my_resid->numb,
    atom->my_atom->atom,
    atom->my_atom->xyz->x,
    atom->my_atom->xyz->y,
    atom->my_atom->xyz->z );

}
















/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/













void set_mol_list_seqs( MOL_LIST *mol_header )
{

  MOL_LIST *mol = mol_header;
  
  while ( mol != NULL )
    set_mol_list_seqs_single_mol_lister_and_step( &mol );

}














void set_mol_list_seqs_single_mol_lister_and_step( MOL_LIST **mol )
{
  set_mol_list_seqs_single_mol_lister( *mol );
  *mol = (*mol)->next;
}













void set_mol_list_seqs_single_mol_lister( MOL_LIST *mol )
{

  RESID_LIST *resid = mol->my_mol->resid_header;

  int i = 0;
  
  while ( resid != NULL ) {
  
    char c = get_nucl_or_aa_from_resid_lister_and_step( &resid );
    
    if ( c != '\0' )
      mol->my_mol->seq[ i++ ] = c;
      
  }
  
  mol->my_mol->seq[ i ] = '\0';
  
}















char get_nucl_or_aa_from_resid_lister_and_step( RESID_LIST **resid )
{
  char c = get_nucl_or_aa_from_resid_lister( *resid );
  *resid = (*resid)->next;
  return c;
}












char get_nucl_or_aa_from_resid_lister( RESID_LIST *resid )
{

  resid->my_resid->aa = get_aa_from_resid_name( resid->my_resid->resid );
  
  switch ( resid->my_resid->aa ) {
  
  case 'a' :
  case 'c' :
  case 'g' :
  case 't' :
    resid->my_resid->mol_type = MOL_TYPE_NUCL;
    break;
    
  case 'Z' :
    resid->my_resid->mol_type = MOL_TYPE_ION;
    break;
    
  default :
    resid->my_resid->mol_type = MOL_TYPE_AA;
    break;
    
  }
  
  return resid->my_resid->aa;

}













char get_aa_from_resid_name( char resid[] )
{

  if ( ! strcmp( resid, "ALA" ))
    return 'A';
    
  if ( ! strcmp( resid, "VAL" ))
    return 'V';
    
  if ( ! strcmp( resid, "LEU" ))
    return 'L';
    
  if ( ! strcmp( resid, "ILE" ))
    return 'I';
    
  if ( ! strcmp( resid, "PHE" ))
    return 'F';
    
  if ( ! strcmp( resid, "TRP" ))
    return 'W';
    
  if ( ! strcmp( resid, "MET" ))
    return 'M';
    
  if ( ! strcmp( resid, "PRO" ))
    return 'P';
    
  if ( ! strcmp( resid, "ASP" ))
    return 'D';
    
  if ( ! strcmp( resid, "GLU" ))
    return 'E';
    
  if ( ! strcmp( resid, "GLY" ))
    return 'G';
    
  if ( ! strcmp( resid, "SER" ))
    return 'S';
    
  if ( ! strcmp( resid, "THR" ))
    return 'T';
    
  if ( ! strcmp( resid, "CYS" ))
    return 'C';
    
  if ( ! strcmp( resid, "TYR" ))
    return 'Y';
    
  if ( ! strcmp( resid, "ASN" ))
    return 'N';
    
  if ( ! strcmp( resid, "GLN" ))
    return 'Q';
    
  if ( ! strcmp( resid, "LYS" ))
    return 'K';
    
  if ( ! strcmp( resid, "ARG" ))
    return 'R';
    
  if ( ! strcmp( resid, "HIS" ))
    return 'H';
    
  if ( ! strcmp( resid, "DA" ))
    return 'a';
    
  if ( ! strcmp( resid, "DC" ))
    return 'c';
    
  if ( ! strcmp( resid, "DG" ))
    return 'g';
    
  if ( ! strcmp( resid, "DT" ))
    return 't';
    
  if ( ! strcmp( resid, "ZN" ))
    return 'Z';
    
  if ( ! strcmp( resid, "HOH" ))
    return '\0';
    
  if ( ! strcmp( resid, "GOL" ))
    return '\0';
    
  printf( "Unrecognized residue type :  %s\n", resid );
    
  return '\0';

}











