#include "cp.h"













/*

    Notes for function :  'do_forward_rev_comprehensive_spec_comp()'
    
    Usage :
      ./phylo -m R -x hg19 -X monDom5 -a gggcgg
    
    This function creates the file
      'data/ft_hg19/occur/monDom5/monDom5.motifComp.hg19.gggcgg.txt.BothStrand.txt'

    Only the motif 'gggcgg' and 'ccgccc' are presently accepted.
    The program call outputs co-occurrence data to the output file
    as above.  The file contains 6 entries, one for each site of the
    motif.  Each entry contains the variable site (Site :  1, 2, ..., 6),
    the motif (with an 'N' at the variable site, such as 'gggNgg'
    for Site == 4), and the number of orthologs containing the
    motif in both species, where the motif is considered to 'occur' when
    it matches the motif at all sites except for the variable site.
    (Eg, 'Ortho = 9774' indicates that the motif was found in both species
    within 9774 gene promoters, although these occurrences may not be
    within the region of overrepresentation.
    

*/











#define GENE_LABEL_FNAME "data/ft_hg19/occur/hg19.geneLabel.txt"
#define INPUT_MAF_FNAME "data/ft_hg19/align/hg19.multiz46way.my_fa.txt"

















void do_single_spec_comparison( void )
{

  extern char species[ BUFFER ], minor_species[ BUFFER ];

  char major_file[ BUFFER ], minor_file[ BUFFER ];
  
  get_species_occur_fname_base( major_file, species );
  get_species_occur_fname_base( minor_file, minor_species );
  
  do_single_spec_comparison_w_fnames( GENE_LABEL_FNAME, major_file, minor_file );

}














void do_single_spec_comparison_w_fnames( char gene_fname[], char major_file[], 
  char minor_file[] )
{

  extern MOTIF major_motif;
  
  MOTIF mf = major_motif;
  
  char site;
  
  for ( site = 0; site < MOTIF_LENGTH; site++ )
    do_single_spec_comparison_single_site( &mf, gene_fname, major_file,
      minor_file, site );
      
}












void do_single_spec_comparison_single_site( MOTIF *mf, char gene_fname[],
  char major_file[], char minor_file[], char site )
{

  MOTIF mf_holder = *mf;
  
  do_my_single_spec_comparison_single_site( mf, gene_fname, major_file,
    minor_file, site );
    
  *mf = mf_holder;

}












void do_my_single_spec_comparison_single_site( MOTIF *mf, char gene_fname[],
  char major_file[], char minor_file[], char site )
{

  GENE_LIST *get_gene_occur_list_fname( GENE_LIST *gene_header, MOTIF *mf,
    char gene_fname[], char fname[] );

  GENE_LIST *major_header = NULL, *minor_header = NULL;
  
  char bp_holder = mf->cons[ site ];
  
  mf->cons[ site ] = 'n';
  
  /*  In occur_list.c:  */
  major_header = (GENE_LIST *)get_gene_occur_list_fname( major_header, mf, 
    gene_fname, major_file );
    
  /*  In occur_list.c:  */
  minor_header = (GENE_LIST *)get_gene_occur_list_fname( minor_header, mf, 
    gene_fname, minor_file );
  
  mf->cons[ site ] = bp_holder;
    
  /*  In this file, below:  */
  do_single_species_comparison_w_occur_lists( major_header, minor_header, mf, 
    gene_fname, major_file, minor_file, site );
    
  kill_gene_list( major_header );
  kill_gene_list( minor_header );

}












void do_single_species_comparison_w_occur_lists( GENE_LIST *major_header,
  GENE_LIST *minor_header, MOTIF *mf, char gene_fname[], char major_file[],
  char minor_file[], char site )
{

  MATRIX_LIST *get_matrix_list_occur_comparison( MATRIX_LIST *mx_header,
    GENE_LIST *major_header, GENE_LIST *minor_header, MOTIF *mf, char site );
    
  extern char species[ BUFFER ], minor_species[ BUFFER ];
    
  MATRIX_LIST *mx_header = NULL;
  /*  In mx_occur.c:  */
  mx_header = (MATRIX_LIST *)get_matrix_list_occur_comparison( mx_header,
    major_header, minor_header, mf, site );
    
  /*  In pr.c:  */
  print_co_occur_matrix_list( mx_header, mf, site, species, minor_species,
    stdout );

  kill_matrix_list( mx_header );

}















/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/
















void do_comprehensive_spec_comparisons( void )
{

  extern char species[ BUFFER ];

  char major_file[ BUFFER ];  
  get_species_occur_fname_base( major_file, species );
  
  do_comprehensive_spec_comparisons_major_fname( major_file );
  
}












void do_comprehensive_spec_comparisons_major_fname( char major_fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  STR_LIST *get_spec_list_align_fh( STR_LIST *spec_header, FILE *fh );

  STR_LIST *spec_header = NULL;
  
  FILE *fh = NULL;
  fh = (FILE *)open_text_file_reading( INPUT_MAF_FNAME, "" );
  
  /*  In ../format/occur.c:  */
  spec_header = (STR_LIST *)get_spec_list_align_fh( spec_header, fh );
  
  close_file( fh );
  
  /*  In this file, below:  */
  do_compr_spec_comp_minor_species_list( spec_header, major_fname );

}












void do_compr_spec_comp_minor_species_list( STR_LIST *spec_header, 
  char major_fname[] )
{

  extern MOTIF major_motif;
  
  GENE_LIST *major_gene_header[ MOTIF_LENGTH ];
  
  MOTIF mf = major_motif;
  
  char site;
  
  for ( site = 0; site < MOTIF_LENGTH; site++ )
    major_gene_header[ site ] = NULL;
  
  for ( site = 0; site < MOTIF_LENGTH; site++ )
    major_gene_header[ site ] = 
      (GENE_LIST *)get_single_gene_list( major_gene_header[ site ], &mf, site,
        major_fname );
        
  do_compr_spec_comp_major_gene_list( major_gene_header, spec_header, &mf );

}











GENE_LIST *get_single_gene_list( GENE_LIST *gene_header, MOTIF *mf, char site,
  char fname[] )
{

  MOTIF mf_holder = *mf;
  
  gene_header = (GENE_LIST *)get_my_single_gene_list( gene_header, mf, site,
    fname );
    
  *mf = mf_holder;
  
  return gene_header;

}











GENE_LIST *get_my_single_gene_list( GENE_LIST *gene_header, MOTIF *mf,
  char site, char fname[] )
{

  GENE_LIST *get_gene_occur_list_fname( GENE_LIST *gene_header, MOTIF *mf,
    char gene_fname[], char fname[] );

  GENE_LIST *major_header = NULL, *minor_header = NULL;
  
  char bp_holder = mf->cons[ site ];
  
  mf->cons[ site ] = 'n';
  
  /*  In occur_list.c:  */
  return get_gene_occur_list_fname( gene_header, mf, GENE_LABEL_FNAME, fname );
    
}












void do_compr_spec_comp_major_gene_list( GENE_LIST *major_gene_header[], 
  STR_LIST *spec_header, MOTIF *mf )
{

  int numb = 0;

  STR_LIST *spec = spec_header;
  
  while ( spec != NULL )
    do_compr_spec_comp_single_spec_pair_and_step( major_gene_header, &spec, mf,
      ++numb );
    
}









void do_compr_spec_comp_single_spec_pair_and_step( GENE_LIST *major_gene_header[],
  STR_LIST **spec, MOTIF *mf, int numb )
{
  do_compr_spec_comp_single_spec_pair( major_gene_header, *spec, mf, numb );
  *spec = (*spec)->next;
}












void do_compr_spec_comp_single_spec_pair( GENE_LIST *major_gene_header[],
  STR_LIST *spec, MOTIF *mf, int numb )
{

  extern char species[ BUFFER ], minor_species[ BUFFER ];
  
  char minor_fname[ BUFFER ], out_fname[ BUFFER ];
  
  if ( ! strcmp( species, spec->str ))
    return;
    
  if (( minor_species[ 0 ] != '\0' ) && ( strcmp( minor_species, spec->str )))
    return;
    
  get_species_occur_fname_base( minor_fname, spec->str );
  get_species_compare_out_fname( out_fname, species, mf, spec->str );
  
  do_my_compr_spec_comp_single_spec_pair( major_gene_header, species, spec->str,
    mf, minor_fname, out_fname, numb );
  
}












void do_my_compr_spec_comp_single_spec_pair( GENE_LIST *major_gene_header[], 
  char major_spec[], char minor_spec[], MOTIF *mf, char minor_fname[],
  char out_fname[], int numb )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( out_fname, "" );
  
  do_compr_spec_comp_single_pair_out_fh( major_gene_header, major_spec, 
    minor_spec, mf, numb, minor_fname, out );
  
  close_file( out );

}











void do_compr_spec_comp_single_pair_out_fh( GENE_LIST *major_gene_header[],
  char major_spec[], char minor_spec[], MOTIF *mf, int numb, char minor_fname[], 
  FILE *out )
{

  char site;
  
  for ( site = 0; site < MOTIF_LENGTH; site++ )
    do_compr_spec_comp_single_pair_single_site( major_gene_header[ site ],
      major_spec, minor_spec, mf, site, numb, minor_fname, out );

}











void do_compr_spec_comp_single_pair_single_site( GENE_LIST *major_header,
  char major_spec[], char minor_spec[], MOTIF *mf, char site, int numb,
  char minor_fname[], FILE *out )
{

  GENE_LIST *get_gene_occur_list_fname( GENE_LIST *gene_header, MOTIF *mf,
    char gene_fname[], char fname[] );

  GENE_LIST *minor_header = NULL;
  
  char bp_holder = mf->cons[ site ];
  
  mf->cons[ site ] = 'n';
  
  printf( "\t\t\t%8s  %8s  (%d)\n", major_spec, minor_spec, numb );
  
  /*  In occur_list.c:  */
  minor_header = (GENE_LIST *)get_gene_occur_list_fname( minor_header, mf, 
    GENE_LABEL_FNAME, minor_fname );
    
  mf->cons[ site ] = bp_holder;
    
  /*  In this file, below:  */
  do_compr_spec_single_spec_pair_two_gene_lists( major_header, minor_header,
    major_spec, minor_spec, mf, site, out );
    
  kill_gene_list( minor_header );
  
}












void do_compr_spec_single_spec_pair_two_gene_lists( GENE_LIST *major_header, 
  GENE_LIST *minor_header, char major_spec[], char minor_spec[], MOTIF *mf, 
  char site, FILE *out )
{

  MATRIX_LIST *get_matrix_list_occur_comparison( MATRIX_LIST *mx_header,
    GENE_LIST *major_header, GENE_LIST *minor_header, MOTIF *mf, char site );
    
  MOTIF mf_holder = *mf;  
    
  MATRIX_LIST *mx_header = NULL;
  /*  In mx_occur.c:  */
  mx_header = (MATRIX_LIST *)get_matrix_list_occur_comparison( mx_header,
    major_header, minor_header, mf, site );
    
  *mf = mf_holder;

  /*  In pr.c:  */
  print_co_occur_matrix_list( mx_header, mf, site, major_spec, minor_spec, out );

  kill_matrix_list( mx_header );

}













/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/














void do_forward_reverse_spec_comparison( void )
{

  extern char species[ BUFFER ], minor_species[ BUFFER ];

  char major_file[ BUFFER ], minor_file[ BUFFER ];
  
  get_species_occur_fname_base( major_file, species );
  get_species_occur_fname_base( minor_file, minor_species );
  
  do_forward_reverse_spec_comp_fnames( major_file, minor_file, GENE_LABEL_FNAME,
    species, minor_species );

}















void do_forward_reverse_spec_comp_fnames( char major_file[], char minor_file[],
  char gene_fname[], char major_spec[], char minor_spec[] )
{


  extern MOTIF major_motif;
  
  MOTIF mf = major_motif;
  
  char site;
  
  for ( site = 0; site < MOTIF_LENGTH; site++ )
    do_forward_reverse_spec_comp_single_site( major_file, minor_file,
      major_spec, minor_spec, gene_fname, &mf, site );

}














void do_forward_reverse_spec_comp_single_site( char major_file[], 
  char minor_file[], char major_spec[], char minor_spec[], char gene_fname[],
  MOTIF *mf, char site )
{

  GENE_LIST *get_gene_for_rev_occur_list_fname( GENE_LIST *major_header, MOTIF *mf, 
    char gene_fname[], char fname[] );

  GENE_LIST *major_header = NULL, *minor_header = NULL;
  
  char bp_holder = mf->cons[ site ];
  
  mf->cons[ site ] = 'n';
  
  /*  In occur_list.c:  */
  major_header = (GENE_LIST *)get_gene_for_rev_occur_list_fname( major_header, mf, 
    gene_fname, major_file );
    
  /*  In occur_list.c:  */
  minor_header = (GENE_LIST *)get_gene_for_rev_occur_list_fname( minor_header, mf, 
    gene_fname, minor_file );
  
  mf->cons[ site ] = bp_holder;
    
  /*  In this file, below:  */
  do_single_species_comp_forward_and_rev_occur_lists( major_header, minor_header,
    mf, gene_fname, major_file, minor_file, major_spec, minor_spec, site );
    
  kill_gene_list( major_header );
  kill_gene_list( minor_header );

}














void do_single_species_comp_forward_and_rev_occur_lists( GENE_LIST *major_header, 
  GENE_LIST *minor_header, MOTIF *mf, char gene_fname[], char major_file[], 
  char minor_file[], char major_spec[], char minor_spec[], char site )
{

  MATRIX_LIST *get_matrix_occur_list_forward_and_rev( MATRIX_LIST *mx, 
    GENE_LIST *major_header, GENE_LIST *minor_header, char major_spec[], 
    char minor_spec[], MOTIF *mf, char site );
  
  MATRIX_LIST *mx_header = NULL;
  /*  In mx_occur.c:  */
  mx_header = (MATRIX_LIST *)get_matrix_occur_list_forward_and_rev( mx_header,
    major_header, minor_header, major_spec, minor_spec, mf, site );
    
  /*  In pr.c:  */
  print_co_occur_matrix_list( mx_header, mf, site, species, minor_species,
    stdout );

  kill_matrix_list( mx_header );

}














/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/












void do_forward_rev_comprehensive_spec_comp( void )
{

  extern char species[ BUFFER ];

  char major_file[ BUFFER ];  
  get_species_occur_fname_base( major_file, species );
  
  do_forward_rev_comprehensive_spec_comp_w_fname( major_file );
  
}














void do_forward_rev_comprehensive_spec_comp_w_fname( char major_file[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  STR_LIST *get_spec_list_align_fh( STR_LIST *spec_header, FILE *fh );

  STR_LIST *spec_header = NULL;
  
  FILE *fh = NULL;
  fh = (FILE *)open_text_file_reading( INPUT_MAF_FNAME, "" );
  
  /*  In ../ft/occur.c:  */
  spec_header = (STR_LIST *)get_spec_list_align_fh( spec_header, fh );
  
  close_file( fh );
  
  /*  In this file, below:  */
  do_forward_rev_compr_spec_comp_w_spec_list( spec_header, major_file );
  
}













void do_forward_rev_compr_spec_comp_w_spec_list( STR_LIST *spec_header,
  char major_fname[] )
{

  extern MOTIF major_motif;
  extern char species[ BUFFER ];
  
  GENE_LIST *major_gene_header[ MOTIF_LENGTH ];
  
  MOTIF mf = major_motif;
  
  char site;
  
  for ( site = 0; site < MOTIF_LENGTH; site++ )
    major_gene_header[ site ] = NULL;
  
  for ( site = 0; site < MOTIF_LENGTH; site++ )
    
    major_gene_header[ site ] = 
      (GENE_LIST *)get_single_gene_list_forward_rev( major_gene_header[ site ], &mf, 
        site, major_fname );
        
  do_compr_spec_comp_forward_rev_major_gene_list( major_gene_header, spec_header, 
    &mf, species, major_fname );

}












GENE_LIST *get_single_gene_list_forward_rev( GENE_LIST *gene_header, MOTIF *mf,
  char site, char major_fname[] )
{

  MOTIF mf_holder = *mf;
  
  gene_header = (GENE_LIST *)get_my_single_gene_list_forward_rev( gene_header,
    mf, site, major_fname );
  
  *mf = mf_holder;
  
  return gene_header;

}











GENE_LIST *get_my_single_gene_list_forward_rev( GENE_LIST *gene_header,
  MOTIF *mf, char site, char major_fname[] )
{

  GENE_LIST *get_gene_for_rev_occur_list_fname( GENE_LIST *major_header, MOTIF *mf, 
    char gene_fname[], char fname[] );

  GENE_LIST *major_header = NULL;
  
  mf->cons[ site ] = 'n';
  
  /*  In occur_list.c:  */
  return get_gene_for_rev_occur_list_fname( gene_header, mf, GENE_LABEL_FNAME, 
    major_fname );
    
}















void do_compr_spec_comp_forward_rev_major_gene_list( GENE_LIST *major_gene_header[], 
  STR_LIST *spec_header, MOTIF *mf, char major_spec[], char major_fname[] )
{

  int numb = 0;

  STR_LIST *spec = spec_header;
  
  while ( spec != NULL )
    do_compr_single_spec_comp_forward_rev_and_step( major_gene_header, &spec,
      mf, major_spec, major_fname, ++numb );

}














void do_compr_single_spec_comp_forward_rev_and_step( GENE_LIST *major_gene_header[],
  STR_LIST **spec, MOTIF *mf, char major_spec[], char major_fname[], int numb )
{
  do_compr_single_spec_comp_forward_rev( major_gene_header, *spec, mf, major_spec,
    major_fname, numb );
  *spec = (*spec)->next;
}











void do_compr_single_spec_comp_forward_rev( GENE_LIST *major_gene_header[],
  STR_LIST *spec, MOTIF *mf, char major_spec[], char major_fname[], int numb )
{

  if ( ! strcmp( spec->str, major_spec ))
    return;
    
  do_my_compr_single_spec_comp_forward_rev( major_gene_header, mf, major_spec,
    spec->str, major_fname, numb );

}













void do_my_compr_single_spec_comp_forward_rev( GENE_LIST *major_gene_header[],
  MOTIF *mf, char major_spec[], char minor_spec[], char major_fname[], int numb )
{

  extern char minor_species[ BUFFER ];
  
  static int i = 0;
  
  char minor_fname[ BUFFER ], out_fname[ BUFFER ];
  
  if (( minor_species[ 0 ] != '\0' ) && ( strcmp( minor_species, minor_spec )))
    return;
    
  if ( ! strcmp( minor_spec, "bosTau4" ))
    i = 1;
    
  if ( i )
    return;
    
  get_species_occur_fname_base( minor_fname, minor_spec );
  get_species_compare_out_fname( out_fname, major_spec, mf, minor_spec );
  
  do_my_compr_spec_comp_single_spec_pair_forward_rev( major_gene_header, major_spec,
    minor_spec, mf, major_fname, minor_fname, out_fname, numb );
  
}













void do_my_compr_spec_comp_single_spec_pair_forward_rev( GENE_LIST *major_header[],
  char major_spec[], char minor_spec[], MOTIF *mf, char major_fname[],
  char minor_fname[], char out_fname[], int numb )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( out_fname, "Total.txt" );
  
//  out = (FILE *)open_text_file_writing( out_fname, "BothStrand.txt" );
  
  do_compr_spec_comp_single_pair_forward_rev_out_fh( major_header, major_spec, 
    minor_spec, mf, numb, major_fname, minor_fname, out );
  
  close_file( out );

}












void do_compr_spec_comp_single_pair_forward_rev_out_fh( GENE_LIST *major_header[],
  char major_spec[], char minor_spec[], MOTIF *mf, int numb, char major_fname[],
  char minor_fname[], FILE *out )
{

  char site;
  
  for ( site = 0; site < MOTIF_LENGTH; site++ )
    do_compr_spec_comp_forward_rev_single_site( major_header[ site ], major_spec,
      minor_spec, mf, numb, site, major_fname, minor_fname, out );

}












void do_compr_spec_comp_forward_rev_single_site( GENE_LIST *major_header,
  char major_spec[], char minor_spec[], MOTIF *mf, int numb, char site,
  char major_fname[], char minor_fname[], FILE *out )
{

  GENE_LIST *get_gene_for_rev_occur_list_fname( GENE_LIST *major_header, MOTIF *mf, 
    char gene_fname[], char fname[] );

  GENE_LIST *minor_header = NULL;
  
  char bp_holder = mf->cons[ site ]; 
  mf->cons[ site ] = 'n';
  
  printf( "\t\t\t%8s  %8s  (%d)\n", major_spec, minor_spec, numb );
  
  /*  In occur_list.c:  */
  minor_header = (GENE_LIST *)get_gene_for_rev_occur_list_fname( minor_header, mf, 
    GENE_LABEL_FNAME, minor_fname );
    
  mf->cons[ site ] = bp_holder;
    
  /*  In this file, below:  */
  do_single_species_comp_forward_and_rev_lists_fh( major_header, minor_header, 
    mf, GENE_LABEL_FNAME, major_fname, minor_fname, major_spec, minor_spec, site,
    out );
    
  kill_gene_list( minor_header );

}














void do_single_species_comp_forward_and_rev_lists_fh( GENE_LIST *major_header, 
  GENE_LIST *minor_header, MOTIF *mf, char gene_fname[], char major_fname[], 
  char minor_fname[], char major_spec[], char minor_spec[], char site, FILE *out )
{

  MATRIX_LIST *get_matrix_occur_list_forward_and_rev( MATRIX_LIST *mx, 
    GENE_LIST *major_header, GENE_LIST *minor_header, char major_spec[], 
    char minor_spec[], MOTIF *mf, char site );
  
  MATRIX_LIST *mx_header = NULL;
  /*  In mx_occur.c:  */
  mx_header = (MATRIX_LIST *)get_matrix_occur_list_forward_and_rev( mx_header,
    major_header, minor_header, major_spec, minor_spec, mf, site );
    
  /*  In pr.c:  */
  print_co_occur_matrix_list( mx_header, mf, site, major_spec, minor_spec, out );

  kill_matrix_list( mx_header );

}







/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/












void get_species_occur_fname_base( char fname[], char spec[] )
{
  /*
  sprintf( fname, "data/ft_hg18/occur/%s/%s.ft_occur", spec, spec );
  */
  sprintf( fname, "data/ft_hg19/occur/%s/%s.ft_occur", spec, spec );
}












void get_species_compare_out_fname( char out_fname[], char major_spec[], MOTIF *mf, 
  char minor_spec[] )
{

  /*
  sprintf( out_fname, "data/ft_hg18/occur/%s/%s.motifComp.%s.%s.txt",
    minor_spec,
    minor_spec,
    major_spec,
    mf->cons );
  */

  sprintf( out_fname, "data/ft_%s/occur/%s/%s.motifComp.%s.%s.txt",
    major_spec,
    minor_spec,
    minor_spec,
    major_spec,
    mf->cons );

  
}





























