#include "mx_occur.h"

















#define OCCUR_POSITION_FNAME "data/ft_hg19/occur/hg19.position"
















MATRIX_LIST *get_matrix_list_occur_comparison( MATRIX_LIST *mx_header,
  GENE_LIST *major_header, GENE_LIST *minor_header, MOTIF *mf, char site )
{

  MATRIX_LIST *occur_mx = NULL;
  
  occur_mx = (MATRIX_LIST *)get_co_occurrence_matrix( occur_mx, major_header,
    minor_header, mf, site, OCCUR_POSITION_FNAME );

  mx_header = occur_mx;

  return mx_header;

}












MATRIX_LIST *get_co_occurrence_matrix( MATRIX_LIST *occur_mx, 
  GENE_LIST *major_header, GENE_LIST *minor_header, MOTIF *mf, char site,
  char pos_fname[] )
{

  WIND my_wind;
  
  /*  In this file, bottom:  */
  set_window_from_motif( &my_wind, mf );
  
  /*  In this file, below:  */
  return get_co_occurrence_matrix_within_window( occur_mx, major_header,
    minor_header, &my_wind, mf, site, pos_fname );

}












MATRIX_LIST *get_co_occurrence_matrix_within_window( MATRIX_LIST *occur_mx,
  GENE_LIST *major_header, GENE_LIST *minor_header, WIND *my_wind, MOTIF *mf,
  char site, char pos_fname[] )
{

  FILE *open_binary_file_reading( char a[], char b[] );
  
  FILE *ft = NULL, *ind = NULL;
  
  ft = (FILE *)open_binary_file_reading( pos_fname, "ft.txt" );
  ind = (FILE *)open_binary_file_reading( pos_fname, "ind.txt" );
  
  occur_mx = (MATRIX_LIST *)get_co_occurrence_matrix_in_wind_w_fhs( occur_mx,
    major_header, minor_header, my_wind, mf, site, ft, ind );
    
  close_file( ft );
  close_file( ind );
  
  return occur_mx;

}











MATRIX_LIST *get_co_occurrence_matrix_in_wind_w_fhs( MATRIX_LIST *occur_mx,
  GENE_LIST *major_header, GENE_LIST *minor_header, WIND *my_wind, MOTIF *mf, 
  char site, FILE *ft, FILE *ind )
{

  MATRIX_LIST *create_matrix_lister( MATRIX_LIST *xx, DIM ii, DIM jj, char tt );
  
  extern char species[ BUFFER ], minor_species[ BUFFER ];
  
  occur_mx = (MATRIX_LIST *)create_matrix_lister( occur_mx, NUMB_OF_NUCL,
    NUMB_OF_NUCL, CO_OCCUR_MATRIX );
  
  return get_my_co_occurrence_matrix_within_window( occur_mx, major_header, 
    minor_header, my_wind, mf, site, ft, ind );
    
}










MATRIX_LIST *get_my_co_occurrence_matrix_within_window( MATRIX_LIST *occur_mx,
  GENE_LIST *major_header, GENE_LIST *minor_header, WIND *my_wind, MOTIF *mf,
  char site, FILE *ft, FILE *ind )
{

  GENE_LIST *major_gene = major_header, *minor_gene = minor_header;
  
  while (( major_gene != NULL ) && ( minor_gene != NULL ))
    get_co_occurrence_matrix_in_wind_two_lists( occur_mx, &major_gene,
      &minor_gene, my_wind, mf, site, ft, ind );

  return set_occur_mx_freqs( occur_mx );

}










MATRIX_LIST *set_occur_mx_freqs( MATRIX_LIST *occur_mx )
{

  ELEM *elem = occur_mx->my_matrix->mx;
  
  occur_mx->my_matrix->total = get_total_from_matrix_lister( occur_mx );
  
  register int i = 0;
  
  while ( i++ < occur_mx->my_matrix->i * occur_mx->my_matrix->j )
    set_single_occur_elem_mx_freqs( elem++, occur_mx->my_matrix->total );

  return occur_mx;

}












COUNT get_total_from_matrix_lister( MATRIX_LIST *mx )
{

  COUNT total = 0;

  register char i, j;
  
  for ( i = 0; i < mx->my_matrix->i; i++ ) {
  
    for ( j = 0; j < mx->my_matrix->j; j++ ) {
    
      total += (mx->my_matrix->mx + (i * mx->my_matrix->j) + j)->x.total;
    
    }
  
  }
  
  return total;

}









void set_single_occur_elem_mx_freqs( ELEM *elem, COUNT total )
{
  elem->x.frac = (STAT)(elem->x.total) / (STAT)total;
}









void get_co_occurrence_matrix_in_wind_two_lists( MATRIX_LIST *occur_mx,
  GENE_LIST **major_gene, GENE_LIST **minor_gene, WIND *my_wind, MOTIF *mf,
  char site, FILE *ft, FILE *ind )
{

  get_co_occurrence_matrix_in_wind_single_major_gene( occur_mx, *major_gene,
    minor_gene, my_wind, mf, site, ft, ind );
  
  *major_gene = (*major_gene)->next;
  
}












void get_co_occurrence_matrix_in_wind_single_major_gene( MATRIX_LIST *occur_mx,
  GENE_LIST *major_gene, GENE_LIST **minor_gene, WIND *my_wind, MOTIF *mf,
  char site, FILE *ft, FILE *ind )
{

  SEQ major_numb = major_gene->my_gene->gene->seq_numb;

  while (( *minor_gene != NULL ) && 
    ( (*minor_gene)->my_gene->gene->seq_numb < major_numb ))
  {
    *minor_gene = (*minor_gene)->next;
  }
  
  if ( *minor_gene == NULL )
    return;
    
  if ( (*minor_gene)->my_gene->gene->seq_numb > major_numb )
    return;
    
  get_co_occurrence_matching_genes( occur_mx, major_gene, *minor_gene, my_wind,
    mf, site, ft, ind );

}











void get_co_occurrence_matching_genes( MATRIX_LIST *occur_mx, 
  GENE_LIST *major_gene, GENE_LIST *minor_gene, WIND *my_wind, MOTIF *mf,
  char site, FILE *ft, FILE *ind )
{

  if ( major_gene->my_gene->gene->tss_column == NON_SEQ )
    return;

  get_my_co_occurrence_matching_genes( occur_mx, major_gene, minor_gene, 
    my_wind, mf, site, ft, ind );
  
  //  For 'BothStrand.txt' file :  
//  ++(occur_mx->my_matrix->total);
    
}















void get_my_co_occurrence_matching_genes( MATRIX_LIST *occur_mx,
  GENE_LIST *major_gene, GENE_LIST *minor_gene, WIND *my_wind, MOTIF *mf,
  char site, FILE *ft, FILE *ind )
{

  WIND column_wind;
  
  SEQ seq_numb = major_gene->my_gene->gene->seq_numb;
  
  get_column_window_position_files( &column_wind, my_wind, seq_numb, ft, ind );
  
  get_co_occurrence_matrix_known_align_columns( occur_mx, &column_wind,
    major_gene, minor_gene, mf, site );
  
}













void get_column_window_position_files( WIND *column_wind, WIND *my_wind, 
  SEQ seq_numb, FILE *ft, FILE *ind )
{

  OCCUR_IND index;

  set_position( (seq_numb - 1) * sizeof(OCCUR_IND), ind );
  
  get( &index, sizeof(OCCUR_IND), ind );
  
  set_position( index.fpos, ft );
  
  get_column_window_position_w_ind_for_ft_file( column_wind, my_wind, &index, ft );

}













void get_column_window_position_w_ind_for_ft_file( WIND *column_wind, WIND *my_wind,
  OCCUR_IND *my_ind, FILE *ft )
{

  OCCUR my_occur;
  
  register int i = 0;
  
  while ( i++ < my_ind->count ) {
  
    get( &my_occur, sizeof(OCCUR), ft );
    
    if ( my_occur.seq_pos == my_wind->min )
      column_wind->min = my_occur.align_pos;
      
    if ( my_occur.seq_pos == my_wind->max )
      column_wind->max = my_occur.align_pos;
  
  }
  
}














void get_co_occurrence_matrix_known_align_columns( MATRIX_LIST *occur_mx, 
  WIND *column_wind, GENE_LIST *major_gene, GENE_LIST *minor_gene, MOTIF *mf, 
  char site )
{

  char major_nucl[ NUMB_OF_NUCL ], minor_nucl[ NUMB_OF_NUCL ];
  
  initialize_nucl_array( major_nucl, NUMB_OF_NUCL );
  initialize_nucl_array( minor_nucl, NUMB_OF_NUCL );
  
  get_occurrence_arrays_within_window( major_nucl, major_gene, site, column_wind );
  get_occurrence_arrays_within_window( minor_nucl, minor_gene, site, column_wind );
  
  update_co_occur_matrix_nucl_arrays( occur_mx, major_nucl, minor_nucl );

}











void initialize_nucl_array( char array[], int buffer )
{

  register char i;
  
  for ( i = 0; i < buffer; i++ )
    array[ i ] = 0;

}









void get_occurrence_arrays_within_window( char nucl_array[], GENE_LIST *gene,
  char site, WIND *column_wind )
{

  OCR_LIST *occur = gene->my_gene->occur_header;
  
  while ( occur != NULL )
    get_occurrence_arrays_in_window_single_occur_and_step( nucl_array, &occur,
      site, column_wind );

}











void get_occurrence_arrays_in_window_single_occur_and_step( char nucl_array[],
  OCR_LIST **occur, char site, WIND *column_wind )
{

  get_occurrence_arrays_in_window_single_occur( nucl_array, *occur, site,
    column_wind );

  *occur = (*occur)->next;

}










void get_occurrence_arrays_in_window_single_occur( char nucl_array[],
  OCR_LIST *occur, char site, WIND *column_wind )
{

  if ( ! ocr_lister_is_in_window( occur, column_wind ))
    return;
    
  get_occurrence_array_occurrence_is_in_window( nucl_array, occur, site );

}










int ocr_lister_is_in_window( OCR_LIST *occur, WIND *column_wind )
{

  if ( occur->my_occur->align_pos < column_wind->min )
    return 0;
    
  if ( occur->my_occur->align_pos > column_wind->max )
    return 0;
    
  return 1;

}












void get_occurrence_array_occurrence_is_in_window( char nucl_array[], 
  OCR_LIST *occur, char site )
{

  char get_nucl_numb_from_char( char nucl );
  
  /*  In ../main/nucl.c:  */
  char bp = get_nucl_numb_from_char( occur->my_occur->mf.cons[ site ] );
  
  nucl_array[ bp ] = 1;

}












void update_co_occur_matrix_nucl_arrays( MATRIX_LIST *occur_mx, char major_nucl[], 
  char minor_nucl[] )
{

  if ( array_is_empty( major_nucl, NUMB_OF_NUCL ))
    return;
    
  if ( array_is_empty( minor_nucl, NUMB_OF_NUCL ))
    return;
    
  ++(occur_mx->my_matrix->genes);
    
  update_my_co_occur_matrix_nucl_arrays( occur_mx, major_nucl, minor_nucl );
  
}











int array_is_empty( char array[], int buffer )
{

  register char i;
  
  for ( i = 0; i < buffer; i++ ) {
    if ( array[ i ] )
      return 0;
  }
  
  return 1;

}













void update_my_co_occur_matrix_nucl_arrays( MATRIX_LIST *occur_mx, 
  char major_nucl[], char minor_nucl[] )
{

  update_co_occur_matrix_struct_nucl_arrays( occur_mx->my_matrix, major_nucl,
    minor_nucl );
    
}










void update_co_occur_matrix_struct_nucl_arrays( MATRIX *mx, char major_nucl[],
  char minor_nucl[] )
{

  int single = one_true_elem_array( major_nucl, NUMB_OF_NUCL );

  single *= one_true_elem_array( minor_nucl, NUMB_OF_NUCL );

  register char i, j;
  
  for ( i = 0; i < mx->i; i++ ) {
    for ( j = 0; j < mx->j; j++ ) {
    
      COUNT x = 0;
      
      if (( major_nucl[ i ] ) && ( minor_nucl[ j ] )) 
        x = 1;
    
      (mx->mx + (i * mx->j) + j)->x.total += x;
      
      if ( single )  
        (mx->mx + (i * mx->j) + j)->x.co_occur += x;
    
    }
  }

}












int one_true_elem_array( char array[], int buffer )
{

  int total = 0;
  
  register char i;
  
  for ( i = 0; i < buffer; i++ ) 
    total += array[ i ];
    
  return ( total == 1 ) ? (1) : (0);

}









/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/












MATRIX_LIST *get_matrix_occur_list_forward_and_rev( MATRIX_LIST *mx, 
  GENE_LIST *major_header, GENE_LIST *minor_header, char major_spec[], 
  char minor_spec[], MOTIF *mf, char site )
{

  WIND my_wind;
  
  /*  In this file, bottom:  */
  set_window_from_motif( &my_wind, mf );
  
  /*  In this file, below:  */
  return get_matrix_forward_and_rev_occurs_within_window( mx, major_header,
    minor_header, major_spec, minor_spec, &my_wind, mf, site, OCCUR_POSITION_FNAME );

}













MATRIX_LIST *get_matrix_forward_and_rev_occurs_within_window( MATRIX_LIST *mx, 
  GENE_LIST *major_header, GENE_LIST *minor_header, char major_spec[], 
  char minor_spec[], WIND *my_wind, MOTIF *mf, char site, char pos_fname[] )
{

  FILE *open_binary_file_reading( char a[], char b[] );
  
  FILE *ft = NULL, *ind = NULL;
  
  ft = (FILE *)open_binary_file_reading( pos_fname, "ft.txt" );
  ind = (FILE *)open_binary_file_reading( pos_fname, "ind.txt" );
  
  mx = (MATRIX_LIST *)get_matrix_forward_and_rev_occurs_in_wind_w_fhs( mx,
    major_header, minor_header, major_spec, minor_spec, my_wind, mf, site, ft, ind );
    
  close_file( ft );
  close_file( ind );
  
  return mx;

}












MATRIX_LIST *get_matrix_forward_and_rev_occurs_in_wind_w_fhs( MATRIX_LIST *mx,
  GENE_LIST *major_header, GENE_LIST *minor_header, char major_spec[],
  char minor_spec[], WIND *my_wind, MOTIF *mf, char site, FILE *ft_fh, 
  FILE *ind_fh )
{

  MATRIX_LIST *create_matrix_lister( MATRIX_LIST *xx, DIM ii, DIM jj, char tt );
  
  mx = (MATRIX_LIST *)create_matrix_lister( mx, NUMB_OF_NUCL,
    NUMB_OF_NUCL, CO_OCCUR_MATRIX );
  
  return get_my_matrix_forward_and_rev_occurs_in_wind_w_fhs( mx, major_header, 
    minor_header, major_spec, minor_spec, my_wind, mf, site, ft_fh, ind_fh );
    
}













MATRIX_LIST *get_my_matrix_forward_and_rev_occurs_in_wind_w_fhs( MATRIX_LIST *mx,
  GENE_LIST *major_header, GENE_LIST *minor_header, char major_spec[],
  char minor_spec[], WIND *my_wind, MOTIF *mf, char site, FILE *ft_fh,
  FILE *ind_fh )
{

  /*  In this file, below:  */
  convert_gene_list_to_forward_strand( major_header, site );
  convert_gene_list_to_forward_strand( minor_header, site );
  
  /*  In this file, above:  */
  return get_my_co_occurrence_matrix_within_window( mx, major_header, minor_header, 
    my_wind, mf, site, ft_fh, ind_fh );

}











void convert_gene_list_to_forward_strand( GENE_LIST *major_header, char site )
{

  GENE_LIST *gene = major_header;
  
  while ( gene != NULL )
    convert_single_gene_to_foward_strand_and_step( &gene, site );

}










void convert_single_gene_to_foward_strand_and_step( GENE_LIST **gene, char site )
{
  convert_single_gene_to_forward_strand( *gene, site );
  *gene = (*gene)->next;
}












void convert_single_gene_to_forward_strand( GENE_LIST *gene, char site )
{

  OCR_LIST *my_occur = gene->my_gene->occur_header;
  
  while ( my_occur != NULL )
    convert_single_occurrence_to_forward_strand_and_step( &my_occur, site );

}











void convert_single_occurrence_to_forward_strand_and_step( OCR_LIST **my_occur,
  char site )
{
  convert_single_occurrence_to_forward_strand( *my_occur, site );
  *my_occur = (*my_occur)->next;
}










void convert_single_occurrence_to_forward_strand( OCR_LIST *my_occur, char site )
{
  
  if ( my_occur->my_occur->strand == FORWARD_STRAND )
    return;
    
  convert_single_rev_compl_occur_to_forward_strand( my_occur, site );
  
  my_occur->my_occur->strand = FORWARD_STRAND;
  
}











void convert_single_rev_compl_occur_to_forward_strand( OCR_LIST *my_occur,
  char site )
{

  MOTIF rev_mf;
  
  /*  In ../main/nucl.c:  */
  set_motif_to_reverse_complement( &rev_mf, &(my_occur->my_occur->mf) );
  
  strcpy( my_occur->my_occur->mf.cons, rev_mf.cons );
  my_occur->my_occur->mf.my_motif = rev_mf.my_motif;

}










/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/













void set_window_from_motif( WIND *my_wind, MOTIF *mf )
{

  char cons[ BUFFER ];
  
  /*  In ../main/nucl.c:  */
  str_to_lower( cons, mf->cons );
  
  if (
    ( ! strcmp( cons, "gggcgg" )) || 
    ( ! strcmp( cons, "ccgccc" ))
    ) 
  {
    my_wind->min = GC_BOX_OVERREP_5_PRIME;
    my_wind->max = GC_BOX_OVERREP_3_PRIME;
    return;
  }
  
  error( 3100, "Unrecognized motif.\n" );

}
































