#include "pr.h"

























void print_co_occur_matrix_list( MATRIX_LIST *mx_header, MOTIF *mf, char site, 
  char major_spec[], char minor_spec[], FILE *fh )
{

  print_co_occur_matrix_lister( mx_header, mf, site, major_spec, minor_spec, fh );
  
  print_nucl_freqs_matrix_lister( mx_header, major_spec, minor_spec, fh );
  
  print_sep( fh );
  fprintf( fh, "\n\n\n" );
  
}












void print_co_occur_matrix_lister( MATRIX_LIST *mx, MOTIF *mf, char site,
  char major_spec[], char minor_spec[], FILE *fh )
{

  print_co_occur_matrix( mx->my_matrix, mf, site, major_spec, minor_spec, fh );

}










void print_co_occur_matrix( MATRIX *mx, MOTIF *mf, char site, char major_spec[], 
  char minor_spec[], FILE *fh )
{
  
  print_sep( fh );
  
  fprintf( fh, "  Site :  %d\n", (int)site + 1 );
  
  print_degenerate_mf( mf, site, fh );
  
  fprintf( fh, " Occur :      %d\n", (int)mx->total );
  fprintf( fh, " Genes :      %d\n", (int)mx->genes );
  
  print_dot_sep( fh );
  
  print_my_co_occur_matrix( mx, mf, site, major_spec, minor_spec, fh );
  
  print_dot_sep( fh );
  
  print_frac_matrix( mx, mf, site, major_spec, minor_spec, fh );
  
  print_sep( fh );
  
}












void print_degenerate_mf( MOTIF *mf, char site, FILE *fh )
{

  fprintf( fh, "    MF :      " );
  
  register char i;
  
  for ( i = 0; i < strlen( mf->cons ); i++ ) {
  
    if ( i == site )
      fprintf( fh, "N" );
      
    else
      fprintf( fh, "%c", mf->cons[ i ] );
  
  }
  
  fprintf( fh, "\n" );

}











void print_my_co_occur_matrix( MATRIX *mx, MOTIF *mf, char site, char major_spec[],
  char minor_spec[], FILE *fh )
{

  print_co_occur_matrix_first_lines( mf, site, minor_spec, fh );

  fprintf( fh, "%-8s\n", major_spec );
  
  print_co_occur_matrix_co_occur_elems( mx, fh );

}










void print_co_occur_matrix_first_lines( MOTIF *mf, char site, char spec[], 
  FILE *fh )
{

  pr_blank( fh, 14 );
  
  fprintf( fh, "%-8s\n", spec );
  
  pr_blank( fh, 16 );
  
  fprintf( fh, "A       C       G       T\n" );

}













void print_co_occur_matrix_co_occur_elems( MATRIX *mx, FILE *fh )
{

  register char i;
  
  for ( i = 0; i < mx->i; i++ )
    print_co_occur_matrix_single_line_co_occur_elems( mx, i, fh );

}











void print_co_occur_matrix_single_line_co_occur_elems( MATRIX *mx, char i,
  FILE *fh )
{

  char get_nucl_char_from_bp( MF bp );
  
  /*  In ../main/nucl.x:  */
  fprintf( fh, "     %c   ", toupper( get_nucl_char_from_bp( i )) );
  
  register char j;
  
  for ( j = 0; j < mx->j; j++ )
    fprintf( fh, "   %5d", (int)((mx->mx + (i * mx->j) + j)->x.total) );
  
  fprintf( fh, "\n" );

}









void print_frac_matrix( MATRIX *mx, MOTIF *mf, char site, char major_spec[], 
  char minor_spec[], FILE *fh )
{

  /*  In this file, above:  */
  print_co_occur_matrix_first_lines( mf, site, minor_spec, fh );
  
  fprintf( fh, "%-8s\n", major_spec );
  
  /*  In this file, below:  */
  print_rest_of_frac_matrix( mx, mf, site, major_spec, minor_spec, fh );

}










void print_rest_of_frac_matrix( MATRIX *mx, MOTIF *mf, char site, 
  char major_spec[], char minor_spec[], FILE *fh )
{

  register char i;
  
  for ( i = 0; i < mx->i; i++ )
    print_frac_matrix_single_line_co_occur_elems( mx, i, fh );

}











void print_frac_matrix_single_line_co_occur_elems( MATRIX *mx, char i,
  FILE *fh )
{

  char get_nucl_char_from_bp( MF bp );
  
  /*  In ../main/nucl.x:  */
  fprintf( fh, "     %c   ", toupper( get_nucl_char_from_bp( i )) );
  
  register char j;
  
  for ( j = 0; j < mx->j; j++ )
    fprintf( fh, "   %5.1f", (float)((mx->mx + (i * mx->j) + j)->x.frac) * 100.0 );
  
  fprintf( fh, "\n" );

}













/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/












void print_nucl_freqs_matrix_lister( MATRIX_LIST *mx, char major_spec[], 
  char minor_spec[], FILE *fh )
{
  print_nucl_freqs_matrix( mx->my_matrix, major_spec, minor_spec, fh );
}












void print_nucl_freqs_matrix( MATRIX *mx, char major_spec[], char minor_spec[],
  FILE *fh )
{

  print_nucl_freqs_major_spec( mx, major_spec, fh );
  
  print_dot_sep( fh );
  
  print_nucl_freqs_minor_spec( mx, minor_spec, fh );
  
  print_dot_sep( fh );

}












void print_nucl_freqs_major_spec( MATRIX *mx, char major_spec[], FILE *fh )
{

  fprintf( fh, "%s\n", major_spec );
  
  print_my_nucl_freqs_major_spec( mx, fh );

}













void print_my_nucl_freqs_major_spec( MATRIX *mx, FILE *fh )
{

  register int i;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) 
    print_nucl_freqs_major_spec_single_nucl( mx, i, fh );

}












void print_nucl_freqs_major_spec_single_nucl( MATRIX *mx, int i, FILE *fh )
{

  char get_nucl_char_from_bp( MF bp );
  
  /*  In ../main/nucl.x:  */
  fprintf( fh, "     %c   ", toupper( get_nucl_char_from_bp( (MF)i )) );
  
  /*  In this file, below:  */
  COUNT total = get_total_count_nucl_freqs_row( mx, i );
  STAT freq = get_total_freq_nucl_freqs_row( mx, i );
  
  fprintf( fh, "   %5d  (%.1f%%)\n", (int)total, (float)freq * 100.0 );
  
}











COUNT get_total_count_nucl_freqs_row( MATRIX *mx, int i )
{

  COUNT total = 0;
  
  register char j;
  
  for ( j = 0; j < mx->j; j++ )
    total += (COUNT)((mx->mx + (i * mx->j) + j)->x.total);
    
  return total;

}









STAT get_total_freq_nucl_freqs_row( MATRIX *mx, int i )
{

  STAT freq = 0.0;
  
  register char j;
  
  for ( j = 0; j < NUMB_OF_NUCL; j++ )
    freq += (STAT)((mx->mx + (i * mx->j) + j)->x.frac);
    
  return freq;

}















void print_nucl_freqs_minor_spec( MATRIX *mx, char minor_spec[], FILE *fh )
{

  fprintf( fh, "%s\n", minor_spec );
  
  print_my_nucl_freqs_minor_spec( mx, fh );

}











void print_my_nucl_freqs_minor_spec( MATRIX *mx, FILE *fh )
{

  register char i;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) 
    print_nucl_freqs_minor_spec_single_nucl( mx, i, fh );

}











void print_nucl_freqs_minor_spec_single_nucl( MATRIX *mx, char i, FILE *fh )
{

  char get_nucl_char_from_bp( MF bp );
  
  /*  In ../main/nucl.x:  */
  fprintf( fh, "     %c   ", toupper( get_nucl_char_from_bp( i )) );
  
  COUNT total = get_total_count_nucl_freqs_column( mx, i );
  STAT freq = get_total_freq_nucl_freqs_column( mx, i );
  
  fprintf( fh, "   %5d  (%.1f%%)\n", (int)total, (float)freq * 100.0 );
  
}











COUNT get_total_count_nucl_freqs_column( MATRIX *mx, char j )
{

  COUNT total = 0;
  
  register char i;
  
  for ( i = 0; i < mx->i; i++ )
    total += (mx->mx + (i * mx->j) + j)->x.total;
    
  return total;

}













STAT get_total_freq_nucl_freqs_column( MATRIX *mx, char j )
{

  STAT freq = 0.0;
  
  register char i;
  
  for ( i = 0; i < mx->i; i++ )
    freq += (mx->mx + (i * mx->j) + j)->x.frac;
    
  return freq;

}









/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/












void pr_blank( FILE *fh, int length )
{

  register int i = 0;
  
  while ( i++ < length )
    fprintf( fh, " " );

}
































