#include "my_enrich.h"





















#define INPUT_FNAME "data/ft_hg19/change/hg19.gcBox.count.txt"
#define SPEC_ONE "hg19"
#define SPEC_TWO "hg19"
#define LEAF_LABEL "Human"


#define GO_GENE_FNAME "data/ft_hg18/go/go_genes.txt"

#define ANC_GENE_LIST_BASE "data/ft_hg18/evol/lists/hg18.Mammalia.converted.sorted"
#define BIRD_GENE_LIST_BASE "data/ft_hg18/evol/lists/bird.hg19.converted.sorted"

#define GO_CONDENSED_FNAME "data/ft_hg19/go/go.hg.condensed.txt"
#define BIRD_HUMAN_BASE "data/ft_hg18/evol/lists/bird.hg19.converted.sorted"

#define GO_GENE_FNAME "data/ft_hg18/go/go_genes.txt"
#define GO_TERM_FNAME "data/ft_hg19/go/go.terms.txt"

#define OUTPUT_FNAME "data/ft_hg18/go/go_enrichment.hg18.txt"

#define MY_NUCL C_NUCL






#define MIN_COUNT 10
#define PVAL_THRESH 1e-3
















void do_go_enrichment_counts( void )
{
  
  GENE_SLIST *anc_list = NULL, *node_list = NULL;
  
  anc_list = (GENE_SLIST *)get_gene_slist_for_go_enrichment( anc_list,
    ANC_GENE_LIST_BASE, SPEC_ONE, SPEC_TWO, INPUT_FNAME, MY_NUCL );
    
  node_list = (GENE_SLIST *)get_gene_slist_for_go_enrichment( node_list,
    BIRD_GENE_LIST_BASE, SPEC_ONE, SPEC_TWO, INPUT_FNAME, MY_NUCL );
  
  do_go_enrichment_counts_gene_lists( anc_list, node_list, GO_GENE_FNAME, 
    OUTPUT_FNAME );
  
}












GENE_SLIST *get_gene_slist_for_go_enrichment( GENE_SLIST *my_list, 
  char gene_list_base[], char spec_one[], char spec_two[], char input_fname[], 
  char my_nucl )
{

  GENE_SLIST *get_gene_slist_struct_from_fname( GENE_SLIST *my_list, 
    char fname_base[] );
  SLIST *get_total_genes_from_input_fname( SLIST *total_genes, char spec_one[],
    char spec_two[], char fname[] );
  SLIST *get_genes_w_motif_type( SLIST *gene_header, char fname[], char spec_one[],
    char spec_two[], char my_nucl );

  /*  In list.h:  */
  my_list = (GENE_SLIST *)get_gene_slist_struct_from_fname( my_list,
    gene_list_base );
    
  /*  In ../func/enrich.c:  */
  my_list->total = (SLIST *)get_total_genes_from_input_fname( my_list->total, 
    spec_one, spec_two, input_fname );
  
  /*  In ../func/total_enrich.c:  */
  my_list->total_target = (SLIST *)get_genes_w_motif_type( my_list->total_target,
    input_fname, spec_one, spec_two, my_nucl );  
    
  return my_list;
  
}











void do_go_enrichment_counts_gene_lists( GENE_SLIST *anc_list, 
  GENE_SLIST *node_list, char go_gene_fname[], char output_fname[] )
{

  SLIST *get_go_category_list_w_fname( SLIST *term_header, char fname[] );

  SLIST *term_header = NULL;
  /*  In ../func/enrich.c:  */
  term_header = (SLIST *)get_go_category_list_w_fname( term_header, 
    go_gene_fname );
    
  /*  In this file, below:  */
  do_go_enrichment_counts_w_term_list( anc_list, node_list, term_header,
    go_gene_fname, output_fname );

}













/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/











void do_go_enrichment_counts_w_term_list( GENE_SLIST *anc_list, 
  GENE_SLIST *node_list, SLIST *term_header, char go_gene_fname[],
  char output_fname[] )
{

  GO_CAT_LIST *go_header = NULL;
  go_header = (GO_CAT_LIST *)get_go_enrichment_counts( go_header, anc_list,
    term_header, go_gene_fname );
    
  print_go_enrichment_counts_output_fname( go_header, anc_list, node_list,
    go_gene_fname, output_fname );

}












GO_CAT_LIST *get_go_enrichment_counts( GO_CAT_LIST *go_header, GENE_SLIST *my_list,
  SLIST *term_header, char go_gene_fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( go_gene_fname, "" );
  
  go_header = (GO_CAT_LIST *)get_go_enrichment_counts_fh( go_header, my_list,
    term_header, in );
    
  close_file( in );
  
  return go_header;

}












GO_CAT_LIST *get_go_enrichment_counts_fh( GO_CAT_LIST *go_header, 
  GENE_SLIST *my_list, SLIST *term_header, FILE *in )
{

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, in ) != NULL ) {
  
    if ( line[ 0 ] != '=' )
      error( 14100, "Expected line sep.\n" );
      
    go_header = (GO_CAT_LIST *)get_and_put_go_cat_lister_counts( go_header,
      my_list, term_header, in );
  
  }
  
  return go_header;

}











GO_CAT_LIST *get_and_put_go_cat_lister_counts( GO_CAT_LIST *go_header,
  GENE_SLIST *my_list, SLIST *term_header, FILE *in )
{

  int str_in_my_slist( char str[], SLIST *header );
  GO_CAT_LIST *empty_entry_go_enrich( GO_CAT_LIST *go_header, FILE *in );

  char line[ BUFFER ], label[ BUFFER ];
  
  if ( fgets( line, BUFFER, in ) == NULL )
    error( 13001, "Unexpected EOF.\n" );
    
  sscanf( line, "%s", label );
  
  /*  In ../func/enrich.c:  */
  if ( ! str_in_my_slist( label, term_header ))
    /*  In ../func/enrich.c:  */
    return empty_entry_go_enrich( go_header, in );
    
  /*  In this file, below:  */
  return get_and_put_my_go_cat_lister_counts( go_header, my_list, label, in );

}












GO_CAT_LIST *get_and_put_my_go_cat_lister_counts( GO_CAT_LIST *go_header,
  GENE_SLIST *my_list, char label[], FILE *in )
{

  SLIST *get_go_gene_slist( SLIST *go_gene_header, FILE *in );
  
  printf( "GO cat :  %s\n", label );
  
  SLIST *go_gene_header = NULL;
  /*  In ../func/en_binom.c:  */
  go_gene_header = (SLIST *)get_go_gene_slist( go_gene_header, in );
  
  /*  In this file, below:  */
  go_header = (GO_CAT_LIST *)get_and_put_new_go_cat_lister_counts( go_header,
    my_list, go_gene_header, label, in );
  
  kill_slist( go_gene_header );
  
  return go_header;

}












GO_CAT_LIST *get_and_put_new_go_cat_lister_counts( GO_CAT_LIST *go_header,
  GENE_SLIST *my_list, SLIST *go_gene_header, char label[], FILE *in )
{

  return get_and_put_my_new_go_cat_lister_counts( go_header, my_list->total_target,
    go_gene_header, my_list->total, label );

}











GO_CAT_LIST *get_and_put_my_new_go_cat_lister_counts( GO_CAT_LIST *go_header,
  SLIST *target_header, SLIST *go_gene_header, SLIST *total_header, char label[] )
{

  int get_numb_elem_slist( SLIST *header );
  int get_intersection_slist_pair( SLIST *a_header, SLIST *b_header );
  PVAL get_hyper_geo_pval( int nn, int n, int gg, int g );
  
  /*  In sep_go.c:  */
  if ( get_numb_elem_slist( go_gene_header ) < MIN_COUNT )
    return go_header;

  /*  In sep_go.c:  */
  int g = get_intersection_slist_pair( target_header, go_gene_header );
  int nn = get_numb_elem_slist( total_header );
  int n = get_numb_elem_slist( target_header );
  int gg = get_intersection_slist_pair( go_gene_header, total_header );
  
  /*  In this file, below:  */
  PVAL pval = get_fishers_exact_test_pval( nn, n, gg, g );
  
  if ( pval >= PVAL_THRESH )
    return go_header;
    
  printf( "\t\t\t\t\t\t\t\t%s :  %.2e\n", label, (double)pval );

  int x_genes = g;
  int go_genes = gg;
  int target_genes = n;
  int total_genes = nn;

  /*  In this file, below:  */
  return get_and_put_significant_go_cat_lister_counts( go_header, label, x_genes,
    target_genes, go_genes, total_genes, pval );

}














GO_CAT_LIST *get_and_put_significant_go_cat_lister_counts( GO_CAT_LIST *go_header,
  char label[], int x_genes, int target_genes, int go_genes, int total_genes,
  PVAL pval )
{

  GO_CAT_LIST *create_go_cat_lister( GO_CAT_LIST *xx );
  GO_CAT_LIST *put_go_cat_into_list_binom( GO_CAT_LIST *go_header, GO_CAT_LIST *go );
  
  GO_CAT_LIST *go = NULL;
  go = (GO_CAT_LIST *)create_go_cat_lister( go );
  
  strcpy( go->my_go->acc, label );
  strcpy( go->my_go->label, label );
  
  go->my_go->x_genes = x_genes;
  go->my_go->total_genes = go_genes;
  go->my_go->pval = pval;
  
  go->my_go->exp_numb = (float)(target_genes * go_genes) / (float)(total_genes);
  go->my_go->ratio = (float)(go->my_go->x_genes) / (go->my_go->exp_numb);
  
  /*  In ../func/en_binom.c:  */
  return put_go_cat_into_list_binom( go_header, go );

}













/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/












void print_go_enrichment_counts_output_fname( GO_CAT_LIST *go_header, 
  GENE_SLIST *anc_list, GENE_SLIST *node_list, char go_gene_fname[],
  char output_fname[] )
{

  print_go_enrichment_counts_summary( go_header, anc_list, node_list, 
    go_gene_fname, output_fname );
    
  print_go_enrichment_counts_full( go_header, anc_list, node_list, 
    go_gene_fname, output_fname );

}












void print_go_enrichment_counts_summary( GO_CAT_LIST *go_header, 
  GENE_SLIST *anc_list, GENE_SLIST *node_list, char go_gene_fname[],
  char output_fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *in = NULL, *out = NULL;
  in = (FILE *)open_text_file_reading( go_gene_fname, "" );
  out = (FILE *)open_text_file_writing( output_fname, "" );
  
  print_go_enrichment_counts_summary_fh( go_header, anc_list, node_list, in, out );
  
  close_file( in );
  close_file( out );

}













void print_go_enrichment_counts_summary_fh( GO_CAT_LIST *go_header,
  GENE_SLIST *anc_list, GENE_SLIST *node_list, FILE *in, FILE *out )
{

  GO_CAT_LIST *go = go_header;
  
  while ( go != NULL )
    print_go_enrichment_counts_single_go_and_step( &go, anc_list, node_list,
      in, out );

}











void print_go_enrichment_counts_single_go_and_step( GO_CAT_LIST **go,
  GENE_SLIST *anc_list, GENE_SLIST *node_list, FILE *in, FILE *out )
{
  print_go_enrichment_counts_single_go( *go, anc_list, node_list, in, out );
  *go = (*go)->next;
}












void print_go_enrichment_counts_single_go( GO_CAT_LIST *go, GENE_SLIST *anc_list,
  GENE_SLIST *node_list, FILE *in, FILE *out )
{
  
  int is_cellular_component( char acc[], char fname[] );
  SLIST *get_go_gene_list_from_go_label( SLIST *gene_header, char acc[], FILE *in );
  
  /*  In ../func/en_binom.c:  */
  if ( is_cellular_component( go->my_go->acc, GO_TERM_FNAME ))
    return;
    
  SLIST *go_gene_header = NULL;
  /*  In ../func/en_binom.c:  */
  go_gene_header = (SLIST *)get_go_gene_list_from_go_label( go_gene_header,
    go->my_go->acc, in );
    
  /*  In this file, below:  */
  print_my_go_enrichment_counts_single_go( go, anc_list, node_list, go_gene_header,
    out );
    
  kill_slist( go_gene_header );
  
}












void print_my_go_enrichment_counts_single_go( GO_CAT_LIST *go, 
  GENE_SLIST *anc_list, GENE_SLIST *node_list, SLIST *go_gene_header, FILE *out )
{

  print_sep( out );
  
  print_go_enrichment_counts_header_entry( go, out );
  
  print_go_enrichment_counts_single_go_sep( go, anc_list, node_list, go_gene_header,
    out );

}












void print_go_enrichment_counts_header_entry( GO_CAT_LIST *go, FILE *out )
{

  fprintf( out, "%s\n", go->my_go->acc );

  char go_term[ LINE_BUFFER ];
  
  /*  In sep_go.c:  */  
  get_go_term_label_term_file( go_term, go->my_go->acc, GO_TERM_FNAME );
  fprintf( out, "%s\n", go_term );
  
  print_dot_sep( out );
  
  fprintf( out, "Pval :    %.3e\n", (float)go->my_go->pval );
  fprintf( out, "Genes :   %d\n", go->my_go->x_genes );
  fprintf( out, "Exp :     %.1f\n", go->my_go->exp_numb );
  fprintf( out, "Ratio :   %.3fx\n", go->my_go->ratio );

}














void print_go_enrichment_counts_single_go_sep( GO_CAT_LIST *go, 
  GENE_SLIST *anc_list, GENE_SLIST *node_list, SLIST *go_gene_header, FILE *out )
{

  print_go_enrichment_counts_sep_total_slist( go, anc_list, node_list,
    go_gene_header, anc_list->total, "All genes", out );
    
  print_go_enrichment_counts_sep_total_slist( go, anc_list, node_list,
    go_gene_header, anc_list->list[ ALL_IND ][ ALL_IND ], "Ancestral genes", out );

}












void print_go_enrichment_counts_sep_total_slist( GO_CAT_LIST *go,
  GENE_SLIST *anc_list, GENE_SLIST *node_list, SLIST *go_gene_header,
  SLIST *total_header, char notes[], FILE *out )
{

  print_dot_sep( out );
  
  fprintf( out, "%s...\n", notes );
  fprintf( out, "\n" );
  
  fprintf( out, "%12s   %12s\n", "Ancestor", LEAF_LABEL );
  
  /*  In ../func/en_binom.c:  */
  
  print_separate_slist_ratios( "All", "GC", 
    anc_list->list[ ALL_IND ][ C_IND ], go_gene_header, total_header, out );

  print_separate_slist_ratios( "GA", "GC", 
    anc_list->list[ A_IND ][ C_IND ], go_gene_header, total_header, out );

  print_separate_slist_ratios( "(No GA)", "GC", 
    anc_list->list[ NO_A_IND ][ C_IND ], go_gene_header, total_header, out );

  print_separate_slist_ratios( "GA", "(No GC)", 
    anc_list->list[ A_IND ][ NO_C_IND ], go_gene_header, total_header, out );

  print_separate_slist_ratios( "GA", "GA", 
    anc_list->list[ A_IND ][ A_IND ], go_gene_header, total_header, out );

  print_separate_slist_ratios( "GC", "GC", 
    anc_list->list[ C_IND ][ C_IND ], go_gene_header, total_header, out );

  print_separate_slist_ratios( "GC (No GA)", "GC", 
    anc_list->list[ C_NO_A_IND ][ C_IND ], go_gene_header, total_header, out );

  print_separate_slist_ratios( "GA (No GC)", "GC", 
    anc_list->list[ A_NO_C_IND ][ C_IND ], go_gene_header, total_header, out );
    
  print_separate_slist_ratios( "GA & GC", "GC",
    anc_list->list[ A_AND_C_IND ][ C_IND ], go_gene_header, total_header, out );
    
  print_separate_slist_ratios( "GA & GC", "GA & GC",
    anc_list->list[ A_AND_C_IND ][ A_AND_C_IND ], go_gene_header, total_header, 
      out );

  print_separate_slist_ratios( "Bird GC", "GC", 
    node_list->list[ C_IND ][ C_IND ], go_gene_header, total_header, out );

}












void print_go_enrichment_counts_full( GO_CAT_LIST *go_header, 
  GENE_SLIST *anc_list, GENE_SLIST *node_list, char go_gene_fname[], 
  char output_fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *in = NULL, *out = NULL;
  in = (FILE *)open_text_file_reading( go_gene_fname, "" );
  out = (FILE *)open_text_file_writing( output_fname, "full.txt" );
  
  print_go_enrichment_counts_full_fh( go_header, anc_list, node_list, in, out );
  
  close_file( in );
  close_file( out );

}













void print_go_enrichment_counts_full_fh( GO_CAT_LIST *go_header, 
  GENE_SLIST *anc_list, GENE_SLIST *node_list, FILE *in, FILE *out )
{

  GO_CAT_LIST *go = go_header;
  
  while ( go != NULL )
    print_go_enrichment_counts_full_single_go_and_step( &go, anc_list,
      node_list, in, out );

}











void print_go_enrichment_counts_full_single_go_and_step( GO_CAT_LIST **go,
  GENE_SLIST *anc_list, GENE_SLIST *node_list, FILE *in, FILE *out )
{
  print_go_enrichment_counts_full_single_go( *go, anc_list, node_list, in, out );
  *go = (*go)->next;
}











void print_go_enrichment_counts_full_single_go( GO_CAT_LIST *go,
  GENE_SLIST *anc_list, GENE_SLIST *node_list, FILE *in, FILE *out )
{
  
  int is_cellular_component( char acc[], char fname[] );
  SLIST *get_go_gene_list_from_go_label( SLIST *gene_header, char acc[], FILE *in );
  
  /*  In ../func/en_binom.c:  */
  if ( is_cellular_component( go->my_go->acc, GO_TERM_FNAME ))
    return;
    
  SLIST *go_gene_header = NULL;
  /*  In ../func/en_binom.c:  */
  go_gene_header = (SLIST *)get_go_gene_list_from_go_label( go_gene_header,
    go->my_go->acc, in );
    
  /*  In this file, below:  */
  print_my_go_enrichment_counts_full_single_go( go, anc_list, node_list, 
    go_gene_header, out );
    
  kill_slist( go_gene_header );
  
}











void print_my_go_enrichment_counts_full_single_go( GO_CAT_LIST *go,
  GENE_SLIST *anc_list, GENE_SLIST *node_list, SLIST *go_gene_header, FILE *out )
{

  print_sep( out );
  
  /*  In this file, above:  */
  print_go_enrichment_counts_header_entry( go, out );
  
  /*  In this file, below:  */
  print_go_enrichment_counts_full_body( go, anc_list, node_list, go_gene_header,
    out );
  
}












void print_go_enrichment_counts_full_body( GO_CAT_LIST *go, GENE_SLIST *anc_list,
  GENE_SLIST *node_list, SLIST *go_gene_header, FILE *out )
{

  print_go_enrichment_counts_full_body_single_node( go, anc_list, go_gene_header,
    "Mammalian ancestor", out );
    
  print_go_enrichment_counts_full_body_single_node( go, node_list, go_gene_header,
    "Bird", out );

}











void print_go_enrichment_counts_full_body_single_node( GO_CAT_LIST *go,
  GENE_SLIST *my_list, SLIST *go_gene_header, char notes[], FILE *out )
{

  print_dot_sep( out );

  fprintf( out, "Comparison node :  %s\n", notes );
  fprintf( out, "\n" );
  
  int i, j;
  
  for ( i = 0; i < SLIST_BUFFER; i++ ) {
  
    for ( j = 0; j < SLIST_BUFFER; j++ ) {
    
      /*  In ../func/en_binom.c:  */
      print_separate_slist_ratios( my_list->label[ i ], my_list->label[ j ],
        my_list->list[ i ][ j ], go_gene_header, my_list->total, out );
    
    }
  
  }

}








/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/













PVAL get_fishers_exact_test_pval( int nn, int n, int gg, int g )
{

  PVAL get_hyper_geo_log_prob( int nn, int n, int gg, int g );

  PVAL pval = 0.0;
  
  int a = g;
  int b = n - g;
  int c = gg - g;
  int d = nn - a - b - c;
  
  while (( b >= 0 ) && ( c >= 0 )) {
  
    int my_g = a;
    int my_gg = a + b;
    int my_n = a + c;
    int my_nn = nn;
    
    /*  In ../gene/sig.c:  */
    pval += exp( get_hyper_geo_log_prob( my_nn, my_n, my_gg, my_g ));
    
    ++a;
    ++d;
    --b;
    --c;
  
  }
  
  return pval;

}



















