#include "anc.h"















#define INPUT_COL_FNAME "data/ft_hg19/change/hg19.gcBox.count.txt"

#define MAM_FNAME "data/ft_hg19/evol/ancestor/mammal_ancestor"
#define EUTH_FNAME "data/ft_hg19/evol/ancestor/euth_ancestor"
#define BIRD_FNAME "data/ft_hg19/evol/ancestor/bird_ancestor"

#define OUTPUT_FNAME "data/ft_hg19/evol/ancestor/ancestor_info.txt"



#define OUTPUT_MF_BUFFER 7



#define ANCESTOR_NODE_LABEL "Mammalia"



#define PARAM_TYPE_UNSET -1
#define EUTH_INDEX 0
#define CONVERT_INDEX 1
#define ANCESTOR_INDEX 2
#define NUMB_OF_PARAMS 3



#define EUTH_GC_ALPHA 1.11e-11
#define EUTH_GC_BETA 1.08e-8
#define CONV_GC_ALPHA 4.37e-11
#define CONV_GC_BETA 8.68e-10
#define ANC_GC_ALPHA 1.19e-11
#define ANC_GC_BETA 7.58e-9

#define EUTH_GA_ALPHA 1.02e-11
#define EUTH_GA_BETA 1.02e-8
#define CONV_GA_ALPHA 1.78e-11
#define CONV_GA_BETA 4.43e-9
#define ANC_GA_ALPHA 1.66e-11
#define ANC_GA_BETA 7.74e-9

#define EUTH_GT_ALPHA 7.58e-12
#define EUTH_GT_BETA 9.66e-9
#define CONV_GT_ALPHA 9.61e-12
#define CONV_GT_BETA 6.28e-9
#define ANC_GT_ALPHA 7.03e-12
#define ANC_GT_BETA 7.84e-9



#define PRESENT_THRESH 0.5











void do_ancestor( void )
{

  FH my_fh;
  
  set_ancestor_fhs( &my_fh );
  
  do_my_ancestor( &my_fh );
  
  close_ancestor_fhs( &my_fh );
  
  do_ancestor_conversions();

}











void set_ancestor_fhs( FH *my_fh )
{
  
  FILE *open_text_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );
  
  my_fh->in = (FILE *)open_text_file_reading( INPUT_COL_FNAME, "" );
  
  my_fh->mam_gc_out = (FILE *)open_text_file_writing( MAM_FNAME, "GCbox.txt" );
  my_fh->mam_ga_out = (FILE *)open_text_file_writing( MAM_FNAME, "GAbox.txt" );
  my_fh->mam_gt_out = (FILE *)open_text_file_writing( MAM_FNAME, "GTbox.txt" );
  
  my_fh->euth_gc_out = (FILE *)open_text_file_writing( EUTH_FNAME, "GCbox.txt" );
  my_fh->euth_ga_out = (FILE *)open_text_file_writing( EUTH_FNAME, "GAbox.txt" );
  my_fh->euth_gt_out = (FILE *)open_text_file_writing( EUTH_FNAME, "GTbox.txt" );

  my_fh->bird_gc_out = (FILE *)open_text_file_writing( BIRD_FNAME, "GCbox.txt" );
  my_fh->bird_ga_out = (FILE *)open_text_file_writing( BIRD_FNAME, "GAbox.txt" );
  my_fh->bird_gt_out = (FILE *)open_text_file_writing( BIRD_FNAME, "GTbox.txt" );
  
}









void close_ancestor_fhs( FH *my_fh )
{

  close_file( my_fh->in );
  
  close_file( my_fh->mam_gc_out );
  close_file( my_fh->mam_ga_out );
  close_file( my_fh->mam_gt_out );
  
  close_file( my_fh->euth_gc_out );
  close_file( my_fh->euth_ga_out );
  close_file( my_fh->euth_gt_out );
  
  close_file( my_fh->bird_gc_out );
  close_file( my_fh->bird_ga_out );
  close_file( my_fh->bird_gt_out );

}














void do_my_ancestor( FH *my_fh )
{

  ND_LIST *get_evol_tree( ND_LIST *root );
  ND_LIST *get_nd_leaf_header( ND_LIST *node );

  ND_LIST *root = NULL, *leaf_header = NULL;

  /*  In tree.c:  */
  root = (ND_LIST *)get_evol_tree( root );
  leaf_header = (ND_LIST *)get_nd_leaf_header( root );
  
  /*  In this file, below:  */
  do_ancestor_w_tree( root, leaf_header, my_fh );
  
}











void do_ancestor_w_tree( ND_LIST *root, ND_LIST *leaf_header, FH *my_fh )
{

  PARAM *gc_param = NULL, *ga_param = NULL, *gt_param = NULL;
  gc_param = (PARAM *)make_and_set_gc_params( gc_param );
  ga_param = (PARAM *)make_and_set_ga_params( ga_param );
  gt_param = (PARAM *)make_and_set_gt_params( gt_param );

  do_ancestor_w_tree_single_nucl( root, leaf_header, C_NUCL, gc_param, my_fh->in,
    my_fh->mam_gc_out, my_fh->euth_gc_out, my_fh->bird_gc_out );
    
  do_ancestor_w_tree_single_nucl( root, leaf_header, A_NUCL, ga_param, my_fh->in,
    my_fh->mam_ga_out, my_fh->euth_ga_out, my_fh->bird_ga_out );

  do_ancestor_w_tree_single_nucl( root, leaf_header, T_NUCL, gt_param, my_fh->in,
    my_fh->mam_gt_out, my_fh->euth_gt_out, my_fh->bird_gt_out );

  free( gc_param );
  free( ga_param );
  free( gt_param );

}













PARAM *make_and_set_gc_params( PARAM *param )
{

  param = (PARAM *)malloc( sizeof(PARAM) * NUMB_OF_PARAMS );

  (param + EUTH_INDEX)->alpha = EUTH_GC_ALPHA;
  (param + EUTH_INDEX)->beta = EUTH_GC_BETA;
  
  (param + CONVERT_INDEX)->alpha = CONV_GC_ALPHA;
  (param + CONVERT_INDEX)->beta = CONV_GC_BETA;
  
  (param + ANCESTOR_INDEX)->alpha = ANC_GC_ALPHA;
  (param + ANCESTOR_INDEX)->beta = ANC_GC_BETA;
  
  return param;

}











PARAM *make_and_set_ga_params( PARAM *param )
{

  param = (PARAM *)malloc( sizeof(PARAM) * NUMB_OF_PARAMS );

  (param + EUTH_INDEX)->alpha = EUTH_GA_ALPHA;
  (param + EUTH_INDEX)->beta = EUTH_GA_BETA;
  
  (param + CONVERT_INDEX)->alpha = CONV_GA_ALPHA;
  (param + CONVERT_INDEX)->beta = CONV_GA_BETA;
  
  (param + ANCESTOR_INDEX)->alpha = ANC_GA_ALPHA;
  (param + ANCESTOR_INDEX)->beta = ANC_GA_BETA;
  
  return param;

}










PARAM *make_and_set_gt_params( PARAM *param )
{

  param = (PARAM *)malloc( sizeof(PARAM) * NUMB_OF_PARAMS );

  (param + EUTH_INDEX)->alpha = EUTH_GT_ALPHA;
  (param + EUTH_INDEX)->beta = EUTH_GT_BETA;
  
  (param + CONVERT_INDEX)->alpha = CONV_GT_ALPHA;
  (param + CONVERT_INDEX)->beta = CONV_GT_BETA;
  
  (param + ANCESTOR_INDEX)->alpha = ANC_GT_ALPHA;
  (param + ANCESTOR_INDEX)->beta = ANC_GT_BETA;
  
  return param;

}










void do_ancestor_w_tree_single_nucl( ND_LIST *root, ND_LIST *leaf_header,
  char my_nucl, PARAM *param, FILE *in, FILE *mam_out, FILE *euth_out, 
  FILE *bird_out )
{

  /*  In this file, below:  */
  set_prior_probs_ancestor_tree( root, leaf_header, my_nucl, in );
  
  /*  In this file, below:  */
  set_ancestor_tree_types( root );
  
  /*  In mammal.c:  */
  put_trans_probs_into_tree_varying_params( root, param );
  
  /*  In this file, below:  */
  get_ancestor_tree_single_nucl( root, leaf_header, my_nucl, param, in, mam_out,
    euth_out, bird_out );

}











void set_prior_probs_ancestor_tree( ND_LIST *root, ND_LIST *leaf_header,
  char my_nucl, FILE *in )
{

  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );

  ND_LIST *mam = NULL, *euth = NULL, *bird = NULL;
  ND_LIST *hg = NULL, *opossum = NULL, *chicken = NULL;
  /*  In tree.c:  */
  mam = (ND_LIST *)get_nd_lister_from_label( root, ANCESTOR_NODE_LABEL );
  euth = (ND_LIST *)get_nd_lister_from_label( root, "Eutheria" );
  bird = (ND_LIST *)get_nd_lister_from_label( root, "Neognathae" );
  hg = (ND_LIST *)get_nd_lister_from_label( root, "hg19" );
  opossum = (ND_LIST *)get_nd_lister_from_label( root, "monDom5" );
  chicken = (ND_LIST *)get_nd_lister_from_label( root, "galGal3" );
  
  set_position( 0, in );
  
  /*  In my_lhood.c:  */
  set_prior_probs_for_lhood_step_ancestor( root, leaf_header, mam, opossum,
    my_nucl, in );
  
  set_position( 0, in );
  
  /*  In my_lhood.c:  */
  set_prior_probs_for_lhood_step_ancestor( root, leaf_header, euth, hg, my_nucl,
    in );
    
  set_position( 0, in );
  
  /*  In my_lhood.c:  */
  set_prior_probs_for_lhood_step_ancestor( root, leaf_header, bird, chicken,
    my_nucl, in );
      
}










void set_ancestor_tree_types( ND_LIST *root )
{

  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );
  
  ND_LIST *euth = NULL, *bird = NULL;
  
  /*  In tree.c:  */
  euth = (ND_LIST *)get_nd_lister_from_label( root, "Eutheria" );
  bird = (ND_LIST *)get_nd_lister_from_label( root, "Neognathae" );
  
  /*  In mammal.c:  */
  set_param_types_descendent_nodes( root, ANCESTOR_INDEX );
  set_param_types_descendent_nodes( euth, EUTH_INDEX );
  set_param_types_descendent_nodes( bird, EUTH_INDEX );
  
  euth->my_node->param_type = CONVERT_INDEX;
  
  char i = 0;
  
  while ( euth->down[ i ] != NULL )
    euth->down[ i++ ]->my_node->param_type = CONVERT_INDEX;
  
}










void get_ancestor_tree_single_nucl( ND_LIST *root, ND_LIST *leaf_header, 
  char my_nucl, PARAM *param, FILE *in, FILE *mam_out, FILE *euth_out,
  FILE *bird_out )
{

  STR_LIST *get_label_list_from_first_line( STR_LIST *label_header, FILE *in );

  STR_LIST *label_header = NULL;

  set_position( 0, in );
  
  /*  In set.c:  */
  label_header = (STR_LIST *)get_label_list_from_first_line( label_header, in );
  
  /*  In this file, below:  */
  get_my_ancestor_tree_single_nucl( root, leaf_header, label_header, param,
    my_nucl, in, mam_out, euth_out, bird_out );
    
}










void get_my_ancestor_tree_single_nucl( ND_LIST *root, ND_LIST *leaf_header,
  STR_LIST *label_header, PARAM *param, char my_nucl, FILE *in, FILE *mam_out,
  FILE *euth_out, FILE *bird_out )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL )
    get_ancestor_tree_single_gene( root, leaf_header, label_header, my_nucl, param, 
      line, mam_out, euth_out, bird_out );

}













void get_ancestor_tree_single_gene( ND_LIST *root, ND_LIST *leaf_header,
  STR_LIST *label_header, char my_nucl, PARAM *param, char line[], FILE *mam_out, 
  FILE *euth_out, FILE *bird_out )
{

  char gene[ BUFFER ], acc[ BUFFER ];
  int numb;
  
  sscanf( line, "%d%s%s", &numb, acc, gene );
  
  printf( "%5d :  %s\n", numb, gene );

  /*  In set.c:  */
  initialize_nd_tree( root );
  
  /*  In set.c:  */
  set_my_evolutionary_tree_single_gene( root, leaf_header, label_header,
    line, my_nucl );
    
  /*  In this file, below:  */
  fill_phylo_ancestor( root );
  
  /*  In this file, below:  */
  put_ancestor_tree_probs_single_gene( root, numb, acc, gene, mam_out, euth_out,
    bird_out );
    
}













void fill_phylo_ancestor( ND_LIST *node )
{

  if ( node->down[ 0 ] == NULL )
    return;
    
  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    fill_phylo_ancestor( node->down[ i++ ] );
    
  fill_phylo_ancestor_single_node( node );

}












void fill_phylo_ancestor_single_node( ND_LIST *node )
{

  if ( node_is_absent_or_euth( node )) {
    node->my_node->exists = 0;
    return;
  }
  
  fill_my_phylo_ancestor_single_node( node );
  
  node->my_node->exists = 1;

}










int node_is_absent_or_euth( ND_LIST *node )
{

  register char i = 0;
  
  while ( node->down[ i ] != NULL ) {

    ND_LIST *nn = node->down[ i++ ];

    if (( nn->my_node->exists ) && ( strcmp( nn->my_node->label, "Eutheria" )))
      return 0;

  }
  
  return 1;

}











void fill_my_phylo_ancestor_single_node( ND_LIST *node )
{

  register char i;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ )
    fill_phylo_ancestor_single_numb_of_mfs( node, i );

}












void fill_phylo_ancestor_single_numb_of_mfs( ND_LIST *node, char i )
{

  TT fill_single_node_single_numb_of_mfs_single_down( ND_LIST *node,
    PARAM *param, char i );

  TT pr = 1.0;
  
  register char k = 0;
  
  while ( node->down[ k ] != NULL ) {
  
    ND_LIST *nn = node->down[ k++ ];
    
    if ( ! strcmp( nn->my_node->label, "Eutheria" ))
      continue;
  
    /*  In v.c:  */    
    pr *= fill_single_node_single_numb_of_mfs_single_down( nn, NULL, i );
  
  }
  
  *(node->my_node->data->pr->x + i) = pr;
  
}











void fill_phylo_ancestor_birds( ND_LIST *root )
{

  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );
  
  return;

  ND_LIST *bird = NULL, *chicken = NULL, *zf = NULL;
  bird = (ND_LIST *)get_nd_lister_from_label( root, "Neognathae" );
  chicken = (ND_LIST *)get_nd_lister_from_label( root, "galGal3" );
  zf = (ND_LIST *)get_nd_lister_from_label( root, "taeGut1" );
  
  if (( ! chicken->my_node->exists ) && ( zf->my_node->exists ))
    set_mf_occur_numb( bird, zf );
    
  if (( ! zf->my_node->exists ) && ( chicken->my_node->exists ))
    set_mf_occur_numb( bird, chicken );

}












void set_mf_occur_numb( ND_LIST *node, ND_LIST *nn )
{

  TT_MX *pr = node->my_node->data->pr;
  TT_MX *pp = nn->my_node->data->pr;

  register char k;
  
  for ( k = 0; k < NUMB_OF_MFS; k++ )
    *(pr->x + k) = *(pp->x + k);

}












void fill_phylo_ancestor_mammals( ND_LIST *root )
{

  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );
  
  return;
  
  ND_LIST *mam = NULL, *platypus = NULL, *opossum = NULL, *wallaby = NULL;
  mam = (ND_LIST *)get_nd_lister_from_label( root, ANCESTOR_NODE_LABEL );
  platypus = (ND_LIST *)get_nd_lister_from_label( root, "ornAna1" );
  opossum = (ND_LIST *)get_nd_lister_from_label( root, "monDom5" );
  wallaby = (ND_LIST *)get_nd_lister_from_label( root, "macEug1" );
  
  if ( first_only_exists( platypus, opossum, wallaby ))  
    set_mf_occur_numb( mam, platypus );

  if ( first_only_exists( opossum, platypus, wallaby ))
    set_mf_occur_numb( mam, opossum );
    
  if ( first_only_exists( wallaby, platypus, opossum ))
    set_mf_occur_numb( mam, wallaby );

}










int first_only_exists( ND_LIST *a, ND_LIST *b, ND_LIST *c )
{

  if ( ! a->my_node->exists )
    return 0;
    
  if ( b->my_node->exists )
    return 0;
    
  if ( c->my_node->exists )
    return 0;
    
  return 1;

}








void put_ancestor_tree_probs_single_gene( ND_LIST *root, int numb, char acc[], 
  char gene[], FILE *mam_out, FILE *euth_out, FILE *bird_out )
{

  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );

  ND_LIST *mam = NULL, *euth = NULL, *bird = NULL;
  /*  In tree.c:  */
  mam = (ND_LIST *)get_nd_lister_from_label( root, ANCESTOR_NODE_LABEL );
  euth = (ND_LIST *)get_nd_lister_from_label( root, "Eutheria" );
  bird = (ND_LIST *)get_nd_lister_from_label( root, "Neognathae" );
  
  /*  In this file, below:  */
  put_ancestor_tree_probs_single_node( mam, numb, acc, gene, mam_out );
  put_ancestor_tree_probs_single_node( euth, numb, acc, gene, euth_out );
  put_ancestor_tree_probs_single_node( bird, numb, acc, gene, bird_out );

}












void put_ancestor_tree_probs_single_node( ND_LIST *node, int numb, char acc[],
  char gene[], FILE *out )
{

  fprintf( out, "%6d\t%s\t%12s\t", numb, acc, gene );
  
  register char k;
  
  if ( ! node->my_node->exists ) {
    for ( k = 0; k < OUTPUT_MF_BUFFER; k++ )
      fprintf( out, "%10s\t", "---" );
    fprintf( out, "\n" );
    return;
  }

  TT_MX *prior = node->my_node->data->prior;
  TT_MX *lhood = node->my_node->data->pr;

  float array[ NUMB_OF_MFS ], sum = 0.0;
  
  for ( k = 0; k < NUMB_OF_MFS; k++ ) {
    array[ k ] = (float)*(lhood->x + k);
    sum += array[ k ];
  }
  
  for ( k = 0; k < OUTPUT_MF_BUFFER; k++ ) 
    fprintf( out, "%10.3e\t", array[ k ] / sum );
    
  fprintf( out, "\n" );

}













/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/















void do_ancestor_conversions( void )
{

  FH my_fh;
  INFO my_info;
  
  set_ancestor_conversions_fh( &my_fh );
  
  initialize_ancestor_conversions_info( &my_info );
  
  do_my_ancestor_conversions( &my_info, &my_fh );
  
  print_ancestor_conversion_info( &my_info, &my_fh );
  
  close_ancestor_conversions_fh( &my_fh );

}









void set_ancestor_conversions_fh( FH *my_fh )
{

  FILE *open_text_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );
  
  my_fh->in = (FILE *)open_text_file_reading( INPUT_COL_FNAME, "" );
  
  my_fh->mam_gc_out = (FILE *)open_text_file_reading( MAM_FNAME, "GCbox.txt" );
  my_fh->mam_ga_out = (FILE *)open_text_file_reading( MAM_FNAME, "GAbox.txt" );
  my_fh->mam_gt_out = (FILE *)open_text_file_reading( MAM_FNAME, "GTbox.txt" );
  
  my_fh->euth_gc_out = (FILE *)open_text_file_reading( EUTH_FNAME, "GCbox.txt" );
  my_fh->euth_ga_out = (FILE *)open_text_file_reading( EUTH_FNAME, "GAbox.txt" );
  my_fh->euth_gt_out = (FILE *)open_text_file_reading( EUTH_FNAME, "GTbox.txt" );
  
  my_fh->bird_gc_out = (FILE *)open_text_file_reading( BIRD_FNAME, "GCbox.txt" );
  my_fh->bird_ga_out = (FILE *)open_text_file_reading( BIRD_FNAME, "GAbox.txt" );
  my_fh->bird_gt_out = (FILE *)open_text_file_reading( BIRD_FNAME, "GTbox.txt" );
  
  my_fh->out = (FILE *)open_text_file_writing( OUTPUT_FNAME, "" );

}









void close_ancestor_conversions_fh( FH *my_fh )
{

  close_file( my_fh->mam_gc_out );
  close_file( my_fh->mam_ga_out );
  close_file( my_fh->mam_gt_out );
  
  close_file( my_fh->euth_gc_out );
  close_file( my_fh->euth_ga_out );
  close_file( my_fh->euth_gt_out );
  
  close_file( my_fh->bird_gc_out );
  close_file( my_fh->bird_ga_out );
  close_file( my_fh->bird_gt_out );
  
  close_file( my_fh->out );

}












void initialize_ancestor_conversions_info( INFO *my_info )
{

  my_info->mammal_gc = 0;
  my_info->mammal_ga = 0;
  my_info->euth_gc = 0;
  my_info->euth_ga = 0;
  my_info->bird_gc = 0;
  my_info->bird_ga = 0;
  
  my_info->total_genes = 0;
  
  my_info->mammal_exists = 0;
  my_info->bird_exists = 0;
  my_info->mammal_bird_exists = 0;
  
  my_info->mammal_gc_euth_gc = 0;
  my_info->mammal_gc_euth_ga = 0;
  my_info->mammal_ga_euth_gc = 0;
  my_info->mammal_ga_euth_ga = 0;
  
  my_info->mammal_gc_hg_gc = 0;
  my_info->mammal_gc_hg_ga = 0;
  my_info->mammal_ga_hg_gc = 0;
  my_info->mammal_ga_hg_ga = 0;
  
  my_info->mammal_gc_only_euth_gc = 0;
  my_info->mammal_gc_only_euth_ga = 0;
  my_info->mammal_ga_only_euth_gc = 0;
  my_info->mammal_ga_only_euth_ga = 0;
  
  my_info->mammal_gc_only_hg_gc = 0;
  my_info->mammal_gc_only_hg_ga = 0;
  my_info->mammal_ga_only_hg_gc = 0;
  my_info->mammal_ga_only_hg_ga = 0;
  
  char i, j, k;
  
  for ( i = 0; i < SP_BUFFER; i++ ) {
  
    my_info->mam_bird_exists[ i ] = 0;

    for ( j = 0; j < SP_BUFFER; j++ ) {

      my_info->mam_euth_bird_exists[ i ][ j ] = 0;
      my_info->mam_bird[ i ][ j ] = 0;

      for ( k = 0; k < SP_BUFFER; k++ ) {
        my_info->bird_array[ i ][ j ][ k ] = 0;
        my_info->bird_exp[ i ][ j ][ k ] = 0.0;
      }

    }

  }

}











void do_my_ancestor_conversions( INFO *my_info, FH *my_fh )
{

  ND_LIST *get_evol_tree( ND_LIST *root );
  ND_LIST *get_nd_leaf_header( ND_LIST *node );

  ND_LIST *root = NULL, *leaf_header = NULL;

  /*  In tree.c:  */
  root = (ND_LIST *)get_evol_tree( root );
  leaf_header = (ND_LIST *)get_nd_leaf_header( root );
  
  /*  In this file, below:  */
  do_ancestor_conversions_w_tree( my_info, root, leaf_header, my_fh );
  
}














void do_ancestor_conversions_w_tree( INFO *my_info, ND_LIST *root, 
  ND_LIST *leaf_header, FH *my_fh )
{

  STR_LIST *get_label_list_from_first_line( STR_LIST *label_header, FILE *in );

  STR_LIST *label_header = NULL;

  set_position( 0, my_fh->in );
  
  /*  In set.c:  */
  label_header = (STR_LIST *)get_label_list_from_first_line( label_header, 
    my_fh->in );
  
  /*  In this file, below:  */
  do_my_ancestor_conversions_w_tree( my_info, root, leaf_header, label_header,
    my_fh );
    
}












void do_my_ancestor_conversions_w_tree( INFO *my_info, ND_LIST *root, 
  ND_LIST *leaf_header, STR_LIST *label_header, FH *my_fh )
{

  while ( get_ancestor_lines( my_info, my_fh ))
    do_ancestor_conversion_single_gene( my_info, root, leaf_header, label_header );

}











int get_ancestor_lines( INFO *my_info, FH *my_fh )
{

  if ( fgets( my_info->mammal_gc_line, LINE_BUFFER, my_fh->mam_gc_out ) == NULL )
    return 0;
    
  fgets( my_info->mammal_ga_line, LINE_BUFFER, my_fh->mam_ga_out );
  fgets( my_info->mammal_gt_line, LINE_BUFFER, my_fh->mam_gt_out );
  
  fgets( my_info->euth_gc_line, LINE_BUFFER, my_fh->euth_gc_out );
  fgets( my_info->euth_ga_line, LINE_BUFFER, my_fh->euth_ga_out );
  fgets( my_info->euth_gt_line, LINE_BUFFER, my_fh->euth_gt_out );
  
  fgets( my_info->bird_gc_line, LINE_BUFFER, my_fh->bird_gc_out );
  fgets( my_info->bird_ga_line, LINE_BUFFER, my_fh->bird_ga_out );
  fgets( my_info->bird_gt_line, LINE_BUFFER, my_fh->bird_gt_out );
  
  fgets( my_info->input_line, LINE_BUFFER, my_fh->in );
  
  return 1;

}










void do_ancestor_conversion_single_gene( INFO *my_info, ND_LIST *root, 
  ND_LIST *leaf_header, STR_LIST *label_header )
{

  CELL present, ortho;
  
  get_ancestor_conversion_single_gene_present( &present, my_info );

  get_ancestor_conversion_single_gene_spec( &present, root, leaf_header,
    label_header, my_info );
  
  get_ancestor_conversion_single_gene_ortho( &ortho, my_info );
  
  do_my_ancestor_conversion_single_gene( my_info, &present, &ortho );

}











void get_ancestor_conversion_single_gene_present( CELL *x, INFO *my_info )
{
  
  x->mammal_gc = is_present( my_info->mammal_gc_line );
  x->mammal_ga = is_present( my_info->mammal_ga_line );
  x->mammal_gt = is_present( my_info->mammal_gt_line );
  
  x->euth_gc = is_present( my_info->euth_gc_line );
  x->euth_ga = is_present( my_info->euth_ga_line );
  x->euth_gt = is_present( my_info->euth_gt_line );
  
  x->bird_gc = is_present( my_info->bird_gc_line );
  x->bird_ga = is_present( my_info->bird_ga_line );
  x->bird_gt = is_present( my_info->bird_gt_line );
  
}














void get_ancestor_conversion_single_gene_spec( CELL *x, ND_LIST *root, 
  ND_LIST *leaf_header, STR_LIST *label_header, INFO *my_info )
{

  x->hg_gc = is_present_spec( my_info->input_line, root, leaf_header,
    label_header, C_NUCL );

  x->hg_ga = is_present_spec( my_info->input_line, root, leaf_header,
    label_header, A_NUCL );

  x->hg_gt = is_present_spec( my_info->input_line, root, leaf_header,
    label_header, T_NUCL );

}












void get_ancestor_conversion_single_gene_ortho( CELL *x, INFO *my_info )
{
  x->mammal_gc = is_ortho( my_info->mammal_gc_line );
  x->mammal_ga = is_ortho( my_info->mammal_ga_line );
  x->euth_gc = is_ortho( my_info->euth_gc_line );
  x->euth_ga = is_ortho( my_info->euth_ga_line );
  x->bird_gc = is_ortho( my_info->bird_gc_line );
  x->bird_ga = is_ortho( my_info->bird_ga_line );
}













int is_present( char line[] )
{

  if ( ! is_ortho( line ))
    return 0;
    
  float x0, x1, x2, x3, x4, x5, x6;
  
  sscanf( line, "%*d%*s%*s%f%f%f%f%f%f", &x0, &x1, &x2, &x3, &x4, &x5, &x6 );
  
  if ( x0 < x1 )  return 1;
  if ( x0 < x2 )  return 1;
  if ( x0 < x3 )  return 1;
  if ( x0 < x4 )  return 1;
  if ( x0 < x5 )  return 1;
  if ( x0 < x6 )  return 1;
  
  return 0;

}










int is_ortho( char line[] )
{

  char entry[ BUFFER ];
  
  sscanf( line, "%*d%*s%*s%s", entry );
  
  return ( strcmp( entry, "---" )) ? (1) : (0);

}











int is_present_spec( char line[], ND_LIST *root, ND_LIST *leaf_header,
  STR_LIST *label_header, char my_nucl )
{

  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );
  
  ND_LIST *hg = NULL;
  /*  In tree.c:  */
  hg = (ND_LIST *)get_nd_lister_from_label( root, "hg19" );

  /*  In set.c:  */
  initialize_nd_tree( root );
  
  /*  In set.c:  */
  set_my_evolutionary_tree_single_gene( root, leaf_header, label_header,
    line, my_nucl );
    
  /*  In this file, below:  */
  return is_present_node( hg );

}











int is_present_node( ND_LIST *node )
{

  TT_MX *pr = node->my_node->data->pr;
  
  if ( *(pr->x) == 1.0 )
    return 0;
    
  return 1;

}












void do_my_ancestor_conversion_single_gene( INFO *my_info, CELL *present, 
  CELL *ortho )
{

  ++(my_info->total_genes);

  update_ancestor_info_ortho( my_info, ortho );
  update_ancestor_info_present( my_info, present );
  update_ancestor_info_mammal_euth( my_info, present );
  update_bird_info( my_info, present, ortho );

}














void update_ancestor_info_ortho( INFO *my_info, CELL *ortho )
{
  my_info->mammal_exists += ortho->mammal_gc;
  my_info->bird_exists += ortho->bird_gc;
  my_info->mammal_bird_exists += ortho->mammal_gc * ortho->bird_gc;
}












void update_ancestor_info_present( INFO *my_info, CELL *present )
{
  
  my_info->mammal_gc += present->mammal_gc;
  my_info->mammal_ga += present->mammal_ga;
  my_info->mammal_gt += present->mammal_gt;
  
  my_info->euth_gc += present->euth_gc;
  my_info->euth_ga += present->euth_ga;
  my_info->euth_gt += present->euth_gt;
  
  my_info->bird_gc += present->bird_gc;
  my_info->bird_ga += present->bird_ga;
  my_info->bird_gt += present->bird_gt;
  
}












void update_ancestor_info_mammal_euth( INFO *my_info, CELL *x )
{

  my_info->mammal_gc_euth_gc += (x->mammal_gc * x->euth_gc);
  my_info->mammal_gc_euth_ga += (x->mammal_gc * x->euth_ga);
  my_info->mammal_ga_euth_gc += (x->mammal_ga * x->euth_gc);
  my_info->mammal_ga_euth_ga += (x->mammal_ga * x->euth_ga);
  
  my_info->mammal_gc_hg_gc += x->mammal_gc * x->hg_gc;
  my_info->mammal_gc_hg_ga += x->mammal_gc * x->hg_ga;
  my_info->mammal_ga_hg_gc += x->mammal_ga * x->hg_gc;
  my_info->mammal_ga_hg_ga += x->mammal_ga * x->hg_ga;
  
  my_info->mammal_gc_only_hg_gc += x->mammal_gc * x->hg_gc * (1 - x->mammal_ga);
  my_info->mammal_gc_only_hg_ga += x->mammal_gc * x->hg_ga * (1 - x->mammal_ga);
  my_info->mammal_ga_only_hg_gc += x->mammal_ga * x->hg_gc * (1 - x->mammal_gc);
  my_info->mammal_ga_only_hg_ga += x->mammal_ga * x->hg_ga * (1 - x->mammal_gc);
  
  my_info->mammal_gc_only_euth_gc += x->mammal_gc * x->euth_gc * (1 - x->mammal_ga);
  my_info->mammal_gc_only_euth_ga += x->mammal_gc * x->euth_ga * (1 - x->mammal_ga);
  my_info->mammal_ga_only_euth_gc += x->mammal_ga * x->euth_gc * (1 - x->mammal_gc);
  my_info->mammal_ga_only_euth_ga += x->mammal_ga * x->euth_ga * (1 - x->mammal_gc);

}













void update_bird_info( INFO *my_info, CELL *present, CELL *ortho )
{
  update_bird_info_joint( my_info, present );
  update_bird_pair_joint( my_info, present, ortho );
}












void update_bird_info_joint( INFO *my_info, CELL *x )
{

  if (( x->mammal_gc ) && ( ! x->mammal_ga )) {
  
      my_info->bird_array[ GC_INDEX ][ GC_INDEX ][ GC_INDEX ] +=
        x->mammal_gc * x->euth_gc * x->bird_gc;
    
      my_info->bird_array[ GC_INDEX ][ GC_INDEX ][ GA_INDEX ] +=
        x->mammal_gc * x->euth_gc * x->bird_ga;
    
      my_info->bird_array[ GC_INDEX ][ GA_INDEX ][ GC_INDEX ] +=
        x->mammal_gc * x->euth_ga * x->bird_gc;
    
      my_info->bird_array[ GC_INDEX ][ GA_INDEX ][ GA_INDEX ] +=
        x->mammal_gc * x->euth_ga * x->bird_ga;
    
      my_info->bird_array[ GC_INDEX ][ GC_INDEX ][ GT_INDEX ] +=
        x->mammal_gc * x->euth_gc * x->bird_gt;
    
      my_info->bird_array[ GC_INDEX ][ GA_INDEX ][ GT_INDEX ] +=
        x->mammal_gc * x->euth_ga * x->bird_gt;
    
      my_info->bird_array[ GC_INDEX ][ GT_INDEX ][ GC_INDEX ] +=
        x->mammal_gc * x->euth_gt * x->bird_gc;
    
      my_info->bird_array[ GC_INDEX ][ GT_INDEX ][ GA_INDEX ] +=
        x->mammal_gc * x->euth_gt * x->bird_ga;
    
      my_info->bird_array[ GC_INDEX ][ GT_INDEX ][ GT_INDEX ] +=
        x->mammal_gc * x->euth_gt * x->bird_gt;
    
  }
  
  if (( x->mammal_ga ) && ( ! x->mammal_gc )) {
  
      my_info->bird_array[ GA_INDEX ][ GC_INDEX ][ GC_INDEX ] +=
        x->mammal_ga * x->euth_gc * x->bird_gc;

      my_info->bird_array[ GA_INDEX ][ GC_INDEX ][ GA_INDEX ] +=
        x->mammal_ga * x->euth_gc * x->bird_ga;

      my_info->bird_array[ GA_INDEX ][ GA_INDEX ][ GC_INDEX ] +=
        x->mammal_ga * x->euth_ga * x->bird_gc;

      my_info->bird_array[ GA_INDEX ][ GA_INDEX ][ GA_INDEX ] +=
        x->mammal_ga * x->euth_ga * x->bird_ga;
      
      my_info->bird_array[ GA_INDEX ][ GC_INDEX ][ GT_INDEX ] +=
        x->mammal_ga * x->euth_gc * x->bird_gt;
    
      my_info->bird_array[ GA_INDEX ][ GA_INDEX ][ GT_INDEX ] +=
        x->mammal_ga * x->euth_ga * x->bird_gt;
    
      my_info->bird_array[ GA_INDEX ][ GT_INDEX ][ GC_INDEX ] +=
        x->mammal_ga * x->euth_gt * x->bird_gc;
    
      my_info->bird_array[ GA_INDEX ][ GT_INDEX ][ GA_INDEX ] +=
        x->mammal_ga * x->euth_gt * x->bird_ga;
    
      my_info->bird_array[ GA_INDEX ][ GT_INDEX ][ GT_INDEX ] +=
        x->mammal_ga * x->euth_gt * x->bird_gt;
    
  }

  if (( x->mammal_gt ) && ( ! x->mammal_gc ) && ( ! x->mammal_ga )) {
  
      my_info->bird_array[ GT_INDEX ][ GC_INDEX ][ GC_INDEX ] +=
        x->mammal_gt * x->euth_gc * x->bird_gc;

      my_info->bird_array[ GT_INDEX ][ GC_INDEX ][ GA_INDEX ] +=
        x->mammal_gt * x->euth_gc * x->bird_ga;

      my_info->bird_array[ GT_INDEX ][ GA_INDEX ][ GC_INDEX ] +=
        x->mammal_gt * x->euth_ga * x->bird_gc;

      my_info->bird_array[ GT_INDEX ][ GA_INDEX ][ GA_INDEX ] +=
        x->mammal_gt * x->euth_ga * x->bird_ga;
      
      my_info->bird_array[ GT_INDEX ][ GC_INDEX ][ GT_INDEX ] +=
        x->mammal_gt * x->euth_gc * x->bird_gt;
    
      my_info->bird_array[ GT_INDEX ][ GA_INDEX ][ GT_INDEX ] +=
        x->mammal_gt * x->euth_ga * x->bird_gt;
    
      my_info->bird_array[ GT_INDEX ][ GT_INDEX ][ GC_INDEX ] +=
        x->mammal_gt * x->euth_gt * x->bird_gc;
    
      my_info->bird_array[ GT_INDEX ][ GT_INDEX ][ GA_INDEX ] +=
        x->mammal_gt * x->euth_gt * x->bird_ga;
    
      my_info->bird_array[ GT_INDEX ][ GT_INDEX ][ GT_INDEX ] +=
        x->mammal_gt * x->euth_gt * x->bird_gt;
    
  }

}














void update_bird_pair_joint( INFO *info, CELL *x, CELL *ortho )
{

  if ( ! ortho->bird_gc )
    return;
    
  if (( x->mammal_gc ) && ( ! x->mammal_ga )) {

    info->mam_bird_exists[ GC_INDEX ] += 1;

    info->mam_euth_bird_exists[ GC_INDEX ][ GC_INDEX ] += x->euth_gc;
    info->mam_euth_bird_exists[ GC_INDEX ][ GA_INDEX ] += x->euth_ga;
    info->mam_euth_bird_exists[ GC_INDEX ][ GT_INDEX ] += x->euth_gt;

    info->mam_bird[ GC_INDEX ][ GC_INDEX ] += x->bird_gc;
    info->mam_bird[ GC_INDEX ][ GA_INDEX ] += x->bird_ga;
    info->mam_bird[ GC_INDEX ][ GT_INDEX ] += x->bird_gt;

  }
  
  if (( x->mammal_ga ) && ( ! x->mammal_gc )) {

    info->mam_bird_exists[ GA_INDEX ] += 1;

    info->mam_euth_bird_exists[ GA_INDEX ][ GC_INDEX ] += x->euth_gc;
    info->mam_euth_bird_exists[ GA_INDEX ][ GA_INDEX ] += x->euth_ga;
    info->mam_euth_bird_exists[ GA_INDEX ][ GT_INDEX ] += x->euth_gt;

    info->mam_bird[ GA_INDEX ][ GC_INDEX ] += x->bird_gc;
    info->mam_bird[ GA_INDEX ][ GA_INDEX ] += x->bird_ga;  
    info->mam_bird[ GA_INDEX ][ GT_INDEX ] += x->bird_gt;

  }

  if (( x->mammal_gt ) && ( ! x->mammal_gc ) && ( ! x->mammal_ga )) {

    info->mam_bird_exists[ GT_INDEX ] += 1;

    info->mam_euth_bird_exists[ GT_INDEX ][ GC_INDEX ] += x->euth_gc;
    info->mam_euth_bird_exists[ GT_INDEX ][ GA_INDEX ] += x->euth_ga;
    info->mam_euth_bird_exists[ GT_INDEX ][ GT_INDEX ] += x->euth_gt;

    info->mam_bird[ GT_INDEX ][ GC_INDEX ] += x->bird_gc;
    info->mam_bird[ GT_INDEX ][ GA_INDEX ] += x->bird_ga;  
    info->mam_bird[ GT_INDEX ][ GT_INDEX ] += x->bird_gt;

  }

}









/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/















void print_ancestor_conversion_info( INFO *my_info, FH *my_fh )
{
  print_ancestor_conversion_info_fh( my_info, my_fh->out );
}











void print_ancestor_conversion_info_fh( INFO *my_info, FILE *fh )
{

  print_ancestor_conversion_total_ortho( my_info, fh );
  print_ancestor_conversion_present( my_info, fh );
  print_ancestor_conversion_mammal_euth( my_info, fh );
  print_ancestor_conversion_bird( my_info, fh );

}













void print_ancestor_conversion_total_ortho( INFO *my_info, FILE *fh )
{

  print_sep( fh );

  fprintf( fh, "Total number of genes :             %d\n", my_info->total_genes );
  fprintf( fh, "Number of mammal root orthologs :   %d\n", my_info->mammal_exists );
  fprintf( fh, "Number of bird orthologs :          %d\n", my_info->bird_exists );

  print_sep( fh );
  fprintf( fh, "\n" );

}












void print_ancestor_conversion_present( INFO *my_info, FILE *fh )
{

  fprintf( fh, "Mammal-root (GC box) :                 %d\n",
    my_info->mammal_gc );

  fprintf( fh, "Mammal-root (GA box) :                 %d\n",
    my_info->mammal_ga );
    
  fprintf( fh, "Mammal root (GT box) :                 %d\n",
    my_info->mammal_gt );

  fprintf( fh, "Eutherian   (GC box) :                 %d\n",
    my_info->euth_gc );

  fprintf( fh, "Eutherian   (GA box) :                 %d\n",
    my_info->euth_ga );

  fprintf( fh, "Eutherian   (GT box) :                 %d\n",
    my_info->euth_gt );

  fprintf( fh, "Bird        (GC box) :                 %d\n",
    my_info->bird_gc );

  fprintf( fh, "Bird        (GA box) :                 %d\n",
    my_info->bird_ga );
    
  fprintf( fh, "Bird        (GT box) :                 %d\n",
    my_info->bird_gt );
    
  fprintf( fh, "\n" );
  
  print_sep( fh );
  
  fprintf( fh, "\n" );

}










void print_ancestor_conversion_mammal_euth( INFO *my_info, FILE *fh )
{

  fprintf( fh, "Mammal-root (GC box)   Euth (GC box) :  %d\n", 
    my_info->mammal_gc_euth_gc );

  fprintf( fh, "Mammal-root (GC box)   Euth (GA box) :  %d\n", 
    my_info->mammal_gc_euth_ga );

  fprintf( fh, "Mammal-root (GA box)   Euth (GC box) :  %d\n", 
    my_info->mammal_ga_euth_gc );

  fprintf( fh, "Mammal-root (GA box)   Euth (GA box) :  %d\n", 
    my_info->mammal_ga_euth_ga );
    
  fprintf( fh, "\n" );
  
  fprintf( fh, "Mammal-root (GC box)   Hum  (GC box) :  %d\n",
    my_info->mammal_gc_hg_gc );

  fprintf( fh, "Mammal-root (GC box)   Hum  (GA box) :  %d\n",
    my_info->mammal_gc_hg_ga );

  fprintf( fh, "Mammal-root (GA box)   Hum  (GC box) :  %d\n",
    my_info->mammal_ga_hg_gc );

  fprintf( fh, "Mammal-root (GA box)   Hum  (GA box) :  %d\n",
    my_info->mammal_ga_hg_ga );

    
  fprintf( fh, "\n" );
  
  fprintf( fh, "Mammal-root (GC only)  Euth (GC box) :  %d\n",
    my_info->mammal_gc_only_euth_gc );

  fprintf( fh, "Mammal-root (GC only)  Euth (GA box) :  %d\n",
    my_info->mammal_gc_only_euth_ga );

  fprintf( fh, "Mammal-root (GA only)  Euth (GC box) :  %d\n",
    my_info->mammal_ga_only_euth_gc );

  fprintf( fh, "Mammal-root (GA only)  Euth (GA box) :  %d\n",
    my_info->mammal_ga_only_euth_ga );
    
  fprintf( fh, "\n" );
  
  fprintf( fh, "Mammal-root (GC only)  Hum  (GC box) :  %d\n",
    my_info->mammal_gc_only_hg_gc );

  fprintf( fh, "Mammal-root (GC only)  Hum  (GA box) :  %d\n",
    my_info->mammal_gc_only_hg_ga );

  fprintf( fh, "Mammal-root (GA only)  Hum  (GC box) :  %d\n",
    my_info->mammal_ga_only_hg_gc );

  fprintf( fh, "Mammal-root (GA only)  Hum  (GA box) :  %d\n",
    my_info->mammal_ga_only_hg_ga );

  fprintf( fh, "\n\n" );

}













void print_ancestor_conversion_bird( INFO *my_info, FILE *fh )
{

  char box[ SP_BUFFER ][ BUFFER ];
  
  set_ancestor_bird_exp_vals( my_info );
  
  print_sep( fh );
  
  fprintf( fh, "\n" );
  
  strcpy( box[ GC_INDEX ], "GC box" );
  strcpy( box[ GA_INDEX ], "GA box" );
  strcpy( box[ GT_INDEX ], "GT box" );
  
  register char i, j, k;
  
  for ( i = 0; i < SP_BUFFER - 1; i++ ) {
  
    fprintf( fh, "Mammal (%s) :\n", box[ i ] );
  
    for ( j = 0; j < SP_BUFFER - 1; j++ ) {
    
      for ( k = 0; k < SP_BUFFER - 1; k++ ) {
      
        fprintf( fh, "  Euth (%s)  Bird (%s)\t",
          box[ j ], box[ k ] );
          
        fprintf( fh, "%4d   (exp = %.2f)  %.3f\n", 
          my_info->bird_array[ i ][ j ][ k ],
          my_info->bird_exp[ i ][ j ][ k ],
          log( my_info->bird_array[ i ][ j ][ k ] / 
            my_info->bird_exp[ i ][ j ][ k ] ) );
      
      }
    
    }
    
    fprintf( fh, "\n" );
  
  }

}














void set_ancestor_bird_exp_vals( INFO *my_info )
{

  INFO *x = my_info;
  
  register char i, j, k;
  
  for ( i = 0; i < SP_BUFFER; i++ ) {
  
    for ( j = 0; j < SP_BUFFER; j++ ) {
    
      for ( k = 0; k < SP_BUFFER; k++ ) {
      
        x->bird_exp[ i ][ j ][ k ] = 
          (float)x->mam_euth_bird_exists[ i ][ j ] * (float)x->mam_bird[ i ][ k ] 
            / (float)x->mam_bird_exists[ i ];
      
      }
    
    }
  
  }

}








