












#include "../ext/struct.h"










#define GC_INDEX 0
#define GA_INDEX 1
#define GT_INDEX 2
#define SP_BUFFER 3









typedef struct fh_struct {
  FILE *in;
  FILE *mam_gc_out;
  FILE *mam_ga_out;
  FILE *mam_gt_out;
  FILE *euth_gc_out;
  FILE *euth_ga_out;
  FILE *euth_gt_out;
  FILE *bird_gc_out;
  FILE *bird_ga_out;
  FILE *bird_gt_out;  
  FILE *out;
} FH;





typedef struct info_struct {

  int mammal_gc;
  int mammal_ga;
  int mammal_gt;
  int euth_gc;
  int euth_ga;
  int euth_gt;
  int bird_gc;
  int bird_ga;
  int bird_gt;
  
  int total_genes;
  
  int mammal_exists;
  int bird_exists;
  int mammal_bird_exists;

  int mammal_gc_euth_gc;
  int mammal_gc_euth_ga;
  int mammal_ga_euth_gc;
  int mammal_ga_euth_ga;
  
  int mammal_gc_hg_gc;
  int mammal_gc_hg_ga;
  int mammal_ga_hg_gc;
  int mammal_ga_hg_ga;
  
  int mammal_gc_only_euth_gc;
  int mammal_gc_only_euth_ga;
  int mammal_ga_only_euth_gc;
  int mammal_ga_only_euth_ga;
  
  int mammal_gc_only_hg_gc;
  int mammal_gc_only_hg_ga;
  int mammal_ga_only_hg_gc;
  int mammal_ga_only_hg_ga;
  
  int bird_array[ SP_BUFFER ][ SP_BUFFER ][ SP_BUFFER ];
  float bird_exp[ SP_BUFFER ][ SP_BUFFER ][ SP_BUFFER ];
  
  int mam_bird_exists[ SP_BUFFER ];
  int mam_euth_bird_exists[ SP_BUFFER ][ SP_BUFFER ];
  int mam_bird[ SP_BUFFER ][ SP_BUFFER ];
  
  char input_line[ LINE_BUFFER ];
  char mammal_gc_line[ LINE_BUFFER ];
  char mammal_ga_line[ LINE_BUFFER ];
  char mammal_gt_line[ LINE_BUFFER ];
  char euth_gc_line[ LINE_BUFFER ];
  char euth_ga_line[ LINE_BUFFER ];
  char euth_gt_line[ LINE_BUFFER ];
  char bird_gc_line[ LINE_BUFFER ];
  char bird_ga_line[ LINE_BUFFER ];
  char bird_gt_line[ LINE_BUFFER ];

} INFO;








typedef struct cell_struct {
  
  int mammal_gc;
  int mammal_ga;
  int mammal_gt;
  
  int euth_gc;
  int euth_ga;
  int euth_gt;
  
  int bird_gc;
  int bird_ga;
  int bird_gt;
  
  int hg_gc;
  int hg_ga;
  int hg_gt;
  
} CELL;










void do_ancestor( void );

void set_ancestor_fhs( FH *my_fh );

void close_ancestor_fhs( FH *my_fh );

void do_my_ancestor( FH *my_fh );

void do_ancestor_w_tree( ND_LIST *root, ND_LIST *leaf_header, FH *my_fh );

PARAM *make_and_set_gc_params( PARAM *param );

PARAM *make_and_set_ga_params( PARAM *param );

PARAM *make_and_set_gt_params( PARAM *param );

void do_ancestor_w_tree_single_nucl( ND_LIST *root, ND_LIST *leaf_header,
  char my_nucl, PARAM *param, FILE *in, FILE *mam_out, FILE *euth_out, 
  FILE *bird_out );

void set_prior_probs_ancestor_tree( ND_LIST *root, ND_LIST *leaf_header,
  char my_nucl, FILE *in );

void set_ancestor_tree_types( ND_LIST *root );

void get_ancestor_tree_single_nucl( ND_LIST *root, ND_LIST *leaf_header, 
  char my_nucl, PARAM *param, FILE *in, FILE *mam_out, FILE *euth_out,
  FILE *bird_out );

void get_my_ancestor_tree_single_nucl( ND_LIST *root, ND_LIST *leaf_header,
  STR_LIST *label_header, PARAM *param, char my_nucl, FILE *in, FILE *mam_out,
  FILE *euth_out, FILE *bird_out );

void get_ancestor_tree_single_gene( ND_LIST *root, ND_LIST *leaf_header,
  STR_LIST *label_header, char my_nucl, PARAM *param, char line[], FILE *mam_out, 
  FILE *euth_out, FILE *bird_out );

void fill_phylo_ancestor( ND_LIST *node );

void fill_phylo_ancestor_single_node( ND_LIST *node );

int node_is_absent_or_euth( ND_LIST *node );

void fill_my_phylo_ancestor_single_node( ND_LIST *node );

void fill_phylo_ancestor_single_numb_of_mfs( ND_LIST *node, char i );

void fill_phylo_ancestor_birds( ND_LIST *root );

void set_mf_occur_numb( ND_LIST *node, ND_LIST *nn );

void fill_phylo_ancestor_mammals( ND_LIST *root );

int first_only_exists( ND_LIST *a, ND_LIST *b, ND_LIST *c );

void put_ancestor_tree_probs_single_gene( ND_LIST *root, int numb, char acc[], 
  char gene[], FILE *mam_out, FILE *euth_out, FILE *bird_out );

void put_ancestor_tree_probs_single_node( ND_LIST *node, int numb, char acc[],
  char gene[], FILE *out );



/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/



void do_ancestor_conversions( void );

void set_ancestor_conversions_fh( FH *my_fh );

void close_ancestor_conversions_fh( FH *my_fh );

void initialize_ancestor_conversions_info( INFO *my_info );

void do_my_ancestor_conversions( INFO *my_info, FH *my_fh );

void do_ancestor_conversions_w_tree( INFO *my_info, ND_LIST *root, 
  ND_LIST *leaf_header, FH *my_fh );

void do_my_ancestor_conversions_w_tree( INFO *my_info, ND_LIST *root, 
  ND_LIST *leaf_header, STR_LIST *label_header, FH *my_fh );

int get_ancestor_lines( INFO *my_info, FH *my_fh );

void do_ancestor_conversion_single_gene( INFO *my_info, ND_LIST *root, 
  ND_LIST *leaf_header, STR_LIST *label_header );

void get_ancestor_conversion_single_gene_present( CELL *x, INFO *my_info );

void get_ancestor_conversion_single_gene_spec( CELL *x, ND_LIST *root, 
  ND_LIST *leaf_header, STR_LIST *label_header, INFO *my_info );

void get_ancestor_conversion_single_gene_ortho( CELL *x, INFO *my_info );

int is_present( char line[] );

int is_ortho( char line[] );

int is_present_spec( char line[], ND_LIST *root, ND_LIST *leaf_header,
  STR_LIST *label_header, char my_nucl );

int is_present_node( ND_LIST *node );

void do_my_ancestor_conversion_single_gene( INFO *my_info, CELL *present, 
  CELL *ortho );

void update_ancestor_info_ortho( INFO *my_info, CELL *ortho );

void update_ancestor_info_present( INFO *my_info, CELL *present );

void update_ancestor_info_mammal_euth( INFO *my_info, CELL *x );

void update_bird_info( INFO *my_info, CELL *x, CELL *ortho );

void update_bird_info_joint( INFO *my_info, CELL *x );

void update_bird_pair_joint( INFO *info, CELL *x, CELL *ortho );



/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/



void print_ancestor_conversion_info( INFO *my_info, FH *my_fh );

void print_ancestor_conversion_info_fh( INFO *my_info, FILE *fh );

void print_ancestor_conversion_total_ortho( INFO *my_info, FILE *fh );

void print_ancestor_conversion_present( INFO *my_info, FILE *fh );

void print_ancestor_conversion_mammal_euth( INFO *my_info, FILE *fh );

void print_ancestor_conversion_bird( INFO *my_info, FILE *fh );

void set_ancestor_bird_exp_vals( INFO *my_info );

























