#include "count.h"
















#define INPUT_COL_FNAME "data/ft_hg19/change/hg19.gcBox.count.txt"

#define OUTPUT_FNAME "data/ft_hg19/evol/motif_count.txt"

#define SPECIAL_NODE "Vertebrata"





#define NON_TREE_SPEC_BUFFER 3
#define TREE_SHREW 0
#define STICKLEBACK 1
#define MEDAKA 2














void do_motif_count( void )
{

  FH my_fh;
  
  initialize_motif_count_fh( &my_fh );
  
  do_my_motif_count( &my_fh );
  
  close_motif_count_fh( &my_fh );

}













void initialize_motif_count_fh( FH *my_fh ) 
{

  FILE *open_text_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );
  
  my_fh->in = (FILE *)open_text_file_reading( INPUT_COL_FNAME, "" );
  my_fh->out = (FILE *)open_text_file_writing( OUTPUT_FNAME, "" );

}













void close_motif_count_fh( FH *my_fh )
{
  close_file( my_fh->in );
  close_file( my_fh->out );
}














void do_my_motif_count( FH *my_fh )
{
  do_motif_count_single_nucl( C_NUCL, "GC box", my_fh ); 
  do_motif_count_single_nucl( A_NUCL, "GA box", my_fh );
  do_motif_count_single_nucl( T_NUCL, "GT box", my_fh );
}













void do_motif_count_single_nucl( char my_nucl, char box[], FH *my_fh )
{

  ND_LIST *get_evol_tree( ND_LIST *root );
  ND_LIST *get_nd_leaf_header( ND_LIST *node );

  ND_LIST *root = NULL, *leaf_header = NULL;

  /*  In tree.c:  */
  root = (ND_LIST *)get_evol_tree( root );
  leaf_header = (ND_LIST *)get_nd_leaf_header( root );
  
  CELL *my_cell = NULL;
  my_cell = (CELL *)make_and_initialize_count_cell( my_cell );

  /*  In this file, below:  */
  do_motif_count_w_tree( root, leaf_header, my_cell, my_nucl, my_fh->in );
  
  /*  In this file, below:  */
  print_motif_count( root, box, my_cell, my_fh->out );
  
  /*  In this file, below:  */
  kill_count_cell( my_cell );

}









CELL *make_and_initialize_count_cell( CELL *my_cell )
{

  my_cell = (CELL *)malloc( sizeof(CELL) );
  
  my_cell->spec = (ENTRY *)malloc( sizeof(ENTRY) * NON_TREE_SPEC_BUFFER );
  my_cell->n = NON_TREE_SPEC_BUFFER;
  
  strcpy( (my_cell->spec + TREE_SHREW)->label, "tupBel1" );
  strcpy( (my_cell->spec + MEDAKA)->label, "oryLat2" );
  strcpy( (my_cell->spec + STICKLEBACK)->label, "gasAcu1" );
  
  char i;
  
  for ( i = 0; i < my_cell->n; i++ ) {
    (my_cell->spec + i)->total_genes = 0;
    (my_cell->spec + i)->x = 0;
    (my_cell->spec + i)->numb_of_mfs = 0;
  }
  
  return my_cell;

}














void kill_count_cell( CELL *my_cell )
{
  free( my_cell->spec );   
  free( my_cell );
}













void do_motif_count_w_tree( ND_LIST *root, ND_LIST *leaf_header, CELL *my_cell,
  char my_nucl, FILE *in )
{

  STR_LIST *get_label_list_from_first_line( STR_LIST *label_header, FILE *in );

  STR_LIST *label_header = NULL;

  set_position( 0, in );
  
  /*  In set.c:  */
  label_header = (STR_LIST *)get_label_list_from_first_line( label_header, in );
  
  /*  In this file, below:  */
  do_motif_count_w_label_list( root, leaf_header, my_cell, label_header, 
    my_nucl, in );

}













void do_motif_count_w_label_list( ND_LIST *root, ND_LIST *leaf_header,
  CELL *my_cell, STR_LIST *label_header, char my_nucl, FILE *in )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL )
    do_motif_count_single_gene( root, leaf_header, my_cell, label_header, line, 
      my_nucl );

}















void do_motif_count_single_gene( ND_LIST *root, ND_LIST *leaf_header,
  CELL *my_cell, STR_LIST *label_header, char line[], char my_nucl )
{

  char gene[ BUFFER ], acc[ BUFFER ];
  int numb;
  
  sscanf( line, "%d%s%s", &numb, acc, gene );
  
  printf( "%5d :  %s\n", numb, gene );

  /*  In set.c:  */
  initialize_nd_tree( root );
  
  /*  In set.c:  */
  set_my_evolutionary_tree_single_gene( root, leaf_header, label_header,
    line, my_nucl );
    
  /*  In this file, below:  */
  update_motif_count_single_gene( leaf_header );
  
  /*  In this file, below:  */
  update_motif_count_cell_species( my_cell, label_header, line, my_nucl );
    
}














void update_motif_count_single_gene( ND_LIST *leaf_header )
{

  ND_LIST *leaf = leaf_header;
  
  while ( leaf != NULL )
    update_motif_count_single_leaf_and_step( &leaf );

}













void update_motif_count_single_leaf_and_step( ND_LIST **leaf )
{
  update_motif_count_single_leaf( *leaf );
  *leaf = (*leaf)->right;
}













void update_motif_count_single_leaf( ND_LIST *leaf )
{

  if ( ! leaf->my_node->exists )
    return;

  TT_MX *x = leaf->my_node->data->x;
  TT_MX *y = leaf->my_node->data->pr;

  register char i;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) 
    *(x->x + i) += *(y->x + i);

}














/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/














void update_motif_count_cell_species( CELL *my_cell, STR_LIST *label_header,
  char line[], char my_nucl )
{

  register char i;
  
  for ( i = 0; i < my_cell->n; i++ )
    update_motif_count_cell_single_spec( my_cell->spec + i, label_header, line,
      my_nucl );

}












void update_motif_count_cell_single_spec( ENTRY *spec, STR_LIST *label_header,
  char line[], char my_nucl )
{

  STR_LIST *label = label_header;
  INDEX index = 0;
  
  while ( label != NULL )
    update_motif_count_cell_single_label_and_step( spec, &label, &index, line,
      my_nucl );  

}











void update_motif_count_cell_single_label_and_step( ENTRY *spec, STR_LIST **label,
  INDEX *index, char line[], char my_nucl )
{
  update_motif_count_cell_single_label( spec, *label, index, line, my_nucl );
  *label = (*label)->next;
}












void update_motif_count_cell_single_label( ENTRY *spec, STR_LIST *label,
  INDEX *index, char line[], char my_nucl )
{

  INDEX get_next_word_str_index( char str[], INDEX curr_index );

  char entry[ BUFFER ];
  
  sscanf( line + (*index), "%s", entry );
  
  if ( ! strcmp( spec->label, label->str ))
    /*  In this file, below:  */
    update_motif_count_cell_single_spec_matching_label( spec, entry, my_nucl );
    
  /*  In set.c:  */
  *index = get_next_word_str_index( line, *index );

}














void update_motif_count_cell_single_spec_matching_label( ENTRY *spec,
  char entry[], char my_nucl )
{

  char get_entry_character_nucl_numb( char entry[], char nucl_numb );

  /*  In set.c:  */
  char entry_char = get_entry_character_nucl_numb( entry, my_nucl );
  
  /*  In this file, below:  */
  update_motif_count_cell_single_spec_w_char( spec, entry_char );

}












void update_motif_count_cell_single_spec_w_char( ENTRY *spec, char entry_char )
{

  if ( entry_char == '-' )
    return;
    
  spec->numb_of_mfs += entry_char;
    
  if ( entry_char > 0 )
    ++(spec->x);

  ++(spec->total_genes);

}










/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
















void print_motif_count( ND_LIST *root, char box[], CELL *my_cell, FILE *fh )
{

  print_sep( fh );
  
  fprintf( fh, "%s\n\n", box );
  
  print_dot_sep( fh );
  
  print_my_motif_count( root, my_cell, fh );
  
  print_sep( fh );
  
  fprintf( fh, "\n\n" );

}















void print_my_motif_count( ND_LIST *root, CELL *my_cell, FILE *fh )
{

  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );
  
  ND_LIST *x = NULL;
  /*  In tree.c:  */
  x = (ND_LIST *)get_nd_lister_from_label( root, SPECIAL_NODE );

  fprintf( fh, "Root :  '%s'\n\n", SPECIAL_NODE );

  /*  In this file, below:  */
  print_x_matrix_root( x, fh );
  
  /*  In this file, below:  */
  print_spec_cell_count( my_cell, fh );

}













void print_x_matrix_root( ND_LIST *node, FILE *fh )
{

  if ( node->down[ 0 ] == NULL ) {
    print_x_matrix_leaf( node, fh );
    return;
  }
  
  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    print_x_matrix_root( node->down[ i++ ], fh );

}













void print_x_matrix_leaf( ND_LIST *node, FILE *fh )
{

  fprintf( fh, "%s :\n", node->my_node->label );
  
  TT_MX *x = node->my_node->data->x;
  
  int total = get_total_x_elem_mx( node );
  int mfs = get_total_numb_of_mfs_node( node );
  
  fprintf( fh, "    Total genes  :  %d\n", total );
  fprintf( fh, "    Target genes :  %d   (%.2f%%)\n", total - (int)*(x->x),
    (float)(total - *(x->x)) * 100.0 / (float)total );
  fprintf( fh, "    Numb of mfs  :  %d\n", mfs );
}














int get_total_x_elem_mx( ND_LIST *node )
{

  TT_MX *x = node->my_node->data->x;

  int sum = 0;
  
  register char i;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ )
    sum += (int)*(x->x + i);
    
  return sum;

}













int get_total_numb_of_mfs_node( ND_LIST *node )
{

  TT_MX *mx = node->my_node->data->x;

  int total = 0;
  
  register char i;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ )
    total += (i * (*(mx->x + i)));
    
  return total;

}










void print_spec_cell_count( CELL *my_cell, FILE *fh )
{

  if ( my_cell == NULL )
    return;
    
  print_my_spec_cell_count( my_cell, fh );

}











void print_my_spec_cell_count( CELL *my_cell, FILE *fh )
{

  char i;
  
  for ( i = 0; i < my_cell->n; i++ )
    print_spec_cell_single_entry( my_cell->spec + i, fh );

}











void print_spec_cell_single_entry( ENTRY *spec, FILE *fh )
{

  fprintf( fh, "%s :\n", spec->label );
  
  fprintf( fh, "    Total genes  :  %d\n", spec->total_genes );
  fprintf( fh, "    Target genes :  %d   (%.2f%%)\n", spec->x,
    (float)(spec->x) * 100.0 / (float)(spec->total_genes) );
  fprintf( fh, "    Numb of mfs  :  %d\n", spec->numb_of_mfs );

}



















