#include "diff.h"












#define INPUT_COL_FNAME "data/ft_hg19/change/hg19.gcBox.count.txt"


#define MAJOR_SPEC "hg19"
#define MINOR_SPEC "mm9"
















void do_motif_numb_diff( void )
{

  float mx[ NUMB_OF_MFS ][ NUMB_OF_MFS ];
  
  initialize_diff_mx( mx );
  
  do_my_motif_numb_diff( mx );
  
  print_motif_numb_diff( mx );
  
}












void initialize_diff_mx( float mx[ NUMB_OF_MFS ][ NUMB_OF_MFS ] )
{

  register char i, j;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {
    for ( j = 0; j < NUMB_OF_MFS; j++ ) {
      mx[ i ][ j ] = 0.0;
    }
  }

}














void do_my_motif_numb_diff( float mx[ NUMB_OF_MFS ][ NUMB_OF_MFS ] )
{

  ND_LIST *get_evol_tree( ND_LIST *root );
  ND_LIST *get_nd_leaf_header( ND_LIST *node );

  ND_LIST *root = NULL, *leaf_header = NULL;

  /*  In tree.c:  */
  root = (ND_LIST *)get_evol_tree( root );
  leaf_header = (ND_LIST *)get_nd_leaf_header( root );
  
  /*  In this file, below:  */
  do_motif_numb_diff_w_tree( mx, root, leaf_header, MAJOR_SPEC, MINOR_SPEC );
  
}












void do_motif_numb_diff_w_tree( float mx[ NUMB_OF_MFS ][ NUMB_OF_MFS ],
  ND_LIST *root, ND_LIST *leaf_header, char major_spec[], char minor_spec[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( INPUT_COL_FNAME, "" );
  
  do_motif_numb_diff_w_fh( mx, root, leaf_header, major_spec, minor_spec, in );
  
  close_file( in );

}












void do_motif_numb_diff_w_fh( float mx[ NUMB_OF_MFS ][ NUMB_OF_MFS ],
  ND_LIST *root, ND_LIST *leaf_header, char major_spec[], char minor_spec[],
  FILE *in )
{

  STR_LIST *get_label_list_from_first_line( STR_LIST *label_header, FILE *in );

  STR_LIST *label_header = NULL;

  set_position( 0, in );
  
  /*  In set.c:  */
  label_header = (STR_LIST *)get_label_list_from_first_line( label_header, in );
  
  /*  In this file, below:  */
  do_my_motif_numb_diff_w_fh( mx, root, leaf_header, label_header, major_spec,
    minor_spec, in );

}












void do_my_motif_numb_diff_w_fh( float mx[ NUMB_OF_MFS ][ NUMB_OF_MFS ],
  ND_LIST *root, ND_LIST *leaf_header, STR_LIST *label_header,
  char major_spec[], char minor_spec[], FILE *in )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL )
    get_motif_numb_diff_single_line( mx, root, leaf_header, label_header, 
      line, major_spec, minor_spec );

}












void get_motif_numb_diff_single_line( float mx[ NUMB_OF_MFS ][ NUMB_OF_MFS ],
  ND_LIST *root, ND_LIST *leaf_header, STR_LIST *label_header, char line[],
  char major_spec[], char minor_spec[] )
{

  char gene[ BUFFER ], acc[ BUFFER ];
  int numb;
  
  sscanf( line, "%d%s%s", &numb, acc, gene );
  
  printf( "%5d :  %s\n", numb, gene );

  /*  In set.c:  */
  initialize_nd_tree( root );
  
  /*  In set.c:  */
  set_my_evolutionary_tree_single_gene( root, leaf_header, label_header,
    line, C_NUCL );
    
  /*  In this file, below:  */
  get_motif_numb_diff_single_set_gene( mx, root, major_spec, minor_spec );
    
}














void get_motif_numb_diff_single_set_gene( float mx[ NUMB_OF_MFS ][ NUMB_OF_MFS ],
  ND_LIST *root, char major_spec[], char minor_spec[] )
{

  int major_numb = get_numb_of_sites_label( root, major_spec );
  int minor_numb = get_numb_of_sites_label( root, minor_spec );
  
  if (( major_numb < 0 ) || ( minor_numb < 0 ))
    return;
  
  ++(mx[ major_numb ][ minor_numb ]);

}













int get_numb_of_sites_label( ND_LIST *root, char label[] )
{

  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );

  ND_LIST *node = NULL;
  /*  In tree.c:  */
  node = (ND_LIST *)get_nd_lister_from_label( root, label );
  
  /*  In this file, below:  */
  return get_numb_of_sites_leaf( node );

}












int get_numb_of_sites_leaf( ND_LIST *node )
{

  if ( ! node->my_node->exists )
    return -1;

  int i;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {
    if ( *(node->my_node->data->pr->x + i) == 1.0 )
      return i;
  }
  
  error( 11000, "Unknown number of sites.\n" );

}













void print_motif_numb_diff( float mx[ NUMB_OF_MFS ][ NUMB_OF_MFS ] )
{

  float total = get_total_diff_mx( mx );
  float n = get_existing_total_diff_mx( mx );
  
  register char i, j;
  
  printf( "\n\n\n" );
  
  print_sep( stdout );
  
  printf( "%3s\t", "" );
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) 
    printf( "%6d   ", i );
    
  printf( "\n" );
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {

    printf( "%3d\t", i );

    for ( j = 0; j < NUMB_OF_MFS; j++ ) 
      printf( "%6d   ", (int)mx[ i ][ j ] );

    printf( "\n" );

  }
  
  print_sep( stdout );

  printf( "%3s\t", "" );
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) 
    printf( "%6d   ", i );
    
  printf( "\n" );
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {

    printf( "%3d\t", i );

    for ( j = 0; j < NUMB_OF_MFS; j++ ) 
      printf( "%6.3f   ", mx[ i ][ j ] / total );

    printf( "\n" );

  }
  
  print_sep( stdout );

  printf( "%3s\t", "" );
  
  for ( i = 1; i < NUMB_OF_MFS; i++ ) 
    printf( "%6d   ", i );
    
  printf( "\n" );
  
  for ( i = 1; i < NUMB_OF_MFS; i++ ) {

    printf( "%3d\t", i );

    for ( j = 1; j < NUMB_OF_MFS; j++ ) 
      printf( "%6.3f   ", mx[ i ][ j ] / n );

    printf( "\n" );

  }
  
  float diff_sum = get_diff_sum( mx, n );
  
  printf( "\n\n" );
  printf( "Diff numbs of binding sites :  %6.4f\n", diff_sum );
  
  printf( "\n\n" );
  
}











float get_total_diff_mx( float mx[ NUMB_OF_MFS ][ NUMB_OF_MFS ] )
{

  float sum = 0.0;
  
  register char i, j;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {
  
    for ( j = 0; j < NUMB_OF_MFS; j++ ) {
    
      sum += mx[ i ][ j ];
    
    }
  
  }
  
  return sum;

}












float get_existing_total_diff_mx( float mx[ NUMB_OF_MFS ][ NUMB_OF_MFS ] )
{

  float sum = 0.0;
  
  register char i, j;
  
  for ( i = 1; i < NUMB_OF_MFS; i++ ) {
  
    for ( j = 1; j < NUMB_OF_MFS; j++ ) {
    
      sum += mx[ i ][ j ];
    
    }
  
  }
  
  return sum;

}













float get_diff_sum( float mx[ NUMB_OF_MFS ][ NUMB_OF_MFS ], float n )
{

  float x = 0.0, total = 0.0;

  register char i, j;
  
  for ( i = 1; i < NUMB_OF_MFS; i++ ) {
  
    for ( j = 1; j < NUMB_OF_MFS; j++ ) {
    
      total += mx[ i ][ j ];

      if ( i != j )
        x += mx[ i ][ j ] / n;
    
    }
  
  }
  
  printf( "total numb :  %d\n", (int)total );
  printf( "total frac :  %.2f\n", total / n );
  
  return x;

}






