#include "get_n.h"














#define MIN_PROB 1e-27
#define MIN_PROB_RATIO 1e-6
#define MIN_NUMB_OF_MFS 5






#define OUTPUT_GENE 4










/*

    The function 'evol_fill_phylo_n_vals()' determines the value for

      Pr( D[root], A[Y] = i, A[Z] = j; theta )

    for each node Z, where Y = z->up is the parent node of Z,
    and each pair of possible number of binding site occurrences i,j.

    These values are stored in :
      z->my_node->data->n[ i,j ]
      
    See below and v.c for details regarding the computation...

*/















void evol_fill_phylo_n_vals( ND_LIST *root, ND_LIST *z )
{

  if (( root == NULL ) || ( z == NULL ))
    error( 9400, "Unexpected NULL value for nodes.\n" );
    
  INDEX i = 0;
  
  while ( z->down[ i ] != NULL )
    evol_fill_my_phylo_n_vals( root, z->down[ i++ ] );
    
}












void evol_fill_my_phylo_n_vals( ND_LIST *root, ND_LIST *z )
{

  evol_fill_phylo_n_vals_single_node( root, z );
  
  register char i = 0;
  
  while ( z->down[ i ] != NULL )
    evol_fill_my_phylo_n_vals( root, z->down[ i++ ] );

}













void evol_fill_phylo_n_vals_single_node( ND_LIST *root, ND_LIST *z )
{

  if ( ! z->my_node->exists )
    return;
    
  if ( ! strcmp( z->up->my_node->label, root->my_node->label )) {
    evol_fill_phylo_n_vals_root_daughter( z );
    return;
  }
  
  evol_fill_my_phylo_n_vals_single_node( root, z );

}














void evol_fill_my_phylo_n_vals_single_node( ND_LIST *root, ND_LIST *z )
{

  TT_MX *create_tt_mx( TT_MX *mm, INDEX ii, INDEX jj );
  
  TT_MX *mx = NULL;
  mx = (TT_MX *)create_tt_mx( mx, NUMB_OF_MFS, 1 );
    
  TT_MX *n_mx = z->my_node->data->n;
  
  TT max = 0.0;

  register INDEX i, j;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {
  
    for ( j = 0; j < NUMB_OF_MFS; j++ ) {
    
      TT n = get_single_n_val( root, z, mx, i, j );
      
      if ( is_below_nval_thresh( n, &max, j ))
        break;
        
      *(n_mx->x + (i * n_mx->j) + j) = n;
    
    }
    
  }
  
  kill_tt_mx( mx );
  
}












int is_below_nval_thresh( TT n, TT *max, INDEX j )
{

  if ( n > *max ) {
    *max = n;
    return 0;
  }

  if (( j <= MIN_NUMB_OF_MFS ) || ( n > MIN_PROB ))
    return 0;
    
  return ( n < (*max) * (MIN_PROB_RATIO) ) ? (1) : (0);

}










TT get_single_n_val( ND_LIST *root, ND_LIST *z, TT_MX *mx, INDEX i, INDEX j )
{

  get_single_n_val_node_wyz( mx, z, i, j );
  
  return get_my_single_n_val_mx( root, z, mx );

}








/*

    Assigns Pr( D[W], A[Z] = j, A[Y] = i | A[W] = k; theta )
      to each element :  mx[k]
      
    Parent node of Z is Y = z->up,
    Parent node of Y is W = y->up.
      
      Pr( D[W], A[Z] = j, A[Y] = i | A[W] = k; theta )
        = 
        Pr( D[W] | A[W] = k; theta ) *
          [ Pr( D[Y], A[Z] = j | A[Y] = i; theta ) * y_p[ k,i ](t[Y]) / 
            sum_x : 
            ( Pr( D[Y] | A[Y] = x; theta ) * y_p[ k,x ](t[Y]) )
          ]
          
      where
    
      Pr( D[Y], A[Z] = j | A[Y] = i; theta )
        = 
        Pr( D[Y] | A[Y] = i; theta ) *
          [ Pr( D[Z] | A[Z] = j; theta ) * z_p[ i,j ](t[Z]) /
            sum_x :
            ( Pr( D[Z] | A[Z] = x; theta ) * z_p[ i,x ](t[Z]) )
          ]
          
    In this function :
      
    y_lhood_val  =  Pr( D[Y] | A[Y] = i; theta )
    y_p[ k,i ]   =  p[ k,i ](t[Y])
    z_joint      =  Pr( D[Z] | A[Z] = j; theta ) * p[ i,j ](t[Z])
    z_sum_val    =  \sum_x  Pr( D[Z] | A[Z] = x; theta ) * p[ i,x ](t[Z])
    y_sum[ k ]   =  \sum_x  Pr( D[Y] | A[Y] = x; theta ) * p[ k,x ](t[Y])

*/







void get_single_n_val_node_wyz( TT_MX *mx, ND_LIST *z, INDEX i, INDEX j )
{

  ND_LIST *y = z->up;
  ND_LIST *w = y->up;

  TT_MX *z_lhood = z->my_node->data->pr;
  TT_MX *z_p = z->my_node->data->p;
  TT_MX *z_sum = z->my_node->data->sum;
  
  TT_MX *y_lhood = y->my_node->data->pr;
  TT_MX *y_sum = y->my_node->data->sum;
  TT_MX *y_p = y->my_node->data->p;
    
  TT_MX *w_lhood = w->my_node->data->pr;
  TT_MX *w_p = w->my_node->data->p;
  TT_MX *w_sum = w->my_node->data->sum;

  register char k;
  
  for ( k = 0; k < NUMB_OF_MFS; k++ ) 
    *(mx->x + k) = (*(z_p->x + (i * z_p->j) + j)) * (*(y_p->x + (k * y_p->j) + i))
      * (*(z_lhood->x + j)) * (*(y_lhood->x + i)) * (*(w_lhood->x + k)) 
      / 
      ( (*(z_sum->x + i)) * (*(y_sum->x + k)) );
    
}













void evol_fill_phylo_n_vals_root_daughter( ND_LIST *z )
{

  ND_LIST *r = z->up;

  TT_MX *n_mx = z->my_node->data->n;
  TT_MX *z_pr = z->my_node->data->pr;
  TT_MX *z_sum = z->my_node->data->sum;
  TT_MX *p = z->my_node->data->p;
  
  TT_MX *prior = r->my_node->data->prior;
  TT_MX *r_pr = r->my_node->data->pr;
  
  TT max = 0.0;

  register char i, j;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {
  
    TT sum = *(z_sum->x + i);
    
    for ( j = 0; j < NUMB_OF_MFS; j++ ) {
    
      TT n = (*(z_pr->x + j)) * (*(p->x + (i * p->j) + j)) * 
        (*(prior->x + i)) * (*(r_pr->x + i)) / (sum);
      
      if ( is_below_nval_thresh( n, &max, j ))
        break;
        
      *(n_mx->x + (i * n_mx->j) + j) = n;
    
    }
  
  }

}














void evol_fill_phylo_n_vals_grand_daughter( ND_LIST *z )
{

  TT_MX *create_tt_mx( TT_MX *mm, INDEX ii, INDEX jj );
  
  TT_MX *mx = NULL;
  mx = (TT_MX *)create_tt_mx( mx, NUMB_OF_MFS, 1 );
    
  TT_MX *n_mx = z->my_node->data->n;
  
  TT max = 0.0;

  register INDEX i, j;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {
  
    for ( j = 0; j < NUMB_OF_MFS; j++ ) {
    
      TT n = get_single_n_val_grand_daughter( z, mx, i, j );
      
      if ( is_below_nval_thresh( n, &max, j ))
        break;
        
      *(n_mx->x + (i * n_mx->j) + j) = n;
    
    }
    
  }
  
  kill_tt_mx( mx );
  
}











TT get_single_n_val_grand_daughter( ND_LIST *z, TT_MX *mx, INDEX i, INDEX j )
{

  get_single_n_val_node_wyz( mx, z, i, j );
  
  return get_my_single_n_val_grand_daughter( z, mx, i, j );

}












TT get_my_single_n_val_grand_daughter( ND_LIST *z, TT_MX *mx, INDEX i, INDEX j )
{

  ND_LIST *y = z->up;
  ND_LIST *w = y->up;

  return get_my_single_n_val_w_is_root( w, w, mx );

}









/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/











TT get_my_single_n_val_mx( ND_LIST *root, ND_LIST *z, TT_MX *mx )
{

  ND_LIST *y = z->up;
  ND_LIST *w = y->up;
  
  if ( ! strcmp( root->my_node->label, z->my_node->label ))
    error( 9401, "Node Z is unexpectedly root node.\n" );
    
  if ( ! strcmp( root->my_node->label, y->my_node->label ))
    error( 9402, "Node Y is unexpectedly root node.\n" );
    
  if ( ! strcmp( root->my_node->label, w->my_node->label ))
    return get_my_single_n_val_w_is_root( root, w, mx );
    
  return get_single_n_val_mx_middle_node( root, z, mx );
  
}












TT get_my_single_n_val_w_is_root( ND_LIST *root, ND_LIST *w, TT_MX *mx )
{

  TT_MX *prior = w->my_node->data->prior;
  
  TT prob = 0.0;
  
  register INDEX k;
  
  for ( k = 0; k < NUMB_OF_MFS; k++ )  
    prob += (*(mx->x + k)) * (*(prior->x + k));
    
  return prob;

}











TT get_single_n_val_mx_middle_node( ND_LIST *root, ND_LIST *z, TT_MX *mx )
{

  ND_LIST *y = z->up;
  ND_LIST *w = y->up;
  
  while ( strcmp( root->my_node->label, w->my_node->label )) {
    /*  In this file, below:  */
    set_single_n_val_middle_node_single_node( y, mx );
    z = y;
    y = w;
    w = w->up;
  }
  
  /*  In this file, above:  */
  return get_my_single_n_val_w_is_root( root, w, mx );

}










/*

    Assigns Pr( D[T], A[Z] = j, A[Y] = i | A[T] = x; theta )
      to each element :  mx[x]
      
    Parent node of W (passed to function) is T = w->up,

    When the function is called, the value of mx[a] is :
      mx[a] = Pr( D[W], A[Z] = j, A[Y] = i | A[W] = a; theta )
      
      
      Pr( D[T], A[Z] = j, A[Y] = i | A[T] = x; theta )
        = 
        Pr( D[T] | A[T] = x; theta ) *
          [ 
            (SUM_k) :
              Pr( D[W], A[Y] = i, A[Z] = j | A[W] = k; theta ) * p_w[ x,k ](t[W])
                /                   (divided by...)
            (SUM_k) :
              Pr( D[W] | A[W] = k; theta ) * p_w[ x,k ](t[W])
          ]
          
      Note that here, 
        w_sum        =  (SUM_k) : Pr( D[W] | A[W] = k; theta ) * p_w[ x,k ](t[W])
        t_lhood[x]   =  Pr( D[T] | A[T] = x; theta )
          
*/









void set_single_n_val_middle_node_single_node( ND_LIST *y, TT_MX *mx )
{

  TT array[ NUMB_OF_MFS ];
  
  copy_tt_mx_to_array( array, mx );

  ND_LIST *w = y->up;
  ND_LIST *t = w->up;
  
  TT_MX *t_lhood = t->my_node->data->pr;
  
  TT_MX *w_p = w->my_node->data->p;
  TT_MX *w_sum = w->my_node->data->sum;
  TT_MX *w_lhood = w->my_node->data->pr;
  
  char end = NUMB_OF_MFS - 1;
  
  register char x, b;
  
  for ( b = MIN_NUMB_OF_MFS; b < NUMB_OF_MFS; b++ ) {
    if ( MIN_PROB > array[ b ] ) {
      end = b;
      break;
    }
  }
  
  for ( x = 0; x <= end; x++ ) {
  
    TT sum = 0.0;
    
    register char k;
    
    for ( k = 0; k <= end; k++ ) 
      sum += (array[ k ]) * (*(w_p->x + (x * w_p->j) + k));
      
    *(mx->x + x) = (sum) * (*(t_lhood->x + x)) / (*(w_sum->x + x));
  
  }
  
}












void copy_tt_mx_to_array( TT array[], TT_MX *mx )
{

  register char i;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ )
    array[ i ] = *(mx->x + i);

}



















