#include "mammal.h"











#define ABS(x) (((x) > 0.0) ? (x) : (-1.0 * (x)))









#define NUMB_OF_PARAMS 3


#define EUTH_INDEX 0
#define CONVERT_INDEX 1
#define ANCESTOR_INDEX 2
#define PARAM_TYPE_UNSET -1


/*
#define ALPHA_EUTH 9e-10
#define BETA_EUTH 5e-8
#define ALPHA_CONVERT (ALPHA_EUTH)
#define BETA_CONVERT (BETA_EUTH)
#define ALPHA_ANCESTOR (ALPHA_EUTH)
#define BETA_ANCESTOR (BETA_EUTH)
*/




#define ALPHA_EUTH 1e-11
#define BETA_EUTH 1e-8
#define ALPHA_CONVERT 6e-11
#define BETA_CONVERT 2e-10
#define ALPHA_ANCESTOR 1.3e-11
#define BETA_ANCESTOR 8.2e-9






#define CONVERGE_THRESH 1e-12

#define MAX_SEQS 500

#define MY_NUCL C_NUCL



#define MAX_ITERATIONS 10



#define PARAM_OUTPUT_FNAME "data/ft_hg19/evol/params/parameter_output.txt"
#define INPUT_COL_FNAME "data/ft_hg19/change/hg19.gcBox.count.txt"























void do_mammal_evol( void )
{

  ND_LIST *get_evol_tree( ND_LIST *root );
  ND_LIST *get_nd_leaf_header( ND_LIST *node );

  ND_LIST *root = NULL, *leaf_header = NULL;

  /*  In tree.c:  */
  root = (ND_LIST *)get_evol_tree( root );
  leaf_header = (ND_LIST *)get_nd_leaf_header( root );
  
  /*  In this file, below:  */
  do_mammal_evol_w_tree( root, leaf_header );  

}













void do_mammal_evol_w_tree( ND_LIST *root, ND_LIST *leaf_header )
{

  PARAM *param = NULL;
  param = (PARAM *)make_and_initialize_mammal_params( param );
  
  do_mammal_evol_w_params( root, leaf_header, param );

}













PARAM *make_and_initialize_mammal_params( PARAM *param )
{

  param = (PARAM *)malloc( sizeof(PARAM) * NUMB_OF_PARAMS );
  
  param->numb = NUMB_OF_PARAMS;
  (param + 1)->numb = NUMB_OF_PARAMS;
  (param + 2)->numb = NUMB_OF_PARAMS;
  
  (param + EUTH_INDEX)->alpha = ALPHA_EUTH;
  (param + EUTH_INDEX)->beta = BETA_EUTH;
  (param + EUTH_INDEX)->curr_numb = EUTH_INDEX;
  
  (param + CONVERT_INDEX)->alpha = ALPHA_CONVERT;
  (param + CONVERT_INDEX)->beta = BETA_CONVERT;
  (param + CONVERT_INDEX)->curr_numb = CONVERT_INDEX;
  
  (param + ANCESTOR_INDEX)->alpha = ALPHA_ANCESTOR;
  (param + ANCESTOR_INDEX)->beta = BETA_ANCESTOR;
  (param + ANCESTOR_INDEX)->curr_numb = ANCESTOR_INDEX;
  
  return param;

}















void do_mammal_evol_w_params( ND_LIST *root, ND_LIST *leaf_header, PARAM *param )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( PARAM_OUTPUT_FNAME, "" );
  
  do_my_mammal_evol_w_params( root, leaf_header, param, out );
  
  close_file( out );

}













void do_my_mammal_evol_w_params( ND_LIST *root, ND_LIST *leaf_header,
  PARAM *param, FILE *out )
{

  int converged = 0;
  INDEX iter = 0;
  
  while (( ! converged ) && ( iter++ < MAX_ITERATIONS ))
    converged = do_mammal_evol_params_single_iter( root, leaf_header, param,
      iter, out );

}













int do_mammal_evol_params_single_iter( ND_LIST *root, ND_LIST *leaf_header,
  PARAM *param, INDEX iter, FILE *out )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( INPUT_COL_FNAME, "" );
  
  int converged = do_mammal_evol_params_single_iter_w_fh( root, leaf_header, param, 
    iter, in, out );
  
  close_file( in );
  
  return converged;

}












int do_mammal_evol_params_single_iter_w_fh( ND_LIST *root, ND_LIST *leaf_header,
  PARAM *param, INDEX iter, FILE *in, FILE *out )
{

  set_prior_probs_for_mammals( root, leaf_header, in );
  
  return do_my_mammal_evol_params_single_iter_w_fh( root, leaf_header, param,
    iter, in, out );

}













void set_prior_probs_for_mammals( ND_LIST *root, ND_LIST *leaf_header, FILE *in )
{

  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );

  ND_LIST *ancestor = NULL;
  /*  In tree.c:  */
  ancestor = (ND_LIST *)get_nd_lister_from_label( root, "Mammalia" );
  
  /*  In v.c:  */
  set_prior_probs_w_label_str( root, ancestor, leaf_header, "monDom5", in );
  
  set_position( 0, in );

}














int do_my_mammal_evol_params_single_iter_w_fh( ND_LIST *root, ND_LIST *leaf_header, 
  PARAM *param, INDEX iter, FILE *in, FILE *out )
{

  set_mammal_param_types( root );

  return do_mammal_evol_set_param_types( root, leaf_header, param, iter, in, out );

}














void set_mammal_param_types( ND_LIST *root )
{

  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );
  
  ND_LIST *euth = NULL, *ancestor = NULL;
  
  /*  In tree.c:  */
  euth = (ND_LIST *)get_nd_lister_from_label( root, "Eutheria" );
  ancestor = (ND_LIST *)get_nd_lister_from_label( root, "Mammalia" );
  
  /*  In this file, below:  */
  set_param_types_descendent_nodes( root, PARAM_TYPE_UNSET );
  set_param_types_descendent_nodes( ancestor, ANCESTOR_INDEX );
  set_param_types_descendent_nodes( euth, EUTH_INDEX );
  
  euth->my_node->param_type = CONVERT_INDEX;
  
  /*
  char i = 0;
  
  while ( euth->down[ i ] != NULL )
    euth->down[ i++ ]->my_node->param_type = CONVERT_INDEX;
  */
  
}













void set_param_types_descendent_nodes( ND_LIST *node, char param_type )
{

  if ( node == NULL )
    return;
    
  node->my_node->param_type = param_type;
  
  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    set_param_types_descendent_nodes( node->down[ i++ ], param_type );

}













int do_mammal_evol_set_param_types( ND_LIST *root, ND_LIST *leaf_header, 
  PARAM *param, INDEX iter, FILE *in, FILE *out )
{

  PARAM *new_param = NULL;
  new_param = (PARAM *)make_and_copy_params( new_param, param );
  
  get_new_params_mammal_evol( new_param, root, leaf_header, iter, in, out );  
  
  int converged = params_have_converged( new_param, param );
  
  copy_params( param, new_param );
  
  free( new_param );
  
  return converged;
  
}













PARAM *make_and_copy_params( PARAM *new, PARAM *old )
{

  new = (PARAM *)malloc( sizeof(PARAM) * old->numb );
  
  PARAM *new_ptr = new, *old_ptr = old;
  
  new->numb = old->numb;
  
  char i;
  
  for ( i = 0; i < old->numb; i++ )
    *new_ptr++ = *old_ptr++;
    
  return new;

}












void copy_params( PARAM *a, PARAM *b )
{

  register char i;
  
  for ( i = 0; i < a->numb; i++ )
    *(a + i) = *(b + i);

}









int params_have_converged( PARAM *param1, PARAM *param2 )
{

  char i;
  
  for ( i = 0; i < param1->numb; i++ ) {
  
    PARAM *p = param1 + i;
    PARAM *pp = param2 + i;
  
    if ( ABS( p->alpha - pp->alpha ) >= CONVERGE_THRESH )
      return 0;
    
    if ( ABS( p->beta - pp->beta ) >= CONVERGE_THRESH )
      return 0;
    
  }  
    
  return 1;

}








void get_new_params_mammal_evol( PARAM *param, ND_LIST *root, ND_LIST *leaf_header, 
  INDEX iter, FILE *in, FILE *out )
{

  put_trans_probs_into_tree_varying_params( root, param );
  
  get_params_mammal_evol_w_trans_probs( param, root, leaf_header, iter, in, out );
  
  get_params_mammal_evol_w_info( param, root, leaf_header, iter, out );

}












/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/













void put_trans_probs_into_tree_varying_params( ND_LIST *node, PARAM *param )
{

  if ( node == NULL )
    return;
    
  put_my_trans_probs_into_tree_varying_params( node, param );
  
  char i = 0;
  
  while ( node->down[ i ] != NULL )
    put_trans_probs_into_tree_varying_params( node->down[ i++ ], param );

}













void put_my_trans_probs_into_tree_varying_params( ND_LIST *node, PARAM *param )
{

  if ( node->my_node->param_type == PARAM_TYPE_UNSET )
    return;

  PARAM *my_param = param + node->my_node->param_type;
  
  /*  In v.c:  */
  put_my_transition_probs_into_single_node( node, my_param );
  
}














/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/













void get_params_mammal_evol_w_trans_probs( PARAM *param, ND_LIST *root, 
  ND_LIST *leaf_header, INDEX iter, FILE *in, FILE *out )
{

  STR_LIST *get_label_list_from_first_line( STR_LIST *label_header, FILE *in );

  STR_LIST *label_header = NULL;
  /*  In set.c:  */
  label_header = (STR_LIST *)get_label_list_from_first_line( label_header, in );
  
  /*  In this file, below:  */
  get_params_mammal_evol_w_label_list( param, root, leaf_header, label_header,
    iter, in, out );
  
}














void get_params_mammal_evol_w_label_list( PARAM *param, ND_LIST *root, 
  ND_LIST *leaf_header, STR_LIST *label_header, INDEX iter, FILE *in, FILE *out )
{

  SEQ seq_numb = 0;

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL )
    get_params_mammal_evol_single_gene( root, leaf_header, label_header, 
      ++seq_numb, iter, line );

}













void get_params_mammal_evol_single_gene( ND_LIST *root, ND_LIST *leaf_header,
  STR_LIST *label_header, SEQ seq_numb, INDEX iter, char line[] )
{

  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );
  
  if ( seq_numb > MAX_SEQS )
    return;
  
  ND_LIST *ancestor = NULL;
  /*  In tree.c:  */
  ancestor = (ND_LIST *)get_nd_lister_from_label( root, "Mammalia" );

  printf( "Iter :  %d      Seq :  %d\n", (int)iter, (int)seq_numb );
  
  /*  In set.c:  */
  initialize_nd_tree( root );
  
  /*  In set.c:  */
  set_my_evolutionary_tree_single_gene( root, leaf_header, label_header,
    line, MY_NUCL );
    
  /*  In v.c:  */
  do_evol_fill_phylo( ancestor, NULL );
    
  /*  In this file, below:  */
  get_params_mammal_evol_single_gene_filled_nodes( ancestor );

}














void get_params_mammal_evol_single_gene_filled_nodes( ND_LIST *ancestor )
{

  /*  In get_n.c:  */
  evol_fill_phylo_n_vals( ancestor, ancestor );
  
  /*  In v.c:  */
  update_phylo_total_n_vals( ancestor );

}













/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/















void get_params_mammal_evol_w_info( PARAM *param, ND_LIST *root, 
  ND_LIST *leaf_header, INDEX iter, FILE *out )
{

  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );
  
  ND_LIST *ancestor = NULL;
  /*  In tree.c:  */
  ancestor = (ND_LIST *)get_nd_lister_from_label( root, "Mammalia" );

  /*  In newton.c:  */
  get_params_newton( param, ancestor, iter, out );

}









