#include "matrix.h"





















void set_matrix_inv( TT_MX *inv, TT_MX *mx )
{

  if (( inv->i != mx->i ) || ( inv->j != mx->j ))
    error( 9600, "Unmatching matrix sizes.\n" );
    
  set_my_matrix_inv( inv, mx );

}













void set_my_matrix_inv( TT_MX *inv, TT_MX *mx )
{

  TT_MX *create_tt_mx( TT_MX *xx, INDEX ii, INDEX jj );
  
  TT_MX *vector = NULL, *sol = NULL;
  vector = (TT_MX *)create_tt_mx( vector, mx->i, 1 );
  sol = (TT_MX *)create_tt_mx( sol, mx->i, 1 );
  
  calc_matrix_inv( inv, mx, vector, sol );
  
  kill_tt_mx( vector );
  kill_tt_mx( sol );

}














void calc_matrix_inv( TT_MX *inv, TT_MX *mx, TT_MX *vector, TT_MX *sol )
{

  register INDEX col = 0;
  
  while ( col < mx->i )
    compute_inv_column( inv, mx, vector, sol, col++ );

}













void compute_inv_column( TT_MX *inv, TT_MX *mx, TT_MX *vector, TT_MX *sol,
  INDEX col )
{

  get_ident_vector_for_inv( vector, col );
  system_of_eq( sol, vector, mx );
  put_column_into_matrix( inv, sol, col );

}










void get_ident_vector_for_inv( TT_MX *vector, INDEX col )
{

  register char i;
  
  for ( i = 0; i < vector->i; i++ )
    *(vector->x + i) = 0.0;
    
  *(vector->x + col) = 1.0;

}














void system_of_eq( TT_MX *sol, TT_MX *vector, TT_MX *mx )
{

  TT_MX *create_tt_mx( TT_MX *xx, INDEX ii, INDEX jj );
  
  TT_MX *L = NULL, *U = NULL;
  L = (TT_MX *)create_tt_mx( L, mx->i, mx->j );
  U = (TT_MX *)create_tt_mx( U, mx->i, mx->j );
  
  do_system_of_eq( sol, vector, mx, L, U );
  
  kill_tt_mx( L );
  kill_tt_mx( U );

}













void do_system_of_eq( TT_MX *sol, TT_MX *vector, TT_MX *mx, TT_MX *L, TT_MX *U )
{

  set_diag_l( L );
  
  do_lu_decomp( L, U, mx );
  
  do_my_system_of_eq( sol, L, U, vector );

}











void set_diag_l( TT_MX *L )
{

  register char col = 0;
  
  do {
    *(L->x + (col * L->j) + col) = 1.0;
  } while ( ++col < L->i );

}















void do_lu_decomp( TT_MX *L, TT_MX *U, TT_MX *mx )
{

  INDEX i, k;
  
  for ( k = 0; k < mx->i; k++ ) {
  
    for ( i = 0; i <= k; i++ )
      set_u_element( L, U, mx, i, k );
      
    for ( i = k + 1; i < mx->i; i++ )
      set_l_element( L, U, mx, i, k );
  
  }

}












void set_u_element( TT_MX *L, TT_MX *U, TT_MX *mx, INDEX i, INDEX k )
{

  INDEX index = (i * mx->j) + k;
  
  *(U->x + index) = *(mx->x + index);
  
  register char kk = 0;
  
  while ( kk < i ) {
    *(U->x + index) -= (*(L->x + (i * L->j) + kk)) * (*(U->x + (kk * U->j) + k));
    ++kk;
  }

}













void set_l_element( TT_MX *L, TT_MX *U, TT_MX *mx, INDEX i, INDEX k )
{

  INDEX index = (i * mx->j) + k;
  
  *(L->x + index) = *(mx->x + index);
  
  register char kk = 0;
  
  while ( kk < k ) {
    *(L->x + index) -= (*(L->x + (i * L->j) + kk)) * (*(U->x + (kk * U->j) + k));
    ++kk;
  }
  
  *(L->x + index) /= (*(U->x + (k * U->j) + k));

}













void do_my_system_of_eq( TT_MX *sol, TT_MX *L, TT_MX *U, TT_MX *vector )
{

  TT_MX *create_tt_mx( TT_MX *xx, INDEX ii, INDEX jj );
  
  TT_MX *y = NULL;
  y = (TT_MX *)create_tt_mx( y, vector->i, 1 );
  
  do_system_of_eq_wy_vec( sol, L, U, vector, y );
  
  kill_tt_mx( y );

}










void do_system_of_eq_wy_vec( TT_MX *sol, TT_MX *L, TT_MX *U, TT_MX *vector,
  TT_MX *y )
{
  find_y_system_of_eq( y, L, vector );
  find_sol_vector_system_of_eq( sol, U, y ); 
}











void find_y_system_of_eq( TT_MX *y, TT_MX *L, TT_MX *vector )
{

  TT *y_ptr = y->x;
  TT *vec_ptr = vector->x;
  
  *y_ptr++ = *vec_ptr++ / *(L->x);
  
  register INDEX j, k = 1;
  
  do {
  
    *y_ptr = *vec_ptr++;
    
    TT *y_ptr2 = y->x;
    
    for ( j = 0; j < k; j++ ) {
      *y_ptr -= (*(L->x + (k * L->j) + j)) * (*y_ptr2++);
    }
    
    *y_ptr++ /= (*(L->x + (k * L->j) + k));
  
  } while ( ++k < y->i );

}













void find_sol_vector_system_of_eq( TT_MX *sol, TT_MX *U, TT_MX *y )
{

  TT *y_ptr = y->x + y->i - 1;
  
  TT *sol_ptr = sol->x + sol->i - 1;
  
  *sol_ptr-- = (*y_ptr--) / (*(U->x + ((U->i - 1) * (U->i + 1))));
  
  register int j, k = sol->i - 2;
  
  do {
  
    *sol_ptr = *y_ptr--;
    
    TT *sol_ptr2 = sol->x + sol->i - 1;
    
    for ( j = U->i - 1; j > k; j-- ) {
      *sol_ptr -= (*(U->x + (k * U->j) + j)) * (*sol_ptr2--);
    }
    
    *sol_ptr-- /= (*(U->x + (k * U->j) + k));
  
  } while ( --k >= 0 );

}














void put_column_into_matrix( TT_MX *inv, TT_MX *sol, INDEX col )
{

  TT *ptr = inv->x + col;
  TT *y_ptr = sol->x;
  
  register char i = 0;
  
  while ( i++ < inv->i ) {
    *ptr = *y_ptr++;
    ptr += inv->j;
  }

}














/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/















void mult_matrices( TT_MX *y, TT_MX *a, TT_MX *b )
{

  if (( y->i != a->i ) || ( y->j != b->j ) || ( a->j != b->i ))
    error( 9700, "Unmatching dimensions for matrix multiplication.\n" );
    
  mult_my_matrices( y, a, b );

}











void mult_my_matrices( TT_MX *y, TT_MX *a, TT_MX *b )
{

  register INDEX i, j, k;
  
  for ( i = 0; i < y->i; i++ ) {
  
    for ( j = 0; j < y->j; j++ ) {
    
      *(y->x + (i * y->j) + j) = 0.0;
      
      for ( k = 0; k < a->j; k++ ) {
        *(y->x + (i * y->j) + j) 
          += (*(a->x + (i * a->j) + k)) * (*(b->x + (k * b->j) + j));
    
      }
    
    }
  
  }

}














void mult_matrix_by_scalar( TT_MX *x, float k )
{

  register INDEX i, j;
  
  for ( i = 0; i < x->i; i++ ) {
  
    for ( j = 0; j < x->j; j++ ) {  
      
      *(x->x + (i * x->j) + j) *= k;
    
    }
  
  }

}






















