#include "max.h"












#define INPUT_COL_FNAME "data/ft_hg19/change/hg19.gcBox.count.txt"

#define BIRD_FNAME "data/ft_hg19/evol/ancestor/bird_ancestor.GAbox.max.txt"
#define EUTH_FNAME "data/ft_hg19/evol/ancestor/euth_ancestor.GAbox.max.txt"
#define MAM_FNAME "data/ft_hg19/evol/ancestor/mammal_ancestor.GAbox.max.txt"

#define MY_NUCL A_NUCL

#define OUTPUT_MF_BUFFER 7



#define MAX(a,b) (((a) > (b)) ? (a) : (b))
#define MIN(a,b) (((a) < (b)) ? (a) : (b))












void do_lineage_max( void )
{

  FH my_fh;
  
  initialize_lineage_max( &my_fh );
  
  do_my_lineage_max( &my_fh );
  
  close_lineage_max( &my_fh );

}











void initialize_lineage_max( FH *my_fh )
{

  FILE *open_text_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );
  
  my_fh->in = (FILE *)open_text_file_reading( INPUT_COL_FNAME, "" );
  my_fh->mam_out = (FILE *)open_text_file_writing( MAM_FNAME, "" );
  my_fh->euth_out = (FILE *)open_text_file_writing( EUTH_FNAME, "" );
  my_fh->bird_out = (FILE *)open_text_file_writing( BIRD_FNAME, "" );

}














void close_lineage_max( FH *my_fh )
{
  close_file( my_fh->in );
  close_file( my_fh->mam_out );
  close_file( my_fh->euth_out );
  close_file( my_fh->bird_out );
}













void do_my_lineage_max( FH *my_fh )
{

  ND_LIST *get_evol_tree( ND_LIST *root );
  ND_LIST *get_nd_leaf_header( ND_LIST *node );

  ND_LIST *root = NULL, *leaf_header = NULL;

  /*  In tree.c:  */
  root = (ND_LIST *)get_evol_tree( root );
  leaf_header = (ND_LIST *)get_nd_leaf_header( root );
  
  /*  In this file, below:  */
  do_lineage_max_w_tree( root, leaf_header, my_fh );

}













void do_lineage_max_w_tree( ND_LIST *root, ND_LIST *leaf_header, FH *my_fh )
{

  STR_LIST *get_label_list_from_first_line( STR_LIST *label_header, FILE *in );

  STR_LIST *label_header = NULL;

  set_position( 0, my_fh->in );
  
  /*  In set.c:  */
  label_header = (STR_LIST *)get_label_list_from_first_line( label_header, 
    my_fh->in );
  
  /*  In this file, below:  */
  do_my_lineage_max_w_tree( root, leaf_header, label_header, my_fh );

}














void do_my_lineage_max_w_tree( ND_LIST *root, ND_LIST *leaf_header, 
  STR_LIST *label_header, FH *my_fh )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, my_fh->in ) != NULL )
    get_lineage_max_single_gene( root, leaf_header, label_header, line, my_fh );

}












void get_lineage_max_single_gene( ND_LIST *root, ND_LIST *leaf_header,
  STR_LIST *label_header, char line[], FH *my_fh )
{

  char gene[ BUFFER ], acc[ BUFFER ];
  int numb;
  
  sscanf( line, "%d%s%s", &numb, acc, gene );

  printf( "%5d :  %s\n", numb, gene );
  
  /*  In set.c:  */
  initialize_nd_tree( root );
  
  /*  In set.c:  */
  set_my_evolutionary_tree_single_gene( root, leaf_header, label_header,
    line, MY_NUCL );
    
  /*  In anc.c:  */
  fill_phylo_ancestor( root );
    
  /*  In this file, below:  */
  get_lineage_max_single_gene_set_tree( root, numb, gene, acc, my_fh );

}












void get_lineage_max_single_gene_set_tree( ND_LIST *root, int numb, char gene[],
  char acc[], FH *my_fh )
{

  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );
  
  ND_LIST *mam = NULL, *euth = NULL, *bird = NULL;
  /*  In tree.c:  */
  mam = (ND_LIST *)get_nd_lister_from_label( root, "Mammalia" );
  euth = (ND_LIST *)get_nd_lister_from_label( root, "Eutheria" );
  bird = (ND_LIST *)get_nd_lister_from_label( root, "Neognathae" );
  
  /*  In this file, below:  */
  get_and_put_lineage_max_single_node( mam, numb, gene, acc, my_fh->mam_out );
  get_and_put_lineage_max_single_node( euth, numb, gene, acc, my_fh->euth_out );
  get_and_put_lineage_max_single_node( bird, numb, gene, acc, my_fh->bird_out );

}













void get_and_put_lineage_max_single_node( ND_LIST *node, int numb, char gene[],
  char acc[], FILE *out )
{

  register char k;
  
  fprintf( out, "%6d\t%s\t%12s\t", numb, acc, gene );

  if ( ! node->my_node->exists ) {
    for ( k = 0; k < OUTPUT_MF_BUFFER; k++ )
      fprintf( out, "%10s\t", "---" );
    fprintf( out, "\n" );
    return;
  }

  int max = get_max_downstream_nodes( node, 0 );  
  
  max = MIN( max, OUTPUT_MF_BUFFER - 1 );
  
  for ( k = 0; k < OUTPUT_MF_BUFFER; k++ ) {
    if ( k == max )
      fprintf( out, "%10.3e\t", 1.0 );
    else
      fprintf( out, "%10.3e\t", 0.0 );
  }
  
  fprintf( out, "\n" );
  
}











int get_max_downstream_nodes( ND_LIST *node, int x )
{

  if ( ! node->my_node->exists )
    return x;

  if ( node->down[ 0 ] == NULL )
    return get_single_downstream_node_count( node );
    
  register char i = 0;  
    
  while ( node->down[ i ] != NULL ) {
    int y = get_max_downstream_nodes( node->down[ i++ ], x );
    x = MAX( x, y );
  }
    
  return x;

}













int get_single_downstream_node_count( ND_LIST *node )
{

  register char i;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {
    if ( *(node->my_node->data->pr->x + i) == 1.0 )
      return i;
  }
  
  error( 10500, "Did not find motif count.\n" );

}














