#include "my_lhood.h"

















#define EUTH_ALPHA 8.5e-12
#define EUTH_BETA 1.0e-8
#define CONVERT_ALPHA 3.3e-11
#define CONVERT_BETA 5.0e-9
#define ANCESTOR_ALPHA 2.1e-11
#define ANCESTOR_BETA 6.6e-9




#define ALPHA_STEP 5e-12
#define BETA_STEP 2e-9

#define ALPHA_STEP_THRESH 1e-13
#define BETA_STEP_THRESH 1e-11

#define PARAM_BUFFER 3

#define MAX_LIST_LENGTH 150

#define MAX_ITER 25

#define POINT_RADIUS 2

#define MAX_SEQS 22000




#define PARAM_COUNT 1





#define EUTH_NODE_LABEL "Eutheria"
#define EUTH_LINEAGE_ONE "Primates"
#define EUTH_LINEAGE_TWO "Laurasiatheria"
#define DO_TWO_LINEAGES 0





#define MY_NUCL C_NUCL



#define INPUT_COL_FNAME "data/ft_hg19/change/hg19.gcBox.count.txt"
#define OUTPUT_FNAME "data/ft_hg19/evol/params_step.GCbox.allGenes.oneParam.txt"















void do_params_lhood_step( void )
{

  ND_LIST *get_evol_tree( ND_LIST *root );
  ND_LIST *get_nd_leaf_header( ND_LIST *node );

  ND_LIST *root = NULL, *leaf_header = NULL;

  /*  In tree.c:  */
  root = (ND_LIST *)get_evol_tree( root );
  leaf_header = (ND_LIST *)get_nd_leaf_header( root );
  
  /*  In this file, below:  */
  do_my_params_lhood_step( root, leaf_header );  

}















void do_my_params_lhood_step( ND_LIST *root, ND_LIST *leaf_header )
{

  FH *my_fh = NULL;
  my_fh = (FH *)malloc( sizeof(FH) );
  
  set_params_lhood_fh_step( my_fh );
  
  set_prior_probs_for_lhood_step( root, leaf_header, MY_NUCL, my_fh->in );
  
  do_params_lhood_step_w_tree( root, leaf_header, my_fh );
  
  close_params_lhood_fh_step( my_fh );

}














void set_params_lhood_fh_step( FH *my_fh )
{

  FILE *open_text_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );
  
  my_fh->in = (FILE *)open_text_file_reading( INPUT_COL_FNAME, "" );
  my_fh->out = (FILE *)open_text_file_writing( OUTPUT_FNAME, "" );

}















void close_params_lhood_fh_step( FH *my_fh )
{

  close_file( my_fh->in );
  close_file( my_fh->out );

}













/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/












void set_prior_probs_for_lhood_step( ND_LIST *root, ND_LIST *leaf_header, 
  char my_nucl, FILE *in )
{

  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );

  ND_LIST *ancestor = NULL;
  /*  In tree.c:  */
  ancestor = (ND_LIST *)get_nd_lister_from_label( root, "Mammalia" );
  
  set_position( 0, in );
  
  /*  In this file, below:  */
  set_prior_probs_for_lhood_step_anc_nd( root, leaf_header, ancestor, 
    my_nucl, "monDom5", in );
    
}














void set_prior_probs_for_lhood_step_anc_nd( ND_LIST *root, ND_LIST *leaf_header,
  ND_LIST *ancestor, char my_nucl, char label[], FILE *in )
{

  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );
  
  ND_LIST *spec = NULL;
  /*  In tree.c:  */
  spec = (ND_LIST *)get_nd_lister_from_label( root, label );
  
  /*  In this file, below:  */
  set_prior_probs_for_lhood_step_ancestor( root, leaf_header, ancestor, spec, 
    my_nucl, in );

}















void set_prior_probs_for_lhood_step_ancestor( ND_LIST *root, ND_LIST *leaf_header,
  ND_LIST *ancestor, ND_LIST *spec, char my_nucl, FILE *in )
{

  TT_MX *create_tt_mx( TT_MX *xx, INDEX ii, INDEX jj );
  
  TT_MX *mx = NULL;
  mx = (TT_MX *)create_tt_mx( mx, NUMB_OF_MFS, 1 );

  set_prior_probs_lhood_step_w_tt_mx( root, leaf_header, ancestor, spec, mx, 
    my_nucl, in );
  
  kill_tt_mx( mx );

}













void set_prior_probs_lhood_step_w_tt_mx( ND_LIST *root, ND_LIST *leaf_header, 
  ND_LIST *ancestor, ND_LIST *spec, TT_MX *mx, char my_nucl, FILE *in )
{

  STR_LIST *get_label_list_from_first_line( STR_LIST *label_header, FILE *in );
  
  STR_LIST *label_header = NULL;
  /*  In set.c:  */
  label_header = (STR_LIST *)get_label_list_from_first_line( label_header, in );
  
  /*  In this file, below:  */
  set_prior_probs_lhood_step_label_list( root, leaf_header, ancestor, spec, 
    label_header, mx, my_nucl, in );

}















void set_prior_probs_lhood_step_label_list( ND_LIST *root, ND_LIST *leaf_header,
  ND_LIST *ancestor, ND_LIST *spec, STR_LIST *label_header, TT_MX *mx, char my_nucl, 
  FILE *in )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL )
    /*  In this file, below:  */
    update_counts_prior_prob_lhood_step_single_gene( root, leaf_header, spec,
      label_header, line, mx, my_nucl );
     
  /*  In v.c:  */    
  set_prior_probs_w_count_mx( ancestor, mx );

}













void update_counts_prior_prob_lhood_step_single_gene( ND_LIST *root,
  ND_LIST *leaf_header, ND_LIST *spec, STR_LIST *label_header, char line[],
  TT_MX *mx, char my_nucl )
{

  /*  In set.c:  */
  initialize_nd_tree( root );

  /*  In set.c:  */
  set_my_evolutionary_tree_single_gene( root, leaf_header, label_header, line,
    my_nucl );
    
  if ( ! spec->my_node->exists )
    return;

  /*  In v.c:  */
  update_my_counts_for_prior_probs_single_gene( mx, spec );
    
}












/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/










void do_params_lhood_step_w_tree( ND_LIST *root, ND_LIST *leaf_header, FH *my_fh )
{

  LHOOD_LIST *lhood_header = NULL;
  
  /*  In mammal.c:  */
  set_mammal_param_types( root );
  
  if ( PARAM_COUNT == 3 )
  /*  In this file, below:  */
  lhood_header = (LHOOD_LIST *)get_params_lhood_step_list( lhood_header, root,
    leaf_header, my_fh );

  if ( PARAM_COUNT == 1 )
    /*  In this file, below:  */
    lhood_header = (LHOOD_LIST *)get_params_lhood_step_list_one_p( lhood_header,
      root, leaf_header, my_fh );

  /*  In this file, below:  */
  print_lhood_list_step_fh( lhood_header, my_fh );
    
}












void setup_two_lineage_euth_lineage( ND_LIST *root )
{

  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );
  
  ND_LIST *euth = NULL, *euth_one = NULL, *euth_two = NULL;
  /*  In tree.c:  */
  euth = (ND_LIST *)get_nd_lister_from_label( root, EUTH_NODE_LABEL );
  euth_one = (ND_LIST *)get_nd_lister_from_label( root, EUTH_LINEAGE_ONE );
  euth_two = (ND_LIST *)get_nd_lister_from_label( root, EUTH_LINEAGE_TWO );
  
  euth->down[ 0 ] = euth_one;
  euth->down[ 1 ] = euth_two;
  euth->down[ 2 ] = NULL;

}










LHOOD_LIST *get_params_lhood_step_list( LHOOD_LIST *lhood_header, ND_LIST *root,
  ND_LIST *leaf_header, FH *my_fh )
{

  int iter = 0;
  
  TT ea = EUTH_ALPHA;
  TT eb = EUTH_BETA;
  TT ca = CONVERT_ALPHA;
  TT cb = CONVERT_BETA;
  TT aa = ANCESTOR_ALPHA;
  TT ab = ANCESTOR_BETA;
  
  TT ea_step = ALPHA_STEP;
  TT eb_step = BETA_STEP;
  TT ca_step = ALPHA_STEP;
  TT cb_step = BETA_STEP;
  TT aa_step = ALPHA_STEP;
  TT ab_step = BETA_STEP;
  
  while (( iter++ < MAX_ITER ) && 
    ( ! steps_have_converged( ea_step, ca_step, aa_step, eb_step, cb_step, 
      ab_step )))
  {
    lhood_header = (LHOOD_LIST *)get_params_lhood_single_step( lhood_header,
      root, leaf_header, &ea, &eb, &ca, &cb, &aa, &ab, &ea_step, &eb_step, 
      &ca_step, &cb_step, &aa_step, &ab_step, my_fh );
  }

  return lhood_header;

}













int steps_have_converged( TT a1, TT a2, TT a3, TT b1, TT b2, TT b3 )
{

  printf( "Euth alpha step :  %.2e\n", (float)a1 );
  printf( "Euth beta step  :  %.2e\n", (float)b1 );
  printf( "Conv alpha step :  %.2e\n", (float)a2 );
  printf( "Conv beta step  :  %.2e\n", (float)b2 );
  printf( "Anc alpha step  :  %.2e\n", (float)a3 );
  printf( "Anc beta step   :  %.2e\n", (float)b3 );

  if ( a1 >= ALPHA_STEP_THRESH )    return 0;
  if ( a2 >= ALPHA_STEP_THRESH )    return 0;
  if ( a3 >= ALPHA_STEP_THRESH )    return 0;

  if ( b1 >= BETA_STEP_THRESH )     return 0;
  if ( b2 >= BETA_STEP_THRESH )     return 0;
  if ( b3 >= BETA_STEP_THRESH )     return 0;
  
  return 1;

}











LHOOD_LIST *get_params_lhood_single_step( LHOOD_LIST *lhood_header,
  ND_LIST *root, ND_LIST *leaf_header, TT *ea, TT *eb, TT *ca, TT *cb, TT *aa, 
  TT *ab, TT *ea_step, TT *eb_step, TT *ca_step, TT *cb_step, TT *aa_step, 
  TT *ab_step, FH *my_fh )
{

  lhood_header = (LHOOD_LIST *)get_params_lhood_step_euth( lhood_header,
    root, leaf_header, ea, eb, ea_step, eb_step, *ca, *cb, *aa, *ab, my_fh );
    
  lhood_header = (LHOOD_LIST *)get_params_lhood_step_anc( lhood_header,
    root, leaf_header, aa, ab, aa_step, ab_step, *ea, *eb, *ca, *cb, my_fh );

  lhood_header = (LHOOD_LIST *)get_params_lhood_step_conv( lhood_header, 
    root, leaf_header, ca, cb, ca_step, cb_step, *ea, *eb, *aa, *ab, my_fh );
    
  return lhood_header;

}














LHOOD_LIST *get_params_lhood_step_euth( LHOOD_LIST *lhood_header, ND_LIST *root,
  ND_LIST *leaf_header, TT *ea, TT *eb, TT *ea_step, TT *eb_step, TT ca, TT cb, 
  TT aa, TT ab, FH *my_fh )
{

  TT ea_min = *ea - (POINT_RADIUS * (*ea_step));
  TT ea_max = *ea + (POINT_RADIUS * (*ea_step));
  TT eb_min = *eb - (POINT_RADIUS * (*eb_step));
  TT eb_max = *eb + (POINT_RADIUS * (*eb_step));

  TT a, b;

  for ( a = ea_min; a <= ea_max; a += *ea_step ) {
  
    for ( b = eb_min; b <= eb_max; b += *eb_step ) {
    
      if (( a <= 0.0 ) || ( b <= 0.0 ))
        continue;
    
      lhood_header = (LHOOD_LIST *)get_params_lhood_single_step_lister( lhood_header,
        root, leaf_header, a, b, ca, cb, aa, ab, my_fh );
    
    }
  
  }
  
  if ( on_the_edge( lhood_header->my_lhood->euth_alpha, 
    lhood_header->my_lhood->euth_beta, ea_min, ea_max, eb_min, eb_max ))
  {
  
    *ea = lhood_header->my_lhood->euth_alpha;
    *eb = lhood_header->my_lhood->euth_beta;
  
    return get_params_lhood_step_euth( lhood_header, root, leaf_header,
      ea, eb, ea_step, eb_step, 
      lhood_header->my_lhood->conv_alpha, lhood_header->my_lhood->conv_beta,
      lhood_header->my_lhood->anc_alpha, lhood_header->my_lhood->anc_beta,
      my_fh );
  }
      
  *ea = lhood_header->my_lhood->euth_alpha;
  *eb = lhood_header->my_lhood->euth_beta;
  
  *ea_step /= (POINT_RADIUS * 2.0);
  *eb_step /= (POINT_RADIUS * 2.0);

  return lhood_header;

}











int on_the_edge( TT alpha, TT beta, TT amin, TT amax, TT bmin, TT bmax )
{

  if ( alpha == amin )   return 1;
  if ( alpha == amax )   return 1;
  if ( beta == bmin )    return 1;
  if ( beta == bmax )    return 1;
  
  return 0;

}












LHOOD_LIST *get_params_lhood_step_anc( LHOOD_LIST *lhood_header, ND_LIST *root, 
  ND_LIST *leaf_header, TT *aa, TT *ab, TT *aa_step, TT *ab_step, TT ea, TT eb, 
  TT ca, TT cb, FH *my_fh )
{

  TT aa_min = *aa - (POINT_RADIUS * (*aa_step));
  TT aa_max = *aa + (POINT_RADIUS * (*aa_step));
  TT ab_min = *ab - (POINT_RADIUS * (*ab_step));
  TT ab_max = *ab + (POINT_RADIUS * (*ab_step));

  TT a, b;

  for ( a = aa_min; a <= aa_max; a += *aa_step ) {
  
    for ( b = ab_min; b <= ab_max; b += *ab_step ) {
    
      if (( a <= 0.0 ) || ( b <= 0.0 ))
        continue;
    
      lhood_header = (LHOOD_LIST *)get_params_lhood_single_step_lister( lhood_header,
        root, leaf_header, ea, eb, ca, cb, a, b, my_fh );
    
    }
  
  }
  
  if ( on_the_edge( lhood_header->my_lhood->anc_alpha, 
    lhood_header->my_lhood->anc_beta, aa_min, aa_max, ab_min, ab_max ))
  {
  
    *aa = lhood_header->my_lhood->anc_alpha;
    *ab = lhood_header->my_lhood->anc_beta;
  
    return get_params_lhood_step_anc( lhood_header, root, leaf_header,
      aa, ab, aa_step, ab_step, 
      lhood_header->my_lhood->euth_alpha, lhood_header->my_lhood->euth_beta,
      lhood_header->my_lhood->conv_alpha, lhood_header->my_lhood->conv_beta,
      my_fh );
  }
      
  *aa = lhood_header->my_lhood->anc_alpha;
  *ab = lhood_header->my_lhood->anc_beta;
  
  *aa_step /= (POINT_RADIUS * 2.0);
  *ab_step /= (POINT_RADIUS * 2.0);

  return lhood_header;

}












LHOOD_LIST *get_params_lhood_step_conv( LHOOD_LIST *lhood_header, ND_LIST *root, 
  ND_LIST *leaf_header, TT *ca, TT *cb, TT *ca_step, TT *cb_step, TT ea, TT eb, 
  TT aa, TT ab, FH *my_fh )
{

  TT ca_min = *ca - (POINT_RADIUS * (*ca_step));
  TT ca_max = *ca + (POINT_RADIUS * (*ca_step));
  TT cb_min = *cb - (POINT_RADIUS * (*cb_step));
  TT cb_max = *cb + (POINT_RADIUS * (*cb_step));

  TT a, b;

  for ( a = ca_min; a <= ca_max; a += *ca_step ) {
  
    for ( b = cb_min; b <= cb_max; b += *cb_step ) {
    
      if (( a <= 0.0 ) || ( b <= 0.0 ))
        continue;
    
      lhood_header = (LHOOD_LIST *)get_params_lhood_single_step_lister( lhood_header,
        root, leaf_header, ea, eb, a, b, aa, ab, my_fh );
    
    }
  
  }
  
  if ( on_the_edge( lhood_header->my_lhood->conv_alpha, 
    lhood_header->my_lhood->conv_beta, ca_min, ca_max, cb_min, cb_max ))
  {
  
    *ca = lhood_header->my_lhood->conv_alpha;
    *cb = lhood_header->my_lhood->conv_beta;
  
    return get_params_lhood_step_conv( lhood_header, root, leaf_header,
      ca, cb, ca_step, cb_step, 
      lhood_header->my_lhood->euth_alpha, lhood_header->my_lhood->euth_beta,
      lhood_header->my_lhood->anc_alpha, lhood_header->my_lhood->anc_beta,
      my_fh );
  }
      
  *ca = lhood_header->my_lhood->conv_alpha;
  *cb = lhood_header->my_lhood->conv_beta;
  
  *ca_step /= (POINT_RADIUS * 2.0);
  *cb_step /= (POINT_RADIUS * 2.0);

  return lhood_header;

}
















LHOOD_LIST *get_params_lhood_step_list_one_p( LHOOD_LIST *lhood_header,
  ND_LIST *root, ND_LIST *leaf_header, FH *my_fh )
{

  int iter = 0;
  
  TT a = EUTH_ALPHA;
  TT b = EUTH_BETA;
  
  TT a_step = ALPHA_STEP;
  TT b_step = BETA_STEP;
  
  while (( iter++ < MAX_ITER ) && 
    ( ! steps_have_converged( a_step, a_step, a_step, b_step, b_step, b_step )))
  {
    lhood_header = (LHOOD_LIST *)get_params_lhood_single_step_one_p( lhood_header,
      root, leaf_header, &a, &b, &a_step, &b_step, my_fh );
  }

  return lhood_header;

}













LHOOD_LIST *get_params_lhood_single_step_one_p( LHOOD_LIST *lhood_header,
  ND_LIST *root, ND_LIST *leaf_header, TT *aa, TT *bb, TT *a_step, TT *b_step,
  FH *my_fh )
{

  TT a_min = *aa - (POINT_RADIUS * (*a_step));
  TT a_max = *aa + (POINT_RADIUS * (*a_step));
  TT b_min = *bb - (POINT_RADIUS * (*b_step));
  TT b_max = *bb + (POINT_RADIUS * (*b_step));

  TT a, b;

  for ( a = a_min; a <= a_max; a += *a_step ) {
  
    for ( b = b_min; b <= b_max; b += *b_step ) {
    
      if (( a <= 0.0 ) || ( b <= 0.0 ))
        continue;
    
      lhood_header = (LHOOD_LIST *)get_params_lhood_single_step_lister( lhood_header,
        root, leaf_header, a, b, a, b, a, b, my_fh );
    
    }
  
  }
  
  if ( on_the_edge( lhood_header->my_lhood->euth_alpha, 
    lhood_header->my_lhood->euth_beta, a_min, a_max, b_min, b_max ))
  {
  
    *aa = lhood_header->my_lhood->euth_alpha;
    *bb = lhood_header->my_lhood->euth_beta;
  
    return get_params_lhood_single_step_one_p( lhood_header, root, leaf_header,
      aa, bb, a_step, b_step, my_fh );
  }
      
  *aa = lhood_header->my_lhood->euth_alpha;
  *bb = lhood_header->my_lhood->euth_beta;
  
  *a_step /= (POINT_RADIUS * 2.0);
  *b_step /= (POINT_RADIUS * 2.0);

  return lhood_header;

}









/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/














LHOOD_LIST *get_params_lhood_single_step_lister( LHOOD_LIST *lhood_header, 
  ND_LIST *root, ND_LIST *leaf_header, TT ea, TT eb, TT ca, TT cb, TT aa, TT ab, 
  FH *my_fh )
{

  LHOOD_LIST *put_lhood_lister_into_list( LHOOD_LIST *header, LHOOD_LIST *lhood );

  LHOOD_LIST *lhood = NULL;
  lhood = (LHOOD_LIST *)create_my_lhood_lister( lhood );
  
  set_my_lhood_params( lhood, ea, eb, ca, cb, aa, ab );
  
  set_params_lhood_step_w_tree( lhood, root, leaf_header, my_fh );

  return put_lhood_lister_step_into_list( lhood_header, lhood );  

}












void set_my_lhood_params( LHOOD_LIST *lhood, TT ea, TT eb, TT ca, TT cb, 
  TT aa, TT ab )
{

  lhood->my_lhood->euth_alpha = ea;
  lhood->my_lhood->euth_beta = eb;
  lhood->my_lhood->conv_alpha = ca;
  lhood->my_lhood->conv_beta = cb;
  lhood->my_lhood->anc_alpha = aa;
  lhood->my_lhood->anc_beta = ab;

}















void set_params_lhood_step_w_tree( LHOOD_LIST *lhood, ND_LIST *root, 
  ND_LIST *leaf_header, FH *my_fh )
{

  PARAM *param = NULL;
  /*  In this file, below:  */
  param = (PARAM *)make_and_set_param_from_lhood_step( param, lhood );
  
  /*  In mammal.c:  */
  put_trans_probs_into_tree_varying_params( root, param );
    
  /*  In this file, below:  */
  lhood->my_lhood->lhood = get_params_lhood_step_w_tree_params( root, leaf_header,
    param, my_fh );
    
  free( param );

}











PARAM *make_and_set_param_from_lhood_step( PARAM *param, LHOOD_LIST *lhood )
{

  param = (PARAM *)malloc( sizeof(PARAM) * PARAM_BUFFER );
  
  PARAM *p = param;
  
  (p + 0)->alpha = lhood->my_lhood->euth_alpha;
  (p + 0)->beta = lhood->my_lhood->euth_beta;
  (p + 0)->numb = PARAM_BUFFER;
  (p + 0)->curr_numb = 0;
  
  (p + 1)->alpha = lhood->my_lhood->conv_alpha;
  (p + 1)->beta = lhood->my_lhood->conv_beta;
  (p + 1)->numb = PARAM_BUFFER;
  (p + 1)->curr_numb = 1;

  (p + 2)->alpha = lhood->my_lhood->anc_alpha;
  (p + 2)->beta = lhood->my_lhood->anc_beta;
  (p + 2)->numb = PARAM_BUFFER;
  (p + 2)->curr_numb = 2;
  
  return param;

}














TT get_params_lhood_step_w_tree_params( ND_LIST *root, ND_LIST *leaf_header,
  PARAM *param, FH *my_fh )
{

  STR_LIST *get_label_list_from_first_line( STR_LIST *label_header, FILE *in );
  TT get_params_lhood_single_gene( ND_LIST *root, ND_LIST *leaf_header,
    PARAM *param, STR_LIST *label_header, char line[], SEQ seq_numb );

  set_position( 0, my_fh->in );

  STR_LIST *label_header = NULL;
  /*  In set.c:  */
  label_header = (STR_LIST *)get_label_list_from_first_line( label_header, 
    my_fh->in );

  SEQ seq_numb = 0;
  TT lh = 0.0;
  
  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, my_fh->in ) != NULL ) {
  
    /*  In this file, below:  */
    lh += get_params_lhood_step_single_gene( root, leaf_header, param, label_header, 
      line, ++seq_numb );
      
    if ( seq_numb >= MAX_SEQS )
      break;
      
  }
  
  kill_str_list( label_header );
  
  return lh;

}











TT get_params_lhood_step_single_gene( ND_LIST *root, ND_LIST *leaf_header,
  PARAM *param, STR_LIST *label_header, char line[], SEQ seq_numb )
{

  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );
  TT get_params_lhood_single_set_tree( ND_LIST *root );
  
  ND_LIST *ancestor = NULL;
  /*  In tree.c:  */
  ancestor = (ND_LIST *)get_nd_lister_from_label( root, "Mammalia" );

  /*  In set.c:  */
  initialize_nd_tree( root );
  
  /*  In set.c:  */
  set_my_evolutionary_tree_single_gene( root, leaf_header, label_header,
    line, MY_NUCL );
    
  /*  In v.c:  */
  do_evol_fill_phylo( root, param );
  
  /*  In lhood.c:  */
  return get_params_lhood_single_set_tree( ancestor );

}









LHOOD_LIST *put_lhood_lister_step_into_list( LHOOD_LIST *header, LHOOD_LIST *lhood )
{

  if ( header == NULL ) {
    print_single_lhood_lister_step_fh( lhood, 0, stdout );
    return lhood;
  }
    
  if ( lhood->my_lhood->lhood > header->my_lhood->lhood ) {
    print_single_lhood_lister_step_fh( lhood, 0, stdout );
    lhood->next = header;
    return lhood;
  }
  
  printf( "lhood :  %.3f\n", (float)lhood->my_lhood->lhood );

  int i = 0;
  
  LHOOD_LIST *curr = header, *prev = NULL;
  
  while (( curr != NULL ) && ( curr->my_lhood->lhood > lhood->my_lhood->lhood )) {

    prev = curr;
    curr = curr->next;    
    
    if ( ++i >= MAX_LIST_LENGTH ) {
      kill_my_lhood_lister( lhood );
      return header;
    }

  }
  
  if ( curr != NULL )
    lhood->next = curr;
  
  if ( prev != NULL )
    prev->next = lhood;
  
  i = 0;
  
  curr = header; 
  prev = NULL;
  
  while ( curr->next != NULL ) {
  
    prev = curr;
    curr = curr->next;
    
    if ( ++i >= MAX_LIST_LENGTH ) {
      kill_my_lhood_lister( curr );
      prev->next = NULL;
      return header;
    }
  
  }
  
  return header;

}














/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/














void print_lhood_list_step_fh( LHOOD_LIST *lhood_header, FH *my_fh )
{
  print_my_lhood_list_step_fh( lhood_header, my_fh->out );
}











void print_my_lhood_list_step_fh( LHOOD_LIST *lhood_header, FILE *fh )
{

  print_lhood_list_step_header_fh( fh );

  int numb = 0;

  LHOOD_LIST *lhood = lhood_header;
  
  while ( lhood != NULL ) 
    print_single_lhood_lister_step_fh_and_step( &lhood, ++numb, fh );

}











void print_lhood_list_step_header_fh( FILE *fh )
{

  print_sep( fh );
  
  fprintf( fh, "Number of seq (MAX) :  %d\n", MAX_SEQS );
  fprintf( fh, "Initial values :\n" );
  fprintf( fh, "  Euth[a] :            %.2e\n", EUTH_ALPHA );
  fprintf( fh, "  Euth[b] :            %.2e\n", EUTH_BETA );
  fprintf( fh, "  Conv[a] :            %.2e\n", CONVERT_ALPHA );
  fprintf( fh, "  Conv[b] :            %.2e\n", CONVERT_BETA );
  fprintf( fh, "  Anc[a]  :            %.2e\n", ANCESTOR_ALPHA );
  fprintf( fh, "  Anc[b]  :            %.2e\n", ANCESTOR_BETA );
  fprintf( fh, "Alpha step size :      %.2e\n", ALPHA_STEP );
  fprintf( fh, "Beta step size  :      %.2e\n", BETA_STEP );
  fprintf( fh, "Min alpha step size :  %.2e\n", ALPHA_STEP_THRESH );
  fprintf( fh, "Min beta step size  :  %.2e\n", ALPHA_STEP_THRESH );
  
  if ( DO_TWO_LINEAGES ) {
    fprintf( fh, "Two eutherian lineages :\n" );
    fprintf( fh, "  %s\n", EUTH_LINEAGE_ONE );
    fprintf( fh, "  %s\n", EUTH_LINEAGE_TWO );
  }

}












void print_single_lhood_lister_step_fh_and_step( LHOOD_LIST **lhood, int numb, 
  FILE *fh )
{
  print_single_lhood_lister_step_fh( *lhood, numb, fh );
  *lhood = (*lhood)->next;
}














void print_single_lhood_lister_step_fh( LHOOD_LIST *lhood, int numb, FILE *fh )
{

  LHOOD *lh = lhood->my_lhood;

  print_sep( fh );
  fprintf( fh, "Numb :  %d\n", numb );
  fprintf( fh, "        Log-lh  = %.3f\n", (float)lh->lhood );
  fprintf( fh, "        Euth[a] = %.2e\n", (float)lh->euth_alpha );
  fprintf( fh, "        Euth[b] = %.2e\n", (float)lh->euth_beta );
  fprintf( fh, "        Conv[a] = %.2e\n", (float)lh->conv_alpha );
  fprintf( fh, "        Conv[b] = %.2e\n", (float)lh->conv_beta );
  fprintf( fh, "        Anc[a]  = %.2e\n", (float)lh->anc_alpha );
  fprintf( fh, "        Anc[b]  = %.2e\n", (float)lh->anc_beta );
  
  fprintf( fh, "\n" );

}












/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/












void do_param_lhood_step_single( void )
{

  ND_LIST *get_evol_tree( ND_LIST *root );
  ND_LIST *get_nd_leaf_header( ND_LIST *node );

  ND_LIST *root = NULL, *leaf_header = NULL;

  /*  In tree.c:  */
  root = (ND_LIST *)get_evol_tree( root );
  leaf_header = (ND_LIST *)get_nd_leaf_header( root );
  
  /*  In this file, below:  */
  do_my_param_lhood_step_single( root, leaf_header );
  
}














void do_my_param_lhood_step_single( ND_LIST *root, ND_LIST *leaf_header )
{

  FH *my_fh = NULL;
  my_fh = (FH *)malloc( sizeof(FH) );
  
  /*  In this file, above:  */
  set_params_lhood_fh_step( my_fh );
  set_prior_probs_for_lhood_step( root, leaf_header, MY_NUCL, my_fh->in );
  
  /*  In this file, below:  */
  do_params_lhood_step_w_tree_single( root, leaf_header, my_fh );
  
  /*  In this file, above:  */
  close_params_lhood_fh_step( my_fh );

}














void do_params_lhood_step_w_tree_single( ND_LIST *root, ND_LIST *leaf_header,
  FH *my_fh )
{

  ;

}











/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/










LHOOD_LIST *create_my_lhood_lister( LHOOD_LIST *xx )
{

  LHOOD *x = NULL;
  x = (LHOOD *)malloc( sizeof(LHOOD) );
  xx = (LHOOD_LIST *)malloc( sizeof(LHOOD_LIST) );
  
  return initialize_my_lhood_lister( xx, x );

}











LHOOD_LIST *initialize_my_lhood_lister( LHOOD_LIST *xx, LHOOD *x )
{

  x->euth_alpha = 0.0;
  x->euth_beta = 0.0;
  x->conv_alpha = 0.0;
  x->conv_beta = 0.0;
  x->anc_alpha = 0.0;
  x->anc_beta = 0.0;
  
  xx->my_lhood = x;
  xx->next = NULL;
  
  return xx;

}












void kill_my_lhood_lister( LHOOD_LIST *x )
{

  if ( x == NULL )
    return;
    
  free( x->my_lhood );
  free( x );

}




















