#include "newton.h"















#define MAX_ITERATIONS 15

#define PARAM_BUFFER 2
#define NUMB_OF_SITES 100

#define MILLION 1000000

#define EUTH_INDEX 0
#define CONVERT_INDEX 1
#define ANCESTOR_INDEX 2
#define PARAM_TYPE_UNSET -1


















void get_params_newton( PARAM *param, ND_LIST *root, INDEX iter, FILE *out )
{

  PARAM *make_and_copy_params( PARAM *new, PARAM *old );

  PARAM *new_param = NULL;
  /*  In mammal.c:  */
  new_param = (PARAM *)make_and_copy_params( new_param, param );
  
  /*  In this file, below:  */
  get_my_params_newton( new_param, param, root, iter, out );
  
  free( new_param );

}














void get_my_params_newton( PARAM *new_param, PARAM *param, ND_LIST *root,
  INDEX iter, FILE *out )
{

  MY_MX *my_mx = NULL;
  my_mx = (MY_MX *)make_and_initialize_my_mx( my_mx, param );
  
  get_params_newton_w_my_mx( my_mx, new_param, param, root, iter, out );
  
}










MY_MX *make_and_initialize_my_mx( MY_MX *my_mx, PARAM *param )
{

  TT_MX *create_tt_mx( TT_MX *xx, INDEX ii, INDEX jj );
  
  TT_MX *x = NULL, *x_old = NULL, *y = NULL, *f = NULL, *j = NULL, *j_inv = NULL;


  my_mx = (MY_MX *)malloc( sizeof(MY_MX) );
  
  x = (TT_MX *)create_tt_mx( x, PARAM_BUFFER, 1 );
  x_old = (TT_MX *)create_tt_mx( x_old, PARAM_BUFFER, 1 );
  y = (TT_MX *)create_tt_mx( y, PARAM_BUFFER, 1 );
  f = (TT_MX *)create_tt_mx( f, PARAM_BUFFER, 1 );
  j = (TT_MX *)create_tt_mx( j, PARAM_BUFFER, PARAM_BUFFER );
  j_inv = (TT_MX *)create_tt_mx( j_inv, PARAM_BUFFER, PARAM_BUFFER );
  
  my_mx->x = x;
  my_mx->x_old = x_old;
  my_mx->y = y;
  my_mx->f = f;
  my_mx->j = j;
  my_mx->j_inv = j_inv;
  
  return my_mx;

}











void get_params_newton_w_my_mx( MY_MX *my_mx, PARAM *new_param, PARAM *param, 
  ND_LIST *root, INDEX iter, FILE *out )
{

  PARAM *pp = param, *new_pp = new_param;
  
  char i;
  
  for ( i = 0; i < param->numb; i++ )
    get_params_newton_single_param_set( my_mx, new_pp++, pp++, root, iter, i,
      out );
    
}











void get_params_newton_single_param_set( MY_MX *my_mx, PARAM *new_param,
  PARAM *param, ND_LIST *root, INDEX iter, char numb, FILE *out )
{

  print_newton_initial_params( new_param, iter, numb, out );

  int diverged = 0;
  INDEX newt_iter = 0;
  
  while (( ! diverged ) && ( newt_iter++ < MAX_ITERATIONS ))
    diverged = get_params_newton_single_iter( my_mx, new_param, param, root,
      iter, newt_iter, numb, out );

}












int get_params_newton_single_iter( MY_MX *my_mx, PARAM *new_param, PARAM *param,
  ND_LIST *root, INDEX iter, INDEX newt_iter, char numb, FILE *out )
{

  get_my_params_newton_single_iter( my_mx, new_param, root, numb );
  
  if ( params_nan( my_mx ))
    return 1;
  
  update_params( my_mx, root, new_param, param, numb );
  
  print_updated_params( my_mx, new_param, newt_iter, out );
  
  return 0;

}












int params_nan( MY_MX *my_mx )
{

  TT a = *(my_mx->y->x);
  TT b = *(my_mx->y->x + 1);

  if (( isnan( a )) || ( isinf( a )))
    return 1;
    
  if (( isnan( b )) || ( isinf( b )))
    return 1;
    
  return 0;

}











void get_my_params_newton_single_iter( MY_MX *my_mx, PARAM *param, ND_LIST *root,
  char numb )
{

  static int i = 0;

  set_x_vector( my_mx, param );
  
  set_newton_matrices( my_mx, root, numb );
  
  do_newton_single_param_set( my_mx );
  
  my_mx->ss = get_sum_of_squares_lhood( my_mx->x, root, numb );
  
  printf( "%3d    %9.2e  %9.2e  (%9.2e    %9.2e)   ss = %f\n", 
    ++i, 
    (float)*(my_mx->x->x),
    (float)*(my_mx->x->x + 1),
    (float)*(my_mx->y->x), 
    (float)*(my_mx->y->x + 1), 
    (float)my_mx->ss );
    
}













void set_x_vector( MY_MX *my_mx, PARAM *param )
{
  *(my_mx->x->x) = param->alpha;
  *(my_mx->x->x + 1) = param->beta;
}














void set_newton_matrices( MY_MX *my_mx, ND_LIST *root, char numb )
{
  set_newton_f_matrix( my_mx, root, numb );
  set_newton_j_matrix( my_mx, root, numb );
//  print_tt_mx( my_mx->f );
//  print_tt_mx( my_mx->j );
}














void do_newton_single_param_set( MY_MX *my_mx )
{

  /*  In matrix.c:  */
  set_matrix_inv( my_mx->j_inv, my_mx->j );
  
  *(my_mx->f->x) *= -1.0;
  *(my_mx->f->x + 1) *= -1.0;
  
  /*  In matrix.c:  */
  mult_matrices( my_mx->y, my_mx->j_inv, my_mx->f );
  
}














void update_params( MY_MX *my_mx, ND_LIST *root, PARAM *new_param, 
  PARAM *param, char numb )
{

  new_param->alpha += *(my_mx->y->x + 0);
  new_param->beta += *(my_mx->y->x + 1);
  
  param->alpha = new_param->alpha;
  param->beta = new_param->beta;
  
  update_node_params_trans_probs( root, param, numb );

}












void update_node_params_trans_probs( ND_LIST *node, PARAM *param, char numb )
{

  if ( node == NULL )
    return;
    
  update_single_node_params_trans_probs( node, param, numb );  
    
  char i = 0;
  
  while ( node->down[ i ] != NULL )
    update_node_params_trans_probs( node->down[ i++ ], param, numb );

}













void update_single_node_params_trans_probs( ND_LIST *node, PARAM *param,
  char numb )
{

  if ( node->my_node->param_type != numb )
    return;
    
  update_my_single_node_params_trans_probs( node, param );

}











void update_my_single_node_params_trans_probs( ND_LIST *node, PARAM *param )
{

  TT get_p( PARAM *param, TT t, INDEX i, INDEX j );

  TT_MX *p = node->my_node->data->p;

  register char i, j;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {
  
    for ( j = 0; j < NUMB_OF_MFS; j++ ) 
      /*  In v.c:  */
      *(p->x + (i * p->j) + j) = get_p( param, node->my_node->t, i, j );
  
  }

}










TT get_sum_of_squares_lhood( TT_MX *x_param, ND_LIST *node, char numb )
{

  if ( node == NULL )
    return 0.0;
    
  TT x = get_sum_of_squares_lhood_single_node( x_param, node, numb );
  
  char i = 0;
  
  while ( node->down[ i ] != NULL )
    x += get_sum_of_squares_lhood( x_param, node->down[ i++ ], numb );
    
  return x;

}












TT get_sum_of_squares_lhood_single_node( TT_MX *x_param, ND_LIST *node, char numb )
{

  if ( node->my_node->param_type != numb )
    return 0.0;
    
  return get_my_sum_of_squares_lhood_single_node( x_param, node );

}











TT get_my_sum_of_squares_lhood_single_node( TT_MX *x_param, ND_LIST *node )
{

  TT sum = 0.0;
  
  register char i, j;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {
  
    for ( j = 0; j < NUMB_OF_MFS; j++ ) {
    
      sum += get_sum_of_squares_single_ij_pair( x_param, node, i, j );
    
    }
  
  }
  
  return sum;
  
}











TT get_sum_of_squares_single_ij_pair( TT_MX *x_param, ND_LIST *node,
  char i, char j )
{

  TT_MX *p_mx = node->my_node->data->p;
  TT_MX *n_mx = node->my_node->data->n_sum;

  TT n = get_total_n_val( node, i, j );
  
  if ( n == 0.0 )
    return 0.0;
  
  TT w = get_row_sum_tt_mx( n_mx, i ) / get_tt_mx_total( n_mx );
  
  TT p = *(p_mx->x + (i * p_mx->j) + j);
  
  return w * (n - p) * (n - p);

}









/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/












void set_newton_f_matrix( MY_MX *my_mx, ND_LIST *root, char numb )
{
  set_my_newton_f_matrix( my_mx->f, root, my_mx->x, numb );
}













void set_my_newton_f_matrix( TT_MX *f, ND_LIST *root, TT_MX *x, char numb )
{
  *(f->x + 0) = get_f_matrix_alpha_deriv( root, x, numb );
  *(f->x + 1) = get_f_matrix_beta_deriv( root, x, numb );
}











TT get_f_matrix_alpha_deriv( ND_LIST *node, TT_MX *x_param, char numb )
{

  if ( node == NULL )
    return 0.0;

  TT x = get_f_matrix_alpha_deriv_single_node( node, x_param, numb );
  
  char i = 0;
  
  while ( node->down[ i ] != NULL )
    x += get_f_matrix_alpha_deriv( node->down[ i++ ], x_param, numb );
    
  return x;

}












TT get_f_matrix_alpha_deriv_single_node( ND_LIST *node, TT_MX *x_param, char numb )
{

  if ( node->my_node->param_type != numb )
    return 0.0;
    
  return get_my_f_matrix_alpha_deriv_single_node( node, x_param );

}











TT get_my_f_matrix_alpha_deriv_single_node( ND_LIST *node, TT_MX *x_param )
{

  TT x = 0.0;

  register char i, j;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {
  
    for ( j = 0; j < NUMB_OF_MFS; j++ ) {
    
      x += get_f_matrix_alpha_deriv_single_ij_pair( node, x_param, i, j );
      
    }
  
  }
  
  return x;

}












TT get_f_matrix_alpha_deriv_single_ij_pair( ND_LIST *node, TT_MX *x_param,
  char i, char j )
{

  TT_MX *p_mx = node->my_node->data->p;
  TT_MX *n_mx = node->my_node->data->n_sum;

  TT n = get_total_n_val( node, i, j );
  
  if ( n == 0.0 )
    return 0.0;
  
  TT w = get_row_sum_tt_mx( n_mx, i ) / get_tt_mx_total( n_mx );
  
  TT p = *(p_mx->x + (i * p_mx->j) + j);
  
  TT deriv = get_p_deriv_alpha( node, x_param, i, j );
  
  return -2.0 * w * deriv * (n - p);

}










TT get_f_matrix_beta_deriv( ND_LIST *node, TT_MX *x_param, char numb )
{

  if ( node == NULL )
    return 0.0;

  TT x = get_f_matrix_beta_deriv_single_node( node, x_param, numb );
  
  char i = 0;
  
  while ( node->down[ i ] != NULL )
    x += get_f_matrix_beta_deriv( node->down[ i++ ], x_param, numb );

  return x;

}












TT get_f_matrix_beta_deriv_single_node( ND_LIST *node, TT_MX *x_param, char numb )
{

  if ( node->my_node->param_type != numb )
    return 0.0;
    
  return get_my_f_matrix_beta_deriv_single_node( node, x_param );

}











TT get_my_f_matrix_beta_deriv_single_node( ND_LIST *node, TT_MX *x_param )
{

  TT x = 0.0;

  register char i, j;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {
  
    for ( j = 0; j < NUMB_OF_MFS; j++ ) {
    
      x += get_f_matrix_beta_deriv_single_ij_pair( node, x_param, i, j );
    
    }
  
  }
  
  return x;

}














TT get_f_matrix_beta_deriv_single_ij_pair( ND_LIST *node, TT_MX *x_param,
  char i, char j )
{

  TT_MX *p_mx = node->my_node->data->p;
  TT_MX *n_mx = node->my_node->data->n_sum;

  TT n = get_total_n_val( node, i, j );
  
  if ( n == 0.0 )
    return 0.0;
  
  TT w = get_row_sum_tt_mx( n_mx, i ) / get_tt_mx_total( n_mx );
  
  TT p = *(p_mx->x + (i * p_mx->j) + j);
  
  TT deriv = get_p_deriv_beta( node, x_param, i, j );
  
  return -2.0 * w * deriv * (n - p);

}
















/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/














void set_newton_j_matrix( MY_MX *my_mx, ND_LIST *root, char numb )
{
  set_my_newton_j_matrix( my_mx->j, root, my_mx->x, numb );
}











void set_my_newton_j_matrix( TT_MX *j, ND_LIST *root, TT_MX *x, char numb )
{

  *(j->x + 0) = get_newton_j_matrix_aa( root, x, numb );
  *(j->x + 1) = get_newton_j_matrix_ab( root, x, numb );
  *(j->x + 2) = *(j->x + 1);
  *(j->x + 3) = get_newton_j_matrix_bb( root, x, numb );

}











TT get_newton_j_matrix_aa( ND_LIST *node, TT_MX *x_param, char numb )
{

  if ( node == NULL )
    return 0.0;

  TT x = get_j_matrix_single_node_aa( node, x_param, numb );
  
  char i = 0;
  
  while ( node->down[ i ] != NULL )
    x += get_newton_j_matrix_aa( node->down[ i++ ], x_param, numb );

  return x;

}












TT get_j_matrix_single_node_aa( ND_LIST *node, TT_MX *x_param, char numb )
{

  if ( node->my_node->param_type != numb )
    return 0.0;
    
  return get_my_j_matrix_single_node_aa( node, x_param );

}











TT get_my_j_matrix_single_node_aa( ND_LIST *node, TT_MX *x_param )
{

  TT x = 0.0;

  register char i, j;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {
  
    for ( j = 0; j < NUMB_OF_MFS; j++ ) {
    
      x += get_j_matrix_single_ij_pair_aa( node, x_param, i, j );
    
    }
  
  }
  
  return x;

}












TT get_j_matrix_single_ij_pair_aa( ND_LIST *node, TT_MX *x_param, char i, char j )
{

  TT_MX *n_mx = node->my_node->data->n_sum;

  TT n = get_total_n_val( node, i, j );
  
  if ( n == 0.0 )
    return 0.0;
  
  TT w = get_row_sum_tt_mx( n_mx, i ) / get_tt_mx_total( n_mx );
  
  TT deriv = get_p_deriv_alpha( node, x_param, i, j );  
  
  return 2.0 * w * deriv * deriv;

}













TT get_newton_j_matrix_ab( ND_LIST *node, TT_MX *x_param, char numb )
{

  if ( node == NULL )
    return 0.0;

  TT x = get_j_matrix_single_node_ab( node, x_param, numb );
  
  char i = 0;
  
  while ( node->down[ i ] != NULL )
    x += get_newton_j_matrix_ab( node->down[ i++ ], x_param, numb );

  return x;

}












TT get_j_matrix_single_node_ab( ND_LIST *node, TT_MX *x_param, char numb )
{

  if ( node->my_node->param_type != numb )
    return 0.0;
    
  return get_my_j_matrix_single_node_ab( node, x_param );

}











TT get_my_j_matrix_single_node_ab( ND_LIST *node, TT_MX *x_param )
{

  TT x = 0.0;

  register char i, j;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {
  
    for ( j = 0; j < NUMB_OF_MFS; j++ ) {
    
      x += get_j_matrix_single_ij_pair_ab( node, x_param, i, j );
    
    }
  
  }
  
  return x;

}










TT get_j_matrix_single_ij_pair_ab( ND_LIST *node, TT_MX *x_param, char i, char j )
{

  TT_MX *n_mx = node->my_node->data->n_sum;

  TT n = get_total_n_val( node, i, j );
  
  if ( n == 0.0 )
    return 0.0;
  
  TT w = get_row_sum_tt_mx( n_mx, i ) / get_tt_mx_total( n_mx );
  
  TT a_deriv = get_p_deriv_alpha( node, x_param, i, j );  
  TT b_deriv = get_p_deriv_beta( node, x_param, i, j );
  
  return 2.0 * w * a_deriv * b_deriv;

}









TT get_newton_j_matrix_bb( ND_LIST *node, TT_MX *x_param, char numb )
{

  if ( node == NULL )
    return 0.0;

  TT x = get_j_matrix_single_node_bb( node, x_param, numb );
  
  char i = 0;
  
  while ( node->down[ i ] != NULL )
    x += get_newton_j_matrix_bb( node->down[ i++ ], x_param, numb );

  return x;

}












TT get_j_matrix_single_node_bb( ND_LIST *node, TT_MX *x_param, char numb )
{

  if ( node->my_node->param_type != numb )
    return 0.0;
    
  return get_my_j_matrix_single_node_bb( node, x_param );

}











TT get_my_j_matrix_single_node_bb( ND_LIST *node, TT_MX *x_param )
{

  TT x = 0.0;

  register char i, j;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {
  
    for ( j = 0; j < NUMB_OF_MFS; j++ ) {
    
      x += get_j_matrix_single_ij_pair_bb( node, x_param, i, j );
    
    }
  
  }
  
  return x;

}












TT get_j_matrix_single_ij_pair_bb( ND_LIST *node, TT_MX *x_param, char i, char j )
{

  TT_MX *n_mx = node->my_node->data->n_sum;

  TT n = get_total_n_val( node, i, j );
  
  if ( n == 0.0 )
    return 0.0;
  
  TT w = get_row_sum_tt_mx( n_mx, i ) / get_tt_mx_total( n_mx );
  
  TT deriv = get_p_deriv_beta( node, x_param, i, j );  
  
  return 2.0 * w * deriv * deriv;

}












/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/










TT get_total_n_val( ND_LIST *node, char i, char j )
{

  TT_MX *n = node->my_node->data->n_sum;
  
  TT n_val = *(n->x + (i * n->j) + j);
  
  if ( n_val == 0.0 )
    return 0.0;
    
  return n_val / get_row_sum_tt_mx( n, i );

}











TT get_row_sum_tt_mx( TT_MX *x, char i )
{

  TT sum = 0.0;
  
  register char k;
  
  for ( k = 0; k < x->i; k++ )
    sum += *(x->x + (i * x->j) + k);
    
  return sum;

}









TT get_tt_mx_total( TT_MX *x )
{

  TT sum = 0.0;
  
  register char i, j;
  
  for ( i = 0; i < x->i; i++ ) {
    for ( j = 0; j < x->j; j++ ) {
      sum += *(x->x + (i * x->j) + j);
    }
  }
  
  return sum;

}










TT get_p_deriv_alpha( ND_LIST *node, TT_MX *x_param, char i, char j )
{

  PARAM param;
  
  copy_param_vector_to_param( &param, x_param );

  TT sum = 0.0;
  
  register char k;
  
  for ( k = 0; k <= j; k++ )
    sum += get_p_deriv_alpha_single_term( node, &param, i, j, k );
  
  return sum;

}











TT get_p_deriv_beta( ND_LIST *node, TT_MX *x_param, char i, char j )
{

  PARAM param;
  
  copy_param_vector_to_param( &param, x_param );
  
  TT sum = 0.0;
  
  register char k;
  
  for ( k = 0; k <= j; k++ )
    sum += get_p_deriv_beta_single_term( node, &param, i, j, k );
    
  return sum;

}










void copy_param_vector_to_param( PARAM *param, TT_MX *mx )
{
  param->alpha = *(mx->x + 0);
  param->beta = *(mx->x + 1);
}









TT get_p_deriv_alpha_single_term( ND_LIST *node, PARAM *param, char i, char j,
  char k )
{

  TT get_transition_u( PARAM *param, TT time );
  TT get_transition_v( PARAM *param, TT time );
  TT get_binom_pr( TT (*pr)( PARAM *pp, TT tt ), INDEX n, INDEX k, PARAM *param, 
    TT t );

  TT (*pr)( PARAM *pp, TT tt );
  TT (*dd)( PARAM *pp, TT tt );
  
  /*  In v.c:  */
  pr = get_transition_u;

  /*  In this file, below:  */
  dd = get_u_deriv_alpha;
  
  /*  In this file, below:  */
  TT u_deriv = get_binom_deriv( pr, dd, i, k, param, node->my_node->t );
  
  /*  In v.c:  */
  TT u = get_binom_pr( pr, i, k, param, node->my_node->t );
  
  /*  In v.c:  */
  pr = get_transition_v;

  /*  In this file, below:  */
  dd = get_v_deriv_alpha;
  
  /*  In this file, below:  */
  TT v_deriv = get_binom_deriv( pr, dd, NUMB_OF_SITES - i, j - k, param,
    node->my_node->t );
    
  /*  In v.c:  */
  TT v = get_binom_pr( pr, NUMB_OF_SITES - i, j - k, param, node->my_node->t );
  
  return (u_deriv * v) + (u * v_deriv);

}












TT get_p_deriv_beta_single_term( ND_LIST *node, PARAM *param, char i, char j, 
  char k )
{

  TT get_transition_u( PARAM *param, TT time );
  TT get_transition_v( PARAM *param, TT time );
  TT get_binom_pr( TT (*pr)( PARAM *pp, TT tt ), INDEX n, INDEX k, PARAM *param, 
    TT t );

  TT (*pr)( PARAM *pp, TT tt );
  TT (*dd)( PARAM *pp, TT tt );
  
  /*  In v.c:  */
  pr = get_transition_u;

  /*  In this file, below:  */
  dd = get_u_deriv_beta;
  
  /*  In this file, below:  */
  TT u_deriv = get_binom_deriv( pr, dd, i, k, param, node->my_node->t );

  /*  In v.c:  */
  TT u = get_binom_pr( pr, i, k, param, node->my_node->t );
  
  /*  In v.c:  */
  pr = get_transition_v;

  /*  In this file, below:  */
  dd = get_v_deriv_beta;
  
  /*  In this file, below:  */
  TT v_deriv = get_binom_deriv( pr, dd, NUMB_OF_SITES - i, j - k, param,
    node->my_node->t );
    
  /*  In v.c:  */
  TT v = get_binom_pr( pr, NUMB_OF_SITES - i, j - k, param, node->my_node->t );
  
  return (u_deriv * v) + (u * v_deriv);

}









TT get_binom_deriv( TT (*pr)( PARAM *pp, TT tt), TT (*dd)( PARAM *pp, TT tt ),
  INDEX n, INDEX k, PARAM *param, TT t )
{

  TT get_transition_u( PARAM *param, TT time );
  TT get_transition_v( PARAM *param, TT time );
  TT n_choose_k( INDEX n, INDEX k );
  
  /*  References in v.c:  */
  TT p = pr( param, t );

  TT q = 1.0 - p;
  
  /*  In v.c:  */
  TT ch = n_choose_k( n, k );
  
  /*  In this file, below:  */
  TT d = dd( param, t );

  return ch * (k - (n * p)) * d * pow( p, k - 1.0 ) * pow( q, n - k - 1.0 );
  
}













TT get_u_deriv_alpha( PARAM *param, TT t )
{

  TT a = param->alpha;
  TT b = param->beta;

  TT sum = a + b;
  TT t_sum = sum * t * MILLION;
  
  TT expt = exp( -1.0 * t_sum );
  
  TT coeff = b / (sum * sum);
  
  return coeff * ( 1.0 - expt - (t_sum * expt) );
  
}











TT get_v_deriv_alpha( PARAM *param, TT t )
{

  TT a = param->alpha;
  TT b = param->beta;
  
  TT sum = a + b;
  TT t_sum = sum * t * MILLION;
  
  TT expt = exp( -1.0 * t_sum );
  
  TT coeff = 1.0 / (sum * sum);
  
  return coeff * ( b - (b * expt) + (a * t_sum * expt) );

}












TT get_u_deriv_beta( PARAM *param, TT t )
{

  TT a = param->alpha;
  TT b = param->beta;
  
  TT sum = a + b;
  TT t_sum = sum * t * MILLION;
  
  TT expt = exp( -1.0 * t_sum );
  
  TT coeff = -1.0 / (sum * sum);
  
  return coeff * ( a - (a * expt) + (b * t_sum * expt) );
  
}














TT get_v_deriv_beta( PARAM *param, TT t )
{

  TT a = param->alpha;
  TT b = param->beta;
  
  TT sum = a + b;
  TT t_sum = sum * t * MILLION;
  
  TT expt = exp( -1.0 * t_sum );
  
  TT coeff = -1.0 * a / (sum * sum);
  
  return coeff * ( 1.0 - expt - (t_sum * expt) );

}













/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/













void print_newton_initial_params( PARAM *param, INDEX iter, char numb, FILE *out )
{

  print_sep( out );
  
  fprintf( out, "EM iteration :  %d\n", (int)iter );
  
  switch ( numb ) {

  case EUTH_INDEX :
    fprintf( out, "Eutherian lineages\n" );
    break;
    
  case CONVERT_INDEX :
    fprintf( out, "Converted branch\n" );
    break;
    
  case ANCESTOR_INDEX :
    fprintf( out, "Ancestral lineages\n" );
    break;
    
  default :
    error( 9701, "Unknown rate type.\n" );
    break;

  }
  
  fprintf( out, "Initial parameter values :\n" );
  fprintf( out, "alpha =  %.3e\n", (float)param->alpha );
  fprintf( out, "beta  =  %.3e\n", (float)param->beta );
  
  print_dot_sep( out );

}













void print_updated_params( MY_MX *my_mx, PARAM *param, INDEX newt_iter, FILE *out )
{

  fprintf( out, "\t%3d iteration\n", (int)newt_iter );
  
  fprintf( out, "\t\talpha :  %9.3e   step = %.2e\n", 
    (float)param->alpha, (float)*(my_mx->y->x) );
  
  fprintf( out, "\t\tbeta  :  %9.3e   step = %.2e\n", 
    (float)param->beta, (float)*(my_mx->y->x + 1) );
  
  fprintf( out, "\t\tSS    :\t\t            %f\n", (float)my_mx->ss );

}













