#include "pr.h"
















#define OUT_DIR "data/ft_hg19/evol/gene"



#define ARRAY_BUFFER 5















void print_evol_node_info( ND_LIST *root, PARAM *param, char line[] )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  char fname[ BUFFER ];
  
  get_print_evol_node_info_fname( fname, line );
  
  FILE *fh = NULL;
  
  fh = (FILE *)open_text_file_writing( fname, "" );
  
  print_evol_node_info_fh( root, param, fh );
  
  close_file( fh );
  
}













void print_evol_node_info_fh( ND_LIST *root, PARAM *param, FILE *fh )
{

  /*  In tree.c:  */
  print_nd_tree( root, fh );
  
  /*  In this file, below:  */
  print_my_evol_node_info_fh( root, param, fh );

}














void print_my_evol_node_info_fh( ND_LIST *root, PARAM *param, FILE *fh )
{

  print_evol_node_info_leaves( root, fh );
  
  print_evol_node_info_each_node( root, fh );

}














void print_evol_node_info_leaves( ND_LIST *root, FILE *fh )
{

  print_evol_node_info_single_leaf( root, fh );

}












void print_evol_node_info_single_leaf( ND_LIST *leaf, FILE *fh )
{

  if ( leaf->down[ 0 ] == NULL ) {
    print_evol_node_info_my_single_leaf( leaf, fh );
    return;
  }
  
  register char i = 0;
  
  while ( leaf->down[ i ] != NULL )
    print_evol_node_info_single_leaf( leaf->down[ i++ ], fh );

}












void print_evol_node_info_my_single_leaf( ND_LIST *leaf, FILE *fh )
{

  print_sep( fh );

  fprintf( fh, ">  %s\n", leaf->my_node->label );
  fprintf( fh, "   %.1f million years (t)\n", (float)leaf->my_node->t );
  
  print_dot_sep( fh );
  
  print_evol_node_leaf_occur_count( leaf, fh );
  
  print_dot_sep( fh );
  
  print_evol_node_info_single_node( leaf, fh );

}













void print_evol_node_leaf_occur_count( ND_LIST *leaf, FILE *fh )
{

  if ( ! leaf->my_node->exists ) {
    fprintf( fh, "   (no gene ortholog...)\n" );
    return;
  }

  fprintf( fh, "     Numb of occur :  %d\n", 
    get_numb_of_occur_nd_lister( leaf ) );

}













int get_numb_of_occur_nd_lister( ND_LIST *node )
{

  register char i;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {
    if ( *(node->my_node->data->pr->x + i) == 1.0 )
      return i;
  }
  
  error( 9500, "Unknown value for number of occurrences.\n" );

}
















void print_evol_node_info_each_node( ND_LIST *node, FILE *fh )
{

  if ( node == NULL )
    return;

  print_evol_node_info_single_each_node( node, fh );
  
  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    print_evol_node_info_each_node( node->down[ i++ ], fh );

}











void print_evol_node_info_single_each_node( ND_LIST *node, FILE *fh )
{

  print_sep( fh );
  
  fprintf( fh, ">> %s\n", node->my_node->label );
  fprintf( fh, "   %.1f million years (t)\n", (float)node->my_node->t );
  fprintf( fh, "   %.1f total mya\n", (float)node->my_node->total_time );
  
  if ( node->my_node->exists )
    fprintf( fh, "   Ortholog exists.\n" );
  else
    fprintf( fh, "   (no ortholog)\n" );
    
  print_dot_sep( fh );
  
  print_evol_node_info_node_tree( node, fh );
    
  print_evol_node_info_single_node( node, fh );
  
  print_sep( fh );

}













void print_evol_node_info_node_tree( ND_LIST *node, FILE *fh )
{

  if ( node->down[ 0 ] == NULL )
    return;
    
  fprintf( fh, "%s :  %.1f\n", node->my_node->label, 
    (float)node->my_node->total_time );
    
  if ( node->up != NULL )  
    fprintf( fh, "%s   (parent node)\n", node->up->my_node->label );
  else
    fprintf( fh, "Root node.\n" );
  
  print_dot_sep( fh );
  
  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    print_evol_single_daughter_node( node->down[ i++ ], fh );
    
  print_dot_sep( fh );

}














void print_evol_single_daughter_node( ND_LIST *node, FILE *fh )
{

  fprintf( fh, "%s  (%.1f)\n", node->my_node->label, (float)node->my_node->t );
  
  print_evol_daughter_node_leaves( node, fh );

}












void print_evol_daughter_node_leaves( ND_LIST *node, FILE *fh )
{

  if ( node->down[ 0 ] == NULL ) {

    if ( ! node->my_node->exists )
      fprintf( fh, "\t\t\t%8s :  (no ortholog...)\n", node->my_node->label );
    else
      fprintf( fh, "\t\t\t%8s :  %d\n", node->my_node->label,
        get_numb_of_occur_nd_lister( node ) );

    return;

  }
  
  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    print_evol_daughter_node_leaves( node->down[ i++ ], fh );

}









/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/














void print_evol_node_info_single_node( ND_LIST *node, FILE *fh )
{

  print_evol_node_info_p_trans( node, fh );
  
  if ( ! node->my_node->exists )
    return;
  
  print_evol_node_info_lhood( node, fh );

  print_evol_node_info_n_mx( node, fh );
  
  print_evol_node_total_n_vals( node, fh );
  
}













void print_evol_node_info_p_trans( ND_LIST *node, FILE *fh )
{

  TT_MX *p = node->my_node->data->p;
    
  fprintf( fh, "p[i,j](t) =\n         " );

  register int i, j;
  
  for ( i = 0; i <= ARRAY_BUFFER; i++ )
    fprintf( fh, "  %6d   ", i );
    
  fprintf( fh, "\n" );

  for ( i = 0; i <= ARRAY_BUFFER; i++ ) {
  
    fprintf( fh, "    " );
    fprintf( fh, "%d    ", i );
    
    for ( j = 0; j <= ARRAY_BUFFER; j++ ) {
      fprintf( fh, "%6.2e   ", (float)*(p->x + (i * p->j) + j) );      
    }
    
    fprintf( fh, "\n" );
    
  }

}













void print_evol_node_info_lhood( ND_LIST *node, FILE *fh )
{

  if ( ! node->my_node->exists )
    return;

  TT_MX *pr = node->my_node->data->pr;
    
  fprintf( fh, "Likelihood =\n" );

  register char i;
  
  for ( i = 0; i <= ARRAY_BUFFER; i++ ) 
    fprintf( fh, "  %d :   %.2e\n", i, (float)*(pr->x + i) );

}











void print_evol_node_info_n_mx( ND_LIST *node, FILE *fh )
{

  if ( ! node->my_node->exists ) 
    return;

  TT_MX *n = node->my_node->data->n;
    
  fprintf( fh, "N[i,j] =\n         " );
  
  register char i, j;
  
  for ( i = 0; i <= ARRAY_BUFFER; i++ )
    fprintf( fh, "  %6d   ", i );
    
  fprintf( fh, "\n" );

  for ( i = 0; i <= ARRAY_BUFFER; i++ ) {
  
    fprintf( fh, "    " );
    fprintf( fh, "%d    ", i );
    
    for ( j = 0; j <= ARRAY_BUFFER; j++ ) {
    
      fprintf( fh, "%6.2e   ", (float)*(n->x + (i * n->j) + j) );
  
    }
    
    fprintf( fh, "\n" );
  
  }

}












void print_evol_node_total_n_vals( ND_LIST *node, FILE *fh )
{

  TT_MX *x = node->my_node->data->n;

  TT sum = 0.0;

  register char i, j;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {
  
    for ( j = 0; j < NUMB_OF_MFS; j++ ) {
    
      sum += *(x->x + (i * x->j) + j);
    
    }
  
  }
  
  fprintf( fh, "Total N val :  %.3e\n", (float)sum );

}












void print_tt_mx( TT_MX *mx )
{

  print_sep( stdout );
  
  TT *ptr = mx->x;
  
  register char i, j;
  
  for ( i = 0; i < mx->i; i++ ) {
  
    for ( j = 0; j < mx->j; j++ ) {
    
      printf( "%9.2e   ", (float)*ptr++ );
    
    }
    
    printf( "\n" );
  
  }
  
  print_sep( stdout );

}










/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/


















void get_print_evol_node_info_fname( char fname[], char line[] )
{

  char gene[ BUFFER ];
  
  sscanf( line, "%*s%*s%s", gene );
  
  sprintf( fname, "%s/%s.info.txt", OUT_DIR, gene );

}







































