#include "tree.h"













#define MIN_T_RATIO 0.00






#define TREE_INPUT "data/ft_hg19/tree/SpeciesHG19.phylogeny.txt"













void do_evolution_tree( void )
{

  printf( "Creating evolution tree...\n" );
  
  do_my_evolution_tree();
  
  printf( "\n" );

}












void do_my_evolution_tree( void )
{

  ND_LIST *root = NULL;
  root = (ND_LIST *)get_evol_tree( root );
  
  print_nd_tree( root, stdout );
  print_nd_leaves( root, stdout );
  
}










ND_LIST *get_evol_tree( ND_LIST *root )
{

  NODE_LIST *get_node_tree_w_fname( NODE_LIST *node_header, char fname[] );

  NODE_LIST *node_header = NULL;
  
  /*  In ../tree/node.c:  */
  node_header = (NODE_LIST *)get_node_tree_w_fname( node_header, TREE_INPUT );
  
  /*  In this file, below:  */
  root = (ND_LIST *)get_evol_tree_from_node_tree( root, node_header );
  
  kill_node_list( node_header );
  
  return root;  

}













/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/











ND_LIST *get_evol_tree_from_node_tree( ND_LIST *root, NODE_LIST *node_header )
{

  root = (ND_LIST *)get_evol_tree_from_single_node_lister( root, node_header );
  
  revise_evol_tree( root );
  
  link_nd_leaves( root );
  
  return root;

}











ND_LIST *get_evol_tree_from_single_node_lister( ND_LIST *node, 
  NODE_LIST *node_lister )
{

  ND_LIST *create_nd_lister( ND_LIST *nn );
  
  node = (ND_LIST *)create_nd_lister( node );
  
  set_nd_lister_from_node_lister( node, node_lister );
  
  register char i;
  
  for ( i = 0; i < NODE_BUFFER; i++ ) {
  
    ND_LIST *nn = NULL;
  
    if ( node_lister->down[ i ] == NULL )
      return node;
      
    nn = (ND_LIST *)get_evol_tree_from_single_node_lister( nn,
      node_lister->down[ i ] );
      
    node->down[ i ] = nn;
    nn->up = node;

  }
  
  return node;

}











void set_nd_lister_from_node_lister( ND_LIST *node, NODE_LIST *node_lister )
{

  strcpy( node->my_node->label, node_lister->my_node->label );
  
  node->my_node->t = (TT)(node_lister->my_node->length);
  node->my_node->total_time = (TT)(node_lister->my_node->time);
  
  printf( "%s :  %.1f\n", node->my_node->label, (float)node->my_node->t );

}












void revise_evol_tree( ND_LIST *node )
{

  if ( node == NULL )
    return;

  register int i = 0;
  
  while ( node->down[ i ] != NULL ) {
  
    ND_LIST *nn = node->down[ i ];
  
    if ( nn->my_node->t / node->my_node->total_time >= MIN_T_RATIO ) {
      ++i;
      continue;
    }
    
    int numb = get_numb_downstream_nodes( nn );
    
    revise_my_evol_tree_single_downstream_node( nn, node, i, numb );
  
  }
  
  i = 0;
  
  while ( node->down[ i ] != NULL ) 
    revise_evol_tree( node->down[ i++ ] );

}










int get_numb_downstream_nodes( ND_LIST *node )
{

  register int i = 0;
  
  while ( node->down[ i ] != NULL )
    ++i;
    
  return i;

}











void revise_my_evol_tree_single_downstream_node( ND_LIST *nn, ND_LIST *node, 
  int index, int numb )
{

  step_evol_tree_nodes( node, index, numb );
  
  put_short_leaves_into_node( node, nn, index, numb );

}












void step_evol_tree_nodes( ND_LIST *node, int index, int numb )
{

  register char i;
  
  for ( i = NODE_BUFFER - numb - 1; i > index; i-- ) 
    node->down[ i + numb - 1 ] = node->down[ i ];

}












void put_short_leaves_into_node( ND_LIST *node, ND_LIST *nn, int index, int numb )
{

  TT time = nn->my_node->t;

  register char i;
  
  for ( i = 0; i < numb; i++ ) {
    node->down[ index + i ] = nn->down[ i ];
    nn->down[ i ]->up = node;
    nn->my_node->t = time + nn->down[ i ]->my_node->t;
  }
  
  kill_nd_lister( nn );

}














/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/














void link_nd_leaves( ND_LIST *root )
{

  STR_LIST *leaf_header = NULL;
  leaf_header = (STR_LIST *)get_str_list_leaf_list( leaf_header, root );
  
  link_nd_leaves_w_label_list( root, leaf_header );
  
}











STR_LIST *get_str_list_leaf_list( STR_LIST *leaf_header, ND_LIST *node )
{

  if ( node->down[ 0 ] != NULL ) {
    
    register char i = 0;
    
    while ( node->down[ i ] != NULL ) 
      leaf_header = (STR_LIST *)get_str_list_leaf_list( leaf_header, 
        node->down[ i++ ] );
        
    return leaf_header;
    
  }
  
  return get_str_list_leaf_list_single_leaf( leaf_header, node );
   
}












STR_LIST *get_str_list_leaf_list_single_leaf( STR_LIST *leaf_header,
  ND_LIST *node )
{

  STR_LIST *create_str_lister( STR_LIST *xx );
  
  STR_LIST *new = NULL;
  new = (STR_LIST *)create_str_lister( new );
  
  strcpy( new->str, node->my_node->label );
  
  return put_str_lister_at_end( leaf_header, new );

}












STR_LIST *put_str_lister_at_end( STR_LIST *header, STR_LIST *new )
{

  if ( header == NULL )
    return new;
    
  STR_LIST *curr = header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = new;
  
  return header;

}













void link_nd_leaves_w_label_list( ND_LIST *root, STR_LIST *leaf_header )
{

  STR_LIST *leaf = leaf_header;
  
  while ( leaf->next != NULL ) 
    link_nd_leaves_single_leaf_lister_and_step( root, &leaf );

}














void link_nd_leaves_single_leaf_lister_and_step( ND_LIST *root, STR_LIST **leaf )
{
  link_nd_leaves_single_leaf_lister( root, *leaf );
  *leaf = (*leaf)->next;
}












void link_nd_leaves_single_leaf_lister( ND_LIST *root, STR_LIST *leaf )
{

  ND_LIST *left = NULL, *right = NULL;
  
  left = (ND_LIST *)get_nd_lister_from_label( root, leaf->str );
  right = (ND_LIST *)get_nd_lister_from_label( root, leaf->next->str );
  
  left->right = right;
  right->left = left;

}











/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/











ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] )
{

  return get_my_nd_lister_from_label( root, label );

}











ND_LIST *get_my_nd_lister_from_label( ND_LIST *node, char label[] )
{

  if ( ! strcmp( node->my_node->label, label ))
    return node;
    
  char i = 0;  
  
  while ( node->down[ i ] != NULL ) {
  
    ND_LIST *x = (ND_LIST *)get_my_nd_lister_from_label( node->down[ i++ ], label );
    
    if ( x != NULL )
      return x;
  
  }    
  
  return NULL;

}









ND_LIST *get_leaf_from_label( ND_LIST *leaf_header, char label[] )
{

  ND_LIST *leaf = leaf_header;
  
  while ( leaf != NULL ) {
  
    if ( ! strcmp( leaf->my_node->label, label ))
      return leaf;
      
    leaf = leaf->right;
  
  }
  
  return NULL;

}








/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/












void print_nd_tree( ND_LIST *root, FILE *fh )
{

  if ( root == NULL )
    return;
    
  print_sep( fh );
    
  print_nd( root, fh );

}












void print_nd( ND_LIST *node, FILE *fh )
{

  print_my_nd( node, fh );
  
  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    print_nd( node->down[ i++ ], fh );
    
}











void print_my_nd( ND_LIST *node, FILE *fh )
{

  if ( node->down[ 0 ] == NULL ) 
    return;
    
  fprintf( fh, "%s :  %.1f\n", node->my_node->label, 
    (float)node->my_node->total_time );
  
  print_dot_sep( fh );
  
  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    print_single_daughter_node( node->down[ i++ ], fh );
    
  print_sep( fh );

}












void print_single_daughter_node( ND_LIST *node, FILE *fh )
{

  fprintf( fh, "%s  (%.1f)\n", node->my_node->label, (float)node->my_node->t );
  
  print_leaves_of_node( node, fh );

}











void print_leaves_of_node( ND_LIST *node, FILE *fh )
{

  if ( node->down[ 0 ] == NULL ) {
    fprintf( fh, "\t\t\t%s\n", node->my_node->label );
    return;
  }
  
  register int i = 0;
  
  while ( node->down[ i ] != NULL )
    print_leaves_of_node( node->down[ i++ ], fh );

}











void print_nd_leaves( ND_LIST *root, FILE *fh )
{

  ND_LIST *leaf_header = NULL;
  leaf_header = (ND_LIST *)get_nd_leaf_header( root );
  
  print_nd_leaves_leaf_list( leaf_header, fh );

}











ND_LIST *get_nd_leaf_header( ND_LIST *node )
{

  ND_LIST *leaf = (ND_LIST *)get_leftest_leaf( node );
  
  return get_nd_leaf_header_from_leaf( leaf );

}









ND_LIST *get_leftest_leaf( ND_LIST *node )
{

  ND_LIST *x = node;

  while ( x->down[ 0 ] != NULL )
    x = x->down[ 0 ];
    
  return x;

}









ND_LIST *get_nd_leaf_header_from_leaf( ND_LIST *node )
{

  ND_LIST *x = node;
  
  while ( x->left != NULL )
    x = x->left;
    
  return x;

}











void print_nd_leaves_leaf_list( ND_LIST *leaf_header, FILE *fh )
{

  ND_LIST *leaf = leaf_header;
  
  while ( leaf != NULL ) {
  
    fprintf( fh, "   %s\n", leaf->my_node->label );
    
    leaf = leaf->right;
  
  }

}





