#include "v.h"












#define INITIAL_ALPHA 7e-10
#define INITIAL_BETA 2e-8

#define MAX_ITERATIONS 10
#define MIN_NUMB_OF_MFS 4
#define PROB_THRESH 1.0e-10

#define MILLION 1000000.0
#define NUMB_OF_SITES 100

#define INPUT_COL_FNAME "data/ft_hg19/change/hg19.gcBox.count.txt"

#define OUTPUT_GENE 4















/*

    The functions called below consider a tree topology and estimate the
    likelihood of the observed data (i.e., observed sequences of the modern
    lineages) given parameter estimates alpha and beta.
    
    At each node, we have several parameter matrices (in substructure
    node->my_node->data...) :
      p    (transition matrix)
      pr   (likelihood matrix)
      n    (joint probability matrix)
      sum  (likelihood sums)
      ...etc
      
    First, the transition probabilities are computed (see function
    'put_transition_probs_into_tree()'...)
    These take the current values for alpha and beta, and calculate
    p[ i,j ](t[Z]) for each node Z.  More specifically, p[ i,j ](t[Z])
    represents the conditional probability of transition from i -> j binding 
    sites.  That is, assuming that parent node Y has i binding sites,
    p[ i,j ](t[Z]) gives the probability that daughter node Z contains j 
    binding sites.  Note that this is a function of t[Z], the amount of
    evolutionary time between node Y and node Y.  In the node structure
    ('ND_LIST'), t[Z] is defined as 'node->my_node->t', and is given in
    millions of years.  A related value is 'node->my_node->total_time', which
    gives the total amount of time since the node existed (e.g., each modern
    leaf gives 'node->my_node->total_time == 0').  Again, this is given in
    millions of years.
    
    After matrix p is created, we iterate across all genes.  For each gene
    (and thus each new set of orthologous sequence data), we compute
    matrix 'pr', which is the likelihood of the subtree given the observed
    data.  Thus, at node Z, 'pr[ i ]' = Pr( D[Z] | A[Z] = i, theta ), 
    where D[Z] is the data of the subtree of Z (i.e., the descendent lineages
    of node Z) and A[Z] is the random variable representing the number
    of binding sites at node Z.  These likelihoods are computed using 
    Felsenstein's pruning algorithm.
    
    The matrix 'sum' for node Z represents the sum :
      sum[ i ] =
        SUM over k :   Pr( D[Z] | A[Z] = k ) * p[ i,k ](t[Z])
        
    The last matrix to be computed is 'n', where
      'n[ i,j ] = Pr( D[root], A[Z] = j, A[Y] = i; theta ).
    These values are computed with the function call
    'evol_fill_phylo_n_vals()', defined in 'get_n.c'.

*/












int iteration;















void do_evol( void )
{

  printf( "Running evolution program call...\n\n" );
  
  do_my_evol();
  
  printf( "\n" );

}










void do_my_evol( void )
{

  ND_LIST *get_evol_tree( ND_LIST *root );

  ND_LIST *root = NULL;

  /*  In tree.c:  */
  root = (ND_LIST *)get_evol_tree( root );
  
  /*  In tree.c:  */
  print_nd_tree( root, stdout );
  
  /*  In this file, below:  */
  do_evol_with_tree( root );  

}










void do_evol_with_tree( ND_LIST *root )
{

  ND_LIST *get_nd_leaf_header( ND_LIST *node );
  
  ND_LIST *leaf_header = NULL;
  /*  In tree.c:  */
  leaf_header = (ND_LIST *)get_nd_leaf_header( root );

  /*  In this file, below:  */
  do_evol_with_tree_and_leaves( root, leaf_header );
  
}












void do_evol_with_tree_and_leaves( ND_LIST *root, ND_LIST *leaf_header )
{

  PARAM *param = NULL;
  param = (PARAM *)make_and_initialize_params( param );
  
  do_my_evol_with_params( root, leaf_header, param );
  
}










PARAM *make_and_initialize_params( PARAM *param )
{

  param = (PARAM *)malloc( sizeof(PARAM) );
  param->alpha = INITIAL_ALPHA;
  param->beta = INITIAL_BETA;
  
  return param;
  
}









void do_my_evol_with_params( ND_LIST *root, ND_LIST *leaf_header, PARAM *param )
{

  int converged = 0;

  register INDEX iter = 0;
  
  while (( ! converged ) && ( iter++ < MAX_ITERATIONS ))
    converged = do_evol_params_single_iter( root, leaf_header, param, iter );
    
}










int do_evol_params_single_iter( ND_LIST *root, ND_LIST *leaf_header, PARAM *param,
  INDEX iter )
{  

  FILE *open_text_file_reading( char a[], char b[] );
  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );
  
  extern int iteration;
  extern SEQ curr_gene;
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( INPUT_COL_FNAME, "" );
  
  iteration = iter;
  curr_gene = 0;
  
  ND_LIST *euth = NULL;
  /*  In tree.c:  */
  euth = (ND_LIST *)get_nd_lister_from_label( root, "Eutheria" );
  
  /*  In this file, below:  */
  set_prior_probs_w_label_str( root, euth, leaf_header, "hg19", in );
  
  set_position( 0, in );
  
  /*  In this file, below:  */
  int converged = do_evol_tree_w_fh( root, leaf_header, param, in );
  
  close_file( in );
  
  return converged;

}











int do_evol_tree_w_fh( ND_LIST *root, ND_LIST *leaf_header, PARAM *param, 
  FILE *in )
{

  STR_LIST *get_label_list_from_first_line( STR_LIST *label_header, FILE *in );
  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );
  
  STR_LIST *label_header = NULL;
  /*  In set.c:  */
  label_header = (STR_LIST *)get_label_list_from_first_line( label_header, in );
  
  /*  In this file, below:  */
  put_transition_probs_into_tree( root, param );
  
  /*  In this file, below:  */  
  do_evol_w_label_list( root, leaf_header, param, label_header, in );
  
  ND_LIST *euth = NULL;
  /*  In tree.c:  */
  euth = (ND_LIST *)get_nd_lister_from_label( root, "Eutheria" );
  
  return 1;

}













void put_transition_probs_into_tree( ND_LIST *root, PARAM *param )
{
  put_transition_probs_into_single_node( root, param );
}











void put_transition_probs_into_single_node( ND_LIST *node, PARAM *param )
{

  if ( node == NULL )
    return;
    
  put_my_transition_probs_into_single_node( node, param );
  
  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    put_transition_probs_into_single_node( node->down[ i++ ], param );

}











void put_my_transition_probs_into_single_node( ND_LIST *node, PARAM *param )
{

  TT_MX *p = node->my_node->data->p;

  register INDEX i, j;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {
    for ( j = 0; j < NUMB_OF_MFS; j++ ) {
      *(p->x + (i * p->j) + j) = get_p( param, node->my_node->t, i, j );
    }
  }
  
}















/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/















void do_evol_w_label_list( ND_LIST *root, ND_LIST *leaf_header, PARAM *param,
  STR_LIST *label_header, FILE *in )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL )
    do_evol_w_label_list_single_gene( root, leaf_header, param, label_header, line );

}












void do_evol_w_label_list_single_gene( ND_LIST *root, ND_LIST *leaf_header,
  PARAM *param, STR_LIST *label_header, char line[] )
{

  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );

  extern int iteration; 
  extern SEQ curr_gene;
  
  printf( "Iter  %d      Seq :  %d\n", iteration, ++curr_gene );
  
  /*  In set.c:  */
  initialize_nd_tree( root );

  /*  In set.c:  */
  set_my_evolutionary_tree_single_gene( root, leaf_header, label_header, line,
    C_NUCL );
    
  /*  In this file, below:  */
  do_evol_fill_phylo( root, param );
  
  ND_LIST *euth = NULL;
  /*  In tree.c:  */
  euth = (ND_LIST *)get_nd_lister_from_label( root, "Eutheria" );
  
  /*  In get_n.c:  */
  evol_fill_phylo_n_vals( euth, euth );
  
  /*  In this file, below:  */
  update_phylo_total_n_vals( euth );
  
  if ( curr_gene == OUTPUT_GENE )
    /*  In pr.c:  */
    print_evol_node_info( root, param, line );

}












/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/












void do_evol_fill_phylo( ND_LIST *root, PARAM *param )
{

  /*  In this file, below:  */
  do_evol_fill_phylo_single_node( root, param );
  
}











void do_evol_fill_phylo_single_node( ND_LIST *node, PARAM *param )
{

  if ( node->down[ 0 ] == NULL ) 
    return;
    
  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    do_evol_fill_phylo_single_node( node->down[ i++ ], param );
    
  fill_single_node_w_params( node, param );
  
}












void fill_single_node_w_params( ND_LIST *node, PARAM *param )
{

  if ( node_is_absent( node )) {
    node->my_node->exists = 0;
    return;
  }

  fill_single_node_w_param_probs( node, param );
  
  node->my_node->exists = 1;
  
}









int node_is_absent( ND_LIST *node )
{

  register char i = 0;
  
  while ( node->down[ i ] != NULL ) {
    if ( node->down[ i++ ]->my_node->exists )
      return 0;
  }
  
  return 1;

}











void fill_single_node_w_param_probs( ND_LIST *node, PARAM *param )
{

  register char i;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) 
    fill_single_node_w_param_single_numb_of_mfs( node, param, i );
  
}












void fill_single_node_w_param_single_numb_of_mfs( ND_LIST *node, PARAM *param,
  char i )
{

  TT pr = 1.0;

  register char k = 0;
  
  while ( node->down[ k ] != NULL ) 
    pr *= fill_single_node_single_numb_of_mfs_single_down( node->down[ k++ ],
      param, i );
      
  *(node->my_node->data->pr->x + i) = pr;
  
}











TT fill_single_node_single_numb_of_mfs_single_down( ND_LIST *node,
  PARAM *param, char i )
{

  if ( ! node->my_node->exists )
    return 1.0;

  TT sum = 0.0;
  
  TT_MX *s = node->my_node->data->sum;

  register char j;
  
  for ( j = 0; j < NUMB_OF_MFS; j++ )
    sum += get_single_node_prob( node, param, i, j );
    
  *(s->x + i) = sum;
    
  return sum;

}










TT get_single_node_prob( ND_LIST *node, PARAM *param, char i, char j )
{

  TT_MX *p = node->my_node->data->p;
  
  return *(node->my_node->data->pr->x + j) * (*(p->x + (i * p->j) + j));

}












/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/












void update_phylo_total_n_vals( ND_LIST *node )
{

  update_my_phylo_total_n_vals( node );
  
  register char i;
  
  while ( node->down[ i ] != NULL )
    update_phylo_total_n_vals( node->down[ i++ ] );

}












void update_my_phylo_total_n_vals( ND_LIST *node )
{

  TT_MX *n = node->my_node->data->n;
  TT_MX *x = node->my_node->data->n_sum;

  register char i, j, k;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) {
  
    TT sum = 0.0;
    
    for ( j = 0; j < NUMB_OF_MFS; j++ ) {
      for ( k = 0; k < NUMB_OF_MFS; k++ ) {
        sum += *(n->x + (i * n->j) + j);
      }
    }
    
    for ( j = 0; j < NUMB_OF_MFS; j++ ) {

      if ( *(n->x + (i * x->j) + j) == 0.0 )
        continue;

      *(x->x + (i * x->j) + j) += *(n->x + (i * n->j) + j) / sum;
      
    }
  
  }

}








/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
















void set_prior_probs_w_label_str( ND_LIST *root, ND_LIST *node, 
  ND_LIST *leaf_header, char label[], FILE *fh )
{

  ND_LIST *get_nd_lister_from_label( ND_LIST *root, char label[] );
  
  ND_LIST *spec = NULL;
  /*  In tree.c:  */
  spec = (ND_LIST *)get_nd_lister_from_label( root, label );
  
  /*  In this file, below:  */
  set_prior_probs_w_spec_node( root, node, leaf_header, spec, fh );

}















void set_prior_probs_w_spec_node( ND_LIST *root, ND_LIST *node, 
  ND_LIST *leaf_header, ND_LIST *spec, FILE *fh )
{

  TT_MX *create_tt_mx( TT_MX *xx, INDEX ii, INDEX jj );
  
  TT_MX *mx = NULL;
  mx = (TT_MX *)create_tt_mx( mx, NUMB_OF_MFS, 1 );

  set_prior_probs_w_spec_node_w_mx( root, node, leaf_header, spec, mx, fh );
  
  kill_tt_mx( mx );

}













void set_prior_probs_w_spec_node_w_mx( ND_LIST *root, ND_LIST *node,
  ND_LIST *leaf_header, ND_LIST *spec, TT_MX *mx, FILE *fh )
{

  STR_LIST *get_label_list_from_first_line( STR_LIST *label_header, FILE *in );
  
  STR_LIST *label_header = NULL;
  /*  In set.c:  */
  label_header = (STR_LIST *)get_label_list_from_first_line( label_header, fh );
  
  /*  In this file, below:  */
  set_prior_probs_w_label_list( root, node, leaf_header, label_header, spec, 
    mx, fh );
  
}













void set_prior_probs_w_label_list( ND_LIST *root, ND_LIST *node, 
  ND_LIST *leaf_header, STR_LIST *label_header, ND_LIST *spec, TT_MX *mx, FILE *fh )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, fh ) != NULL )
    update_counts_for_prior_probs_single_gene( root, node, leaf_header, line, 
      label_header, spec, mx, fh );
      
  set_prior_probs_w_count_mx( node, mx );

}














void update_counts_for_prior_probs_single_gene( ND_LIST *root, ND_LIST *node,
  ND_LIST *leaf_header, char line[], STR_LIST *label_header, ND_LIST *spec, 
  TT_MX *mx, FILE *fh )
{

  /*  In set.c:  */
  initialize_nd_tree( root );

  /*  In set.c:  */
  set_my_evolutionary_tree_single_gene( root, leaf_header, label_header, line,
    C_NUCL );

  /*  In this file, below:  */
  update_my_counts_for_prior_probs_single_gene( mx, spec );
    
}













void update_my_counts_for_prior_probs_single_gene( TT_MX *mx, ND_LIST *node )
{

  register char i;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) 
    *(mx->x + i) += *(node->my_node->data->pr->x + i);
  
}















void set_prior_probs_w_count_mx( ND_LIST *node, TT_MX *mx )
{

  TT total = get_total_from_tt_mx( mx );
  
  set_prior_probs_w_count_mx_total( node, mx, total );

}














TT get_total_from_tt_mx( TT_MX *mx )
{

  TT total = 0.0;
  
  TT *ptr = mx->x;
  
  INDEX end = mx->i * mx->j;
  
  register INDEX i = 0;
  
  while ( i++ < end )
    total += *ptr++;
    
  return total;

}











void set_prior_probs_w_count_mx_total( ND_LIST *node, TT_MX *mx, TT total )
{

  register char i;
  
  for ( i = 0; i < NUMB_OF_MFS; i++ ) 
    *(node->my_node->data->prior->x + i) = *(mx->x + i) / total;

}










/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/










TT get_p( PARAM *param, TT t, INDEX i, INDEX j )
{

  TT p = 0.0;
  
  register INDEX k;
  
  for ( k = 0; k <= j; k++ ) 
    p += get_p_single_k_value( param, t, i, j, k );
  
  return p;

}










TT get_p_single_k_value( PARAM *param, TT t, INDEX i, INDEX j, INDEX k )
{

  TT (*pr)( PARAM *pp, TT tt );
  
  pr = get_transition_u;
  
  TT u = get_binom_pr( pr, i, k, param, t );
  
  pr = get_transition_v;
  
  TT v = get_binom_pr( pr, NUMB_OF_SITES - i, j - k, param, t );  
  
  return u * v;

}










TT get_transition_u( PARAM *param, TT time )
{

  TT alpha = param->alpha;
  TT beta = param->beta;
  
  TT t = time * MILLION;
  
  TT coeff = 1.0 / (alpha + beta);
  
  return coeff * (alpha + (beta * exp( -1.0 * (alpha + beta) * t )));

}











TT get_transition_v( PARAM *param, TT time )
{

  TT alpha = param->alpha;
  TT beta = param->beta;
  
  TT t = time * MILLION;
  
  TT coeff = alpha / (alpha + beta);
  
  return coeff * (1.0 - exp( -1.0 * (alpha + beta) * t ));

}













TT get_binom_pr( TT (*pr)( PARAM *pp, TT tt ), INDEX n, INDEX k, PARAM *param, 
  TT t )
{

  TT p = pr( param, t );

  return n_choose_k( n, k ) * pow( (double)p, (double)k ) 
    * pow( 1.0 - p, (double)(n - k) );

}









TT n_choose_k( INDEX n, INDEX k )
{

  if (( k == 0 ) || ( n == k ))
    return 1.0;

  if (( n < k ) || ( k < 0 ))
    return 0.0;
    
  TT x = 1; 
    
  register long i = n, j = 1;
  
  while ( j <= k ) {
    x *= (TT)i--;
    x /= (TT)j++;
  }

  return x;
  
}






















