#include "my_occur.h"


















#define INPUT_MAF_FNAME "data/ft_hg19/align/hg19.multiz46way.my_fa.txt"
#define OUTPUT_BASE_NAME "data/ft_hg19/occur"
#define GENE_LABEL_FNAME "data/ft_hg19/occur/hg19.geneLabel.txt"
#define POS_FT_FNAME "data/ft_hg19/occur/hg19.position"
#define NUMB_OF_SPEC 46




/*

    Note :  All indices for (SEQ) sequence numbers
    are 1-based, i.e., start from :  i = 1, 2, 3, ..., etc
    
    gene_label->tss_column is also 1-based; tss at the first nucl site
    alignment would be gene_label->tss_column = 1, etc.
    This value gives the alignment column at which the TSS occurs
    within the MAF aligned sequence.
    
    COORD indices equal zero at TSS column of humans (hg18),
    negative numbers are upstream, etc.
    
    occur->align_pos = index according to alignments, including gaps
    occur->seq_pos = index according to nucleotides, based upon
      the TSS column (index 0), does not include gaps.

*/









void do_binary_occur_ft( void )
{

  printf( "Formatting occurrences in binary mode...\n\n" );
  
  do_my_binary_occur_ft();
  
  printf( "\n" );

}











void do_my_binary_occur_ft( void )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( INPUT_MAF_FNAME, "" );
  
  do_binary_occur_ft_w_input( in );
  
  close_file( in );

}












void do_binary_occur_ft_w_input( FILE *in )
{

  STR_LIST *get_spec_list_align_fh( STR_LIST *spec_header, FILE *fh );
  STR_LIST *create_str_lister( STR_LIST *xx );
  
  extern char species[ BUFFER ];

  STR_LIST *spec_header = NULL;
  
  if ( species[ 0 ] != '\0' ) {
    spec_header = (STR_LIST *)create_str_lister( spec_header );
    strcpy( spec_header->str, species );
  }
  
  else {
    /*  In occur.c:  */
    spec_header = (STR_LIST *)get_spec_list_align_fh( spec_header, in );
  }
  
  /*  In this file, below:  */
  do_binary_occur_ft_w_species_list( spec_header, in );
  
  kill_str_list( spec_header );

}










void set_gene_label_binary_file( char spec[], FILE *in )
{

  FILE *open_binary_file_writing( char a[], char b[] );
  
  FH my_fh = { .ind = NULL, .ft = NULL, .label = NULL, 
    .pos = NULL, .ind_pos = NULL };
  
  my_fh.label = (FILE *)open_binary_file_writing( GENE_LABEL_FNAME, "" );
  my_fh.pos = (FILE *)open_binary_file_writing( POS_FT_FNAME, "ft.txt" );
  my_fh.ind_pos = (FILE *)open_binary_file_writing( POS_FT_FNAME, "ind.txt" );

  /*  In this file, below:  */
  do_gene_label_binary_file_w_fh( spec, in, &my_fh );
  
  close_file( my_fh.label );  
  close_file( my_fh.pos );
  close_file( my_fh.ind_pos );

}












void do_gene_label_binary_file_w_fh( char spec[], FILE *in, FH *my_fh )
{

  set_position( 0, in );

  SEQ seq_numb = 0;

  char line[ BUFFER ], seq[ LINE_BUFFER ];
  char label[ BUFFER ];
  
  while ( fgets( line, BUFFER, in ) != NULL )
    get_gene_label_binary_entry_single_line( line, seq, label, spec, &seq_numb, 
      my_fh );

}











void get_gene_label_binary_entry_single_line( char line[], char seq[], 
  char label[], char spec[], SEQ *seq_numb, FH *my_fh )
{

  if (( line[ 0 ] == '.' ) || ( line[ 0 ] == '\n' ))
    return;
    
  if ( line[ 0 ] == '>' ) 
    strcpy( label, line );
  
  else   
    get_my_gene_label_binary_entry_single_line( line, seq, label, spec, 
      seq_numb, my_fh );
    
}









void get_my_gene_label_binary_entry_single_line( char line[], char seq[],
  char label[], char spec[], SEQ *seq_numb, FH *my_fh )
{
    
  if ( line[ 0 ] == '=' )
    new_gene_label_binary_sequence( seq, ++(*seq_numb), label, my_fh );
  
  else
    do_binary_occur_ft_sequence_update( line, seq, spec );
    
}










void new_gene_label_binary_sequence( char seq[], SEQ seq_numb, char label_line[],
  FH *my_fh )
{

  GENE_LABEL label = { .gene = "", .seq_numb = seq_numb, .tss_column = 0 };
  
  get_and_put_gene_name_from_label_line( &label, label_line );
  
  label.tss_column = get_alignment_tss_column_numb( seq );
  
  do_position_align_nucls( seq, seq_numb, &label, my_fh );

  printf( "Finished label gene :  %6d  %s  %d\n", seq_numb, label.gene,
    (int)label.tss_column );
  
  put( &label, sizeof(GENE_LABEL), my_fh->label );
  
  seq[ 0 ] = '\0';
  
}









void get_and_put_gene_name_from_label_line( GENE_LABEL *label, char label_line[] )
{

  register int i = 0, j = 0;
  
  while ( label_line[ i++ ] != '.' )
    ;
    
  while ( label_line[ i ] != '.' )
    label->gene[ j++ ] = label_line[ i++ ];
    
  label->gene[ j ] = '\0';
  
}











COORD get_alignment_tss_column_numb( char seq[] )
{

  int is_nucl( char x );

  register COORD nucl_numb = 0, i = 0, tss_column = NON_SEQ;
  
  while ( seq[ i ] != '\0' ) {
  
    /*  In ../main/nucl.c:  */
    if ( is_nucl( seq[ i++ ] )) {
    
      if ( ++nucl_numb == TSS_INDEX )
        tss_column = i;
    
    }
    
  }
  
  return tss_column;
  
}
















/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/










void do_binary_occur_ft_w_species_list( STR_LIST *spec_header, FILE *in )
{

  STR_LIST *spec = spec_header;
  
  while ( spec != NULL )
    do_binary_occur_ft_single_spec_and_step( &spec, in );

}









void do_binary_occur_ft_single_spec_and_step( STR_LIST **spec, FILE *in )
{
  do_binary_occur_ft_single_spec( *spec, in );
  *spec = (*spec)->next;
}










void do_binary_occur_ft_single_spec( STR_LIST *spec, FILE *in )
{

  static int i = 0;

  printf( "Species (%d) :  %s\n", ++i, spec->str );
    
  set_position( 0, in );
  
  do_my_binary_occur_ft_single_spec( spec, in );

}











void do_my_binary_occur_ft_single_spec( STR_LIST *spec, FILE *in )
{

  FILE *open_binary_file_writing( char a[], char b[] );
  FILE *open_binary_file_reading( char a[], char b[] );
  
  char fname[ BUFFER ];
  
  /*  In occur.c:  */
  get_occur_format_output_fnames( fname, spec );
  
  FH my_fh = { .ind = NULL, .ft = NULL, .label = NULL };
  
  my_fh.ind = (FILE *)open_binary_file_reading( fname, "ind.txt" );
  my_fh.label = (FILE *)open_binary_file_reading( GENE_LABEL_FNAME, "" );
  my_fh.ft = (FILE *)open_binary_file_writing( fname, "ft.txt" );
    
  do_binary_occur_ft_single_spec_w_fhs( spec->str, in, &my_fh );
  
  close_file( my_fh.ind );
  close_file( my_fh.ft );
  close_file( my_fh.label );

}











void do_binary_occur_ft_single_spec_w_fhs( char spec[], FILE *in, FH *my_fh )
{

  OCCUR_IND_CELL *create_occur_ind_cell( OCCUR_IND_CELL *xx, INDEX nn );
  
  OCCUR_IND_CELL *occur_cell = NULL;
  occur_cell = (OCCUR_IND_CELL *)create_occur_ind_cell( occur_cell,
    NUMB_OF_EIGHTMERS );
    
  occur_cell = (OCCUR_IND_CELL *)set_occur_ind_cell_from_ind_file( occur_cell,
    my_fh );
    
  do_binary_occur_ft_single_spec_w_occur_ind_cell( occur_cell, spec, in, my_fh );
  
  check_occur_cell_counts( occur_cell, my_fh );
  
  kill_occur_ind_cell( occur_cell );
  
}











OCCUR_IND_CELL *set_occur_ind_cell_from_ind_file( OCCUR_IND_CELL *occur_cell,
  FH *my_fh )
{

  OCCUR_IND *my_occur = occur_cell->array;

  register INDEX i = 0;
  
  while ( i++ < occur_cell->n )
    set_single_occur_ind_struct_from_ind_file( my_occur++, my_fh );
    
  return occur_cell;

}












void set_single_occur_ind_struct_from_ind_file( OCCUR_IND *my_occur, FH *my_fh )
{
  get( my_occur, sizeof(OCCUR_IND), my_fh->ind ); 
  my_occur->count = 0;
}









void do_binary_occur_ft_single_spec_w_occur_ind_cell( OCCUR_IND_CELL *occur_cell,
  char spec[], FILE *in, FH *my_fh )
{

  GENE_LABEL gene_label;

  char line[ BUFFER ], seq[ LINE_BUFFER ];
  seq[ 0 ] = '\0';
  
  while ( fgets( line, BUFFER, in ) != NULL )
    do_binary_occur_ft_single_spec_single_line( occur_cell, &gene_label, line, 
      seq, spec, my_fh );

}












void do_binary_occur_ft_single_spec_single_line( OCCUR_IND_CELL *occur_cell,
  GENE_LABEL *gene_label, char line[], char seq[], char spec[], FH *my_fh )
{

  printf( "Species :   %s\n", spec );

  if (( line[ 0 ] == '.' ) || ( line[ 0 ] == '\n' ))
    return;
    
  if ( line[ 0 ] == '>' )
    get( gene_label, sizeof(GENE_LABEL), my_fh->label );
    
  else
    do_my_binary_occur_ft_single_spec_single_line( occur_cell, gene_label,
      line, seq, spec, my_fh );

}













void do_my_binary_occur_ft_single_spec_single_line( OCCUR_IND_CELL *occur_cell,
  GENE_LABEL *gene_label, char line[], char seq[], char spec[], FH *my_fh )
{

  if ( line[ 0 ] == '=' )
    get_and_put_binary_sequence_info_occur_struct( occur_cell, gene_label,
      seq, my_fh );
      
  else
    do_binary_occur_ft_sequence_update( line, seq, spec );

}













void do_binary_occur_ft_sequence_update( char line[], char seq[], char spec[] )
{

  char curr_spec[ BUFFER ], curr_seq[ BUFFER ];
  
  sscanf( line, "%s%s", curr_spec, curr_seq );
  
  if ( strcmp( curr_spec, spec ))
    return;
    
  do_binary_occur_ft_matching_spec_seq_update( seq, curr_seq );

}












void do_binary_occur_ft_matching_spec_seq_update( char seq[], char curr_seq[] )
{
  strcat( seq, curr_seq );
}












/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/











void get_and_put_binary_sequence_info_occur_struct( OCCUR_IND_CELL *occur_cell,
  GENE_LABEL *gene_label, char seq[], FH *my_fh )
{

  static int i = 0;

  char pure_seq[ LINE_BUFFER ];
  
  /*  In occur.c:  */
  get_aligned_seq_no_gaps( pure_seq, seq );
  
  printf( "\t\t\t\t%d :  %s\n", ++i, pure_seq );
  
  /*  In this file, below:  */
  get_and_put_binary_seq_info_occur_two_seqs( occur_cell, pure_seq, seq, 
    gene_label, my_fh );
    
  seq[ 0 ] = '\0';
  
}










void get_and_put_binary_seq_info_occur_two_seqs( OCCUR_IND_CELL *occur_cell,
  char pure_seq[], char seq[], GENE_LABEL *gene_label, FH *my_fh )
{

  COORD tss = gene_label->tss_column;

  COORD rel_align_tss = (tss * -1) + 1;
  
  COORD rel_site_tss = get_relative_site_tss_column( seq, tss );
  
  get_and_put_binary_seq_info_w_start_indices( occur_cell, pure_seq, seq,
    gene_label, rel_align_tss, rel_site_tss, my_fh );

}













COORD get_relative_site_tss_column( char seq[], COORD tss )
{

  COORD site = 0, column = 0;
  COORD end = tss - 1;
    
  while ( column < end ) {
  
    switch ( tolower( seq[ column++ ] )) {
    
    case 'a' :
    case 'c' :
    case 'g' :
    case 't' :
    case 'n' :
      --site;
      break;
      
    default :
      break;
    
    }
  
  }
  
  printf( "First site index :  %d\n", (int)site );
  
  return site;

}














void get_and_put_binary_seq_info_w_start_indices( OCCUR_IND_CELL *occur_cell, 
  char pure_seq[], char seq[], GENE_LABEL *gene_label, COORD rel_align_tss, 
  COORD rel_site_tss, FH *my_fh )
{

  COORD align_pos = rel_align_tss, site_pos = rel_site_tss;

  int end = (int)strlen( seq ) - MOTIF_LENGTH + 1;
  
  MF numb = 0;
  
  int align_index = 0, pure_index = 0, pure_nucl = MOTIF_LENGTH - 1;
  
  while ( align_index < end )
    get_and_put_binary_seq_info_single_mf( occur_cell, pure_seq, seq, gene_label, 
      align_index++, &pure_index, &pure_nucl, align_pos++, &site_pos, &numb, my_fh );
      
}










void get_and_put_binary_seq_info_single_mf( OCCUR_IND_CELL *occur_cell,
  char pure_seq[], char seq[], GENE_LABEL *gene_label, int align_index,
  int *pure_index, int *pure_nucl, COORD align_pos, COORD *site_pos, MF *numb, 
  FH *fh )
{

  switch ( tolower( seq[ align_index ] )) {
  
  case 'a' :
  case 'c' :
  case 'g' :
  case 't' :
  case 'n' :
  
    if ( *pure_index < (int)strlen( pure_seq ) - MOTIF_LENGTH + 1 )
    
      do_binary_seq_single_nucl_is_nucl( occur_cell, pure_seq, seq, gene_label,
        align_index, (*pure_index)++, (*pure_nucl)++, align_pos, (*site_pos)++, 
        numb, fh );
    
    break;
    
  default :
    break;
  
  }

}











void do_binary_seq_single_nucl_is_nucl( OCCUR_IND_CELL *occur_cell, char pure_seq[],
  char seq[], GENE_LABEL *gene_label, int align_index, int pure_index,
  int pure_nucl, COORD align_pos, COORD site_pos, MF *numb, FH *fh )
{

  *numb = get_motif_numb_binary_seq( pure_seq, align_index, pure_index, 
    pure_nucl, *numb );
    
  if ( *numb == NON_MF )
    return;
  
  do_binary_seq_single_nucl_motif_numb( occur_cell, *numb, align_pos,
    site_pos, gene_label, fh );

}










MF get_motif_numb_binary_seq( char pure_seq[], int align_index, int pure_index, 
  int pure_nucl, MF numb )
{

  MF get_mf_numb_from_string( char mm[], int ll );
  char get_nucl_numb_from_char( char nucl );
  
  char mf[ BUFFER ];
  
  strncpy( mf, pure_seq + pure_index, MOTIF_LENGTH );

  /*  In ../main/nucl.c:  */
  return get_mf_numb_from_string( mf, MOTIF_LENGTH );
  
}










void do_binary_seq_single_nucl_motif_numb( OCCUR_IND_CELL *occur_cell,
  MF numb, COORD align_pos, COORD site_pos, GENE_LABEL *gene_label, FH *fh )
{
  
  FPOS fpos = (occur_cell->array + (INDEX)numb)->fpos;
    
  fpos += ((occur_cell->array + (INDEX)numb)->count) * sizeof(OCCUR);  
    
  set_position( fpos, fh->ft );
  
  put_binary_occur_info_ft_file( align_pos, site_pos, gene_label, fh );
  
  ++((occur_cell->array + (INDEX)numb)->count);
  
}












void put_binary_occur_info_ft_file( COORD align_pos, COORD site_pos,
  GENE_LABEL *gene_label, FH *fh )
{

  OCCUR my_occur;
  
  my_occur.seq_numb = gene_label->seq_numb;
  my_occur.align_pos = align_pos;
  my_occur.seq_pos = site_pos;
  
  put( &my_occur, sizeof(OCCUR), fh->ft ); 

}











/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/











void do_position_align_nucls( char seq[], SEQ seq_numb, GENE_LABEL *gene, 
  FH *my_fh )
{

  OCCUR_IND my_index;

  get_position_align_nucls_occur_ind( &my_index, seq, my_fh );
  
  do_position_align_nucls_w_sequence( seq, seq_numb, gene, my_fh );
  
}












void get_position_align_nucls_occur_ind( OCCUR_IND *my_index, char seq[], 
  FH *my_fh )
{

  long get_position( FILE *fh );
  int is_nucl( char x );
  
  my_index->fpos = (FPOS)get_position( my_fh->pos );
  my_index->count = 0;
  
  register int i = 0;
  
  while ( seq[ i ] != '\0' ) {
    /*  In ../main/nucl.c:  */
    if ( is_nucl( seq[ i++ ] ))
      ++(my_index->count);
  }
  
  put( my_index, sizeof(OCCUR_IND), my_fh->ind_pos );

}













void do_position_align_nucls_w_sequence( char seq[], SEQ seq_numb,
  GENE_LABEL *gene, FH *my_fh )
{

  int is_nucl( char x );
  
  register COORD i = 0, pos = 0;
  
  while ( seq[ i ] != '\0' ) {
  
    /*  In ../main/nucl.c:  */
    if ( is_nucl( seq[ i ] ))
      /*  In this file, below:  */
      do_position_align_nucls_w_sequence_bp( seq_numb, gene, i, pos++, my_fh );

    ++i;

  }
  
}













void do_position_align_nucls_w_sequence_bp( SEQ seq_numb, GENE_LABEL *gene,
  COORD column, COORD pos, FH *my_fh )
{

  OCCUR my_occur;
  
  my_occur.seq_numb = seq_numb;
  my_occur.align_pos = (column - gene->tss_column) + 1;
  my_occur.seq_pos = (pos - TSS_INDEX) + 1;
  
  put( &my_occur, sizeof(OCCUR), my_fh->pos );
  
  fflush( my_fh->pos );

}










/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/















void check_occur_cell_counts( OCCUR_IND_CELL *occur_cell, FH *my_fh )
{

  set_position( 0, my_fh->ind );

  OCCUR_IND *occur = occur_cell->array;
  
  register int i = 0;
  
  while ( i++ < occur_cell->n )
    check_occur_cell_single_count( occur++, my_fh );

}











void check_occur_cell_single_count( OCCUR_IND *occur, FH *my_fh )
{

  OCCUR_IND my_occur;
  
  get( &my_occur, sizeof(OCCUR_IND), my_fh->ind );
  
  printf( "\t\t%6d\t%6d\n", (int)my_occur.count, (int)occur->count );
  
  if ( occur->count != my_occur.count )
    error( 2600, "Occurrence counts do not match.\n" );

}











