#include "put_extract.h"












#define LINE_LENGTH 50













void put_alignment_extract_single_gene( MAF_LIST *maf_header, 
  GENCOORD_ENTRY *entry, FILE *out )
{
  
  int numb_of_sections = get_numb_of_sections_maf_list( maf_header );
  
  condense_alignment_all_species( maf_header, numb_of_sections );
  
  set_reverse_complement_alignments( maf_header, entry );
  
  put_alignment_extract_correct_maf_list( maf_header, out );
  
}












int get_numb_of_sections_maf_list( MAF_LIST *maf_header )
{

  int numb_of_sections = 0;

  MAF_LIST *maf = maf_header;
  
  while ( maf != NULL )
    numb_of_sections = get_numb_of_sections_single_spec_and_step( &maf, 
      numb_of_sections );
      
  return numb_of_sections;

}











int get_numb_of_sections_single_spec_and_step( MAF_LIST **maf, 
  int numb_of_sections )
{

  int numb = get_numb_of_sections_single_spec( *maf, numb_of_sections );
  
  *maf = (*maf)->next;
  
  return numb;

}











int get_numb_of_sections_single_spec( MAF_LIST *maf, int numb_of_sections )
{

  int numb = 0;

  MAF_LIST *section = maf->same;
  
  while ( section != NULL ) {
    ++numb;
    section = section->same;
  }
  
  if ( ! numb_of_sections )
    return numb;
    
  if ( numb_of_sections != numb )
    error( 2300, "Number of sections differs across species.\n" );

  return numb;

}
















/*======================================================================*/
/*======================================================================*/
/*======================================================================*/
/*======================================================================*/
/*======================================================================*/














void condense_alignment_all_species( MAF_LIST *maf_header, int numb_of_sections )
{

  int i;
  
  for ( i = 1; i <= numb_of_sections; i++ )
    condense_alignment_single_section_all_species( maf_header, i, numb_of_sections );

}











void condense_alignment_single_section_all_species( MAF_LIST *maf_header,
  int section, int numb_of_sections )
{

  int align_length = get_alignment_length_major_species( maf_header, section );
  
  MAF_LIST *maf = maf_header;
  
  while ( maf != NULL )
    condense_alignment_single_section_single_spec_and_step( maf_header, &maf,
      section, align_length );
      
}











int get_alignment_length_major_species( MAF_LIST *maf_header, int section )
{

  MAF_LIST *maf = (MAF_LIST *)get_section_maf_lister( maf_header, section );
  
  return strlen( maf->my_maf->seq );

}










void condense_alignment_single_section_single_spec_and_step( MAF_LIST *maf_header,
  MAF_LIST **maf, int section, int align_length )
{
  condense_alignment_single_section_single_spec( maf_header, *maf, section,
    align_length );
  *maf = (*maf)->next;
}











void condense_alignment_single_section_single_spec( MAF_LIST *maf_header,
  MAF_LIST *maf, int section, int align_length )
{

  MAF_LIST *sect_maf = (MAF_LIST *)get_section_maf_lister( maf, section );
  
  condense_alignment_single_section_maf_listers( maf_header, maf, sect_maf,
    align_length );

}











MAF_LIST *get_section_maf_lister( MAF_LIST *maf, int section )
{

  MAF_LIST *sect_maf = maf;
  
  register int i = 0;
  
  while ( i++ < section )
    sect_maf = sect_maf->same;
    
  if ( sect_maf == NULL )
    error( 2401, "MAF section numb exceeds buffer.\n" );

  return sect_maf;
}











void condense_alignment_single_section_maf_listers( MAF_LIST *maf_header, 
  MAF_LIST *maf, MAF_LIST *sect_maf, int align_length )
{

  if ( sect_maf->my_maf->seq[ 0 ] == '\0' )
    update_blank_maf( sect_maf, align_length );

  strcat( maf->my_maf->seq, sect_maf->my_maf->seq );

}














void update_blank_maf( MAF_LIST *sect_maf, int align_length )
{

  register int i;
  
  for ( i = 0; i < align_length; i++ )
    sect_maf->my_maf->seq[ i ] = '-';
    
  sect_maf->my_maf->seq[ align_length ] = '\0';

}















/*======================================================================*/
/*======================================================================*/
/*======================================================================*/
/*======================================================================*/
/*======================================================================*/














void set_reverse_complement_alignments( MAF_LIST *maf_header, 
  GENCOORD_ENTRY *entry )
{
  if ( entry->strand == '-' )
    set_reverse_complement_known_rev_strand_maf_list( maf_header );
}














void set_reverse_complement_known_rev_strand_maf_list( MAF_LIST *maf_header )
{

  MAF_LIST *maf = maf_header;
  
  while ( maf != NULL )
    set_rev_compl_single_maf_lister_and_step( &maf );
    
}












void set_rev_compl_single_maf_lister_and_step( MAF_LIST **maf )
{
  set_rev_compl_single_maf_lister( *maf );
  *maf = (*maf)->next;
}











void set_rev_compl_single_maf_lister( MAF_LIST *maf )
{

  char rev_compl[ LINE_BUFFER ];
  
  /*  In ../main/nucl.c:  */
  get_rev_complement_seq( rev_compl, maf->my_maf->seq );
  
  strcpy( maf->my_maf->seq, rev_compl );

}













/*======================================================================*/
/*======================================================================*/
/*======================================================================*/
/*======================================================================*/
/*======================================================================*/













void put_alignment_extract_correct_maf_list( MAF_LIST *maf_header, FILE *out )
{

  COORD get_numb_of_nucl_maf_lister( MAF_LIST *maf );
  
  /*  In ../main/nucl.c:  */
  if ( get_numb_of_nucl_maf_lister( maf_header ) != SEQ_LENGTH )
    return;

  fprintf( out, ">%s.%s.%s.%d.%d.%c\n",
    maf_header->my_maf->entry->spec,
    maf_header->my_maf->entry->gene,
    maf_header->my_maf->entry->chr,
    (int)maf_header->my_maf->entry->start,
    (int)maf_header->my_maf->entry->end,
    maf_header->my_maf->entry->strand );
    
  /*  In this file, below:  */  
  put_my_alignment_extract_correct_maf_list( maf_header, out );
  
}












void put_my_alignment_extract_correct_maf_list( MAF_LIST *maf_header, FILE *out )
{

  int seq_length = strlen( maf_header->my_maf->seq );

  register int i;
  
  for ( i = 0; i < seq_length; i += LINE_LENGTH )
    put_alignment_extract_correct_maf_list_single_line( maf_header, i, out );
    
  fprintf( out, "\n" );
  print_sep( out );
  fprintf( out, "\n" );

}











void put_alignment_extract_correct_maf_list_single_line( MAF_LIST *maf_header,
  int site, FILE *out )
{

  MAF_LIST *maf = maf_header;
  
  while ( maf != NULL )
    put_align_extract_single_line_single_maf_lister_and_step( &maf, site, out );
    
  print_dot_sep( out );

}












void put_align_extract_single_line_single_maf_lister_and_step( MAF_LIST **maf,
  int site, FILE *out )
{
  put_align_extract_single_line_single_maf_lister( *maf, site, out );
  *maf = (*maf)->next;
}











void put_align_extract_single_line_single_maf_lister( MAF_LIST *maf, int site,
  FILE *out )
{

  int seq_is_empty( char seq[] );

  /*  In ../main/nucl.c:  */
  if ( seq_is_empty( maf->my_maf->seq ))
    return;
    
  /*  In thisf file, below:  */
  put_my_align_extract_single_line_single_maf_lister( maf, site, out );
  
}












void put_my_align_extract_single_line_single_maf_lister( MAF_LIST *maf,
  int site, FILE *out )
{

  put_align_extract_label_column( maf, out );
  
  put_align_extract_seq_column( maf, site, out );
  
}










void put_align_extract_label_column( MAF_LIST *maf, FILE *out )
{
  fprintf( out, "%-10s\t\t", maf->my_maf->entry->spec, maf->my_maf->entry->gene );
}










void put_align_extract_seq_column( MAF_LIST *maf, int site, FILE *out )
{

  int end = site + LINE_LENGTH;
  int i = site;
  
  while (( i < end ) && ( maf->my_maf->seq[ i ] != '\0' ))
    fprintf( out, "%c", tolower( maf->my_maf->seq[ i++ ] ));

  fprintf( out, "\n" );

}


























