#include "cpg.h"















#define CPG_FNAME_BASE "data/ft_hg18/cpg/data/hg18.CpG"
#define FASTA_SEQ_FNAME "data/ft_hg18/hg18.refSeq.500.100.Mar2006.txt.final.txt"
#define LIST_FNAME_BASE "data/ft_hg18/evol/lists/hg18.Mammalia.converted.sorted"

#define CPG_OUTPUT "data/ft_hg18/cpg/hg18.CpG_islands.results.txt"



#define PROM_START 300
#define PROM_END 500
#define SEQ_LENGTH 600










void do_cpg_islands( void )
{

  condense_cpg_island_files();
  
  do_my_cpg_islands();

}












void condense_cpg_island_files( void )
{
  /*  In methyl.c:  */
  condense_methylation_files( CPG_FNAME_BASE );
}











void do_my_cpg_islands( void )
{

  FILE *open_text_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );
  
  FH my_fh = { .fasta = NULL, .cpg_fh = NULL, .conv_fh = NULL,
    .no_conv_fh = NULL, .all_fh = NULL, .out = NULL };
  
  my_fh.fasta = (FILE *)open_text_file_reading( FASTA_SEQ_FNAME, "" );
  my_fh.cpg_fh = (FILE *)open_text_file_reading( CPG_FNAME_BASE, "total.txt" );
  my_fh.conv_fh = (FILE *)open_text_file_reading( LIST_FNAME_BASE, "C.A.txt" );
  my_fh.no_conv_fh = (FILE *)open_text_file_reading( LIST_FNAME_BASE, "C.noA.txt" );
  my_fh.all_fh = (FILE *)open_text_file_reading( LIST_FNAME_BASE, "C.all.txt" );
  my_fh.out = (FILE *)open_text_file_writing( CPG_OUTPUT, "" );

  INFO my_info = { .total_genes = 0, .cpg_genes = 0, .conv_genes = 0,
    .conv_cpg = 0, .no_conv_genes = 0, .no_conv_cpg = 0 };
    
  do_cpg_island_correlation( &my_info, &my_fh );  
  
  print_cpg_island_correlation_output( &my_info, my_fh.out );
    
  close_file( my_fh.fasta );
  close_file( my_fh.cpg_fh );
  close_file( my_fh.conv_fh );
  close_file( my_fh.no_conv_fh );
  close_file( my_fh.all_fh );
  close_file( my_fh.out );
    
}











void do_cpg_island_correlation( INFO *my_info, FH *my_fh )
{

  int get_fasta_seq_and_gene_label( GENCOORD_ENTRY *my_entry, char seq[], 
    FILE *in );

  GENCOORD_ENTRY my_entry;
  char seq[ LINE_BUFFER ];
  
  GG gg_conv, gg_no_conv;
  
  seq[ 0 ] = '\0';
  
  /*  In this file, below:  */
  get_gg_struct_list_fh( &gg_conv, my_fh->conv_fh );
  get_gg_struct_list_fh( &gg_no_conv, my_fh->no_conv_fh );
  
  /*  In methyl.:  */
  while ( get_fasta_seq_and_gene_label( &my_entry, seq, my_fh->fasta )) {  

    /*  In this file, below:  */
    do_cpg_island_correlation_single_gene( my_info, &my_entry, seq, &gg_conv,
      &gg_no_conv, my_fh );

    seq[ 0 ] = '\0';

  }

}














void get_gg_struct_list_fh( GG *gg, FILE *fh )
{

  char line[ LINE_BUFFER ];
  
  if ( fgets( line, LINE_BUFFER, fh ) == NULL ) {
    gg->seq_numb = gg->index = 0;
    return;
  }
  
  sscanf( line, "%d%d%s%s", &(gg->index), &(gg->seq_numb), gg->acc, gg->gene );

}










void do_cpg_island_correlation_single_gene( INFO *my_info, GENCOORD_ENTRY *my_entry,
  char seq[], GG *gg_conv, GG *gg_no_conv, FH *my_fh )
{

  ++(my_info->total_genes);
  
  /*  In this file, below:  */
  int gene_has_cpg = cpg_in_promoter_region( my_entry, my_fh->cpg_fh );
  
  my_info->cpg_genes += gene_has_cpg;
  
  if ( ! strcmp( gg_conv->acc, my_entry->gene )) {
  
    ++(my_info->conv_genes);
    my_info->conv_cpg += gene_has_cpg;
  
    /*  In this file, above:  */
    get_gg_struct_list_fh( gg_conv, my_fh->conv_fh );

  }

  if ( ! strcmp( gg_no_conv->acc, my_entry->gene )) {

    ++(my_info->no_conv_genes);
    my_info->no_conv_cpg += gene_has_cpg;
    
    /*  In this file, above:  */
    get_gg_struct_list_fh( gg_no_conv, my_fh->no_conv_fh );

  }

}













int cpg_in_promoter_region( GENCOORD_ENTRY *my_entry, FILE *fh )
{

  set_position( 0, fh );
  
  int correct_chr = 0;
  
  char chr[ BUFFER ];
  int start, end;
  
  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, fh ) != NULL ) {
  
    if ( line[ 0 ] == '#' )
      continue;
  
    sscanf( line, "%s%d%d", chr, &start, &end );
    
    if ( strcmp( chr, my_entry->chr )) {

      if ( correct_chr )
        return 0;

      continue;

    }
    
    correct_chr = 1;
    
    if ( cpg_in_single_promoter_region( my_entry, start, end ))
      return 1;
  
  }
  
  return 0;

}














int cpg_in_single_promoter_region( GENCOORD_ENTRY *my_entry, int start, int end )
{

  switch ( my_entry->strand ) {
  
  case '+' :
    
    if ( end < my_entry->start + PROM_START )
      return 0;
      
    if ( start > my_entry->start + PROM_END )
      return 0;
      
    return 1;
    
  case '-' :
  
    if ( end < my_entry->start + SEQ_LENGTH - PROM_END )
      return 0;
      
    if ( start > my_entry->start + SEQ_LENGTH - PROM_START )
      return 0;
      
    return 1;
    
  default :
    error( 11400, "Unrecognized strand type.\n" );
    break;
  
  }

}















void print_cpg_island_correlation_output( INFO *my_info, FILE *out )
{

  PVAL get_hyper_geo_pval( int nn, int n, int gg, int g );

  float p = (float)my_info->cpg_genes / (float)my_info->total_genes;
  float conv_p = (float)my_info->conv_cpg / (float)my_info->conv_genes;
  float no_conv_p = (float)my_info->no_conv_cpg / (float)my_info->no_conv_genes;
    
  int nn = my_info->total_genes;
  int n = my_info->conv_genes;
  int gg = my_info->cpg_genes;
  int g = my_info->conv_cpg;  
    
  /*  In ../gene/sig.c:  */  
  PVAL conv_pval = get_hyper_geo_pval( nn, n, gg, g );

  nn = my_info->total_genes;
  n = my_info->no_conv_genes;
  gg = my_info->cpg_genes;
  g = my_info->no_conv_cpg;  
    
  /*  In ../gene/sig.c:  */  
  PVAL no_conv_pval = get_hyper_geo_pval( nn, n, gg, g );

  fprintf( out, "Total genes :     %d\n", my_info->total_genes );
  fprintf( out, "CpG promoters :   %d\n", my_info->cpg_genes );
  fprintf( out, "  Frac :  %.1f\n", p * 100.0 );
    
  print_dot_sep( out );
  
  fprintf( out, "Converted genes :   %d\n", my_info->conv_genes );
  fprintf( out, "Conv CpG prom :     %d\n", my_info->conv_cpg );
  fprintf( out, "Expected CpG :      %.2f\n", p * my_info->conv_genes );
  fprintf( out, "  Frac :  %.1f\n", conv_p * 100.0 );
  fprintf( out, "  Ratio : %.2f\n", conv_p / p );
  fprintf( out, "  Pval  : %.2e\n", (double)conv_pval );

  print_dot_sep( out );
  
  fprintf( out, "Non-conv genes :    %d\n", my_info->no_conv_genes );
  fprintf( out, "Non-conv CpG p :    %d\n", my_info->no_conv_cpg );
  fprintf( out, "Expected CpG :      %.2f\n", p * my_info->no_conv_genes );
  fprintf( out, "  Frac :  %.1f\n", no_conv_p * 100.0 );
  fprintf( out, "  Ratio : %.2f\n", no_conv_p / p );
  fprintf( out, "  Pval :  %.2e\n", (double)no_conv_pval );

  print_dot_sep( out );

}












