#include "en_binom.h"














#define INPUT_FNAME "data/ft_hg19/change/hg19.gcBox.count.txt"
#define SPEC_ONE "hg19"
#define SPEC_TWO "hg19"
#define LEAF_LABEL "Human"


#define GO_GENE_FNAME "data/ft_hg18/go/go_genes.txt"

#define GENE_LIST_BASE "data/ft_hg18/evol/lists/hg18.Mammalia.converted.sorted"
#define GO_CONDENSED_FNAME "data/ft_hg19/go/go.hg.condensed.txt"
#define BIRD_HUMAN_BASE "data/ft_hg18/evol/lists/bird.hg19.converted.sorted"

#define GO_GENE_FNAME "data/ft_hg18/go/go_genes.txt"
#define GO_TERM_FNAME "data/ft_hg19/go/go.terms.txt"

#define OUTPUT_FNAME "data/ft_hg18/go/enrich_go.binom.hg18.txt"








#define MIN_COUNT 10
#define PVAL_THRESH 1e-3










void do_enrich_binomial( void )
{

  LIST_STRUCT my_list = { .total = NULL, .total_gc = NULL, 
    .all_and_all = NULL, .all_and_c = NULL, .a_and_c = NULL, 
    .a_and_no_c = NULL, .no_a_and_c = NULL, .a_no_c_and_c = NULL,
    .a_and_a_no_c = NULL, .c_no_a_and_c = NULL, .c_and_c = NULL,
    .node_c_and_c = NULL };
    
  get_enrich_binom_gene_slists( &my_list, SPEC_ONE, SPEC_TWO, GENE_LIST_BASE, 
    INPUT_FNAME, GO_CONDENSED_FNAME, BIRD_HUMAN_BASE );
    
  do_enrich_binom_gene_list_struct( &my_list, GO_GENE_FNAME, OUTPUT_FNAME );

}















void get_enrich_binom_gene_slists( LIST_STRUCT *my_list, char spec_one[],
  char spec_two[], char list_base[], char input_col_fname[], char go_gene_fname[], 
  char bird_hg_base[] )
{

  SLIST *get_total_genes_from_input_fname( SLIST *total_genes, char spec_one[],
    char spec_two[], char fname[] );
  SLIST *get_genes_w_motif_type( SLIST *gene_header, char fname[], char spec_one[],
    char spec_two[], char my_nucl );
  
  /*  In enrich.c:  */
  my_list->total = (SLIST *)get_total_genes_from_input_fname( my_list->total, 
    spec_one, spec_two, input_col_fname );
  
  /*  In total_enrich.c:  */
  my_list->total_gc = (SLIST *)get_genes_w_motif_type( my_list->total_gc,
    input_col_fname, spec_one, spec_two, C_NUCL );
    
  /*  In this file, below:  */
  get_my_enrich_binom_gene_slists( my_list, list_base );
  
  /*  In this file, below:  */
  set_enrich_two_node_fname_base( my_list, bird_hg_base );
  
}













void get_my_enrich_binom_gene_slists( LIST_STRUCT *my_list, char list_base[] )
{

  SLIST *get_gene_list_from_fnames( SLIST *header, char base[], char ext[] );
  SLIST *get_gene_list_complement( SLIST *header, char base[], char ext_yes[],
    char ext_no[] );

  /*  In sep_go.c:  */

  my_list->all_and_all = (SLIST *)get_gene_list_from_fnames( my_list->all_and_all, 
    list_base, "all.all.txt" );
    
  my_list->all_and_c = (SLIST *)get_gene_list_from_fnames( my_list->all_and_c,
    list_base, "C.all.txt" );
    
  my_list->a_and_c = (SLIST *)get_gene_list_from_fnames( my_list->a_and_c,
    list_base, "C.A.txt" );
    
  my_list->c_and_c = (SLIST *)get_gene_list_from_fnames( my_list->c_and_c,
    list_base, "C.C.txt" );
    
  my_list->a_and_no_c = (SLIST *)get_gene_list_complement( my_list->a_and_no_c,
    list_base, "all.A.txt", "C.A.txt" );
    
  my_list->no_a_and_c = (SLIST *)get_gene_list_complement( my_list->no_a_and_c,
    list_base, "C.all.txt", "C.A.txt" );
    
  my_list->a_no_c_and_c = (SLIST *)get_gene_list_complement( my_list->a_no_c_and_c,
    list_base, "C.A.txt", "C.C.txt" );
    
  my_list->c_no_a_and_c = (SLIST *)get_gene_list_complement( my_list->c_no_a_and_c,
    list_base, "C.C.txt", "C.A.txt" );
    
  my_list->a_and_a_no_c = (SLIST *)get_gene_list_complement( my_list->a_and_a_no_c,
    list_base, "A.A.txt", "C.A.txt" );

}
















void set_enrich_two_node_fname_base( LIST_STRUCT *my_list, char bird_hg_base[] )
{

  SLIST *get_gene_list_from_fnames( SLIST *header, char base[], char ext[] );
  
  /*  In sep_go.c:  */
  my_list->node_c_and_c = (SLIST *)get_gene_list_from_fnames( my_list->node_c_and_c,
    bird_hg_base, "C.C.txt" );

}
















/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/












void do_enrich_binom_gene_list_struct( LIST_STRUCT *my_list, char go_gene_fname[],
  char output_fname[] )
{

  SLIST *get_go_category_list_w_fname( SLIST *term_header, char fname[] );
  
  int i = 0;

  SLIST *term_header = NULL;
  /*  In enrich.c:  */
  term_header = (SLIST *)get_go_category_list_w_fname( term_header, 
    go_gene_fname );
    
  /*  In this file, below:  */
  do_enrich_binom_gene_list_w_term_list( term_header, my_list, go_gene_fname,
    output_fname );
    
}














void do_enrich_binom_gene_list_w_term_list( SLIST *term_header, LIST_STRUCT *my_list, 
  char go_gene_fname[], char output_fname[] )
{

  GO_CAT_LIST *go_header = NULL;
  go_header = (GO_CAT_LIST *)get_go_enrich_list_binom( go_header, my_list,
    term_header, go_gene_fname );
    
  do_enrich_binom_gene_list_w_go_list( go_header, my_list, go_gene_fname,
    output_fname );

}














GO_CAT_LIST *get_go_enrich_list_binom( GO_CAT_LIST *go_header, LIST_STRUCT *my_list,
  SLIST *term_header, char go_gene_fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( go_gene_fname, "" );
  
  go_header = (GO_CAT_LIST *)get_go_enrich_list_binom_fh( go_header, my_list,
    term_header, in );
    
  close_file( in );
  
  return go_header;

}













GO_CAT_LIST *get_go_enrich_list_binom_fh( GO_CAT_LIST *go_header, 
  LIST_STRUCT *my_list, SLIST *term_header, FILE *in )
{

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, in ) != NULL ) {
  
    if ( line[ 0 ] != '=' )
      error( 13000, "Expected line sep.\n" );
      
    go_header = (GO_CAT_LIST *)get_and_put_go_cat_lister_binom( go_header, my_list,
      term_header, in );
      
  }

  return go_header;

}













GO_CAT_LIST *get_and_put_go_cat_lister_binom( GO_CAT_LIST *go_header,
  LIST_STRUCT *my_list, SLIST *term_header, FILE *in )
{

  int str_in_my_slist( char str[], SLIST *header );
  GO_CAT_LIST *empty_entry_go_enrich( GO_CAT_LIST *go_header, FILE *in );

  char line[ BUFFER ], label[ BUFFER ];
  
  if ( fgets( line, BUFFER, in ) == NULL )
    error( 13001, "Unexpected EOF.\n" );
    
  sscanf( line, "%s", label );
  
  /*  In enrich.c:  */
  if ( ! str_in_my_slist( label, term_header ))
    /*  In enrich.c:  */
    return empty_entry_go_enrich( go_header, in );
    
  /*  In this file, below:  */
  return get_and_put_new_go_cat_lister_binom( go_header, my_list, label, in );

}













GO_CAT_LIST *get_and_put_new_go_cat_lister_binom( GO_CAT_LIST *go_header,
  LIST_STRUCT *my_list, char label[], FILE *in )
{

  SLIST *go_gene_header = NULL;
  go_gene_header = (SLIST *)get_go_gene_slist( go_gene_header, in );
  
  go_header = (GO_CAT_LIST *)get_and_put_my_new_go_cat_lister_binom( go_header,
    my_list, go_gene_header, label );
    
  kill_slist( go_gene_header );
  
  return go_header;

}













SLIST *get_go_gene_slist( SLIST *go_gene_header, FILE *in )
{

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, in ) != NULL ) {
  
    if ( line[ 0 ] == '=' )
      return go_gene_header;
      
    go_gene_header = (SLIST *)get_and_put_go_gene_slister( go_gene_header, line );
  
  }

}













SLIST *get_and_put_go_gene_slister( SLIST *go_gene_header, char line[] )
{

  SLIST *create_slister( SLIST *xx );
  
  SLIST *x = NULL;
  x = (SLIST *)create_slister( x );
  
  sscanf( line, "%s", x->str );
  
  x->next = go_gene_header;
  
  return x;

}
















GO_CAT_LIST *get_and_put_my_new_go_cat_lister_binom( GO_CAT_LIST *go_header,
  LIST_STRUCT *my_list, SLIST *go_gene_header, char label[] )
{

  return get_and_put_go_cat_lister_binom_w_slists( go_header, my_list->total_gc,
    go_gene_header, my_list->total, label );

}



















GO_CAT_LIST *get_and_put_go_cat_lister_binom_w_slists( GO_CAT_LIST *go_header,
  SLIST *gc_gene_header, SLIST *go_gene_header, SLIST *all_gene_header, 
  char go_label[] )
{

  int get_numb_elem_slist( SLIST *header );
  int get_intersection_slist_pair( SLIST *a_header, SLIST *b_header );
  PVAL get_hyper_geo_pval( int nn, int n, int gg, int g );
  
  /*  In sep_go.c:  */
  if ( get_numb_elem_slist( go_gene_header ) < MIN_COUNT )
    return go_header;

  /*  In sep_go.c:  */
  int g = get_intersection_slist_pair( gc_gene_header, go_gene_header );
  int nn = get_numb_elem_slist( all_gene_header );
  int n = get_numb_elem_slist( gc_gene_header );
  int gg = get_intersection_slist_pair( go_gene_header, all_gene_header );
  
  /*  In ../gene/sig.c:  */
  PVAL pval = get_hyper_geo_pval( nn, n, gg, g );
  
  printf( "%s :  %.2e  %d  %d  %d  %d\n", go_label, (double)pval,
    nn, n, gg, g );
  
  if ( pval >= PVAL_THRESH )
    return go_header;
    
  printf( "\t\t\t\t\t\t\t\t%.2e\n", (double)pval );

  /*  In this file, below:  */
  return get_and_put_my_go_cat_lister_binom( go_header, go_label, g, gg, n, nn,
    pval );

}














GO_CAT_LIST *get_and_put_my_go_cat_lister_binom( GO_CAT_LIST *go_header,
  char go_label[], int g, int gg, int n, int nn, PVAL pval )
{

  GO_CAT_LIST *create_go_cat_lister( GO_CAT_LIST *xx );
  PVAL get_hyper_geo_pval( int nn, int n, int gg, int g );
  
  GO_CAT_LIST *go = NULL;
  go = (GO_CAT_LIST *)create_go_cat_lister( go );
  
  strcpy( go->my_go->acc, go_label );
  strcpy( go->my_go->label, go_label );
  
  go->my_go->x_genes = g;
  go->my_go->total_genes = gg;
  
  /*  In ../gene/sig.c:  */
  go->my_go->pval = get_hyper_geo_pval( nn, n, gg, g );
  /*  In this file, below:  */
  go->my_go->pval2 = get_binomial_pvalue( g, n, (PVAL)gg / (PVAL)nn );
  go->my_go->pval3 = get_binomial_pvalue( g, gg, (PVAL)n / (PVAL)nn );
  
  go->my_go->exp_numb = (float)(n * gg) / (float)(nn);
  
  go->my_go->ratio = (float)(go->my_go->x_genes) / (go->my_go->exp_numb);
  
  return put_go_cat_into_list_binom( go_header, go );

}














GO_CAT_LIST *put_go_cat_into_list_binom( GO_CAT_LIST *go_header, GO_CAT_LIST *go )
{

  if ( go_header == NULL )
    return go;
    
  if ( go_header->my_go->pval > go->my_go->pval ) {
    go->next = go_header;
    return go;
  }
  
  GO_CAT_LIST *curr = go_header, *prev = NULL;
  
  while (( curr != NULL ) && ( curr->my_go->pval <= go->my_go->pval )) {
    prev = curr;
    curr = curr->next;
  }
  
  if ( prev != NULL )
    prev->next = go;
    
  if ( curr != NULL )
    go->next = curr;
    
  return go_header;

}














/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/













PVAL get_binomial_pvalue( int k, int n, PVAL pr )
{

  PVAL pval = 0.0;
  
  register int i;
  
  for ( i = k; i <= n; i++ )
    pval += exp( get_binomial_log_single_val( i, n, pr ) );
    
  return pval;

}













PVAL get_binomial_log_single_val( int k, int n, PVAL pr )
{

  PVAL get_log_factorial( int n, int k );
  
  /*  In ../gene/sig.c:  */
  PVAL ch = get_log_factorial( n, k );
  
  return ch + ( (PVAL)k * log( pr )) + ((PVAL)( n - k ) * ( log( 1.0 - pr )));

}















/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/

















void do_enrich_binom_gene_list_w_go_list( GO_CAT_LIST *go_header, 
  LIST_STRUCT *my_list, char go_gene_fname[], char output_fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *in = NULL, *out = NULL;
  in = (FILE *)open_text_file_reading( go_gene_fname, "" );
  out = (FILE *)open_text_file_writing( output_fname, "" );
  
  do_enrich_binom_gene_list_w_go_list_w_fh( go_header, my_list, in, out );
  
  close_file( in );
  close_file( out );

}















void do_enrich_binom_gene_list_w_go_list_w_fh( GO_CAT_LIST *go_header,
  LIST_STRUCT *my_list, FILE *in, FILE *out )
{

  GO_CAT_LIST *go = go_header;
  
  while ( go != NULL ) 
    do_enrich_binom_gene_list_single_go_lister_and_step( &go, my_list, in, out );

}














void do_enrich_binom_gene_list_single_go_lister_and_step( GO_CAT_LIST **go,
  LIST_STRUCT *my_list, FILE *in, FILE *out )
{
  do_enrich_binom_gene_list_single_go_lister( *go, my_list, in, out );
  *go = (*go)->next;
}













void do_enrich_binom_gene_list_single_go_lister( GO_CAT_LIST *go, 
  LIST_STRUCT *my_list, FILE *in, FILE *out )
{

  if ( go->my_go->pval >= PVAL_THRESH )
    return;
    
  do_my_enrich_binom_gene_list_single_go_lister( go, my_list, in, out );

}














void do_my_enrich_binom_gene_list_single_go_lister( GO_CAT_LIST *go, 
  LIST_STRUCT *my_list, FILE *in, FILE *out )
{

  if ( is_cellular_component( go->my_go->acc, GO_TERM_FNAME ))  
    return;

  SLIST *go_gene_header = NULL;
  go_gene_header = (SLIST *)get_go_gene_list_from_go_label( go_gene_header,
    go->my_go->acc, in );
  
  print_enrich_binom_single_go_lister_go_gene_list( go, my_list, go_gene_header, 
    out );
    
  kill_slist( go_gene_header );

}














SLIST *get_go_gene_list_from_go_label( SLIST *gene_header, char acc[], FILE *in )
{

  set_position( 0, in );

  char line[ BUFFER ], label[ BUFFER ];
  
  while ( fgets( line, BUFFER, in ) != NULL ) {
    sscanf( line, "%s", label );
    if ( ! strcmp( label, acc ))
      return get_go_gene_list_from_set_fh( gene_header, in );
  }
  
  error( 13010, "Did not find GO category label in file.\n" );

}













SLIST *get_go_gene_list_from_set_fh( SLIST *gene_header, FILE *in )
{

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, in ) != NULL ) {
  
    if ( line[ 0 ] == '=' )
      return gene_header;
      
    /*  In this file, above:  */  
    gene_header = (SLIST *)get_and_put_go_gene_slister( gene_header, line );
  
  }

}


















void print_enrich_binom_single_go_lister_go_gene_list( GO_CAT_LIST *go, 
  LIST_STRUCT *my_list, SLIST *go_gene_header, FILE *out )
{

  print_sep( out );

  print_enrich_binom_go_cat_header_entry( go, out );
  
  print_enrich_binom_separate_ratios( go, my_list, go_gene_header, out );
  
}













void print_enrich_binom_go_cat_header_entry( GO_CAT_LIST *go, FILE *out )
{

  fprintf( out, "%s\n", go->my_go->acc );

  char go_term[ LINE_BUFFER ];
  
  /*  In sep_go.c:  */  
  get_go_term_label_term_file( go_term, go->my_go->acc, GO_TERM_FNAME );
  fprintf( out, "%s\n", go_term );
  
  print_dot_sep( out );
  
  fprintf( out, "Pval :    %.3e\n", (float)go->my_go->pval );
  fprintf( out, "Genes :   %d\n", go->my_go->x_genes );
  fprintf( out, "Exp :     %.1f\n", go->my_go->exp_numb );
  fprintf( out, "Ratio :   %.3fx\n", go->my_go->ratio );
  fprintf( out, "Pval (Binom:Pr=GOgenes/N) :  %.3e\n", (float)go->my_go->pval2 );
  fprintf( out, "Pval (Binom:Pr=GCgenes/N) :  %.3e\n", (float)go->my_go->pval3 );
  
}














void print_enrich_binom_separate_ratios( GO_CAT_LIST *go, LIST_STRUCT *my_list, 
  SLIST *go_gene_header, FILE *out )
{

  print_enrich_binom_separate_ratios_total_gene_slist( go, my_list, go_gene_header,
    my_list->total, "All genes", out );
    
  print_enrich_binom_separate_ratios_total_gene_slist( go, my_list, go_gene_header,
    my_list->all_and_all, "Ancestral genes", out );
    
}
















void print_enrich_binom_separate_ratios_total_gene_slist( GO_CAT_LIST *go,
  LIST_STRUCT *my_list, SLIST *go_gene_header, SLIST *total_genes, char notes[],
  FILE *out )
{

  print_dot_sep( out );
  
  fprintf( out, "%s...\n", notes );
  fprintf( out, "\n" );

  fprintf( out, "%12s   %12s\n", "Ancestor", LEAF_LABEL );
  
  print_separate_slist_ratios( "All", "GC", my_list->all_and_c, go_gene_header,
    total_genes, out );

  print_separate_slist_ratios( "GA", "GC", my_list->a_and_c, go_gene_header,
    total_genes, out );

  print_separate_slist_ratios( "(No GA)", "GC", my_list->no_a_and_c, go_gene_header,
    total_genes, out );

  print_separate_slist_ratios( "GA", "(No GC)", my_list->a_and_no_c, go_gene_header,
    total_genes, out );

  print_separate_slist_ratios( "GA", "GA (No GC)", my_list->a_and_a_no_c, 
    go_gene_header, total_genes, out );

  print_separate_slist_ratios( "GC", "GC", my_list->c_and_c, go_gene_header,
    total_genes, out );

  print_separate_slist_ratios( "GC (No GA)", "GC", my_list->c_no_a_and_c, 
    go_gene_header, total_genes, out );

  print_separate_slist_ratios( "GA (No GC)", "GC", my_list->a_no_c_and_c, 
    go_gene_header, total_genes, out );
    
  print_separate_slist_ratios( "Bird GC", "GC", my_list->node_c_and_c, 
    go_gene_header, total_genes, out );
    
}












void print_separate_slist_ratios( char anc_notes[], char leaf_notes[],
  SLIST *gene_header, SLIST *go_header, SLIST *total_genes, FILE *out )
{

  int get_numb_elem_slist( SLIST *header );
  int get_intersection_slist_pair( SLIST *a_header, SLIST *b_header );

  /*  In sep_go.c:  */
  int a = get_numb_elem_slist( gene_header );
  int b = get_intersection_slist_pair( go_header, total_genes );
  int n = get_numb_elem_slist( total_genes );
  
  int x = get_intersection_slist_pair( gene_header, go_header );
  
  float exp = (float)(a * b) / (float)(n);
  
  fprintf( out, "%12s   %12s     Genes:  %5d   Exp:  %6.1f   Ratio:  %.3fx\n",
    anc_notes, leaf_notes, x, exp, (float)(x) / (float)(exp) );

}
  













int is_cellular_component( char acc[], char fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( fname, "" );
  
  int is = is_cellular_component_fh( acc, in );
  
  close_file( in );
  
  return is;

}












int is_cellular_component_fh( char acc[], FILE *in )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL ) {
  
    if ( ! strncmp( line, "id:", 3 )) {

      char term[ BUFFER ];

      sscanf( line, "%*s%s", term );

      if ( ! strcmp( term, acc )) {
      
        fgets( line, LINE_BUFFER, in );
        fgets( line, LINE_BUFFER, in );
        
        if ( strncmp( line, "namespace:", 10 )) {
          printf( "%s\n", line );
          error( 13020, "Did not find namespace string.\n" );
        }
        
        char type[ BUFFER ];
        sscanf( line, "%*s%s", type );
        
        return ( ! strcmp( type, "cellular_component" )) ? (1) : (0);
      
      }

    }
  
  }

}













