#include "enrich.h"













#define GENE_LIST_BASE "data/ft_hg18/evol/lists/taeGut1.Tetropoda.converted.sorted"
#define GO_GENE_FNAME "data/ft_hg18/go/go_genes.txt"
#define GO_TERM_FNAME "data/ft_hg19/go/go.terms.txt"
#define GO_CONDENSED_FNAME "data/ft_hg19/go/go.hg.condensed.txt"
#define OUTPUT_FNAME "data/ft_hg18/go/results_go_enrich.new.taeGut1.Tetropoda.txt"




#define COUNT_MIN 10

#define PVAL_THRESH 1e-3
















void do_gene_enrich( void )
{

  SLIST *term_header = NULL;
  term_header = (SLIST *)get_go_category_list_w_fname( term_header, GO_GENE_FNAME );
  
  do_gene_enrich_w_go_list( term_header, GENE_LIST_BASE, GO_GENE_FNAME, 
    GO_TERM_FNAME, OUTPUT_FNAME );

}















SLIST *get_go_category_list_w_fname( SLIST *term_header, char fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  SLIST *get_go_term_list_w_fh( SLIST *term_header, FILE *in );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( fname, "" );
  
  term_header = (SLIST *)get_go_category_list_w_fh( term_header, in );
  
  close_file( in );
  
  return term_header;

}














SLIST *get_go_category_list_w_fh( SLIST *term_header, FILE *in )
{

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, in ) != NULL )
    term_header = (SLIST *)get_go_category_single_entry( term_header, line, in );

  return term_header;

}













SLIST *get_go_category_single_entry( SLIST *term_header, char line[], FILE *in )
{

  if ( line[ 0 ] != '=' )
    error( 12701, "Expected line separator at beginning of entry.\n" );
    
  char my_line[ BUFFER ], term[ BUFFER ];
  
  fgets( my_line, BUFFER, in );
  
  sscanf( my_line, "%s", term );
  
  int numb_of_genes = get_numb_of_lines_until_sep( in );
  
  if ( numb_of_genes < COUNT_MIN )
    return term_header;
    
  return make_and_put_go_cat( term_header, term );

}












int get_numb_of_lines_until_sep( FILE *in )
{

  char line[ BUFFER ];

  int numb = 0;
  
  while ( fgets( line, BUFFER, in ) != NULL ) {
  
    if ( line[ 0 ] == '=' )
      return numb;
      
    ++numb;
  
  }
  
  error( 12703, "Unexpected EOF.\n" );

}














SLIST *make_and_put_go_cat( SLIST *term_header, char term[] )
{

  SLIST *create_slister( SLIST *xx );
  
  SLIST *x = NULL;
  x = (SLIST *)create_slister( x );
  
  strcpy( x->str, term );
  
  if ( term_header == NULL )
    return x;
    
  SLIST *curr = term_header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = x;
  
  return term_header;

}











/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/















void do_gene_enrich_w_go_list( SLIST *term_header, char list_base[],
  char go_gene_fname[], char go_term_fname[], char output_fname[] )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( output_fname, "" );
  
  do_gene_enrich_w_fh( term_header, list_base, go_gene_fname, go_term_fname, out );
  
  close_file( out );
  
}















void do_gene_enrich_w_fh( SLIST *term_header, char list_base[], char go_gene_fname[],
  char go_term_fname[], FILE *out )
{

  MY_LIST my_list = { .total = NULL, .all_and_all = NULL, .all_and_c = NULL, 
    .a_and_c = NULL, .a_and_no_c = NULL, .no_a_and_c = NULL, .a_no_c_and_c = NULL, 
    .c_no_a_and_c = NULL, .c_and_c = NULL };
    
  get_my_list_gene_enrich( &my_list, list_base );
  
  do_gene_enrich_w_gene_lists( &my_list, term_header, go_gene_fname, go_term_fname, 
    out );

}














void get_my_list_gene_enrich( MY_LIST *my_list, char list_base[] )
{

  SLIST *get_gene_list_from_fnames( SLIST *header, char base[], char ext[] );
  SLIST *get_gene_list_complement( SLIST *header, char base[], char ext_yes[],
    char ext_no[] );

  /*  In this file, below:  */
  my_list->total = (SLIST *)get_total_gene_list( my_list->total, 
    GO_CONDENSED_FNAME );
  
  /*  In sep_go.c:  */
  
  my_list->all_and_all = (SLIST *)get_gene_list_from_fnames( my_list->all_and_all, 
    list_base, "all.all.txt" );
    
  my_list->all_and_c = (SLIST *)get_gene_list_from_fnames( my_list->all_and_c,
    list_base, "C.all.txt" );
    
  my_list->a_and_c = (SLIST *)get_gene_list_from_fnames( my_list->a_and_c,
    list_base, "C.A.txt" );
    
  my_list->c_and_c = (SLIST *)get_gene_list_from_fnames( my_list->c_and_c,
    list_base, "C.C.txt" );
    
  my_list->a_and_no_c = (SLIST *)get_gene_list_complement( my_list->a_and_no_c,
    list_base, "all.A.txt", "C.A.txt" );
    
  my_list->no_a_and_c = (SLIST *)get_gene_list_complement( my_list->no_a_and_c,
    list_base, "C.all.txt", "C.A.txt" );
    
  my_list->a_no_c_and_c = (SLIST *)get_gene_list_complement( my_list->a_no_c_and_c,
    list_base, "C.A.txt", "C.C.txt" );
    
  my_list->c_no_a_and_c = (SLIST *)get_gene_list_complement( my_list->c_no_a_and_c,
    list_base, "C.C.txt", "C.A.txt" );
    
  my_list->a_and_a_no_c = (SLIST *)get_gene_list_complement( my_list->a_and_a_no_c,
    list_base, "A.A.txt", "C.A.txt" );
    
  print_slist( my_list->total );

}














SLIST *get_total_gene_list( SLIST *header, char fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( fname, "" );
  
  header = (SLIST *)get_total_gene_list_fh( header, in );
  
  close_file( in );
  
  return header;
  
}










SLIST *get_total_gene_list_fh( SLIST *header, FILE *in )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL )
    header = (SLIST *)get_total_gene_list_my_single_line( header, line );
    
  return header;

}











SLIST *get_total_gene_list_my_single_line( SLIST *header, char line[] )
{

  SLIST *create_slister( SLIST *xx );

  char gene[ BUFFER ];
  
  sscanf( line, "%s", gene );
  
  if ( str_in_my_slist( gene, header ))
    return header;
    
  SLIST *x = NULL;
  x = (SLIST *)create_slister( x );
  
  strcpy( x->str, gene );
  
  if ( header == NULL )
    return x;
    
  SLIST *curr = header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = x;
  
  return header;

}















void do_gene_enrich_w_gene_lists( MY_LIST *my_list, SLIST *term_header,
  char go_gene_fname[], char go_term_fname[], FILE *out )
{

  GO_CAT_LIST *go_header = NULL;
  go_header = (GO_CAT_LIST *)get_go_enrich_list( go_header, my_list, term_header,
    go_gene_fname );
    
  do_gene_enrich_w_go_category_list( go_header, my_list, go_gene_fname,
    go_term_fname, out );

}















GO_CAT_LIST *get_go_enrich_list( GO_CAT_LIST *go_header, MY_LIST *my_list,
  SLIST *term_header, char go_gene_fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( go_gene_fname, "" );
  
  go_header = (GO_CAT_LIST *)get_go_enrich_list_w_fh( go_header, my_list,
    term_header, in );
    
  close_file( in );
  
  return go_header;
  
}















GO_CAT_LIST *get_go_enrich_list_w_fh( GO_CAT_LIST *go_header, MY_LIST *my_list,
  SLIST *term_header, FILE *in )
{

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, in ) != NULL )
    go_header = (GO_CAT_LIST *)get_go_enrich_list_single_entry( go_header, my_list,
      term_header, line[ 0 ], in );
      
  return go_header;

}












GO_CAT_LIST *get_go_enrich_list_single_entry( GO_CAT_LIST *go_header, 
  MY_LIST *my_list, SLIST *term_header, char x, FILE *in )
{

  if ( x != '=' )
    error( 12705, "Expected separator.\n" );
    
  char line[ BUFFER ], term[ BUFFER ];  
    
  fgets( line, BUFFER, in );
  
  sscanf( line, "%s", term );
  
  if ( ! str_in_my_slist( term, term_header ))
    return empty_entry_go_enrich( go_header, in );
    
  return get_my_go_enrich_list_single_entry( go_header, term, my_list, in );

}













int str_in_my_slist( char str[], SLIST *header )
{

  SLIST *x = header;
  
  while ( x != NULL ) {
  
    if ( ! strcmp( str, x->str ))
      return 1;
      
    x = x->next;
  
  }
  
  return 0;

}














GO_CAT_LIST *empty_entry_go_enrich( GO_CAT_LIST *go_header, FILE *in )
{

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, in ) != NULL ) {
  
    if ( line[ 0 ] == '=' )
      return go_header;
  
  }
  
  error( 1207, "Unexpected EOF.\n" );

}













GO_CAT_LIST *get_my_go_enrich_list_single_entry( GO_CAT_LIST *go_header, char term[], 
  MY_LIST *my_list, FILE *in )
{

  SLIST *get_go_gene_list_correct_entry( SLIST *go_gene_header, FILE *in );

  GO_CAT_LIST *go = NULL;

  SLIST *go_gene_header = NULL;
  /*  In sep_go.c:  */
  go_gene_header = (SLIST *)get_go_gene_list_correct_entry( go_gene_header, in );
  
  /*  In this file, below:  */
  go = (GO_CAT_LIST *)make_and_put_go_enrich_w_go_gene_slist( go, my_list, term, 
    go_gene_header );
    
  kill_slist( go_gene_header );
  
  if ( go == NULL )
    return go_header;
  
  /*  In this file, below:  */
  return put_go_cat_lister_into_sorted_list( go_header, go );

}












GO_CAT_LIST *make_and_put_go_enrich_w_go_gene_slist( GO_CAT_LIST *go, 
  MY_LIST *my_list, char term[], SLIST *go_gene_header )
{

  GO_CAT_LIST *create_go_cat_lister( GO_CAT_LIST *xx );
  int get_numb_elem_slist( SLIST *header );
  int get_intersection_slist_pair( SLIST *a_header, SLIST *b_header );
  PVAL get_hyper_geo_pval( int nn, int n, int gg, int g );

  /*  In go_genes.c:  */
  int nn = get_numb_elem_slist( my_list->total );
  int n = get_intersection_slist_pair( my_list->total, go_gene_header );
  int gg = get_numb_elem_slist( my_list->all_and_c );
  /*  In this file, below:  */
  int g = get_intersection_slist_pair( go_gene_header, my_list->all_and_c );
  
  static int i = 0;
  
  printf( "%d :  %s\n", ++i, term );
  
  if ( g < COUNT_MIN )
    return NULL;
    
  go = (GO_CAT_LIST *)create_go_cat_lister( go );
  
  strcpy( go->my_go->acc, term );
  strcpy( go->my_go->label, term );
  
  /*  In ../gene/sig.c:  */
  go->my_go->pval = get_hyper_geo_pval( nn, gg, n, g );
  
  if ( go->my_go->pval < 1e-4 )
    printf( "\t\t\t\t\t\tpval :  %.3e\n", (float)go->my_go->pval );
    
  go->my_go->x_genes = g;
  go->my_go->exp_numb = (float)(n * gg) / (float)(nn);
  go->my_go->ratio = (float)go->my_go->x_genes / (float)go->my_go->exp_numb;

  if ( go->my_go->ratio > 1.2 )
    printf( "\t\t\t\t%.3f\n", go->my_go->ratio );

  return go;

}













int get_intersection_three_slists( SLIST *a_header, SLIST *b_header, 
  SLIST *c_header )
{

  int x = 0;
  
  SLIST *a = a_header;
  
  while ( a != NULL ) {
  
    if (( str_in_my_slist( a->str, b_header )) &&
      ( str_in_my_slist( a->str, c_header )))
    {
      ++x;
    }
    
    a = a->next;
  
  }
  
  return x;

}














GO_CAT_LIST *put_go_cat_lister_into_sorted_list( GO_CAT_LIST *go_header, 
  GO_CAT_LIST *go )
{

  if ( go_header == NULL )
    return go;
    
  if ( go == NULL )
    return go_header;
    
  if ( go_header->my_go->pval > go->my_go->pval ) {
    go->next = go_header;
    return go;
  }  
    
  GO_CAT_LIST *prev = NULL, *curr = go_header;
  
  while (( curr != NULL ) && ( curr->my_go->pval <= go->my_go->pval )) {
    prev = curr;
    curr = curr->next;
  }
  
  if ( prev != NULL )
    prev->next = go;
    
  if ( curr != NULL )
    go->next = curr;
    
  return go_header;

}









/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/












void do_gene_enrich_w_go_category_list( GO_CAT_LIST *go_header, MY_LIST *my_list, 
  char go_gene_fname[], char go_term_fname[], FILE *out )
{

  GO_CAT_LIST *go = go_header;
  
  while ( go != NULL )
    do_gene_enrich_go_cat_list_single_go_and_step( &go, my_list, go_gene_fname,
      go_term_fname, out );

}













void do_gene_enrich_go_cat_list_single_go_and_step( GO_CAT_LIST **go,
  MY_LIST *my_list, char go_gene_fname[], char go_term_fname[], FILE *out )
{
  do_gene_enrich_go_cat_list_single_go( *go, my_list, go_gene_fname, 
    go_term_fname, out );
  *go = (*go)->next;
}













void do_gene_enrich_go_cat_list_single_go( GO_CAT_LIST *go, MY_LIST *my_list,
  char go_gene_fname[], char go_term_fname[], FILE *out )
{

  if ( go->my_go->pval > PVAL_THRESH )
    return;
    
  do_my_gene_enrich_go_cat_list_single_go( go, my_list, go_gene_fname, 
    go_term_fname, out );

}












void do_my_gene_enrich_go_cat_list_single_go( GO_CAT_LIST *go, MY_LIST *my_list,
  char go_gene_fname[], char go_term_fname[], FILE *out )
{

  char go_term[ LINE_BUFFER ];
  
  /*  In sep_go.c:  */  
  get_go_term_label_term_file( go_term, go->my_go->label, go_term_fname );
  
  print_sep( out );
  fprintf( out, "%s\n", go->my_go->label );
  fprintf( out, "%s\n", go_term );
  
  print_dot_sep( out );
  
  /*  In this file, below:  */
  do_gene_enrich_single_go_cat( go, my_list, go_gene_fname, out );
  
  print_sep( out );

}














void do_gene_enrich_single_go_cat( GO_CAT_LIST *go, MY_LIST *my_list,
  char go_gene_fname[], FILE *out )
{

  SLIST *get_go_gene_list_from_file( SLIST *go_gene_header, GO_CAT_LIST *go, 
    char fname[] );

  SLIST *go_gene_header = NULL;
  /*  In sep_go.c:  */
  go_gene_header = (SLIST *)get_go_gene_list_from_file( go_gene_header, go, 
    go_gene_fname );
    
  /*  In this file, below:  */
  do_gene_enrich_single_go_cat_w_gene_list( go, my_list, go_gene_header, out );  
    
  kill_slist( go_gene_header );

}














void do_gene_enrich_single_go_cat_w_gene_list( GO_CAT_LIST *go, MY_LIST *my_list,
  SLIST *go_gene_header, FILE *out )
{

  print_gene_enrich_go_cat_lister( go, out );
  
  print_dot_sep( out );
  
  do_my_enrich_single_go_cat_w_gene_list( go, my_list, my_list->total, 
    "All genes", go_gene_header, out );

  do_my_enrich_single_go_cat_w_gene_list( go, my_list, my_list->all_and_all, 
    "All existing genes", go_gene_header, out );

}














void print_gene_enrich_go_cat_lister( GO_CAT_LIST *go, FILE *out )
{

  fprintf( out, "P-value :      %.2e\n", (float)go->my_go->pval );
  fprintf( out, "Total genes :  %d\n", go->my_go->x_genes );
  fprintf( out, "Exp genes :    %.2f\n", go->my_go->exp_numb );
  fprintf( out, "Ratio :        %.3f\n", go->my_go->ratio );

}













void do_my_enrich_single_go_cat_w_gene_list( GO_CAT_LIST *go, MY_LIST *my_list, 
  SLIST *all_header, char x[], SLIST *go_gene_header, FILE *out )
{

  fprintf( out, "\n" );
  fprintf( out, "%s...\n", x );

  print_enrich_single_go_cat_single_list( "All -> GC", my_list->all_and_c,
    all_header, go_gene_header, out );

  print_enrich_single_go_cat_single_list( "GA -> GC", my_list->a_and_c,
    all_header, go_gene_header, out );

  print_enrich_single_go_cat_single_list( "(No GA) -> GC", my_list->no_a_and_c,
    all_header, go_gene_header, out );

  print_enrich_single_go_cat_single_list( "GA -> (No GC)", my_list->a_and_no_c,
    all_header, go_gene_header, out );

  print_enrich_single_go_cat_single_list( "GC (No GA) -> GC", my_list->c_no_a_and_c,
    all_header, go_gene_header, out );

  print_enrich_single_go_cat_single_list( "GA -> GA (No GC)", my_list->a_and_a_no_c,
    all_header, go_gene_header, out );

  print_enrich_single_go_cat_single_list( "GC -> GC", my_list->c_and_c,
    all_header, go_gene_header, out );

  print_enrich_single_go_cat_single_list( "GA (No GC) -> GC", my_list->a_no_c_and_c,
    all_header, go_gene_header, out );
    
}












void print_enrich_single_go_cat_single_list( char x[], SLIST *gene_header,
  SLIST *all_header, SLIST *go_gene_header, FILE *out )
{

  int get_numb_elem_slist( SLIST *header );
  int get_intersection_slist_pair( SLIST *a_header, SLIST *b_header );
  
  /*  In sep_go.c:  */
  int nn = get_numb_elem_slist( all_header );
  int n = get_numb_elem_slist( gene_header );
  int gg = get_intersection_slist_pair( go_gene_header, all_header );
  int g = get_intersection_slist_pair( gene_header, go_gene_header );

  
  fprintf( out, "%20s\t", x );
  
  float exp = (float)(n * gg) / (float)(nn);
  
  fprintf( out, "Obs :  %5d\tExp :  %5.2f\tRatio :  %.3f\n",
    g, exp, (float)g / exp );

}




























