#include "ft_methyl.h"













#define FASTA_FNAME "data/ft_hg18/hg18.refSeq.500.100.Mar2006.txt.final.txt"
#define GENE_FNAME "data/ft_hg18/hg18.gene.info.txt"

#define FNAME_DIR "data/ft_hg18/methyl/methylSeq"

#define SEQ_LENGTH 600
#define UPSTREAM_DIST 370
#define DOWNSTREAM_DIST 470

#define REGION_BUFFER 10


#define CPG_FNAME "data/ft_hg18/cpg/data/hg18.CpG.total.txt"
#define CPG_OUTPUT "data/ft_hg18/cpg/data/hg18.CpG.geneList.txt"











void do_format_methyl( void )
{
  do_format_methyl_single_cell_line( "BG02ES", FNAME_DIR, FASTA_FNAME );
  do_format_methyl_single_cell_line( "H1hESC", FNAME_DIR, FASTA_FNAME );
  do_format_methyl_single_cell_line( "HAL", FNAME_DIR, FASTA_FNAME );
}















void do_format_methyl_single_cell_line( char cell_line[], char dir[], 
  char fasta_fname[] )
{

  SLIST *chr_header = NULL;
  chr_header = (SLIST *)get_chr_list_methyl_all_file( chr_header, cell_line, dir );
  
  print_slist( chr_header );
  
  do_format_methyl_single_cell_line_chr_list( chr_header, cell_line, dir, 
    fasta_fname );
  
}















SLIST *get_chr_list_methyl_all_file( SLIST *chr_header, char cell_line[], 
  char dir[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  char fname[ BUFFER ];
  
  sprintf( fname, "%s/%s/hg18.methylSeq.%s", dir, cell_line, cell_line );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( fname, "all.txt" );
  
  chr_header = (SLIST *)get_chr_list_methyl_all_fh( chr_header, in );
  
  close_file( in );

  return chr_header;

}

















SLIST *get_chr_list_methyl_all_fh( SLIST *chr_header, FILE *in )
{

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, in ) != NULL )
    chr_header = (SLIST *)get_chr_list_methyl_single_line( chr_header, line );
    
  return chr_header;

}














SLIST *get_chr_list_methyl_single_line( SLIST *chr_header, char line[] )
{

  if ( line[ 0 ] == '#' )
    return chr_header;
    
  return get_my_chr_list_methyl_single_line( chr_header, line );

}














SLIST *get_my_chr_list_methyl_single_line( SLIST *chr_header, char line[] )
{

  char chr[ BUFFER ];
  
  sscanf( line, "%s", chr );
  
  if ( ! str_in_slist( chr_header, chr ))
    return make_and_put_new_slister_end_of_list( chr_header, chr );
    
  return chr_header;

}














int str_in_slist( SLIST *header, char str[] )
{

  SLIST *x = header;
  
  while ( x != NULL ) {
  
    if ( ! strcmp( x->str, str ))
      return 1;
      
    x = x->next;
  
  }
  
  return 0;

}














SLIST *make_and_put_new_slister_end_of_list( SLIST *chr_header, char chr[] )
{

  SLIST *create_slister( SLIST *xx );
  
  SLIST *x = NULL;
  x = (SLIST *)create_slister( x );
  
  strcpy( x->str, chr );
  
  if ( chr_header == NULL )
    return x;
    
  SLIST *curr = chr_header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = x;
  
  return chr_header;

}















void print_slist( SLIST *header )
{

  print_sep( stdout );
  
  SLIST *x = header;
  
  while ( x != NULL ) {
    printf( "%s\n", x->str );
    x = x->next;
  }
  
  print_sep( stdout );

}











/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/














void do_format_methyl_single_cell_line_chr_list( SLIST *chr_header, char cell_line[],
  char dir[], char fasta_fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );
  
  char output_fname[ BUFFER ];
  
  sprintf( output_fname, "%s/%s/hg18.methylSeq.%s.geneList.txt",
    dir, cell_line, cell_line );
  
  FILE *fasta_fh = NULL, *out = NULL;
  fasta_fh = (FILE *)open_text_file_reading( fasta_fname, "" );
  out = (FILE *)open_text_file_writing( output_fname, "" );
  
  do_format_methyl_single_cell_line_w_fhs( chr_header, cell_line, dir, fasta_fh,
    out );
  
  close_file( fasta_fh );
  close_file( out );
  
}














void do_format_methyl_single_cell_line_w_fhs( SLIST *chr_header, char cell_line[],
  char dir[], FILE *fasta_fh, FILE *out )
{

  int seq_index = 0;

  SLIST *chr = chr_header;
  
  while ( chr != NULL )
    do_format_methyl_single_chr_and_step( &chr, cell_line, dir, &seq_index, fasta_fh, 
      out );

}














void do_format_methyl_single_chr_and_step( SLIST **chr, char cell_line[], 
  char dir[], int *seq_index, FILE *fasta_fh, FILE *out )
{
  do_format_methyl_single_chr( *chr, cell_line, dir, seq_index, fasta_fh, out );
  *chr = (*chr)->next;
}













void do_format_methyl_single_chr( SLIST *chr, char cell_line[], char dir[],
  int *seq_index, FILE *fasta_fh, FILE *out )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  char methyl_fname[ BUFFER ];
  
  sprintf( methyl_fname, "%s/%s/hg18.methylSeq.%s.%s.txt",
    dir, cell_line, cell_line, chr->str );
  
  FILE *methyl_fh = NULL;
  methyl_fh = (FILE *)open_text_file_reading( methyl_fname, "" );
  
  do_format_methyl_single_chr_w_fh( chr, seq_index, methyl_fh, fasta_fh, out );
  
  close_file( methyl_fh );

}














void do_format_methyl_single_chr_w_fh( SLIST *chr, int *seq_index, FILE *methyl_fh, 
  FILE *fasta_fh, FILE *out )
{

  int get_fasta_seq_and_gene_label( GENCOORD_ENTRY *my_entry, char seq[], FILE *in );

  set_position( 0, fasta_fh );
  
  GENCOORD_ENTRY my_entry;
  char seq[ LINE_BUFFER ];
  
  int seq_numb = 0;
  
  seq[ 0 ] = '\0';
  
  /*  In methyl.c:  */
  while ( get_fasta_seq_and_gene_label( &my_entry, seq, fasta_fh )) {
  
    /*  In this file, below:  */
    do_format_methyl_single_gene( &my_entry, chr, ++seq_numb, seq_index, methyl_fh, 
      out );  
    
    seq[ 0 ] = '\0';
    
  }

}














void do_format_methyl_single_gene( GENCOORD_ENTRY *my_entry, SLIST *chr,
  int seq_numb, int *seq_index, FILE *methyl_fh, FILE *out )
{

  if ( strcmp( my_entry->chr, chr->str ))
    return;
    
  int prom_start, prom_end;
  
  switch ( my_entry->strand ) {
  
  case '+' :
    prom_start = my_entry->start + UPSTREAM_DIST;
    prom_end = my_entry->start + DOWNSTREAM_DIST;
    break;
    
  case '-' :
    prom_start = my_entry->end - DOWNSTREAM_DIST;
    prom_end = my_entry->end - UPSTREAM_DIST;
    break;
    
  default :
    error( 12600, "Unrecognized strand type.\n" );
    break;
  
  }
  
  do_my_format_methyl_single_gene( my_entry, prom_start, prom_end, seq_numb,
    seq_index, methyl_fh, out );

}













void do_my_format_methyl_single_gene( GENCOORD_ENTRY *my_entry, int prom_start, 
  int prom_end, int seq_numb, int *seq_index, FILE *methyl_fh, FILE *out )
{

  set_position( 0, methyl_fh );

  int found = 0;
  char line[ LINE_BUFFER ];
  
  while (( ! found ) && ( fgets( line, LINE_BUFFER, methyl_fh ) != NULL ))
    found = do_format_methyl_single_methyl_region( my_entry, line, prom_start, 
      prom_end, seq_numb, seq_index, out );
      
  if ( found )
    printf( "\t\t\t\t\t%s\n", my_entry->gene );
    
}













int do_format_methyl_single_methyl_region( GENCOORD_ENTRY *my_entry, char line[], 
  int prom_start, int prom_end, int seq_numb, int *seq_index, FILE *out )
{

  if ( line[ 0 ] == '#' )
    return 0;
    
  int start, end, score;
  
  sscanf( line, "%*s%d%d%*s%d", &start, &end, &score );

  if (( prom_start + REGION_BUFFER > end ) || ( prom_end - REGION_BUFFER < start ))
    return 0;
    
  char my_gene[ BUFFER ];
  get_gene_from_acc_gene_info_file( my_gene, my_entry->gene, GENE_FNAME );
    
  fprintf( out, "%d\t%d\t%s\t%s\t%d\n", ++(*seq_index), seq_numb, my_entry->gene, 
    my_gene, score );
  
  return 1;

}














void get_gene_from_acc_gene_info_file( char my_gene[], char acc[], char fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( fname, "" );
  
  get_gene_from_acc_gene_info_fh( my_gene, acc, in );
  
  close_file( in );

}












void get_gene_from_acc_gene_info_fh( char my_gene[], char acc[], FILE *in )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL ) {
  
    if ( line[ 0 ] == '#' )
      continue;
      
    char my_acc[ BUFFER ];
    
    sscanf( line, "%s%*s%*s%*d%*d%s", my_acc, my_gene );
    
    if ( ! strcmp( acc, my_acc ))
      return;
  
  }
  
  strcmp( my_gene, "---" );

}















/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/














void do_cpg_island_format( void )
{

  FILE *open_text_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *fasta_fh = NULL, *cpg_fh = NULL, *out = NULL;
  fasta_fh = (FILE *)open_text_file_reading( FASTA_FNAME, "" );
  cpg_fh = (FILE *)open_text_file_reading( CPG_FNAME, "" );
  out = (FILE *)open_text_file_writing( CPG_OUTPUT, "" );
  
  do_cpg_island_format_w_fhs( cpg_fh, fasta_fh, out );
  
  close_file( fasta_fh );
  close_file( cpg_fh );
  close_file( out );

}













void do_cpg_island_format_w_fhs( FILE *cpg_fh, FILE *fasta_fh, FILE *out )
{

  int get_fasta_seq_and_gene_label( GENCOORD_ENTRY *my_entry, char seq[], FILE *in );

  set_position( 0, fasta_fh );
  
  GENCOORD_ENTRY my_entry;
  char seq[ LINE_BUFFER ];
  
  int seq_numb = 0, seq_index = 0;
  
  seq[ 0 ] = '\0';
  
  /*  In methyl.c:  */
  while ( get_fasta_seq_and_gene_label( &my_entry, seq, fasta_fh )) {
  
    /*  In this file, below:  */
    do_cpg_island_format_single_gene( &my_entry, ++seq_numb, &seq_index,
      cpg_fh, out );
      
    seq[ 0 ] = '\0';
    
  }
  
}














void do_cpg_island_format_single_gene( GENCOORD_ENTRY *my_entry, int seq_numb,
  int *seq_index, FILE *cpg_fh, FILE *out )
{

  /*  In this file, below:  */
  if ( gene_is_in_cpg_island( my_entry, cpg_fh )) {

    char my_gene[ BUFFER ];
    /*  In this file, above:  */
    get_gene_from_acc_gene_info_file( my_gene, my_entry->gene, GENE_FNAME );

    fprintf( out, "%d\t%d\t%s\t%s\n", ++(*seq_index), seq_numb, my_entry->gene,
      my_gene );
    
  }

}












int gene_is_in_cpg_island( GENCOORD_ENTRY *my_entry, FILE *cpg_fh )
{

  set_position( 0, cpg_fh );
  
  int in_chr = 0;
  
  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, cpg_fh ) != NULL ) {
  
    if ( line[ 0 ] == '#' )
      continue;
  
    char chr[ BUFFER ];
    
    sscanf( line, "%s", chr );
  
    if ( strcmp( chr, my_entry->chr )) {
      if ( in_chr )
        return 0;
      continue;
    }
    
    in_chr = 1;
    
    if ( cpg_island_line_in_promoter( line, my_entry ))
      return 1;
  
  }

  return 0;

}













int cpg_island_line_in_promoter( char line[], GENCOORD_ENTRY *my_entry )
{

  int prom_start, prom_end;

  switch ( my_entry->strand ) {
  
  case '+' :
    prom_start = my_entry->start + UPSTREAM_DIST;
    prom_end = my_entry->start + DOWNSTREAM_DIST;
    break;
    
  case '-' :
    prom_start = my_entry->end - DOWNSTREAM_DIST;
    prom_end = my_entry->end - UPSTREAM_DIST;
    break;
    
  default :
    error( 12640, "Unrecognized strand type.\n" );
    break;
  
  }
  
  return cpg_island_in_start_end_region( line, prom_start, prom_end );

}












int cpg_island_in_start_end_region( char line[], int prom_start, int prom_end )
{

  int start, end;
  
  sscanf( line, "%*s%d%d", &start, &end );
  
  return (( prom_start > end ) || ( prom_end < start )) ? (0) : (1);

}























