#include "methyl.h"














#define METHYL_FNAME_BASE "data/ft_hg18/methyl/hg18.methyl.GM12878"
#define FASTA_FNAME "data/ft_hg18/hg18.refSeq.500.100.Mar2006.txt.final.txt"
#define CONV_FNAME_BASE "data/ft_hg18/evol/lists/hg18.Mammalia.converted.sorted"

#define OUTPUT_FNAME "data/ft_hg18/evol/methylation_results.txt"

#define METHYL_SCORE_THRESH 200

#define SEQ_STEP 500

#define SEQ_LENGTH 600

#define FOR_MOTIF "gggcgg"
#define REV_MOTIF "ccgccc"

#define MF_LENGTH 6

#define START_REGION 370
#define REGION_WIDTH 100
#define END_REGION ((START_REGION) + (REGION_WIDTH))
#define REGION_BUFFER 0













void do_methylation( void )
{

  condense_methylation_files( METHYL_FNAME_BASE );
  
  do_methylation_w_file( METHYL_FNAME_BASE );

}














void condense_methylation_files( char fname_base[] )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( fname_base, "total.txt" );
  
  condense_my_methylation_files( fname_base, out );
  
  close_file( out );
  
  printf( "Methylation files have been condensed...\n\n" );

}














void condense_my_methylation_files( char fname_base[], FILE *out )
{

  int numb = 0;

  while ( 1 ) {
  
    char fname[ BUFFER ];
    sprintf( fname, "%s.%d.txt", fname_base, numb );
  
    FILE *in = NULL;
    in = (FILE *)fopen( fname, "r" );
    
    if ( in == NULL )
      return;
      
    condense_methylation_single_file( numb, in, out );
    
    numb += SEQ_STEP;
    
    close_file( in );
  
  }

}













void condense_methylation_single_file( int numb, FILE *in, FILE *out )
{

  static int i = 0;

  char line[ LINE_BUFFER ];
  
  if ( fgets( line, LINE_BUFFER, in ) == NULL )
    error( 11200, "Unexpected EOF in methylation file.\n" );
    
  if ( ! numb )
    fprintf( out, "%s", line );
    
  while ( fgets( line, LINE_BUFFER, in ) != NULL ) {
    printf( "methyl :  %d\n", ++i );
    fprintf( out, "%s", line );
  }
  
}















/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
















void do_methylation_w_file( char fname_base[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );
  
  FH my_fh = { .fasta = NULL, .methyl_fh = NULL, .all_genes_fh = NULL,
    .conv_fh = NULL, .no_conv_fh = NULL, .out = NULL };
  
  my_fh.fasta = (FILE *)open_text_file_reading( FASTA_FNAME, "" );
  my_fh.methyl_fh = (FILE *)open_text_file_reading( fname_base, "total.txt" );
  my_fh.all_genes_fh = 
    (FILE *)open_text_file_reading( CONV_FNAME_BASE, "C.all.txt" );
  my_fh.conv_fh = 
    (FILE *)open_text_file_reading( CONV_FNAME_BASE, "C.A.txt" );
  my_fh.no_conv_fh = 
    (FILE *)open_text_file_reading( CONV_FNAME_BASE, "C.noA.txt" );
  my_fh.out = (FILE *)open_text_file_writing( OUTPUT_FNAME, "" );
  
  INFO my_info;
  
  initialize_methyl_info( &my_info );
  
  do_methyl( &my_info, &my_fh );
  
//  do_methylation_w_total_fh( &my_info, &my_fh );
  
  close_file( my_fh.fasta );
  close_file( my_fh.methyl_fh );
  close_file( my_fh.all_genes_fh );
  close_file( my_fh.conv_fh );
  close_file( my_fh.no_conv_fh );
  close_file( my_fh.out );

}













void initialize_methyl_info( INFO *my_info )
{
  my_info->total_gc = 0;
  my_info->total_methyl = 0;
  my_info->total_gc_box = 0;
  my_info->conv = 0;
  my_info->conv_hits = 0;
  my_info->non_conv = 0;
  my_info->non_conv_hits = 0;
}












void do_methyl( INFO *my_info, FH *my_fh )
{

  GG gg_all, gg_conv, gg_no_conv;

  get_gg_from_conversion_line( &gg_conv, my_fh->conv_fh );
  get_gg_from_conversion_line( &gg_no_conv, my_fh->no_conv_fh );
  
  while ( get_gg_from_conversion_line( &gg_all, my_fh->all_genes_fh ) ) 
    do_methyl_single_conversion_line( my_info, &gg_all, &gg_conv, &gg_no_conv,
      my_fh );

  print_methylation_results( my_info, my_fh->out );

}













void do_methyl_single_conversion_line( INFO *my_info, GG *gg_all, GG *gg_conv,
  GG *gg_no_conv, FH *my_fh )
{

  GENCOORD_ENTRY my_entry;
  char seq[ LINE_BUFFER ];

  do {
    seq[ 0 ] = '\0';
    get_fasta_seq_and_gene_label( &my_entry, seq, my_fh->fasta );
    do_methyl_single_gene( my_info, &my_entry, seq, my_fh );
  } while ( strcmp( my_entry.gene, gg_all->acc ));

  update_methyl_matching_seq( my_info, &my_entry, seq, gg_all, gg_conv, gg_no_conv, 
    my_fh );

}














void do_methyl_single_gene( INFO *my_info, GENCOORD_ENTRY *my_entry, char seq[],
  FH *my_fh )
{

  int i = 0, end = strlen( seq ) - MF_LENGTH + 1;
  
  while ( i < end ) {
  
    if ( is_in_region( my_entry, i )) {
      ++i;
      continue;
    }
      
    if (( ! strncmp( seq + i, FOR_MOTIF, MF_LENGTH )) 
      || ( ! strncmp( seq + i, REV_MOTIF, MF_LENGTH )))
    {
      
      my_info->total_gc += is_methylated( my_entry, i, 0, my_fh );
      
      my_info->total_methyl += is_methylated( my_entry, i, METHYL_SCORE_THRESH, 
        my_fh );
      
      i += MF_LENGTH + 1;
    
    }
    
    else {
      ++i;
    }
  
  }

}













int is_in_region( GENCOORD_ENTRY *my_entry, int i )
{

  switch ( my_entry->strand ) {
  
  case '+' :

    if ( i < START_REGION - REGION_BUFFER )
      return 0;

    if ( i > END_REGION + REGION_BUFFER )
      return 0;

    return 1;
  
  case '-' :
  
    if ( i < SEQ_LENGTH - END_REGION - REGION_BUFFER )
      return 0;
      
    if ( i > SEQ_LENGTH - START_REGION + REGION_BUFFER )
      return 0;
      
    return 1;
  
  default :
    error( 11205, "Unrecognized strand.\n" );
  
  }

}

















int is_methylated( GENCOORD_ENTRY *my_entry, int i, int score_min, FH *my_fh )
{

  set_position( 0, my_fh->methyl_fh );
  
  char line[ LINE_BUFFER ];
  
  int methyl = 0, still_going = 1, same_chrom = 0;
  
  while (( still_going ) && ( fgets( line, LINE_BUFFER, my_fh->methyl_fh )))
    still_going = is_methylated_single_line( my_entry, line, i, &methyl,
      &same_chrom, score_min );
    
  return methyl;
  
}













int is_methylated_single_line( GENCOORD_ENTRY *my_entry, char line[], int i,
  int *methyl, int *same_chrom, int score_min )
{

  if ( line[ 0 ] == '#' )
    return 1;
    
  char chr[ BUFFER ], name[ BUFFER ];
  long start, end, score;
  
  sscanf( line, "%s", chr );

  if ( strcmp( chr, my_entry->chr )) {
    if ( *same_chrom )   return 0;
    return 1;
  }
    
  sscanf( line, "%s%ld%ld%s%ld", chr, &start, &end, name, &score );

  *same_chrom = 1;
    
  if ( start < my_entry->start + i )
    return 1;
    
  if ( start > my_entry->start + i + MF_LENGTH - 1 )
    return 1;
    
  if ( score < score_min )
    return 1;
    
  *methyl = 1;
  
  return 0;

}














/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/











void do_methylation_w_total_fh( INFO *my_info, FH *my_fh )
{

  GG gg_all, gg_conv, gg_no_conv;
  
  SEQ seq_numb = 0;
  
  get_gg_from_conversion_line( &gg_conv, my_fh->conv_fh );
  get_gg_from_conversion_line( &gg_no_conv, my_fh->no_conv_fh );
  
  while ( get_gg_from_conversion_line( &gg_all, my_fh->all_genes_fh ) ) 
    do_methylation_single_conv_gene( my_info, &gg_all, &gg_conv, &gg_no_conv, 
      &seq_numb, my_fh );
      
  do_end_methylation( my_info, my_fh );
  
  print_methylation_results( my_info, my_fh->out );

}














int get_gg_from_conversion_line( GG *my_gg, FILE *fh )
{

  char line[ LINE_BUFFER ];
  
  if ( fgets( line, LINE_BUFFER, fh ) == NULL )
    return 0;
    
  sscanf( line, "%d%d%s%s", &(my_gg->index), &(my_gg->seq_numb), my_gg->acc, 
    my_gg->gene );
    
  return 1;

}















void do_methylation_single_conv_gene( INFO *my_info, GG *gg_all, GG *gg_conv, 
  GG *gg_no_conv, SEQ *seq_numb, FH *my_fh )
{

  GENCOORD_ENTRY my_entry;
  char seq[ LINE_BUFFER ];

  do {
    get_fasta_seq_and_gene_label( &my_entry, seq, my_fh->fasta );
    do_methylation_single_gene( my_info, &my_entry, seq, my_fh );
    seq[ 0 ] = '\0';
  } while ( strcmp( my_entry.gene, gg_all->acc ));
  
  update_methyl_matching_seq( my_info, &my_entry, seq, gg_all, gg_conv, gg_no_conv, 
    my_fh );

}















int get_fasta_seq_and_gene_label( GENCOORD_ENTRY *my_entry, char seq[], FILE *in )
{

  char label[ LINE_BUFFER ];
  
  if ( fgets( label, LINE_BUFFER, in ) == NULL )
    return 0;
    
  if ( label[ 0 ] != '>' )
    error( 11201, "Expected angle character at beginning of label line.\n" );
    
  /*  In ../format/gen_coord.c:  */
  get_gencoord_entry_fasta_label( my_entry, label );
  
  /*  In this file, below:  */
  get_seq_from_fasta_file( seq, SEQ_LENGTH, in );
  
  if ( my_entry->strand == '-' ) {

    char my_seq[ LINE_BUFFER ];
    strcpy( my_seq, seq );
    
    /*  In ../main/nucl.c:  */
    get_rev_complement_seq( seq, my_seq ); 

  }
  
  return 1;

}













void get_seq_from_fasta_file( char seq[], int seq_length, FILE *in )
{

  char line[ BUFFER ];

  while ( strlen( seq ) < seq_length ) {
  
    if ( fgets( line, BUFFER, in ) == NULL )
      error( 11202, "Unexpected EOF.\n" );
      
    strcat( seq, line );
    
    if ( seq[ strlen( seq ) - 1 ] != '\n' )
      error( 11203, "Expected newline character at end of line.\n" );
      
    seq[ strlen( seq ) - 1 ] = '\0';
  
  }

}
















void do_methylation_single_gene( INFO *my_info, GENCOORD_ENTRY *my_entry, 
  char seq[], FH *my_fh )
{

  int going = 1, correct_chr = 0;

  set_position( 0, my_fh->methyl_fh );

  char methyl_line[ BUFFER ];
  
  while (( going ) && ( fgets( methyl_line, BUFFER, my_fh->methyl_fh ) != NULL )) 
    going = do_methylation_single_methyl_line( my_info, my_entry, seq, methyl_line, 
      &correct_chr );
      
  do_my_methyl_single_gene( my_info, seq );

}














int do_methylation_single_methyl_line( INFO *my_info, GENCOORD_ENTRY *my_entry,
  char seq[], char methyl_line[], int *correct_chr )
{

  if ( methyl_line[ 0 ] == '#' )
    return;
    
  char chr[ BUFFER ], name[ BUFFER ];
  long start, end, score;
  
  sscanf( methyl_line, "%s", chr );

  if ( strcmp( chr, my_entry->chr )) {
    if ( *correct_chr )   return 0;
    return 1;
  }
    
  sscanf( methyl_line, "%s%ld%ld%s%ld", chr, &start, &end, name, &score );

  *correct_chr = 1;
    
  if ( start < my_entry->start )
    return 1;
    
  if ( end > my_entry->end )
    return 1;
    
  printf( "\t\t\t\t\t\t%s\n", name );
  
  do_methyl_single_matching_methyl_line( my_info, my_entry, seq, start, end,
    score );
  
  return 1;

}














void do_methyl_single_matching_methyl_line( INFO *my_info, GENCOORD_ENTRY *my_entry,
  char seq[], long start, long end, long score )
{

  long x = my_entry->start - start;

  if ( score >= METHYL_SCORE_THRESH ) 
    ++(my_info->total_methyl);

}


















void do_my_methyl_single_gene( INFO *my_info, char seq[] )
{

  short end = strlen( seq ) - 3;

  register short i = 0;
  
  while ( i < end ) {

    if ( is_gc( seq + i )) {
      ++(my_info->total_gc);
      i += 3;
    }
    
    else {
      ++i;
    }

  }

}













int is_gc( char di[] )
{

  if ( ! strncmp( di, "cgc", 3 ))
    return 1;
    
  if ( ! strncmp( di, "gcg", 3 ))
    return 1;
    
  return 0;

}











void update_methyl_matching_seq( INFO *my_info, GENCOORD_ENTRY *my_entry, 
  char seq[], GG *gg_all, GG *gg_conv, GG *gg_no_conv, FH *my_fh )
{
  
  int going = 1, correct_chr = 0;

  set_position( 0, my_fh->methyl_fh );

  char methyl_line[ BUFFER ];
  
  while (( going ) && ( fgets( methyl_line, BUFFER, my_fh->methyl_fh ) != NULL )) 
    going = update_methylation_single_methyl_line( my_info, my_entry, seq, 
      gg_all, gg_conv, gg_no_conv, methyl_line, &correct_chr );
      
  update_total_gc_boxes_in_overrep_region( my_info, my_entry, seq, gg_all,
    gg_conv, gg_no_conv );

  get_new_methyl_gg_struct( gg_all, gg_conv, gg_no_conv, my_fh );
  
}













int update_methylation_single_methyl_line( INFO *my_info, GENCOORD_ENTRY *my_entry,
  char seq[], GG *gg_all, GG *gg_conv, GG *gg_no_conv, char methyl_line[], 
  int *correct_chr )
{

  if ( methyl_line[ 0 ] == '#' )
    return;
    
  char chr[ BUFFER ], name[ BUFFER ];
  long start, end, score;
  
  sscanf( methyl_line, "%s", chr );

  if ( strcmp( chr, my_entry->chr )) {
    if ( *correct_chr )   return 0;
    return 1;
  }
    
  sscanf( methyl_line, "%s%ld%ld%s%ld", chr, &start, &end, name, &score );

  *correct_chr = 1;
    
  if ( start < my_entry->start )
    return 1;
    
  if ( end > my_entry->end )
    return 1;
    
  if ( score < METHYL_SCORE_THRESH )
    return 1;
    
  update_methyl_single_matching_methyl_line( my_info, my_entry, seq, gg_all,
    gg_conv, gg_no_conv, start, score );
    
  return 1;
  
}












void update_methyl_single_matching_methyl_line( INFO *my_info, 
  GENCOORD_ENTRY *my_entry, char seq[], GG *gg_all, GG *gg_conv, GG *gg_no_conv,
  long start, long score )
{

  int x = start - my_entry->start;
  register short i;
  
  for ( i = x - 4; i <= x; i++ ) {

    if (( ! strncmp( FOR_MOTIF, seq + i, MF_LENGTH )) 
      || ( ! strncmp( REV_MOTIF, seq + i, MF_LENGTH ))) {
    
      if ( ! strcmp( gg_all->acc, gg_conv->acc ))
        ++(my_info->conv_hits);
        
      if ( ! strcmp( gg_all->acc, gg_no_conv->acc ))
        ++(my_info->non_conv_hits);    
      
    }

  }
  
}















void update_total_gc_boxes_in_overrep_region( INFO *my_info, 
  GENCOORD_ENTRY *my_entry, char seq[], GG *gg_all, GG *gg_conv, GG *gg_no_conv )
{

  int start = get_start_of_gc_box_region( my_entry );
  
  register int i;
  
  for ( i = start; i <= start + REGION_WIDTH; i++ ) {
  
    if ( ! strncmp( seq + i, FOR_MOTIF, MF_LENGTH )) {
    
      if ( ! strcmp( gg_all->acc, gg_conv->acc ))
        ++(my_info->conv);

      if ( ! strcmp( gg_all->acc, gg_no_conv->acc ))
        ++(my_info->non_conv);
        
    }
    
    if ( ! strncmp( seq + i, REV_MOTIF, MF_LENGTH )) {
    
      if ( ! strcmp( gg_all->acc, gg_conv->acc ))
        ++(my_info->conv);

      if ( ! strcmp( gg_all->acc, gg_no_conv->acc ))
        ++(my_info->non_conv);
        
    }
  
  }

}













int get_start_of_gc_box_region( GENCOORD_ENTRY *my_entry )
{

  switch ( my_entry->strand ) {
  
  case '+' :
    return START_REGION;
    
  case '-' :
    return SEQ_LENGTH - END_REGION;
    
  default :
    error( 11209, "Unrecognized strand type in gencoord entry.\n" );
    break;
  
  }

}













void get_new_methyl_gg_struct( GG *gg_all, GG *gg_conv, GG *gg_no_conv, FH *my_fh )
{    

  if ( ! strcmp( gg_all->acc, gg_conv->acc ))
    get_gg_from_conversion_line( gg_conv, my_fh->conv_fh );

  if ( ! strcmp( gg_all->acc, gg_no_conv->acc ))
    get_gg_from_conversion_line( gg_no_conv, my_fh->no_conv_fh );
  
}















void do_end_methylation( INFO *my_info, FH *my_fh )
{

  GENCOORD_ENTRY my_entry;
  char seq[ LINE_BUFFER ];
  
  while ( get_fasta_seq_and_gene_label( &my_entry, seq, my_fh->fasta )) {
    do_methylation_single_gene( my_info, &my_entry, seq, my_fh );
    seq[ 0 ] = '\0';
  }

}














/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/













void print_methylation_results( INFO *my_info, FILE *out )
{

  PVAL pval, p = (PVAL)my_info->total_methyl / (PVAL)(my_info->total_gc);

  fprintf( out, "Total CpG dinucleotides :  %ld\n", (long)my_info->total_gc );
  fprintf( out, "Total methylated sites  :  %ld\n", (long)my_info->total_methyl );
  fprintf( out, "  Frac :  %.2f\n", 
    (float)my_info->total_methyl * 100.0 / (float)my_info->total_gc );
    
  fprintf( out, "Methyl score threshold  :  %d\n", METHYL_SCORE_THRESH );
  
  print_dot_sep( out );
  
  fprintf( out, "Total converted GC box  :  %ld\n", (long)my_info->conv );
  fprintf( out, "Methyl converted GC box :  %ld\n", (long)my_info->conv_hits );
  fprintf( out, "  Frac :  %.2f\n", 
    (float)my_info->conv_hits * 100.0 / (float)my_info->conv );

  fprintf( out, "  Exp  :  %.2f\n", (float)p * my_info->conv );
  fprintf( out, "  Ratio   %.2fx\n", 
    (float)my_info->conv_hits / (float)(p * my_info->conv) );
    
  pval = get_binom_pval( (int)my_info->conv, (int)my_info->conv_hits, p );  
  
  fprintf( out, "  Pval :  %.2e\n", (double)pval );
    
  print_dot_sep( out );
    
  fprintf( out, "Total non-conv GC box   :  %ld\n", (long)my_info->non_conv );
  fprintf( out, "Methyl non-conv GC box  :  %ld\n", (long)my_info->non_conv_hits );
  fprintf( out, "  Frac :  %.2f\n",
    (float)my_info->non_conv_hits * 100.0 / (float)my_info->non_conv );
    
  fprintf( out, "  Exp  :  %.2f\n", (float)p * my_info->non_conv );
  fprintf( out, "  Ratio   %.2fx\n", 
    (float)my_info->non_conv_hits / (float)(p * my_info->non_conv) );
    
  pval = get_binom_pval( (int)my_info->non_conv, (int)my_info->non_conv_hits, p );  
  
  fprintf( out, "  Pval :  %.2e\n", (double)pval );
    
  print_dot_sep( out );
  
  p = (PVAL)my_info->non_conv_hits / (PVAL)my_info->non_conv;
  
  pval = get_binom_pval( (int)my_info->conv, (int)my_info->conv_hits, p );
  
  fprintf( out, "Converted vs Non-converted...\n" );
  fprintf( out, "  Exp   :  %.2f\n", (float)my_info->conv * (float)p );
  fprintf( out, "  Ratio :  %.2fx\n", 
    (float)my_info->conv_hits / (float)(my_info->conv * p) );
  fprintf( out, "  Pval  :  %.2e\n", (double)pval );

}















PVAL get_binom_pval( int n, int k, PVAL p )
{

  TT n_choose_k( INDEX n, INDEX k );

  PVAL pval = 0.0;
  
  register int i;
  
  for ( i = 0; i <= k; i++ )
    /*  In ../evol/v.c:  */
    pval += n_choose_k( n, i ) * pow( (double)p, (double)i )
      * pow( 1.0 - p, (double)(n - i) );
      
  return pval;

}











PVAL get_binom_pval_up( int n, int k, PVAL p )
{

  TT n_choose_k( INDEX n, INDEX k );

  PVAL pval = 0.0;
  
  register int i;
  
  for ( i = k; i <= n; i++ )
    /*  In ../evol/v.c:  */
    pval += (PVAL)n_choose_k( n, i ) * pow( (double)p, (double)i )
      * pow( 1.0 - p, (double)(n - i) );
      
  return pval;

}









