#include "methyl_seq.h"





















#define METHYL_BASE "data/ft_hg18/methyl/methylSeq/H1hESC/hg18.methylSeq.H1hESC"
#define FASTA_FNAME "data/ft_hg18/hg18.refSeq.500.100.Mar2006.txt.final.txt"
#define CONV_FNAME_BASE "data/ft_hg18/evol/lists/hg18.Mammalia.converted.sorted"
#define OUTPUT_FNAME "data/ft_hg18/methyl/methylSeq/hg18.methylSeq.H1hESC.txt"



#define FOR_MOTIF "gggcgg"
#define REV_MOTIF "ccgccc"
#define MF_LENGTH 6

#define SEQ_LENGTH 600

#define START 300
#define END 500

#define SITE_TOTAL_MIN 10
#define METHYL_FRAC 0.70

#define MIN(a,b) (((a) > (b)) ? (b) : (a))
#define MAX(a,b) (((a) > (b)) ? (a) : (b))












void do_methyl_seq( void )
{

  STR_LIST *chr_header = NULL;

  chr_header = (STR_LIST *)do_methyl_seq_separate_chr( chr_header );
  
  do_my_methyl_seq( chr_header );

}















STR_LIST *do_methyl_seq_separate_chr( STR_LIST *chr_header )
{

  chr_header = (STR_LIST *)get_chr_list_methyl_seq( chr_header );
  
  STR_LIST *str = chr_header;
  
  printf( "Separating files...\n\n" );
  
  while ( str != NULL ) {
    printf( "str :  %s\n", str->str );
    str = str->next;
  }
  
  print_methyl_seq_separate_chrs( chr_header );
  
  return chr_header;

}












STR_LIST *get_chr_list_methyl_seq( STR_LIST *chr_header )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( METHYL_BASE, "all.txt" );
  
  chr_header = (STR_LIST *)get_chr_list_methyl_seq_w_fh( chr_header, in );
  
  close_file( in );
  
  return chr_header;

}












STR_LIST *get_chr_list_methyl_seq_w_fh( STR_LIST *chr_header, FILE *in )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL )
    chr_header = (STR_LIST *)get_chr_list_methyl_seq_single_line( chr_header, line );
    
  return chr_header;

}











STR_LIST *get_chr_list_methyl_seq_single_line( STR_LIST *chr_header, 
  char line[] )
{

  if ( line[ 0 ] == '#' )
    return chr_header;

  char chr[ BUFFER ];
  
  sscanf( line, "%s", chr );

  if ( ! chr_is_in_chr_list( chr, chr_header ))
    return add_new_chr_to_list( chr_header, chr );
    
  return chr_header; 

}












int chr_is_in_chr_list( char chr[], STR_LIST *chr_header )
{

  if ( chr_header == NULL )
    return 0;
    
  STR_LIST *curr = chr_header;
  
  while ( curr != NULL ) {
  
    if ( ! strcmp( curr->str, chr ))
      return 1;
      
    curr = curr->next;
  
  }
  
  return 0;

}











STR_LIST *add_new_chr_to_list( STR_LIST *chr_header, char chr[] )
{

  STR_LIST *create_str_lister( STR_LIST *cc );
  
  STR_LIST *str = NULL;
  str = (STR_LIST *)create_str_lister( str );
  
  strcpy( str->str, chr );
  
  return put_new_chr_into_list( chr_header, str );

}













STR_LIST *put_new_chr_into_list( STR_LIST *chr_header, STR_LIST *chr )
{

  if ( chr_header == NULL )
    return chr;
    
  STR_LIST *curr = chr_header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = chr;
  
  return chr_header;

}












void print_methyl_seq_separate_chrs( STR_LIST *chr_header )
{

  STR_LIST *chr = chr_header;
  
  while ( chr != NULL )
    print_methyl_seq_separate_single_chr_and_step( &chr );

}











void print_methyl_seq_separate_single_chr_and_step( STR_LIST **chr )
{
  print_methyl_seq_separate_single_chr( *chr );
  *chr = (*chr)->next;
}












void print_methyl_seq_separate_single_chr( STR_LIST *chr )
{

  FILE *open_text_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *in = NULL, *out = NULL, *seq_in = NULL, *seq_out = NULL;
  
  char ext[ BUFFER ];
  sprintf( ext, "%s.txt", chr->str );
  
  printf( "chrom :  %s\n", chr->str );
  
  in = (FILE *)open_text_file_reading( METHYL_BASE, "all.txt" );
  out = (FILE *)open_text_file_writing( METHYL_BASE, ext );
  
  print_methyl_seq_single_chrom( chr->str, in, out );
  
  close_file( in );
  close_file( out );

}














void print_methyl_seq_single_chrom( char chrom[], FILE *in, FILE *out )
{

  char line[ LINE_BUFFER ];
  
  fgets( line, LINE_BUFFER, in );
  
  fprintf( out, "%s", line );
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL )
    print_methyl_seq_single_line( chrom, line, out );

}












void print_methyl_seq_single_line( char chrom[], char line[], FILE *out )
{

  char x[ BUFFER ];
  int score;
  
  sscanf( line, "%s%*d%*d%*s%d", x, &score );
  
  if ( ! strcmp( x, chrom ))
    fprintf( out, "%s", line );
    
}















/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/















void do_my_methyl_seq( STR_LIST *chr_header )
{

  INFO my_info = { .total_prom = 0, .methyl_prom = 0, .total_conv = 0,
    .methyl_conv = 0, .total_no_conv = 0, .methyl_no_conv = 0 };
    
  get_methyl_seq_info( &my_info, chr_header );
  
  print_methyl_seq( &my_info );
  
  printf( "Total prom :  %d\n", my_info.total_prom );
  printf( "Methyl prom : %d\n", my_info.methyl_prom );
  printf( "Frac :  %.1f\n", 
    (float)my_info.methyl_prom * 100.0 / (float)my_info.total_prom );
      
}













void get_methyl_seq_info( INFO *my_info, STR_LIST *chr_header )
{

  STR_LIST *chr = chr_header;
  
  while ( chr != NULL )
    get_methyl_seq_info_single_chr_and_step( my_info, &chr );

}












void get_methyl_seq_info_single_chr_and_step( INFO *my_info, STR_LIST **chr )
{
  get_methyl_seq_info_single_chr( my_info, *chr );
  *chr = (*chr)->next;
}










void get_methyl_seq_info_single_chr( INFO *my_info, STR_LIST *chr )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FH my_fh = { .fasta = NULL, .seq_fh = NULL, .methyl_fh = NULL,
    .all_fh = NULL, .conv_fh = NULL, .no_conv_fh = NULL };
    
  char ext[ BUFFER ];
  sprintf( ext, "%s.txt", chr->str );  
    
  my_fh.fasta = (FILE *)open_text_file_reading( FASTA_FNAME, "" );
  my_fh.methyl_fh = (FILE *)open_text_file_reading( METHYL_BASE, ext );
  my_fh.all_fh = (FILE *)open_text_file_reading( CONV_FNAME_BASE, "C.all.txt" );
  my_fh.conv_fh = (FILE *)open_text_file_reading( CONV_FNAME_BASE, "C.A.txt" );
  my_fh.no_conv_fh = (FILE *)open_text_file_reading( CONV_FNAME_BASE, "C.noA.txt" );
  
  get_methyl_seq_info_single_chr_w_fhs( my_info, chr, &my_fh );
  
  close_file( my_fh.fasta );
  close_file( my_fh.methyl_fh );
  close_file( my_fh.all_fh );
  close_file( my_fh.conv_fh );
  close_file( my_fh.no_conv_fh );

}












void get_methyl_seq_info_single_chr_w_fhs( INFO *my_info, STR_LIST *chr, FH *my_fh )
{

  int get_fasta_seq_and_gene_label( GENCOORD_ENTRY *my_entry, char seq[], FILE *in );

  FPOS get_position( FILE *ff );

  GENCOORD_ENTRY my_entry;
  char seq[ LINE_BUFFER ];
  
  seq[ 0 ] = '\0';
  
  FPOS fpos = 0;
  
  int correct_chr = 0;
  
  /*  In methyl.c:  */
  while ( get_fasta_seq_and_gene_label( &my_entry, seq, my_fh->fasta )) {
  
    if ( strcmp( my_entry.chr, chr->str )) {
      if ( correct_chr ) {
        set_position( fpos, my_fh->fasta );
        return;
      }
      
      seq[ 0 ] = '\0';
      continue;
      
    }
    
    fpos = get_position( my_fh->fasta );
    
    correct_chr = 1;
    
    /*  In this file, below:  */
    get_methyl_seq_info_single_gene( my_info, &my_entry, seq, chr, my_fh );
    
    seq[ 0 ] = '\0';
  
  }

}












void get_methyl_seq_info_single_gene( INFO *my_info, GENCOORD_ENTRY *my_entry,
  char seq[], STR_LIST *chr, FH *my_fh )
{

  set_position( 0, my_fh->methyl_fh );
  
  int going = 1;
  int total = 0, methyl = 0;
  
  int prom_start = my_entry->start + START;
  int prom_end = my_entry->start + END;
  
  char line[ LINE_BUFFER ];
  
  while (( going ) && ( fgets( line, LINE_BUFFER, my_fh->methyl_fh )))
    going = get_methyl_seq_info_single_methyl_line( my_info, my_entry, seq,
      line, &total, &methyl, prom_start, prom_end, my_fh );
      
  update_methyl_seq_info_single_gene( my_info, my_entry, seq, chr, total,
    methyl, my_fh );

}













int get_methyl_seq_info_single_methyl_line( INFO *my_info, GENCOORD_ENTRY *my_entry,
  char seq[], char line[], int *total, int *methyl, int prom_start,
  int prom_end, FH *my_fh )
{

  if ( line[ 0 ] == '#' )
    return 1;

  int start, end, score;
  
  sscanf( line, "%*s%d%d%*s%d", &start, &end, &score );
  
  if ( end < prom_start )
    return 1;
    
  if ( start > prom_end )
    return 1;
    
  printf( "\t\t\t\t\t%s :  %d\n", my_entry->gene, score );
  
  int x = MIN( prom_end, end ) - MAX( prom_start, start );
  
  *total += x;
  
  if ( score == 1000 )
    *methyl += x;
  
  return 1;

}

















void update_methyl_seq_info_single_gene( INFO *my_info, GENCOORD_ENTRY *my_entry, 
  char seq[], STR_LIST *chr, int total, int methyl, FH *my_fh )
{

  if ( update_total_methyl_seq_info( my_info, total, methyl ))
    update_gc_box_promoter_info( my_info, my_entry, total, methyl, my_fh );

}













int update_total_methyl_seq_info( INFO *my_info, int total, int methyl )
{

  if ( SITE_TOTAL_MIN > total )
    return 0;
    
  ++(my_info->total_prom);
    
  if ( (float)methyl / (float)total > METHYL_FRAC )
    ++(my_info->methyl_prom);

  return 1;

}
















void update_gc_box_promoter_info( INFO *my_info, GENCOORD_ENTRY *my_entry, 
  int total, int methyl, FH *my_fh )
{

  int mm = ( (float)methyl / (float)total > METHYL_FRAC ) ? (1) : (0);

  if ( acc_exists_in_list_fh( my_entry->gene, my_fh->conv_fh )) {

    ++(my_info->total_conv);

    if ( mm )
      ++(my_info->methyl_conv);

  }
  
  else {
  
    if ( acc_exists_in_list_fh( my_entry->gene, my_fh->no_conv_fh )) {
    
      ++(my_info->total_no_conv);
      
      if ( mm )
        ++(my_info->methyl_no_conv);
    
    }
  
  }

}















int acc_exists_in_list_fh( char acc[], FILE *fh )
{

  set_position( 0, fh );
  
  char my_acc[ BUFFER ], line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, fh ) != NULL ) {
  
    if ( line[ 0 ] == '#' )
      continue;
      
    sscanf( line, "%*d%*d%s", my_acc );
    
    if ( ! strcmp( my_acc, acc )) {
      printf( "\t\t\t\t\t\t\t\tgene found :  %s\n", acc );
      return 1;
    }
  
  }
  
  return 0;

}











/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/















void print_methyl_seq( INFO *my_info )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *fh = NULL;
  fh = (FILE *)open_text_file_writing( OUTPUT_FNAME, "" );
  
  print_methyl_seq_fh( my_info, fh );
  
  close_file( fh );

}













void print_methyl_seq_fh( INFO *my_info, FILE *fh )
{

  PVAL get_hyper_geo_pval_down( int nn, int n, int gg, int g );

  float pr = (float)my_info->methyl_prom / (float)my_info->total_prom;
  float conv_pr = (float)my_info->methyl_conv / (float)my_info->total_conv;
  float no_conv_pr = (float)my_info->methyl_no_conv / (float)my_info->total_no_conv;
  float total_pr = (float)(my_info->methyl_conv + my_info->methyl_no_conv) 
    / (float)(my_info->total_conv + my_info->total_no_conv);
  
  /*  In ../gene/sig.c:  */
  PVAL conv_pval = get_hyper_geo_pval_down( my_info->total_prom, 
    my_info->methyl_prom, my_info->total_conv, my_info->methyl_conv );
    
  /*  In ../gene/sig.c:  */  
  PVAL no_conv_pval = get_hyper_geo_pval_down( my_info->total_prom,
    my_info->methyl_prom, my_info->total_no_conv, my_info->methyl_no_conv );
    
  /*  In ../gene/sig.c:  */
  PVAL pval = get_hyper_geo_pval_down( my_info->total_prom,
    my_info->methyl_prom, 
    my_info->total_conv + my_info->total_no_conv, 
    my_info->methyl_conv + my_info->methyl_no_conv );
    

  fprintf( fh, "Total promoters :    %d\n", my_info->total_prom );
  fprintf( fh, "Methyl promoters :   %d\n", my_info->methyl_prom );
  fprintf( fh, "  Frac :   %.2f\n", 100.0 * pr );
  
  print_dot_sep( fh );
  
  fprintf( fh, "Converted promoters :    %d\n", my_info->total_conv );
  fprintf( fh, "Methyl conv promoters :  %d\n", my_info->methyl_conv );
  fprintf( fh, "  Frac :   %.2f\n", 100.0 * conv_pr );
  fprintf( fh, "  Ratio :  %.4f\n", conv_pr / pr );
  fprintf( fh, "  Pval :   %.2e\n", (double)conv_pval );

  print_dot_sep( fh );
  
  fprintf( fh, "Non-conv promoters :     %d\n", my_info->total_no_conv );
  fprintf( fh, "Methyl non-conv prom :   %d\n", my_info->methyl_no_conv );
  fprintf( fh, "  Frac :   %.2f\n", 100.0 * no_conv_pr );
  fprintf( fh, "  Ratio :  %.4f\n", no_conv_pr / pr );
  fprintf( fh, "  Pval :   %.2e\n", (double)no_conv_pval );

  print_dot_sep( fh );
  
  fprintf( fh, "Total GC promoters :     %d\n", 
    my_info->total_no_conv + my_info->total_conv );
  
  fprintf( fh, "Methyl total prom  :     %d\n", 
    my_info->methyl_conv + my_info->methyl_no_conv );
  fprintf( fh, "  Frac :   %.2f\n", 100.0 * total_pr );
  fprintf( fh, "  Ratio :  %.4f\n", total_pr / pr );
  fprintf( fh, "  Pval :   %.2e\n", (double)pval );

}


















