#include "sep_go.h"















#define GO_CAT_FNAME "data/ft_hg18/go/signif_go_terms.bird.C.all.txt"
#define GO_GENE_FNAME "data/ft_hg19/go/go.hg.condensed.txt"
#define GO_FT_FNAME "data/ft_hg18/go/go_genes.txt"
#define GO_TERM_FNAME "data/ft_hg19/go/go.terms.txt"
#define GENE_LIST_BASE "data/ft_hg18/evol/lists/bird.Mammalia.converted.sorted"
#define OUTPUT_FNAME "data/ft_hg18/go/results_go_enrichment.bird.txt"



#define PVAL_THRESH 1e-3
#define COUNT_MIN 10












void do_sep_go( void )
{

  GO_CAT_LIST *go_header = NULL;
  go_header = (GO_CAT_LIST *)get_go_cat_signif_w_fname( go_header, GO_CAT_FNAME );
  
  do_sep_go_w_list( go_header );

}












GO_CAT_LIST *get_go_cat_signif_w_fname( GO_CAT_LIST *go_header, char fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( fname, "" );
  
  go_header = (GO_CAT_LIST *)get_go_cat_signif_w_fh( go_header, in );
  
  close_file( in );
  
  return go_header;

}











GO_CAT_LIST *get_go_cat_signif_w_fh( GO_CAT_LIST *go_header, FILE *in )
{

  int still_going = 1;
  
  while ( still_going )
    go_header = (GO_CAT_LIST *)get_single_go_cat_input( go_header, &still_going, 
      in );
      
  return go_header;

}











GO_CAT_LIST *get_single_go_cat_input( GO_CAT_LIST *go_header, int *still_going, 
  FILE *in )
{

  char label[ BUFFER ];
  double pval;
  int count;
  
  if ( ! get_go_cat_info_from_input_file( label, &pval, &count, in )) {
    *still_going = 0;
    return go_header;
  }
  
  if ( count < COUNT_MIN )
    return go_header;
    
  return make_and_put_go_cat_from_label( go_header, label );

}











int get_go_cat_info_from_input_file( char label[], double *pval, int *count, 
  FILE *in )
{

  char line[ BUFFER ];
  char x[ BUFFER ], equals[ BUFFER ];
  
  if ( fgets( line, BUFFER, in ) == NULL )
    error( 12000, "Unexpected EOF.\n" );
    
  sscanf( line, "%s%s%s%lf", label, x, equals, pval );
  
  if (( strcmp( x, "pval" )) || ( strcmp( equals, "=" )))
    error( 12001, "Unexpected format of GO line.\n" );
    
  if ( fgets( line, BUFFER, in ) == NULL )
    error( 12002, "Unexpected my EOF.\n" );
    
  sscanf( line, "%s%s%d", x, equals, count );
  
  if (( strcmp( x, "Obs" )) || ( strcmp( equals, "=" )))
    error( 12003, "Unexpected format GO count line.\n" );

  while ( line[ 0 ] != '.' )
    fgets( line, BUFFER, in );
    
  if ( *pval > PVAL_THRESH )
    return 0;
    
  return 1;

}












GO_CAT_LIST *make_and_put_go_cat_from_label( GO_CAT_LIST *go_header, char label[] )
{

  GO_CAT_LIST *create_go_cat_lister( GO_CAT_LIST *xx );
  
  GO_CAT_LIST *go = NULL;
  go = (GO_CAT_LIST *)create_go_cat_lister( go );
  
  strcpy( go->my_go->label, label );
  
  return put_go_cat_at_end_of_list( go_header, go );

}












GO_CAT_LIST *put_go_cat_at_end_of_list( GO_CAT_LIST *go_header, GO_CAT_LIST *go )
{

  if ( go_header == NULL )
    return go;
    
  GO_CAT_LIST *curr = go_header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = go;
  
  return go_header;

}















/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/

















void do_sep_go_w_list( GO_CAT_LIST *go_header )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( OUTPUT_FNAME, "" );
  
  fprintf( out, "# GO_results_file :\n" );
  fprintf( out, "# %s\n", GO_CAT_FNAME );
  fprintf( out, "# Gene lists file :\n" );
  fprintf( out, "# %s\n", GENE_LIST_BASE );
  fprintf( out, "\n\n" );
  
  int total_genes = get_total_genes_go_file( GO_GENE_FNAME );
  
  do_sep_go_w_fh( go_header, total_genes, out );
  
  close_file( out );
  
}












void do_sep_go_w_fh( GO_CAT_LIST *go_header, int total_genes, FILE *out )
{

  LIST my_list = { .a_and_c = NULL, .no_a_and_c = NULL, .a_and_no_c = NULL, 
    .no_a_and_no_c = NULL, .c_no_a_and_c = NULL, .c_and_c = NULL,
    .a_no_c_and_c = NULL, .total = NULL };
  
  set_sep_go_list_struct( &my_list );
  
  do_sep_go_w_separation_lists( &my_list, go_header, total_genes, out );

}













void set_sep_go_list_struct( LIST *my_list )
{

  my_list->a_and_c = (SLIST *)get_gene_list_from_fnames( my_list->a_and_c, 
    GENE_LIST_BASE, "C.A.txt" );
    
  my_list->c_and_c = (SLIST *)get_gene_list_from_fnames( my_list->c_and_c, 
    GENE_LIST_BASE, "C.C.txt" );
    
  my_list->no_a_and_c = (SLIST *)get_gene_list_complement( my_list->no_a_and_c,
    GENE_LIST_BASE, "C.all.txt", "C.A.txt" );
    
  my_list->a_and_no_c = (SLIST *)get_gene_list_complement( my_list->a_and_no_c,
    GENE_LIST_BASE, "all.A.txt", "C.A.txt" );
    
  my_list->c_no_a_and_c = (SLIST *)get_gene_list_complement( my_list->c_no_a_and_c,
    GENE_LIST_BASE, "C.C.txt", "C.A.txt" );

  my_list->a_no_c_and_c = (SLIST *)get_gene_list_complement( my_list->a_no_c_and_c,
    GENE_LIST_BASE, "C.A.txt", "C.C.txt" );
    
}














SLIST *get_gene_list_from_fnames( SLIST *header, char base[], char ext[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( base, ext );
  
  header = (SLIST *)get_gene_list_from_fh( header, in );
  
  close_file( in );
  
  return header;

}














SLIST *get_gene_list_from_fh( SLIST *header, FILE *in )
{

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, in ) != NULL )
    header = (SLIST *)get_gene_list_single_line( header, line );
    
  return header;

}












SLIST *get_gene_list_single_line( SLIST *header, char line[] )
{

  SLIST *create_slister( SLIST *xx );
  int str_in_my_slist( char str[], SLIST *header );
  
  char gene[ BUFFER ];
  
  sscanf( line, "%*d%*d%*s%s", gene );
  
  /*  In enrich.c:  */
  if ( str_in_my_slist( gene, header ))
    return header;
  
  SLIST *x = NULL;
  x = (SLIST *)create_slister( x );
  
  sscanf( line, "%*d%*d%*s%s", x->str );
  
  if ( header == NULL )
    return x;
    
  SLIST *curr = header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = x;  
    
  return header;

}














SLIST *get_gene_list_complement( SLIST *header, char base[], char ext_yes[],
  char ext_no[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *yes = NULL, *no = NULL;
  yes = (FILE *)open_text_file_reading( base, ext_yes );
  no = (FILE *)open_text_file_reading( base, ext_no );
  
  header = (SLIST *)get_gene_list_complement_w_fh( header, yes, no );
  
  close_file( yes );
  close_file( no );
  
  return header;

}













SLIST *get_gene_list_complement_w_fh( SLIST *header, FILE *yes, FILE *no )
{

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, yes ) != NULL )
    header = (SLIST *)get_gene_list_complement_single_line( header, line, no );
    
  return header;

}












SLIST *get_gene_list_complement_single_line( SLIST *header, char line[], FILE *no )
{

  int str_in_my_slist( char str[], SLIST *header );

  char label[ BUFFER ];
  
  sscanf( line, "%*d%*d%*s%s", label );
  
  /*  In enrich.c:  */
  if ( str_in_my_slist( label, header ))
    return header;
  
  set_position( 0, no );
  
  char my_line[ BUFFER ];
  
  while ( fgets( my_line, BUFFER, no ) != NULL ) {
  
    char my_label[ BUFFER ];
    
    sscanf( my_line, "%*d%*d%*s%s", my_label );
    
    if ( ! strcmp( label, my_label ))
      return header;
  
  }
  
  /*  In this file, below:  */
  return set_and_put_gene_lister_end_of_list( header, label );

}













SLIST *set_and_put_gene_lister_end_of_list( SLIST *header, char str[] )
{

  SLIST *create_slister( SLIST *x );
  
  SLIST *x = NULL;
  x = (SLIST *)create_slister( x );
  
  strcpy( x->str, str );
  
  if ( header == NULL )
    return x;
    
  SLIST *curr = header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = x;
  
  return header;

}















void do_sep_go_w_separation_lists( LIST *my_list, GO_CAT_LIST *go_header, 
  int total_genes, FILE *out )
{

  GO_CAT_LIST *go = go_header;
  
  while ( go != NULL )
    do_sep_separation_lists_single_go_lister_and_step( my_list, &go, total_genes,
      out );
      
}














void do_sep_separation_lists_single_go_lister_and_step( LIST *my_list,
  GO_CAT_LIST **go, int total_genes, FILE *out )
{
  do_sep_separation_lists_single_go_lister( my_list, *go, total_genes, out );
  *go = (*go)->next;
}












void do_sep_separation_lists_single_go_lister( LIST *my_list, GO_CAT_LIST *go,
  int total_genes, FILE *out )
{

  char go_term[ LINE_BUFFER ];
  
  get_go_term_label_term_file( go_term, go->my_go->label, GO_TERM_FNAME );

  print_sep( out );
  fprintf( out, "%s\n", go->my_go->label );
  fprintf( out, "%s\n", go_term );
  
  print_dot_sep( out );

  SLIST *go_gene_header = NULL;
  go_gene_header = (SLIST *)get_go_gene_list_from_file( go_gene_header, go, 
    GO_FT_FNAME );
    
  do_go_category_enrichments_for_sep_lists( my_list, go_gene_header, 
    total_genes, out );
  
  print_dot_sep( out );
    
  put_separation_lists_single_go_lister( "GA -> GC", my_list->a_and_c, 
    go_gene_header, total_genes, out );
    
  put_separation_lists_single_go_lister( "(No GA) -> GC", my_list->no_a_and_c,
    go_gene_header, total_genes, out );
    
  put_separation_lists_single_go_lister( "GA -> (No GC)", my_list->a_and_no_c,
    go_gene_header, total_genes, out );
    
  put_separation_lists_single_go_lister( "GC (No GA) -> GC", my_list->c_no_a_and_c,
    go_gene_header, total_genes, out );
    
  put_separation_lists_single_go_lister( "GC -> GC", my_list->c_and_c,
    go_gene_header, total_genes, out );
    
  put_separation_lists_single_go_lister( "GA (No GC) -> GC", my_list->a_no_c_and_c,
    go_gene_header, total_genes, out );
    
  print_dot_sep( out );
    
}














void get_go_term_label_term_file( char go_term[], char label[], char fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *fh = NULL;
  fh = (FILE *)open_text_file_reading( fname, "" );
  
  get_go_term_label_w_fh( go_term, label, fh );
  
  close_file( fh );

}













void get_go_term_label_w_fh( char go_term[], char label[], FILE *fh )
{

  int found = 0;

  char line[ LINE_BUFFER ];
  
  while (( ! found ) && ( fgets( line, LINE_BUFFER, fh ) != NULL ))
    found = get_go_term_label_single_line( go_term, line, label, fh );

}












int get_go_term_label_single_line( char go_term[], char line[], char label[],
  FILE *fh )
{

  if ( strncmp( line, "id:", 3 ))
    return 0;
    
  char file_label[ BUFFER ];
  
  sscanf( line, "%*s%s", file_label );
  
  if ( ! strcmp( label, file_label )) 
    return get_go_term_label_matching_label( go_term, fh );
  
  return 0;

}













int get_go_term_label_matching_label( char go_term[], FILE *fh )
{

  char new_line[ LINE_BUFFER ];

  fgets( new_line, LINE_BUFFER, fh );

  strcpy( go_term, new_line + 6 );

  printf( "%s\n", go_term );

  return 1;

}








/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/












void do_go_category_enrichments_for_sep_lists( LIST *my_list, SLIST *go_gene_header, 
  int total_genes, FILE *out )
{

  SLIST *gc_header = NULL;
  gc_header = (SLIST *)merge_two_slists( gc_header, my_list->a_and_c,
    my_list->no_a_and_c );
    
  do_go_category_enrichments_total_gc_box_genes( gc_header, go_gene_header,
    total_genes, out );
    
  kill_slist( gc_header );

}













SLIST *merge_two_slists( SLIST *x_header, SLIST *a_header, SLIST *b_header )
{

  x_header = (SLIST *)copy_slist( x_header, a_header );
  
  return cat_slist_at_end( x_header, b_header );

}











SLIST *copy_slist( SLIST *x_header, SLIST *a_header )
{

  SLIST *a = a_header;
  
  while ( a != NULL ) 
    x_header = (SLIST *)copy_single_slister_and_step( x_header, &a );
    
  return x_header;
  
}












SLIST *copy_single_slister_and_step( SLIST *x_header, SLIST **a )
{
  x_header = (SLIST *)copy_single_slister( x_header, *a );
  *a = (*a)->next;
  return x_header;
}













SLIST *copy_single_slister( SLIST *x_header, SLIST *a )
{

  SLIST *create_slister( SLIST *xx );
  
  SLIST *x = NULL;
  x = (SLIST *)create_slister( x );
  
  strcpy( x->str, a->str );
  
  if ( x_header == NULL )
    return x;
    
  SLIST *curr = x_header;
  
  while ( curr->next != NULL ) {
  
    if ( ! strcmp( x->str, curr->next->str ))
      return x_header;
         
    curr = curr->next;

  }
  
  curr->next = x;
  
  return x_header;

}












SLIST *cat_slist_at_end( SLIST *x_header, SLIST *b_header )
{

  SLIST *b = b_header;
  
  while ( b != NULL )
    /*  In this file, above:  */
    x_header = (SLIST *)copy_single_slister_and_step( x_header, &b );
    
  return x_header;

}













void do_go_category_enrichments_total_gc_box_genes( SLIST *gc_header, 
  SLIST *go_gene_header, int total_genes, FILE *out )
{

  int get_numb_elem_slist( SLIST *header );
  int get_intersection_slist_pair( SLIST *a_header, SLIST *b_header );
  PVAL get_hyper_geo_pval( int nn, int n, int gg, int g );
  
  int nn = total_genes;
  /*  In go_genes.c:  */
  int n = get_numb_elem_slist( go_gene_header );
  int g = get_intersection_slist_pair( go_gene_header, gc_header );
  int gg = get_numb_elem_slist( gc_header );
  
  float exp = (float)(gg * n) / (float)(nn);
  
  /*  In ../gene/sig.c:  */
  PVAL pval = get_hyper_geo_pval( nn, n, gg, g );
  
  fprintf( out, "Total GC box gene enrichment :\n" );
  fprintf( out, "\tObs   :  %d\n", g );
  fprintf( out, "\tExp   :  %.1f\n", exp );
  fprintf( out, "\tRatio :  %.3fx\n", (float)g / (float)exp );
  fprintf( out, "\tP-val :  %.3Le\n", (long double)pval );

}











/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/














void put_separation_lists_single_go_lister( char x[], SLIST *gene_header,
  SLIST *go_gene_header, int total_genes, FILE *out )
{

  int get_numb_elem_slist( SLIST *header );
  int get_intersection_slist_pair( SLIST *a_header, SLIST *b_header );
  PVAL get_hyper_geo_pval( int nn, int n, int gg, int g );
  
  int nn = total_genes;
  /*  In go_genes.c:  */
  int n = get_numb_elem_slist( go_gene_header );
  int g = get_intersection_slist_pair( go_gene_header, gene_header );
  int gg = get_numb_elem_slist( gene_header );
  
  float exp = (float)(gg * n) / (float)(nn);
  
  /*  In ../gene/sig.c:  */
  PVAL pval = get_hyper_geo_pval( nn, n, gg, g );
  
  fprintf( out, "%20s\tObs :  %5d  Exp :  %6.1f  Ratio :  %.3f\n",
    x,
    g, 
    exp,
    (float)g / exp );
    
}













/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
















void do_sep_go_single_go_cat_lister( GO_CAT_LIST *go, int total_genes, FILE *out )
{

  fprintf( out, "GO_category %s\n", go->my_go->label );
  
  fflush( out );
  
  SLIST *go_gene_header = NULL;
  go_gene_header = (SLIST *)get_go_gene_list_from_file( go_gene_header, go, 
    GO_FT_FNAME );
    
  do_my_sep_go_single_go_cat_lister( go, go_gene_header, total_genes, out );
  
  kill_slist( go_gene_header );
  
  print_dot_sep( out );

}













int get_total_genes_go_file( char fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  int get_total_genes_go_fh( FILE *genes_fh );

  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( fname, "" );
  
  /*  In go_genes.c:  */
  int total_genes = get_total_genes_go_fh( in );
  
  close_file( in );
  
  return total_genes;

}










void do_my_sep_go_single_go_cat_lister( GO_CAT_LIST *go, SLIST *go_gene_header,
  int total_genes, FILE *out )
{

  do_sep_go_single_go_nucl_pair( go, go_gene_header, GENE_LIST_BASE, total_genes, 
    A_NUCL, A_NUCL, out );

  do_sep_go_single_go_nucl_pair( go, go_gene_header, GENE_LIST_BASE, total_genes, 
    A_NUCL, C_NUCL, out );

  do_sep_go_single_go_nucl_pair( go, go_gene_header, GENE_LIST_BASE, total_genes, 
    C_NUCL, A_NUCL, out );

  do_sep_go_single_go_nucl_pair( go, go_gene_header, GENE_LIST_BASE, total_genes, 
    C_NUCL, C_NUCL, out );


}











void do_sep_go_single_go_nucl_pair( GO_CAT_LIST *go, SLIST *go_gene_header,
  char fname_base[], int total_genes, char nucl_a, char nucl_b, FILE *out )
{

  char get_nucl_char_from_bp( MF bp );

  printf( "Go category :  %s     (%c, %c)\n", 
    go->my_go->label,
    get_nucl_char_from_bp( nucl_a ),
    get_nucl_char_from_bp( nucl_b ) );
  
  SLIST *gene_header = NULL;
  gene_header = (SLIST *)get_gene_list_sep_go_nucl_pair( gene_header, fname_base,
    nucl_a, nucl_b );
    
  do_sep_go_single_nucl_pair_w_gene_lists( go, gene_header, go_gene_header,
    total_genes, nucl_a, nucl_b, out );
    
  kill_slist( gene_header );
    
}











SLIST *get_gene_list_sep_go_nucl_pair( SLIST *gene_header, char fname_base[],
  char nucl_a, char nucl_b )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  char nucl_not_a = ( nucl_a == A_NUCL ) ? ( C_NUCL ) : ( A_NUCL );
  char nucl_not_b = ( nucl_b == A_NUCL ) ? ( C_NUCL ) : ( A_NUCL );
  
  char fname[ BUFFER ], fname_not_a[ BUFFER ], fname_not_b[ BUFFER ];
  
  get_sep_go_fname_string_from_nucl_pair( fname, fname_base, nucl_a, nucl_b );  
  get_sep_go_fname_string_from_nucl_pair( fname_not_a, fname_base, nucl_not_a, 
    nucl_b );
  get_sep_go_fname_string_from_nucl_pair( fname_not_b, fname_base, nucl_a, 
    nucl_not_b );
  
  FILE *in = NULL, *not_a = NULL, *not_b = NULL;
  in = (FILE *)open_text_file_reading( fname, "" );
  not_a = (FILE *)open_text_file_reading( fname_not_a, "" );
  not_b = (FILE *)open_text_file_reading( fname_not_b, "" );
  
  gene_header = (SLIST *)do_sep_go_single_go_nucl_pair_w_fh( gene_header, in, not_a, 
    not_b );
  
  close_file( in );  
  close_file( not_a );
  close_file( not_b );
  
  return gene_header;

}












SLIST *do_sep_go_single_go_nucl_pair_w_fh( SLIST *gene_header, FILE *in, 
  FILE *not_a, FILE *not_b )
{

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, in ) != NULL )
    gene_header = (SLIST *)get_sep_go_single_gene_in_list( gene_header, line, in,
      not_a, not_b );
      
  return gene_header;

}













SLIST *get_sep_go_single_gene_in_list( SLIST *gene_header, char line[], FILE *in,
  FILE *not_a, FILE *not_b )
{

  char label[ BUFFER ];
  
  sscanf( line, "%*d%*d%*s%s", label );
  
  if ( gene_name_in_list_file( label, not_a ))
    return gene_header;
    
  if ( gene_name_in_list_file( label, not_b ))
    return gene_header;
    
  return make_and_put_sep_go_single_gene( gene_header, label );

}
















int gene_name_in_list_file( char label[], FILE *fh )
{

  set_position( 0, fh );
  
  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, fh ) != NULL ) {
  
    char gene[ BUFFER ];
    
    sscanf( line, "%*d%*d%*s%s", gene );
    
    if ( ! strcmp( gene, label ))
      return 1;
  
  }
  
  return 0;

}










SLIST *make_and_put_sep_go_single_gene( SLIST *gene_header, char label[] )
{

  SLIST *create_slister( SLIST *xx );
  
  SLIST *gene = NULL;
  gene = (SLIST *)create_slister( gene );
  
  strcpy( gene->str, label );
  
  if ( gene_header == NULL )
    return gene;
    
  SLIST *curr = gene_header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = gene;
  
  return gene_header;

}













/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/











SLIST *get_go_gene_list_from_file( SLIST *go_gene_header, GO_CAT_LIST *go, 
  char fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( fname, "" );
  
  go_gene_header = (SLIST *)get_go_gene_list_w_fh( go_gene_header, go, in );
  
  close_file( in );
  
  return go_gene_header;

}













SLIST *get_go_gene_list_w_fh( SLIST *go_gene_header, GO_CAT_LIST *go, FILE *in )
{

  set_position( 0, in );
  
  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, in ) != NULL ) {
  
    char label[ BUFFER ];
    
    sscanf( line, "%s", label );
  
    if ( ! strcmp( label, go->my_go->label ))
      return get_go_gene_list_correct_entry( go_gene_header, in );

  
  }    
  
  return go_gene_header;

}













SLIST *get_go_gene_list_correct_entry( SLIST *go_gene_header, FILE *in )
{

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, in ) != NULL ) {
  
    if ( line[ 0 ] == '=' )
      return go_gene_header;
      
    go_gene_header = (SLIST *)get_and_put_single_go_gene_slister( go_gene_header,
      line );
  
  }
  
  return go_gene_header;
  
}














SLIST *get_and_put_single_go_gene_slister( SLIST *go_gene_header, char line[] )
{

  SLIST *create_slister( SLIST *xx );
  int str_in_my_slist( char str[], SLIST *header );
  
  char term[ BUFFER ];
  
  sscanf( line, "%s", term );
  
  /*  In enrich.c:  */
  if ( str_in_my_slist( term, go_gene_header ))
    return go_gene_header;
  
  SLIST *gene = NULL;
  gene = (SLIST *)create_slister( gene );
  
  strcpy( gene->str, term );

  gene->next = go_gene_header;
  
  return gene;

}













/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/















void do_sep_go_single_nucl_pair_w_gene_lists( GO_CAT_LIST *go, SLIST *gene_header, 
  SLIST *go_gene_header, int total_genes, char nucl_a, char nucl_b, FILE *out )
{

  char get_nucl_char_from_bp( MF bp );

  /*  In ../main/nucl.c:  */
  fprintf( out, "\tG%c -> G%c\t", 
    toupper( get_nucl_char_from_bp( nucl_b )),
    toupper( get_nucl_char_from_bp( nucl_a )) );
    
  /*  In this file, below:  */
  do_my_sep_go_single_nucl_pair_w_gene_lists( go, gene_header, go_gene_header,
    total_genes, out );

}














void do_my_sep_go_single_nucl_pair_w_gene_lists( GO_CAT_LIST *go, SLIST *gene_header,
  SLIST *go_gene_header, int total_genes, FILE *out )
{

  int get_numb_elem_slist( SLIST *header );
  int get_intersection_slist_pair( SLIST *a_header, SLIST *b_header );
  
  int nn = total_genes;
  /*  In go_genes.c:  */
  int n = get_numb_elem_slist( go_gene_header );
  int g = get_intersection_slist_pair( go_gene_header, gene_header );
  int gg = get_numb_elem_slist( gene_header );
  
  float exp = (float)(gg * n) / (float)(nn);
  
  fprintf( out, "Obs :  %5d  Exp :  %5.1f  Ratio :  %.3f\n", 
    g,
    exp,
    (float)g / exp ); 

}









/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/











void get_sep_go_fname_string_from_nucl_pair( char fname[], char fname_base[],
  char nucl_a, char nucl_b )
{

  char get_nucl_char_from_bp( MF bp );

  /*  In ../main/nucl.c:  */
  sprintf( fname, "%s.%c.%c.txt", fname_base,
    toupper( get_nucl_char_from_bp( nucl_a )),
    toupper( get_nucl_char_from_bp( nucl_b )) );

}





































