#include "sep_methyl.h"












#define FASTA_FNAME "data/ft_hg18/hg18.refSeq.500.100.Mar2006.txt.final.txt"

#define GENE_LIST_BASE "data/ft_hg18/evol/lists/hg18.Mammalia.converted.sorted"
#define GO_CONDENSED_FNAME "data/ft_hg19/go/go.hg.condensed.txt"
#define METHYL_DIR "data/ft_hg18/methyl/methylSeq"
#define CPG_FNAME "data/ft_hg18/cpg/data/hg18.CpG.geneList.txt"

#define SPEC_LIST_BASE "data/ft_hg18/evol/lists/bird.hg19.converted.sorted"


#define OUTPUT_FNAME "data/ft_hg18/methyl/methylSeq/results_methyl.txt"






/*

  Make sure file input names in sep_go.c are set to 
  #define GENE_LIST_BASE "data/ft_hg18/evol/lists/hg18.Mammalia.converted.sorted"

*/













void do_sep_methyl_seq( void )
{

  LIST my_list = { .total = NULL, .a_and_c = NULL, .no_a_and_c = NULL, 
    .a_and_no_c = NULL, .no_a_and_no_c = NULL, .c_no_a_and_c = NULL, .c_and_c = NULL,
    .a_no_c_and_c = NULL, .a_and_a_no_c = NULL, .all_and_c = NULL };
    
  SPEC_LIST spec_list = { .c_and_c = NULL, .no_c_and_c = NULL, .c_and_no_c = NULL };
  
  /*  In sep_go.c:  */
  set_sep_methyl_seq_lists( &my_list );
  
  /*  In total_cat.c:  */
  get_spec_lists( &spec_list, SPEC_LIST_BASE );

  /*  In this file, below:  */
  do_my_sep_methyl_seq( &my_list, &spec_list, METHYL_DIR );

}














void set_sep_methyl_seq_lists( LIST *my_list )
{

  SLIST *get_total_gene_list( SLIST *header, char fname[] );
  SLIST *get_gene_list_from_fnames( SLIST *header, char base[], char ext[] );
  SLIST *get_gene_list_complement( SLIST *header, char base[], char ext_yes[],
    char ext_no[] );

  /*  In enrich.c:  */
  my_list->total = (SLIST *)get_total_gene_list( my_list->total, 
    GO_CONDENSED_FNAME );

  /*  In sep_go.c:  */

  my_list->all_and_c = (SLIST *)get_gene_list_from_fnames( my_list->all_and_c,
    GENE_LIST_BASE, "C.all.txt" );

  my_list->a_and_c = (SLIST *)get_gene_list_from_fnames( my_list->a_and_c,
    GENE_LIST_BASE, "C.A.txt" );

  my_list->c_and_c = (SLIST *)get_gene_list_from_fnames( my_list->c_and_c,
    GENE_LIST_BASE, "C.C.txt" );

  my_list->a_and_no_c = (SLIST *)get_gene_list_complement( my_list->a_and_no_c,
    GENE_LIST_BASE, "all.A.txt", "C.A.txt" );
    
  my_list->no_a_and_c = (SLIST *)get_gene_list_complement( my_list->no_a_and_c,
    GENE_LIST_BASE, "C.all.txt", "C.A.txt" );
    
  my_list->a_no_c_and_c = (SLIST *)get_gene_list_complement( my_list->a_no_c_and_c,
    GENE_LIST_BASE, "C.A.txt", "C.C.txt" );
    
  my_list->c_no_a_and_c = (SLIST *)get_gene_list_complement( my_list->c_no_a_and_c,
    GENE_LIST_BASE, "C.C.txt", "C.A.txt" );
    
  my_list->a_and_a_no_c = (SLIST *)get_gene_list_complement( my_list->a_and_a_no_c,
    GENE_LIST_BASE, "A.A.txt", "C.A.txt" );
    
}












void do_my_sep_methyl_seq( LIST *my_list, SPEC_LIST *spec_list, char dir[] )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( OUTPUT_FNAME, "" );
  
  do_sep_methyl_seq_w_output_fh( my_list, spec_list, dir, out );
  
  close_file( out );

}
















void do_sep_methyl_seq_w_output_fh( LIST *my_list, SPEC_LIST *spec_list,
  char dir[], FILE *out )
{

  do_sep_methyl_seq_single_cell_line( my_list, spec_list, "BG02ES", dir, out );
  do_sep_methyl_seq_single_cell_line( my_list, spec_list, "H1hESC", dir, out );
  do_sep_methyl_seq_single_cell_line( my_list, spec_list, "HAL", dir, out );
  
  do_sep_cpg_islands_output_fh( my_list, spec_list, CPG_FNAME, out );
  
}















void do_sep_methyl_seq_single_cell_line( LIST *my_list, SPEC_LIST *spec_list,
  char cell_line[], char dir[], FILE *out )
{

  fprintf( out, "Cell line :  %s\n", cell_line );
  
  print_dot_sep( out );

  SLIST *methyl_header = NULL, *total_methyl = NULL;
  methyl_header = (SLIST *)get_methyl_slist_cell_line( methyl_header, cell_line,
    dir );
    
  total_methyl = (SLIST *)get_total_methyl_slist_cell_line( total_methyl, cell_line, 
    dir );
  
  do_sep_methyl_seq_w_methyl_gene_list( my_list, spec_list, methyl_header, 
    total_methyl, cell_line, out );
    
  print_sep( out );
  
  kill_slist( methyl_header );

}












SLIST *get_methyl_slist_cell_line( SLIST *methyl_header, char cell_line[],
  char dir[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  char methyl_fname[ BUFFER ];
  
  sprintf( methyl_fname, "%s/%s/hg18.methylSeq.%s.geneList.txt", 
    dir, cell_line, cell_line );
    
  FILE *methyl_fh = NULL;
  methyl_fh = (FILE *)open_text_file_reading( methyl_fname, "" );
  
  methyl_header = (SLIST *)get_methyl_slist_cell_line_w_fh( methyl_header,
    methyl_fh );
    
  close_file( methyl_fh );

  return methyl_header;

}












SLIST *get_methyl_slist_cell_line_w_fh( SLIST *methyl_header, FILE *fh )
{

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, fh ) != NULL )
    methyl_header = (SLIST *)get_methyl_slist_single_line( methyl_header, line );
    
  return methyl_header;

}












SLIST *get_methyl_slist_single_line( SLIST *methyl_header, char line[] )
{

  char acc[ BUFFER ];
  int score;
  
  sscanf( line, "%*d%*d%*s%s%d", acc, &score );
  
  if ( score == 0 )
    return methyl_header;
    
  printf( "Promoter %s\n", acc );
    
  return make_and_put_methyl_slister( methyl_header, acc );

}















SLIST *make_and_put_methyl_slister( SLIST *methyl_header, char acc[] )
{

  SLIST *create_slister( SLIST *xx );
  
  SLIST *x = NULL;
  x = (SLIST *)create_slister( x );
  
  strcpy( x->str, acc );
  
  if ( methyl_header == NULL )
    return x;
    
  SLIST *curr = methyl_header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = x;
  
  return methyl_header;

}













SLIST *get_total_methyl_slist_cell_line( SLIST *total_methyl, char cell_line[], 
  char dir[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  char methyl_fname[ BUFFER ];
  
  sprintf( methyl_fname, "%s/%s/hg18.methylSeq.%s.geneList.txt", 
    dir, cell_line, cell_line );
    
  FILE *methyl_fh = NULL;
  methyl_fh = (FILE *)open_text_file_reading( methyl_fname, "" );
  
  total_methyl = (SLIST *)get_total_methyl_slist_cell_line_w_fh( total_methyl, 
    methyl_fh );
    
  close_file( methyl_fh );

  return total_methyl;

}











SLIST *get_total_methyl_slist_cell_line_w_fh( SLIST *total_methyl, FILE *fh )
{

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, fh ) != NULL )
    total_methyl = (SLIST *)get_total_methyl_slist_single_line( total_methyl, line );
    
  return total_methyl;

}












SLIST *get_total_methyl_slist_single_line( SLIST *total_methyl, char line[] )
{

  char acc[ BUFFER ];
  int score;
  
  sscanf( line, "%*d%*d%*s%s%d", acc, &score );
  
  /*  In this file, above:  */
  return make_and_put_methyl_slister( total_methyl, acc );

}











void do_sep_methyl_seq_w_methyl_gene_list( LIST *my_list, SPEC_LIST *spec_list,
  SLIST *methyl_header, SLIST *total_methyl, char cell_line[], FILE *out )
{

  /*  In this file, below:  */  
  do_total_sep_methyl_gene_list( my_list, methyl_header, total_methyl, out );
  
  print_dot_sep( out );

  /*  In sep_go.c:  */
  
  do_sep_methyl_seq_single_gene_list( "All -> GC", my_list->all_and_c, 
    methyl_header, total_methyl, out );

  do_sep_methyl_seq_single_gene_list( "GA -> GC", my_list->a_and_c, 
    methyl_header, total_methyl, out );

  do_sep_methyl_seq_single_gene_list( "(No GA) -> GC", my_list->no_a_and_c, 
    methyl_header, total_methyl, out );

  do_sep_methyl_seq_single_gene_list( "GA -> (No GC)", my_list->a_and_no_c, 
    methyl_header, total_methyl, out );

  do_sep_methyl_seq_single_gene_list( "GC (No GA) -> GC", my_list->c_no_a_and_c, 
    methyl_header, total_methyl, out );

  do_sep_methyl_seq_single_gene_list( "GA -> GA (No GC)", my_list->a_and_a_no_c, 
    methyl_header, total_methyl, out );

  do_sep_methyl_seq_single_gene_list( "GC -> GC", my_list->c_and_c, 
    methyl_header, total_methyl, out );

  do_sep_methyl_seq_single_gene_list( "GA (No GC) -> GC", my_list->a_no_c_and_c, 
    methyl_header, total_methyl, out );

  print_dot_sep( out );
    
  fprintf( out, "\n" );
  fprintf( out, "Human vs bird gene sets...\n" );
  
  /*  In enrich.c:  */

  print_enrich_single_go_cat_single_list( "Human GC, Bird GC", spec_list->c_and_c,
    total_methyl, methyl_header, out );

  print_enrich_single_go_cat_single_list( "Human GC, B No GC", 
    spec_list->c_and_no_c, total_methyl, methyl_header, out );

  print_enrich_single_go_cat_single_list( "Hm No GC, Bird GC", 
    spec_list->no_c_and_c, total_methyl, methyl_header, out );

}
















void do_total_sep_methyl_gene_list( LIST *my_list, SLIST *methyl_header, 
  SLIST *total_methyl, FILE *out )
{

  SLIST *merge_two_slists( SLIST *x_header, SLIST *a_header, SLIST *b_header );

  SLIST *gc_header = NULL;
  /*  In sep_go.c:  */
  gc_header = (SLIST *)merge_two_slists( gc_header, my_list->a_and_c,
    my_list->no_a_and_c );
    
  /*  In this file, below:  */
  do_my_total_sep_methyl_gene_list( gc_header, methyl_header, total_methyl, out );
  
  kill_slist( gc_header );

}














void do_my_total_sep_methyl_gene_list( SLIST *gc_header, SLIST *methyl_header,
  SLIST *total_methyl, FILE *out )
{

  int get_numb_elem_slist( SLIST *header );
  int get_intersection_slist_pair( SLIST *a_header, SLIST *b_header );
  PVAL get_hyper_geo_pval_down( int nn, int n, int gg, int g );
  
  /*  In go_genes.c:  */
  int nn = get_numb_elem_slist( total_methyl );
  int n = get_numb_elem_slist( methyl_header );
  int gg = get_intersection_slist_pair( gc_header, total_methyl );
  int g = get_intersection_slist_pair( gc_header, methyl_header );
  
  float exp = (float)(gg * n) / (float)(nn);
  
  /*  In ../gene/sig.c:  */
  PVAL pval = get_hyper_geo_pval_down( nn, n, gg, g );
  
  fprintf( out, "Total GC box gene enrichment :\n" );
  fprintf( out, "\tObs   :  %d\n", g );
  fprintf( out, "\tExp   :  %.1f\n", exp );
  fprintf( out, "\tRatio :  %.3fx\n", (float)g / (float)exp );
  fprintf( out, "\tP-val :  %.3e\n", (float)pval );  

}












void do_sep_methyl_seq_single_gene_list( char x[], SLIST *gene_header,
  SLIST *methyl_header, SLIST *total_methyl, FILE *out )
{

  int get_numb_elem_slist( SLIST *header );
  int get_intersection_slist_pair( SLIST *a_header, SLIST *b_header );
  
  /*  In go_genes.c:  */
  int nn = get_numb_elem_slist( total_methyl );
  int n = get_numb_elem_slist( methyl_header );
  int gg = get_intersection_slist_pair( gene_header, total_methyl );
  int g = get_intersection_slist_pair( gene_header, methyl_header );
  
  float exp = (float)(gg * n) / (float)(nn);
  
  fprintf( out, "%20s\t", x );
  
  fprintf( out, "Obs :  %5d  Exp :  %5.1f  Ratio :  %.3f\n", 
    g,
    exp,
    (float)g / exp ); 

}
















/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
















void do_sep_cpg_islands_output_fh( LIST *my_list, SPEC_LIST *spec_list, char fname[], 
  FILE *out )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( fname, "" );
  
  do_sep_cpg_islands_w_fhs( my_list, spec_list, in, out );
  
  close_file( in );

}














void do_sep_cpg_islands_w_fhs( LIST *my_list, SPEC_LIST *spec_list, FILE *in, 
  FILE *out )
{

  SLIST *get_gene_list_from_fh( SLIST *header, FILE *in );
  
  SLIST *cpg_header = NULL;
  /*  In sep_go.c:  */
  cpg_header = (SLIST *)get_gene_list_from_fh( cpg_header, in );
  
  /*  In this file, below:  */
  do_sep_cpg_islands_w_gene_list( my_list, spec_list, cpg_header, out );
  
  kill_slist( cpg_header );

}














void do_sep_cpg_islands_w_gene_list( LIST *my_list, SPEC_LIST *spec_list,
  SLIST *cpg_header, FILE *out )
{

  int total_genes = get_total_genes_fasta_file( FASTA_FNAME );
  
  put_sep_cpg_islands( my_list, spec_list, cpg_header, total_genes, out );

}













int get_total_genes_fasta_file( char fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( fname, "" );
  
  int total_genes = get_total_genes_w_fh( in );
  
  close_file( in );
  
  return total_genes;

}













int get_total_genes_w_fh( FILE *in )
{

  int total = 0;

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL ) {
  
    if ( line[ 0 ] == '>' )
      ++total;
  
  }
  
  return total;

}
















void put_sep_cpg_islands( LIST *my_list, SPEC_LIST *spec_list, SLIST *cpg_header, 
  int total_genes, FILE *out )
{

  fprintf( out, "CpG island promoters\n" );
  
  print_dot_sep( out );
  
  /*  In sep_go.c:  */
  do_go_category_enrichments_for_sep_lists( my_list, cpg_header, total_genes, out );
  
  /*  In sep_go.c:  */

  put_separation_lists_single_go_lister( "All -> GC", my_list->all_and_c,
    cpg_header, total_genes, out );
  
  put_separation_lists_single_go_lister( "GA -> GC", my_list->a_and_c,
    cpg_header, total_genes, out );

  put_separation_lists_single_go_lister( "(No GA) -> GC", my_list->no_a_and_c,
    cpg_header, total_genes, out );

  put_separation_lists_single_go_lister( "GA -> (No GC)", my_list->a_and_no_c,
    cpg_header, total_genes, out );

  put_separation_lists_single_go_lister( "GC (No GA) -> GC", my_list->c_no_a_and_c,
    cpg_header, total_genes, out );

  put_separation_lists_single_go_lister( "GA -> GA (No GC)", my_list->a_and_a_no_c,
    cpg_header, total_genes, out );

  put_separation_lists_single_go_lister( "GC -> GC", my_list->c_and_c,
    cpg_header, total_genes, out );

  put_separation_lists_single_go_lister( "GA (No GC) -> GC", my_list->a_no_c_and_c,
    cpg_header, total_genes, out );
    
  print_dot_sep( out );
    
  fprintf( out, "\n" );
  fprintf( out, "Human vs bird gene sets...\n" );
  
  /*  In enrich.c:  */

  print_enrich_single_go_cat_single_list( "Human GC, Bird GC", spec_list->c_and_c,
    my_list->total, cpg_header, out );

  print_enrich_single_go_cat_single_list( "Human GC, B No GC", 
    spec_list->c_and_no_c, my_list->total, cpg_header, out );

  print_enrich_single_go_cat_single_list( "Hm No GC, Bird GC", 
    spec_list->no_c_and_c, my_list->total, cpg_header, out );

  print_sep( out );

}








