#include "total_enrich.h"











#define INPUT_FNAME "data/ft_hg19/change/hg19.gcBox.count.txt"
#define GO_GENE_FNAME "data/ft_hg18/go/go_genes.txt"
#define GO_TERM_FNAME "data/ft_hg19/go/go.terms.txt"

#define OUTPUT_FNAME "data/ft_hg18/go/enrich.GCbox.total.hg19.txt"








#define SPEC_ONE "galGal3"
#define SPEC_TWO "taeGut1"


#define SEARCHING -1

#define COUNT_MIN 10
#define PVAL_THRESH 1e-3





#define MY_NUCL C_NUCL










void do_total_enrich( void )
{

  SLIST *total_genes = NULL, *gene_header = NULL;
  
  total_genes = (SLIST *)get_total_genes_from_input_fname( total_genes, 
    SPEC_ONE, SPEC_TWO, INPUT_FNAME );

  gene_header = (SLIST *)get_genes_w_motif_type( gene_header, INPUT_FNAME, SPEC_ONE, 
    SPEC_TWO, MY_NUCL );
    
  print_slist( gene_header );
  
  do_total_enrich_w_gene_lists( gene_header, total_genes, GO_GENE_FNAME,
    GO_TERM_FNAME, OUTPUT_FNAME );

}













SLIST *get_total_genes_from_input_fname( SLIST *total_genes, char spec_one[],
  char spec_two[], char fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( fname, "" );
  
  total_genes = (SLIST *)get_total_genes_from_fh( total_genes, spec_one,
    spec_two, in );
  
  close_file( in );
  
  return total_genes;

}














SLIST *get_total_genes_from_fh( SLIST *total_genes, char spec_one[],
  char spec_two[], FILE *in )
{

  STR_LIST *get_label_list_from_first_line( STR_LIST *label_header, FILE *in );

  STR_LIST *label_header = NULL;
  /*  In set.c:  */
  label_header = (STR_LIST *)get_label_list_from_first_line( label_header, in );
  
  /*  In this file, below:  */
  return get_total_genes_w_label_list( total_genes, label_header, spec_one,
    spec_two, in );
  
}












SLIST *get_total_genes_w_label_list( SLIST *total_genes, STR_LIST *label_header,
  char spec_one[], char spec_two[], FILE *in )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL )
    total_genes = (SLIST *)get_total_genes_single_line( total_genes, label_header,
      spec_one, spec_two, line );
      
  return total_genes;

}












SLIST *get_total_genes_single_line( SLIST *total_genes, STR_LIST *label_header,
  char spec_one[], char spec_two[], char line[] )
{

  char gene[ BUFFER ];
  
  sscanf( line, "%*d%*s%s", gene );
  
  printf( "My gene :  %s\n", gene );

  if ( gene_exists_in_col_line( spec_one, label_header, line ))
    return make_and_put_gene_into_slist( total_genes, gene );
    
  if ( gene_exists_in_col_line( spec_two, label_header, line ))
    return make_and_put_gene_into_slist( total_genes, gene );
    
  return total_genes;

}













int gene_exists_in_col_line( char spec[], STR_LIST *label_header, char line[] )
{

  return 1;

  int searching = SEARCHING;
  INDEX index = 0;
  
  STR_LIST *label = label_header;
  
  while (( searching == SEARCHING ) && ( label != NULL ))
    searching = gene_exists_in_single_column_and_step( &label, line, spec, &index );
    
  return searching;

}











int gene_exists_in_single_column_and_step( STR_LIST **label, char line[], 
  char spec[], INDEX *index )
{
  int searching = gene_exists_in_single_column( *label, line, spec, index );
  *label = (*label)->next;
  return searching;
}











int gene_exists_in_single_column( STR_LIST *label, char line[], char spec[],
  INDEX *index )
{

  INDEX get_next_word_str_index( char str[], INDEX curr_index );
  char get_entry_character_nucl_numb( char entry[], char nucl_numb );

  char entry[ BUFFER ];
  
  sscanf( line + (*index), "%s", entry );
  
  /*  In ../evol/set.c:  */
  *index = get_next_word_str_index( line, *index );
  
  if ( strcmp( label->str, spec ))
    return SEARCHING;
    
  /*  In ../evol/set.c:  */  
  char entry_char = get_entry_character_nucl_numb( entry, A_NUCL );
    
  if ( entry_char == '-' ) 
    return 0;
    
  return 1;

}








SLIST *get_genes_w_motif_type( SLIST *gene_header, char fname[], char spec_one[],
  char spec_two[], char my_nucl )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( fname, "" );
  
  gene_header = (SLIST *)get_genes_motif_type_w_fh( gene_header, spec_one,
    spec_two, my_nucl, in );
  
  close_file( in );
  
  return gene_header;

}












SLIST *get_genes_motif_type_w_fh( SLIST *gene_header, char spec_one[],
  char spec_two[], char my_nucl, FILE *in )
{

  STR_LIST *get_label_list_from_first_line( STR_LIST *label_header, FILE *in );

  STR_LIST *label_header = NULL;
  /*  In set.c:  */
  label_header = (STR_LIST *)get_label_list_from_first_line( label_header, in );
  
  /*  In this file, below:  */
  return get_genes_motif_type_w_label_list( gene_header, label_header, spec_one,
    spec_two, my_nucl, in );

}













SLIST *get_genes_motif_type_w_label_list( SLIST *gene_header, STR_LIST *label_header, 
  char spec_one[], char spec_two[], char my_nucl, FILE *in )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL )
    gene_header = (SLIST *)get_genes_motif_type_single_line( gene_header,
      label_header, line, spec_one, spec_two, my_nucl );
      
  return gene_header;

}













SLIST *get_genes_motif_type_single_line( SLIST *gene_header, STR_LIST *label_header,
  char line[], char spec_one[], char spec_two[], char my_nucl )
{

  char gene[ BUFFER ];
  
  sscanf( line, "%*d%*s%s", gene );
  
  printf( "Gene :  %s\n", gene );

  if ( motif_type_in_col_line( label_header, line, spec_one, my_nucl ))
    return make_and_put_gene_into_slist( gene_header, gene );
    
  if ( motif_type_in_col_line( label_header, line, spec_two, my_nucl ))
    return make_and_put_gene_into_slist( gene_header, gene );
    
  return gene_header;

}













int motif_type_in_col_line( STR_LIST *label_header, char line[], char spec[], 
  char my_nucl )
{

  int searching = SEARCHING;
  INDEX index = 0;
  
  STR_LIST *label = label_header;
  
  while (( searching == SEARCHING ) && ( label != NULL ))
    searching = motif_type_in_single_column_and_step( &label, line, spec, 
      &index, my_nucl );
    
  return searching;

}










int motif_type_in_single_column_and_step( STR_LIST **label, char line[], char spec[],
  INDEX *index, char my_nucl )
{
  int searching = motif_type_in_single_column( *label, line, spec, index, my_nucl );
  *label = (*label)->next;
  return searching;
}











int motif_type_in_single_column( STR_LIST *label, char line[], char spec[],
  INDEX *index, char my_nucl )
{

  INDEX get_next_word_str_index( char str[], INDEX curr_index );
  char get_entry_character_nucl_numb( char entry[], char nucl_numb );

  char entry[ BUFFER ];
  
  sscanf( line + (*index), "%s", entry );
  
  /*  In ../evol/set.c:  */
  *index = get_next_word_str_index( line, *index );
  
  if ( strcmp( label->str, spec ))
    return SEARCHING;
    
  /*  In ../evol/set.c:  */  
  char entry_char = get_entry_character_nucl_numb( entry, my_nucl );
    
  if (( entry_char == '-' ) || ( entry_char == 0 ))
    return 0;
    
  return 1;

}











SLIST *make_and_put_gene_into_slist( SLIST *gene_header, char gene[] )
{

  SLIST *create_slister( SLIST *xx );
  int str_in_my_slist( char xx[], SLIST *xy );

  if ( str_in_my_slist( gene, gene_header ))
    return gene_header;
    
  SLIST *g = NULL;
  g = (SLIST *)create_slister( g );
  
  strcpy( g->str, gene );
  
  if ( gene_header == NULL )
    return g;
    
  SLIST *curr = gene_header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = g;
  
  return gene_header;

}













/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/













void do_total_enrich_w_gene_lists( SLIST *gene_header, SLIST *total_genes, 
  char go_gene_fname[], char go_term_fname[], char output_fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *in = NULL, *out = NULL;
  in = (FILE *)open_text_file_reading( go_gene_fname, "" );
  out = (FILE *)open_text_file_writing( output_fname, "" );
  
  do_total_enrich_w_gene_lists_fhs( gene_header, total_genes, go_term_fname, in, 
    out );
  
  close_file( in );
  close_file( out );

}













void do_total_enrich_w_gene_lists_fhs( SLIST *gene_header, SLIST *total_genes,
  char go_term_fname[], FILE *in, FILE *out )
{

  SLIST *get_go_category_list_w_fh( SLIST *term_header, FILE *in );
  
  SLIST *term_header = NULL;
  /*  In enrich.c:  */
  term_header = (SLIST *)get_go_category_list_w_fh( term_header, in );
  
  /*  In this file, below:  */
  do_my_total_enrich_w_gene_lists( term_header, gene_header, total_genes,
    go_term_fname, in, out );

}














void do_my_total_enrich_w_gene_lists( SLIST *term_header, SLIST *gene_header,
  SLIST *total_genes, char go_term_fname[], FILE *in, FILE *out )
{

  GO_CAT_LIST *go_header = NULL;
  go_header = (GO_CAT_LIST *)get_go_cat_list_total_enrich( go_header, term_header,
    gene_header, total_genes, in );
    
  print_go_cat_list_total( go_header, gene_header, go_term_fname, out );

}











GO_CAT_LIST *get_go_cat_list_total_enrich( GO_CAT_LIST *go_header,
  SLIST *term_header, SLIST *gene_header, SLIST *total_genes, FILE *in )
{

  SLIST *term = term_header;
  
  while ( term != NULL )
    go_header = (GO_CAT_LIST *)get_go_cat_list_single_term_and_step( go_header,
      &term, gene_header, total_genes, in );
      
  return go_header;

}











GO_CAT_LIST *get_go_cat_list_single_term_and_step( GO_CAT_LIST *go_header,
  SLIST **term, SLIST *gene_header, SLIST *total_genes, FILE *in )
{

  go_header = (GO_CAT_LIST *)get_go_cat_list_single_term( go_header, *term,
    gene_header, total_genes, in );
    
  *term = (*term)->next;
  
  return go_header;

}












GO_CAT_LIST *get_go_cat_list_single_term( GO_CAT_LIST *go_header, SLIST *term,
  SLIST *gene_header, SLIST *total_genes, FILE *in )
{

  static int i = 0;

  printf( "GO term (%d) :  %s\n", ++i, term->str );

  SLIST *go_gene_header = NULL;
  go_gene_header = (SLIST *)get_go_gene_list_total( go_gene_header, term, in );
  
  go_header = (GO_CAT_LIST *)get_my_go_cat_list_single_term( go_header,
    term, go_gene_header, gene_header, total_genes );
  
  kill_slist( go_gene_header );
  
  return go_header;
  
}











SLIST *get_go_gene_list_total( SLIST *go_gene_header, SLIST *term, FILE *in )
{

  SLIST *get_go_gene_list_correct_entry( SLIST *go_gene_header, FILE *in );

  set_position( 0, in );
  
  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, in ) != NULL ) {
  
    char label[ BUFFER ];
    
    sscanf( line, "%s", label );
    
    if ( ! strcmp( label, term->str ))
      /*  In sep_go.c:  */
      return get_go_gene_list_correct_entry( go_gene_header, in );
  
  }
  
  return go_gene_header;

}














GO_CAT_LIST *get_my_go_cat_list_single_term( GO_CAT_LIST *go_header,
  SLIST *term, SLIST *go_gene_header, SLIST *gene_header, SLIST *total_genes )
{

  GO_CAT_LIST *create_go_cat_lister( GO_CAT_LIST *xx );
  int get_numb_elem_slist( SLIST *header );
  int get_intersection_slist_pair( SLIST *a_header, SLIST *b_header );
  PVAL get_hyper_geo_pval( int nn, int n, int gg, int g );
  GO_CAT_LIST *put_go_cat_lister_into_sorted_list( GO_CAT_LIST *go_header, 
    GO_CAT_LIST *go );

  /*  In sep_go.c:  */
  int nn = get_numb_elem_slist( total_genes );
  int n = get_numb_elem_slist( gene_header );
  int gg = get_intersection_slist_pair( go_gene_header, total_genes );
  int g = get_intersection_slist_pair( gene_header, go_gene_header );

  /*  In ../gene/sig.c:  */
  PVAL pval = get_hyper_geo_pval( nn, n, gg, g );

  if (( g < COUNT_MIN ) || ( pval > PVAL_THRESH ))
    return go_header;
    
  if ( pval < 1e-4 )
    printf( "\t\t\t\t\tpval :  %.2e\n", (float)pval );
    
  GO_CAT_LIST *go = NULL;
  go = (GO_CAT_LIST *)create_go_cat_lister( go );
  
  strcpy( go->my_go->acc, term->str );
  strcpy( go->my_go->label, term->str );

  go->my_go->pval = pval;
  go->my_go->total_genes = gg;
  go->my_go->x_genes = g;
  go->my_go->exp_numb = (float)(n * gg) / (float)(nn);
  go->my_go->ratio = (float)go->my_go->x_genes / (float)go->my_go->exp_numb;
  
  /*  In enrich.c:  */
  return put_go_cat_lister_into_sorted_list( go_header, go );
    
}












/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/













void print_go_cat_list_total( GO_CAT_LIST *go_header, SLIST *gene_header, 
  char go_term_fname[], FILE *out )
{

  int get_numb_elem_slist( SLIST *header );
  
  int g = get_numb_elem_slist( gene_header );

  fprintf( out, "Species :  %s and %s\n", SPEC_ONE, SPEC_TWO );
  fprintf( out, "Total target genes :  %d\n", g );
  fprintf( out, "\n" );
  
  print_sep( out );
  
  print_my_go_cat_list_total( go_header, go_term_fname, out );

}











void print_my_go_cat_list_total( GO_CAT_LIST *go_header, char go_term_fname[],
  FILE *out )
{

  GO_CAT_LIST *go = go_header;
  
  while ( go != NULL )
    print_go_cat_list_total_single_go_and_step( &go, go_term_fname, out );

}










void print_go_cat_list_total_single_go_and_step( GO_CAT_LIST **go, 
  char go_term_fname[], FILE *out )
{
  print_go_cat_list_total_single_go( *go, go_term_fname, out );
  *go = (*go)->next;
}











void print_go_cat_list_total_single_go( GO_CAT_LIST *go, char go_term_fname[],
  FILE *out )
{

  char go_term[ LINE_BUFFER ];
  
  /*  In sep_go.c:  */  
  get_go_term_label_term_file( go_term, go->my_go->label, go_term_fname );
  
  fprintf( out, "%s\n", go->my_go->label );  
  fprintf( out, "%s\n", go_term );
  
  fprintf( out, "\n" );
  
  fprintf( out, "Pval :   %.2e\n", (float)(go->my_go->pval) );
  fprintf( out, "Obs :    %d\n", go->my_go->x_genes );
  fprintf( out, "Exp :    %.1f\n", go->my_go->exp_numb );
  fprintf( out, "Ratio :  %.3f\n", go->my_go->ratio );
  fprintf( out, "Total :  %d\n", go->my_go->total_genes );
  
  print_sep( out );

}















