#include "two_nd_enrich.h"


















#define TOTAL_GENE_LIST_BASE "data/ft_hg18/evol/lists/bird.Mammalia.converted.sorted"
#define GENE_LIST_BASE "data/ft_hg18/evol/lists/bird.hg19.converted.sorted"
#define GO_GENE_FNAME "data/ft_hg18/go/go_genes.txt"
#define GO_TERM_FNAME "data/ft_hg19/go/go.terms.txt"
#define GO_CONDENSED_FNAME "data/ft_hg19/go/go.hg.condensed.txt"
#define OUTPUT_FNAME "data/ft_hg18/go/results_go_enrich.new.bird.hg19.txt"






#define COUNT_MIN 10
#define PVAL_THRESH 1e-3












void do_two_node_enrich( void )
{

  SLIST *get_go_category_list_w_fname( SLIST *term_header, char fname[] );

  SLIST *term_header = NULL;
  /*  In enrich.c:  */
  term_header = (SLIST *)get_go_category_list_w_fname( term_header, GO_GENE_FNAME );
  
  /*  In this file, below:  */
  do_my_two_node_enrich( term_header, TOTAL_GENE_LIST_BASE, GENE_LIST_BASE, 
    GO_GENE_FNAME, GO_TERM_FNAME, OUTPUT_FNAME );

}














void do_my_two_node_enrich( SLIST *term_header, char total_gene_list_base[],
  char gene_list_base[], char go_gene_fname[], char go_term_fname[], 
  char output_fname[] )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( output_fname, "" );
  
  do_two_node_enrich_w_output_fh( term_header, total_gene_list_base, gene_list_base, 
    go_gene_fname, go_term_fname, out );
    
  close_file( out );
  
}











void do_two_node_enrich_w_output_fh( SLIST *term_header, char total_list_base[],
  char list_base[], char go_gene_fname[], char go_term_fname[], FILE *out )
{

  MY_LIST my_list = { .all_and_c = NULL, .c_and_c = NULL, 
    .c_and_no_c = NULL, .no_c_and_c = NULL, .total = NULL };
  
  get_list_two_node_enrich( &my_list, total_list_base, list_base );
  
  do_two_node_enrich_w_gene_lists( term_header, &my_list, go_gene_fname, 
    go_term_fname, out );

}













void get_list_two_node_enrich( MY_LIST *my_list, char total_list_base[],
  char list_base[] )
{

  SLIST *get_total_gene_list( SLIST *header, char fname[] );
  SLIST *get_gene_list_from_fnames( SLIST *header, char base[], char ext[] );
  SLIST *get_gene_list_complement( SLIST *header, char base[], char ext_yes[],
    char ext_no[] );

  /*  In enrich.c:  */
  my_list->total = (SLIST *)get_total_gene_list( my_list->total,
    GO_CONDENSED_FNAME );
    
  /*  In sep_go.c:  */  
  my_list->all_and_c = (SLIST *)get_gene_list_from_fnames( my_list->all_and_c,
    total_list_base, "C.all.txt" );
    
  my_list->c_and_c = (SLIST *)get_gene_list_from_fnames( my_list->c_and_c,
    list_base, "C.C.txt" );
    
  my_list->c_and_no_c = (SLIST *)get_gene_list_complement( my_list->c_and_no_c,
    list_base, "all.C.txt", "C.C.txt" );
    
  my_list->no_c_and_c = (SLIST *)get_gene_list_complement( my_list->no_c_and_c,
    list_base, "C.all.txt", "C.C.txt" );

}











void do_two_node_enrich_w_gene_lists( SLIST *term_header, MY_LIST *my_list, 
  char go_gene_fname[], char go_term_fname[], FILE *out )
{

  GO_CAT_LIST *go_header = NULL;
  go_header = (GO_CAT_LIST *)get_go_enrich_list_two_node( go_header, term_header,
    my_list, go_gene_fname );
    
  do_gene_two_node_enrich_w_go_cat_list( go_header, my_list, go_gene_fname,
    go_term_fname, out );

}












GO_CAT_LIST *get_go_enrich_list_two_node( GO_CAT_LIST *go_header, SLIST *term_header,
  MY_LIST *my_list, char go_gene_fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( go_gene_fname, "" );
  
  go_header = (GO_CAT_LIST *)get_go_enrich_list_two_node_w_fh( go_header, 
    term_header, my_list, in );
    
  close_file( in );
  
  return go_header;

}











GO_CAT_LIST *get_go_enrich_list_two_node_w_fh( GO_CAT_LIST *go_header, 
  SLIST *term_header, MY_LIST *my_list, FILE *in )
{

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, in ) != NULL )
    go_header = (GO_CAT_LIST *)get_go_enrich_two_node_single_entry( go_header,
      my_list, term_header, line[ 0 ], in );
      
  return go_header;

}












GO_CAT_LIST *get_go_enrich_two_node_single_entry( GO_CAT_LIST *go_header,
  MY_LIST *my_list, SLIST *term_header, char x, FILE *in )
{

  int str_in_my_slist( char str[], SLIST *header );
  GO_CAT_LIST *empty_entry_go_enrich( GO_CAT_LIST *go_header, FILE *in );

  if ( x != '=' )
    error( 12800, "Expected separator.\n" );
    
  char line[ BUFFER ], term[ BUFFER ];
  
  fgets( line, BUFFER, in );
  
  sscanf( line, "%s", term );
  
  /*  In enrich.c:  */
  if ( ! str_in_my_slist( term, term_header ))
    /*  In enrich.c:  */
    return empty_entry_go_enrich( go_header, in );
    
  /*  In this file, below:  */
  return get_go_enrich_two_node_single_correct_entry( go_header, term, my_list, in );

}














GO_CAT_LIST *get_go_enrich_two_node_single_correct_entry( GO_CAT_LIST *go_header,
  char term[], MY_LIST *my_list, FILE *in )
{

  SLIST *get_go_gene_list_correct_entry( SLIST *go_gene_header, FILE *in );
  GO_CAT_LIST *put_go_cat_lister_into_sorted_list( GO_CAT_LIST *go_header, 
    GO_CAT_LIST *go );

  GO_CAT_LIST *go = NULL;

  SLIST *go_gene_header = NULL;
  /*  In sep_go.c:  */
  go_gene_header = (SLIST *)get_go_gene_list_correct_entry( go_gene_header, in );
  
  /*  In this file, below:  */
  go = (GO_CAT_LIST *)make_and_put_go_enrich_two_node_gene_slist( go, my_list,
    term, go_gene_header );
    
  kill_slist( go_gene_header );
  
  if ( go == NULL )
    return go_header;
    
  /*  In enrich.c:  */  
  return put_go_cat_lister_into_sorted_list( go_header, go );

}











GO_CAT_LIST *make_and_put_go_enrich_two_node_gene_slist( GO_CAT_LIST *go,
  MY_LIST *my_list, char term[], SLIST *go_gene_header )
{

  GO_CAT_LIST *create_go_cat_lister( GO_CAT_LIST *xx );
  int get_numb_elem_slist( SLIST *header );
  int get_intersection_slist_pair( SLIST *a_header, SLIST *b_header );
  PVAL get_hyper_geo_pval( int nn, int n, int gg, int g );

  /*  In go_genes.c:  */
  int nn = get_numb_elem_slist( my_list->total );
  int n = get_intersection_slist_pair( my_list->total, go_gene_header );
  int gg = get_numb_elem_slist( my_list->all_and_c );
  /*  In this file, below:  */
  int g = get_intersection_slist_pair( go_gene_header, my_list->all_and_c );
  
  static int i = 0;
  
  printf( "%d :  %s\n", ++i, term );
  
  if ( g < COUNT_MIN )
    return NULL;
    
  go = (GO_CAT_LIST *)create_go_cat_lister( go );
  
  strcpy( go->my_go->acc, term );
  strcpy( go->my_go->label, term );
  
  /*  In ../gene/sig.c:  */
  go->my_go->pval = get_hyper_geo_pval( nn, gg, n, g );
  
  if ( go->my_go->pval < 1e-4 )
    printf( "\t\t\t\t\t\tpval :  %.3e\n", (float)go->my_go->pval );
    
  go->my_go->x_genes = g;
  go->my_go->exp_numb = (float)(n * gg) / (float)(nn);
  go->my_go->ratio = (float)go->my_go->x_genes / (float)go->my_go->exp_numb;

  return go;

}













/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/


















void do_gene_two_node_enrich_w_go_cat_list( GO_CAT_LIST *go_header, 
  MY_LIST *my_list, char go_gene_fname[], char go_term_fname[], FILE *out )
{

  GO_CAT_LIST *go = go_header;
  
  while ( go != NULL )
    do_gene_two_node_enrich_single_go_and_step( &go, my_list, go_gene_fname,
      go_term_fname, out );

}













void do_gene_two_node_enrich_single_go_and_step( GO_CAT_LIST **go, MY_LIST *my_list,
  char go_gene_fname[], char go_term_fname[], FILE *out )
{
  do_gene_two_node_enrich_single_go( *go, my_list, go_gene_fname, go_term_fname,
    out );
  *go = (*go)->next;
}













void do_gene_two_node_enrich_single_go( GO_CAT_LIST *go, MY_LIST *my_list,
  char go_gene_fname[], char go_term_fname[], FILE *out )
{

  if ( go->my_go->pval > PVAL_THRESH )
    return;
    
  do_my_gene_two_node_enrich_single_go( go, my_list, go_gene_fname, go_term_fname,
    out );

}













void do_my_gene_two_node_enrich_single_go( GO_CAT_LIST *go, MY_LIST *my_list,
  char go_gene_fname[], char go_term_fname[], FILE *out )
{

  char go_term[ LINE_BUFFER ];
  
  /*  In sep_go.c:  */  
  get_go_term_label_term_file( go_term, go->my_go->label, go_term_fname );
  
  print_sep( out );
  fprintf( out, "%s\n", go->my_go->label );
  fprintf( out, "%s\n", go_term );
  
  print_dot_sep( out );
  
  /*  In this file, below:  */
  do_gene_two_node_enrich_single_go_cat( go, my_list, go_gene_fname, out );
  
  print_sep( out );

}














void do_gene_two_node_enrich_single_go_cat( GO_CAT_LIST *go, MY_LIST *my_list,
  char go_gene_fname[], FILE *out )
{

  SLIST *get_go_gene_list_from_file( SLIST *go_gene_header, GO_CAT_LIST *go, 
    char fname[] );

  SLIST *go_gene_header = NULL;
  /*  In sep_go.c:  */
  go_gene_header = (SLIST *)get_go_gene_list_from_file( go_gene_header, go, 
    go_gene_fname );
    
  /*  In this file, below:  */
  do_gene_two_node_enrich_single_go_w_gene_list( go, my_list, go_gene_header,
    out );
    
  kill_slist( go_gene_header );
  
}













void do_gene_two_node_enrich_single_go_w_gene_list( GO_CAT_LIST *go,
  MY_LIST *my_list, SLIST *go_gene_header, FILE *out )
{

  print_gene_enrich_go_cat_lister( go, out );
  
  print_dot_sep( out );
  
  do_my_gene_two_node_enrich_single_go_w_gene_list( go, my_list, go_gene_header,
    out );

}














void do_my_gene_two_node_enrich_single_go_w_gene_list( GO_CAT_LIST *go,
  MY_LIST *my_list, SLIST *go_gene_header, FILE *out )
{

  print_enrich_single_go_cat_single_list( "All, GC", my_list->all_and_c,
    my_list->total, go_gene_header, out );

  print_enrich_single_go_cat_single_list( "GC, GC", my_list->c_and_c,
    my_list->total, go_gene_header, out );

  print_enrich_single_go_cat_single_list( "(No GC), GC", my_list->no_c_and_c,
    my_list->total, go_gene_header, out );

  print_enrich_single_go_cat_single_list( "GC, (No GC)", my_list->c_and_no_c,
    my_list->total, go_gene_header, out );

}

























