#include "converge.h"














#define BIRD "galGal3"
#define HM "mm9"
#define MARSUPIAL "monDom5"















void do_hack_converge( void )
{

  printf( "Running convergent evolution hack...\n\n" );
  
  do_my_hack_converge();
  
  printf( "\n" );

}













void do_my_hack_converge( void )
{

  SS my_struct;
  
  initialize_ss_struct( &my_struct );
  
  do_hack_converge_w_struct( &my_struct );

}













void initialize_ss_struct( SS *my_struct )
{

  int i, j, k;

  my_struct->bird_total = 0;
  my_struct->hg_total = 0;
  my_struct->mar_total = 0;
  
  my_struct->bird_w_nucl = 0;
  my_struct->hg_w_nucl = 0;
  my_struct->mar_w_nucl = 0;
  
  my_struct->hg_bird_w_nucl = 0;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
    my_struct->bird_nucl[ i ] = 0;
    my_struct->hg_nucl[ i ] = 0;
    my_struct->mar_nucl[ i ] = 0;
    my_struct->overlap[ i ] = 0;
  }
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
  
    for ( j = 0; j < NUMB_OF_NUCL; j++ ) {
    
      for ( k = 0; k < NUMB_OF_NUCL; k++ ) 
      
        my_struct->co_occur[ i ][ j ][ k ] = 0;
    
    }
  
  }
  
  my_struct->hg_c_mar_a = 0;
  
  my_struct->bird_fh = NULL;
  my_struct->hg_fh = NULL;
  my_struct->mar_fh = NULL;

}












void do_hack_converge_w_struct( SS *my_struct )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  char bird_fname[ BUFFER ], hg_fname[ BUFFER ], mar_fname[ BUFFER ];
  
  get_hack_converge_input_fname( bird_fname, BIRD );
  get_hack_converge_input_fname( hg_fname, HM );
  get_hack_converge_input_fname( mar_fname, MARSUPIAL );
  
  my_struct->bird_fh = (FILE *)open_text_file_reading( bird_fname, "" );
  my_struct->hg_fh = (FILE *)open_text_file_reading( hg_fname, "" );
  my_struct->mar_fh = (FILE *)open_text_file_reading( mar_fname, "" );
  
  do_hack_converge_w_fhs( my_struct );
  
  close_file( my_struct->bird_fh );
  close_file( my_struct->hg_fh );
  close_file( my_struct->mar_fh );
  
  print_hack_converge( my_struct );

}














void get_hack_converge_input_fname( char fname[], char spec[] )
{
  sprintf( fname, "data/ft_hg19/occur/%s/%s.ft_occur.column.txt",
    spec, spec );
}














void do_hack_converge_w_fhs( SS *my_struct )
{
 
  char hg_line[ LINE_BUFFER ], bird_line[ LINE_BUFFER ], mar_line[ LINE_BUFFER ];
  
  while ( fgets( hg_line, LINE_BUFFER, my_struct->hg_fh ) != NULL ) {

    fgets( bird_line, LINE_BUFFER, my_struct->bird_fh );
    fgets( mar_line, LINE_BUFFER, my_struct->mar_fh );

    do_hack_converge_single_gene( my_struct, hg_line, bird_line, mar_line );
    
  }

}













void do_hack_converge_single_gene( SS *my_struct, char hg_line[], char bird_line[],
  char mar_line[] )
{

  char hg_entry[ BUFFER ], bird_entry[ BUFFER ], mar_entry[ BUFFER ];
  
  sscanf( hg_line, "%*s%*s%s", hg_entry );
  sscanf( bird_line, "%*s%*s%s", bird_entry );
  sscanf( mar_line, "%*s%*s%s", mar_entry );
  
  do_my_hack_converge_single_gene( my_struct, hg_entry, bird_entry, mar_entry );

}














void do_my_hack_converge_single_gene( SS *my_struct, char hg_entry[], 
  char bird_entry[], char mar_entry[] )
{
  
  get_bird_info_single_gene( my_struct, bird_entry );
  get_hg_info_single_gene( my_struct, hg_entry );
  get_mar_info_single_gene( my_struct, mar_entry );
  
  get_spec_nucl_overlap( my_struct, hg_entry, bird_entry, mar_entry );
  
//  get_hg_mar_a_to_c_cooccur( my_struct, hg_entry, mar_entry );
  
}













void get_bird_info_single_gene( SS *my_struct, char bird_entry[] )
{

  if ( bird_entry[ 0 ] == '-' )
    return;

  int nucl = 0;

  register char i;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
    if ( bird_entry[ i ] != '.' )
      nucl = 1;
  }
  
  my_struct->bird_w_nucl += nucl;

  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
    if ( bird_entry[ i ] != '.' )
      ++(my_struct->bird_nucl[ i ]);
  }
  
  ++(my_struct->bird_total);

}











void get_hg_info_single_gene( SS *my_struct, char hg_entry[] )
{

  if ( hg_entry[ 0 ] == '-' )
    return;

  int nucl = 0;

  register char i;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
    if ( hg_entry[ i ] != '.' )
      nucl = 1;
  }
  
  my_struct->hg_w_nucl += nucl;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
    if ( hg_entry[ i ] != '.' )
      ++(my_struct->hg_nucl[ i ]);
  }
  
  ++(my_struct->hg_total);

}











void get_mar_info_single_gene( SS *my_struct, char mar_entry[] )
{

  if ( mar_entry[ 0 ] == '-' )
    return;

  register char i;
  
  int nucl = 0;

  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
    if ( mar_entry[ i ] != '.' )
      nucl = 1;
  }
  
  my_struct->mar_w_nucl += nucl;

  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
    if ( mar_entry[ i ] != '.' )
      ++(my_struct->mar_nucl[ i ]);
  }
  
  ++(my_struct->mar_total);

}
















void get_hg_mar_a_to_c_cooccur( SS *my_struct, char hg_entry[], char mar_entry[] )
{

  char i;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
  
    if ( i == A_NUCL ) {
      if ( mar_entry[ i ] == '.' )
        return;
    }
    else {
      if ( mar_entry[ i ] != '.' )
        return;
    }
    
    if ( i == C_NUCL ) {
      if ( hg_entry[ i ] == '.' )
        return;
    }
    else {
      if ( hg_entry[ i ] != '.' )
        return;
    }
  
  }
  
  ++(my_struct->hg_c_mar_a);

}















void get_spec_nucl_overlap( SS *my_struct, char hg_entry[], char bird_entry[], 
  char mar_entry[] )
{

  if (( hg_entry[ 0 ] == '-' ) || ( bird_entry[ 0 ] == '-' ))
    return;
    
  get_spec_co_occurs( my_struct, hg_entry, bird_entry, mar_entry );

  register char i;
 
  int x = 0, y = 0;
 
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
  
    if ( hg_entry[ i ] != '.' )
      x = 1;
      
    if ( bird_entry[ i ] != '.' )
      y = 1;
  
  }
  
  my_struct->hg_bird_w_nucl = x * y;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
  
    if (( hg_entry[ i ] == '-' ) || ( hg_entry[ i ] == '.' ))
      continue;
      
    if (( bird_entry[ i ] == '-' ) || ( bird_entry[ i ] == '.' ))
      continue;
      
    ++(my_struct->overlap[ i ]);
  
  }

}














void get_spec_co_occurs( SS *my_struct, char hg_entry[], char bird_entry[], 
  char mar_entry[] )
{

  if (( hg_entry[ 0 ] == '-' ) || ( bird_entry[ 0 ] == '-' ) ||
    ( mar_entry[ 0 ] == '-' ))
  {
    return;
  }
  
  
  ++(my_struct->total);
  
  
  register char i, j, k;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
  
    if ( mar_entry[ i ] == '.' )
      continue;
  
    char m, total = 0;
    
    for ( m = 0; m < NUMB_OF_NUCL; m++ ) {
      if ( mar_entry[ m ] != '.' )
        ++total;
    }
    
//    if ( total != 1 )
//      continue;
  
    for ( j = 0; j < NUMB_OF_NUCL; j++ ) {
    
      if ( hg_entry[ j ] == '.' )
        continue;
    
      for ( k = 0; k < NUMB_OF_NUCL; k++ ) {
      
        if ( bird_entry[ k ] != '.' )
          ++(my_struct->co_occur[ i ][ j ][ k ]);
      
      }
    
    }
  
  }

}







/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/














void print_hack_converge( SS *my_struct )
{

  print_hack_converge_bird_info( my_struct );
  print_hack_converge_hg_info( my_struct );
  print_hack_converge_mar_info( my_struct );
  
  print_hack_converge_three_spec_co_occur( my_struct );
  
//  print_hack_converge_spec_overlap( my_struct );
  
}













void print_hack_converge_bird_info( SS *my_struct )
{

  printf( "Total bird orthologs :  %d\n", my_struct->bird_total );
  
  register char i;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
  
    printf( "Nucl (%d) :    %5d  (%.1f%%)\n",
      i,
      my_struct->bird_nucl[ i ],
      (float)my_struct->bird_nucl[ i ] * 100.0 / (float)my_struct->bird_total );
  
  }
  
  printf( "\n\n" );

}














void print_hack_converge_hg_info( SS *my_struct )
{

  printf( "Total human orthologs :  %d\n", my_struct->hg_total );
  
  register char i;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
  
    printf( "Nucl (%d) :    %5d  (%.1f%%)\n",
      i,
      my_struct->hg_nucl[ i ],
      (float)my_struct->hg_nucl[ i ] * 100.0 / (float)my_struct->hg_total );
  
  }

  printf( "\n\n" );

}













void print_hack_converge_mar_info( SS *my_struct )
{

  printf( "Total marsupial orthologs :  %d\n", my_struct->mar_total );
  
  register char i;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
  
    printf( "Nucl (%d) :    %5d  (%.1f%%)\n",
      i,
      my_struct->mar_nucl[ i ],
      (float)my_struct->mar_nucl[ i ] * 100.0 / (float)my_struct->mar_total );
  
  }

  printf( "\n\n" );

}

















void print_hack_converge_spec_overlap( SS *my_struct )
{

  char i;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
  
    printf( "Nucl (%d) overlap bird and hg :  %d   (%.1f%%)\n", i, 
      my_struct->overlap[ i ],
      0.0 );
  
  }

}















void print_hack_converge_three_spec_co_occur( SS *my_struct )
{

  char i;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ )
    print_hack_converge_three_spec_co_occur_single_mx( my_struct, i );

}













void print_hack_converge_three_spec_co_occur_single_mx( SS *my_struct, char i )
{

  print_sep( stdout );

  printf( "Non-eutherian nucl :  %d\n\n", i );
  
  printf( "\t\tBird\n" );
  printf( "HG\n" );
  
  char j, k;
  
  for ( j = 0; j < NUMB_OF_NUCL; j++ ) {
  
    for ( k = 0; k < NUMB_OF_NUCL; k++ ) {
    
      printf( "   %5d", my_struct->co_occur[ i ][ j ][ k ] );
    
    }
    
    printf( "\n" );
  
  }

}













