#include "nucl.h"























char complement_nucl( char nucl )
{

  switch ( tolower( nucl )) {
  
  case 'a' :
    return 't';

  case 'c' :
    return 'g';
    
  case 'g' :
    return 'c';
    
  case 't' :
    return 'a';
    
  case '-' :
    return '-';
    
  case 'n' :
    return 'n';
    
  default :
    error( 700, "Unrecognized nucleotide type.\n" );
    break;
  
  }

}













void get_rev_complement_seq( char rev_compl[], char seq[] )
{

  int begin = strlen( seq ) - 1;
  
  register int i, j = 0;
  
  for ( i = begin; i >= 0; --i )
    /*  In this file, above:  */
    rev_compl[ j++ ] = complement_nucl( seq[ i ] );
    
  rev_compl[ j ] = '\0';

}












MF mf_queue_to_stack( MF mf_queue )
{

  MF mf_stack = 0, my_queue = mf_queue;
  
  register char i;
  
  for ( i = 0; i < MOTIF_LENGTH; i++ ) {

    mf_stack = mf_stack << BITS_PER_NUCL;
    mf_stack += my_queue % NUMB_OF_NUCL;
    
    my_queue = my_queue >> BITS_PER_NUCL;

  }
  
  return mf_stack;

}










MF mf_stack_to_queue( MF mf_stack )
{
  return mf_queue_to_stack( mf_stack );
}











int seq_is_empty( char seq[] )
{

  int end = strlen( seq );

  register int i;
  
  for ( i = 0; i < end; i++ ) {
    
    switch ( tolower( seq[ i ] )) {
      
    case 'a' :
    case 'c' :
    case 'g' :
    case 't' :
      return 0;
      
    default :
      break;  
      
    }
    
  }
  
  return 1;

}












int is_nucl( char x )
{

  switch ( tolower( x )) {
  
  case 'a' :
  case 'c' :
  case 'g' :
  case 't' :
  case 'n' :
    return 1;
    
  default :
    return 0;
  
  }

}










COORD get_numb_of_nucl_maf_lister( MAF_LIST *maf )
{

  COORD numb_of_nucl = 0;
  
  register int i = 0;
  
  while ( maf->my_maf->seq[ i ] != '\0' ) {
    /*  In this file, above:  */
    if ( is_nucl( maf->my_maf->seq[ i++ ] ))
      ++numb_of_nucl;
  }
  
  return numb_of_nucl;

}













/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/













MF get_mf_numb_from_string( char mf[], int length )
{

  MF numb = 0;

  register char i;
  
  for ( i = 0; i < length; i++ ) {
  
    numb = numb << BITS_PER_NUCL;
    
    /*  In this file, above:  */
    MF bp = get_nucl_numb_from_char( mf[ i ] );
    
    if ( bp == NON_NUCL )
      return NON_MF;
      
    numb += bp;
    
  }
  
  return numb;

}











char get_nucl_numb_from_char( char nucl )
{

  switch ( tolower( nucl )) {
  
  case 'a' :
    return A_NUCL;
    
  case 'c' :
    return C_NUCL;
    
  case 'g' :
    return G_NUCL;
    
  case 't' :
    return T_NUCL;
    
  case 'n' :
    return T_NUCL;
    
  default :
    printf( "char :  (%c)\n", nucl );
    printf( "Is motif set?\n" );
    error( 701, "Unknown nucleotide type.\n" );
    return NON_NUCL;
  
  }

}










char get_nucl_char_from_bp( MF bp )
{

  switch ( bp ) {
  
  case A_NUCL :
    return 'a';
    
  case C_NUCL :
    return 'c';
    
  case G_NUCL :
    return 'g';
    
  case T_NUCL :
    return 't';
    
  default :
    error( 703, "Unknown numeric bp.\n" );
    break;
  
  }

}







int bp_matches_nucl( MF bp, char nucl )
{

  switch ( tolower( nucl )) {
  
  case 'a' :
    if ( bp == A_NUCL )
      return 1;
    return 0;

  case 'c' :
    if ( bp == C_NUCL )
      return 1;
    return 0;
    
  case 'g' :
    if ( bp == G_NUCL )
      return 1;
    return 0;
    
  case 't' :
    if ( bp == T_NUCL )
      return 1;
    return 0;
    
  case 'n' :
    return 1;
    
  default :
    printf( "Char :  (%c)\n", nucl );
    error( 702, "Unknown bp nucl type.\n" );
    break;
  
  }

}








int mf_stack_equals_motif_struct( MOTIF *motif, MF mf_stack )
{

  /*  In this file, above:  */
  MF mf_queue = mf_stack_to_queue( mf_stack );

  register char i;
  
  for ( i = 0; i < MOTIF_LENGTH; i++ ) {
  
    char bp = mf_queue % NUMB_OF_NUCL;
    
    if ( ! bp_matches_nucl( bp, motif->cons[ i ] ))
      return 0;
      
    mf_queue = mf_queue >> BITS_PER_NUCL;
  
  }
  
  return 1;

}












void get_motif_str_from_mf_stack( char cons[], MF mf_stack )
{

  register char i = 0, j = MOTIF_LENGTH;
  
  while ( i++ < MOTIF_LENGTH ) {
  
    /*  In this file, above:  */
    cons[ --j ] = get_nucl_char_from_bp( mf_stack % NUMB_OF_NUCL );
    
    mf_stack = mf_stack >> BITS_PER_NUCL;
  
  }
  
  cons[ MOTIF_LENGTH ] = '\0';

}














void str_to_lower( char str[], char my_str[] )
{

  register int i = 0;
  
  while ( my_str[ i ] != '\0' ) {
    str[ i ] = tolower( my_str[ i ] );
    ++i;
  }

}














void set_motif_from_stack( MOTIF *motif, MF stack )
{

  motif->my_motif = stack;

  /*  In this file, above:  */
  get_motif_str_from_mf_stack( motif->cons, stack );

}













void set_motif_to_reverse_complement( MOTIF *rev, MOTIF *mf )
{

  /*  In this file, above:  */
  get_rev_complement_seq( rev->cons, mf->cons );
  
  /*  In this file, above:  */
  rev->my_motif = get_mf_numb_from_string( rev->cons, strlen( mf->cons ) );

}


















