#include "ancestor.h"
















#define TREE_INPUT "data/ft_hg19/tree/SpeciesHG19.phylogeny.txt"

#define DATA_FNAME_BASE "data/ft_hg19/occur"
#define DATA_FNAME_EXT "ft_occur.column.txt"





#define SPLIT_NODE "Eutheria"



















void do_ancestral_nucls( void )
{

  NODE_LIST *get_node_tree_w_fname( NODE_LIST *node_header, char fname[] );

  printf( "Creating ancestral nucleotide phylogeny...\n" );
  
  NODE_LIST *node_header = NULL;
  
  /*  In node.c:  */
  node_header = (NODE_LIST *)get_node_tree_w_fname( node_header, TREE_INPUT );
  
  /*  In node.c:  */
  print_node_tree( node_header );
  
  /*  In this file, below:  */
  do_ancestral_nucls_w_node_phylo( node_header );
  
  /*  In print.c:  */
  put_phylo_tree( node_header );
  
  kill_node_list( node_header );
  
  printf( "\n" );

}













void do_ancestral_nucls_w_node_phylo( NODE_LIST *node_header )
{

  /*  In node.c:  */
  set_split_node( node_header, SPLIT_NODE );
  
  /*  In this file, below:  */
  do_my_ancestral_nucls_w_node_phylo( node_header );
  
}













void do_my_ancestral_nucls_w_node_phylo( NODE_LIST *node_header )
{

  NODE_LIST *get_leaf_node_header( NODE_LIST *root );

  NODE_LIST *leaf_header = NULL;
  /*  In node.c:  */
  leaf_header = (NODE_LIST *)get_leaf_node_header( node_header );

  /*  In this file, below:  */
  do_ancestral_nucls_phylo_w_leaf_list( node_header, leaf_header );

}












void do_ancestral_nucls_phylo_w_leaf_list( NODE_LIST *node_header,
  NODE_LIST *leaf_header )
{

  FPOS_CELL *create_fpos_cell( FPOS_CELL *ff, INDEX nn );
  INDEX get_numb_of_leaves_leaf_header( NODE_LIST *leaf_header );
  
  /*  In node.c:  */
  INDEX numb_of_spec = get_numb_of_leaves_leaf_header( leaf_header );
  
  /*  In this file, below:  */
//  make_nucl_co_occur_node_data( leaf_header );

  /*  In this file, below:  */
  make_nucl_co_occur_tree_data( node_header );

  FPOS_CELL *fpos_cell = NULL;
  fpos_cell = (FPOS_CELL *)create_fpos_cell( fpos_cell, numb_of_spec );
  
  /*  In this file, below:  */
  do_my_ancestral_nucls_phylo_w_leaf_list( node_header, leaf_header, fpos_cell );
  
  kill_fpos_cell( fpos_cell );
  
}











void make_nucl_co_occur_tree_data( NODE_LIST *node )
{

  if ( node == NULL )
    return;

  make_nucl_co_occur_matrix_single_leaf( node );
  
  make_nucl_co_occur_tree_data_recursion( node );

}









void make_nucl_co_occur_tree_data_recursion( NODE_LIST *node )
{

  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    make_nucl_co_occur_tree_data( node->down[ i++ ] );

}









void make_nucl_co_occur_node_data( NODE_LIST *leaf_header )
{

  NODE_LIST *leaf = leaf_header;
  
  while ( leaf != NULL )
    make_nucl_co_occur_matrix_single_leaf_and_step( &leaf );
    
}














void make_nucl_co_occur_matrix_single_leaf_and_step( NODE_LIST **leaf )
{
  make_nucl_co_occur_matrix_single_leaf( *leaf );
  *leaf = (*leaf)->right;
}












void make_nucl_co_occur_matrix_single_leaf( NODE_LIST *leaf )
{

  INDEX n = get_numb_of_upstream_nodes( leaf );
  INDEX nn = n * NUMB_OF_NUCL * NUMB_OF_NUCL;
  
  leaf->my_node->data->numb_of_spec = n;
  
  leaf->my_node->data->x = (COUNT *)malloc( sizeof(COUNT) * nn );
  
  COUNT *ptr = leaf->my_node->data->x;
  
  register char i = 0;
  
  while ( i++ < nn )
    *ptr++ = 0;
  
  leaf->my_node->data->i = NUMB_OF_NUCL;
  leaf->my_node->data->j = NUMB_OF_NUCL;

}












INDEX get_numb_of_upstream_nodes( NODE_LIST *leaf )
{

  INDEX numb = 0;
  
  NODE_LIST *node = leaf;
  
  while ( node != NULL ) {
    ++numb;
    node = node->up;
  }
  
  return numb;

}











void do_my_ancestral_nucls_phylo_w_leaf_list( NODE_LIST *node_header,
  NODE_LIST *leaf_header, FPOS_CELL *fpos_cell )
{

  int still_going = 1; 
  SEQ seq_numb = 0;
   
  while ( still_going )
    still_going = do_ancestral_nucls_leaf_list_single_gene( node_header, leaf_header, 
      fpos_cell, ++seq_numb );
      
  set_node_data_total_vals( node_header );

}












int do_ancestral_nucls_leaf_list_single_gene( NODE_LIST *node_header,
  NODE_LIST *leaf_header, FPOS_CELL *fpos_cell, SEQ seq_numb )
{

  initialize_ancestral_nucls( node_header );

  int still_going = 1;
  INDEX spec_numb = 0;

  NODE_LIST *leaf = leaf_header;
  
  while (( still_going ) && ( leaf != NULL ))
    still_going = do_ancestral_nucls_single_leaf_and_step( &leaf, fpos_cell,
      seq_numb, spec_numb++ );
      
  if ( ! still_going )
    return 0;

  printf( "Seq :  %d\n", seq_numb );
  
  do_ancestral_nucls_all_nodes_set_leaves( node_header );
  
//  update_leaf_co_occur_data_single_gene( leaf_header );
  
  update_tree_co_occur_data_single_gene( node_header );

  return 1;

}












void initialize_ancestral_nucls( NODE_LIST *node )
{

  initialize_my_ancestral_nucls( node );
  
  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    initialize_ancestral_nucls( node->down[ i++ ] );

}












void initialize_my_ancestral_nucls( NODE_LIST *node )
{

  register char i;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ )
    node->my_node->data->nucl[ i ] = PHYLO_UNSET;

}










int do_ancestral_nucls_single_leaf_and_step( NODE_LIST **leaf, FPOS_CELL *fpos_cell, 
  SEQ seq_numb, INDEX spec_numb )
{

  int still_going = do_ancestral_nucls_single_leaf( *leaf, fpos_cell, seq_numb,
    spec_numb );
  
  *leaf = (*leaf)->right;
  
  return still_going;

}














int do_ancestral_nucls_single_leaf( NODE_LIST *leaf, FPOS_CELL *fpos_cell,
  SEQ seq_numb, INDEX spec_numb )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  char fname[ BUFFER ];
  get_ancetral_phylo_column_data_fname( fname, leaf->my_node->label );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( fname, "" );
  
  int still_going = do_ancestral_nucls_single_leaf_w_fh( leaf, fpos_cell, seq_numb, 
    spec_numb, in );
  
  close_file( in );
  
  return still_going;

}












int do_ancestral_nucls_single_leaf_w_fh( NODE_LIST *leaf, FPOS_CELL *fpos_cell,
  SEQ seq_numb, INDEX spec_numb, FILE *in )
{

  FPOS get_position( FILE *ff );

  set_position( *(fpos_cell->array + spec_numb), in );
  
  char line[ BUFFER ];
  
  if ( fgets( line, BUFFER, in ) == NULL )
    return 0;  
    
  do_ancestral_nucls_single_leaf_single_line( leaf, seq_numb, line );
  
  *(fpos_cell->array + spec_numb) = get_position( in );
  
  return 1;
  
}











void do_ancestral_nucls_single_leaf_single_line( NODE_LIST *leaf, SEQ seq_numb,
  char line[] )
{
  
  char gene[ BUFFER ], nucl_occur[ BUFFER ];
  int seq;
  
  sscanf( line, "%d%s%s", &seq, gene, nucl_occur );
  
  if ( seq != seq_numb )
    error( 4600, "Sequence numbers do not match.\n" );
  
  set_single_leaf_w_line_data( leaf, nucl_occur );
  
}













void set_single_leaf_w_line_data( NODE_LIST *leaf, char nucl_occur[] )
{

  register char nucl;
  
  for ( nucl = 0; nucl < NUMB_OF_NUCL; nucl++ )
    leaf->my_node->data->nucl[ nucl ] = get_leaf_single_nucl( nucl_occur[ nucl ] );
    
  leaf->my_node->data->set = 1;

}













char get_leaf_single_nucl( char my_nucl )
{

  switch ( my_nucl ) {
  
  case '.' :
    return PHYLO_ABSENT;
    
  case '-' :
    return PHYLO_NO_ORTHO;
    
  default :
    return PHYLO_PRESENT;
  
  }

}















void set_node_data_total_vals( NODE_LIST *node )
{

  if ( node == NULL )
    return;
    
  COUNT *total_array = NULL;
  total_array = (COUNT *)malloc( sizeof(COUNT) * node->my_node->data->numb_of_spec );

  node->my_node->data->total = total_array;
  
  set_node_data_total_vals_single_node( node );
  
  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    set_node_data_total_vals( node->down[ i++ ] );

}










void set_node_data_total_vals_single_node( NODE_LIST *node )
{

  INDEX spec_numb = 0;
  
  while ( spec_numb < node->my_node->data->numb_of_spec )
    get_node_data_total_vals_single_ancestor( node, spec_numb++ );

}











void get_node_data_total_vals_single_ancestor( NODE_LIST *node, INDEX spec_numb )
{

  NODE_DATA *data = node->my_node->data;

  INDEX n = spec_numb * NUMB_OF_NUCL * NUMB_OF_NUCL;
  
  COUNT *mx = data->x + n;

  *(data->total + spec_numb) = get_node_data_total_single_mx( node, mx );

}











COUNT get_node_data_total_single_mx( NODE_LIST *node, COUNT *mx )
{

  COUNT x = 0, *total = mx;

  char n = NUMB_OF_NUCL * NUMB_OF_NUCL;

  register char i = 0;
  
  while ( i++ < n )
    x += *total++;
    
  return x;

}









/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/












void do_ancestral_nucls_all_nodes_set_leaves( NODE_LIST *node_header )
{

  do_my_ancestral_nucls_all_nodes( node_header );

}












void do_my_ancestral_nucls_all_nodes( NODE_LIST *node_header )
{
  do_ancestral_nucls_all_nodes_down_to_up( node_header );
//  do_ancestral_nucls_all_nodes_up_to_down( node_header );
}












void do_ancestral_nucls_all_nodes_down_to_up( NODE_LIST *node )
{

  if ( node->down[ 0 ] == NULL ) 
    return;

  register char i = 0;
  
  while ( node->down[ i ] != NULL ) 
    do_ancestral_nucls_all_nodes_down_to_up( node->down[ i++ ] );
  
//  if ( ! node->my_node->split )
    set_ancestral_nucl_array_from_downstream_nodes( node, node->down );
      
}













void set_ancestral_nucl_array_from_downstream_nodes( NODE_LIST *node, 
  NODE_LIST *down_nodes[] )
{
  
  register char nucl;
  
  for ( nucl = 0; nucl < NUMB_OF_NUCL; nucl++ )
    set_ancestral_nucl_array_single_site( node, down_nodes, nucl );
  
}











void set_ancestral_nucl_array_single_site( NODE_LIST *node, NODE_LIST *down_nodes[], 
  char nucl )
{

  NODE_DATA *x = node->my_node->data;

  STAT score = 0.0, total = 0.0;
    
  register char i = 0;
  
  while ( down_nodes[ i ] != NULL ) {
  
    if ( down_nodes[ i ]->my_node->split ) {
      ++i;
      continue;
    }
  
    switch ( down_nodes[ i ]->my_node->data->nucl[ nucl ] ) {
    
    case PHYLO_UNSET :
      printf( "%s\n", down_nodes[ i ]->my_node->label );
      error( 4660, "Unset phylo value.\n" );
      
    case PHYLO_ABSENT :
      score += 0.0;
      ++total;
      break;
      
    case PHYLO_PRESENT :
      score += 1.0;
      ++total;
      break;
      
    case PHYLO_UNKNOWN :
      score += 0.5;
      ++total;
      break;
      
    case PHYLO_NO_ORTHO :
      break;
      
    default :
      error( 4661, "Unrecognized phylo value.\n" ); 
      break;
    
    }
    
    ++i;
    
  }
  
  if ( total == 0.0 ) {
    x->nucl[ nucl ] = PHYLO_NO_ORTHO;
    return;
  }
  
  if ( score / total > 0.5 )
    x->nucl[ nucl ] = PHYLO_PRESENT;
    
  if ( score / total == 0.5 )
    x->nucl[ nucl ] = PHYLO_UNKNOWN;
    
  if ( score / total < 0.5 )
    x->nucl[ nucl ] = PHYLO_ABSENT;

}












char update_ancestral_nucl_downstream_node( char curr_node, char down_node )
{

  if (( curr_node == down_node ) || ( curr_node == PHYLO_UNSET ))
    return down_node;
    
  if ( down_node == PHYLO_UNSET )
    return curr_node;
    
  return update_my_ancestral_nucl_downstream_node( curr_node, down_node );
  
}













char update_my_ancestral_nucl_downstream_node( char curr_node, char down_node )
{

  switch ( curr_node ) {
  
  case PHYLO_ABSENT :
    return update_ancestral_nucl_phylo_absent( down_node );
    
  case PHYLO_PRESENT :
    return update_ancestral_nucl_phylo_present( down_node );
    
  case PHYLO_UNKNOWN :
    return update_ancestral_nucl_phylo_unknown( down_node );
    
  case PHYLO_NO_ORTHO :
    return update_ancestral_nucl_phylo_no_ortho( down_node );
    
  case PHYLO_UNSET :
    return down_node;

  default :
    error( 4620, "Unrecognized ancestral nucleotide version.\n" );
    break;
    
  }
  
}





  
  
  
  
  
  
char update_ancestral_nucl_phylo_absent( char nucl )
{

  switch ( nucl ) {
    
  case PHYLO_UNSET :
    return PHYLO_ABSENT;
      
  case PHYLO_PRESENT :
    return PHYLO_UNKNOWN;
      
  case PHYLO_UNKNOWN :
    return PHYLO_ABSENT;
    
  case PHYLO_NO_ORTHO :
    return PHYLO_ABSENT;
      
  default :
    error( 4611, "Unrecognized ancestral nucleotide version.\n" );
    break;
      
  }
  
}












char update_ancestral_nucl_phylo_present( char nucl )
{
  
  switch ( nucl ) {
    
  case PHYLO_UNSET :
    return PHYLO_PRESENT;

  case PHYLO_ABSENT :
    return PHYLO_UNKNOWN;
      
  case PHYLO_UNKNOWN :
    return PHYLO_PRESENT;
    
  case PHYLO_NO_ORTHO :
    return PHYLO_PRESENT;
      
  default :
    error( 4613, "Unrecognized ancestral nucleotide version.\n" );
    break;
    
  }
  
}
  
    
    











char update_ancestral_nucl_phylo_unknown( char nucl )
{

  switch ( nucl ) {
  
  case PHYLO_UNSET :
    return PHYLO_UNKNOWN;
    
  case PHYLO_PRESENT :
    return PHYLO_PRESENT;
 
  case PHYLO_ABSENT :
    return PHYLO_ABSENT;

  case PHYLO_NO_ORTHO :
    return PHYLO_UNKNOWN;
    
  default :
    error( 4614, "Unrecognized ancestral nucl version.\n" );
  
  }

}














char update_ancestral_nucl_phylo_no_ortho( char nucl )
{

  switch ( nucl ) {
  
  case PHYLO_UNSET :
    return PHYLO_UNKNOWN;
    
  case PHYLO_ABSENT :
    return PHYLO_ABSENT;
    
  case PHYLO_PRESENT :
    return PHYLO_PRESENT;
  
  case PHYLO_UNKNOWN :
    return PHYLO_UNKNOWN;
    
  default :
    error( 4615, "Unrecognized nucl version.\n" );
    break;
  
  }

}










void do_ancestral_nucls_all_nodes_up_to_down( NODE_LIST *node )  
{

  if ( node->down[ 0 ] == NULL )
    return;
    
  if ( node->up != NULL )
    do_my_ancestral_nucls_up_to_down( node );
    
  do_ancestral_nucls_up_to_down_recursion( node );
  
}












void do_ancestral_nucls_up_to_down_recursion( NODE_LIST *node )
{
  
  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    do_ancestral_nucls_all_nodes_up_to_down( node->down[ i++ ] );

}













void do_my_ancestral_nucls_up_to_down( NODE_LIST *node )
{

  if (( node->my_node->split ) || ( node->up->my_node->split ))
    return;
    
  register char i;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ )
    do_ancestral_nucls_up_to_down_single_nucl( node, i );
    
}












void do_ancestral_nucls_up_to_down_single_nucl( NODE_LIST *node, char nucl )
{

  char x = node->my_node->data->nucl[ nucl ];
  char y = node->up->my_node->data->nucl[ nucl ];

  node->my_node->data->nucl[ nucl ] = nucl_up_to_down( x, y );

}













char nucl_up_to_down( char curr_nucl, char up_nucl )
{

  if (( curr_nucl == PHYLO_UNKNOWN ) && ( up_nucl == PHYLO_PRESENT ))
    return PHYLO_PRESENT;
    
  if (( curr_nucl == PHYLO_UNKNOWN ) && ( up_nucl == PHYLO_ABSENT ))
    return PHYLO_ABSENT;
    
  return curr_nucl;

}







void update_tree_co_occur_data_single_gene( NODE_LIST *node )
{

  if ( node == NULL )
    return;
    
  INDEX spec_numb = 0;
  NODE_LIST *curr_node = node;
  
  while ( curr_node != NULL )
    update_leaf_co_occur_data_upstream_node_and_step( node, &curr_node, 
      spec_numb++ );
    
  update_tree_co_occur_data_single_gene_recursion( node );

}










void update_tree_co_occur_data_single_gene_recursion( NODE_LIST *node )
{

  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    update_tree_co_occur_data_single_gene( node->down[ i++ ] );

}













void update_leaf_co_occur_data_single_gene( NODE_LIST *leaf_header )
{

  NODE_LIST *leaf = leaf_header;
  
  while ( leaf != NULL )
    update_leaf_co_occur_data_single_leaf_and_step( &leaf );

}













void update_leaf_co_occur_data_single_leaf_and_step( NODE_LIST **leaf )
{
  update_leaf_co_occur_data_single_leaf( *leaf );
  *leaf = (*leaf)->right;
}












void update_leaf_co_occur_data_single_leaf( NODE_LIST *leaf )
{

  INDEX spec_numb = 0;

  NODE_LIST *node = leaf;
  
  while ( node != NULL )
    update_leaf_co_occur_data_upstream_node_and_step( leaf, &node, spec_numb++ );

}













void update_leaf_co_occur_data_upstream_node_and_step( NODE_LIST *leaf,
  NODE_LIST **node, INDEX spec_numb )
{
  update_leaf_co_occur_data_upstream_node( leaf, *node, spec_numb );
  *node = (*node)->up;
}












void update_leaf_co_occur_data_upstream_node( NODE_LIST *leaf, NODE_LIST *node,
  INDEX spec_numb )
{

  NODE_DATA *data = leaf->my_node->data;

  COUNT *mx = data->x + (spec_numb * NUMB_OF_NUCL * NUMB_OF_NUCL);
  
  update_leaf_co_occur_data_upstream_node_matrix( leaf, node, mx );

}











void update_leaf_co_occur_data_upstream_node_matrix( NODE_LIST *leaf,
  NODE_LIST *node, COUNT *mx )
{

  NODE_DATA *a = leaf->my_node->data;
  NODE_DATA *b = node->my_node->data;

  COUNT *elem = mx;
  
  register char i, j;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
  
    COUNT x = ( a->nucl[ i ] == PHYLO_PRESENT ) ? (1) : (0);

    for ( j = 0; j < NUMB_OF_NUCL; j++ ) {
    
      COUNT y = ( b->nucl[ j ] == PHYLO_PRESENT ) ? (1) : (0);
      
      *elem += x * y;
      
      ++elem;
    
    }
  
  }

}










/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/













void get_ancetral_phylo_column_data_fname( char fname[], char spec[] )
{
  sprintf( fname, "%s/%s/%s.%s", DATA_FNAME_BASE, spec, spec, DATA_FNAME_EXT );
}






















