#include "asy.h"















#define EUTH_OUTPUT_CONS_FNAME "data/ft_hg19/gene/eutherian.cons.gene.txt"
#define OUTGROUP_OUTPUT_CONS_FNAME "data/ft_hg19/gene/outgroup.cons.gene.txt"




#define MAJOR_SPEC "hg19"
#define MINOR_SPEC "monDom5"





#define HUMAN_STR "hg19"
#define MACAQUE_STR "rheMac2"
#define MARMOSET_STR "calJac1"
#define MOUSE_STR "mm9"
#define RAT_STR "rn4"
#define COW_STR "bosTau4"
#define DOG_STR "canFam2"


#define OPOSSUM_STR "monDom5"
#define WALLABY_STR "macEug1"
#define PLATYPUS_STR "ornAna1"






















void do_asymmetry_cons( void )
{

  printf( "Doing asymmetry conservation...\n" );
  
  STRUCT my_struct;
  
  initialize_my_my_struct( &my_struct );  
  
  do_my_asymmetry_cons( &my_struct );
  
  printf( "\n" );

}












void initialize_my_my_struct( STRUCT *my_struct )
{

  my_struct->euth_fh = NULL;
  my_struct->outgroup_fh = NULL;
  
  register int i, j;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
  
    for ( j = 0; j < NUMB_OF_NUCL; j++ ) {
    
      my_struct->mx[ i ][ j ].co_occur = 0;
      my_struct->mx[ i ][ j ].cons = 0;
      my_struct->mx[ i ][ j ].frac = 0.0;
      my_struct->mx[ i ][ j ].major_cons = 0;
      my_struct->mx[ i ][ j ].minor_cons = 0;
    
    }
  
  }
  
  my_struct->numb_of_orthologs = 0;
  
}














void do_my_asymmetry_cons( STRUCT *my_struct )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  my_struct->euth_fh = (FILE *)open_text_file_reading( EUTH_OUTPUT_CONS_FNAME, "" );

  my_struct->outgroup_fh = 
    (FILE *)open_text_file_reading( OUTGROUP_OUTPUT_CONS_FNAME, "" );
  
  do_asymmetry_cons_w_input_fhs( my_struct );
  
  close_file( my_struct->euth_fh );
  close_file( my_struct->outgroup_fh );

}











void do_asymmetry_cons_w_input_fhs( STRUCT *my_struct )
{

  char major_line[ LINE_BUFFER ], minor_line[ LINE_BUFFER ];
  
  while ( fgets( major_line, LINE_BUFFER, my_struct->euth_fh ) != NULL ) {
  
    if ( fgets( minor_line, LINE_BUFFER, my_struct->outgroup_fh ) == NULL )
      error( 4200, "Unexpected EOF of outgroup file.\n" );
      
    do_asymmetry_cons_single_line( my_struct, major_line, minor_line );
  
  }
  
  get_and_put_asymmetry_cons( my_struct );

}













void do_asymmetry_cons_single_line( STRUCT *my_struct, char major_line[],
  char minor_line[] )
{

  if ( major_line[ 0 ] == '#' )
    return;
    
  do_my_asymmetry_cons_single_line( my_struct, major_line, minor_line );

}












void do_my_asymmetry_cons_single_line( STRUCT *my_struct, char major_line[],
  char minor_line[] )
{

  int major_seq, minor_seq;
  char major_entry[ BUFFER ], minor_entry[ BUFFER ];
  
  sscanf( major_line, "%d%*s%*s%s", &major_seq, major_entry );
  sscanf( minor_line, "%d%*s%s", &minor_seq, minor_entry );
  
  do_asymmetry_cons_single_co_occur( my_struct, major_line, minor_line,
    major_entry, minor_entry );

}













void do_asymmetry_cons_single_co_occur( STRUCT *my_struct, char major_line[],
  char minor_line[], char major_entry[], char minor_entry[] )
{

  if (( ! strcmp( major_entry, "...." )) || ( ! strcmp( minor_entry, "...." )))
    return;
    
  register int i, j;

  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
  
    for ( j = 0; j < NUMB_OF_NUCL; j++ ) {
    
      do_asymmetry_cons_single_nucl_co_occur( my_struct, major_line, minor_line,
        major_entry, minor_entry, i, j );
    
    }
  
  }

}












int numb_of_nucls_gene_entry( char entry[] )
{

  int numb = 0;
  
  register char i;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
    if ( entry[ i ] != '.' )
      ++numb;
  }
  
  return numb;

}











void do_asymmetry_cons_single_nucl_co_occur( STRUCT *my_struct, char major_line[],
  char minor_line[], char major_entry[], char minor_entry[], int i, int j )
{

  if (( major_entry[ i ] == '.' ) || ( minor_entry[ j ] == '.' ))
    return;
    
  do_asymmetry_cons_single_matching_nucl_pair( my_struct, major_line, minor_line,
    i, j );

}










void do_asymmetry_cons_single_matching_nucl_pair( STRUCT *my_struct,
  char major_line[], char minor_line[], int i, int j )
{

  ++(my_struct->mx[ i ][ j ].co_occur);
  
  do_my_asymmetry_cons_single_matching_nucl_pair( my_struct, major_line, minor_line,
    i, j );

}












void do_my_asymmetry_cons_single_matching_nucl_pair( STRUCT *my_struct,
  char major_line[], char minor_line[], int i, int j )
{
  
  int major_cons = line_shows_conserved_nucl( major_line, i, MAJOR_SPEC_NUMB );
  int minor_cons = line_shows_conserved_nucl( minor_line, j, MINOR_SPEC_NUMB );
  
  my_struct->mx[ i ][ j ].cons += major_cons * minor_cons;
  
}












int line_shows_conserved_nucl( char line[], int nucl, int numb_of_spec )
{

  char my_line[ LINE_BUFFER ];
  
  get_positioned_gene_line( my_line, line );
  
  return positioned_line_shows_conserved_nucl( my_line, nucl, numb_of_spec );

}











void get_positioned_gene_line( char my_line[], char line[] )
{

  int is_space = 1, numb_of_entries = 0;
  
  register int i = 0;
  
  while ( line[ i ] != '\0' ) {
  
    if (( is_space ) && ( ! isspace( line[ i ] ))) {
      if ( ++numb_of_entries == 3 )
        break;
    }
    
    if ( isspace( line[ i ] ))
      is_space = 1;
    else
      is_space = 0;
      
    ++i;
  
  }
  
  strcpy( my_line, line + i );
  
  printf( "Line :  %s\n", my_line );

}













int positioned_line_shows_conserved_nucl( char my_line[], int nucl, 
  int numb_of_spec )
{

  register int spec = 0, i = 0;
  
  while ( spec++ < numb_of_spec ) {
  
    while ( isspace( my_line[ i ] ))
      ++i;
      
    if ( my_line[ i + nucl ] == '.' )
      return 0;
      
    i += NUMB_OF_NUCL;
  
  }
  
  return 1;

}















void get_and_put_asymmetry_cons( STRUCT *my_struct )
{

  set_structure_fracs( my_struct );
  
  print_asymmetry_cons( my_struct );

}












void set_structure_fracs( STRUCT *my_struct )
{

  register int i, j;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
  
    for ( j = 0; j < NUMB_OF_NUCL; j++ ) {
    
      my_struct->mx[ i ][ j ].frac =
        (STAT)my_struct->mx[ i ][ j ].cons / (STAT)my_struct->mx[ i ][ j ].co_occur;
    
    }
  
  }

}













void print_asymmetry_cons( STRUCT *my_struct )
{

  register int i, j;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
  
    for ( j = 0; j < NUMB_OF_NUCL; j++ ) {
    
      printf( "%4d / %4d  (%4.1f%%)\t", 
        (int)my_struct->mx[ i ][ j ].cons,
        (int)my_struct->mx[ i ][ j ].co_occur,
        100.0 * my_struct->mx[ i ][ j ].frac );
    
    }
    
    printf( "\n" );
  
  }
  
  printf( "\n\n\n" );

}















/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/





















