#include "bias.h"

































void do_cons_bias( void )
{

  printf( "Doing conservation bias...\n" );
  
  do_my_cons_bias();
  
  printf( "\n" );

}











void do_my_cons_bias( void )
{

  ST my_st;
  
  initialize_st_bias( &my_st );
  
  do_cons_bias_w_struct( &my_st );

}











void initialize_st_bias( ST *my_st )
{

  my_st->outgroup_fh = NULL;
  my_st->euth_fh = NULL;
  
  register char i, j;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
  
    for ( j = 0; j < NUMB_OF_NUCL; j++ ) {    
      my_st->mx[ i ][ j ].both = 0;
      my_st->mx[ i ][ j ].joint = 0;
    }
  
  }

}










void do_cons_bias_w_struct( ST *my_st )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  my_st->euth_fh = (FILE *)open_text_file_reading( EUTH_FNAME, "" );
  my_st->outgroup_fh = (FILE *)open_text_file_reading( OUTGROUP_FNAME, "" );
  
  do_cons_bias_w_fh_struct( my_st );
  
  close_file( my_st->euth_fh );
  close_file( my_st->outgroup_fh );

}










void do_cons_bias_w_fh_struct( ST *my_st )
{

  get_cons_bias( my_st );
  
  print_cons_bias_results( my_st );
  
}









void get_cons_bias( ST *my_st )
{

  set_position( 0, my_st->euth_fh );
  set_position( 0, my_st->outgroup_fh );
  
  get_cons_bias_positioned_fhs( my_st );

}










void get_cons_bias_positioned_fhs( ST *my_st )
{

  char major_line[ LINE_BUFFER ], minor_line[ LINE_BUFFER ];
  
  while ( fgets( major_line, LINE_BUFFER, my_st->euth_fh ) != NULL ) {
  
    if ( fgets( minor_line, LINE_BUFFER, my_st->outgroup_fh ) == NULL )
      error( 4300, "Unexpected EOF in outgroup species file.\n" );
      
    do_cons_bias_single_line( my_st, major_line, minor_line );
  
  }

}











void do_cons_bias_single_line( ST *my_st, char euth_line[], char outgroup_line[] )
{

  if ( euth_line[ 0 ] == '#' )
    return;
    
  printf( "Line : %s", euth_line );
  printf( "Line : %s\n", outgroup_line );
  
  do_my_cons_bias_single_line( my_st, euth_line, outgroup_line );

}










void do_my_cons_bias_single_line( ST *my_st, char euth_line[],
  char outgroup_line[] )
{

  char euth_major_entry[ BUFFER ], outgroup_major_entry[ BUFFER ];

  get_major_species_bias_entries( euth_major_entry, outgroup_major_entry,
    euth_line, outgroup_line );
  
  do_cons_bias_single_line_major_entries( my_st, euth_line, outgroup_line,
    euth_major_entry, outgroup_major_entry );
  
}













void get_major_species_bias_entries( char euth_entry[], char outgroup_entry[],
  char euth_line[], char outgroup_line[] )
{

  int euth_numb = set_major_species_bias_single_entry( euth_entry, euth_line,
    EUTH_MAJOR_SPEC, "hg19", "mm9" );
  
  int outgroup_numb = set_major_species_bias_single_entry( outgroup_entry,
    outgroup_line, OUTGROUP_MAJOR_SPEC, "monDom5", "macEug1" );
  
  if ( euth_numb != outgroup_numb )
    error( 4301, "Eutherian and outgroup seq numbers do not match.\n" );
    
}










int set_major_species_bias_single_entry( char entry[], char line[],
  char spec[], char spec_a[], char spec_b[] )
{

  int numb;

  if ( ! strcmp( spec, spec_a ))    sscanf( line, "%d%*s%s", &numb, entry );
  if ( ! strcmp( spec, spec_b ))    sscanf( line, "%d%*s%*s%s", &numb, entry );
  
  if (( strcmp( spec, spec_a )) && ( strcmp( spec, spec_b )))
    error( 4310, "Unrecognized major species.\n" );
    
  return numb;

}

  







void do_cons_bias_single_line_major_entries( ST *my_st, char euth_line[],
  char outgroup_line[], char euth_major_entry[], char outgroup_major_entry[] )
{

  int euth_pos = get_minor_string_index( euth_line, euth_major_entry );
  int outgroup_pos = get_minor_string_index( outgroup_line, outgroup_major_entry );
  
  do_cons_bias_new_string( my_st, euth_line + euth_pos, outgroup_line + outgroup_pos, 
    euth_major_entry, outgroup_major_entry );

}










int get_minor_string_index( char line[], char major_entry[] )
{

  register INDEX i = 0;
  
  while ( line[ i ] != '\0' ) {
  
    if ( ! strncmp( line + i, major_entry, NUMB_OF_NUCL ))
      return i;
      
    ++i;
  
  }
  
  error( 4302, "Major species entry not found.\n" );

}












void do_cons_bias_new_string( ST *my_st, char euth_line[], char outgroup_line[], 
  char euth_entry[], char outgroup_entry[] )
{
  
  register char i, j;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
  
    for ( j = 0; j < NUMB_OF_NUCL; j++ ) {
    
      do_cons_bias_single_nucl_pair( my_st, euth_line, outgroup_line, euth_entry, 
        outgroup_entry, i, j );
    
    }
  
  }

}










void do_cons_bias_single_nucl_pair( ST *my_st, char euth_line[], 
  char outgroup_line[], char euth_entry[], char outgroup_entry[], char i, char j )
{

  COUNT a = conserved_nucl_single_branch( euth_line, i, EUTH_SPEC_NUMB );
  COUNT b = conserved_nucl_single_branch( outgroup_line, j, OUTGROUP_SPEC_NUMB );
  
  my_st->mx[ i ][ j ].both += a * b;
  
  if (( euth_entry[ i ] != '.' ) && ( outgroup_entry[ j ] != '.' ))
    ++(my_st->mx[ i ][ j ].joint);

}











COUNT conserved_nucl_single_branch( char line[], char nucl, int numb_of_spec )
{

  int spec = 0;
  
  while ( spec < numb_of_spec ) {
  
    if ( line[ (spec * (NUMB_OF_NUCL + 1)) + nucl ] == '.' )
      return 0;
      
    ++spec;
  
  }
  
  return 1;

}














void print_cons_bias_results( ST *my_st )
{

  char i, j;
  
  print_sep( stdout );
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
  
    for ( j = 0; j < NUMB_OF_NUCL; j++ ) {
    
      printf( "%5d / %5d  (%4.1f%%)    ",
        (int)my_st->mx[ i ][ j ].both,
        (int)my_st->mx[ i ][ j ].joint,
        (float)(my_st->mx[ i ][ j ].both * 100.0) / 
          (float)(my_st->mx[ i ][ j ].joint) );
    
    }
    
    printf( "\n" );
  
  }
  
  print_sep( stdout );
  
  printf( "\n\n" );
  
}











/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/























