#include "cons.h"
















#define HUMAN_STR "hg19"
#define MACAQUE_STR "rheMac2"
#define MARMOSET_STR "calJac1"
#define MOUSE_STR "mm9"
#define RAT_STR "rn4"
#define COW_STR "bosTau4"
#define DOG_STR "canFam2"

#define EUTH_OUTPUT_CONS_FNAME "data/ft_hg19/gene/eutherian.cons"



#define OPOSSUM_STR "monDom5"
#define WALLABY_STR "macEug1"
#define PLATYPUS_STR "ornAna1"

#define OUTGROUP_OUTPUT_CONS_FNAME "data/ft_hg19/gene/outgroup.cons"


#define GENE_LABEL_FNAME "data/ft_hg19/occur/hg19.geneLabel.txt"
#define INPUT_FNAME_BASE "data/ft_hg19/occur"
#define OCCUR_POSITION_FNAME "data/ft_hg19/occur/hg19.position"







#define GC_BOX "gggcgg"
#define GC_SITE 3





#define NUMB_OF_START_COLS 2













void do_cons_patterns( void )
{

  printf( "Executing conservation patterns...\n" );
  
  STRUCT my_struct;
  
  initialize_my_struct( &my_struct );
  
  do_my_cons_patterns( &my_struct );
  
  printf( "\n\n" );

}













void initialize_my_struct( STRUCT *my_struct )
{
  
  MY_GROUP *x = NULL, *y = NULL;
  
  x = (MY_GROUP *)malloc( sizeof(MY_GROUP) );
  y = (MY_GROUP *)malloc( sizeof(MY_GROUP) );
  
  my_struct->out = x;
  my_struct->euth = y;
  
  x->type = OUT_TYPE;
  y->type = EUTH_TYPE;
  
  register char i;
  
  for ( i = 0; i < GROUP_TYPE_BUFFER; i++ ) {
    x->x.list[ i ] = NULL;
    y->x.list[ i ] = NULL;
    x->x.spec[ i ][ 0 ] = '\0';
    y->x.spec[ i ][ 0 ] = '\0';
  }
  
  strcpy( x->x.spec[ PLATYPUS ], PLATYPUS_STR );
  strcpy( x->x.spec[ OPOSSUM ], OPOSSUM_STR );
  strcpy( x->x.spec[ WALLABY ], WALLABY_STR );
  
  strcpy( y->x.spec[ HUMAN ], HUMAN_STR );
  strcpy( y->x.spec[ MACAQUE ], MACAQUE_STR );
  strcpy( y->x.spec[ MARMOSET ], MARMOSET_STR );
  strcpy( y->x.spec[ MOUSE ], MOUSE_STR );
  strcpy( y->x.spec[ RAT ], RAT_STR );
  strcpy( y->x.spec[ COW ], COW_STR );
  strcpy( y->x.spec[ DOG ], DOG_STR );
  
}











void do_my_cons_patterns( STRUCT *my_struct )
{

  MOTIF mf = { .cons = GC_BOX };
  
  do_cons_patterns_w_motif( my_struct, &mf );
  
}












void do_cons_patterns_w_motif( STRUCT *my_struct, MOTIF *mf )
{

  get_cons_patterns_w_motif_outgroup( my_struct, mf );
  get_cons_patterns_w_motif_euth( my_struct, mf );
  
  do_cons_patterns_w_outgroup_gene_lists( my_struct, mf );
  
}













void get_cons_patterns_w_motif_outgroup( STRUCT *my_struct, MOTIF *mf )
{

  my_struct->out->x.list[ PLATYPUS ] = 
    (GENE_LIST *)get_gene_list_spec_name( my_struct->out->x.list[ PLATYPUS ], mf, 
    PLATYPUS_STR );
    
  my_struct->out->x.list[ OPOSSUM ] = 
    (GENE_LIST *)get_gene_list_spec_name( my_struct->out->x.list[ OPOSSUM ], mf, 
    OPOSSUM_STR );
    
  my_struct->out->x.list[ WALLABY ] = 
    (GENE_LIST *)get_gene_list_spec_name( my_struct->out->x.list[ WALLABY ], mf, 
    WALLABY_STR );
    
}










void get_cons_patterns_w_motif_euth( STRUCT *my_struct, MOTIF *mf )
{

  my_struct->euth->x.list[ HUMAN ] = 
    (GENE_LIST *)get_gene_list_spec_name( my_struct->euth->x.list[ HUMAN ], mf, 
    HUMAN_STR );
    
  my_struct->euth->x.list[ MACAQUE ] = 
    (GENE_LIST *)get_gene_list_spec_name( my_struct->euth->x.list[ MACAQUE ], mf, 
    MACAQUE_STR );
    
  my_struct->euth->x.list[ MARMOSET ] = 
    (GENE_LIST *)get_gene_list_spec_name( my_struct->euth->x.list[ MARMOSET ], mf,
    MARMOSET_STR );
    
  my_struct->euth->x.list[ MOUSE ] = 
    (GENE_LIST *)get_gene_list_spec_name( my_struct->euth->x.list[ MOUSE ], mf, 
    MOUSE_STR );
    
  my_struct->euth->x.list[ RAT ] = 
    (GENE_LIST *)get_gene_list_spec_name( my_struct->euth->x.list[ RAT ], mf, 
    RAT_STR );
    
  my_struct->euth->x.list[ COW ] = 
    (GENE_LIST *)get_gene_list_spec_name( my_struct->euth->x.list[ COW ], mf, 
    COW_STR );
    
  my_struct->euth->x.list[ DOG ] = 
    (GENE_LIST *)get_gene_list_spec_name( my_struct->euth->x.list[ DOG ], mf, 
    DOG_STR );
    
}











GENE_LIST *get_gene_list_spec_name( GENE_LIST *gene_header, MOTIF *mf,
  char spec[] )
{

  GENE_LIST *get_gene_for_rev_occur_list_fname( GENE_LIST *gene_header, MOTIF *mf, 
    char gene_fname[], char fname[] );

  char in_fname[ BUFFER ];
  
  printf( "\t\t\t\tSpecies :  %s\n", spec );
  
  /*  In this file, below:  */
  get_cons_pattern_input_fname_base( in_fname, spec );
  
  char bp_holder = mf->cons[ GC_SITE ];
  
  mf->cons[ GC_SITE ] = 'n';
  
  /*  In ../compare/occur_list.c:  */
  gene_header = (GENE_LIST *)get_gene_for_rev_occur_list_fname( gene_header, mf,
    GENE_LABEL_FNAME, in_fname );
    
  /*  In ../compare/mx_occur.c:  */  
  convert_gene_list_to_forward_strand( gene_header, GC_SITE );
    
  mf->cons[ GC_SITE ] = bp_holder;
  
  return gene_header;

}













void do_cons_patterns_w_outgroup_gene_lists( STRUCT *my_struct, MOTIF *mf )
{

  put_lineage_patterns_output_file( my_struct->out, mf, 
    OUTGROUP_OUTPUT_CONS_FNAME );
  
  put_lineage_patterns_output_file( my_struct->euth, mf,
    EUTH_OUTPUT_CONS_FNAME );
    
}












void put_lineage_patterns_output_file( MY_GROUP *my_group, MOTIF *mf, 
  char fname[] )
{

  FILE *open_binary_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );
  int get_my( void *x, size_t size, FILE *fh );
  
  FILE *label_fh = NULL, *out_fh = NULL;
  
  label_fh = (FILE *)open_binary_file_reading( GENE_LABEL_FNAME, "" );
  out_fh = (FILE *)open_text_file_writing( fname, "gene.txt" );
  
  my_group->ft_fh = (FILE *)open_binary_file_reading( OCCUR_POSITION_FNAME,
    "ft.txt" );
  my_group->ind_fh = (FILE *)open_binary_file_reading( OCCUR_POSITION_FNAME,
    "ind.txt" );
  
  int buffer = get_spec_buffer_my_group( my_group );
  
  put_lineage_species_from_group( my_group, buffer, out_fh );
  
  GENE_LABEL my_gene;
  
  SEQ seq_numb = 0;
  
  while ( get_my( &my_gene, sizeof(GENE_LABEL), label_fh ))
    put_lineage_patterns_output_file_gene_label( my_group, &my_gene, mf, 
      ++seq_numb, buffer, out_fh );
    
  close_file( label_fh );
  close_file( out_fh );
  close_file( my_group->ft_fh );
  close_file( my_group->ind_fh );
  
}









int get_spec_buffer_my_group( MY_GROUP *my_group )
{

  switch ( my_group->type ) {
  
  case OUT_TYPE :
    return OUTGROUP_BUFFER;
    
  case EUTH_TYPE :
    return EUTH_BUFFER;
    
  default :
    error( 4100, "Unrecognized group type in group structure.\n" );
    break;
  
  }

}











void put_lineage_species_from_group( MY_GROUP *my_group, int buffer, FILE *out_fh )
{

  fprintf( out_fh, "#SEQ\tGENE_NAME\t" );
  
  register int i;
  
  for ( i = 0; i < buffer; i++ )
    fprintf( out_fh, "%s\t", my_group->x.spec[ i ] );
    
  fprintf( out_fh, "\n" );

}









void put_lineage_patterns_output_file_gene_label( MY_GROUP *my_group,
  GENE_LABEL *my_gene, MOTIF *mf, SEQ seq_numb, int buffer, FILE *out_fh )
{
  
  printf( "%s\n", my_gene->gene );
  
  fprintf( out_fh, "%6d\t%15s\t", (int)seq_numb, my_gene->gene );
  
  register char i;
  
  for ( i = 0; i < buffer; i++ )
    put_lineage_patterns_output_file_single_spec( my_group->x.list[ i ],
      my_group, my_gene, mf, seq_numb, out_fh );
  
  fprintf( out_fh, "\n" );
  
}











void put_lineage_patterns_output_file_single_spec( GENE_LIST *gene_header,
  MY_GROUP *my_group, GENE_LABEL *my_gene, MOTIF *mf, SEQ seq_numb, FILE *out_fh )
{

  GENE_LIST *gene = gene_header;
  
  while (( gene != NULL ) && ( gene->my_gene->gene->seq_numb < my_gene->seq_numb ))
    gene = gene->next;
    
  if ( gene == NULL ) {
    fprintf( out_fh, "....\t" );  
    return;
  }
  
  if ( gene->my_gene->gene->seq_numb == my_gene->seq_numb )
    put_matching_gene_lister_lineage_patterns( gene, my_group, mf, out_fh );
    
  else
    fprintf( out_fh, "....\t" );

}











void put_matching_gene_lister_lineage_patterns( GENE_LIST *gene, MY_GROUP *my_group,
  MOTIF *mf, FILE *out_fh )
{

  printf( "\t\t\tmatch...\n" );
  
  char nucl[ NUMB_OF_NUCL ];  WIND my_wind;
  
  /*  In ../compare/mx_occur.c:  */
  initialize_nucl_array( nucl, NUMB_OF_NUCL );
  
  /*  In ../compare/mx_occur.c:  */
  set_window_from_motif( &my_wind, mf );
  
  /*  In this file, below:  */
  put_my_matching_gene_lister_lineage_patterns( gene, my_group, nucl, &my_wind, mf, 
    out_fh );

}













void put_my_matching_gene_lister_lineage_patterns( GENE_LIST *gene, 
  MY_GROUP *my_group, char nucl[], WIND *my_wind, MOTIF *mf, FILE *out_fh )
{

  WIND column_wind;
  
  SEQ seq_numb = gene->my_gene->gene->seq_numb;
  
  /*  In ../compare/mx_occur.c:  */
  get_column_window_position_files( &column_wind, my_wind, seq_numb, 
    my_group->ft_fh, my_group->ind_fh );
    
  /*  In this file, below:  */
  put_matching_gene_lister_lineage_w_wind( gene, &column_wind, my_group, nucl, 
    my_wind, mf, out_fh );

}












void put_matching_gene_lister_lineage_w_wind( GENE_LIST *gene, WIND *column_wind, 
  MY_GROUP *my_group, char nucl[], WIND *my_wind, MOTIF *mf, FILE *out_fh )
{

  /*  In ../compare/mx_occur.c:  */
  get_occurrence_arrays_within_window( nucl, gene, GC_SITE, column_wind );
  
  /*  In this file, below:  */
  put_matching_gene_lister_lineage_w_nucl_array( gene, nucl, out_fh );

}











void put_matching_gene_lister_lineage_w_nucl_array( GENE_LIST *gene, char nucl[],
  FILE *out_fh )
{

  char i;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ )
    put_matching_gene_lister_lineage_single_nucl( nucl[ i ], i, out_fh );
    
  fprintf( out_fh, "\t" );

}










void put_matching_gene_lister_lineage_single_nucl( char nucl, char site,
  FILE *out_fh )
{

  char get_nucl_char_from_bp( MF bp );

  if ( ! nucl )
    fprintf( out_fh, "." );
    
  else
    /*  In ../main/nucl.c:  */
    fprintf( out_fh, "%c", toupper( get_nucl_char_from_bp( site )) );

}












/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/










void count_conserved_nucls( void )
{

  char out_fname[ BUFFER ], euth_fname[ BUFFER ];

  sprintf( out_fname, "%s.gene.txt", OUTGROUP_OUTPUT_CONS_FNAME );
  printf( "File :  %s\n", out_fname );
  
  count_conserved_nucls_fname( out_fname );
  
  sprintf( euth_fname, "%s.gene.txt", EUTH_OUTPUT_CONS_FNAME );
  printf( "File :  %s\n", euth_fname );  
  
  count_conserved_nucls_fname( euth_fname );
  
  conserved_nucls_compare_lineages( euth_fname, out_fname );
  
}










void count_conserved_nucls_fname( char fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( fname, "" );
  
  count_conserved_nucls_fh( in );
  
  close_file( in );

}









void count_conserved_nucls_fh( FILE *in )
{

  char nucl;
  
  for ( nucl = 0; nucl < NUMB_OF_NUCL; nucl++ )
    count_conserved_nucls_single_nucl_type( nucl, in );
    
}











void count_conserved_nucls_single_nucl_type( char nucl_numb, FILE *in )
{

  char get_nucl_char_from_bp( MF bp );
  
  /*  In this file, below:  */
  int cons_numb = get_numb_of_conserved_nucls( nucl_numb, in );
  
  /*  In this file, below:  */
  int total_spec_numb = get_total_numb_single_spec_nucls( nucl_numb, in );
  
  /*  In ../main/nucl.c:  */
  printf( "\tNumber of conserved genes (%c) :  %d / %d  (%.1f%%)\n", 
    toupper( get_nucl_char_from_bp( nucl_numb ) ), cons_numb,
    total_spec_numb,
    (float)cons_numb * 100.0 / (float)total_spec_numb );

}










int get_numb_of_conserved_nucls( char nucl_numb, FILE *in )
{

  set_position( 0, in );
  
  char line[ LINE_BUFFER ];

  int numb = 0;
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL )
    numb += gene_line_is_conserved( line, nucl_numb, in );
    
  return numb;

}











int gene_line_is_conserved( char line[], char nucl_numb, FILE *in )
{  

  if ( line[ 0 ] == '#' )
    return 0;
    
  return my_gene_line_is_conserved( line, nucl_numb, in );
  
}









int my_gene_line_is_conserved( char line[], char nucl_numb, FILE *in )
{

  int col = 0, is_space = 1;
  
  register int i = 0;

  while ( line[ i ] != '\0' ) {
  
    if (( is_space ) && ( ! isspace( line[ i ] ))) {
      
      if ( col++ == NUMB_OF_START_COLS )
        break;
        
    }
      
    if ( isspace( line[ i ] ))
      is_space = 1;
    else
      is_space = 0;
      
    ++i;
  
  }

  return gene_line_is_conserved_set_line( line + i, nucl_numb, in );
  
}










int gene_line_is_conserved_set_line( char line[], char nucl_numb, FILE *in )
{

  char entry[ SHORT_BUFFER ];
  int is_space = 0;
  
  register int i = 0;
  
  while ( line[ i ] != '\0' ) {
  
    switch ( tolower( line[ i ] )) {
    
    case 'a' :
    case '.' :
    
      if ( line[ i + nucl_numb ] == '.' )
        return 0;
    
      i += NUMB_OF_NUCL;
      break;
      
    default :
      ++i;
      break;
    
    }
  
  }

  return 1;

}













int get_total_numb_single_spec_nucls( char nucl_numb, FILE *in )
{

  set_position( 0, in );

  int total_numb = 0;

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL )
    total_numb += line_is_nucl_single_first_spec( line, nucl_numb );

  return total_numb;  

}











int line_is_nucl_single_first_spec( char line[], char nucl_numb )
{

  if ( line[ 0 ] == '#' )
    return 0;
    
  char entry[ BUFFER ];
  
  sscanf( line, "%*s%*s%s", entry );
  
  return ( entry[ nucl_numb ] != '.' ) ? (1) : (0);

}











void conserved_nucls_compare_lineages( char euth_fname[], char out_fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *euth_fh = NULL, *out_fh = NULL;
  euth_fh = (FILE *)open_text_file_reading( euth_fname, "" );
  out_fh = (FILE *)open_text_file_reading( out_fname, "" );
  
  printf( "\n" );
  
  conserved_nucls_compare_lineages_w_fhs( euth_fh, out_fh );
  
  printf( "\n" );
  
  close_file( euth_fh );
  close_file( out_fh );

}











void conserved_nucls_compare_lineages_w_fhs( FILE *euth_fh, FILE *out_fh )
{
  
  int nucl_array[ NUMB_OF_NUCL ];
  
  char nucl;
  
  for ( nucl = 0; nucl < NUMB_OF_NUCL; nucl++ )
    conserved_nucls_compare_lineages_single_major_nucl( nucl_array, nucl,
      euth_fh, out_fh );
  
}











void conserved_nucls_compare_lineages_single_major_nucl( int nucl_array[],
  char nucl, FILE *euth_fh, FILE *out_fh )
{

  initialize_comp_nucl_array( nucl_array );
  
  set_position( 0, euth_fh );
  set_position( 0, out_fh );
  
  conserved_nucls_compare_lineages_my_major_nucl( nucl_array, nucl, euth_fh,
    out_fh );
  
  print_nucl_array_compare_lineages( nucl_array, nucl );
  
}












void initialize_comp_nucl_array( int nucl_array[] )
{

  char i;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ )
    nucl_array[ i ] = 0;

}










void conserved_nucls_compare_lineages_my_major_nucl( int nucl_array[],
  char nucl, FILE *euth_fh, FILE *out_fh )
{

  char major_line[ BUFFER ], minor_line[ BUFFER ];

  if ( fgets( major_line, BUFFER, euth_fh ) == NULL )
    error( 4110, "Unexpected empty file.\n" );
    
  if ( fgets( minor_line, BUFFER, out_fh ) == NULL ) 
    error( 4111, "Unexpected empty file.\n" );
    
  conserved_nucls_compare_across_lines( nucl_array, nucl, euth_fh, out_fh );

}











void conserved_nucls_compare_across_lines( int nucl_array[], char nucl,
  FILE *euth_fh, FILE *out_fh )
{

  char major_line[ LINE_BUFFER ], minor_line[ LINE_BUFFER ];
  
  while ( fgets( major_line, LINE_BUFFER, euth_fh ) != NULL ) {
  
    if ( fgets( minor_line, LINE_BUFFER, out_fh ) == NULL )
      error( 4112, "Unexpected end of minor species file.\n" );
      
    register char i;
    
    for ( i = 0; i < NUMB_OF_NUCL; i++ )
      nucl_array[ i ] += double_conservation( major_line, minor_line, nucl, i );      
  
  }
  
  if ( fgets( minor_line, LINE_BUFFER, out_fh ) != NULL )
    error( 4114, "Early end of file for major species.\n" );

}











int double_conservation( char major_line[], char minor_line[], char nucl, char i )
{

  /*  In this file, above:  */
  int major = my_gene_line_is_conserved( major_line, nucl, NULL );
  int minor = my_gene_line_is_conserved( minor_line, i, NULL );
  
  return major * minor;

}











void print_nucl_array_compare_lineages( int nucl_array[], char nucl )
{

  char get_nucl_char_from_bp( MF bp );
  
  /*  In ../main/nucl.c:  */
  printf( "Eutherian nucl '%c' :\t", toupper( get_nucl_char_from_bp( nucl )) );
  
  int i;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ )
    printf( "%5d\t", nucl_array[ i ] );
  
  printf( "\n" );

}









/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/










void get_cons_pattern_input_fname_base( char fname[], char spec[] )
{
  sprintf( fname, "%s/%s/%s.ft_occur", INPUT_FNAME_BASE, spec, spec );
}



























