#include "conv.h"



















#define TREE_INPUT "data/ft_hg19/tree/SpeciesHG19.phylogeny.txt"
















void do_gc_box_converge( void )
{

  printf( "Assessing GC box convergence...\n\n" );
  
  do_my_gc_box_converge();
  
  printf( "\n\n" );

}













void do_my_gc_box_converge( void )
{

  NODE_LIST *get_node_tree_w_fname( NODE_LIST *node_header, char fname[] );

  NODE_LIST *node_header = NULL;
  /*  In node.c:  */
  node_header = (NODE_LIST *)get_node_tree_w_fname( node_header, TREE_INPUT );
  
  /*  In node.c:  */
  print_node_tree( node_header );
  
  /*  In this file, below:  */
  do_gc_box_converge_w_phylo( node_header );
  
  kill_node_list( node_header );

}












void do_gc_box_converge_w_phylo( NODE_LIST *node_header )
{

  CONV *create_conv( CONV *xx );
  
  CONV *my_conv = NULL;
  my_conv = (CONV *)create_conv( my_conv );
  
  set_conv_struct( my_conv );
  
  do_gc_box_converge_w_struct( my_conv, node_header );
  
}













void set_conv_struct( CONV *my_conv )
{

  CONV_DATA *create_conv_data( CONV_DATA *xx );
  
  my_conv->major_conv = (CONV_DATA *)create_conv_data( my_conv->major_conv );
  my_conv->comp_conv = (CONV_DATA *)create_conv_data( my_conv->comp_conv );
  my_conv->spec_conv = (CONV_DATA *)create_conv_data( my_conv->spec_conv );

}














void do_gc_box_converge_w_struct( CONV *my_conv, NODE_LIST *node_header )
{

  /*  In node.c:  */
  set_split_node( node_header, "Eutheria" );
  set_split_node( node_header, "Neognathae" );
  
  /*  In this file, below:  */
  do_my_gc_box_converge_w_struct( my_conv, node_header );
  
  /*  In this file, below:  */
  print_gc_box_converge( my_conv );

}












void do_my_gc_box_converge_w_struct( CONV *my_conv, NODE_LIST *node_header )
{

  NODE_LIST *get_leaf_node_header( NODE_LIST *root );

  NODE_LIST *leaf_header = NULL;
  /*  In node.c:  */
  leaf_header = (NODE_LIST *)get_leaf_node_header( node_header );
  
  /*  In this file, below:  */
  do_gc_box_converge_set_leaf_list( my_conv, node_header, leaf_header );

}













void do_gc_box_converge_set_leaf_list( CONV *my_conv, NODE_LIST *node_header,
  NODE_LIST *leaf_header )
{

  FPOS_CELL *create_fpos_cell( FPOS_CELL *ff, INDEX nn );
  INDEX get_numb_of_leaves_leaf_header( NODE_LIST *leaf_header );
  
  /*  In node.c:  */
  INDEX numb_of_spec = get_numb_of_leaves_leaf_header( leaf_header );
  
  /*  In this file, below:  */
  make_nucl_co_occur_tree_data( node_header );

  FPOS_CELL *fpos_cell = NULL;
  fpos_cell = (FPOS_CELL *)create_fpos_cell( fpos_cell, numb_of_spec );
  
  /*  In this file, below:  */
  do_my_gc_converge_set_leaf_list( my_conv, node_header, leaf_header, fpos_cell );
  
  kill_fpos_cell( fpos_cell );
  
}













void do_my_gc_converge_set_leaf_list( CONV *my_conv, NODE_LIST *node_header,
  NODE_LIST *leaf_header, FPOS_CELL *fpos_cell )
{

  int still_going = 1;
  SEQ seq_numb = 0;
  
  while ( still_going )
    still_going = do_gc_converge_single_gene( my_conv, node_header, leaf_header,
      fpos_cell, ++seq_numb );

}












int do_gc_converge_single_gene( CONV *my_conv, NODE_LIST *node_header,
  NODE_LIST *leaf_header, FPOS_CELL *fpos_cell, SEQ seq_numb )
{

  /*  In ancestor.c:  */
  initialize_ancestral_nucls( node_header );
  
  /*  In this file, below:  */
  return do_my_gc_converge_single_gene( my_conv, node_header, leaf_header,
    fpos_cell, seq_numb );
    
}













int do_my_gc_converge_single_gene( CONV *my_conv, NODE_LIST *node_header,
  NODE_LIST *leaf_header, FPOS_CELL *fpos_cell, SEQ seq_numb )
{

  int do_ancestral_nucls_single_leaf_and_step( NODE_LIST **leaf, 
    FPOS_CELL *fpos_cell, SEQ seq_numb, INDEX spec_numb );

  int still_going = 1;
  INDEX spec_numb = 0;

  NODE_LIST *leaf = leaf_header;
  
  while (( still_going ) && ( leaf != NULL ))
    /*  In ancestor.c:  */
    still_going = do_ancestral_nucls_single_leaf_and_step( &leaf, fpos_cell,
      seq_numb, spec_numb++ );
      
  if ( ! still_going )
    return 0;

  printf( "Seq :  %d\n", seq_numb );
  
  do_gc_converge_single_gene_set_phylo_tree( my_conv, node_header );
  
  return 1;
  
}












void do_gc_converge_single_gene_set_phylo_tree( CONV *my_conv, 
  NODE_LIST *node_header )
{
  
  /*  In ancestor.c:  */
  do_ancestral_nucls_all_nodes_set_leaves( node_header );
  
  /*  In ancestor.c:  */
  update_tree_co_occur_data_single_gene( node_header );
  
  /*  In this file, below:  */
  update_conv_struct_for_gc_converge_single_gene( my_conv, node_header );

}













void update_conv_struct_for_gc_converge_single_gene( CONV *my_conv,
  NODE_LIST *node_header )
{
  
  update_conv_data_struct_major_conv( my_conv->major_conv, node_header );
  update_conv_data_struct_comp_conv( my_conv->comp_conv, node_header );
  update_conv_data_struct_spec_conv( my_conv->spec_conv, node_header );
  
}














/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/












void update_conv_data_struct_major_conv( CONV_DATA *x, NODE_LIST *node_header )
{

  NODE_LIST *get_node_from_label( NODE_LIST *node, char label[] );
  
  /*  In node.c:  */
  NODE_LIST *neo = (NODE_LIST *)get_node_from_label( node_header, "Neognathae" );
  NODE_LIST *pri = (NODE_LIST *)get_node_from_label( node_header, "Eutheria" );
  NODE_LIST *mam = (NODE_LIST *)get_node_from_label( node_header, "Mammalia" );
  NODE_LIST *the = (NODE_LIST *)get_node_from_label( node_header, "Theria" );
  NODE_LIST *amn = (NODE_LIST *)get_node_from_label( node_header, "Amniota" );
  NODE_LIST *gal = (NODE_LIST *)get_node_from_label( node_header, "galGal3" );
  NODE_LIST *tae = (NODE_LIST *)get_node_from_label( node_header, "taeGut1" );
  NODE_LIST *ano = (NODE_LIST *)get_node_from_label( node_header, "anoCar1" );
  
  /*  In this file, below:  */
  update_conv_data_struct_major_conv_w_nodes( x, neo, pri, mam, the, amn, gal,
    tae, ano );
  
}













void update_conv_data_struct_major_conv_w_nodes( CONV_DATA *x, NODE_LIST *neo,
  NODE_LIST *pri, NODE_LIST *mam, NODE_LIST *the, NODE_LIST *amn, NODE_LIST *gal,
  NODE_LIST *tae, NODE_LIST *ano )
{

  int bird = 0, mammals = 0, euth = 0;
  int bird_total, mammals_total, euth_total;

  register char i, j, k;
  
  for ( i = 0; i < DBUFF; i++ ) {
  
    if ( mam->my_node->data->nucl[ i ] == PHYLO_PRESENT )
      mammals = 1;
    else
      mammals = 0;
      
    if ( mam->my_node->data->nucl[ i ] != PHYLO_NO_ORTHO )
      mammals_total = 1;
    else
      mammals_total = 0;
    
      
    for ( j = 0; j < DBUFF; j++ ) {
    
      if (( tae->my_node->data->nucl[ j ] == PHYLO_PRESENT ) ||
        ( gal->my_node->data->nucl[ j ] == PHYLO_PRESENT ))
      {
        bird = 1;
      }
      else 
      {
        bird = 0;
      }
      
      if (( tae->my_node->data->nucl[ j ] != PHYLO_NO_ORTHO ) ||
        ( gal->my_node->data->nucl[ j ] != PHYLO_NO_ORTHO ))
      {
        bird_total = 1;
      }
      else
      {
        bird_total = 0;
      }
      
      for ( k = 0; k < DBUFF; k++ ) {
      
        if ( pri->my_node->data->nucl[ k ] == PHYLO_PRESENT )
          euth = 1;
        else
          euth = 0;
          
        if ( pri->my_node->data->nucl[ k ] != PHYLO_NO_ORTHO )
          euth_total = 1;
        else
          euth_total = 0;
          
        x->co_occur[ i ][ j ][ k ] += (mammals * bird * euth);
        
      }
    
    }
      
  }

}















void update_conv_data_struct_comp_conv( CONV_DATA *conv, NODE_LIST *node_header )
{

  NODE_LIST *get_node_from_label( NODE_LIST *node, char label[] );
  
  /*  In node.c:  */
//  NODE_LIST *mam = (NODE_LIST *)get_node_from_label( node_header, "Mammalia" );

  NODE_LIST *mam = (NODE_LIST *)get_node_from_label( node_header, "Amniota" );

//  NODE_LIST *mam = (NODE_LIST *)get_node_from_label( node_header, "Neognathae" );

  NODE_LIST *gal = (NODE_LIST *)get_node_from_label( node_header, "galGal3" );
  NODE_LIST *tae = (NODE_LIST *)get_node_from_label( node_header, "taeGut1" );
  NODE_LIST *hum = (NODE_LIST *)get_node_from_label( node_header, "mm9" );
  NODE_LIST *mou = (NODE_LIST *)get_node_from_label( node_header, "hg19" );
  NODE_LIST *cow = (NODE_LIST *)get_node_from_label( node_header, "bosTau4" );
  
  /*  In this file, below:  */
  update_conv_data_struct_comp_conv_w_nodes( conv, mam, gal, tae, hum, mou, cow );

}















void update_conv_data_struct_comp_conv_w_nodes( CONV_DATA *x, NODE_LIST *mam,
  NODE_LIST *gal, NODE_LIST *tae, NODE_LIST *hum, NODE_LIST *mou,
  NODE_LIST *cow )
{

  int mammals, bird, human, mouse;

  register char i, j, k;
  
  for ( i = 0; i < DBUFF; i++ ) {
  
    if ( mam->my_node->data->nucl[ i ] == PHYLO_PRESENT )
      mammals = 1;
    else
      mammals = 0;
      
    for ( j = 0; j < DBUFF; j++ ) {
    
      if ( mou->my_node->data->nucl[ j ] == PHYLO_PRESENT ) 
        mouse = 1;
      else
        mouse = 0;
        
      if ( mou->my_node->data->nucl[ j ] != PHYLO_NO_ORTHO )
        ++(x->spec_b_total[ j ]);
        
      if (( gal->my_node->data->nucl[ j ] == PHYLO_PRESENT ) ||
        ( tae->my_node->data->nucl[ j ] == PHYLO_PRESENT ))
        bird = 1;
      else
        bird = 0;  
        
      if (( gal->my_node->data->nucl[ j ] != PHYLO_NO_ORTHO ) ||
        ( tae->my_node->data->nucl[ j ] != PHYLO_NO_ORTHO ))
        ++(x->spec_a_total[ j ]);
        
      for ( k = 0; k < DBUFF; k++ ) {
      
        if ( hum->my_node->data->nucl[ k ] == PHYLO_PRESENT )
          human = 1;
        else
          human = 0;
          
        if ( hum->my_node->data->nucl[ k ] != PHYLO_NO_ORTHO )
          ++(x->spec_c_total[ k ]);
          
        x->co_occur[ i ][ j ][ k ] += (mammals * mouse * human);
        x->comp_occur[ i ][ j ][ k ] += (mammals * bird * human);
      
      }
    
    }      
  
  }

}

















void update_conv_data_struct_spec_conv( CONV_DATA *conv, NODE_LIST *node_header )
{

  NODE_LIST *get_node_from_label( NODE_LIST *node, char label[] );
  
  /*  In node.c:  */
  NODE_LIST *gal = (NODE_LIST *)get_node_from_label( node_header, "galGal3" );
  NODE_LIST *tae = (NODE_LIST *)get_node_from_label( node_header, "taeGut1" );
  
  NODE_LIST *hum = (NODE_LIST *)get_node_from_label( node_header, "hg19" );
  NODE_LIST *mou = (NODE_LIST *)get_node_from_label( node_header, "mm9" );
  
  NODE_LIST *cow = (NODE_LIST *)get_node_from_label( node_header, "bosTau4" );
  
  NODE_LIST *mon = (NODE_LIST *)get_node_from_label( node_header, "monDom5" );
  NODE_LIST *mac = (NODE_LIST *)get_node_from_label( node_header, "macEug1" );
  NODE_LIST *xen = (NODE_LIST *)get_node_from_label( node_header, "xenTro2" );
  NODE_LIST *ano = (NODE_LIST *)get_node_from_label( node_header, "anoCar1" );
  NODE_LIST *orn = (NODE_LIST *)get_node_from_label( node_header, "ornAna1" );
  
  NODE_LIST *amn = (NODE_LIST *)get_node_from_label( node_header, "Amniota" );
  NODE_LIST *mam = (NODE_LIST *)get_node_from_label( node_header, "Mammalia" );
  
  check_null( 12, gal, tae, hum, mou, mon, mac, xen, ano, orn, cow, amn, mam );
  
  /*  In this file, below:  */
  update_conv_data_struct_spec_conv_w_nodes( conv, gal, tae, hum, mou, mon, mac,
    xen, ano, orn, cow, amn, mam );

}












#define MIN_ANCESTRAL_MATCH 2














void update_conv_data_struct_spec_conv_w_nodes( CONV_DATA *conv, NODE_LIST *gal,
  NODE_LIST *tae, NODE_LIST *hum, NODE_LIST *mou, NODE_LIST *mon, NODE_LIST *mac,
  NODE_LIST *xen, NODE_LIST *ano, NODE_LIST *orn, NODE_LIST *cow, NODE_LIST *amn,
  NODE_LIST *mam )
{

  register char i;
  
  for ( i = 0; i < DBUFF; i++ ) {
  
    int ancestral_count = get_ancestral_vert_count( mon, mac, xen, ano, orn, 
      amn, mam, i );
      
    update_conv_data_struct_w_ancestral_count( conv, gal, tae, hum, mou, cow,
      ancestral_count, i );
  
  }

}











int get_ancestral_vert_count( NODE_LIST *mon, NODE_LIST *mac, NODE_LIST *xen,
  NODE_LIST *ano, NODE_LIST *orn, NODE_LIST *amn, NODE_LIST *mam, char nucl )
{

  int numb = 0;
  
  if ( present( xen, nucl ))
    ++numb;
  
  if ( present( mon, nucl ) || present( mac, nucl ))
    ++numb;
    
  if ( present( ano, nucl ))
    ++numb;
    
  if ( present( orn, nucl ))
    ++numb;
  
  
  return numb;

}











int present_only( NODE_LIST *x, char nucl )
{

  int z = 0, y = 0;
  
  register char i;
  
  for ( i = 0; i < DBUFF; i++ ) {
    if ( x->my_node->data->nucl[ i ] == PHYLO_PRESENT ) {
      if ( i == nucl )
        y = 1;
      ++z;
    }
  }
  
  return ( z == 1 && y ) ? (1) : (0);

}









int present( NODE_LIST *x, char nucl )
{

  return ( x->my_node->data->nucl[ nucl ] == PHYLO_PRESENT ) ? (1) : (0);

}










void update_conv_data_struct_w_ancestral_count( CONV_DATA *conv, NODE_LIST *gal, 
  NODE_LIST *tae, NODE_LIST *hum, NODE_LIST *mou, NODE_LIST *cow,
  int ancestral_count, char nucl )
{

  int bird, euth, mouse, human;

  register int i, j;
  
  for ( i = 0; i < DBUFF; i++ ) {
  
    bird = mouse = human = 0;
  
    if ( present( tae, i ) || present( gal, i ))
      bird = 1;

    if ( present( mou, i ))
      mouse = 1;
      
    if ( present( hum, i ))
      human = 1;
      
    for ( j = 0; j < DBUFF; j++ ) {
    
      euth = 0;
      
      if ( present( hum, j ))
        euth = 1;
        
      if ( ancestral_count >= MIN_ANCESTRAL_MATCH ) {
        conv->co_occur[ nucl ][ i ][ j ] += bird * euth;
        conv->comp_occur[ nucl ][ i ][ j ] += mouse * euth;
        
      }
        
    }
          
  }

}











/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
/*========================================================================*/
















void print_gc_box_converge( CONV *my_conv )
{

//  print_gc_box_converge_conv_data( my_conv->major_conv );
//  print_gc_box_converge_comp_data( my_conv->comp_conv );



  print_gc_box_converge_spec_data( my_conv->spec_conv );

}













void print_gc_box_converge_spec_data( CONV_DATA *conv )
{

  set_gc_box_converge_spec_data_notes( conv );
  
  set_gc_box_non_func_converge_spec_data( conv );

  print_gc_box_converge_spec_data_fh( conv, stdout );

}














void set_gc_box_converge_spec_data_notes( CONV_DATA *conv )
{

  char notes[ LINE_BUFFER ];

  strcpy( notes, "This file was created using the command:\n" );
  strcat( notes, "  phylo -m t -M v\n" );
  strcat( notes, "The purpose of this file is to determine convergent\n" );
  strcat( notes, "evolution between eutherians and birds, following\n" );
  strcat( notes, "a shift from gggAgg -> gggCgg in the GC box.\n" );
  strcat( notes, "The contents of this file were derived\n" );
  strcat( notes, "using orthologous sequences across multiple species.\n" );
  strcat( notes, "This includes all non-bird and non-eutherian lineages\n" );
  strcat( notes, "between birds and eutherians,\n" );
  strcat( notes, "including Anolis, Platypus, Opossum, and Wallaby.\n" );
  strcat( notes, "We determine, for each gene, whether a given nucleotide\n" );
  strcat( notes, "is present at the 4th site of the GC box:  'gggNgg'\n" );
  strcat( notes, "If a gene contains an occurrence of the GC box in the\n" );
  strcat( notes, "region of overrepresentation with the same nucleotide at\n" );
  strcat( notes, "this site within 2 of the three lineages:\n" );
  strcat( notes, "1)  Anolis\n" );
  strcat( notes, "2)  Platypus\n" );
  strcat( notes, "3)  Opossum OR Wallaby\n" );
  strcat( notes, "then we consider this gene for the following analysis.\n" );
  strcat( notes, "For such a gene, we conduct a pairwise species comparison\n" );
  strcat( notes, "across both human and mouse, as well as human and birds.\n" );
  strcat( notes, "" );
  
}













void set_gc_box_non_func_converge_spec_data( CONV_DATA *conv )
{

  register char i, j;
  
  for ( i = 0; i < DBUFF; i++ ) {
  
    for ( j = 0; j < DBUFF; j++ ) {
    
      conv->gt_co_occur[ i ][ j ] = 
        conv->co_occur[ G_NUCL ][ i ][ j ] + conv->co_occur[ T_NUCL ][ i ][ j ];
        
      conv->gt_comp_occur[ i ][ j ] =
        conv->comp_occur[ G_NUCL ][ i ][ j ] + conv->co_occur[ T_NUCL ][ i ][ j ];
    
    }
  
  }

}










void print_gc_box_converge_spec_data_fh( CONV_DATA *conv, FILE *fh )
{

  get_expected_gc_box_converge_mx( conv->exp_co_occur, conv->co_occur,
    conv->pval );
    
  print_gc_box_converge_single_mx_fh( conv, conv->co_occur, conv->exp_co_occur,
    conv->pval, "Human", "Bird", fh );

  fprintf( fh, "\n\n\n" );
  
  get_expected_gc_box_converge_mx( conv->exp_comp_occur, conv->comp_occur,
    conv->pval );

  print_gc_box_converge_single_mx_fh( conv, conv->comp_occur, conv->exp_comp_occur,
    conv->pval, "Human", "Mouse", fh );

}












void get_expected_gc_box_converge_mx( STAT exp[ DBUFF ][ DBUFF ][ DBUFF ],
  COUNT mx[ DBUFF ][ DBUFF ][ DBUFF ], PVAL pval[ DBUFF ][ DBUFF ][ DBUFF ] )
{

  PVAL get_pval_gc_box_conv( COUNT mx[ DBUFF ][ DBUFF ][ DBUFF ], STAT row_array[], 
    STAT col_array[], COUNT total, char i, char j, char k );

  STAT row_array[ DBUFF ], col_array[ DBUFF ], diag[ DBUFF ];
  STAT total, whole_total;
  
  register char i, j, k, m;
  
  for ( i = 0; i < DBUFF; i++ ) {
  
    for ( m = 0; m < DBUFF; m++ ) {
      row_array[ m ] = 0.0;
      col_array[ m ] = 0.0;
    }
    
    total = 0.0;
    whole_total = 0.0;
  
    for ( j = 0; j < DBUFF; j++ ) {
    
      for ( k = 0; k < DBUFF; k++ ) {
        row_array[ j ] += (STAT)mx[ i ][ j ][ k ];
        col_array[ j ] += (STAT)mx[ i ][ k ][ j ];
        whole_total += (STAT)mx[ i ][ j ][ k ];
      }
      
    }
    
    for ( j = 0; j < DBUFF; j++ ) {
    
      for ( k = 0; k < DBUFF; k++ ) {
      
        exp[ i ][ j ][ k ] = 
          row_array[ j ] * col_array[ k ] / whole_total;
          
        /*  In sig.c:  
        pval[ i ][ j ][ k ] = get_pval_gc_box_conv( mx, row_array, col_array,
          (COUNT)whole_total, i, j, k );
        */
        
      }
    
    }
  
  }

}














void print_gc_box_converge_single_mx_fh( CONV_DATA *conv, 
  COUNT mx[ DBUFF ][ DBUFF ][ DBUFF ], STAT exp[ DBUFF ][ DBUFF ][ DBUFF ],
  PVAL pval[ DBUFF ][ DBUFF ][ DBUFF ], char major_spec[], char minor_spec[], 
  FILE *fh )
{

  char get_nucl_char_from_bp( MF bp );

  register char i, j, k;
  
  for ( i = 0; i < DBUFF; i++ ) {
  
    print_sep( fh );
    
    fprintf( fh, "Ancestral element lineages :  %c\n",
      toupper( get_nucl_char_from_bp( i )) );
      
    print_dot_sep( fh );
    
    STAT conservation = get_conservation_mx( mx[ i ] );
    
    fprintf( fh, "Conservation :  %6.2f%%\n", conservation * 100.0 );
    
    print_dot_sep( fh );
    
    fprintf( fh, "\t\t%s\n", major_spec );
    fprintf( fh, "%s\n", minor_spec );
      
    for ( j = 0; j < DBUFF; j++ ) {
    
      for ( k = 0; k < DBUFF; k++ ) {
      
        fprintf( fh, "   %6d", (int)mx[ i ][ j ][ k ] );
      
      }
      
      fprintf( fh, "\n" );
    
    }
    
    print_dot_sep( fh );
    
    for ( j = 0; j < DBUFF; j++ ) {
    
      for ( k = 0; k < DBUFF; k++ ) {
      
        fprintf( fh, "   %6.3f", log( mx[ i ][ j ][ k ] / exp[ i ][ j ][ k ] ) );
      
      }
      
      fprintf( fh, "\n" );
    
    }
    
    print_dot_sep( fh );
    
    for ( j = 0; j < DBUFF; j++ ) {
    
      for ( k = 0; k < DBUFF; k++ ) {
      
        fprintf( fh, "   %6.1f", (float)exp[ i ][ j ][ k ] );
      
      }
      
      fprintf( fh, "\n" );
    
    }
    
  }
  
  print_sep( fh );

}













STAT get_conservation_mx( COUNT mx[ DBUFF ][ DBUFF ] )
{

  STAT total = 0.0, cons = 0.0;
  
  register char i, j;
  
  for ( i = 0; i < DBUFF; i++ ) {
  
    cons += mx[ i ][ i ];
  
    for ( j = 0; j < DBUFF; j++ )
      total += mx[ i ][ j ];
  
  }
  
  return cons / total;

}







void print_gc_box_converge_conv_data( CONV_DATA *conv )
{

  register char i, j, k;
  
  for ( i = 0; i < DBUFF; i++ ) {
  
    print_sep( stdout );
    
    printf( "Mammals   Nucl :  %d\n", i );
    print_dot_sep( stdout );
    
    for ( j = 0; j < DBUFF; j++ ) {
    
      for ( k = 0; k < DBUFF; k++ ) {
      
        printf( "  %8d", (int)conv->co_occur[ i ][ j ][ k ] );
      
      }
      
      printf( "\n" );
    
    }
    
    print_dot_sep( stdout );
    
    for ( j = 0; j < DBUFF; j++ ) {
    
      for ( k = 0; k < DBUFF; k++ ) {
      
//        printf( "  %8.3f", (float)conv->co_occur[ i ][ j ][ k ] * 100.0 /
//          (float)conv->total[ i ][ j ][ k ] );
      
      }
      
      printf( "\n" );
    
    }
    
    continue;
    
    print_dot_sep( stdout );
    
    for ( j = 0; j < DBUFF; j++ ) {
    
      for ( k = 0; k < DBUFF; k++ ) {
      
//        printf( "  %8d", (int)conv->total[ i ][ j ][ k ] );
      
      }
      
      printf( "\n" );
    
    }
    
    print_sep( stdout );
    
    printf( "\n\n" );
  
  }

}













void print_gc_box_converge_comp_data( CONV_DATA *conv )
{

  register char i, j, k;
  
  for ( i = 0; i < DBUFF; i++ ) {
  
    print_sep( stdout );
    
    printf( "Mammals   Nucl :  %d\n", i );
    print_dot_sep( stdout );
    
    for ( j = 0; j < DBUFF; j++ ) {
    
      for ( k = 0; k < DBUFF; k++ ) {
      
        printf( "  %8d", (int)conv->co_occur[ i ][ j ][ k ] );
      
      }
      
      printf( "\n" );
    
    }
        
  }
  
  print_sep( stdout );
  
  printf( "\n\n\n" );
  
  for ( i = 0; i < DBUFF; i++ ) {
  
    print_sep( stdout );
    
    printf( "Mammals  Nucl :  %d\n", i );
    print_dot_sep( stdout );
    
    for ( j = 0; j < DBUFF; j++ ) {
    
      for ( k = 0; k < DBUFF; k++ ) {
      
        printf( "  %8d", (int)conv->comp_occur[ i ][ j ][ k ] );
      
      }
      
      printf( "\n" );
    
    }
  
  }
  
}













