#include "node.h"

















#define TREE_INPUT "data/ft_hg19/tree/SpeciesHG19.phylogeny.txt"














void make_node_tree( void )
{

  printf( "Creating node tree...\n" );

  NODE_LIST *node_header = NULL;
  
  node_header = (NODE_LIST *)get_node_tree_w_fname( node_header, TREE_INPUT );
  
  printf( "\n" );
  
  print_node_tree( node_header );
  
  printf( "\n" );

}












NODE_LIST *get_node_tree_w_fname( NODE_LIST *node_header, char fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *fh = NULL;
  fh = (FILE *)open_text_file_reading( fname, "" );
  
  node_header = (NODE_LIST *)get_node_tree_w_fh( node_header, fh );
  
  close_file( fh );
  
  return node_header;

}













NODE_LIST *get_node_tree_w_fh( NODE_LIST *node_header, FILE *fh )
{

  NODE_LIST *leaf_header = NULL;
  leaf_header = (NODE_LIST *)get_leaf_node_list( leaf_header, fh );
  
  node_header = (NODE_LIST *)get_node_tree_w_species_leaf_list( node_header, 
    leaf_header, fh );
    
  get_node_tree_time( node_header );
  
  return node_header;

}












NODE_LIST *get_leaf_node_list( NODE_LIST *leaf_header, FILE *fh )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, fh ) != NULL ) {
  
    if ( line[ 0 ] == '=' )
      return leaf_header;
      
    if ( line[ 0 ] == '#' )
      continue;
      
    leaf_header = (NODE_LIST *)get_and_put_single_leaf_node_lister( leaf_header,
      line );
  
  }
  
  error( 4500, "Unexpected EOF w/in species list in phylogeny file.\n" );

}













NODE_LIST *get_and_put_single_leaf_node_lister( NODE_LIST *leaf_header,
  char line[] )
{

  NODE_LIST *create_node_lister( NODE_LIST *xx );

  NODE_LIST *node = NULL;
  node = (NODE_LIST *)create_node_lister( node );
  
  node = (NODE_LIST *)get_single_leaf_node_lister_from_line( node, line );
  
  return put_single_leaf_node_lister_into_leaf_list( leaf_header, node );

}













NODE_LIST *get_single_leaf_node_lister_from_line( NODE_LIST *node, char line[] )
{
  sscanf( line, "%s%s", node->my_node->label, node->my_node->spec );
  return node;
}














NODE_LIST *put_single_leaf_node_lister_into_leaf_list( NODE_LIST *leaf_header, 
  NODE_LIST *node )
{

  if ( leaf_header == NULL )
    return node;
    
  NODE_LIST *curr = leaf_header;
  
  while ( curr->right != NULL ) 
    curr = curr->right;

  node->left = curr;
  curr->right = node;
  
  return leaf_header;

}













NODE_LIST *get_node_tree_w_species_leaf_list( NODE_LIST *node_header, 
  NODE_LIST *leaf_header, FILE *fh )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, fh ) != NULL )
    get_node_tree_single_line( leaf_header, line, fh );
    
  return get_node_tree_header( leaf_header );

}












void get_node_tree_single_line( NODE_LIST *leaf_header, char line[], FILE *fh )
{

  if (( absent_line( line )) || ( line[ 0 ] == '#' ))
    return;
    
  if ( line[ 0 ] == '>' )
    get_node_tree_multi_branch_entry( leaf_header, line, fh );
    
  else
    get_my_node_tree_single_line( leaf_header, line );

}











int absent_line( char line[] )
{

  register char i = 0;
  
  while ( line[ i ] != '\0' ) {
    if ( ! isspace( line[ i++ ] ))
      return 0;
  }
  
  return 1;

}














void get_node_tree_multi_branch_entry( NODE_LIST *leaf_header, char line[],
  FILE *fh )
{

  char angle[ BUFFER ], lin[ BUFFER ], label[ BUFFER ];
  int numb;
  
  sscanf( line, "%s%d%s%s", angle, &numb, lin, label );
  
  if (( strcmp( angle, ">" )) || ( strcmp( lin, "Lineages" )))
    error( 4501, "Formatting error in multiple lineage label line.\n" );
    
  get_my_node_tree_multi_branch_entry( leaf_header, label, numb, fh );

}












void get_my_node_tree_multi_branch_entry( NODE_LIST *leaf_header, char label[],
  int numb, FILE *fh )
{

  NODE_LIST *create_node_lister( NODE_LIST *xx );
  
  NODE_LIST *node = NULL;
  node = (NODE_LIST *)create_node_lister( node );
  
  strcpy( node->my_node->label, label );
  
  set_node_tree_multi_branch_entry( leaf_header, node, numb, fh );
  
}













void set_node_tree_multi_branch_entry( NODE_LIST *leaf_header, NODE_LIST *node,
  int numb, FILE *fh )
{

  register int i = 0;
  
  while ( i < numb )
    get_node_tree_multi_branch_entry_single_line( leaf_header, node, i++, fh );
    
}














void get_node_tree_multi_branch_entry_single_line( NODE_LIST *leaf_header,
  NODE_LIST *node, int index, FILE *fh )
{

  char line[ BUFFER ], label[ BUFFER ];
  float length;

  if ( fgets( line, BUFFER, fh ) == NULL )
    error( 4502, "Unexpected EOF in multi-lineage entry.\n" );
    
  sscanf( line, "%s%f", label, &length );
  
  remove_brackets( label );

  NODE_LIST *a = NULL;
  a = (NODE_LIST *)find_node_lister_label_w_leaf_list( leaf_header, label );
  
  node->down[ index ] = a;
  
  a->up = node;
  a->my_node->length = (STAT)length;

}













void get_my_node_tree_single_line( NODE_LIST *leaf_header, char line[] )
{

  char spec1[ BUFFER ], spec2[ BUFFER ], ancestor[ BUFFER ];
  char spec1_length[ BUFFER ], spec2_length[ BUFFER ];
  
  sscanf( line, "%s%s%s%s%s", spec1, spec2, 
    spec1_length, spec2_length, ancestor );
    
  remove_brackets( spec1 );
  remove_brackets( spec2 );
  
  NODE_LIST *a = NULL, *b = NULL;
  
  a = (NODE_LIST *)find_node_lister_label_w_leaf_list( leaf_header, spec1 );
  b = (NODE_LIST *)find_node_lister_label_w_leaf_list( leaf_header, spec2 );
  
  if (( a == NULL ) || ( b == NULL )) 
    error( 4504, "Node label is absent from tree.\n" );
    
  build_single_node_from_downstream_nodes( a, b, atof( spec1_length ),
    atof( spec2_length ), ancestor );
  
}













void remove_brackets( char x[] )
{

  if ( x[ 0 ] != '(' )
    return;
    
  char y[ BUFFER ];
  
  strcpy( y, x );
  
  register char i = 0;
  
  while ( y[ i + 1 ] != '\0' ) {
  
    if ( y[ i + 1 ] == ')' ) 
      x[ i ] = '\0';
  
    else
      x[ i ] = y[ i + 1 ];
      
    ++i;
  
  }

}













void build_single_node_from_downstream_nodes( NODE_LIST *a, NODE_LIST *b, 
  TIME length_a, TIME length_b, char ancestor[] )
{

  NODE_LIST *create_node_lister( NODE_LIST *xx );
  
  NODE_LIST *x = NULL;
  x = (NODE_LIST *)create_node_lister( x );
  
  strcpy( x->my_node->label, ancestor );
  
  a->my_node->length = length_a;
  b->my_node->length = length_b;
  
  put_single_node_to_downstream_nodes( a, b, x );

}












void put_single_node_to_downstream_nodes( NODE_LIST *a, NODE_LIST *b,
  NODE_LIST *x )
{

  a->up = x;
  b->up = x;
  
  x->down[ 0 ] = a;
  x->down[ 1 ] = b;

}

















/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/
/*=========================================================================*/














NODE_LIST *find_node_lister_label_w_leaf_list( NODE_LIST *leaf_header, 
  char label[] )
{

  NODE_LIST *match = NULL, *leaf = leaf_header;
  
  while ( leaf != NULL ) {
  
    match = (NODE_LIST *)find_node_lister_label_match_upstream( leaf, label );
    
    if ( match != NULL ) 
      return match;
      
    leaf = leaf->right;
  
  }
  
  return NULL;

}












NODE_LIST *find_node_lister_label_match_upstream( NODE_LIST *node, char label[] )
{

  if ( node == NULL )
    return NULL;
    
  if ( ! strcmp( node->my_node->label, label ))
    return node;
    
  return find_node_lister_label_match_upstream( node->up, label );

}












NODE_LIST *get_node_tree_header( NODE_LIST *node )
{

  NODE_LIST *nn = node;
  
  while ( nn->up != NULL )
    nn = nn->up;
    
  return nn;
  
}











NODE_LIST *get_leaf_node_header( NODE_LIST *root )
{

  if ( root == NULL )
    return NULL;

  NODE_LIST *node = root;
  
  while ( node->down[ 0 ] != NULL )
    node = node->down[ 0 ];
    
  while ( node->left != NULL )
    node = node->left;
    
  return node;

}











TIME get_node_tree_time( NODE_LIST *node )
{

  if ( node->down[ 0 ] == NULL )
    return 0.0;
    
  TIME x[ NODE_BUFFER ];
  
  int numb_down_nodes = get_node_time_array( x, node );
  
  check_node_time_array( x, numb_down_nodes );
  
  node->my_node->time = x[ 0 ];
  
  return node->my_node->time;
  
}












int get_node_time_array( TIME x[], NODE_LIST *node )
{

  register char i = 0;
  
  while ( node->down[ i ] != NULL ) {
    x[ i ] = current_node_tree_time( node, i );
    ++i;
  }
  
  return i;
  
}










TIME current_node_tree_time( NODE_LIST *node, char i )
{
  return  node->down[ i ]->my_node->length + get_node_tree_time( node->down[ i ] );
}
  











void check_node_time_array( TIME x[], int numb_down_nodes )
{

  register char j;

  for ( j = 1; j < numb_down_nodes; j++ ) {
  
    TIME diff = x[ 0 ] - x[ j ];
  
    if (( diff > 1e-4 ) || ( diff < -1.0e-4 ) ) 
      error( 4511, "Unmatching phylo tree times.\n" );
  
  }
  
}













void print_node_tree( NODE_LIST *node )
{
  print_node_tree_fh( node, stdout );
}











void print_node_tree_fh( NODE_LIST *node, FILE *fh )
{

  print_sep( fh );
  
  print_my_node_tree_fh( node, fh );
  
}











void print_my_node_tree_fh( NODE_LIST *node, FILE *fh )
{

  if ( node == NULL )
    return;
    
  print_node_tree_existing_node_fh( node, fh );

}











void print_node_tree_existing_node_fh( NODE_LIST *node, FILE *fh )
{

  print_curr_ancestor_node_fh( node, fh );
  
  fprintf( fh, "\n" );

  if ( node->down[ 0 ] != NULL )
    print_node_tree_leaves_ancestor_node( node, fh );
  
  print_sep( fh );
  
  do_print_node_recursion( node, fh );

}














void print_curr_ancestor_node_fh( NODE_LIST *node, FILE *fh )
{

  fprintf( fh, "%-15s\t\t", node->my_node->label );
  
  if ( node->my_node->time != 0.0 ) 
    fprintf( fh, "%.1f mya\n", (float)node->my_node->time );
  
  else 
    fprintf( fh, "\n" );

}









void print_node_tree_leaves_ancestor_node( NODE_LIST *node, FILE *fh )
{
  
  if ( node->down[ 0 ] == NULL ) {
    fprintf( fh, "\t%-15s\t(%s)\n", node->my_node->label, node->my_node->spec );
    return;
  }
    
  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    print_node_tree_leaves_ancestor_node( node->down[ i++ ], fh );
  
}










void do_print_node_recursion( NODE_LIST *node, FILE *fh )
{

  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    /*  In this file, above:  */
    print_my_node_tree_fh( node->down[ i++ ], fh );

}












void set_split_node( NODE_LIST *node, char split_label[] )
{

  if ( node == NULL )
    return;
    
  if ( ! strcmp( node->my_node->label, split_label ))
    node->my_node->split = 1;

  do_split_mode_recursion( node, split_label );

}











void do_split_mode_recursion( NODE_LIST *node, char split_label[] )
{

  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    set_split_node( node->down[ i++ ], split_label );

}













INDEX get_numb_of_leaves_leaf_header( NODE_LIST *leaf_header )
{

  INDEX numb = 0;
  
  NODE_LIST *leaf = leaf_header;
  
  while ( leaf != NULL ) {
    ++numb;
    leaf = leaf->right;
  }
  
  return numb;

}
















NODE_LIST *get_node_from_label( NODE_LIST *node, char label[] )
{

  if ( node == NULL )
    return NULL;

  if ( ! strcmp( node->my_node->label, label ))
    return node;
    
  register int i = 0;
  
  while ( node->down[ i ] != NULL ) {
  
    NODE_LIST *a = (NODE_LIST *)get_node_from_label( node->down[ i ], label );
    
    if ( a != NULL ) 
      return a;
      
    ++i;
  
  }
  
  return NULL;

}




