#include "print.h"















#define OUT_FNAME_BASE "data/ft_hg19/tree/data"











void put_phylo_tree( NODE_LIST *node )
{

  if ( node == NULL )
    return;

  put_my_phylo_tree( node );
  
  put_phylo_tree_recursion( node );

}










void put_phylo_tree_recursion( NODE_LIST *node )
{

  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    put_phylo_tree( node->down[ i++ ] );

}











void put_my_phylo_tree( NODE_LIST *node )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  char fname[ BUFFER ];
  get_phylo_tree_output_fname( fname, node->my_node->label );
  
  FILE *fh = NULL;
  fh = (FILE *)open_text_file_writing( fname, "" );
  
  put_phylo_tree_w_fh( node, fh );
  
  close_file( fh );
  
  put_ancestral_nucl_data_single_spec_tree( node );

}












void put_phylo_tree_w_fh( NODE_LIST *node, FILE *fh )
{

  put_phylo_tree_intro_comments( node, fh );

  put_ancestral_nucl_data_single_spec_tree_fh( node, fh );
  
  put_leaves_under_node( node, fh );

}











void put_phylo_tree_intro_comments( NODE_LIST *node, FILE *fh )
{

  char spec_line[ BUFFER ];
  
  sprintf( spec_line, "\t%s", node->my_node->label );

  comment( fh, "This file contains phylogenetic co-occurrence data" );
  comment( fh, "at each ancestor node of lineage :" );
  comment( fh, spec_line );
  comment( fh, "" );
  
  comment( fh, "Each entry gives pairwise co-occurrence data" );
  comment( fh, "between this lineage and the node labeled" );
  comment( fh, "in each entry." );
  comment( fh, "" );

}











void comment( FILE *fh, char str[] )
{
  fprintf( fh, "#   %s\n", str );
}











void put_leaves_under_node( NODE_LIST *node, FILE *fh )
{

  fprintf( fh, "\n\n" );
  
  print_sep( fh );

  fprintf( fh, "Node %s\n", node->my_node->label );
  
  print_dot_sep( fh );
  
  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    fprintf( fh, "  %s\n", node->down[ i++ ]->my_node->label );
    
  print_dot_sep( fh );
  
  put_my_leaves_under_node( node, fh );

}











void put_my_leaves_under_node( NODE_LIST *node, FILE *fh )
{

  if ( node->down[ 0 ] == NULL )
    fprintf( fh, "\t%8s   %s\n", node->my_node->label, node->my_node->spec );
    
  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    put_my_leaves_under_node( node->down[ i++ ], fh );

}









/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/












void put_ancestral_nucl_data_single_spec_tree( NODE_LIST *node )
{
  
  extern char species[ BUFFER ];
  
  if ( ! strcmp( species, node->my_node->label ))
    put_my_ancestral_nucl_data_single_spec_tree( node );

}











void put_my_ancestral_nucl_data_single_spec_tree( NODE_LIST *node )
{
  put_ancestral_nucl_data_single_spec_tree_fh( node, stdout );
}










void put_ancestral_nucl_data_single_spec_tree_fh( NODE_LIST *node, FILE *fh )
{

  if ( node == NULL )    
    return;
    
  put_my_ancestral_nucl_data_single_spec_tree_fh( node, fh );
  
}










void put_my_ancestral_nucl_data_single_spec_tree_fh( NODE_LIST *node, FILE *fh )
{
  put_ancestral_nucl_data_single_spec_w_node_fh( node, fh );
}












void put_ancestral_nucl_data_single_spec_tree_fh_recursion( NODE_LIST *node,
  FILE *fh )
{

  register char i = 0;
  
  while ( node->down[ i ] != NULL )
    put_ancestral_nucl_data_single_spec_tree_fh( node->down[ i++ ], fh );

}













void put_ancestral_nucl_data_single_spec_w_node_fh( NODE_LIST *my_node, FILE *fh )
{

  NODE_LIST *node = my_node;

  INDEX i = 0;
  
  while ( i < my_node->my_node->data->numb_of_spec )
    put_ancestral_nucl_data_single_up_node_fh_and_step( my_node, &node, i++, fh );

}













void put_ancestral_nucl_data_single_up_node_fh_and_step( NODE_LIST *my_node, 
  NODE_LIST **node, INDEX i, FILE *fh )
{
  put_ancestral_nucl_data_single_up_node_fh( my_node, *node, i, fh );
  *node = (*node)->up;
}










void put_ancestral_nucl_data_single_up_node_fh( NODE_LIST *my_node, NODE_LIST *node,
  INDEX i, FILE *fh )
{

  if ( node == NULL )
    error( 4651, "Unexpected null value for node leaf element.\n" );
    
  NODE_DATA *data = my_node->my_node->data;

  COUNT *mx = data->x + (i * NUMB_OF_NUCL * NUMB_OF_NUCL);
  COUNT *total = data->total + i;

  put_ancestral_nucl_data_single_mx_fh( my_node, node, mx, *total, i, fh );

}












void put_ancestral_nucl_data_single_mx_fh( NODE_LIST *my_node, NODE_LIST *node, 
  COUNT *mx, COUNT total, INDEX i, FILE *fh )
{

  print_sep( fh );
  
  fprintf( fh, "%15s\t(%d)\tTotal co-occur :  %d\n", node->my_node->label, (int)i,
    (int)total );
  
  print_dot_sep( fh );
  
//  if ( node->my_node->split ) {
//    fprintf( fh, "Unused node.\n" );
//  }
//  else {
    put_ancestral_nucl_data_full_matrix_fh( mx, fh );
    print_dot_sep( fh );
    put_ancestral_nucl_data_full_freq_matrix_fh( mx, total, fh );
//  }
  
  print_sep( fh );

}













void put_ancestral_nucl_data_full_matrix_fh( COUNT *mx, FILE *fh )
{

  register int i, j;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
    for ( j = 0; j < NUMB_OF_NUCL; j++ ) {
      fprintf( fh, "    %6d", (int)(*(mx + (i * NUMB_OF_NUCL) + j)) );
    }    
    fprintf( fh, "\n" );
  }

}











void put_ancestral_nucl_data_full_freq_matrix_fh( COUNT *mx, COUNT total,
  FILE *fh )
{

  float x = (float)total;

  register int i, j;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {

    for ( j = 0; j < NUMB_OF_NUCL; j++ )

      fprintf( fh, "    %5.1f%%", 
        (float)(*(mx + (i * NUMB_OF_NUCL) + j)) * 100.0 / x );
 
    fprintf( fh, "\n" );
 
  }

}








/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/












void get_phylo_tree_output_fname( char fname[], char label[] )
{
  sprintf( fname, "%s/%s.phylo.txt", OUT_FNAME_BASE, label );
}







