#include "deter.h"















#define OUTPUT_FNAME "data/results/deter/determin_results.3.txt"





#define NUMB_OF_GENERATIONS 500000

#define INIT_P ((2000.0 - 1.0) / (2000.0))
#define INIT_Q (1.0 / 2000.0)

#define INIT_A_FREQ 0.80
#define INIT_C_FREQ 0.35

#define ALPHA 1e-11
#define BETA 1e-8
#define GEN_TIME 5.0
#define WIND_WIDTH 100.0
#define BIRTH (GEN_TIME * WIND_WIDTH * ALPHA)
#define DEATH (GEN_TIME * BETA)
#define TRANS BIRTH

#define S0 0.001
#define SC 0.001











void do_deterministic( void )
{

  GG *create_gg( GG *xx, int ii, int jj, int hh );

  GG *my_gg = NULL;
  my_gg = (GG *)create_gg( my_gg, HAPL_BUFFER, NUMB_OF_GENES, HAPL_BUFFER );
  
  do_my_deterministic( my_gg );
  
  kill_gg( my_gg );

}














void do_my_deterministic( GG *my_gg )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( OUTPUT_FNAME, "" );
  
  do_deter_w_fh( my_gg, out );
  
  close_file( out );

}












void do_deter_w_fh( GG *my_gg, FILE *out )
{

  print_deter_header_comments( out );
  
  do_my_deter_w_fh( my_gg, out );

}















void do_my_deter_w_fh( GG *my_gg, FILE *out )
{

  initialize_deter_gg( my_gg );
  
  int t;
  
  for ( t = 0; t < NUMB_OF_GENERATIONS; t++ )
    do_deter_single_generation( my_gg, t, out );

}













void initialize_deter_gg( GG *my_gg )
{

  initialize_deter_gg_y_matrix( my_gg->y );

  my_gg->p = INIT_P;
  my_gg->q = INIT_Q;  
  
  set_deter_w_matrix( my_gg );

}












void initialize_deter_gg_y_matrix( STAT_CELL *y )
{

  int n = y->j;

  int ac = (int)(INIT_A_FREQ * INIT_C_FREQ * n);
  int aa = ac + (int)(INIT_A_FREQ * (1.0 - INIT_C_FREQ) * n);
  int cc = aa + (int)(INIT_C_FREQ * (1.0 - INIT_A_FREQ) * n);
  
  int j;
  
  for ( j = 0; j < ac; j++ )
    *(y->x + (AC * y->j) + j) = 1.0;
    
  for ( j = ac; j < aa; j++ )
    *(y->x + (AA * y->j) + j) = 1.0;
    
  for ( j = aa; j < cc; j++ )
    *(y->x + (CC * y->j) + j) = 1.0;
    
  for ( j = cc; j < n; j++ )
    *(y->x + (NO * y->j) + j) = 1.0;

}












void set_deter_w_matrix( GG *my_gg )
{
  set_deter_w_matrix_pq( my_gg->w, my_gg->p, my_gg->q );
}











void set_deter_w_matrix_pq( STAT_CELL *w, STAT p, STAT q )
{

  STAT no = 1.0 - S0;
  STAT aa = (p * p) + (2.0 * p * q) + ((q * q) * (1.0 - S0));
  STAT cc = ((p * p) * (1.0 - S0)) + (((2.0 * p * q) + (q * q)) * (1.0 + SC));
  STAT ac = (p * p) + (((2.0 * p * q) + (q * q)) * (1.0 + SC));
  
  STAT *wptr = w->x;
  
  int i = 0;
  
  while ( i++ < w->i * w->j )
    *wptr++ = ac;
    
  *(w->x + (NO * w->j) + NO) = no;
  
  *(w->x + (NO * w->j) + AA) = *(w->x + (AA * w->j) + NO) = aa;
  *(w->x + (AA * w->j) + AA) = aa;
  
  *(w->x + (NO * w->j) + CC) = *(w->x + (CC * w->j) + NO) = cc;
  *(w->x + (CC * w->j) + CC) = cc;
  
  
  
}














/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/















void do_deter_single_generation( GG *my_gg, int t, FILE *out )
{

  GG *create_gg( GG *xx, int ii, int jj, int hh );

  GG *gg = NULL;
  gg = (GG *)create_gg( gg, HAPL_BUFFER, NUMB_OF_GENES, HAPL_BUFFER );

  update_deter_single_gen_freq( my_gg );
  
  print_deter_gg( my_gg, t, out );
  
  update_deter_single_gen_loci( my_gg, gg );

  update_deter_single_gen_pq( my_gg, gg );
  
  update_deter_single_gen_mutation( gg );
  
  update_deter_single_gen_freq( gg );
  
  copy_gg( my_gg, gg );
  
  kill_gg( gg );
  
}















void update_deter_single_gen_loci( GG *my_gg, GG *gg )
{

  /*  In this file, above:  */
  set_deter_w_matrix( my_gg );
  
  /*  In this file, below:  */
  update_my_deter_single_gen_loci( my_gg, gg );

}













void update_my_deter_single_gen_loci( GG *my_gg, GG *gg )
{

  int a;
  
  for ( a = 0; a < my_gg->y->j; a++ )
    update_deter_single_gen_single_locus( my_gg, gg, a );

}














void update_deter_single_gen_single_locus( GG *my_gg, GG *gg, int a )
{

  STAT f = get_avg_fitness_deter_single_locus( my_gg->y, my_gg->w, a );
  
  int x;  
  
  for ( x = 0; x < HAPL_BUFFER; x++ ) 
    update_deter_single_gen_single_locus_single_hapl( my_gg, gg, f, x, a );  
  
}













STAT get_avg_fitness_deter_single_locus( STAT_CELL *y, STAT_CELL *w, int a )
{

  STAT f = 0.0;

  int i, j;
  
  for ( i = 0; i < HAPL_BUFFER; i++ ) {
    
    for ( j = 0; j < HAPL_BUFFER; j++ ) {
    
      STAT term = (*(y->x + (i * y->j) + a)) * (*(y->x + (j * y->j) + a));
      term *= (*(w->x + (i * w->j) + j));
      
      f += term;
    
    }
  
  }
  
  return f;

}












void update_deter_single_gen_single_locus_single_hapl( GG *my_gg, GG *gg, 
  STAT f, int x, int a )
{

  STAT y_x = *(my_gg->y->x + (x * my_gg->y->j) + a);

  *(gg->y->x + (x * gg->y->j) + a) = y_x * get_sum_fitness_locus( my_gg, x, a ) / f;
  
}












STAT get_sum_fitness_locus( GG *my_gg, int x, int a )
{

  STAT_CELL *y = my_gg->y, *w = my_gg->w;

  STAT sum = 0.0;
  
  int j;
  
  for ( j = 0; j < HAPL_BUFFER; j++ )
    sum += (*(y->x + (j * y->j) + a)) * (*(w->x + (x * w->j) + j));
    
  return sum;

}











void update_deter_single_gen_pq( GG *my_gg, GG *gg )
{

  STAT la = get_lval( my_gg->y, AA, AC );
  STAT lc = get_lval( my_gg->y, CC, AC );
  STAT l = my_gg->y->j;
  STAT lac = la * lc / l;
  
  STAT waa = get_ww_val( 1.0, 1.0 - S0, la, l - la );
  STAT wac = get_ww_val( 1.0 + SC, 1.0 - S0, lc, l - la - lc + lac );
  STAT wcc = get_ww_val( 1 - S0, 1 + S0, l - lc, lc );
  
  STAT p = my_gg->p;
  STAT q = my_gg->q;
  
  STAT term_aa = p * p * waa;
  STAT term_ac = p * q * wac;
  STAT term_cc = q * q * wcc;
  
  gg->p = (term_aa + term_ac) / (term_aa + (2.0 * term_ac) + term_cc);
  gg->q = 1.0 - gg->p;
  
}














STAT get_lval( STAT_CELL *y, char a, char b )
{

  STAT lval = 0.0;
  
  register int j;
  
  for ( j = 0; j < y->j; j++ )
    lval += get_pr_lval_single_term( y, a, b, j );
    
  return lval;

}










STAT get_pr_lval_single_term( STAT_CELL *y, char a, char b, int j )
{

  STAT prob = (*(y->x + (a * y->j) + j)) + (*(y->x + (b * y->j) + j));
  
  return (prob * prob) + (2.0 * prob * (1.0 - prob)); 

}













STAT get_ww_val( STAT a, STAT b, STAT x, STAT y )
{
  return pow( a, x ) * pow( b, y );
}











void update_deter_single_gen_mutation( GG *gg )
{

  STAT_CELL *create_stat_cell_matrix( STAT_CELL *mm, int ii, int jj );
  
  STAT_CELL *mx = NULL, *a = NULL, *b = NULL;
  mx = (STAT_CELL *)create_stat_cell_matrix( mx, HAPL_BUFFER, HAPL_BUFFER );
  a = (STAT_CELL *)create_stat_cell_matrix( a, HAPL_BUFFER, 1 );
  b = (STAT_CELL *)create_stat_cell_matrix( b, HAPL_BUFFER, 1 );
  
  update_deter_single_gen_mutation_w_mx( gg, mx, a, b );
  
  kill_stat_cell( mx );
  kill_stat_cell( a );
  kill_stat_cell( b );
  
}












void update_deter_single_gen_mutation_w_mx( GG *gg, STAT_CELL *mx, STAT_CELL *a, 
  STAT_CELL *b )
{

  set_deter_transition_mx( mx );
  
  update_my_deter_single_gen_mutation_w_mx( gg, mx, a, b );

}











void set_deter_transition_mx( STAT_CELL *x )
{

  *(x->x + (NO * x->j) + NO) = 1.0 - (2.0 * BIRTH);
  *(x->x + (NO * x->j) + AA) = DEATH;
  *(x->x + (NO * x->j) + CC) = DEATH;
  *(x->x + (NO * x->j) + AC) = 0.0;

  *(x->x + (AA * x->j) + NO) = BIRTH;
  *(x->x + (AA * x->j) + AA) = 1.0 - BIRTH - TRANS - DEATH;
  *(x->x + (AA * x->j) + CC) = TRANS;
  *(x->x + (AA * x->j) + AC) = DEATH; // + TRANS;

  *(x->x + (CC * x->j) + NO) = BIRTH;
  *(x->x + (CC * x->j) + AA) = TRANS;
  *(x->x + (CC * x->j) + CC) = 1.0 - BIRTH - TRANS - DEATH;
  *(x->x + (CC * x->j) + AC) = DEATH; // + TRANS;

  *(x->x + (AC * x->j) + NO) = 0.0;
  *(x->x + (AC * x->j) + AA) = BIRTH;
  *(x->x + (AC * x->j) + CC) = BIRTH;
  *(x->x + (AC * x->j) + AC) = 1.0 - (2.0 * DEATH); // - (2.0 * TRANS);

}










void update_my_deter_single_gen_mutation_w_mx( GG *gg, STAT_CELL *mx, STAT_CELL *a, 
  STAT_CELL *b )
{

  int j;
  
  for ( j = 0; j < gg->y->j; j++ )
    update_my_deter_single_gen_mutation_single_locus( gg, mx, a, b, j );

}














void update_my_deter_single_gen_mutation_single_locus( GG *gg, STAT_CELL *mx, 
  STAT_CELL *a, STAT_CELL *b, int j )
{

  set_deter_freq_vector_single_locus( a, gg, j );
  
  multiply_matrices( b, mx, a );
  
  copy_deter_freq_vector_y_mx( gg->y, b, j );

}











void set_deter_freq_vector_single_locus( STAT_CELL *a, GG *my_gg, int j )
{

  *(a->x + (NO * a->j)) = *(my_gg->y->x + (NO * my_gg->y->j) + j);
  *(a->x + (AA * a->j)) = *(my_gg->y->x + (AA * my_gg->y->j) + j);
  *(a->x + (CC * a->j)) = *(my_gg->y->x + (CC * my_gg->y->j) + j);
  *(a->x + (AC * a->j)) = *(my_gg->y->x + (AC * my_gg->y->j) + j);

}











void multiply_matrices( STAT_CELL *c, STAT_CELL *a, STAT_CELL *b )
{

  int i, j, k;
  
  for ( i = 0; i < c->i; i++ ) {
  
    for ( j = 0; j < c->j; j++ ) {
    
      STAT x = 0.0;
      
      for ( k = 0; k < a->j; k++ )
        x += (*(a->x + (i * a->j) + k)) * (*(b->x + (k * b->j) + j));
        
      *(c->x + (i * c->j) + j) = x;
    
    }
  
  }

}













void copy_deter_freq_vector_y_mx( STAT_CELL *y, STAT_CELL *b, int j )
{

  int nucl;
  
  for ( nucl = 0; nucl < HAPL_BUFFER; nucl++ )
    *(y->x + (nucl * y->j) + j) = *(b->x + nucl);
    
}
















void update_deter_single_gen_freq( GG *gg )
{

  gg->a_freq = 0.0;
  gg->c_freq = 0.0;
  
  int j;
  
  for ( j = 0; j < gg->y->j; j++ )
    update_deter_single_gen_freq_single_locus( gg, j );

} 












void update_deter_single_gen_freq_single_locus( GG *gg, int j )
{

  STAT_CELL *y = gg->y;

  gg->a_freq += *(y->x + (AA * y->j) + j) + *(y->x + (AC * y->j) + j);
  gg->c_freq += *(y->x + (CC * y->j) + j) + *(y->x + (AC * y->j) + j);

}










STAT copy_gg( GG *my_gg, GG *gg )
{

  my_gg->p = gg->p;
  my_gg->q = gg->q;
  
  my_gg->a_freq = gg->a_freq;
  my_gg->c_freq = gg->c_freq;
  
  copy_stat_cell_matrix( my_gg->y, gg->y );
  copy_stat_cell_matrix( my_gg->w, gg->w );

}











void copy_stat_cell_matrix( STAT_CELL *a, STAT_CELL *b )
{

  STAT *aptr = a->x, *bptr = b->x;

  int i, j;
  
  for ( i = 0; i < a->i; i++ ) {
    for ( j = 0; j < a->j; j++ ) {
      *aptr++ = *bptr++;
    }
  }

}








/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/












void print_deter_header_comments( FILE *out )
{

  fprintf( out, "#  Deterministic model...\n" );
  fprintf( out, "#  ALPHA:  %.2e\n", ALPHA );
  fprintf( out, "#  BETA:   %.2e\n", BETA );
  fprintf( out, "#  GEN_TIME:  %.1f\n", GEN_TIME );
  fprintf( out, "#  BIRTH:     %.2e\n", BIRTH );
  fprintf( out, "#  DEATH:     %.2e\n", DEATH );
  fprintf( out, "#  TRANS:     %.2e\n", TRANS );
  fprintf( out, "#  S[0]:      %.5f\n", S0 );
  fprintf( out, "#  S[C]:      %.5f\n", SC );
  fprintf( out, "#\n" );
  fprintf( out, "\"T\"\t\"P\"\t\"A_FREQ\"\t\"C_FREQ\"\n" );

}











void print_deter_gg( GG *my_gg, int t, FILE *out )
{
  print_my_deter_gg( my_gg, t, stdout );
  print_my_deter_gg( my_gg, t, out );
}












void print_my_deter_gg( GG *my_gg, int t, FILE *out )
{

  fprintf( out, "%-7d\t%.12f\t%.7f\t%.7f\n", t, my_gg->p, my_gg->a_freq,
    my_gg->c_freq );
    
  fflush( out );

}








