#include "diploid.h"













#define OUTPUT_FNAME "data/results/simulation_results.curr.txt"








#define NUMB_OF_GENERATIONS 100000000


#define INIT_A_FREQ 0.85
#define INIT_C_FREQ 0.30

#define S0 0.001
#define SC 0.001

//#define ALPHA 1e-11
//#define BETA 1e-8

#define ALPHA 1e-8
#define BETA 1e-5
#define GENERATION_TIME 10.0
#define WIND_WIDTH 100.0
#define K_VAL 3.0













void do_diploid_model( void )
{

  GENERATION *create_generation( GENERATION *gg, int nn );

  srand( (unsigned)time( NULL ) );
  
  GENERATION *gg = NULL;
  gg = (GENERATION *)create_generation( gg, NUMB_OF_GENES );
  
  set_diploid_params();
  set_initial_diploid_generation( gg ); 
  
  do_my_diploid_model( gg );
  
  kill_generation( gg );

}














void set_diploid_params( void )
{

  extern PARAMS my_params;
  
  my_params.numb_of_genes = NUMB_OF_GENES;
  my_params.pop_size = POP_SIZE;
  
  my_params.init_a_freq = INIT_A_FREQ;
  my_params.init_c_freq = INIT_C_FREQ;
  
  my_params.alpha = ALPHA * GENERATION_TIME * WIND_WIDTH;
  my_params.beta = BETA * GENERATION_TIME;
  my_params.k = K_VAL;
  
  my_params.death = my_params.beta;
  my_params.trans = K_VAL * my_params.alpha;
  my_params.birth = my_params.alpha;

}












void set_initial_diploid_generation( GENERATION *gg )
{

  extern PARAMS my_params;
  
  int pop_size = my_params.pop_size;
  STAT init_a_freq = my_params.init_a_freq;
  STAT init_c_freq = my_params.init_c_freq;
  
  gg->p = (STAT)((2 * pop_size) - 1) / (STAT)(2 * pop_size);
  gg->q = (1.0) / (STAT)(2 * pop_size);
  
  set_initial_box_cell( gg->box_a, init_a_freq );
  set_initial_box_cell( gg->box_c, init_c_freq );

}














void set_initial_box_cell( GENE_CELL *cell, STAT freq )
{

  int get_binary_freq_sim( STAT p );

  GENE *gene = cell->x;
  
  register int i = 0;
  
  while ( i++ < cell->n ) {
    /*  In sim.c:  */
    gene->y = (STAT)get_binary_freq_sim( freq );
    gene->b = (gene->y * gene->y) + (2.0 * gene->y * (1.0 - gene->y)); 
    cell->y_total += gene->y;
    cell->b_total += gene->b;
    ++gene;
  }
  
  cell->x->y = 1.0 / 2000.0;
  set_b_freq_gene_elem( cell->x );  
  
}

















/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
















void do_my_diploid_model( GENERATION *gg )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( OUTPUT_FNAME, "" );

  do_diploid_model_fh( gg, out );
  
  close_file( out );

}













void do_diploid_model_fh( GENERATION *gg, FILE *fh )
{

  print_model_header( fh );

  print_model_generation( gg, 0, fh );

  int t;
  
  for ( t = 1; t <= NUMB_OF_GENERATIONS; t++ )
    do_diploid_model_single_generation( gg, t, fh );

}















void do_diploid_model_single_generation( GENERATION *gg, int t, FILE *fh )
{

  update_diploid_model_single_generation( gg );

  print_model_generation( gg, t, fh );

}













void update_diploid_model_single_generation( GENERATION *gg )
{

  update_my_diploid_model_single_generation( gg );
  
  do_diploid_model_mutations( gg );

}













void update_my_diploid_model_single_generation( GENERATION *gg )
{

  GENERATION *create_generation( GENERATION *gg, int nn );
  
  extern PARAMS my_params;
  
  GENERATION *new_gg = NULL;
  new_gg = (GENERATION *)create_generation( new_gg, my_params.numb_of_genes );
  
  update_diploid_model_two_gen_structs( new_gg, gg );
  
  copy_generations( gg, new_gg );
  
  kill_generation( new_gg );
  
}














void update_diploid_model_two_gen_structs( GENERATION *new_gg, GENERATION *gg )
{

  update_diploid_model_tf_freqs( new_gg, gg );
  
  update_diploid_model_box_freqs( new_gg, gg );

}













void update_diploid_model_tf_freqs( GENERATION *new_gg, GENERATION *gg )
{

  STAT f_aa = get_diploid_f_aa( gg );
  STAT f_cc = get_diploid_f_cc( gg );
  STAT f_ac = get_diploid_f_ac( gg );
  
  STAT aa = gg->p * gg->p * f_aa;
  STAT cc = gg->q * gg->q * f_cc;
  STAT ac = gg->p * gg->q * f_ac;
  STAT denom = aa + (2.0 * ac) + cc;
  
  new_gg->p = (aa + ac) / denom;
  new_gg->q = (cc + ac) / denom;

}












STAT get_diploid_f_aa( GENERATION *gg )
{
  return pow( 1.0 - S0, gg->box_a->n - gg->box_a->b_total );
}












STAT get_diploid_f_cc( GENERATION *gg )
{

  return pow( 1.0 + SC, gg->box_c->b_total ) 
    * pow( 1.0 - S0, gg->box_c->n - gg->box_c->b_total );

}














STAT get_diploid_f_ac( GENERATION *gg )
{

  STAT l_ac = 
    (STAT)(gg->box_c->b_total * gg->box_a->b_total) / (STAT)(gg->box_c->n);

  return pow( 1.0 + SC, gg->box_c->b_total ) 
    * pow( 1.0 - S0, gg->box_c->n - gg->box_c->b_total - gg->box_a->b_total + l_ac );

}














void update_diploid_model_box_freqs( GENERATION *new_gg, GENERATION *gg )
{

  GENE *a = gg->box_a->x, *c = gg->box_c->x;
  GENE *new_a = new_gg->box_a->x, *new_c = new_gg->box_c->x;
  
  register int i = 0;
  
  while ( i++ < gg->box_c->n )
    update_diploid_model_single_box_gene( new_a++, new_c++, a++, c++, gg->p, gg->q );

}













void update_diploid_model_single_box_gene( GENE *new_a, GENE *new_c, GENE *a,
  GENE *c, STAT p, STAT q )
{

  STAT f_no = 1.0 - S0;

  STAT f_aa = pow( 1.0 - S0, q * q );

  STAT f_cc = pow( 1.0 + SC, (2.0 * p * q) + (q * q) )
    * pow( 1 - S0, (p * p) );

  STAT f_ac = pow( 1.0 + SC, (2.0 * p * q) + (q * q) );
  
  STAT b_a = a->b;
  STAT b_c = c->b;
  STAT d_a = 1.0 - a->b;
  STAT d_c = 1.0 - c->b;
  
  STAT denom = (f_no * d_a * d_c) + (f_aa * b_a * d_c) + (f_cc * b_c * d_a)
    + (f_ac * b_a * b_c);
  
  new_a->b = ((b_a * d_c * f_aa) + (b_a * b_c * f_ac)) / (denom);
  new_c->b = ((b_c * d_a * f_cc) + (b_a * b_c * f_ac)) / (denom);
  
  set_y_freq_gene_elem( new_a );
  set_y_freq_gene_elem( new_c );
  
}














void set_b_freq_gene_elem( GENE *x )
{
  x->b = (x->y * x->y) + (2.0 * x->y * (1.0 - x->y));
}













void set_y_freq_gene_elem( GENE *x )
{
  x->y = 1.0 - sqrt( 1.0 - x->b );
}











void copy_generations( GENERATION *a, GENERATION *b )
{

  copy_gene_cell( a->box_a, b->box_a );
  copy_gene_cell( a->box_c, b->box_c );
  
  a->p = b->p;
  a->q = b->q;

}















void copy_gene_cell( GENE_CELL *a, GENE_CELL *b )
{

  GENE *aptr = a->x, *bptr = b->x;
  
  register int i = 0;
  
  while ( i++ < a->n )
    *aptr++ = *bptr++;
    
  a->n = b->n;
  a->y_total = b->y_total;
  a->b_total = b->b_total;

}














/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/

















void do_diploid_model_mutations( GENERATION *gg )
{

  GENE *a = gg->box_a->x, *c = gg->box_c->x;
  
  register int i = 0;
  
  while ( i++ < gg->box_a->n )
    do_diploid_model_mutation_single_gene( a++, c++ );
    
  update_totals_gene_cell( gg->box_a );
  update_totals_gene_cell( gg->box_c );

}














void do_diploid_model_mutation_single_gene( GENE *a, GENE *c )
{

  void do_diploid_model_mutation_single_gene_vals( GENE *a, GENE *c, 
    PARAMS *my_params );

  extern PARAMS my_params;
  
  /*  In sim.c:  */
  do_diploid_model_mutation_single_gene_vals( a, c, &my_params );

}















void update_totals_gene_cell( GENE_CELL *cell )
{

  cell->y_total = cell->b_total = 0.0;
  
  GENE *gene = cell->x;
  
  register int i = 0;
  
  while ( i++ < cell->n ) {
    cell->y_total += gene->y;
    cell->b_total += gene->b;
    ++gene;
  }

}









/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/














void print_model_header( FILE *fh )
{

  print_model_header_comments( fh );

  fprintf( fh, "\"T\"\t\"P\"\t\"Q\"\t\"Total\"\t\"BOXA\"\t\"BOXC\"\t" );
  fprintf( fh, "\"Abirth\"\t\"Adeath\"\t\"Cbirth\"\t\"Cdeath\"\n" );

}















void print_model_header_comments( FILE *fh )
{

  extern PARAMS my_params;

  fprintf( fh, "# NumbOfGenes:  %d\n", my_params.numb_of_genes );
  fprintf( fh, "# PopSize:      %d\n", my_params.pop_size );
  fprintf( fh, "# InitA:        %.3f\n", my_params.init_a_freq );
  fprintf( fh, "# InitC:        %.3f\n", my_params.init_c_freq );
  fprintf( fh, "# Birth:        %.2e\n", my_params.birth );
  fprintf( fh, "# Death:        %.2e\n", my_params.death );
  fprintf( fh, "# Trans:        %.2e\n", my_params.trans );
  fprintf( fh, "# S0:           %.5f\n", S0 );
  fprintf( fh, "# SC:           %.5f\n", SC );

}












void print_model_generation( GENERATION *gg, int t, FILE *fh )
{

  int a_birth = 0, a_death = 0, c_birth = 0, c_death = 0;

  GENE *a = gg->box_a->x, *c = gg->box_c->x;
  
  int i = 0;
  
  while ( i++ < gg->box_a->n ) {
    a_birth += a->birth;
    a_death += a->death;
    c_birth += c->birth;
    c_death += c->death;
    ++a;
    ++c;
  }

  fprintf( fh, "%d\t%.12f\t%.12f\t%.2f\t%.12f\t%.12f\t%d\t%d\t%d\t%d\n", 
    t, gg->p, gg->q, 
    gg->p + gg->q,
    gg->box_a->y_total,
    gg->box_c->y_total,
    a_birth, a_death, c_birth, c_death );

  printf( "%d\t%.12f\t%.12f\t%.2f\t%.12f\t%.12f\t%d\t%d\t%d\t%d\n", 
    t, gg->p, gg->q, 
    gg->p + gg->q,
    gg->box_a->y_total,
    gg->box_c->y_total,
    a_birth, a_death, c_birth, c_death );
    
  fflush( fh );

}
















