#include "equil.h"













#define OUTPUT_FNAME "data/results/init_equil/equil_initial.SPC.txt"


#define NUMB_OF_GENERATIONS 10000

#define BIRTH_ARRAY { 1e-5, 5e-6, 1e-6, 5e-7 }
#define BIRTH_BUFFER 4
#define DEATH 1e-5
#define BIRTH_DEATH_STEP 0.1
#define BIRTH_DEATH_NUMB 3

#define INIT_A_FREQ 0.9
#define INIT_C_FREQ 0.15

#define MAX_MIN_BUFFER 2
#define MAX_MIN_INIT { 0.0, NUMB_OF_GENES }

#define THRESH 1.0e-3

#define MAX(a,b) (((a) > (b)) ? (a) : (b))
#define MIN(a,b) (((a) < (b)) ? (a) : (b))
#define ABS(a) (((a) > 0.0) ? (a) : ((a) * -1.0))

#define P_FREQ 0.0
#define Q_FREQ (1.0 - (P_FREQ))









void do_equil( void )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( OUTPUT_FNAME, "" );
  
  do_equil_w_fh( out );
  
  close_file( out );

}












void do_equil_w_fh( FILE *out )
{

  STAT birth_array[ BIRTH_BUFFER ] = BIRTH_ARRAY;

  int i, j;
  
  for ( i = 0; i < BIRTH_DEATH_NUMB; i++ ) {
    
    for ( j = 0; j < BIRTH_BUFFER; j++ ) {
    
      STAT ratio = pow( BIRTH_DEATH_STEP, (float)i );
    
      do_equil_single_birth_death_rate( birth_array[ j ] * ratio, DEATH * ratio, 
        out );
    
    }
  
  }

}












void do_equil_single_birth_death_rate( STAT birth, STAT death, FILE *out )
{

  fprintf( out, ">  Birth:   %8.2e   Death:  %8.2e\n", birth, death );

  STAT box_a[ MAX_MIN_BUFFER ] = MAX_MIN_INIT;
  STAT box_c[ MAX_MIN_BUFFER ] = MAX_MIN_INIT;
  
  do_equil_init_box_vals( box_a, box_c, birth, death );
  
  print_equil_step( box_a, box_c, 0, out );
  
  int converged = 0, iter = 0;
  
  while ( ! converged ) 
    converged = do_equil_single_birth_death_single_window( box_a, box_c, birth,
      death, ++iter, out );
    
  fprintf( out, "\n\n\n" );
  
}














void do_equil_init_box_vals( STAT box_a[], STAT box_c[], STAT birth, STAT death )
{

  STAT after_box_a[ MAX_MIN_BUFFER ] = { NUMB_OF_GENES, 0.0 };
  STAT after_box_c[ MAX_MIN_BUFFER ] = { NUMB_OF_GENES, 0.0 };

  int a, c;
  
  for ( a = 0; a < MAX_MIN_BUFFER; a++ ) {
  
    for ( c = 0; c < MAX_MIN_BUFFER; c++ ) {
    
      STAT box_a_val, box_c_val;
    
      get_new_equil_box_vals( &box_a_val, &box_c_val, box_a[ a ], box_c[ c ], birth, 
        death );
        
      update_box_ac_values_init( after_box_a, after_box_c, box_a_val, box_c_val,
        a, c );
        
    }
  
  }
  
  do_equil_init_box_vals_center( box_a, box_c, after_box_a, after_box_c, 
    birth, death );

  switch_box_vals_two( box_a, box_c );

}
















void update_box_ac_values_init( STAT after_box_a[], STAT after_box_c[], 
  STAT box_a_val, STAT box_c_val, int a, int c )
{

  update_single_box_ac_values_init( after_box_a, box_a_val, a );
  update_single_box_ac_values_init( after_box_c, box_c_val, c );

}












void update_single_box_ac_values_init( STAT after_box[], STAT box_val, int index )
{

  switch ( index ) {
  
  case 0 :
    after_box[ 0 ] = MIN( after_box[ 0 ], box_val );
    break;
    
  case 1 :
    after_box[ 1 ] = MAX( after_box[ 1 ], box_val );
    break;
  
  default :
    error( 2000, "Unrecognized max/min value.\n" );
    break;
        
  }
      

}
        









void update_box_ac_values( STAT after_box_a[], STAT after_box_c[], STAT box_a_val, 
  STAT box_c_val, int a, int c )
{
  update_single_box_ac_values( after_box_a, box_a_val, a );
  update_single_box_ac_values( after_box_c, box_c_val, c ); 
}













void update_single_box_ac_values( STAT after_box[], STAT box_val, int index )
{

  switch ( index ) {
      
  case 0 :
    after_box[ 0 ] = MAX( after_box[ 0 ], box_val );
    break;
    
  case 1 :
    after_box[ 1 ] = MIN( after_box[ 1 ], box_val );
    break;
  
  default :
    error( 2000, "Unrecognized max/min value.\n" );
    break;
        
  }
      
}










void do_equil_init_box_vals_center( STAT box_a[], STAT box_c[], STAT a[], STAT c[], 
  STAT birth, STAT death )
{

  int i;
  
  for ( i = 0; i < MAX_MIN_BUFFER; i++ ) {
    box_a[ i ] = a[ i ];
    box_c[ i ] = c[ i ];
  }

  STAT middle_a = (a[ 0 ] + a[ 1 ]) / 2.0;
  STAT middle_c = (c[ 0 ] + c[ 1 ]) / 2.0;
  
  STAT new_a, new_c;
  
  get_new_equil_box_vals( &new_a, &new_c, middle_a, middle_c, birth, death );
  
  set_box_array_from_center( box_a, box_c, new_a, new_c, middle_a, middle_c );
  
}










void set_box_array_from_center( STAT box_a[], STAT box_c[], STAT new_a, STAT new_c,
  STAT middle_a, STAT middle_c )
{
    
  if ( new_a > middle_a ) 
    box_a[ 0 ] = new_a;
  else
    box_a[ 1 ] = new_a;
    
  if ( new_c > middle_c )
    box_c[ 0 ] = new_c;
  else
    box_c[ 1 ] = new_c;

}














void switch_box_vals_two( STAT box_a[], STAT box_c[] )
{
  switch_box_vals( box_a );
  switch_box_vals( box_c );
}












void switch_box_vals( STAT box[] )
{

  if ( box[ 1 ] > box[ 0 ] )
    return;
    
  STAT x = box[ 0 ];
  box[ 0 ] = box[ 1 ];
  box[ 1 ] = x;

}









/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/













int do_equil_single_birth_death_single_window( STAT box_a[], STAT box_c[], 
  STAT birth, STAT death, int iter, FILE *out )
{

  do_my_equil_single_birth_death_single_window( box_a, box_c, birth, death );
  
  switch_box_vals_two( box_a, box_c );
  
  print_equil_step( box_a, box_c, iter, out );
  
  return equil_window_has_converged( box_a, box_c );

}













void do_my_equil_single_birth_death_single_window( STAT box_a[], STAT box_c[],
  STAT birth, STAT death )
{

  STAT box_a_val, box_c_val;

  STAT middle_a = (box_a[ 0 ] + box_a[ 1 ]) / 2.0;
  STAT middle_c = (box_c[ 0 ] + box_c[ 1 ]) / 2.0;

  get_new_equil_box_vals( &box_a_val, &box_c_val, middle_a, middle_c, birth, 
    death );
        
  set_box_array_from_center( box_a, box_c, box_a_val, box_c_val, middle_a, 
    middle_c );

}











int equil_window_has_converged( STAT box_a[], STAT box_c[] )
{

  if ( ABS( box_a[ 1 ] - box_a[ 0 ] ) >= THRESH )
    return 0;
    
  if ( ABS( box_c[ 1 ] - box_c[ 0 ] ) >= THRESH )
    return 0;
    
  return 1;

}











/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/














void get_new_equil_box_vals( STAT *box_a_val, STAT *box_c_val, STAT init_box_a,
  STAT init_box_c, STAT birth, STAT death )
{

  GG *create_gg( GG *xx, int ii, int jj, int hh );

  GG *my_gg = NULL;
  my_gg = (GG *)create_gg( my_gg, HAPL_BUFFER, NUMB_OF_GENES, HAPL_BUFFER );
  
  do_equil_single_rates_w_gg( my_gg, init_box_a, init_box_c, birth, death );
  
  *box_a_val = my_gg->a_freq;
  *box_c_val = my_gg->c_freq;
  
  kill_gg( my_gg );

}













void do_equil_single_rates_w_gg( GG *my_gg, STAT init_box_a, STAT init_box_c,
  STAT birth, STAT death )
{

  initialize_equil_gg( my_gg, init_box_a, init_box_c );
  
  int t;
  
  for ( t = 0; t <= NUMB_OF_GENERATIONS; t++ )
    do_equil_single_rates_single_gen( my_gg, init_box_a, init_box_c, birth, 
      death, t );

}












void initialize_equil_gg( GG *my_gg, STAT init_box_a, STAT init_box_c )
{

  /*  In this file, below:  */
  initialize_equil_gg_y_matrix( my_gg->y, init_box_a, init_box_c );

  my_gg->p = P_FREQ;
  my_gg->q = Q_FREQ;
  
  /*  In deter.c:  */
  set_deter_w_matrix( my_gg );

}











void initialize_equil_gg_y_matrix( STAT_CELL *y, STAT init_box_a, STAT init_box_c )
{

  register int j;
  
  for ( j = 0; j < y->j; j++ ) 
    initialize_equil_gg_y_matrix_single_locus( y, init_box_a, init_box_c, j );

}












void initialize_equil_gg_y_matrix_single_locus( STAT_CELL *y, STAT a, STAT c, 
  int j )
{

  STAT n = NUMB_OF_GENES;

  *(y->x + (NO * y->j) + j) = (n - a) * (n - c) / n;
  *(y->x + (AA * y->j) + j) = (a) * (n - c) / n;
  *(y->x + (CC * y->j) + j) = (n - a) * (c) / n;
  *(y->x + (AC * y->j) + j) = (a) * (c) / n;

}










void do_equil_single_rates_single_gen( GG *my_gg, STAT init_box_a, STAT init_box_c,
  STAT birth, STAT death, int t )
{

  GG *create_gg( GG *xx, int ii, int jj, int hh );
  void update_deter_single_gen_loci( GG *my_gg, GG *gg );
  
  GG *gg = NULL;
  gg = (GG *)create_gg( gg, HAPL_BUFFER, NUMB_OF_GENES, HAPL_BUFFER );
  
  /*  In deter.c:  */
  copy_gg( gg, my_gg );
  update_deter_single_gen_freq( my_gg );

  printf( "%6d\t%13.8f\t%13.8f\t\t%13.8f  %13.8f   (%.1e, %.1e)\n", 
    t, my_gg->a_freq, my_gg->c_freq, init_box_a, init_box_c, birth, death );

  /*  In deter.c:  */
  update_deter_single_gen_loci( my_gg, gg );
  
  gg->p = my_gg->p;
  gg->q = my_gg->q;
  
  /*  In this file, below:  */
  update_equil_single_gen_mutation( gg, birth, death );
  
  /*  In deter.c:  */
  update_deter_single_gen_freq( gg );
  copy_gg( my_gg, gg );
  
  kill_gg( gg );

}














void update_equil_single_gen_mutation( GG *gg, STAT birth, STAT death )
{

  STAT_CELL *create_stat_cell_matrix( STAT_CELL *mm, int ii, int jj );
  
  STAT_CELL *mx = NULL, *a = NULL, *b = NULL;
  mx = (STAT_CELL *)create_stat_cell_matrix( mx, HAPL_BUFFER, HAPL_BUFFER );
  a = (STAT_CELL *)create_stat_cell_matrix( a, HAPL_BUFFER, 1 );
  b = (STAT_CELL *)create_stat_cell_matrix( b, HAPL_BUFFER, 1 );
  
  update_equil_single_gen_mutation_w_mx( gg, mx, a, b, birth, death );
  
  kill_stat_cell( mx );
  kill_stat_cell( a );
  kill_stat_cell( b );

}













void update_equil_single_gen_mutation_w_mx( GG *gg, STAT_CELL *mx, STAT_CELL *a,
  STAT_CELL *b, STAT birth, STAT death )
{

  /*  In this file, below:  */
  set_equil_transition_mx( mx, birth, death );
  
  /*  In deter.c:  */
  update_my_deter_single_gen_mutation_w_mx( gg, mx, a, b );

}













void set_equil_transition_mx( STAT_CELL *x, STAT birth, STAT death )
{

  STAT trans = birth;

  *(x->x + (NO * x->j) + NO) = 1.0 - (2.0 * birth);
  *(x->x + (NO * x->j) + AA) = death;
  *(x->x + (NO * x->j) + CC) = death;
  *(x->x + (NO * x->j) + AC) = 0.0;

  *(x->x + (AA * x->j) + NO) = birth;
  *(x->x + (AA * x->j) + AA) = 1.0 - birth - trans - death;
  *(x->x + (AA * x->j) + CC) = trans;
  *(x->x + (AA * x->j) + AC) = death;

  *(x->x + (CC * x->j) + NO) = birth;
  *(x->x + (CC * x->j) + AA) = trans;
  *(x->x + (CC * x->j) + CC) = 1.0 - birth - trans - death;
  *(x->x + (CC * x->j) + AC) = death;

  *(x->x + (AC * x->j) + NO) = 0.0;
  *(x->x + (AC * x->j) + AA) = birth;
  *(x->x + (AC * x->j) + CC) = birth;
  *(x->x + (AC * x->j) + AC) = 1.0 - (2.0 * death);

}














/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/













void print_equil_step( STAT box_a[], STAT box_c[], int iter, FILE *out )
{

  fprintf( out, "%-4d\t%13.8f  %13.8f\t%13.8f  %13.8f\n",
    iter, box_a[ 0 ], box_a[ 1 ], box_c[ 0 ], box_c[ 1 ] );
    
  fflush( out );

}











