#include "mut.h"
























void do_partition_model_mutations( INFO *my_info, INFO *curr_info,
  PARAMS *my_params )
{

  STAT_CELL *create_stat_cell_matrix( STAT_CELL *xx, int ii, int jj );
  
  STAT_CELL *t = NULL, *a = NULL, *b = NULL;
  t = (STAT_CELL *)create_stat_cell_matrix( t, HAPL_BUFFER, HAPL_BUFFER );
  a = (STAT_CELL *)create_stat_cell_matrix( a, HAPL_BUFFER, 1 );
  b = (STAT_CELL *)create_stat_cell_matrix( b, HAPL_BUFFER, 1 );
  
  do_partition_model_mutations_w_matrices( my_info, curr_info, t, a, b, my_params );
  
  kill_stat_cell( t );
  kill_stat_cell( a );
  kill_stat_cell( b );

}













void do_partition_model_mutations_w_matrices( INFO *my_info, INFO *curr_info,
  STAT_CELL *t, STAT_CELL *a, STAT_CELL *b, PARAMS *my_params )
{

  set_mutation_matrix( t, my_params );
  
  do_partition_model_mutations_mx( my_info, curr_info, t, a, b, my_params );

}














void set_mutation_matrix( STAT_CELL *t, PARAMS *my_params )
{

  STAT birth = my_params->birth;
  STAT death = my_params->death;
  STAT trans = my_params->trans;

  *(t->x + (NO * t->j) + NO) = 1.0 - (2.0 * birth);
  *(t->x + (NO * t->j) + AA) = death;
  *(t->x + (NO * t->j) + CC) = death;
  *(t->x + (NO * t->j) + AC) = 0.0;

  *(t->x + (AA * t->j) + NO) = birth;
  *(t->x + (AA * t->j) + AA) = 1.0 - birth - trans - death;
  *(t->x + (AA * t->j) + CC) = trans;
  *(t->x + (AA * t->j) + AC) = death; 

  *(t->x + (CC * t->j) + NO) = birth;
  *(t->x + (CC * t->j) + AA) = trans;
  *(t->x + (CC * t->j) + CC) = 1.0 - birth - trans - death;
  *(t->x + (CC * t->j) + AC) = death; 

  *(t->x + (AC * t->j) + NO) = 0.0;
  *(t->x + (AC * t->j) + AA) = birth;
  *(t->x + (AC * t->j) + CC) = birth;
  *(t->x + (AC * t->j) + AC) = 1.0 - (2.0 * death); 

}













void do_partition_model_mutations_mx( INFO *my_info, INFO *curr_info, 
  STAT_CELL *t, STAT_CELL *a, STAT_CELL *b, PARAMS *my_params )
{

  PART *part = curr_info->b->x;
  
  int i = 0;
  
  while ( i++ < curr_info->b->n )
    do_mutation_single_partition( part++, t, a, b );

}













void do_mutation_single_partition( PART *part, STAT_CELL *t, STAT_CELL *a,
  STAT_CELL *b )
{

  void multiply_matrices( STAT_CELL *c, STAT_CELL *a, STAT_CELL *b );

  if ( part->count == 0 )
    return;

  /*  In this file, below:  */
  set_freq_vector_partition( a, part );
  
  /*  In ../model/deter.c:  */
  multiply_matrices( b, t, a );
  
  /*  In this file, below:  */
  set_haplotype_freqs_partition( part, b );
  
}













void set_freq_vector_partition( STAT_CELL *a, PART *part )
{

  *(a->x + NO) = part->y[ NO ];
  *(a->x + AA) = part->y[ AA ];
  *(a->x + CC) = part->y[ CC ];
  *(a->x + AC) = part->y[ AC ];

}















void set_haplotype_freqs_partition( PART *part, STAT_CELL *b )
{

  part->y[ NO ] = *(b->x + NO);
  part->y[ AA ] = *(b->x + AA);
  part->y[ CC ] = *(b->x + CC);
  part->y[ AC ] = *(b->x + AC);

}
















/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/




















void do_one_partition_model_mutations( INFO *my_info, INFO *curr_info,
  PARAMS *my_params )
{

  STAT_CELL *create_stat_cell_matrix( STAT_CELL *xx, int ii, int jj );
  
  STAT_CELL *t = NULL, *a = NULL, *b = NULL;
  t = (STAT_CELL *)create_stat_cell_matrix( t, HAPL_BUFFER, HAPL_BUFFER );
  a = (STAT_CELL *)create_stat_cell_matrix( a, HAPL_BUFFER, 1 );
  b = (STAT_CELL *)create_stat_cell_matrix( b, HAPL_BUFFER, 1 );
  
  do_one_partition_model_mutations_w_matrices( my_info, curr_info, t, a, b, 
    my_params );
  
  kill_stat_cell( t );
  kill_stat_cell( a );
  kill_stat_cell( b );

}

















void do_one_partition_model_mutations_w_matrices( INFO *my_info, INFO *curr_info,
  STAT_CELL *t, STAT_CELL *a, STAT_CELL *b, PARAMS *my_params )
{

  /*  In this file, above:  */
  set_mutation_matrix( t, my_params );
  
  /*  In this file, below:  */
  do_one_partition_model_mutations_mx( my_info, curr_info, t, a, b, my_params );

}















void do_one_partition_model_mutations_mx( INFO *my_info, INFO *curr_info,
  STAT_CELL *t, STAT_CELL *a, STAT_CELL *b, PARAMS *my_params )
{

  void multiply_matrices( STAT_CELL *c, STAT_CELL *a, STAT_CELL *b );

  /*  In this file, above:  */
  set_freq_vector_partition( a, curr_info->B );
  
  /*  In ../model/deter.c:  */
  multiply_matrices( b, t, a );
  
  /*  In this file, above:  */
  set_haplotype_freqs_partition( curr_info->B, b );
  
}



















/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
















void do_partition_model_mutations_no_ac( INFO *my_info, INFO *curr_info,
  PARAMS *my_params )
{

  STAT_CELL *create_stat_cell_matrix( STAT_CELL *xx, int ii, int jj );
  
  STAT_CELL *t = NULL, *a = NULL, *b = NULL;
  t = (STAT_CELL *)create_stat_cell_matrix( t, HAPL_BUFFER, HAPL_BUFFER );
  a = (STAT_CELL *)create_stat_cell_matrix( a, HAPL_BUFFER, 1 );
  b = (STAT_CELL *)create_stat_cell_matrix( b, HAPL_BUFFER, 1 );
  
  do_partition_model_mutations_no_ac_w_matrices( my_info, curr_info, t, a, b, 
    my_params );
  
  kill_stat_cell( t );
  kill_stat_cell( a );
  kill_stat_cell( b );

}













void do_partition_model_mutations_no_ac_w_matrices( INFO *my_info, 
  INFO *curr_info, STAT_CELL *t, STAT_CELL *a, STAT_CELL *b, PARAMS *my_params )
{

  /*  In this file, below:  */
  set_mutation_matrix_no_ac( t, my_params );
  
  /*  In this file, above:  */
  do_one_partition_model_mutations_mx( my_info, curr_info, t, a, b, my_params );

}














void set_mutation_matrix_no_ac( STAT_CELL *t, PARAMS *my_params )
{

  STAT birth = my_params->birth;
  STAT death = my_params->death;
  STAT trans = my_params->trans;
  
  *(t->x + (NO * t->j) + NO) = 1.0 - (2.0 * birth);
  *(t->x + (NO * t->j) + AA) = death;
  *(t->x + (NO * t->j) + CC) = death;
  *(t->x + (NO * t->j) + AC) = 0.0;

  *(t->x + (AA * t->j) + NO) = birth;
  *(t->x + (AA * t->j) + AA) = 1.0 - trans - death;
  *(t->x + (AA * t->j) + CC) = trans;
  *(t->x + (AA * t->j) + AC) = 0.0; 

  *(t->x + (CC * t->j) + NO) = birth;
  *(t->x + (CC * t->j) + AA) = trans;
  *(t->x + (CC * t->j) + CC) = 1.0 - trans - death;
  *(t->x + (CC * t->j) + AC) = 0.0; 

  *(t->x + (AC * t->j) + NO) = 0.0;
  *(t->x + (AC * t->j) + AA) = 0.0;
  *(t->x + (AC * t->j) + CC) = 0.0;
  *(t->x + (AC * t->j) + AC) = 0.0;
  
}






