#include "pq.h"














#define ZERO_THRESH 1e-10





















void set_new_deter_info_pq( INFO *my_info, INFO *curr_info, PARAMS *my_params )
{

  STAT_CELL *create_stat_cell( STAT_CELL *xx, int nn );
  
  int nn = my_params->numb_of_genes;
  
  STAT_CELL *n_c = NULL, *n_a = NULL;
  n_c = (STAT_CELL *)create_stat_cell( n_c, nn + 1 );
  n_a = (STAT_CELL *)create_stat_cell( n_a, nn + 1 );
  
  DISTR my_distr = { .n_c = n_c, .n_a = n_a };
  
  set_new_deter_info_pq_n_cells( my_info, curr_info, my_params, &my_distr );
    
  kill_stat_cell( n_c );
  kill_stat_cell( n_a );

}














void set_new_deter_info_pq_n_cells( INFO *my_info, INFO *curr_info, 
  PARAMS *my_params, DISTR *my_distr )
{

  set_new_deter_n_cells( my_distr, my_info );
  
  set_new_deter_fitnesses( my_distr, my_info, my_params );
  
  set_new_deter_pq_vals( my_distr, my_info, curr_info );

}














void set_new_deter_n_cells( DISTR *my_distr, INFO *my_info )
{

  STAT y_0 = *(my_info->y->x + NO);
  STAT y_a = *(my_info->y->x + AA);
  STAT y_c = *(my_info->y->x + CC);
  STAT y_ac = *(my_info->y->x + AC);
  
  STAT p_a = y_a + y_ac;
  STAT p_c = y_c + y_ac;
  
  set_new_deter_cell_binom( my_distr->n_c, 
    (p_c * p_c) + (2.0 * p_c * (1.0 - p_c)) );
    
  set_new_deter_cell_binom( my_distr->n_a, 
    (p_a * p_a) + (2.0 * p_a * (1.0 - p_a)) );
  
}

















void set_new_deter_cell_binom( STAT_CELL *cell, STAT pr )
{
  set_new_deter_cell_binom_from_mean( cell, pr );  
}















void set_new_deter_cell_binom_from_mean( STAT_CELL *cell, STAT pr )
{

  int mean = (int)( pr * (cell->n - 1) );
  
  set_new_deter_cell_binom_mean_up( cell, pr, mean );
  set_new_deter_cell_binom_mean_down( cell, pr, mean );

}













void set_new_deter_cell_binom_mean_up( STAT_CELL *cell, STAT pr, int mean )
{

  int k = mean, n = cell->n - 1;

  STAT x = get_binom_prob( k, pr, cell->n - 1 );
  
  STAT ratio = (pr) / (1.0 - pr);
  STAT a = (STAT)(k + 1.0);
  STAT b = (STAT)(n - k);
  
  STAT *ptr = cell->x + k;
  
  *ptr++ = x;
  
  int i;
  
  for ( i = k + 1; i < cell->n; i++ ) {
  
    x *= ratio * b / a;
    
    *ptr++ = x;
    
    --b;
    ++a;
    
  }  

}
















void set_new_deter_cell_binom_mean_down( STAT_CELL *cell, STAT pr, int mean )
{

  int k = mean, n = cell->n - 1;

  STAT x = get_binom_prob( k, pr, cell->n - 1 );
  
  STAT ratio = (1.0 - pr) / (pr);
  STAT a = (STAT)(k);
  STAT b = (STAT)(n - k + 1.0);
  
  STAT *ptr = cell->x + k;
  
  *ptr-- = x;
  
  int i;
  
  for ( i = k - 1; i >= 0; i-- ) {
  
    x *= ratio * a / b;
    
    *ptr-- = x;
    
    ++b;
    --a;
    
  }  

}









STAT get_binom_prob( int k, STAT pr, int n )
{

  STAT choose = get_log_choose( k, n );

  return exp( choose + log( pow( pr, k )) + log( pow( 1.0 - pr, n - k ) ));

}













STAT get_log_choose( int k, int n )
{

  STAT choose = 0.0;
  
  int kk = 1, nn = n;

  while ( kk <= k ) {
    choose += log( nn-- );
    choose -= log( kk++ );
  }
  
  return choose;

}
















/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/


















void set_new_deter_fitnesses( DISTR *my_distr, INFO *my_info, PARAMS *my_params )
{

  my_distr->w_aa = get_new_deter_waa( my_distr, my_info, my_params );
  my_distr->w_cc = get_new_deter_wcc( my_distr, my_info, my_params );
  my_distr->w_ac = get_new_deter_wac( my_distr, my_info, my_params );
  
}














STAT get_new_deter_waa( DISTR *my_distr, INFO *my_info, PARAMS *my_params )
{

  STAT w = 0.0;
  
  STAT_CELL *n_a = my_distr->n_a;
  
  int i;
  
  for ( i = 0; i < n_a->n; i++ )
    w += get_new_deter_single_waa_term( n_a, my_info, my_params, i );
    
  return w;

}














STAT get_new_deter_single_waa_term( STAT_CELL *cell, INFO *my_info, 
  PARAMS *my_params, int i )
{

  STAT s0 = my_params->s0;
  STAT y0 = *(my_info->y->x + NO);
  STAT yc = *(my_info->y->x + CC);
  
  STAT Y = ( y0 == 0.0 ) ? (0.0) : ((y0) / (yc + y0));
      
  STAT x = pow( 1.0 - s0, my_params->numb_of_genes - i );
  
  STAT pr = Y * Y;
  STAT qr = ( pr == 0.0 ) ? (0.0) : (pow( pr, my_params->numb_of_genes - i ));
  
  return (1.0 - qr) * (*(cell->x + i)) * x;

}















STAT get_new_deter_wcc( DISTR *my_distr, INFO *my_info, PARAMS *my_params )
{

  STAT w = 0.0;
  
  STAT_CELL *n_c = my_distr->n_c;

  int i;
  
  for ( i = 0; i < n_c->n; i++ )
    w += get_new_deter_single_wcc_term( n_c, my_info, my_params, i );
    
  return w;

}
















STAT get_new_deter_single_wcc_term( STAT_CELL *n_c, INFO *my_info, 
  PARAMS *my_params, int i )
{

  STAT sc = my_params->sc;
  STAT y0 = *(my_info->y->x + NO);
  STAT ya = *(my_info->y->x + AA);
  
  STAT Y = ( y0 == 0.0 ) ? (0.0) : ((y0) / (ya + y0));
    
  STAT x = pow( 1.0 + sc, i );
  
  STAT pr = Y * Y;
  STAT qr = ( pr == 0.0 ) ? (0.0) : (pow( pr, my_params->numb_of_genes - i ));
  
  return (1.0 - qr) * (*(n_c->x + i)) * x;

}














STAT get_new_deter_wac( DISTR *my_distr, INFO *my_info, PARAMS *my_params )
{

  STAT w = 0.0;
  
  STAT_CELL *n_c = my_distr->n_c;
  STAT_CELL *n_a = my_distr->n_a;

  int i;
  
  for ( i = 0; i < n_c->n; i++ )
    w += get_new_deter_single_wac_term( n_c, n_a, my_info, my_params, i );
    
  return w;

}

















STAT get_new_deter_single_wac_term( STAT_CELL *n_c, STAT_CELL *n_a, INFO *my_info,
  PARAMS *my_params, int i )
{

  STAT sc = my_params->sc;
  STAT s0 = my_params->s0;
  
  STAT y0 = *(my_info->y->x + NO);
  STAT ya = *(my_info->y->x + AA);
  
  STAT Y = ( y0 == 0.0 ) ? (0.0) : ((y0) / (ya + y0));
  
  STAT x = pow( 1.0 + sc, i );
  STAT z = pow( 1.0 - s0, my_params->numb_of_genes - i );
  
  STAT pr = Y * Y;
  STAT qr = ( pr == 0.0 ) ? (0.0) : (pow( pr, my_params->numb_of_genes - i ));
    
  return (1.0 - qr) * (*(n_c->x + i)) * x * z;

}


















/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/


















void set_new_deter_pq_vals( DISTR *my_distr, INFO *my_info, INFO *curr_info )
{

  STAT z_aa = my_distr->w_aa * my_info->p * my_info->p;
  STAT z_cc = my_distr->w_cc * my_info->q * my_info->q;
  STAT z_ac = my_distr->w_ac * my_info->p * my_info->q;
  
  STAT denom = z_aa + (2.0 * z_ac) + z_cc;
  
  curr_info->p = (z_aa + z_ac) / (denom);
  curr_info->q = (z_cc + z_ac) / (denom);

}























