#include "print.h"



























void open_fh_part_partition_model( FH_PART *my_fh, char output_fname[],
  char output_dir[], char part_dir_fname[], char ww_name[], PARAMS *my_params )
{

  if ( my_params->output == 0 )
    return;
    
  open_my_fh_part_partition_model( my_fh, output_fname, output_dir, part_dir_fname,
    ww_name, my_params );
    
  print_partition_model_headers( my_fh, my_params );
    
}

















void open_my_fh_part_partition_model( FH_PART *my_fh, char output_fname[],
  char output_dir[], char part_dir_fname[], char ww_name[], PARAMS *my_params )
{
  
  FILE *open_text_file_writing( char a[], char b[] );
  
  char fname[ BUFFER ], file_name[ BUFFER ];
  
  sprintf( file_name, "%s_%.0e_%.0e_%.0e", output_fname,
    my_params->birth,
    my_params->death,
    my_params->trans );
    
  sprintf( fname, "%s/%s", output_dir, file_name );

  my_fh->out = (FILE *)open_text_file_writing( fname, "txt" );
  
  sprintf( fname, "%s/%s/%s", output_dir, part_dir_fname, file_name );  

  char *hapl_array[ HAPL_BUFFER ] = { "NO.txt", "AA.txt", "CC.txt", "AC.txt" };
  
  int i;
  for ( i = 0; i < HAPL_BUFFER; i++ ) 
    my_fh->y_fh[ i ] = (FILE *)open_text_file_writing( fname, hapl_array[ i ] );
    
  sprintf( fname, "%s/%s/%s", output_dir, ww_name, file_name );

  my_fh->w_fh = (FILE *)open_text_file_writing( fname, "w.txt" );

}














void get_partition_model_output_fname( char fname[], char output_fname[],
  PARAMS *my_params )
{

  sprintf( fname, "%s_%.0e_%.0e_%.0e", output_fname,
    my_params->birth,
    my_params->death,
    my_params->trans );

}

















void print_partition_model_headers( FH_PART *my_fh, PARAMS *my_params )
{

  print_partition_model_header_single_fh( my_params, my_fh->out );
  
  int i;
  
  for ( i = 0; i < HAPL_BUFFER; i++ )
    print_partition_model_header_single_fh( my_params, my_fh->y_fh[ i ] );

  print_partition_model_header_single_fh( my_params, my_fh->w_fh );

}
















void print_partition_model_header_single_fh( PARAMS *my_params, FILE *fh )
{

  fprintf( fh, "#  Deterministic partition model\n" );
  fprintf( fh, "#  \n" );
  fprintf( fh, "#  Numb of genes:  %d\n", my_params->numb_of_genes );
  fprintf( fh, "#  Pop size:       %d\n", my_params->pop_size );
  fprintf( fh, "#  Init p freq:    %.6f\n", my_params->init_p );
  fprintf( fh, "#  Birth rate:     %.1e\n", my_params->birth );
  fprintf( fh, "#  Death rate:     %.1e\n", my_params->death );
  fprintf( fh, "#  Trans rate:     %.1e\n", my_params->trans );
  fprintf( fh, "#  s0:             %.1e\n", my_params->s0 );
  fprintf( fh, "#  sc:             %.1e\n", my_params->sc );
  fprintf( fh, "#  \n" );
  fprintf( fh, "#  \n" );
  
}










void close_fh_part_partition_model( FH_PART *my_fh )
{

  if ( my_fh->out != NULL )
    close_file( my_fh->out );
    
  int i;
  
  for ( i = 0; i < HAPL_BUFFER; i++ ) {
    if ( my_fh->y_fh[ i ] != NULL )
      close_file( my_fh->y_fh[ i ] );
  }
  
  if ( my_fh->w_fh != NULL )
    close_file( my_fh->w_fh );

}










void flush_fh_part( FH_PART *my_fh )
{

  fflush( my_fh->out );
  fflush( my_fh->w_fh );
  
  int i;
  
  for ( i = 0; i < HAPL_BUFFER; i++ )
    fflush( my_fh->y_fh[ i ] );

}
















/*==============================================================================*/
/*==============================================================================*/
/*==============================================================================*/
/*==============================================================================*/
/*==============================================================================*/



















void print_partition_model_single_generation( INFO *my_info, PARAMS *my_params, 
  int t, FH_PART *my_fh )
{

  printf( "t = %-8d\t\t%.10f\t%.8f\t%.8f\n", 
    t,
    my_info->q,
    my_info->a_freq,
    my_info->c_freq );
    
  if ( my_params->output == 0 )
    return;
    
  if ( t == 0 )
    print_partition_model_column_header( my_fh );
    
  if ( ! output_generation( t ))
    return;
    
  print_my_partition_model_single_generation( my_info, t, my_fh );
    
}












void print_partition_model_single_generation_one_part( INFO *my_info,
  PARAMS *my_params, int t, FH_PART *my_fh )
{

  printf( "t = %-8d\t\t%.10f\t%.8f\t%.8f\n", 
    t,
    my_info->q,
    my_info->a_freq,
    my_info->c_freq );
    
  if ( my_params->output == 0 )
    return;
    
  if ( t == 0 )
    print_partition_model_column_header( my_fh );
    
  if ( ! output_generation( t ))
    return;
    
  print_my_partition_model_single_generation_one_part( my_info, t, my_fh );
  
}













void print_my_partition_model_single_generation_one_part( INFO *my_info,
  int t, FH_PART *my_fh )
{

  print_partition_model_single_gen_out( my_info, t, my_fh->out );
  
  print_partition_model_single_gen_fitness( my_info, t, my_fh->w_fh );
  
  flush_fh_part( my_fh );

}











void print_partition_model_column_header( FH_PART *my_fh )
{

  fprintf( my_fh->out, "%-15s\t%-15s\t%-15s\t%-15s\n", 
    "\"T\"",
    "\"Q\"",
    "\"A_FREQ\"",
    "\"C_FREQ\"" );
    
  int i;
  
  for ( i = 0; i < HAPL_BUFFER; i++ )    
    fprintf( my_fh->y_fh[ i ], "%-15s\t%-15s\t%-15s\t%-15s\t%-15s\n",
      "\"T\"",
      "\"NoBOXes\"",
      "\"BOX_A\"",
      "\"BOX_C\"",
      "\"BothBOX\"" );
      
  fprintf( my_fh->w_fh, "%-15s\t%-15s\t%-15s\t%-15s\n",
    "\"T\"",
    "\"W[AA]\"",
    "\"W[CC]\"",
    "\"W[AC]\"" );
    
  flush_fh_part( my_fh );

}
















int output_generation( int t )
{

  if ( t < 100 )
    return 1;
    
  if ( t < 1000 )
    return ( t % 10 == 0 ) ? (1) : (0);
    
  if ( t < 10000 )
    return ( t % 100 == 0 ) ? (1) : (0);
  
  if ( t < 100000 )
    return ( t % 1000 == 0 ) ? (1) : (0);
    
  return ( t % 10000 == 0 ) ? (1) : (0);

}












void print_my_partition_model_single_generation( INFO *my_info, int t, 
  FH_PART *my_fh )
{

  print_partition_model_single_gen_out( my_info, t, my_fh->out );
  
  print_partition_model_single_gen_partitions( my_info, t, my_fh->y_fh );
  
  print_partition_model_single_gen_fitness( my_info, t, my_fh->w_fh );
  
  flush_fh_part( my_fh );

}














void print_partition_model_single_gen_out( INFO *my_info, int t, FILE *out )
{

  fprintf( out, "%-15d\t%-15.12f\t%-15.10f\t%-15.10f\n",
    t,
    my_info->q,
    my_info->a_freq,
    my_info->c_freq );
    
}

















void print_partition_model_single_gen_partitions( INFO *my_info, int t, 
  FILE *fh[] )
{

  int i;
  
  for ( i = 0; i < HAPL_BUFFER; i++ )
    print_partition_model_single_gen_single_part( my_info->b->x + i, t, fh[ i ] );

}














void print_partition_model_single_gen_single_part( PART *b, int t, FILE *fh )
{

  fprintf( fh, "%-15d\t%-15.8f\t%-15.8f\t%-15.9f\t%-15.8f\n",
    t,
    b->y[ NO ],
    b->y[ AA ],
    b->y[ CC ],
    b->y[ AC ] );

}















void print_partition_model_single_gen_fitness( INFO *my_info, int t, FILE *fh )
{

  fprintf( fh, "%15d\t%-15.6f\t%-15.6f\t%-15.6f\n",
    t,
    my_info->w_aa,
    my_info->w_cc,
    my_info->w_ac );

}






