#include "tf_part.h"
















#define BP_BUFFER 2

#define ZERO_THRESH 1e-10











void do_partition_model_generation_tf( INFO *my_info, INFO *curr_info, 
  PARAMS *my_params )
{

  STAT_CELL *create_stat_cell( STAT_CELL *xx, int nn );
  
  if (( my_info->p == 1.0 ) || ( my_info->p == 0.0 ))
    return;
  
  STAT_CELL *n_c = NULL, *n_a = NULL;
  STAT_CELL *u = NULL, *y = NULL, *z = NULL;
  
  int nn = my_params->numb_of_genes + 1;
  
  n_c = (STAT_CELL *)create_stat_cell( n_c, nn );
  n_a = (STAT_CELL *)create_stat_cell( n_a, nn );
  u = (STAT_CELL *)create_stat_cell( u, nn );
  y = (STAT_CELL *)create_stat_cell( y, nn );
  z = (STAT_CELL *)create_stat_cell( z, nn );
  
  DISTR my_distr = { .n_a = n_a, .n_c = n_c, .u = u, .y = y, .z = z,
    .w_aa = 0.0, .w_ac = 0.0, .w_cc = 0.0 };
  
  do_my_partition_model_generation_tf( my_info, curr_info, &my_distr, my_params );
  
  curr_info->w_aa = my_distr.w_aa;
  curr_info->w_ac = my_distr.w_ac;
  curr_info->w_cc = my_distr.w_cc;
  
  kill_stat_cell( n_c );
  kill_stat_cell( n_a );
  kill_stat_cell( u );
  kill_stat_cell( y );
  kill_stat_cell( z );

}
















void do_my_partition_model_generation_tf( INFO *my_info, INFO *curr_info,
  DISTR *my_distr, PARAMS *my_params )
{

  set_partition_model_fitnesses( my_info, curr_info, my_distr, my_params );
  set_partition_fitness_w_distr( my_info, curr_info, my_distr, my_params );

}













void set_partition_model_fitnesses( INFO *my_info, INFO *curr_info,
  DISTR *my_distr, PARAMS *my_params )
{
  set_partition_model_distr( my_info, curr_info, my_distr ); 
}













void set_partition_model_distr( INFO *my_info, INFO *curr_info,
  DISTR *my_distr )
{

  if ( (my_info->b->x + NO)->count != 0 )
    error( 5000, "Expected zero value for y_0 array.\n" );
    
  if ( my_info->b->n != HAPL_BUFFER )
    error( 5001, "Expected four elements in partition cell.\n" );
    
  set_my_partition_model_distr( my_info, curr_info, my_distr );

}














void set_my_partition_model_distr( INFO *my_info, INFO *curr_info,
  DISTR *my_distr )
{

  int a_array[ BP_BUFFER ] = { AA, AC };
  int a_not = CC;
  
  int c_array[ BP_BUFFER ] = { CC, AC };
  int c_not = AA;

  set_partition_model_single_distr( my_distr->n_a, my_distr, my_info, a_array,
    a_not );
    
  set_partition_model_single_distr( my_distr->n_c, my_distr, my_info, c_array,
    c_not );

}













void set_partition_model_single_distr( STAT_CELL *d, DISTR *my_distr, INFO *my_info,
  int nucl_array[], int nucl_not )
{

  int aa_count = (my_info->b->x + AA)->count;
  int cc_count = (my_info->b->x + CC)->count;
  int ac_count = (my_info->b->x + AC)->count;

  int y_mean = set_binom_array_distr( my_distr->y, my_info->b->x + AA, nucl_array,
    nucl_not );
    
  int z_mean = set_binom_array_distr( my_distr->z, my_info->b->x + CC, nucl_array,
    nucl_not );
  
  set_binom_array_sum_distr( my_distr->u, my_distr->y, my_distr->z, aa_count, 
    cc_count, y_mean + z_mean );
    
  int x_mean = set_binom_array_distr( my_distr->z, my_info->b->x + AC, nucl_array,
    nucl_not );
  
  set_binom_array_sum_distr( d, my_distr->u, my_distr->z, d->n, ac_count, 
    x_mean + y_mean + z_mean );
    
}















int set_binom_array_distr( STAT_CELL *z, PART *b, int nucl_array[], int nucl_not )
{

  init_stat_cell( z );

  STAT pr = get_prob_box_occur( b, nucl_array );

  int mean = (int)(pr * b->count);

  set_binom_array_distr_up( z, b, mean, pr, nucl_not );
  set_binom_array_distr_down( z, b, mean, pr, nucl_not );
  
  return mean;
  
}















void init_stat_cell( STAT_CELL *z )
{

  STAT *zptr = z->x;
  
  int i = 0;
  
  while ( i++ < z->n )
    *zptr++ = 0.0;

}















STAT get_prob_box_occur( PART *b, int nucl_array[] )
{

  STAT y = b->y[ nucl_array[ 0 ]] + b->y[ nucl_array[ 1 ]];
  
  return (y * y) + (2.0 * y * (1.0 - y));

}















void set_binom_array_distr_up( STAT_CELL *z, PART *b, int mean, STAT pr,
  int nucl_not )
{

  STAT get_binom_prob( int k, STAT pr, int n );
  
  int k = mean, n = b->count;
  
  /*  In pq.c:  */
  STAT x = get_binom_prob( k, pr, n );
  
  STAT y = b->y[ NO ] / (b->y[ NO ] + b->y[ nucl_not ]);
  STAT yy = y * y;
  
  STAT frac = 1.0 - yy;
  
  if ( isnan( frac ) || isinf( frac ))
    frac = 1.0;
  
  x *= pow( frac, n - k );
  
  STAT ratio = (pr) / (1.0 - pr);
  STAT a = (STAT)(k + 1.0);
  STAT c = (STAT)(n - k);
  
  STAT *zptr = z->x + k;
  
  *zptr++ = x;
  
  int i;
  
  for ( i = k + 1; i <= b->count; i++ ) {
  
    x *= (ratio * c) / (a * frac);
    
    *zptr++ = x;
    
    --c;
    ++a;
  
  }
  
}



















void set_binom_array_distr_down( STAT_CELL *z, PART *b, int mean, STAT pr,
  int nucl_not )
{

  STAT get_binom_prob( int k, STAT pr, int n );
  
  int k = mean, n = b->count;
  
  /*  In pq.c:  */
  STAT x = get_binom_prob( k, pr, n );
  
  STAT y = b->y[ NO ] / (b->y[ NO ] + b->y[ nucl_not ]);
  STAT yy = y * y;
  
  STAT frac = 1.0 - yy;
  
  if ( isnan( frac ) || isinf( frac ))
    frac = 1.0;
  
  x *= pow( frac, n - k );
  
  STAT ratio = (1.0 - pr) / (pr);
  STAT a = (STAT)(k);
  STAT c = (STAT)(n - k + 1.0);
  
  STAT *zptr = z->x + k;
  
  *zptr-- = x;
  
  int i;
  
  for ( i = k - 1; i >= 0; i-- ) {
  
    x *= (ratio * a * frac) / (c);
    
    *zptr-- = x;
    
    ++c;
    --a;
    
    if ( x < ZERO_THRESH )
      break;
    
  }
  
}



















void set_binom_array_sum_distr( STAT_CELL *u, STAT_CELL *y, STAT_CELL *z,
  int y_n, int z_n, int mean )
{

  set_binom_array_sum_distr_up( u, y, z, y_n + z_n, mean );  
  set_binom_array_sum_distr_down( u, y, z, y_n + z_n, mean );
  
}












void set_binom_array_sum_distr_up( STAT_CELL *u, STAT_CELL *y, STAT_CELL *z,
  int total, int mean )
{

  int i, j;
  
  for ( i = mean; i <= total; i++ ) {
  
    *(u->x + i) = 0.0;
    
    for ( j = 0; j <= i; j++ ) {
    
      STAT yy = *(y->x + j);
      STAT zz = *(z->x + i - j);
      
      *(u->x + i) += yy * zz;
      
    }
    
    if ( i == mean )
      continue;
    
    if (( *(u->x + i) < ZERO_THRESH ) && ( *(u->x + i) < *(u->x + i - 1) ))
      break;
    
  }
  
}















void set_binom_array_sum_distr_down( STAT_CELL *u, STAT_CELL *y, STAT_CELL *z,
  int total, int mean )
{

  int i, j;

  for ( i = mean - 1; i >= 0; i-- ) {
  
    *(u->x + i) = 0.0;
    
    for ( j = 0; j <= i; j++ ) {
    
      STAT yy = *(y->x + j);
      STAT zz = *(z->x + i - j);
      
      *(u->x + i) += yy * zz;
      
    }
    
    if ( i == mean - 1 )
      continue;
    
    if (( *(u->x + i) < ZERO_THRESH ) && ( *(u->x + i) < *(u->x + i + 1) ))
      break;
    
  }

}
















/*==============================================================================*/
/*==============================================================================*/
/*==============================================================================*/
/*==============================================================================*/
/*==============================================================================*/


















void set_partition_fitness_w_distr( INFO *my_info, INFO *curr_info, 
  DISTR *my_distr, PARAMS *my_params )
{

  get_partition_relative_fitnesses( my_info, my_distr, my_params );
  
  curr_info->p = get_new_p_freq_partition( my_distr, my_info );
  curr_info->q = 1.0 - curr_info->p;
  
}













void get_partition_relative_fitnesses( INFO *my_info, DISTR *my_distr,
  PARAMS *my_params )
{

  my_distr->w_aa = get_fitness_waa_partition( my_info, my_distr, my_params );
  my_distr->w_cc = get_fitness_wcc_partition( my_info, my_distr, my_params );
  my_distr->w_ac = get_fitness_wac_partition( my_info, my_distr, my_params );
  
}














STAT get_fitness_waa_partition( INFO *my_info, DISTR *my_distr, PARAMS *my_params )
{

  STAT w = 0.0;
  
  STAT_CELL *n_a = my_distr->n_a;

  int i;
  
  for ( i = 0; i < n_a->n; i++ ) 
    w += get_fitness_waa_partition_single_term( n_a, my_info, my_params, i );
  
  return w;

}













STAT get_fitness_waa_partition_single_term( STAT_CELL *n_a, INFO *my_info,
  PARAMS *my_params, int i )
{

  STAT s0 = my_params->s0;
  int n = my_params->numb_of_genes;
  
  STAT x = pow( 1.0 - s0, n - i );
  
  STAT pr = *(n_a->x + i);
  
  return pr * x;

}















STAT get_fitness_wcc_partition( INFO *my_info, DISTR *my_distr, PARAMS *my_params )
{

  STAT w = 0.0;
  
  STAT_CELL *n_c = my_distr->n_c;

  int i;
  
  for ( i = 0; i < n_c->n; i++ ) 
    w += get_fitness_wcc_partition_single_term( n_c, my_info, my_params, i );
  
  return w;

}













STAT get_fitness_wcc_partition_single_term( STAT_CELL *n_c, INFO *my_info,
  PARAMS *my_params, int i )
{

  STAT sc = my_params->sc;
  STAT s0 = my_params->s0;

  int n = my_params->numb_of_genes;
  
  STAT x = pow( 1.0 + sc, i ) * pow( 1.0 - s0, n - i );
  
  STAT pr = *(n_c->x + i);
  
  return pr * x;

}












STAT get_fitness_wac_partition( INFO *my_info, DISTR *my_distr, PARAMS *my_params )
{

  STAT w = 0.0;
  
  STAT_CELL *n_c = my_distr->n_c;

  int i;
  
  for ( i = 0; i < n_c->n; i++ ) 
    w += get_fitness_wac_partition_single_term( n_c, my_info, my_params, i );
  
  return w;

}













STAT get_fitness_wac_partition_single_term( STAT_CELL *n_c, INFO *my_info,
  PARAMS *my_params, int i )
{

  STAT s0 = my_params->s0;
  STAT sc = my_params->sc;
  
  STAT x = pow( 1.0 + sc, i );
  
  STAT pr = *(n_c->x + i);
  
  return pr * x;

}















STAT get_new_p_freq_partition( DISTR *my_distr, INFO *my_info )
{

  STAT p = my_info->p;
  STAT q = my_info->q;

  STAT aa = my_distr->w_aa * p * p;
  STAT ac = my_distr->w_ac * p * q;
  STAT cc = my_distr->w_cc * q * q;
  
  return (aa + ac) / (aa + (2.0 * ac) + cc);

}
















