#include "box_count.h"















#define INPUT_FNAME "data/occur/hg19.gcBox.count.txt"
#define OUTPUT_FNAME "data/occur/gcBox.count.results.txt"





#define MAX_COUNT 10

#define MAJOR_NUCL A_NUCL












void do_gc_box_spec_count( void )
{

  do_my_gc_box_spec_count( INPUT_FNAME, OUTPUT_FNAME );

}












void do_my_gc_box_spec_count( char input_fname[], char output_fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *in = NULL, *out = NULL;
  in = (FILE *)open_text_file_reading( input_fname, "" );
  out = (FILE *)open_text_file_writing( output_fname, "" );
  
  do_gc_box_spec_count_w_fh( in, out );
  
  close_file( in );
  close_file( out ); 

}












void do_gc_box_spec_count_w_fh( FILE *in, FILE *out )
{

  print_sep( out );

  do_gc_box_spec_count_single_spec_pair( "macEug1", "macEug1", MAJOR_NUCL, in, out );
  do_gc_box_spec_count_single_spec_pair( "monDom5", "monDom5", MAJOR_NUCL, in, out );
  do_gc_box_spec_count_single_spec_pair( "ornAna1", "ornAna1", MAJOR_NUCL, in, out );

}











void do_gc_box_spec_count_single_spec_pair( char major_spec[], char minor_spec[],
  char major_nucl, FILE *in, FILE *out )
{

  char get_nucl_from_numb( char numb );

  /*  In ../main/nucl.c:  */
  fprintf( out, "Major:  %-7s   Minor:  %-7s   MajorMf:  G%c box\n", 
    major_spec, 
    minor_spec,
    get_nucl_from_numb( major_nucl ));
  
  int i;
  
  for ( i = 0; i < NUMB_OF_NUCL; i++ ) {
  
    /*  In ../main/nucl.c:  */
    fprintf( out, "%8sMinorMf:  G%c box\n", "", get_nucl_from_numb( i ) );
    
    /*  In this file, below:  */
    do_gc_box_spec_count_single_mf( major_spec, minor_spec, major_nucl, i, 
      in, out );
  
  }
  
  print_sep( out );
  
}













void do_gc_box_spec_count_single_mf( char major_spec[], char minor_spec[],
  char major_nucl, char minor_nucl, FILE *in, FILE *out )
{

  COUNT_CELL *create_count_cell( COUNT_CELL *x, int nn );
  
  COUNT_CELL *my_cell = NULL;
  my_cell = (COUNT_CELL *)create_count_cell( my_cell, MAX_COUNT );
  
  get_and_put_gc_box_spec_count( my_cell, major_spec, minor_spec, major_nucl,
    minor_nucl, in, out );
    
  kill_count_cell( my_cell );
  
}











void get_and_put_gc_box_spec_count( COUNT_CELL *my_cell, char major_spec[],
  char minor_spec[], char major_nucl, char minor_nucl, FILE *in, FILE *out )
{

  get_gc_box_spec_count( my_cell, major_spec, minor_spec, major_nucl, minor_nucl,
    in );
    
  put_gc_box_spec_count( my_cell, major_spec, minor_spec, out );
    
}










void get_gc_box_spec_count( COUNT_CELL *my_cell, char major_spec[], 
  char minor_spec[], char major_nucl, char minor_nucl, FILE *in )
{

  int get_entry_index_from_str( char line[], char str[] );
  
  char line[ LINE_BUFFER ];
  
  set_position( 0, in );
  
  if ( fgets( line, LINE_BUFFER, in ) == NULL )
    error( 1100, "Unexpectedly empty file.\n" );
  
  /*  Index numbers are 1-based...  */
  /*  In ../main/parse.c:  */
  int major_index = get_entry_index_from_str( line, major_spec );
  int minor_index = get_entry_index_from_str( line, minor_spec );
  
  /*  In this file, below:  */
  get_gc_box_spec_count_entry_index( my_cell, major_index, minor_index, major_nucl,
    minor_nucl, in );

}














void get_gc_box_spec_count_entry_index( COUNT_CELL *my_cell, int major_index,
  int minor_index, char major_nucl, char minor_nucl, FILE *in )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL )
    get_gc_box_spec_count_single_line( my_cell, line, major_index, minor_index,
      major_nucl, minor_nucl );

}












void get_gc_box_spec_count_single_line( COUNT_CELL *my_cell, char line[],
  int major_index, int minor_index, char major_nucl, char minor_nucl )
{

  void get_entry_from_index( char entry[], char line[], int index );
  int get_nucl_entry_count( char entry[], char numb );
  
  char major_entry[ BUFFER ], minor_entry[ BUFFER ];
  
  /*  In ../main/parse.c:  */
  get_entry_from_index( major_entry, line, major_index );
  
  /*  In ../main/parse.c:  */
  if ( ! get_nucl_entry_count( major_entry, major_nucl ))
    return;
  
  /*  In this file, below:  */
  get_gc_box_spec_count_existing_box( my_cell, line, minor_index, minor_nucl );

}












void get_gc_box_spec_count_existing_box( COUNT_CELL *my_cell, char line[],
  int minor_index, char minor_nucl )
{

  void get_entry_from_index( char entry[], char line[], int index );
  int get_nucl_entry_count( char entry[], char numb );
  
  char major_entry[ BUFFER ], minor_entry[ BUFFER ];
  
  /*  In ../main/parse.c:  */
  get_entry_from_index( minor_entry, line, minor_index );
  
  printf( "%s\n", minor_entry );
  
  /*  In ../main/parse.c:  */
  int x = get_nucl_entry_count( minor_entry, minor_nucl );
    
  if (( x < 0 ) || ( x >= my_cell->n ))
    return;
    
  ++(*(my_cell->x + x));

}













void put_gc_box_spec_count( COUNT_CELL *my_cell, char major_spec[], 
  char minor_spec[], FILE *out )
{

  COUNT total = get_total_count_from_cell( my_cell );
  
  int i;
  
  for ( i = 0; i < my_cell->n; i++ ) 
    fprintf( out, "\t\t%2d\t%d\t(%.1f%%)\n", i, 
      (int)*(my_cell->x + i),
      (float)*(my_cell->x + i) * 100.0 / (float)(total) );
      
  fprintf( out, "\t\tTotal:\t%d\n", total );
  fprintf( out, "\t\tExist:\t%d\t(%.1f%%)\n",
    total - (int)*(my_cell->x),
    (float)(total - *(my_cell->x)) * 100.0 / (float)(total) );

}













COUNT get_total_count_from_cell( COUNT_CELL *my_cell )
{

  int i = 0;
  
  COUNT total = 0, *ptr = my_cell->x;
  
  while ( i++ < my_cell->n )
    total += *ptr++;
    
  return total;

}







