#include "atype.h"

















#define INPUT_FNAME "data/c2h2/raw_data/1ARD.pdb"
#define PHI_PSI_FNAME "data/c2h2/params/aa_phi_psi.txt"
#define PHI_PSI_BIN_FNAME "data/c2h2/params/aa_phi_psi_bins.txt"


#define DEGREES_TO_RADIANS(a) (PI * (a) / 180.0)











void do_atom_type( void )
{

  DATA *my_data = NULL;
  
  my_data = (DATA *)make_data_struct( my_data );
  
  my_data = (DATA *)update_data_struct_w_fname( my_data, INPUT_FNAME );
  
}












DATA *make_data_struct( DATA *my_data )
{

  DATA *create_data( DATA *xx );
  
  my_data = (DATA *)create_data( my_data );
  
  return make_and_set_data_struct_type_cell( my_data );  
  
}













DATA *make_and_set_data_struct_type_cell( DATA *my_data )
{  
  
  my_data->type = (TYPE_CELL *)make_and_set_type_cell( my_data->type );
  
  return my_data;
  
}











TYPE_CELL *make_and_set_type_cell( TYPE_CELL *my_type )
{

  TYPE_CELL *create_type_cell( TYPE_CELL *x, int nn );
  
  my_type = (TYPE_CELL *)create_type_cell( my_type, NUMB_OF_TYPES );
  
  return set_type_cell( my_type );

}













TYPE_CELL *set_type_cell( TYPE_CELL *my_type )
{

  TYPE *t = my_type->x;

  char i;
  
  for ( i = 0; i < my_type->n; i++ )
    set_type_cell_elem( t++, i );
    
  return my_type; 

}












void set_type_cell_elem( TYPE *t, int i )
{

  switch ( i ) {
  
  case T_C_NYL :    strcpy( t->description, "Side chain carbonyl C" );
  case T_C_OXL :    strcpy( t->description, "Side chain carboxyl C" );
    t->r = 2.0;
    t->e = 0.1200;
    break;
    
  case T_C_1_H :    strcpy( t->description, "Aliphatic C with 1 H" );
    t->r = 2.0;
    t->e = 0.0486;
    break;
    
  case T_C_2_H :    strcpy( t->description, "Aliphatic C with 2 H" );
    t->r = 2.0;
    t->e = 0.1142;
    break;
    
  case T_C_3_H :    strcpy( t->description, "Aliphatic C with 3 H" );
    t->r = 2.0;
    t->e = 0.1811;
    break;
    
  case T_C_ARO :    strcpy( t->description, "Aromatic ring C" );
    t->r = 2.0;
    t->e = 0.1200;
    break;
    
  case T_N_PTN :    strcpy( t->description, "Protonated N" );
  case T_N_DEP :    strcpy( t->description, "Deprotonated N" );
  case T_N_NEU :    strcpy( t->description, "Neutral NH2 Nitrogen" );
  case T_N_LYS :    strcpy( t->description, "Lysine N" );
  case T_N_ARG :    strcpy( t->description, "Arginine N" );
  case T_N_PRO :    strcpy( t->description, "Proline N" );
    t->r = 1.75;
    t->e = 0.2384;
    t->acc = 1;
    break;
  
  case T_O_HYD :    strcpy( t->description, "Hydroxyl O" );
  case T_O_NYL :    strcpy( t->description, "Side chain carbonyl O" );
  case T_O_XYL :    strcpy( t->description, "Side chain carboxyl O" );
    t->r = 1.55;
    t->e = 0.1591;
    t->don = 1;
    t->acc = ( i == T_O_HYD ) ? (1) : (0);
    break;
  
  case T_S_SUL :    strcpy( t->description, "Sulfur" );
    t->r = 1.90;
    t->e = 0.1600;
    break;
    
  case T_N_BAC :    strcpy( t->description, "Backbone N" );
    t->r = 1.75;
    t->e = 0.2384;
    break;
    
  case T_C_CAL :    strcpy( t->description, "Backbone alpha C" );
    t->r = 2.0;
    t->e = 0.0486;
    break;
    
  case T_C_BBC :    strcpy( t->description, "Backbone carbonyl C" );
    t->r = 2.0;
    t->e = 0.1400;
    break;
    
  case T_O_BBO :    strcpy( t->description, "Backbone O" );
    t->r = 1.55;
    t->e = 0.1591;
    break;
    
  case T_Z_ZIN :    strcpy( t->description, "Zinc ion" );
    break;
    
  case T_H_POL :    strcpy( t->description, "Polar H" );
    t->r = 0.10;
    t->e = 0.0500;
    break;
    
  case T_H_NON :    strcpy( t->description, "Nonpolar H" );
  case T_H_ARO :    strcpy( t->description, "Aromatic H" );
    t->r = 0.70;
    t->e = 0.0500;
    break;
    
  case T_H_BBH :    strcpy( t->description, "Backbone NH2 H" );
    t->r = 0.10;
    t->e = 0.0500;
    break;

  default :
    error( 3100, "Atom type not found.\n" );
    break;
  
  }
  
  int j;
  
  STAT pow = 1.0;
  
  for ( j = 0; j < SHORT_BUFFER; j++ ) {
    t->r_pow[ j ] = pow;
    pow *= t->r;
  }

}















RTYPE_CELL *make_and_set_rtype_cell( RTYPE_CELL *my_rtype )
{

  RTYPE_CELL *create_rtype_cell( RTYPE_CELL *rr, int nn );
  
  my_rtype = (RTYPE_CELL *)create_rtype_cell( my_rtype, NUMB_OF_AA );

  my_rtype = (RTYPE_CELL *)set_rtype_cell_w_fname( my_rtype, PHI_PSI_FNAME );
  
  return set_rtype_cell_w_fname_bin( my_rtype, PHI_PSI_BIN_FNAME );

}











RTYPE_CELL *set_rtype_cell_w_fname( RTYPE_CELL *my_rtype, char fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( fname, "" );
  
  my_rtype = (RTYPE_CELL *)set_rtype_cell_w_fh( my_rtype, in );
  
  close_file( in );
  
  return my_rtype;
  
}











RTYPE_CELL *set_rtype_cell_w_fh( RTYPE_CELL *my_rtype, FILE *in )
{

  int i = 0;

  RTYPE *r = my_rtype->x;

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, in ) != NULL ) {
  
    if ( line[ 0 ] == '=' )
      return my_rtype;
      
    if ( line[ 0 ] == '#' )
      continue;  
      
    if ( ++i > my_rtype->n )
      error( 3120, "Too many residue types in resid type file.\n" );  
      
    set_rtype_cell_single_line( r++, line );
  
  }
  
  return my_rtype;

}












void set_rtype_cell_single_line( RTYPE *r, char line[] )
{

  float phi[ CO_BUFFER ], psi[ CO_BUFFER ];
  float phi_gamma, psi_gamma;

  sscanf( line, "%*d%s%f%f%f%f%f%f%f%f",
    r->resid,
    phi + 0,
    phi + 1,
    phi + 2,
    psi + 0,
    psi + 1,
    psi + 2,
    &phi_gamma,
    &psi_gamma );
    
  int i;
  
  for ( i = 0; i < CO_BUFFER; i++ ) {
    r->phi_coeff[ i ] = phi[ i ];
    r->psi_coeff[ i ] = psi[ i ];
  }
  
  r->phi_gamma = DEGREES_TO_RADIANS( phi_gamma );
  r->psi_gamma = DEGREES_TO_RADIANS( psi_gamma );

}















RTYPE_CELL *make_and_set_rtype_cell_bin( RTYPE_CELL *my_rtype )
{

  RTYPE_CELL *create_rtype_cell( RTYPE_CELL *rr, int nn );
  
  my_rtype = (RTYPE_CELL *)create_rtype_cell( my_rtype, NUMB_OF_AA );

  return set_rtype_cell_w_fname_bin( my_rtype, PHI_PSI_BIN_FNAME );

}












RTYPE_CELL *set_rtype_cell_w_fname_bin( RTYPE_CELL *my_rtype, char fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( fname, "" );
  
  my_rtype = (RTYPE_CELL *)set_rtype_cell_bin_w_fh( my_rtype, in );
  
  close_file( in );

  return my_rtype;

}















RTYPE_CELL *set_rtype_cell_bin_w_fh( RTYPE_CELL *my_rtype, FILE *in )
{

  RTYPE *r = my_rtype->x;

  char line[ BUFFER ];

  while ( fgets( line, BUFFER, in ) != NULL )
    set_rtype_cell_single_aa_bin( r++, line, in );
    
  return my_rtype;

}













void set_rtype_cell_single_aa_bin( RTYPE *r, char line[], FILE *in )
{

  if ( line[ 0 ] != '>' )
    error( 3160, "Expected angle char at beginning of residue entry.\n" );
    
  strncpy( r->resid, line + 1, strlen( line + 1 ) - 1 );
  
  int i;
  
  for ( i = 0; i < NUMB_OF_ANGLES; i++ )
    set_rtype_cell_single_aa_single_angle_bin( r->phi_array + i, r->psi_array + i,
      in );

}














void set_rtype_cell_single_aa_single_angle_bin( STAT *phi, STAT *psi, FILE *in )
{

  char line[ BUFFER ];
  float my_phi, my_psi;

  fgets( line, BUFFER, in );
  
  sscanf( line, "%*d%*d%*d%f%*d%f", &my_phi, &my_psi );
  
  *phi = my_phi;
  *psi = my_psi;

}









/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
















DATA *update_data_struct_w_fname( DATA *my_data, char fname[] )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  
  MOL_LIST *mol_header = NULL;
  /*  In ../mm/mol.c:  */
  mol_header = (MOL_LIST *)get_mol_list_w_fname( mol_header, fname );
  
  /*  In this file, below:  */
  return update_data_struct_w_new_mol_list( my_data, mol_header );

}














DATA *update_data_struct_w_new_mol_list( DATA *my_data, MOL_LIST *mol_header )
{

  set_atom_types_mol_list( mol_header, my_data->type );
  
  return make_and_put_new_mm_lister_into_data( my_data, mol_header );

}














DATA *make_and_put_new_mm_lister_into_data( DATA *my_data, MOL_LIST *mol_header )
{

  MM_LIST *create_mm_lister( MM_LIST *xx );
  
  MM_LIST *mm = NULL;
  mm = (MM_LIST *)create_mm_lister( mm );
  
  mm->my_mm->mol_header = mol_header;
  
  return put_mm_lister_into_data( my_data, mm );

}












DATA *put_mm_lister_into_data( DATA *my_data, MM_LIST *mm )
{

  if ( my_data->mm_header == NULL ) {
    my_data->mm_header = mm;
    return my_data;
  }
  
  MM_LIST *curr = my_data->mm_header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = mm;
  
  return my_data;

}














/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/














void set_atom_types_mol_list( MOL_LIST *mol_header, TYPE_CELL *my_type )
{

  MOL_LIST *mol = mol_header;
  
  while ( mol != NULL )
    set_atom_types_single_mol_lister_and_step( &mol, my_type );

}














void set_atom_types_single_mol_lister_and_step( MOL_LIST **mol,
  TYPE_CELL *my_type )
{
  set_atom_types_single_mol_lister( *mol, my_type );
  *mol = (*mol)->next;
}













void set_atom_types_single_mol_lister( MOL_LIST *mol, TYPE_CELL *my_type )
{

  RESID_LIST *resid = mol->my_mol->resid_header;
  
  while ( resid != NULL )
    set_atom_types_single_resid_lister_and_step( &resid, my_type );

}












void set_atom_types_single_resid_lister_and_step( RESID_LIST **resid,
  TYPE_CELL *my_type )
{
  set_atom_types_single_resid_lister( *resid, my_type );
  *resid = (*resid)->next;
}













void set_atom_types_single_resid_lister( RESID_LIST *resid, TYPE_CELL *my_type )
{

  if ( resid->my_resid->mol_type == MOL_TYPE_NUCL )
    return;
    
  ATOM_LIST *atom = resid->my_resid->atom_header;
  char aa = resid->my_resid->aa;
  
  while ( atom != NULL )
    set_atom_types_single_atom_lister_and_step( &atom, aa, my_type );

}













void set_atom_types_single_atom_lister_and_step( ATOM_LIST **atom, char aa,
  TYPE_CELL *my_type )
{
  set_atom_types_single_atom_lister( *atom, aa, my_type );
  *atom = (*atom)->next;
}














void set_atom_types_single_atom_lister( ATOM_LIST *atom, char aa, 
  TYPE_CELL *my_type )
{

  if ( set_atom_types_backbone( atom, aa, my_type ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "ZN", my_type->x + T_Z_ZIN ))
    return;
    
  set_atom_types_single_sidechain_atom_lister( atom, aa, my_type );

}














int set_atom_types_backbone( ATOM_LIST *atom, char aa, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CA", my_type->x + T_C_CAL ))
    return 1;
    
  if ( atom_lister_is_atom_type( atom, "C", my_type->x + T_C_BBC ))
    return 1;
    
  if ( atom_lister_is_atom_type( atom, "O", my_type->x + T_O_BBO ))
    return 1;
    
  if ( atom_lister_is_atom_type( atom, "OXT", my_type->x + T_O_BBO ))
    return 1;
    
  if ( atom_lister_is_atom_type( atom, "HA", my_type->x + T_H_NON ))
    return 1;
    
  if ( atom_lister_is_atom_type( atom, "N", my_type->x + T_N_BAC ))
    return 1;
    
  if ( atom_lister_is_atom_type( atom, "H", my_type->x + T_H_BBH ))
    return 1;
    
  if ( atom_lister_is_atom_type( atom, "H1", my_type->x + T_H_BBH ))
    return 1;
    
  if ( atom_lister_is_atom_type( atom, "H2", my_type->x + T_H_BBH ))
    return 1;

  if ( atom_lister_is_atom_type( atom, "H3", my_type->x + T_H_BBH ))
    return 1;
    
  return 0;

}














int atom_lister_is_atom_type( ATOM_LIST *atom, char x[], TYPE *type )
{

  if ( strcmp( atom->my_atom->atom, x ))
    return 0;
    
  atom->my_atom->t = type;
  
  return 1;

}
















void set_atom_types_single_sidechain_atom_lister( ATOM_LIST *atom, char aa,
  TYPE_CELL *my_type )
{

  switch ( aa ) {
  
  case 'A' :
    set_atom_type_ala_resid( atom, my_type );
    break;
  
  case 'V' :
    set_atom_type_val_resid( atom, my_type );
    break;
    
  case 'L' :
    set_atom_type_leu_resid( atom, my_type );
    break;
    
  case 'I' :
    set_atom_type_ise_resid( atom, my_type );
    break;
    
  case 'F' :
    set_atom_type_phe_resid( atom, my_type );
    break;
    
  case 'W' :
    set_atom_type_trp_resid( atom, my_type );
    break;
    
  case 'M' :
    set_atom_type_met_resid( atom, my_type );
    break;
    
  case 'P' :
    set_atom_type_pro_resid( atom, my_type );
    break;
    
  case 'D' :
    set_atom_type_asp_resid( atom, my_type );
    break;
    
  case 'E' :
    set_atom_type_glu_resid( atom, my_type );
    break;
    
  case 'G' :
    set_atom_type_gly_resid( atom, my_type );
    break;
    
  case 'S' :
    set_atom_type_ser_resid( atom, my_type );
    break;
    
  case 'T' :
    set_atom_type_thr_resid( atom, my_type );
    break;
    
  case 'C' :
    set_atom_type_cys_resid( atom, my_type );
    break;
    
  case 'Y' :
    set_atom_type_tyr_resid( atom, my_type );
    break;
    
  case 'N' :
    set_atom_type_asn_resid( atom, my_type );
    break;
    
  case 'Q' :
    set_atom_type_gln_resid( atom, my_type );
    break;
    
  case 'K' :
    set_atom_type_lys_resid( atom, my_type );
    break;
    
  case 'R' :
    set_atom_type_arg_resid( atom, my_type );
    break;
    
  case 'H' :
    set_atom_type_his_resid( atom, my_type );
    break;
    
  case '\0' : 
    break;
    
  default :
    error( 3101, "Unrecognized amino acid type.\n" );
    break;
  
  }

}
















/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/












void set_resid_types_mol_list( MOL_LIST *mol_header, RTYPE_CELL *my_rtype )
{

  MOL_LIST *mol = mol_header;
  
  while ( mol != NULL )
    set_resid_types_single_mol_lister_and_step( &mol, my_rtype );

}











void set_resid_types_single_mol_lister_and_step( MOL_LIST **mol, 
  RTYPE_CELL *my_rtype )
{
  set_resid_types_single_mol_lister( *mol, my_rtype );
  *mol = (*mol)->next;
}











void set_resid_types_single_mol_lister( MOL_LIST *mol, RTYPE_CELL *my_rtype )
{

  RESID_LIST *resid = mol->my_mol->resid_header;
  
  while ( resid != NULL )
    set_resid_types_single_resid_lister_and_step( &resid, my_rtype );

}












void set_resid_types_single_resid_lister_and_step( RESID_LIST **resid,
  RTYPE_CELL *my_rtype )
{
  set_resid_types_single_resid_lister( *resid, my_rtype );
  *resid = (*resid)->next;
}










void set_resid_types_single_resid_lister( RESID_LIST *resid, RTYPE_CELL *my_rtype )
{

  if ( resid->my_resid->mol_type != MOL_TYPE_AA )
    return;

  resid->my_resid->type = (RTYPE *)get_rtype_from_cell_w_resid( my_rtype, 
    resid->my_resid->resid );

}









RTYPE *get_rtype_from_cell_w_resid( RTYPE_CELL *my_rtype, char res[] )
{

  RTYPE *r = my_rtype->x;
  
  int i = 0;
  
  while ( i++ < my_rtype->n ) {
  
    if ( ! strcmp( r->resid, res )) 
      return r;
      
    ++r;
  
  }
  
  error( 3121, "Could not find resid type in residue cell.\n" );
  
  return NULL;

}









/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
















void set_atom_type_ala_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CB", my_type->x + T_C_3_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB1", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HB2", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HB3", my_type->x + T_H_NON ))
    return;
    
  error( 3110, "Unknown atom type for alanine residue.\n" );

}















void set_atom_type_val_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CB", my_type->x + T_C_1_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CG1", my_type->x + T_C_3_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CG2", my_type->x + T_C_3_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG11", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG12", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG13", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG21", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG22", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG23", my_type->x + T_H_NON ))
    return;

  error( 3111, "Unknown atom type for valine residue.\n" );

}












void set_atom_type_leu_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CB", my_type->x + T_C_2_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CG", my_type->x + T_C_1_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CD1", my_type->x + T_C_3_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CD2", my_type->x + T_C_3_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB2", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HB3", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HD11", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HD12", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HD13", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HD21", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HD22", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HD23", my_type->x + T_H_NON ))
    return;
    
  error( 3112, "Unknown atom type for leucine residue.\n" );

}















void set_atom_type_ise_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CB", my_type->x + T_C_1_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CG1", my_type->x + T_C_2_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CG2", my_type->x + T_C_3_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CD1", my_type->x + T_C_3_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG12", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG13", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG21", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG22", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG23", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HD11", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HD12", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HD13", my_type->x + T_H_NON ))
    return;
    
  error( 3113, "Unknown atom type for isoleucine residue.\n" );

}















void set_atom_type_phe_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CB", my_type->x + T_C_2_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CG", my_type->x + T_C_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "CD1", my_type->x + T_C_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "CE1", my_type->x + T_C_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "CZ", my_type->x + T_C_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "CD2", my_type->x + T_C_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "CE2", my_type->x + T_C_ARO ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB2", my_type->x + T_H_NON ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB3", my_type->x + T_H_NON ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HD1", my_type->x + T_H_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "HE1", my_type->x + T_H_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "HZ", my_type->x + T_H_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "HD2", my_type->x + T_H_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "HE2", my_type->x + T_H_ARO ))
    return;

  error( 3113, "Unknown atom type for phenylalanine residue.\n" );

}
















void set_atom_type_trp_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CB", my_type->x + T_C_2_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CG", my_type->x + T_C_ARO ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CD1", my_type->x + T_C_ARO ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CE2", my_type->x + T_C_ARO ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CD2", my_type->x + T_C_ARO ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CZ2", my_type->x + T_C_ARO ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CE3", my_type->x + T_C_ARO ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CZ3", my_type->x + T_C_ARO ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CH2", my_type->x + T_C_ARO ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "NE1", my_type->x + T_N_PTN ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB2", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HB3", my_type->x + T_H_NON ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HD1", my_type->x + T_H_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "HE1", my_type->x + T_H_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "HZ2", my_type->x + T_H_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "HH2", my_type->x + T_H_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "HZ3", my_type->x + T_H_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "HE3", my_type->x + T_H_ARO ))
    return;

  error( 3113, "Unknown atom type for phenylalanine residue.\n" );

}










void set_atom_type_met_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CB", my_type->x + T_C_2_H ))
    return;

  if ( atom_lister_is_atom_type( atom, "CG", my_type->x + T_C_2_H ))
    return;

  if ( atom_lister_is_atom_type( atom, "CE", my_type->x + T_C_3_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "SD", my_type->x + T_S_SUL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB2", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HB3", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG2", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG3", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HE1", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HE2", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HE3", my_type->x + T_H_NON ))
    return;
    
  error( 3114, "Unknown atom type for methionine residue.\n" );

}
















void set_atom_type_pro_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CB", my_type->x + T_C_2_H ))
    return;

  if ( atom_lister_is_atom_type( atom, "CG", my_type->x + T_C_2_H ))
    return;

  if ( atom_lister_is_atom_type( atom, "CD", my_type->x + T_C_2_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "N", my_type->x + T_N_PRO ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "H", my_type->x + T_H_POL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB2", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HB3", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG2", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG3", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HD2", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HD3", my_type->x + T_H_NON ))
    return;

  error( 3115, "Unknown atom type for methionine residue.\n" );

}















void set_atom_type_asp_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CB", my_type->x + T_C_2_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CG", my_type->x + T_C_OXL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "OD1", my_type->x + T_O_XYL ))
    return;

  if ( atom_lister_is_atom_type( atom, "OD2", my_type->x + T_O_XYL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HD2", my_type->x + T_H_POL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB2", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HB3", my_type->x + T_H_NON ))
    return;

  error( 3116, "Unknown atom type for aspartic acid residue.\n" );

}














void set_atom_type_glu_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CB", my_type->x + T_C_2_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CG", my_type->x + T_C_2_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CD", my_type->x + T_C_OXL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "OE1", my_type->x + T_O_XYL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "OE2", my_type->x + T_O_XYL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HE2", my_type->x + T_H_POL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB2", my_type->x + T_H_NON ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB3", my_type->x + T_H_NON ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HG2", my_type->x + T_H_NON ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HG3", my_type->x + T_H_NON ))
    return;
    
  error( 3117, "Unknown atom type for glutamine acid residue.\n" );
    
}














void set_atom_type_gly_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "HA2", my_type->x + T_H_NON ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HA3", my_type->x + T_H_NON ))
    return;
        
  error( 3118, "Unknown atom type for glycine residue.\n" );

}














void set_atom_type_ser_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CB", my_type->x + T_C_2_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "OG", my_type->x + T_O_HYD ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HG", my_type->x + T_H_POL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB2", my_type->x + T_H_NON ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB3", my_type->x + T_H_NON ))
    return;
    
  error( 3119, "Unknown atom type for serine residue.\n" );

}














void set_atom_type_thr_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CB", my_type->x + T_C_1_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CG2", my_type->x + T_C_3_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "OG1", my_type->x + T_O_HYD ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HG1", my_type->x + T_H_POL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG21", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG22", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG23", my_type->x + T_H_NON ))
    return;
    
  error( 3120, "Unknown atom type for threonine residue.\n" );

}















void set_atom_type_cys_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CB", my_type->x + T_C_2_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "SG", my_type->x + T_S_SUL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HG", my_type->x + T_H_POL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB2", my_type->x + T_H_NON ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB3", my_type->x + T_H_NON ))
    return; 

  error( 3121, "Unknown atom type for cysteine residue.\n" );

}









void set_atom_type_tyr_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CB", my_type->x + T_C_2_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CG", my_type->x + T_C_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "CD1", my_type->x + T_C_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "CE1", my_type->x + T_C_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "CZ", my_type->x + T_C_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "CD2", my_type->x + T_C_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "CE2", my_type->x + T_C_ARO ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "OH", my_type->x + T_O_HYD ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HD1", my_type->x + T_H_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "HE1", my_type->x + T_H_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "HH", my_type->x + T_H_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "HD2", my_type->x + T_H_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "HE2", my_type->x + T_H_ARO ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB2", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HB3", my_type->x + T_H_NON ))
    return;

  error( 3122, "Unknown atom type for tyrosine residue.\n" );

}















void set_atom_type_asn_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CB", my_type->x + T_C_2_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CG", my_type->x + T_C_NYL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "ND2", my_type->x + T_N_NEU ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "OD1", my_type->x + T_O_NYL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HD21", my_type->x + T_H_POL ))
    return;

  if ( atom_lister_is_atom_type( atom, "HD22", my_type->x + T_H_POL ))
    return;

  if ( atom_lister_is_atom_type( atom, "HB2", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HB3", my_type->x + T_H_NON ))
    return;

  error( 3123, "Unknown atom type for asparagine residue.\n" );

}















void set_atom_type_gln_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CB", my_type->x + T_C_2_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CG", my_type->x + T_C_2_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CD", my_type->x + T_C_NYL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "NE2", my_type->x + T_N_NEU ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "OE1", my_type->x + T_O_NYL ))
    return;

  if ( atom_lister_is_atom_type( atom, "HE21", my_type->x + T_H_POL ))
    return; 

  if ( atom_lister_is_atom_type( atom, "HE22", my_type->x + T_H_POL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB2", my_type->x + T_H_NON ))
    return; 
    
  if ( atom_lister_is_atom_type( atom, "HB3", my_type->x + T_H_NON ))
    return; 
    
  if ( atom_lister_is_atom_type( atom, "HG2", my_type->x + T_H_NON ))
    return; 
    
  if ( atom_lister_is_atom_type( atom, "HG3", my_type->x + T_H_NON ))
    return; 
    
  error( 3124, "Unknown atom type for glutamine residue.\n" );

}














void set_atom_type_lys_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CB", my_type->x + T_C_2_H ))
    return;

  if ( atom_lister_is_atom_type( atom, "CG", my_type->x + T_C_2_H ))
    return;

  if ( atom_lister_is_atom_type( atom, "CD", my_type->x + T_C_2_H ))
    return;

  if ( atom_lister_is_atom_type( atom, "CE", my_type->x + T_C_2_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "NZ", my_type->x + T_N_LYS ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HZ1", my_type->x + T_H_POL ))
    return;

  if ( atom_lister_is_atom_type( atom, "HZ2", my_type->x + T_H_POL ))
    return;

  if ( atom_lister_is_atom_type( atom, "HZ3", my_type->x + T_H_POL ))
    return;

  if ( atom_lister_is_atom_type( atom, "HB2", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HB3", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG2", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG3", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HD2", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HD3", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HE2", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HE3", my_type->x + T_H_NON ))
    return;

  error( 3125, "Unknown atom type for lysine residue.\n" );

}














void set_atom_type_arg_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CB", my_type->x + T_C_2_H ))
    return; 

  if ( atom_lister_is_atom_type( atom, "CG", my_type->x + T_C_2_H ))
    return; 

  if ( atom_lister_is_atom_type( atom, "CD", my_type->x + T_C_2_H ))
    return; 

  if ( atom_lister_is_atom_type( atom, "CZ", my_type->x + T_C_NYL ))
    return; 
    
  if ( atom_lister_is_atom_type( atom, "NE", my_type->x + T_N_ARG ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "NH1", my_type->x + T_N_ARG ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "NH2", my_type->x + T_N_ARG ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HH11", my_type->x + T_H_POL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HH12", my_type->x + T_H_POL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HH21", my_type->x + T_H_POL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HH22", my_type->x + T_H_POL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HE", my_type->x + T_H_POL ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB2", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HB3", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG2", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HG3", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HD2", my_type->x + T_H_NON ))
    return;

  if ( atom_lister_is_atom_type( atom, "HD3", my_type->x + T_H_NON ))
    return;
    
  error( 3126, "Unknown atom type for arginine residue.\n" );

}

















void set_atom_type_his_resid( ATOM_LIST *atom, TYPE_CELL *my_type )
{

  if ( atom_lister_is_atom_type( atom, "CB", my_type->x + T_C_2_H ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "CG", my_type->x + T_C_ARO ))
    return; 

  if ( atom_lister_is_atom_type( atom, "CD2", my_type->x + T_C_ARO ))
    return; 

  if ( atom_lister_is_atom_type( atom, "CE1", my_type->x + T_C_ARO ))
    return; 
    
  if ( atom_lister_is_atom_type( atom, "ND1", my_type->x + T_N_DEP ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "NE2", my_type->x + T_N_PTN ))
    return;   
    
  if ( atom_lister_is_atom_type( atom, "HD2", my_type->x + T_H_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "HE2", my_type->x + T_H_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "HE1", my_type->x + T_H_ARO ))
    return;

  if ( atom_lister_is_atom_type( atom, "HD1", my_type->x + T_H_ARO ))
    return;
    
  if ( atom_lister_is_atom_type( atom, "HB2", my_type->x + T_H_NON ))
    return; 

  if ( atom_lister_is_atom_type( atom, "HB3", my_type->x + T_H_NON ))
    return; 

  error( 3127, "Unknown atom type for histidine residue.\n" );

}


















