#include "der.h"















#define DIFF_MX_DIM 25

#define H_VAL 2.0
#define THRESHOLD 1.0e-3

#define H_VAL_PHI_PSI 0.05




#define ABS(a) (((a) > 0.0) ? (a) : (-1.0 * (a)))



























/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/













STAT get_double_deriv_angles( MOL_LIST *mol, RESID_LIST *resid_a, 
  RESID_LIST *resid_b,
  void (*a_func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh ),
  void (*b_func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh ) )
{

  MATRIX *create_matrix( MATRIX *mm, int ii, int jj );
  
  MATRIX *mx = NULL;
  mx = (MATRIX *)create_matrix( mx, DIFF_MX_DIM, DIFF_MX_DIM );
  
  STAT x = get_my_double_deriv_angles( mx, mol, resid_a, resid_b, a_func, b_func );
  
  kill_matrix( mx );

  return x;

}













STAT get_my_double_deriv_angles( MATRIX *mx, MOL_LIST *mol, RESID_LIST *resid_a,
  RESID_LIST *resid_b,
  void (*a_func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh ),
  void (*b_func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh ) )
{

  STAT h = H_VAL;
  
  *(mx->x) = get_numerical_deriv_angles( mol, resid_a, resid_b, a_func, b_func, h );
  
  int i, j;
  
  for ( i = 1; i < mx->i; i++ ) {
  
    h /= 2.0;
    
    /*  In this file, below:  */
    *(mx->x + (i * mx->j)) = get_numerical_deriv_angles( mol, resid_a, resid_b,
      a_func, b_func, h );
      
    for ( j = 1; j <= i; j++ )
      /*  In this file, above:  */
      *(mx->x + (i * mx->j) + j) = get_deriv_mx_elem( mx, i, j );
      
    STAT diff = *(mx->x + (i * mx->j) + i) - *(mx->x + ((i-1) * mx->j) + (i-1));
    
    if ( ABS(diff) < THRESHOLD ) 
      return *(mx->x + (i * mx->j) + i);
      
  }
  
  return *(mx->x);

}











STAT get_numerical_deriv_angles( MOL_LIST *mol, RESID_LIST *resid_a,
  RESID_LIST *resid_b, 
  void (*a_func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh ),
  void (*b_func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh ), STAT h )
{

  STAT get_free_energy_mol_list( MOL_LIST *mol_header );
  void rotate_psi_angle( MOL_LIST *mol, RESID_LIST *resid, STAT angle );
  void rotate_phi_angle( MOL_LIST *mol, RESID_LIST *resid, STAT angle );
  MOL_LIST *copy_mol_lister( MOL_LIST *major, MOL_LIST *minor );
  RESID_LIST *get_resid_lister_w_numb( MOL_LIST *mol, int numb );


  MOL_LIST *copy = NULL;
  copy = (MOL_LIST *)copy_mol_lister( copy, mol );

  set_mol_list_neighbors( copy );

  RESID_LIST *r = (RESID_LIST *)get_resid_lister_w_numb( copy,
    resid_a->my_resid->numb );
  RESID_LIST *rb = (RESID_LIST *)get_resid_lister_w_numb( copy,
    resid_b->my_resid->numb );


  /*  In phi_psi.c:  */
  a_func( copy, r, h );
  
  /*  In this file, below:  */
  STAT a = get_numerical_deriv_one_angle( copy, rb, b_func );
  
  /*  In phi_psi.c:  */
  a_func( copy, r, -2.0 * h );
  
  /*  In free_energy.c:  */
  STAT b = get_numerical_deriv_one_angle( copy, rb, b_func );
  
  kill_mol_list( copy );
  
  return (a - b) / (2.0 * h);

}












STAT get_numerical_deriv_one_angle( MOL_LIST *mol, RESID_LIST *resid,
  void (*func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh ) )
{

  MATRIX *create_matrix( MATRIX *mm, int ii, int jj );
  
  MATRIX *mx = NULL;
  mx = (MATRIX *)create_matrix( mx, DIFF_MX_DIM, DIFF_MX_DIM );
  
  STAT x = get_my_numerical_deriv_one_angle( mx, mol, resid, func );
  
  kill_matrix( mx );
  
  return x;

}












STAT get_my_numerical_deriv_one_angle( MATRIX *mx, MOL_LIST *mol, 
  RESID_LIST *resid, void (*func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh ) )
{

  STAT h = H_VAL;
  
  *(mx->x) = get_deriv_one_angle( mol, resid, func, h );
  
  int i, j;
  
  for ( i = 1; i < mx->i; i++ ) {
  
    h /= 2.0;
    
    *(mx->x + (i * mx->j)) = get_deriv_one_angle( mol, resid, func, h );
    
    for ( j = 1; j <= i; j++ )
      *(mx->x + (i * mx->j) + j) = get_deriv_mx_elem( mx, i, j );
      
    STAT diff = *(mx->x + (i * mx->j) + i) - *(mx->x + ((i-1) * mx->j) + (i-1));
    
    if ( ABS(diff) < THRESHOLD )
      return *(mx->x + (i * mx->j) + i);

  }
  
//  error( 3203, "Unable to obtain derivative in max steps.\n" );
  
  return *(mx->x);

}










STAT get_deriv_one_angle( MOL_LIST *mol, RESID_LIST *resid,
  void (*func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh ), STAT h )
{

  void rotate_phi_angle( MOL_LIST *mol, RESID_LIST *resid, STAT angle );
  void rotate_psi_angle( MOL_LIST *mol, RESID_LIST *resid, STAT angle );
  STAT get_free_energy_mol_list( MOL_LIST *mol_header );
  MOL_LIST *copy_mol_lister( MOL_LIST *major, MOL_LIST *minor );
  RESID_LIST *get_resid_lister_w_numb( MOL_LIST *mol, int numb );


  MOL_LIST *copy = NULL;
  /*  In ../mm/mol.c:  */
  copy = (MOL_LIST *)copy_mol_lister( copy, mol );

  /*  In free_energy.c:  */
  set_mol_list_neighbors( copy );

  /*  In peptide.c:  */
  RESID_LIST *r = (RESID_LIST *)get_resid_lister_w_numb( copy,
    resid->my_resid->numb );

  /*  In phi_psi.c:  */
  func( copy, r, h );
  
  /*  In free_energy.c:  */
  STAT a = get_free_energy_mol_list( copy );
  
  /*  In phi_psi.c:  */
  func( copy, r, -2.0 * h );
  
  /*  In free_energy.c:  */
  STAT b = get_free_energy_mol_list( copy );
    
  STAT x = (a - b) / (2.0 * h);
  
  kill_mol_list( copy );
  
  return x;

}















STAT get_deriv_mx_elem( MATRIX *mx, int i, int j )
{

  STAT aa = *(mx->x + (i * mx->j) + (j-1));
  STAT bb = *(mx->x + ((i-1) * mx->j) + (j-1));
  
  STAT x = pow( 4.0, (double)j );
  
  return aa + ((aa - bb) / (x - 1.0));

}
















/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/















STAT get_phi_psi_single_deriv( MATRIX *p, MOL_LIST *sep, int i )
{

  MATRIX *create_matrix( MATRIX *mm, int ii, int jj );
  
  MATRIX *mx = NULL;
  mx = (MATRIX *)create_matrix( mx, DIFF_MX_DIM, DIFF_MX_DIM );
  
  STAT x = get_my_phi_psi_single_deriv( mx, p, sep, i );
  
  kill_matrix( mx );

  return x;

}













STAT get_my_phi_psi_single_deriv( MATRIX *mx, MATRIX *p, MOL_LIST *sep, int k )
{

  STAT h = H_VAL_PHI_PSI;
  
  *(mx->x) = get_numerical_deriv_phi_psi( p, sep, k, h );
  
  int i, j;
  
  for ( i = 1; i < mx->i; i++ ) {
  
    h /= 2.0;
    
    /*  In this file, below:  */
    *(mx->x + (i * mx->j)) = get_numerical_deriv_phi_psi( p, sep, k, h );
      
    for ( j = 1; j <= i; j++ )
      /*  In this file, above:  */
      *(mx->x + (i * mx->j) + j) = get_deriv_mx_elem( mx, i, j );
      
    STAT diff = *(mx->x + (i * mx->j) + i) - *(mx->x + ((i-1) * mx->j) + (i-1));
    
    if ( ABS(diff) < THRESHOLD ) 
      return *(mx->x + (i * mx->j) + i);
      
  }
  
  return *(mx->x);

}















STAT get_numerical_deriv_phi_psi( MATRIX *p, MOL_LIST *sep, int i, STAT h )
{

  STAT get_phi_psi_prob_sep_mol_list_mx( MATRIX *p, MOL_LIST *sep );

  *(p->x + i) += h;

  /*  In pep_params.c:  */
  STAT a = get_phi_psi_prob_sep_mol_list_mx( p, sep );
  
  *(p->x + i) -= 2.0 * h;
  
  /*  In pep_params.c:  */
  STAT b = get_phi_psi_prob_sep_mol_list_mx( p, sep );
  
  *(p->x + i) += h;
  
  return (a - b) / (2.0 * h);

}














STAT get_phi_psi_double_deriv( MATRIX *p, MOL_LIST *sep, int i, int j )
{

  MATRIX *create_matrix( MATRIX *mm, int ii, int jj );
  
  MATRIX *mx = NULL;
  mx = (MATRIX *)create_matrix( mx, DIFF_MX_DIM, DIFF_MX_DIM );
  
  STAT x = get_my_phi_psi_double_deriv( mx, p, sep, i, j );
  
  kill_matrix( mx );

  return x;

}













STAT get_my_phi_psi_double_deriv( MATRIX *mx, MATRIX *p, MOL_LIST *sep, int a,
  int b )
{

  STAT h = H_VAL_PHI_PSI;
  
  *(mx->x) = get_numerical_double_deriv_phi_psi( p, sep, a, b, h );
  
  int i, j;
  
  for ( i = 1; i < mx->i; i++ ) {
  
    h /= 2.0;
    
    /*  In this file, below:  */
    *(mx->x + (i * mx->j)) = get_numerical_double_deriv_phi_psi( p, sep, a, b, h );
      
    for ( j = 1; j <= i; j++ )
      /*  In this file, above:  */
      *(mx->x + (i * mx->j) + j) = get_deriv_mx_elem( mx, i, j );
      
    STAT diff = *(mx->x + (i * mx->j) + i) - *(mx->x + ((i-1) * mx->j) + (i-1));
    
    if ( ABS(diff) < THRESHOLD ) 
      return *(mx->x + (i * mx->j) + i);
      
  }
  
  return *(mx->x);

}











STAT get_numerical_double_deriv_phi_psi( MATRIX *p, MOL_LIST *sep, int i, int j,
  STAT h )
{

  *(p->x + i) += h;

  STAT a = get_phi_psi_single_deriv( p, sep, j );
  
  *(p->x + i) -= 2.0 * h;
  
  STAT b = get_phi_psi_single_deriv( p, sep, j );
  
  *(p->x + i) += h;
  
  return (a - b) / (2.0 * h);

}













/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/














STAT get_deriv_rot_params( MATRIX *p, XYZ_CELL *major_cell, XYZ_CELL *minor_cell, 
  int i )
{

  MATRIX *create_matrix( MATRIX *mm, int ii, int jj );
  
  MATRIX *mx = NULL;
  mx = (MATRIX *)create_matrix( mx, DIFF_MX_DIM, DIFF_MX_DIM );
  
  STAT x = get_my_deriv_rot_params( mx, p, major_cell, minor_cell, i );
  
  kill_matrix( mx );

  return x;

}













STAT get_my_deriv_rot_params( MATRIX *mx, MATRIX *p, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell, int a )
{

  STAT h = H_VAL_PHI_PSI;
  
  /*  In this file, below:  */
  *(mx->x) = get_deriv_single_rot_param( p, major_cell, minor_cell, a, h );
  
  int i, j;
  
  for ( i = 1; i < mx->i; i++ ) {
  
    h /= 2.0;
    
    /*  In this file, below:  */
    *(mx->x + (i * mx->j)) = get_deriv_single_rot_param( p, major_cell, minor_cell, 
      a, h );
    
    for ( j = 1; j <= i; j++ )
      /*  In this file, above:  */
      *(mx->x + (i * mx->j) + j) = get_deriv_mx_elem( mx, i, j );
      
    STAT diff = *(mx->x + (i * mx->j) + i) - *(mx->x + ((i-1) * mx->j) + (i-1));
    
    if ( ABS(diff) < THRESHOLD )
      return *(mx->x + (i * mx->j) + i);

  }
  
  return *(mx->x);

}













STAT get_deriv_single_rot_param( MATRIX *p, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell, int a, STAT h )
{

  STAT get_rmsd_rotation_matrix( XYZ_CELL *major_cell, XYZ_CELL *minor_cell, 
    MATRIX *p );

  *(p->x + a) += h;

  /*  In rmsd.c:  */
  STAT f = get_rmsd_rotation_matrix( major_cell, minor_cell, p );
  
  *(p->x + a) -= 2 * h;
  
  /*  In rmsd.c:  */
  STAT g = get_rmsd_rotation_matrix( major_cell, minor_cell, p );
  
  *(p->x + a) += h;
    
  return (f - g) / (2.0 * h);

}













STAT get_double_deriv_rot_params( MATRIX *p, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell, int a, int b )
{

  MATRIX *create_matrix( MATRIX *mm, int ii, int jj );
  
  MATRIX *mx = NULL;
  mx = (MATRIX *)create_matrix( mx, DIFF_MX_DIM, DIFF_MX_DIM );
  
  STAT x = get_my_double_deriv_rot_params( mx, p, major_cell, minor_cell, a, b );
  
  kill_matrix( mx );

  return x;

}














STAT get_my_double_deriv_rot_params( MATRIX *mx, MATRIX *p, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell, int a, int b )
{

  STAT h = H_VAL_PHI_PSI;
  
  /*  In this file, below:  */
  *(mx->x) = get_double_deriv_single_rot_param( p, major_cell, minor_cell, a, b, h );
  
  int i, j;
  
  for ( i = 1; i < mx->i; i++ ) {
  
    h /= 2.0;
    
    /*  In this file, below:  */
    *(mx->x + (i * mx->j)) = get_double_deriv_single_rot_param( p, major_cell, 
      minor_cell, a, b, h );
    
    for ( j = 1; j <= i; j++ )
      /*  In this file, above:  */
      *(mx->x + (i * mx->j) + j) = get_deriv_mx_elem( mx, i, j );
      
    STAT diff = *(mx->x + (i * mx->j) + i) - *(mx->x + ((i-1) * mx->j) + (i-1));
    
    if ( ABS(diff) < THRESHOLD )
      return *(mx->x + (i * mx->j) + i);

  }
  
  return *(mx->x);

}












STAT get_double_deriv_single_rot_param( MATRIX *p, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell, int a, int b, STAT h )
{

  *(p->x + a) += h;
  
  /*  In this file, above:  */
  STAT f = get_deriv_rot_params( p, major_cell, minor_cell, b );

  *(p->x + a) -= 2.0 * h;

  /*  In this file, above:  */
  STAT g = get_deriv_rot_params( p, major_cell, minor_cell, b );
  
  *(p->x + a) += h;
  
  return (f - g) / (2.0 * h);
  

}












