#include "free_energy.h"

















#define DIST_THRESH 20.0

#define ZINC_K 640.0
//#define ZINC_K 1000.0

#define ZINC_SG_DIST 2.298
#define ZINC_NE_DIST 2.053

#define W_ANGLE 0.594
#define W_ATTR 0.777
#define W_REP 0.627
#define REP_ATTR_DIST 1.12

#define DEGREES_TO_RADIANS(a) (PI * (a) / 180.0)
#define RADIANS_TO_DEGREES(a) (180.0 * (a) / PI)












STAT do_free_energy_mol_list( MOL_LIST *mol_header ) 
{   

  DATA *make_data_struct( DATA *my_data );
  RTYPE_CELL *make_and_set_rtype_cell( RTYPE_CELL *my_rtype );
  DATA *update_data_struct_w_fname( DATA *my_data, char fname[] );

  DATA *my_data = NULL;
  RTYPE_CELL *my_rtype = NULL;
  
  /*  In atype.c:  */
  my_data = (DATA *)make_data_struct( my_data );
  my_rtype = (RTYPE_CELL *)make_and_set_rtype_cell( my_rtype );
    
  /*  In atype.c:  */
  set_atom_types_mol_list( mol_header, my_data->type );
  
  /*  In this file, below:  */
  set_mol_list_neighbors( mol_header );
  
  /*  In this file, below:  */
  return get_free_energy_mol_w_two_type_cells( mol_header, my_data->type,
    my_rtype );
  
}















STAT get_free_energy_mol_w_type_cell( MOL_LIST *mol_header, TYPE_CELL *my_type )
{

  /*  In atype.c:  */
  set_atom_types_mol_list( mol_header, my_type );
  
  /*  In this file, below:  */
  set_mol_list_neighbors( mol_header );
  
  /*  In this file, below:  */
  return get_free_energy_mol_list( mol_header );

}













STAT get_free_energy_mol_w_two_type_cells( MOL_LIST *mol_header, TYPE_CELL *my_type,
  RTYPE_CELL *my_rtype )
{

  /*  In atype.c:  */
  set_atom_types_mol_list( mol_header, my_type );
  set_resid_types_mol_list( mol_header, my_rtype );
  
  /*  In this file, below:  */
  set_mol_list_neighbors( mol_header );
  
  /*  In this file, below:  */
  return get_free_energy_mol_list( mol_header );

}







/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/














void set_mol_list_neighbors( MOL_LIST *mol_header )
{

  MOL_LIST *mol = mol_header;
  
  while ( mol != NULL ) 
    set_single_mol_lister_neighbors_and_step( &mol );

}














void set_single_mol_lister_neighbors_and_step( MOL_LIST **mol )
{
  set_single_mol_lister_neighbors( *mol );
  *mol = (*mol)->next;
}














void set_single_mol_lister_neighbors( MOL_LIST *mol )
{

  RESID_LIST *resid = mol->my_mol->resid_header;
  
  while ( resid != NULL )
    set_single_resid_lister_neighbors_and_step( &resid, mol->my_mol->resid_header );

}














void set_single_resid_lister_neighbors_and_step( RESID_LIST **resid,
  RESID_LIST *resid_header )
{
  set_single_resid_lister_neighbors( *resid, resid_header );
  *resid = (*resid)->next;
}













void set_single_resid_lister_neighbors( RESID_LIST *resid, 
  RESID_LIST *resid_header )
{

  if ( resid->my_resid->mol_type != MOL_TYPE_AA )
    return;

  RESID_LIST *neigh_header = NULL;
  RESID_LIST *minor_resid = resid_header;
  
  while ( minor_resid != NULL )
    neigh_header = (RESID_LIST *)get_neigh_resid_single_pair_and_step( neigh_header,
      resid, &minor_resid );
      
  resid->my_resid->neigh_header = neigh_header;

}













RESID_LIST *get_neigh_resid_single_pair_and_step( RESID_LIST *neigh_header,
  RESID_LIST *resid, RESID_LIST **minor_resid )
{

  neigh_header = (RESID_LIST *)get_neigh_resid_single_pair( neigh_header,
    resid, *minor_resid );
    
  *minor_resid = (*minor_resid)->next;
  
  return neigh_header;

}














RESID_LIST *get_neigh_resid_single_pair( RESID_LIST *neigh_header, 
  RESID_LIST *resid, RESID_LIST *minor_resid )
{

  if ( resid->my_resid->numb == minor_resid->my_resid->numb )
    return neigh_header;
    
  if ( minor_resid->my_resid->mol_type != MOL_TYPE_AA )
    return neigh_header;

  return get_my_neigh_resid_single_pair( neigh_header, resid, minor_resid );

}













RESID_LIST *get_my_neigh_resid_single_pair( RESID_LIST *neigh_header,
  RESID_LIST *resid, RESID_LIST *minor_resid )
{

  ATOM_LIST *ca = NULL, *ca_minor = NULL;
  ca = (ATOM_LIST *)get_atom_from_name( resid->my_resid->atom_header, "CA" );
  ca_minor = (ATOM_LIST *)get_atom_from_name( minor_resid->my_resid->atom_header,
    "CA" );
    
  if (( ca == NULL ) || ( ca_minor == NULL )) 
    error( 2100, "Did not find alpha carbon in atom list.\n" );
  
  return get_neigh_resid_atom_pair( neigh_header, minor_resid, ca, ca_minor );

}














ATOM_LIST *get_atom_from_name( ATOM_LIST *header, char name[] )
{

  ATOM_LIST *atom = header;
  
  while ( atom != NULL ) {
  
    if ( ! strcmp( atom->my_atom->atom, name ))
      return atom;
      
    atom = atom->next;
  
  }
  
  return NULL;

}













RESID_LIST *get_neigh_resid_atom_pair( RESID_LIST *neigh_header, 
  RESID_LIST *minor_resid, ATOM_LIST *ca, ATOM_LIST *ca_minor )
{

  STAT get_dist_xyz( XYZ *a, XYZ *b );
  
  /*  In ../main/xyz.c:  */
  STAT dist = get_dist_xyz( ca->my_atom->xyz, ca_minor->my_atom->xyz );
  
  if ( dist < DIST_THRESH )
    /*  In this file, below:  */
    return set_neigh_resid_to_neighbor_list( neigh_header, minor_resid );
    
  return neigh_header;

}












RESID_LIST *set_neigh_resid_to_neighbor_list( RESID_LIST *neigh_header,
  RESID_LIST *minor_resid )
{

  void check_null( int numb, ... );
  
  RESID_LIST *new = NULL;
  new = (RESID_LIST *)malloc( sizeof(RESID_LIST) );
  
  check_null( 1, new );
  
  new->my_resid = minor_resid->my_resid;
  new->next = NULL;
  
  if ( neigh_header == NULL )
    return new;
    
  RESID_LIST *curr = neigh_header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = new;
  
  return neigh_header;

}















/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/












STAT get_free_energy_mol_list( MOL_LIST *mol_header )
{

  /*  In phi_psi.c:  */
  set_phi_psi_angles_w_mol_list( mol_header );

  STAT g = 0.0;
  
  MOL_LIST *mol = mol_header;
  
  while ( mol != NULL )
    /*  In this file, below:  */
    g += get_free_energy_single_mol_and_step( &mol );
    
  return g;

}















STAT get_free_energy_single_mol_and_step( MOL_LIST **mol )
{
  STAT g = get_free_energy_single_mol( *mol );
  *mol = (*mol)->next;
  return g;
}














STAT get_free_energy_single_mol( MOL_LIST *mol )
{

  STAT g = 0.0;

  RESID_LIST *resid = mol->my_mol->resid_header;
  
  while ( resid != NULL )
    g += get_free_energy_single_resid_lister_and_step( &resid );
    
  return g;

}













STAT get_free_energy_single_resid_lister_and_step( RESID_LIST **resid )
{
  STAT g = get_free_energy_single_resid_lister( *resid );
  *resid = (*resid)->next;
  return g;
}













STAT get_free_energy_single_resid_lister( RESID_LIST *resid )
{

  if ( ! strcmp( resid->my_resid->resid, "ZN" )) 
    return get_free_energy_zinc_ion( resid );

  STAT g = 0.0;
  STAT a = get_free_energy_resid_phi_psi_angles( resid );
    
  RESID_LIST *neigh = resid->my_resid->neigh_header;
  
  while ( neigh != NULL )
    g += get_free_energy_single_resid_pair_and_step( resid, &neigh );
    
  return a + g;

}














STAT get_free_energy_zinc_ion( RESID_LIST *resid )
{

  ATOM_LIST *zinc = resid->my_resid->atom_header;
  ATOM_LIST *atom = zinc->my_atom->con_header;
  
  STAT g = 0.0;
  
  while ( atom != NULL )
    g += get_free_energy_zinc_ion_single_atom_lister_and_step( zinc, &atom );
    
  return g;

}














STAT get_free_energy_zinc_ion_single_atom_lister_and_step( ATOM_LIST *zinc,
  ATOM_LIST **atom )
{
  STAT g = get_free_energy_zinc_ion_single_atom_lister( zinc, *atom );
  *atom = (*atom)->next;
  return g;
}













STAT get_free_energy_zinc_ion_single_atom_lister( ATOM_LIST *zinc, 
  ATOM_LIST *atom )
{

  STAT get_dist_xyz( XYZ *a, XYZ *b );
  
  /*  In ../main/xyz.c:  */
  STAT d = get_dist_xyz( zinc->my_atom->xyz, atom->my_atom->xyz );
  
  if ( ! strcmp( atom->my_atom->atom, "NE2" ))
    return (ZINC_K) * (d - ZINC_NE_DIST) * (d - ZINC_NE_DIST);
  
  if ( ! strcmp( atom->my_atom->atom, "SG" ))  
    return (ZINC_K) * (d - ZINC_SG_DIST) * (d - ZINC_SG_DIST);
    
  error( 3100, "Unrecognized atom type bound to zinc.\n" );

}












STAT get_free_energy_resid_phi_psi_angles( RESID_LIST *resid )
{

  if ( resid->my_resid->type == NULL )
    return 0.0;
    
  STAT phi_g = get_free_energy_angle( resid->my_resid->phi, 
    resid->my_resid->type->phi_coeff,
    DEGREES_TO_RADIANS(resid->my_resid->type->phi_gamma) );
    
  STAT psi_g = get_free_energy_angle( resid->my_resid->psi,
    resid->my_resid->type->psi_coeff,
    DEGREES_TO_RADIANS(resid->my_resid->type->psi_gamma) );
    
  return (W_ANGLE) * (phi_g + psi_g);

}












STAT get_free_energy_angle( STAT angle, STAT coeff[], STAT gamma )
{

  STAT x = 1.0 / (2.0 * PI);
  
  int i;
  
  for ( i = 0; i < CO_BUFFER; i++ ) 
    x += get_prob_angle( angle, coeff[ i ], gamma, i );
    
  return -1.0 * log( x );

}













STAT get_prob_angle( STAT angle, STAT coeff, STAT gamma, int i )
{

  STAT z = (i == 1) ? (-1.0) : (1.0);
  
  return coeff * z * cos( (angle * (i+1.0)) - gamma ) / (2.0 * PI);
  
}








STAT get_free_energy_single_resid_pair_and_step( RESID_LIST *resid,
  RESID_LIST **neigh )
{
  STAT g = get_free_energy_single_resid_pair( resid, *neigh );
  *neigh = (*neigh)->next;
  return g;
}













STAT get_free_energy_single_resid_pair( RESID_LIST *resid, RESID_LIST *neigh )
{

  if ( resid->my_resid->numb < neigh->my_resid->numb )
    return 0.0;
    
  int is_next = ( resid->my_resid->numb == neigh->my_resid->numb + 1) ? (1) : (0);

  STAT g = 0.0;
  
  ATOM_LIST *atom = resid->my_resid->atom_header;
  
  while ( atom != NULL )
    g += get_free_energy_single_atom_lister_and_step( &atom, neigh, is_next );
    
  return g;

}












STAT get_free_energy_single_atom_lister_and_step( ATOM_LIST **atom,
  RESID_LIST *neigh, int is_next )
{

  STAT g = get_free_energy_single_atom_lister( *atom, neigh, is_next );
  
  *atom = (*atom)->next;
  
  return g;

}












STAT get_free_energy_single_atom_lister( ATOM_LIST *atom, RESID_LIST *neigh,
  int is_next )
{

  STAT g = 0.0;

  ATOM_LIST *neigh_atom = neigh->my_resid->atom_header;
  
  while ( neigh_atom != NULL )
    g += get_free_energy_single_atom_pair_and_step( atom, &neigh_atom, is_next );
    
  return g;

}











STAT get_free_energy_single_atom_pair_and_step( ATOM_LIST *atom,
  ATOM_LIST **neigh_atom, int is_next )
{
  STAT g = get_free_energy_single_atom_pair( atom, *neigh_atom, is_next );
  *neigh_atom = (*neigh_atom)->next;
  return g;
}














STAT get_free_energy_single_atom_pair( ATOM_LIST *a, ATOM_LIST *b, int is_next )
{

  STAT get_dist_xyz( XYZ *a, XYZ *b );
  
  if ( atom_in_backbone( a ) && atom_in_backbone( b ) && is_next )
    return 0.0;
    
  /*  In ../main/xyz.c:  */
  STAT d = get_dist_xyz( a->my_atom->xyz, b->my_atom->xyz );
  
  /*  In this file, below:  */
  STAT z = get_lennard_jones( d, a->my_atom->t, b->my_atom->t );
  
  STAT w = ( d > REP_ATTR_DIST ) ? (W_REP) : (W_ATTR);
  
  return w * z;

}













int atoms_are_bonded( ATOM_LIST *a, ATOM_LIST *b )
{

  if (( ! strcmp( a->my_atom->atom, "N" )) && ( ! strcmp( b->my_atom->atom, "O" )))
    return 1;

  if (( ! strcmp( a->my_atom->atom, "O" )) && ( ! strcmp( b->my_atom->atom, "N" )))
    return 1;
    
  if (( ! strcmp( a->my_atom->atom, "C" )) && ( ! strcmp( b->my_atom->atom, "N" )))
    return 1;
    
  if (( ! strcmp( a->my_atom->atom, "N" )) && ( ! strcmp( b->my_atom->atom, "C" )))
    return 1;
    
  if (( ! strcmp( a->my_atom->atom, "O" )) && ( ! strcmp( b->my_atom->atom, "C" )))
    return 1;
    
  if (( ! strcmp( a->my_atom->atom, "C" )) && ( ! strcmp( b->my_atom->atom, "O" )))
    return 1;
    
  return 0;

}












int atom_in_backbone( ATOM_LIST *a )
{

  if ( ! strcmp( a->my_atom->atom, "CA" ))
    return 1;

  if ( ! strcmp( a->my_atom->atom, "C" ))
    return 1;
  
  if ( ! strcmp( a->my_atom->atom, "O" ))
    return 1;

  if ( ! strcmp( a->my_atom->atom, "OXT" ))
    return 1;
  
  if ( ! strcmp( a->my_atom->atom, "HA" ))
    return 1;

  if ( ! strcmp( a->my_atom->atom, "N" ))
    return 1;
    
  return 0;

}














STAT get_lennard_jones( STAT d, TYPE *a, TYPE *b )
{

  STAT e = sqrt( a->e * b->e );
  STAT r = a->r + b->r;
  
  STAT z = r / d;
  
  return e * ( pow( z, 12.0 ) - (2.0 * pow( z, 6.0 )) );
    
}
















