#include "g.h"




















#define INPUT_FNAME "data/sp1/1VA2.txt"





#define MAX_ITERATIONS 10




#define ANGLE 2.0
#define G_MIN_THRESH 1.0
#define ANGLE_BUFFER 2












void do_minimum_free_energy_mol( void )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  
  printf( "Doing molecule minimum free energy...\n\n" );
  
  MOL_LIST *mol_header = NULL, *best_mol = NULL;
  /*  In ../parse/mol_list.c:  */
  mol_header = (MOL_LIST *)get_mol_list_w_fname( mol_header, INPUT_FNAME );
  
  /*  In this file, below:  */
  best_mol = (MOL_LIST *)do_my_minimum_free_energy_mol( best_mol, mol_header );

}














MOL_LIST *do_my_minimum_free_energy_mol( MOL_LIST *best_mol, MOL_LIST *mol_header )
{

  TYPE_CELL *make_and_set_type_cell( TYPE_CELL *my_type );
  RTYPE_CELL *make_and_set_rtype_cell( RTYPE_CELL *rr );
  MOL_LIST *copy_mol_lister( MOL_LIST *major, MOL_LIST *minor );
  
  TYPE_CELL *my_type = NULL;
  RTYPE_CELL *my_rtype = NULL;
  MOL_LIST *saved_mol = NULL;

  /*  In atype.c:  */
  my_type = (TYPE_CELL *)make_and_set_type_cell( my_type );
  my_rtype = (RTYPE_CELL *)make_and_set_rtype_cell( my_rtype );
  
  /*  In ../mm/mol.c:  */
  saved_mol = (MOL_LIST *)copy_mol_lister( saved_mol, mol_header );
  
  /*  In this file, below:  */
  best_mol = (MOL_LIST *)do_min_free_energy_mol_w_type_cell( best_mol,
    mol_header, my_type, my_rtype );
    
  kill_type_cell( my_type );
  
  return best_mol;

}














MOL_LIST *do_min_free_energy_mol_w_type_cell( MOL_LIST *best_mol,
  MOL_LIST *mol_header, TYPE_CELL *my_type, RTYPE_CELL *my_rtype )
{

  STAT get_free_energy_mol_w_two_type_cells( MOL_LIST *mol_header, TYPE_CELL *my_type,
    RTYPE_CELL *my_rtype );

  /*  In free_energy.c:  */
  STAT g = get_free_energy_mol_w_two_type_cells( mol_header, my_type, my_rtype );
  
  printf( "GG = %.3f\n", g );
    
  /*  In this file, below:  */  
  return get_min_free_energy_mol( best_mol, mol_header, my_type, my_rtype );

}
















/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/













MOL_LIST *get_min_free_energy_mol( MOL_LIST *best_mol, MOL_LIST *mol, 
  TYPE_CELL *my_type, RTYPE_CELL *my_rtype )
{

  STAT g = 100000000.0;

  int iter = 0, converged = 0;
  
  while (( iter++ < MAX_ITERATIONS ) && ( ! converged ))
    best_mol = (MOL_LIST *)get_min_free_energy_single_iter( best_mol, mol, my_type, 
      my_rtype, &g, &converged, iter );

  return best_mol;

}











MOL_LIST *get_min_free_energy_single_iter( MOL_LIST *best_mol, MOL_LIST *mol,
  TYPE_CELL *my_type, RTYPE_CELL *my_rtype, STAT *g, int *converged, int iter )
{

  STAT get_free_energy_mol_w_two_type_cells( MOL_LIST *mol_header, TYPE_CELL *my_type,
    RTYPE_CELL *my_rtype );

  /*  In this file, below:  */
  set_resid_list_use_vals( mol, mol->my_mol->resid_header );
  
  printf( "Iteration :  %d\n", iter );

  /*  In this file, below:  */
  get_min_free_energy_iteration_use_resid( mol, my_type );
  
  /*  In free_energy.c:  */
  STAT gg = get_free_energy_mol_w_two_type_cells( mol, my_type, my_rtype );
  
  printf( "GG = %.3f\n", gg );

    
  return best_mol;


}













void set_resid_list_use_vals( MOL_LIST *mol, RESID_LIST *resid_header )
{

  clear_resid_list_use_vals( resid_header );
  
  set_my_resid_list_use_vals( mol, resid_header );

}











void clear_resid_list_use_vals( RESID_LIST *resid_header )
{

  RESID_LIST *resid = resid_header;
  
  while ( resid != NULL ) {
    resid->my_resid->use = 0;
    resid = resid->next;
  }

}











void set_my_resid_list_use_vals( MOL_LIST *mol, RESID_LIST *resid_header )
{
  
  RESID_LIST *resid = 
    resid_header->next->next->next->next->next->next;
  
  /*
  RESID_LIST *resid = 
    resid_header->next->next;
  */
  while ( resid->next != NULL )
    set_single_resid_lister_use_val_and_step( mol, &resid );

}












void set_single_resid_lister_use_val_and_step( MOL_LIST *mol, RESID_LIST **resid )
{
  set_single_resid_lister_use_val( mol, *resid );
  *resid = (*resid)->next;
}












void set_single_resid_lister_use_val( MOL_LIST *mol, RESID_LIST *resid )
{

  if ( resid->my_resid->mol_type != MOL_TYPE_AA )
    return;
  
  if ( use_single_resid_lister_phi( mol, resid ))
    resid->my_resid->use = 1;

  else if ( use_single_resid_lister_psi( mol, resid ))
    resid->my_resid->use = 1;
    
}













int use_single_resid_lister_phi( MOL_LIST *mol, RESID_LIST *resid )
{

  STAT get_free_energy_mol_list( MOL_LIST *mol_header );
  void rotate_phi_angle( MOL_LIST *mol, RESID_LIST *resid, STAT angle );


  STAT array[ ANGLE_BUFFER ] = { -1.0 * ANGLE, ANGLE };
  
  int i;
  
  for ( i = 0; i < ANGLE_BUFFER; i++ ) {
  
    /*  In phi_psi.c:  */
    rotate_phi_angle( mol, resid, array[ i ] );
    
    /*  In free_energy.c:  */    
    STAT g = get_free_energy_mol_list( mol );
    
    /*  In phi_psi.c:  */
    rotate_phi_angle( mol, resid, -1.0 * array[ i ] );
    
    if ( g < G_MIN_THRESH ) 
      return 1;
    
  }
  
  return 0;

}













int use_single_resid_lister_psi( MOL_LIST *mol, RESID_LIST *resid )
{

  STAT get_free_energy_mol_list( MOL_LIST *mol_header );
  void rotate_psi_angle( MOL_LIST *mol, RESID_LIST *resid, STAT angle );


  STAT array[ ANGLE_BUFFER ] = { -1.0 * ANGLE, ANGLE };
  
  int i;
  
  for ( i = 0; i < ANGLE_BUFFER; i++ ) {
  
    /*  In phi_psi.c:  */
    rotate_psi_angle( mol, resid, array[ i ] );
    
    /*  In free_energy.c:  */    
    STAT g = get_free_energy_mol_list( mol );
    
    /*  In phi_psi.c:  */
    rotate_psi_angle( mol, resid, -1.0 * array[ i ] );
    
    if ( g < G_MIN_THRESH ) 
      return 1;
    
  }
  
  return 0;

}















void get_min_free_energy_iteration_use_resid( MOL_LIST *mol, 
  TYPE_CELL *my_type )
{

  MATRIX *create_matrix( MATRIX *xx, int ii, int jj );
  
  MX_STRUCT my_mx = { .p = NULL, .f = NULL, .j = NULL, .b = NULL, .d = NULL,
    .z = NULL };
    
  int nn = 2 * get_numb_of_used_resids( mol->my_mol->resid_header );
  
  my_mx.p = (MATRIX *)create_matrix( my_mx.p, nn, 1 );
  my_mx.f = (MATRIX *)create_matrix( my_mx.f, nn, 1 );
  my_mx.j = (MATRIX *)create_matrix( my_mx.j, nn, nn );
  my_mx.b = (MATRIX *)create_matrix( my_mx.b, nn, nn );
  my_mx.d = (MATRIX *)create_matrix( my_mx.d, nn, 1 );
  my_mx.z = (MATRIX *)create_matrix( my_mx.z, nn, 1 );
  
  get_min_free_energy_new_mol_iteration( mol, &my_mx, my_type );
  
  kill_matrix( my_mx.p );
  kill_matrix( my_mx.f );
  kill_matrix( my_mx.j );
  kill_matrix( my_mx.b );
  kill_matrix( my_mx.d );
  kill_matrix( my_mx.z );

}











int get_numb_of_used_resids( RESID_LIST *resid_header )
{

  int n = 0;
  
  RESID_LIST *resid = resid_header;
  
  while ( resid != NULL ) {
  
    if ( resid->my_resid->use )
      ++n;
      
    resid = resid->next;
  
  }
  
  return n;

}










void get_min_free_energy_new_mol_iteration( MOL_LIST *mol, MX_STRUCT *my_mx,
  TYPE_CELL *my_type )
{

  do_min_free_energy_newton( mol, my_mx, my_type );
  
  update_mol_min_free_energy( mol, my_mx );

}














void do_min_free_energy_newton( MOL_LIST *mol, MX_STRUCT *my_mx, 
  TYPE_CELL *my_type )
{

  get_step_mx_min_free_energy_newton( mol, my_mx, my_type );

}












void update_mol_min_free_energy( MOL_LIST *mol, MX_STRUCT *my_mx )
{

  int i = 0;

  RESID_LIST *resid = mol->my_mol->resid_header, *prev = NULL;
  
  while ( resid != NULL ) {
  
    if ( resid->my_resid->use ) 
      update_resid_lister_min_free_energy( mol, resid, prev, my_mx, i++ );
    
    prev = resid;
    resid = resid->next;
  
  }

}












void update_resid_lister_min_free_energy( MOL_LIST *mol, RESID_LIST *resid, 
  RESID_LIST *prev, MX_STRUCT *my_mx, int i )
{

  void rotate_phi_angle( MOL_LIST *mol, RESID_LIST *resid, STAT angle );
  void rotate_psi_angle( MOL_LIST *mol, RESID_LIST *resid, STAT angle );

  MATRIX *d = my_mx->d;
  
  STAT phi = *(d->x + (i * d->j));
  STAT psi = *(d->x + (i * d->j) + 1);
  
  /*  In phi_psi.c:  */
  rotate_phi_angle( mol, resid, phi );
  rotate_psi_angle( mol, resid, psi );  
  
  /*  In phi_psi.c:  */
  set_phi_psi_angles_single_resid_lister( resid, prev );
  
}











void get_step_mx_min_free_energy_newton( MOL_LIST *mol, MX_STRUCT *my_mx,
  TYPE_CELL *my_type )
{

  void multiply_scalar_matrix( MATRIX *a, STAT x );
  
  /*  In this file, below:  */
  set_f_matrix_free_energy_newton( my_mx, mol, my_type );
  
  /*  In this file, below:   */
  set_j_matrix_free_energy_newton( my_mx, mol, my_type );

  /*  In matrix.c:  */
  get_matrix_inv( my_mx->b, my_mx->j );
  
  /*  In matrix.c:  */
  multiply_scalar_matrix( my_mx->f, -1.0 );
  
  /*  In matrix.c:  */
  multiply_matrices( my_mx->d, my_mx->b, my_mx->f );
  
  print_matrix( my_mx->b );
  print_matrix( my_mx->d );
  
}












void set_f_matrix_free_energy_newton( MX_STRUCT *my_mx, MOL_LIST *mol,
  TYPE_CELL *my_type )
{

  MATRIX *f = my_mx->f;
  
  RESID_LIST *resid = (RESID_LIST *)first_used_resid( mol->my_mol->resid_header );
  
  STAT *phi = f->x;
  STAT *psi = f->x + 1;
  
  int end = f->i / 2;
  
  register int j = 0;
  
  while ( j++ < end ) {
  
    *phi = get_f_matrix_elem_phi( mol, resid, my_type );
    *psi = get_f_matrix_elem_psi( mol, resid, my_type );
    
    step_aa_resid_lister( &resid );    
    
    phi += 2;
    psi += 2;
    
  }
  
  print_matrix( f );

}










RESID_LIST *first_used_resid( RESID_LIST *resid_header )
{

  RESID_LIST *resid = resid_header;
  
  while ( resid != NULL ) {

    if ( resid->my_resid->use )
      return resid;

    resid = resid->next;

  }
  
  error( 3200, "No residues have true used value.\n" );
  
  return NULL;

}









void step_aa_resid_lister( RESID_LIST **resid )
{

  *resid = (*resid)->next;
  
  while (( *resid != NULL ) && ( ! (*resid)->my_resid->use ))
    *resid = (*resid)->next;

}











STAT get_f_matrix_elem_phi( MOL_LIST *mol, RESID_LIST *resid, TYPE_CELL *my_type )
{

  STAT get_numerical_deriv_one_angle( MOL_LIST *mol, RESID_LIST *resid,
    void (*func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh ) );
  void rotate_phi_angle( MOL_LIST *mol, RESID_LIST *resid, STAT angle );

  void (*func)( MOL_LIST *mm, RESID_LIST *rr, STAT aa );
  
  func = rotate_phi_angle;

  /*  In der.c:  */
  return get_numerical_deriv_one_angle( mol, resid, func );
  
}











STAT get_f_matrix_elem_psi( MOL_LIST *mol, RESID_LIST *resid, TYPE_CELL *my_type )
{

  STAT get_numerical_deriv_one_angle( MOL_LIST *mol, RESID_LIST *resid,
    void (*func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh ) );
  void rotate_psi_angle( MOL_LIST *mol, RESID_LIST *resid, STAT angle );

  void (*func)( MOL_LIST *mm, RESID_LIST *rr, STAT aa );
  
  func = rotate_psi_angle;

  /*  In der.c:  */
  return get_numerical_deriv_one_angle( mol, resid, func );
  
}












void set_j_matrix_free_energy_newton( MX_STRUCT *my_mx, MOL_LIST *mol, 
  TYPE_CELL *my_type )
{

  MATRIX *j = my_mx->j;
  
  /*  In this file, above:  */
  RESID_LIST *resid_a = (RESID_LIST *)first_used_resid( mol->my_mol->resid_header );
  
  int a, b, end = j->i / 2;
  
  for ( a = 0; a < end; a++ ) {
  
    /*  In this file, above:  */
    RESID_LIST *resid_b = 
      (RESID_LIST *)first_used_resid( mol->my_mol->resid_header );
  
    for ( b = a; b < end; b++ ) {
    
      STAT *f = j->x + (2 * a * j->j) + (2 * b);
      STAT *g = j->x + (2 * a * j->j) + (2 * b) + 1;
      STAT *h = j->x + (((2 * a) + 1) * j->j) + (2 * b);
      STAT *k = j->x + (((2 * a) + 1) * j->j) + (2 * b) + 1;
    
      STAT *m = j->x + (2 * b * j->j) + (2 * a);
      STAT *n = j->x + (2 * b * j->j) + (2 * a) + 1;
      STAT *p = j->x + (((2 * b) + 1) * j->j) + (2 * a);
      STAT *q = j->x + (((2 * b) + 1) * j->j) + (2 * a) + 1;
    
      /*  In this file, below:  */
      *f = get_j_elem_phi_phi( mol, resid_a, resid_b );
      *g = get_j_elem_phi_psi( mol, resid_a, resid_b );
      *h = get_j_elem_psi_phi( mol, resid_a, resid_b );
      *k = get_j_elem_psi_psi( mol, resid_a, resid_b );
      
      *m = *f;
      *n = *h;
      *p = *g;
      *q = *k;
      
      printf( "\t\t\tJ (%d,%d)     %.5f  %.5f  %.5f  %.5f\n", a, b, *f, *g, *h, *k );
      
      step_aa_resid_lister( &resid_b );
    
    }
    
    step_aa_resid_lister( &resid_a );
  
  }  
  
  print_matrix( j );
  
}











STAT get_j_elem_phi_phi( MOL_LIST *mol, RESID_LIST *resid_a, RESID_LIST *resid_b )
{

  void rotate_phi_angle( MOL_LIST *mol, RESID_LIST *resid, STAT angle );

  void (*a_func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh );
  void (*b_func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh );
  
  /*  In phi_psi.c:  */  
  a_func = rotate_phi_angle;
  b_func = rotate_phi_angle;
  
  /*  In this file, below:  */
  return get_j_elem( mol, resid_a, resid_b, a_func, b_func );

}













STAT get_j_elem_phi_psi( MOL_LIST *mol, RESID_LIST *resid_a, RESID_LIST *resid_b )
{

  void rotate_phi_angle( MOL_LIST *mol, RESID_LIST *resid, STAT angle );
  void rotate_psi_angle( MOL_LIST *mol, RESID_LIST *resid, STAT angle );

  void (*a_func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh );
  void (*b_func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh );
  
  /*  In phi_psi.c:  */  
  a_func = rotate_phi_angle;
  b_func = rotate_psi_angle;
  
  /*  In this file, below:  */
  return get_j_elem( mol, resid_a, resid_b, a_func, b_func );

}













STAT get_j_elem_psi_phi( MOL_LIST *mol, RESID_LIST *resid_a, RESID_LIST *resid_b )
{

  void rotate_phi_angle( MOL_LIST *mol, RESID_LIST *resid, STAT angle );
  void rotate_psi_angle( MOL_LIST *mol, RESID_LIST *resid, STAT angle );

  void (*a_func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh );
  void (*b_func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh );
  
  /*  In phi_psi.c:  */  
  a_func = rotate_psi_angle;
  b_func = rotate_phi_angle;
  
  /*  In this file, below:  */
  return get_j_elem( mol, resid_a, resid_b, a_func, b_func );

}













STAT get_j_elem_psi_psi( MOL_LIST *mol, RESID_LIST *resid_a, RESID_LIST *resid_b )
{

  void rotate_psi_angle( MOL_LIST *mol, RESID_LIST *resid, STAT angle );

  void (*a_func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh );
  void (*b_func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh );
  
  /*  In phi_psi.c:  */  
  a_func = rotate_psi_angle;
  b_func = rotate_psi_angle;
  
  /*  In this file, below:  */
  return get_j_elem( mol, resid_a, resid_b, a_func, b_func );

}














STAT get_j_elem( MOL_LIST *mol, RESID_LIST *resid_a, RESID_LIST *resid_b,
  void (*a_func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh ),
  void (*b_func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh ))
{

  STAT get_double_deriv_angles( MOL_LIST *mol, RESID_LIST *resid_a, 
    RESID_LIST *resid_b,
    void (*a_func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh ),
    void (*b_func)( MOL_LIST *mm, RESID_LIST *rr, STAT hh ) );

  /*  In der.c:  */
  return get_double_deriv_angles( mol, resid_a, resid_b, a_func, b_func );

}


















