#include "pep_params.h"















#define DATA_FILE_FNAME "data/c2h2/data_files.txt"
#define DIR_NAME "data/c2h2/raw_data"
#define OUTPUT_FNAME "data/c2h2/params/aa_phi_psi.txt"

#define OUTPUT_BIN_FNAME "data/c2h2/params/aa_phi_psi_bins.txt"



#define RADIANS_TO_DEGREES(a) (180.0 * (a) / PI)
#define DEGREES_TO_RADIANS(a) (PI * (a) / 180.0)

#define GAMMA_STEP (PI / 6.0)

#define COEFF_BUFFER 3
#define WIND_RADIUS (DEGREES_TO_RADIANS(30.0))

#define MAX_ITERATIONS 25
#define H_VAL 0.1
#define MATRIX_BUFFER 20
#define THRESHOLD 5.0e-3
#define LHOOD_THRESH 0.05
#define ABS(a) (((a) > 0) ? (a) : (-1.0 * (a)))

#define BIN_SIZE 10
#define NUMB_OF_BINS 36
#define LOWEST_BIN -180








void do_peptide_params( void )
{

  MOL_LIST *header = NULL;
  header = (MOL_LIST *)get_sep_resid_mol_list( header, DATA_FILE_FNAME, DIR_NAME );
  
  print_sep_resid_list_phi_psi_angles( header, OUTPUT_FNAME );

}













MOL_LIST *get_sep_resid_mol_list( MOL_LIST *header, char fname[], char dir[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( fname, "" );
  
  header = (MOL_LIST *)get_sep_resid_mol_list_fh( header, dir, in );
  
  close_file( in );
  
  return header;

}













MOL_LIST *get_sep_resid_mol_list_fh( MOL_LIST *header, char dir[], FILE *in )
{

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, in ) != NULL )
    header = (MOL_LIST *)get_sep_resid_mol_list_single_file( header, line, dir );
    
  return header;

}











MOL_LIST *get_sep_resid_mol_list_single_file( MOL_LIST *header, char line[],
  char dir[] )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  
  char fname[ BUFFER ], key[ BUFFER ];  
  int numb;
  
  MOL_LIST *mol_header = NULL;

  sscanf( line, "%d%s", &numb, key );
  sprintf( fname, "%s/%s.pdb", dir, key );
  
  /*  In ../parse/mol_list.c:  */
  mol_header = (MOL_LIST *)get_mol_list_w_fname( mol_header, fname );
  
  printf( "%d :  %s\n", numb, key );
  
  /*  In phi_psi.c:  */
  set_phi_psi_angles_w_mol_list( mol_header );
  
  /*  In this file, below:  */
  header = (MOL_LIST *)add_residues_to_sep_resid_mol_list( header, mol_header );
  
  kill_mol_list( mol_header );
  
  return header;

}














MOL_LIST *add_residues_to_sep_resid_mol_list( MOL_LIST *header, 
  MOL_LIST *mol_header )
{

  MOL_LIST *mol = mol_header;
  
  while ( mol != NULL )
    header = (MOL_LIST *)add_sep_aa_residues_single_mol_lister_and_step( header, 
      &mol );
    
  return header;

}











MOL_LIST *add_sep_aa_residues_single_mol_lister_and_step( MOL_LIST *header,
  MOL_LIST **mol )
{

  header = (MOL_LIST *)add_sep_aa_residues_single_mol_lister( header, *mol );
  
  *mol = (*mol)->next;
  
  return header;

}













MOL_LIST *add_sep_aa_residues_single_mol_lister( MOL_LIST *header, MOL_LIST *mol )
{

  if ( mol->my_mol->resid_header->my_resid->mol_type != MOL_TYPE_AA )
    return header;

  return add_sep_aa_residues_my_single_mol_lister( header, mol );
  
}











MOL_LIST *add_sep_aa_residues_my_single_mol_lister( MOL_LIST *header, 
  MOL_LIST *mol )
{

  RESID_LIST *resid = mol->my_mol->resid_header;
  
  while ( resid != NULL )
    header = (MOL_LIST *)add_sep_aa_single_residue_lister_and_step( header,
      &resid );
      
  return header;

}












MOL_LIST *add_sep_aa_single_residue_lister_and_step( MOL_LIST *header,
  RESID_LIST **resid )
{

  header = (MOL_LIST *)add_sep_aa_single_residue_lister( header, *resid );
  
  *resid = (*resid)->next;
  
  return header;

}













MOL_LIST *add_sep_aa_single_residue_lister( MOL_LIST *header, RESID_LIST *resid )
{

  MOL_LIST *x = NULL;
  x = (MOL_LIST *)get_mol_lister_residue_label( header, resid->my_resid->resid );
  
  if ( x == NULL )
    return create_sep_aa_single_residue_lister( header, resid );

  return add_my_sep_aa_single_residue_lister( header, x, resid );

}













MOL_LIST *get_mol_lister_residue_label( MOL_LIST *header, char label[] )
{

  MOL_LIST *curr = header;
  
  while ( curr != NULL ) {
  
    if ( ! strcmp( curr->my_mol->mol, label ))
      return curr;
      
    curr = curr->next;
  
  }
  
  return NULL;

}














MOL_LIST *create_sep_aa_single_residue_lister( MOL_LIST *header, RESID_LIST *resid )
{

  MOL_LIST *create_mol_lister( MOL_LIST *xx );
  RESID_LIST *create_resid_lister( RESID_LIST *rr );
  
  MOL_LIST *new = NULL;
  RESID_LIST *r = NULL;
  
  new = (MOL_LIST *)create_mol_lister( new );
  r = (RESID_LIST *)create_resid_lister( r );
  
  strcpy( new->my_mol->mol, resid->my_resid->resid );
  new->my_mol->resid_header = r;
  
  copy_sep_aa_resid_listers( r, resid );
  
  if ( header == NULL )
    return new;
    
  MOL_LIST *curr = header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = new;
  
  return header;

}













void copy_sep_aa_resid_listers( RESID_LIST *r, RESID_LIST *b )
{
  
  strcpy( r->my_resid->resid, b->my_resid->resid );
  
  r->my_resid->phi = RADIANS_TO_DEGREES(b->my_resid->phi);
  r->my_resid->psi = RADIANS_TO_DEGREES(b->my_resid->psi);

}













MOL_LIST *add_my_sep_aa_single_residue_lister( MOL_LIST *header, MOL_LIST *x, 
  RESID_LIST *resid )
{

  RESID_LIST *create_resid_lister( RESID_LIST *xx );
  
  RESID_LIST *r = NULL;
  r = (RESID_LIST *)create_resid_lister( r );
  
  /*  In this file, above:  */
  copy_sep_aa_resid_listers( r, resid );  
  
  RESID_LIST *curr = x->my_mol->resid_header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = r;
  
  return header;

}













/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/














void print_sep_resid_list_phi_psi_angles( MOL_LIST *header, char fname[] )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( fname, "" );
  
  print_sep_resid_list_phi_psi_angles_fh( header, out );
  
  close_file( out );

}













void print_sep_resid_list_phi_psi_angles_fh( MOL_LIST *header, FILE *out )
{

  /*  In this file, bottom:  */
  get_and_print_aa_phi_psi_angle_params( header, out );
  
  /*  In this file, below:  */
  print_my_sep_resid_list_phi_psi_angles_fh( header, out );
  
}












void print_my_sep_resid_list_phi_psi_angles_fh( MOL_LIST *header, FILE *out )
{

  MOL_LIST *sep = header;
  
  while ( sep != NULL )
    print_resid_list_single_aa_phi_psi_and_step( &sep, out );

}












void print_resid_list_single_aa_phi_psi_and_step( MOL_LIST **sep, FILE *out )
{
  print_resid_list_single_aa_phi_psi( *sep, out );
  *sep = (*sep)->next;
}











void print_resid_list_single_aa_phi_psi( MOL_LIST *sep, FILE *out )
{

  fprintf( out, ">AA: %s\tphi\tpsi\n", sep->my_mol->mol );
  
  print_my_resid_list_single_aa_phi_psi( sep, out );
  
  fprintf( out, "\n" );

}












void print_my_resid_list_single_aa_phi_psi( MOL_LIST *sep, FILE *out )
{

  RESID_LIST *resid = sep->my_mol->resid_header;
  
  while ( resid != NULL )
    print_single_resid_lister_phi_psi_and_step( &resid, out );

}











void print_single_resid_lister_phi_psi_and_step( RESID_LIST **resid, FILE *out )
{
  print_single_resid_lister_phi_psi( *resid, out );
  *resid = (*resid)->next;
}











void print_single_resid_lister_phi_psi( RESID_LIST *resid, FILE *out )
{
  fprintf( out, "%8.3f\t%8.3f\n", resid->my_resid->phi, resid->my_resid->psi );
}













/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/















void get_and_print_aa_phi_psi_angle_params( MOL_LIST *header, FILE *out )
{

  fprintf( out, "#NUMB\t%3s\t%8s\t%8s\t%8s\t%8s\t%8s\t%8s\t%8s\t%s\n",
    "AA", "PHI[1]", "PHI[2]", "PHI[3]", "PSI[1]", "PSI[2]", "PSI[3]",
    "GMA[PHI]", "GMA[PSI]" );
    
  MOL_LIST *sep = header;
  int i = 0;
  
  while ( sep != NULL )
    get_and_print_aa_phi_psi_params_single_aa_and_step( &sep, &i, out );
    
  print_sep( out );
    
}














void get_and_print_aa_phi_psi_params_single_aa_and_step( MOL_LIST **sep, int *i,
  FILE *out )
{
  get_and_print_aa_phi_psi_params_single_aa( *sep, i, out );
  *sep = (*sep)->next;
}











void get_and_print_aa_phi_psi_params_single_aa( MOL_LIST *sep, int *i, FILE *out )
{

  if ( ! strcmp( sep->my_mol->mol, "ZN" ))
    return;

  STAT coeff[ COEFF_BUFFER * 2 ], phi_gamma, psi_gamma;
  
  printf( "Residue :  %s\n", sep->my_mol->mol );
  
  get_aa_phi_psi_params_single_aa( coeff, &phi_gamma, &psi_gamma, sep );
  
  print_aa_phi_psi_params_single_aa( sep->my_mol->mol, coeff, phi_gamma, psi_gamma, 
    ++(*i), out );

}

















void print_aa_phi_psi_params_single_aa( char aa[], STAT coeff[], STAT phi_gamma,
  STAT psi_gamma, int k, FILE *out )
{

  fprintf( out, "%5d\t%s\t", k, aa );
  
  int i;
  
  for ( i = 0; i < COEFF_BUFFER * 2; i++ )
    fprintf( out, "%8.3f\t", coeff[ i ] );
    
  fprintf( out, "%8.3f\t%8.3f\n", get_angle_in_range(phi_gamma), 
    get_angle_in_range(psi_gamma) );
  
  fflush( out );

}












STAT get_angle_in_range( STAT gamma )
{

  STAT a = RADIANS_TO_DEGREES( gamma );
  
  while ( a > 180.0 ) 
    a -= 360.0;
    
  while ( a < -180.0 )
    a += 360.0;
    
  return a;

}










void get_aa_phi_psi_params_single_aa( STAT coeff[], STAT *phi_gamma, 
  STAT *psi_gamma, MOL_LIST *sep )
{

  STAT lhood = -100000000.0;
  
  STAT curr_coeff[ 2 * COEFF_BUFFER ];

  int i, j, k;
  STAT phi = get_max_wind_phi_gamma( sep );
  STAT psi = get_max_wind_psi_gamma( sep );
  
  for ( i = 1; i < COEFF_BUFFER; i++ ) {
  
    for ( j = COEFF_BUFFER + 1; j < 2 * COEFF_BUFFER; j++ ) {
    
      for ( k = 0; k < COEFF_BUFFER * 2; k++ ) {
        if (( k == i ) || ( k == j ))
          curr_coeff[ k ] = 0.0;
        else
          curr_coeff[ k ] = 1.0;
      }
    
      get_aa_phi_psi_params_single_init_value( coeff, curr_coeff, phi_gamma, 
        psi_gamma, phi, psi, sep, &lhood );
      
    }
  
  }

}













STAT get_max_wind_phi_gamma( MOL_LIST *sep )
{

  STAT i, best = -1.0 * PI;
  
  int numb = 0;
  
  for ( i = -1.0 * PI; i < PI; i += GAMMA_STEP )
    get_max_wind_single_window_phi_gamma( sep, &best, &numb, i );

  return best;

}













void get_max_wind_single_window_phi_gamma( MOL_LIST *sep, STAT *best, int *numb,
  STAT angle )
{

  int curr_numb = get_numb_wind_phi_gamma( sep, angle );
  
  if ( curr_numb > *numb ) {
    *numb = curr_numb;
    *best = angle;
  }  

}











int get_numb_wind_phi_gamma( MOL_LIST *sep, STAT angle )
{

  int numb = 0;

  RESID_LIST *resid = sep->my_mol->resid_header;
  
  while ( resid != NULL ) {
  
    STAT phi = resid->my_resid->phi;
    
    if (( phi > angle - WIND_RADIUS ) && ( phi < angle + WIND_RADIUS ))
      ++numb;
      
    resid = resid->next;
  
  }
  
  return numb;

}











STAT get_max_wind_psi_gamma( MOL_LIST *sep )
{

  STAT i, best = -1.0 * PI;
  
  int numb = 0;
  
  for ( i = -1.0 * PI; i < PI; i += GAMMA_STEP )
    get_max_wind_single_window_psi_gamma( sep, &best, &numb, i );

  return best;

}













void get_max_wind_single_window_psi_gamma( MOL_LIST *sep, STAT *best, int *numb,
  STAT angle )
{

  int curr_numb = get_numb_wind_psi_gamma( sep, angle );
  
  if ( curr_numb > *numb ) {
    *numb = curr_numb;
    *best = angle;
  }  

}











int get_numb_wind_psi_gamma( MOL_LIST *sep, STAT angle )
{

  int numb = 0;

  RESID_LIST *resid = sep->my_mol->resid_header;
  
  while ( resid != NULL ) {
  
    STAT phi = resid->my_resid->phi;
    
    if (( phi > angle - WIND_RADIUS ) && ( phi < angle + WIND_RADIUS ))
      ++numb;
      
    resid = resid->next;
  
  }
  
  return numb;

}










void get_aa_phi_psi_params_single_init_value( STAT coeff[], STAT curr_coeff[],
  STAT *phi_gamma, STAT *psi_gamma, STAT init_phi_gamma, STAT init_psi_gamma, 
  MOL_LIST *sep, STAT *lhood )
{

  STAT curr_phi_gamma = init_phi_gamma;
  STAT curr_psi_gamma = init_psi_gamma;
  
//  initialize_curr_phi_psi_params( curr_coeff, &curr_phi_gamma, &curr_psi_gamma,
//    init_phi_gamma, init_psi_gamma );
    
  STAT curr_lhood = get_my_aa_phi_psi_single_init_value( curr_coeff, &curr_phi_gamma, 
    &curr_psi_gamma, sep );
    
  if ( curr_lhood > *lhood )
    update_aa_phi_psi_single_init_val_loop( coeff, phi_gamma, psi_gamma,
      curr_coeff, curr_phi_gamma, curr_psi_gamma, lhood, curr_lhood );
    
}












void initialize_curr_phi_psi_params( STAT curr_coeff[], STAT *curr_phi_gamma,
  STAT *curr_psi_gamma, STAT init_phi_gamma, STAT init_psi_gamma )
{

  int i;

  for ( i = 0; i < COEFF_BUFFER * 2; i++ )
    curr_coeff[ i ] = 1.0;
    
  *curr_phi_gamma = init_phi_gamma;
  *curr_psi_gamma = init_psi_gamma;
  
}










STAT get_my_aa_phi_psi_single_init_value( STAT coeff[], STAT *phi_gamma,
  STAT *psi_gamma, MOL_LIST *sep )
{

  MATRIX *create_matrix( MATRIX *xx, int ii, int jj );
  
  MX_STRUCT my_mx = { .p = NULL, .f = NULL, .j = NULL, .b = NULL, .d = NULL,
    .z = NULL };
    
  int nn = 2 * (COEFF_BUFFER + 1);
  
  my_mx.p = (MATRIX *)create_matrix( my_mx.p, nn, 1 );
  my_mx.f = (MATRIX *)create_matrix( my_mx.f, nn, 1 );
  my_mx.j = (MATRIX *)create_matrix( my_mx.j, nn, nn );
  my_mx.b = (MATRIX *)create_matrix( my_mx.b, nn, nn );
  my_mx.d = (MATRIX *)create_matrix( my_mx.d, nn, 1 );
  my_mx.z = (MATRIX *)create_matrix( my_mx.z, nn, 1 );
  
  STAT lhood = get_aa_phi_psi_newton( &my_mx, coeff, phi_gamma, psi_gamma, sep );
  
  update_aa_phi_psi_params_newton( coeff, phi_gamma, psi_gamma, &my_mx );
  
  print_matrix( my_mx.z );
  
  kill_matrix( my_mx.p );
  kill_matrix( my_mx.f );
  kill_matrix( my_mx.j );
  kill_matrix( my_mx.b );
  kill_matrix( my_mx.d );
  kill_matrix( my_mx.z );
  
  return lhood;

}












STAT get_aa_phi_psi_newton( MX_STRUCT *my_mx, STAT coeff[], STAT *phi_gamma,
  STAT *psi_gamma, MOL_LIST *sep )
{

  initialize_p_matrix_phi_psi( my_mx->p, coeff, *phi_gamma, *psi_gamma );
  
  return get_my_aa_phi_psi_newton( my_mx, sep );
  
}












void initialize_p_matrix_phi_psi( MATRIX *p, STAT coeff[], STAT phi_gamma,
  STAT psi_gamma )
{

  int i;
  
  for ( i = 0; i < COEFF_BUFFER * 2; i++ )
    *(p->x + i) = coeff[ i ];
    
  *(p->x + (COEFF_BUFFER * 2)) = phi_gamma;
  *(p->x + (COEFF_BUFFER * 2) + 1) = psi_gamma;

}












STAT get_my_aa_phi_psi_newton( MX_STRUCT *my_mx, MOL_LIST *sep )
{

  int iter = 0, converged = 0;
  
  STAT lhood = -10000000.0;
  
  while (( iter++ < MAX_ITERATIONS ) && ( ! converged ))
    converged = get_aa_phi_psi_newton_single_iter( my_mx, sep, iter, &lhood );
    
  return lhood;

}













int get_aa_phi_psi_newton_single_iter( MX_STRUCT *my_mx, MOL_LIST *sep, int iter,
  STAT *lhood )
{

  printf( "\t\tIteration :  %d\n", iter );

  get_aa_phi_psi_newton_step_mx( my_mx, sep );
  
  return do_aa_phi_psi_update_mxs( my_mx, sep, lhood );

}













void get_aa_phi_psi_newton_step_mx( MX_STRUCT *my_mx, MOL_LIST *sep )
{

  void multiply_scalar_matrix( MATRIX *a, STAT x );

  /*  In this file, below:  */
  get_f_matrix_aa_phi_psi_newton( my_mx->f, my_mx->p, sep );
  
  /*  In this file, below:  */
  get_j_matrix_aa_phi_psi_newton( my_mx->j, my_mx->p, sep );
  
  /*  In matrix.c:  */
  get_matrix_inv( my_mx->b, my_mx->j );
  
  /*  In matrix.c:  */
  multiply_scalar_matrix( my_mx->f, -1.0 );
  
  /*  In matrix.c:  */
  multiply_matrices( my_mx->d, my_mx->b, my_mx->f );
  
}













void get_f_matrix_aa_phi_psi_newton( MATRIX *f, MATRIX *p, MOL_LIST *sep )
{

  STAT get_phi_psi_single_deriv( MATRIX *p, MOL_LIST *sep, int ii );

  int i;
  
  for ( i = 0; i < f->i; i++ )
    /*  In der.c:  */
    *(f->x + i) = get_phi_psi_single_deriv( p, sep, i );

}

















void get_j_matrix_aa_phi_psi_newton( MATRIX *j, MATRIX *p, MOL_LIST *sep )
{

  STAT get_phi_psi_double_deriv( MATRIX *p, MOL_LIST *sep, int ii, int kk );
  
  int i, k;
  
  for ( i = 0; i < j->i; i++ ) {
  
    for ( k = 0; k < j->j; k++ ) {
    
      *(j->x + (i * j->j) + k) = get_phi_psi_double_deriv( p, sep, i, k );
    
    }
  
  }

}













int do_aa_phi_psi_update_mxs( MX_STRUCT *my_mx, MOL_LIST *sep, STAT *lhood )
{

  MATRIX *create_matrix( MATRIX *mm, int ii, int jj );
  
  MATRIX *my_p = NULL, *my_z = NULL;
  my_p = (MATRIX *)create_matrix( my_p, my_mx->f->i, my_mx->f->j );
  my_z = (MATRIX *)create_matrix( my_z, my_mx->f->i, my_mx->f->j );
  
  /*  In ../mm/matrix.c:  */
  copy_matrix( my_p, my_mx->p );
    
  /*  In ../mm/matrix.c:  */
  add_matrices( my_mx->p, my_p, my_mx->d );
  
  /*  In this file, below:  */
  get_phi_psi_z_matrix( my_z, my_mx->p );
  
  /*  In this file, below:  */
  int converged = phi_psi_matrices_converged( my_mx->z, my_z );
  
  /*  In this file, below:  */
  STAT curr_lhood = get_phi_psi_prob_sep_mol_list_mx( my_mx->p, sep );
  
  printf( "\t\t\t\t\t\tlhood :  %.3f\n", curr_lhood );
  
  if ( ABS( curr_lhood - *lhood ) < LHOOD_THRESH )
    converged = 1;
    
  if ( curr_lhood > *lhood ) {
    /*  In ../mm/matrix.c:  */
    copy_matrix( my_mx->z, my_z );
    *lhood = curr_lhood;
  }
  
  kill_matrix( my_z );
  kill_matrix( my_p );
  
  return converged;

}













void get_phi_psi_z_matrix( MATRIX *z, MATRIX *p )
{

  STAT total_phi = 0.0, total_psi = 0.0;
  
  int i;
  
  for ( i = 0; i < COEFF_BUFFER; i++ ) {
    total_phi += ABS(*(p->x + i));
    total_psi += ABS(*(p->x + COEFF_BUFFER + i));
  }
  
  for ( i = 0; i < COEFF_BUFFER; i++ ) {
    *(z->x + i) = *(p->x + i) / total_phi;
    *(z->x + COEFF_BUFFER + i) = *(p->x + COEFF_BUFFER + i) / total_psi;
  }
  
  *(z->x + (COEFF_BUFFER * 2)) = *(p->x + (COEFF_BUFFER * 2));
  *(z->x + (COEFF_BUFFER * 2) + 1) = *(p->x + (COEFF_BUFFER * 2) + 1);

}












int phi_psi_matrices_converged( MATRIX *z, MATRIX *my_z )
{

  int i, converged = 1;
  
  for ( i = 0; i < z->i; i++ ) {

    if ( ABS( *(z->x + i) - *(my_z->x + i) ) >= THRESHOLD ) {
      converged = 0;
      break;
    }

  }
  
  return converged;

}















void update_aa_phi_psi_params_newton( STAT coeff[], STAT *phi_gamma, 
  STAT *psi_gamma, MX_STRUCT *my_mx )
{

  int i;
  
  for ( i = 0; i < COEFF_BUFFER * 2; i++ )
   coeff[ i ] = *(my_mx->z->x + i);
   
  *phi_gamma = *(my_mx->z->x + (COEFF_BUFFER * 2));
  *psi_gamma = *(my_mx->z->x + (COEFF_BUFFER * 2) + 1);

}














void update_aa_phi_psi_single_init_val_loop( STAT coeff[], STAT *phi_gamma, 
  STAT *psi_gamma, STAT curr_coeff[], STAT curr_phi_gamma, STAT curr_psi_gamma, 
  STAT *lhood, STAT curr_lhood )
{

  int i;
  
  for ( i = 0; i < COEFF_BUFFER * 2; i++ )
    coeff[ i ] = curr_coeff[ i ];
    
  *phi_gamma = curr_phi_gamma;
  *psi_gamma = curr_psi_gamma;
  
  *lhood = curr_lhood;

}













/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/













STAT get_phi_psi_prob_sep_mol_list_mx( MATRIX *p, MOL_LIST *sep )
{

  RESID_LIST *resid = sep->my_mol->resid_header;
  
  STAT x = 0.0;
  
  while ( resid != NULL )
    x += get_phi_psi_prob_single_resid_lister_and_step( &resid, p );
    
  return x;

}














STAT get_phi_psi_prob_single_resid_lister_and_step( RESID_LIST **resid, 
  MATRIX *p )
{
  STAT x = get_phi_psi_prob_single_resid_lister( *resid, p );
  *resid = (*resid)->next;
  return x;
}













STAT get_phi_psi_prob_single_resid_lister( RESID_LIST *resid, MATRIX *p )
{

  STAT phi = resid->my_resid->phi;
  STAT psi = resid->my_resid->psi;
  
  STAT total_a = abs_total_coeff( p->x );
  STAT total_b = abs_total_coeff( p->x + COEFF_BUFFER );
  STAT a = total_a;
  STAT b = total_b;
  
  int i;
  
  for ( i = 0; i < COEFF_BUFFER; i++ ) {
    a += get_cos_angle_prob( p, i, total_a, phi, 0, 0 );   
    b += get_cos_angle_prob( p, i, total_b, psi, COEFF_BUFFER, 1 );   
  }
  
  return log( a / (2.0 * PI * total_a) ) + log( b / (2.0 * PI * total_b) );

}













STAT abs_total_coeff( STAT *a )
{

  STAT x = 0.0, *ptr = a;
  
  char i = 0;
  
  while ( i++ < COEFF_BUFFER ) {
    x += ABS(*ptr);
    ++ptr;
  }
  
  return x;

}











STAT get_cos_angle_prob( MATRIX *p, int i, STAT total, STAT phi_psi, int x, int y )
{

  STAT z = ( i == 1 ) ? (-1.0) : (1.0);
  STAT angle = (phi_psi * (i + 1.0)) - *(p->x + (COEFF_BUFFER *2) + y);
  
  return *(p->x + x + i) * z * cos( angle );

}











STAT get_lhood_phi_psi( MATRIX *z, MOL_LIST *sep )
{

  RESID_LIST *resid = sep->my_mol->resid_header;

  STAT lhood = 0.0;
  
  while ( resid != NULL )
    lhood += get_lhood_phi_psi_single_resid_lister_and_step( &resid, z );
    
  printf( "\t\t\t\tlhood :  %.3f\n", lhood );
    
  return lhood;

}










STAT get_lhood_phi_psi_single_resid_lister_and_step( RESID_LIST **resid,
  MATRIX *z )
{
  STAT lhood = get_lhood_phi_psi_single_resid_lister( *resid, z );
  *resid = (*resid)->next;
  return lhood;
}












STAT get_lhood_phi_psi_single_resid_lister( RESID_LIST *resid, MATRIX *z )
{

  STAT phi = resid->my_resid->phi, psi = resid->my_resid->psi;
  STAT phi_gamma = *(z->x + (COEFF_BUFFER * 2));
  STAT psi_gamma = *(z->x + (COEFF_BUFFER * 2) + 1);
  
  STAT total_a = abs_total_coeff( z->x );
  STAT total_b = abs_total_coeff( z->x + COEFF_BUFFER );

  STAT a = 0.0, b = 0.0;

  int i;
  
  for ( i = 0; i < COEFF_BUFFER; i++ ) {
  
    STAT y = pow( -1.0, i );
  
    a += (total_a + *(z->x + i) * y * cos( (phi * (i+1.0)) - phi_gamma )) 
      / (2.0 * PI * total_a) ;
    
    b += (total_b + *(z->x + COEFF_BUFFER + i) * y * 
      cos( (psi * (i+1.0)) - psi_gamma ))
      / (2.0 * PI * total_b);
  
  }
  
  return log(a) + log(b);

}














/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/


















void do_peptide_phi_psi_bins( void )
{

  MOL_LIST *header = NULL;
  header = (MOL_LIST *)get_sep_resid_mol_list( header, DATA_FILE_FNAME, DIR_NAME );
  
  get_and_put_peptide_phi_psi_bins( header, OUTPUT_BIN_FNAME );

}













void get_and_put_peptide_phi_psi_bins( MOL_LIST *header, char fname[] )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( fname, "" );
  
  get_and_put_peptide_phi_psi_bins_w_fh( header, out );
  
  close_file( out );

}















void get_and_put_peptide_phi_psi_bins_w_fh( MOL_LIST *header, FILE *out )
{

  /*  In phi_psi.c:  */
  set_phi_psi_angles_w_mol_list( header );
  
  MOL_LIST *sep = header;
  
  while ( sep != NULL )
    /*  In this file, below:  */
    get_and_put_peptide_phi_psi_bins_single_mol_and_step( &sep, out );

}












void get_and_put_peptide_phi_psi_bins_single_mol_and_step( MOL_LIST **sep,
  FILE *out )
{
  get_and_put_peptide_phi_psi_bins_single_mol( *sep, out );
  *sep = (*sep)->next;
}












void get_and_put_peptide_phi_psi_bins_single_mol( MOL_LIST *sep, FILE *out )
{

  int phi_array[ NUMB_OF_BINS ];
  int psi_array[ NUMB_OF_BINS ];
  
  initialize_bin_array( phi_array );
  initialize_bin_array( psi_array );
  
  get_and_put_peptide_phi_psi_bins_w_array( sep, phi_array, psi_array, out );

}












void initialize_bin_array( int bin_array[] )
{

  int i;
  
  for ( i = 0; i < NUMB_OF_BINS; i++ )
    bin_array[ i ] = 1;

}













void get_and_put_peptide_phi_psi_bins_w_array( MOL_LIST *sep, int phi_array[], 
  int psi_array[], FILE *out )
{

  get_peptide_phi_psi_bins( sep, phi_array, psi_array );
  
  put_peptide_phi_psi_bins( sep, phi_array, psi_array, out );

}












void get_peptide_phi_psi_bins( MOL_LIST *sep, int phi_array[], int psi_array[] )
{

  RESID_LIST *res = sep->my_mol->resid_header;
  
  if ( ! strcmp( res->my_resid->resid, "ZN" ))
    return;
  
  while ( res != NULL )
    get_peptide_phi_psi_bins_single_resid_and_step( &res, phi_array, psi_array );

}












void get_peptide_phi_psi_bins_single_resid_and_step( RESID_LIST **res,
  int phi_array[], int psi_array[] )
{
  get_peptide_phi_psi_bins_single_resid( *res, phi_array, psi_array );
  *res = (*res)->next;
}












void get_peptide_phi_psi_bins_single_resid( RESID_LIST *res, int phi_array[],
  int psi_array[] )
{

  update_single_phi_psi_array( phi_array, res->my_resid->phi );
  update_single_phi_psi_array( psi_array, res->my_resid->psi );

}












void update_single_phi_psi_array( int array[], STAT angle )
{
  int index = get_phi_psi_bin_index( angle );
  ++(array[ index ]);
}












void put_peptide_phi_psi_bins( MOL_LIST *sep, int phi_array[], int psi_array[], 
  FILE *out )
{

  if ( ! strcmp( sep->my_mol->mol, "ZN" ))
    return;

  fprintf( out, ">%s\n", sep->my_mol->mol );
  printf( ">%s\n", sep->my_mol->mol );
  
  int phi_total = get_phi_psi_array_total( phi_array );
  int psi_total = get_phi_psi_array_total( psi_array );
  
  printf( "\t\ttotal :  %d  %d\n", phi_total, psi_total );
  
  int i, degrees = LOWEST_BIN;
  
  for ( i = 0; i < NUMB_OF_BINS; i++ ) {
  
    fprintf( out, "\t%2d\t%4d\t%4d\t%7.3f\t%4d\t%7.3f\n",
      i,
      degrees,
      phi_array[ i ],
      -1.0 * log( (float)phi_array[ i ] / (float)phi_total ),
      psi_array[ i ],
      -1.0 * log( (float)psi_array[ i ] / (float)psi_total ) );
      
    degrees += BIN_SIZE;
  
  }

}













int get_phi_psi_array_total( int array[] )
{

  int total = 0, i;
  
  for ( i = 0; i < NUMB_OF_BINS; i++ )
    total += array[ i ];
    
  return total;

}









/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/












int get_phi_psi_bin_index( STAT angle )
{
  
  int index = (int)(angle - LOWEST_BIN) / BIN_SIZE;
  
  return (index >= NUMB_OF_BINS) ? (NUMB_OF_BINS - 1) : (index);
  
}













