#include "peptide.h"
















#define INPUT_FNAME "data/sp1/1VA2.txt"
#define DATA_FILE_FNAME "data/c2h2/data_files.txt"
#define DIR_NAME "data/c2h2/raw_data"

#define BIG 1000000000

#define ANGLE 2.0
#define ANGLE_BUFFER 3
#define ANGLE_STEP 0.7
#define ANGLE_THRESH 0.2

#define RADIANS_TO_DEGREES(a) (180.0 * (a) / PI)
















void do_peptide( void )
{

  extern char minor_mode;
  
  switch ( minor_mode ) {
  
  case 'a' :
    /*  In atype.c:  */
    do_atom_type();
    break;
    
  case 'b' :
    /*  In ../parse/data_files.c:  */
    do_raw_data_files();
    break;
    
  case 'c' :
    /*  In this file, below:  */
    do_phi_psi_angles();
    break;
    
  case 'u' :
    /*  In sub.c:  */
    do_amino_acid_substitution();
    break;

  case 'z' :
    /*  In zinc.c:  */
    output_zinc_distances();
    break;
    
  case 'p' :
    /*  In pep_params.c  */
    do_peptide_params();
    break;
    
  case 'P' :
    /*  In pep_params.c:  */
    do_peptide_phi_psi_bins();
    break;
  
  case 'd' :
    /*  In this file, below:  */
    do_free_energy();
    break;
    
  case 'r' :
    /*  In this file, below:  */
    do_rotate_phi_psi_angles();
    break;
    
  case 'R' :
    /*  In this file, below:  */
    do_rotate_phi_psi_and_free_energy();
    break;
    
  case 'g' :
    /*  In this file, below:  */
    do_minimum_free_energy_rotate_mol();
    break;
    
  case 'm' :
    /*  In rmsd.c:  */
    do_rmsd_struct();
    break;
    
  /*  
  case 'g' :
    /*  In this file, below:  
    do_minimum_free_energy_mol();
    break;  
  */
    
  default :
    error( 3000, "Unrecognized minor mode for peptide run.\n" );
    break;
  
  }

}
















/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/














void do_phi_psi_angles( void )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  
  extern char input_file[ BUFFER ];

  MOL_LIST *mol_header = NULL;
  /*  In ../parse/mol_list.c:  */
  mol_header = (MOL_LIST *)get_mol_list_w_fname( mol_header, input_file );
  
  /*  In phi_psi.c:  */
  set_phi_psi_angles_w_mol_list( mol_header );

}














/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/













void do_free_energy( void )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *fh = NULL;
  fh = (FILE *)open_text_file_reading( DATA_FILE_FNAME, "" );
  
  do_free_energy_w_fh( fh );
  
  close_file( fh );
  
}














void do_free_energy_w_fh( FILE *fh )
{

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, fh ) != NULL )
    do_free_energy_single_data_file_line( line );

}












void do_free_energy_single_data_file_line( char line[] )
{

  char key[ BUFFER ], fname[ BUFFER ];
  
  sscanf( line, "%*d%s", key );
  
  printf( "key :  %s\n\n", key );
  
  sprintf( fname, "%s/%s.pdb", DIR_NAME, key );
  
  do_my_free_energy_single_data_file_line( fname );

}


  








void do_my_free_energy_single_data_file_line( char fname[] )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  STAT do_free_energy_mol_list( MOL_LIST *mol_header );
  
  MOL_LIST *mol_header = NULL;
  /*  In ../parse/mol_list.c:  */
  mol_header = (MOL_LIST *)get_mol_list_w_fname( mol_header, fname );
  
  /*  In free_energy.c:  */
  do_free_energy_mol_list( mol_header );  

}
















/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
















void do_rotate_phi_psi_angles( void )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  
  MOL_LIST *mol_header = NULL;
  RESID_LIST *resid = NULL;
  
  /*  In ../parse/mol_list.c:  */
  mol_header = (MOL_LIST *)get_mol_list_w_fname( mol_header, INPUT_FNAME ); 
  
  /*  In this file, below:  */
  resid = (RESID_LIST *)get_resid_lister_w_numb( mol_header, 571 );
  
  /*  In this file, below:  */
  do_rotate_phi_psi_angles_resid_lister( mol_header, resid );

}














RESID_LIST *get_resid_lister_w_numb( MOL_LIST *mol, int numb )
{

  RESID_LIST *resid = mol->my_mol->resid_header;
  
  while ( resid != NULL ) {
  
    if ( resid->my_resid->numb == numb )
      return resid;
      
    resid = resid->next;
  
  }
  
  return NULL;

}















void do_rotate_phi_psi_angles_resid_lister( MOL_LIST *mol, RESID_LIST *resid )
{

  /*  In phi_psi.c:  */
  rotate_phi_angle( mol, resid, ANGLE );
  rotate_psi_angle( mol, resid, ANGLE );

}















/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/














void do_rotate_phi_psi_and_free_energy( void )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  
  MOL_LIST *mol_header = NULL;  
  /*  In ../parse/mol_list.c:  */
  mol_header = (MOL_LIST *)get_mol_list_w_fname( mol_header, INPUT_FNAME ); 
  
  /*  In this file, below:  */
  do_my_rotate_phi_psi_and_free_energy( mol_header );
  
}













void do_my_rotate_phi_psi_and_free_energy( MOL_LIST *mol )
{

  RESID_LIST *resid = mol->my_mol->resid_header;
  
  while ( resid != NULL ) 
    do_rotate_phi_psi_free_energy_single_resid_and_step( mol, &resid );

}














void do_rotate_phi_psi_free_energy_single_resid_and_step( MOL_LIST *mol,
  RESID_LIST **resid )
{
  do_rotate_phi_psi_free_energy_single_resid( mol, *resid );
  *resid = (*resid)->next;
}












void do_rotate_phi_psi_free_energy_single_resid( MOL_LIST *mol, RESID_LIST *resid )
{

  MOL_LIST *copy_mol_lister( MOL_LIST *major, MOL_LIST *minor );

  MOL_LIST *new_mol = NULL;
  /*  In ../mm/mol.c:  */
  new_mol = (MOL_LIST *)copy_mol_lister( new_mol, mol );
  
  printf( "Resid changed :  %s\t\t", resid->my_resid->resid );
  
  /*  In this file, below:  */
  do_rotate_phi_psi_free_energy_new_mol( new_mol, resid );
  
  kill_mol_list( new_mol );

}













void do_rotate_phi_psi_free_energy_new_mol( MOL_LIST *mol, RESID_LIST *resid )
{

  RESID_LIST *my_resid = NULL;
  /*  In this file, above:  */
  my_resid = (RESID_LIST *)get_resid_lister_w_numb( mol, resid->my_resid->numb );
  
  /*  In phi_psi.c:  */
  rotate_phi_angle( mol, my_resid, ANGLE );
  rotate_psi_angle( mol, my_resid, ANGLE );
  
  /*  In this file, below:  */
  do_my_rotate_phi_psi_free_energy_new_mol( mol, resid );
  
}













void do_my_rotate_phi_psi_free_energy_new_mol( MOL_LIST *mol, RESID_LIST *resid )
{
  
  STAT do_free_energy_mol_list( MOL_LIST *mol_header );

  /*  In free_energy.c:  */
  STAT g = do_free_energy_mol_list( mol );   
  
  printf( "GG = %.3f\n", g );

}














/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
















void do_minimum_free_energy_rotate_mol( void )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  
  extern char input_file[ BUFFER ];
  
  MOL_LIST *mol = NULL;  
  /*  In ../parse/mol_list.c:  */
  mol = (MOL_LIST *)get_mol_list_w_fname( mol, input_file ); 
  
  /*  In this file, below:  */
  do_my_minimum_free_energy_rotate_mol( mol );

}













void do_my_minimum_free_energy_rotate_mol( MOL_LIST *mol )
{

  MOL_LIST *copy_mol_lister( MOL_LIST *major, MOL_LIST *minor );

  MOL_LIST *ori_mol = NULL;
  /*  In ../mm/mol.c:  */
  ori_mol = (MOL_LIST *)copy_mol_lister( ori_mol, mol );
  
  /*  In this file, below:  */
  do_minimum_free_energy_rotation( mol );
  
  /*  In phi_psi.c:  */
  set_phi_psi_angles_w_mol_list( ori_mol );
  set_phi_psi_angles_w_mol_list( mol );

  /*  In this file, below:  */
  print_two_mol_lists_diff_phi_psi( mol, ori_mol );
  
}














void do_minimum_free_energy_rotation( MOL_LIST *mol )
{

  STAT angle = ANGLE;
  
  while ( angle > ANGLE_THRESH )
    do_minimum_free_energy_rotate_mol_single_iter( mol, &angle );

}














void do_minimum_free_energy_rotate_mol_single_iter( MOL_LIST *mol, STAT *angle )
{

  printf( "Angle :  %.3f\n", *angle );

  int new = 0;
  int numb = 0;
  
  RESID_LIST *resid = mol->my_mol->resid_header->next->next->next->next;
  RESID_LIST *end = resid;
  RESID_LIST *rr = mol->my_mol->resid_header->next;
  
  while ( resid->next != NULL ) {
    if ( do_min_g_rotate_single_resid_lister_and_step( mol, &resid, *angle, ++numb ))
      new = 1;
  }
  
  while ( strcmp( rr->my_resid->resid, end->my_resid->resid )) {
    if ( do_min_g_rotate_single_resid_lister_and_step( mol, &rr, *angle, ++numb ))
      new = 1;
  }
    
  if ( ! new )
    *angle *= ANGLE_STEP;

}















int do_min_g_rotate_single_resid_lister_and_step( MOL_LIST *mol, 
  RESID_LIST **resid, STAT angle, int numb )
{
  int new = do_min_g_rotate_single_resid_lister( mol, *resid, angle, numb );
  *resid = (*resid)->next;
  return new;
}













int do_min_g_rotate_single_resid_lister( MOL_LIST *mol, RESID_LIST *resid,
  STAT angle, int numb )
{

  if ( resid->my_resid->mol_type != MOL_TYPE_AA )
    return 0;

  STAT angle_array[ ANGLE_BUFFER ] = { angle, 0.0, -1.0 * angle };
  
  STAT g = BIG;
  
  int phi_index, psi_index;
  STAT phi_best, psi_best;
  
  /*  In this file, below:  */
  do_min_g_rotate_single_phi_psi_pair( mol, resid, 0.0, 0.0, &phi_best, 
    &psi_best, &g );
    
  for ( phi_index = 0; phi_index < ANGLE_BUFFER; phi_index++ ) {
  
    for ( psi_index = 0; psi_index < ANGLE_BUFFER; psi_index++ ) {
    
      /*  In this file, below:  */
      do_min_g_rotate_single_phi_psi_pair( mol, resid, angle_array[ phi_index ],
        angle_array[ psi_index ], &phi_best, &psi_best, &g );
    
    }
  
  }
  
  if ( phi_best != 0.0 ) 
    /*  In phi_psi.c:  */
    rotate_phi_angle( mol, resid, phi_best );

  if ( psi_best != 0.0 ) 
    /*  In phi_psi.c:  */
    rotate_psi_angle( mol, resid, psi_best );
   
  if (( phi_best != 0.0 ) || ( psi_best != 0.0 ))
    printf( "\t\t\t\t%d  %s    %.3f  %.3f\n", numb, resid->my_resid->resid,
      phi_best, psi_best );
  
  return (( phi_best == 0.0 ) && ( psi_best == 0.0 )) ? (0) : (1);

}










void do_min_g_rotate_single_phi_psi_pair( MOL_LIST *mol, RESID_LIST *resid,
  STAT phi, STAT psi, STAT *phi_best, STAT *psi_best, STAT *g )
{

  MOL_LIST *copy_mol_lister( MOL_LIST *major, MOL_LIST *minor );

  MOL_LIST *new_mol = NULL;
  /*  In ../mm/mol.c:  */
  new_mol = (MOL_LIST *)copy_mol_lister( new_mol, mol );
  
  /*  In this file, below:  */
  do_my_min_g_rotate_single_phi_psi_pair( new_mol, resid, phi, psi, phi_best,
    psi_best, g );
    
  kill_mol_list( new_mol );
  
}














void do_my_min_g_rotate_single_phi_psi_pair( MOL_LIST *mol, RESID_LIST *resid,
  STAT phi, STAT psi, STAT *phi_best, STAT *psi_best, STAT *g )
{

  STAT do_free_energy_mol_list( MOL_LIST *mol_header );

  RESID_LIST *my_resid = NULL;
  /*  In this file, above:  */
  my_resid = (RESID_LIST *)get_resid_lister_w_numb( mol, resid->my_resid->numb );
  
  /*  In phi_psi.c:  */
  rotate_phi_angle( mol, my_resid, phi );
  rotate_psi_angle( mol, my_resid, psi );
  
  /*  In free_energy.c:  */
  STAT gg = do_free_energy_mol_list( mol );
  
  if ( gg < *g ) {
    *phi_best = phi;
    *psi_best = psi;
    *g = gg;
  }
  
}














void print_two_mol_lists_diff_phi_psi( MOL_LIST *mol_a, MOL_LIST *mol_b )
{

  RESID_LIST *resid_a = mol_a->my_mol->resid_header;
  RESID_LIST *resid_b = mol_b->my_mol->resid_header;
  
  while ( resid_a != NULL )
    print_single_resid_lister_diff_phi_psi_and_step( &resid_a, &resid_b );

}













void print_single_resid_lister_diff_phi_psi_and_step( RESID_LIST **resid_a,
  RESID_LIST **resid_b )
{
  print_single_resid_lister_diff_phi_psi( *resid_a, *resid_b );
  *resid_a = (*resid_a)->next;
  *resid_b = (*resid_b)->next;
}












void print_single_resid_lister_diff_phi_psi( RESID_LIST *resid_a, 
  RESID_LIST *resid_b )
{

  printf( "%s\t\t%.2f  %.2f\t%.2f  %.2f\t\t", resid_a->my_resid->resid,
    resid_a->my_resid->phi,
    resid_a->my_resid->psi,
    resid_b->my_resid->phi,
    resid_b->my_resid->psi );
    
  if (( resid_a->my_resid->phi != resid_b->my_resid->phi ) ||
    ( resid_a->my_resid->psi != resid_b->my_resid->psi ))
  {
    printf( "%.3f  %.3f", 
      RADIANS_TO_DEGREES(resid_a->my_resid->phi - resid_b->my_resid->phi),
      RADIANS_TO_DEGREES(resid_a->my_resid->psi - resid_b->my_resid->psi) );
  }
  
  printf( "\n" );

}











