#include "phi_psi.h" 
















#define RADIANS_TO_DEGREES(a) (180.0 * (a) / PI)
#define DEGREES_TO_RADIANS(a) (PI * (a) / 180.0)
















void set_phi_psi_angles_w_mol_list( MOL_LIST *mol_header ) 
{

  MOL_LIST *mol = mol_header;
  
  while ( mol != NULL )
    set_phi_psi_angles_single_mol_lister_and_step( &mol );

}















void set_phi_psi_angles_single_mol_lister_and_step( MOL_LIST **mol )
{
  set_phi_psi_angles_single_mol_lister( *mol );
  *mol = (*mol)->next;
}













void set_phi_psi_angles_single_mol_lister( MOL_LIST *mol )
{

  RESID_LIST *prev = NULL, *resid = mol->my_mol->resid_header;
  
  while ( resid != NULL )
    set_phi_psi_angles_single_resid_lister_and_step( &resid, &prev );

}














void set_phi_psi_angles_single_resid_lister_and_step( RESID_LIST **resid,
  RESID_LIST **prev )
{
  set_phi_psi_angles_single_resid_lister( *resid, *prev );
  *prev = *resid;
  *resid = (*resid)->next;
}













void set_phi_psi_angles_single_resid_lister( RESID_LIST *resid, RESID_LIST *prev )
{

  if ( resid->my_resid->mol_type != MOL_TYPE_AA )
    return;

  set_phi_angle_single_resid_lister( resid, prev );
  set_psi_angle_single_resid_lister( resid, resid->next );
  
}














void set_phi_angle_single_resid_lister( RESID_LIST *resid, RESID_LIST *prev )
{

  resid->my_resid->phi = get_phi_angle_single_resid_lister( resid, prev );
  
}














STAT get_phi_angle_single_resid_lister( RESID_LIST *resid, RESID_LIST *prev )
{

  if ( prev == NULL )
    return 0.0;

  ATOM_LIST *cc = NULL, *nn = NULL, *ca = NULL, *nc = NULL;

  cc = (ATOM_LIST *)get_atom_lister_resid_atom( prev, "C" );
  nn = (ATOM_LIST *)get_atom_lister_resid_atom( resid, "N" );
  ca = (ATOM_LIST *)get_atom_lister_resid_atom( resid, "CA" );
  nc = (ATOM_LIST *)get_atom_lister_resid_atom( resid, "C" );
  
  if (( cc == NULL ) || ( nn == NULL ) || ( ca == NULL ) || ( nc == NULL ))
    return 0.0;
    
  return calculate_angle_four_atoms( cc, nn, ca, nc );

}

















void set_psi_angle_single_resid_lister( RESID_LIST *resid, RESID_LIST *next )
{
  resid->my_resid->psi = get_psi_angle_single_resid_lister( resid, next );
}












STAT get_psi_angle_single_resid_lister( RESID_LIST *resid, RESID_LIST *next )
{

  if ( next == NULL )
    return 0.0;
    
  ATOM_LIST *nn = NULL, *ca = NULL, *cc = NULL, *zn;

  nn = (ATOM_LIST *)get_atom_lister_resid_atom( resid, "N" );
  ca = (ATOM_LIST *)get_atom_lister_resid_atom( resid, "CA" );
  cc = (ATOM_LIST *)get_atom_lister_resid_atom( resid, "C" );
  zn = (ATOM_LIST *)get_atom_lister_resid_atom( next, "N" );
  
  if (( nn == NULL ) || ( ca == NULL ) || ( cc == NULL ) || ( zn == NULL ))
    return 0.0;
    
  return calculate_angle_four_atoms( nn, ca, cc, zn );

}















ATOM_LIST *get_atom_lister_resid_atom( RESID_LIST *resid, char name[] )
{

  ATOM_LIST *atom = resid->my_resid->atom_header;
  
  while ( atom != NULL ) {
  
    if ( ! strcmp( atom->my_atom->atom, name ))
      return atom;
      
    atom = atom->next;
  
  }
  
  return NULL;

}















STAT calculate_angle_four_atoms( ATOM_LIST *a_atom, ATOM_LIST *b_atom,
  ATOM_LIST *c_atom, ATOM_LIST *d_atom )
{

  return calculate_angle_four_coords( a_atom->my_atom->xyz, b_atom->my_atom->xyz,
    c_atom->my_atom->xyz, d_atom->my_atom->xyz );

}













STAT calculate_angle_four_coords( XYZ *a, XYZ *b, XYZ *c, XYZ *d )
{

  XYZ b_a = get_vector_two_coords( b, a );
  XYZ c_b = get_vector_two_coords( c, b );
  XYZ d_c = get_vector_two_coords( d, c );
  XYZ b_c = get_vector_two_coords( b, c );
    
  XYZ j = get_cross_product( b_a, c_b );
  XYZ k = get_cross_product( d_c, b_c );
  XYZ s = get_cross_product( j, k );
  
  STAT dot_prod = get_dot_product( &j, &k );
  STAT j_length = sqrt( get_dot_product( &j, &j ));
  STAT k_length = sqrt( get_dot_product( &k, &k ));
  
  STAT p = dot_prod / (j_length * k_length);
  
  STAT angle = acos( p );
  
  return ( get_dot_product( &s, &c_b ) > 0.0 ) ? (angle) : (-1.0 * angle);

}














XYZ get_vector_two_coords( XYZ *a, XYZ *b )
{
  
  XYZ c;
  
  c.x = a->x - b->x;
  c.y = a->y - b->y;
  c.z = a->z - b->z;
  
  return c;
  
}













XYZ get_cross_product( XYZ a, XYZ b )
{

  XYZ c;
  
  c.x = (a.y * b.z) - (a.z * b.y);
  c.y = (a.z * b.x) - (a.x * b.z);
  c.z = (a.x * b.y) - (a.y * b.x);
  
  return c;

}












STAT get_dot_product( XYZ *j, XYZ *k )
{
  return (j->x * k->x) + (j->y * k->y) + (j->z * k->z);
}














/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/

















void rotate_phi_angle( MOL_LIST *mol, RESID_LIST *resid, STAT angle )
{
  rotate_phi_angle_radians( mol, resid, DEGREES_TO_RADIANS(angle) );
}














void rotate_phi_angle_radians( MOL_LIST *mol, RESID_LIST *resid, STAT angle )
{

  if ( resid->my_resid->mol_type != MOL_TYPE_AA )
    return;

  ROT_PARAMS my_params;
  
  get_phi_rot_params( &my_params, resid, angle );

  rotate_phi_backbone( &my_params, resid );
  
  rotate_mol_phi( mol, &my_params, resid );

}














void get_phi_rot_params( ROT_PARAMS *my_params, RESID_LIST *resid,
  STAT angle )
{

  ATOM_LIST *ca = NULL, *n = NULL;
  ca = (ATOM_LIST *)get_atom_lister_atom( resid->my_resid->atom_header, "CA" );
  n = (ATOM_LIST *)get_atom_lister_atom( resid->my_resid->atom_header, "N" );
  
  get_rot_params_two_atoms( my_params, ca, n, angle );

}













ATOM_LIST *get_atom_lister_atom( ATOM_LIST *atom_header, char name[] )
{

  ATOM_LIST *atom = atom_header;
  
  while ( atom != NULL ) {
  
    if ( ! strcmp( atom->my_atom->atom, name ))
      return atom;
      
    atom = atom->next;
  
  }
  
  return NULL;

}














void get_rot_params_two_atoms( ROT_PARAMS *my_params, ATOM_LIST *ca, 
  ATOM_LIST *b, STAT angle )
{

  XYZ *xyz = ca->my_atom->xyz;
  XYZ *b_xyz = b->my_atom->xyz;

  my_params->c[ X_INDEX ] = xyz->x;
  my_params->c[ Y_INDEX ] = xyz->y;
  my_params->c[ Z_INDEX ] = xyz->z;
  
  my_params->vector[ X_INDEX ] = b_xyz->x - xyz->x;
  my_params->vector[ Y_INDEX ] = b_xyz->y - xyz->y;
  my_params->vector[ Z_INDEX ] = b_xyz->z - xyz->z;
  
  scale_vec( my_params->vector );
  
  my_params->theta = angle;

}











void scale_vec( STAT v[] )
{

  STAT x = v[ X_INDEX ] * v[ X_INDEX ];
  STAT y = v[ Y_INDEX ] * v[ Y_INDEX ];
  STAT z = v[ Z_INDEX ] * v[ Z_INDEX ];
  
  STAT length = sqrt( x + y + z );
  
  v[ X_INDEX ] /= length;
  v[ Y_INDEX ] /= length;
  v[ Z_INDEX ] /= length;

}








void rotate_phi_backbone( ROT_PARAMS *my_params, RESID_LIST *resid )
{

  ATOM_LIST *atom = resid->my_resid->atom_header;
  
  while ( atom != NULL )
    rotate_atom_phi_backbone_and_step( my_params, &atom );

}















void rotate_atom_phi_backbone_and_step( ROT_PARAMS *my_params, ATOM_LIST **atom )
{
  rotate_atom_phi_backbone( my_params, *atom );
  *atom = (*atom)->next;

}














void rotate_atom_phi_backbone( ROT_PARAMS *my_params, ATOM_LIST *atom )
{

  if ( atom_is_in_phi_backbone( atom ))
    rotate_atom_lister_vector( my_params, atom );

}















int atom_is_in_phi_backbone( ATOM_LIST *atom )
{
  
  if ( ! strcmp( atom->my_atom->atom, "H" ))
    return 1;
    
  if ( ! strcmp( atom->my_atom->atom, "H2" ))
    return 1;
    
  return 0;

}











void rotate_atom_lister_vector( ROT_PARAMS *my_params, ATOM_LIST *atom )
{
  
  translate_atom_xyz_rot_params( my_params, atom, -1.0 );
  
  rotate_my_atom_lister_vector( my_params, atom );
  
  translate_atom_xyz_rot_params( my_params, atom, 1.0 );
  
}














void translate_atom_xyz_rot_params( ROT_PARAMS *my_params, ATOM_LIST *atom,
  STAT coeff )
{

  XYZ *xyz = atom->my_atom->xyz;
  
  xyz->x += coeff * my_params->c[ X_INDEX ];
  xyz->y += coeff * my_params->c[ Y_INDEX ];
  xyz->z += coeff * my_params->c[ Z_INDEX ];

}














void rotate_my_atom_lister_vector( ROT_PARAMS *my_params, ATOM_LIST *atom )
{

  /*  In ../main/xyz.c:  */
  rotate_xyz_vector( my_params, atom->my_atom->xyz );

}













void rotate_mol_phi( MOL_LIST *mol, ROT_PARAMS *my_params, RESID_LIST *resid )
{

  RESID_LIST *r = mol->my_mol->resid_header;
  
  while (( r != NULL ) && ( r->my_resid->numb != resid->my_resid->numb ))       
    rotate_mol_phi_single_resid_lister_and_step( &r, my_params );

}













void rotate_mol_phi_single_resid_lister_and_step( RESID_LIST **r, 
  ROT_PARAMS *my_params )
{
  rotate_mol_phi_single_resid_lister( *r, my_params );
  *r = (*r)->next;
}











void rotate_mol_phi_single_resid_lister( RESID_LIST *r, ROT_PARAMS *my_params )
{

  ATOM_LIST *atom = r->my_resid->atom_header;
  
  while ( atom != NULL )
    rotate_mol_phi_single_atom_lister_and_step( &atom, my_params );

}












void rotate_mol_phi_single_atom_lister_and_step( ATOM_LIST **atom,
  ROT_PARAMS *my_params )
{
  rotate_mol_phi_single_atom_lister( *atom, my_params );
  *atom = (*atom)->next;
}











void rotate_mol_phi_single_atom_lister( ATOM_LIST *atom, ROT_PARAMS *my_params )
{

  /*  In this file, above:  */
  rotate_atom_lister_vector( my_params, atom );

}














void rotate_psi_angle( MOL_LIST *mol, RESID_LIST *resid, STAT angle )
{
  rotate_psi_angle_radians( mol, resid, DEGREES_TO_RADIANS( angle ));
}















void rotate_psi_angle_radians( MOL_LIST *mol, RESID_LIST *resid, STAT angle )
{

  if ( resid->my_resid->mol_type != MOL_TYPE_AA )
    return;

  ROT_PARAMS my_params;
  
  get_psi_rot_params( &my_params, resid, angle );

  rotate_psi_backbone( &my_params, resid );
  
  rotate_mol_psi( mol, &my_params, resid );
  
}














void get_psi_rot_params( ROT_PARAMS *my_params, RESID_LIST *resid, STAT angle )
{

  ATOM_LIST *ca = NULL, *c = NULL;
  /*  In this file, above:  */
  ca = (ATOM_LIST *)get_atom_lister_atom( resid->my_resid->atom_header, "CA" );
  c = (ATOM_LIST *)get_atom_lister_atom( resid->my_resid->atom_header, "C" );
  
  /*  In this file, above:  */
  get_rot_params_two_atoms( my_params, ca, c, angle );

}











void rotate_psi_backbone( ROT_PARAMS *my_params, RESID_LIST *resid )
{

  ATOM_LIST *atom = resid->my_resid->atom_header;
  
  while ( atom != NULL )
    rotate_single_atom_lister_psi_backbone_and_step( &atom, my_params );

}













void rotate_single_atom_lister_psi_backbone_and_step( ATOM_LIST **atom,
  ROT_PARAMS *my_params )
{
  rotate_single_atom_lister_psi_backbone( *atom, my_params );
  *atom = (*atom)->next;
}












void rotate_single_atom_lister_psi_backbone( ATOM_LIST *atom,
  ROT_PARAMS *my_params )
{

  /*  In this file, below:  */
  if ( atom_is_in_psi_backbone( atom ))
    /*  In this file, above:  */
    rotate_atom_lister_vector( my_params, atom );

}












int atom_is_in_psi_backbone( ATOM_LIST *atom )
{

  if ( ! strcmp( atom->my_atom->atom, "O" ))
    return 1;

  if ( ! strcmp( atom->my_atom->atom, "OXT" ))
    return 1;

  if ( ! strcmp( atom->my_atom->atom, "HXT" ))
    return 1;

  return 0;

}














void rotate_mol_psi( MOL_LIST *mol, ROT_PARAMS *my_params, RESID_LIST *resid )
{

  if ( resid == NULL )
    return;

  RESID_LIST *r = resid->next;
  
  while ( r != NULL )
    rotate_mol_psi_single_resid_lister_and_step( &r, my_params );

}












void rotate_mol_psi_single_resid_lister_and_step( RESID_LIST **r,
  ROT_PARAMS *my_params )
{
  rotate_mol_psi_single_resid_lister( *r, my_params );
  *r = (*r)->next;
}












void rotate_mol_psi_single_resid_lister( RESID_LIST *r, ROT_PARAMS *my_params )
{

  ATOM_LIST *atom = r->my_resid->atom_header;
  
  while ( atom != NULL )
    rotate_mol_psi_single_atom_lister_and_step( &atom, my_params );

}












void rotate_mol_psi_single_atom_lister_and_step( ATOM_LIST **atom,
  ROT_PARAMS *my_params )
{
  rotate_mol_psi_single_atom_lister( *atom, my_params );
  *atom = (*atom)->next;
}












void rotate_mol_psi_single_atom_lister( ATOM_LIST *atom, ROT_PARAMS *my_params )
{

  /*  In this file, above:  */
  rotate_atom_lister_vector( my_params, atom );

}











