#include "rmsd.h"
















#define SP_INPUT_FNAME "data/sp1/1VA2.txt"
#define ZF_INPUT_FNAME "data/zif268/pdb1p47.txt"

#define SP_SUBSTR "RSDELQR"
#define ZF_SUBSTR "RSDELTR"

#define ABS(a) (((a) > 1.0) ? (a) : (-1.0 * (a)))


#define BIG 1000000
#define SMALL 1.0e-3
#define MAX_ITERATIONS 15
#define DOT_PROD_THRESH 0.001
#define DIFF_THRESH 0.001





#define SEQUENCE_LENGTH 30









void do_rmsd_struct( void )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  MOL_LIST *get_mol_lister_from_list_label_char( MOL_LIST *header, char label_char );
  
  extern char input_file[ BUFFER ], minor_input_file[ BUFFER ];
  
  MOL_LIST *a = NULL, *b = NULL;  
  /*  In ../parse/mol_list.c:  */
  a = (MOL_LIST *)get_mol_list_w_fname( a, input_file ); 
  b = (MOL_LIST *)get_mol_list_w_fname( b, minor_input_file );

  /*  In this file, below:  */
  float rmsd = get_rmsd_struct( a, b );  
  
  printf( "RMSD :  %.3f\n", rmsd );

}














STAT get_rmsd_struct( MOL_LIST *a, MOL_LIST *b )
{

  XYZ_CELL *create_xyz_cell( XYZ_CELL *xx, int nn );
  
  XYZ_CELL *a_cell = NULL, *b_cell = NULL;
  a_cell = (XYZ_CELL *)create_xyz_cell( a_cell, SEQUENCE_LENGTH );
  b_cell = (XYZ_CELL *)create_xyz_cell( b_cell, SEQUENCE_LENGTH );
  
  STAT rmsd = get_rmsd_struct_w_cells( a, b, a_cell, b_cell );
    
  kill_xyz_cell( a_cell );
  kill_xyz_cell( b_cell );
  
  return rmsd;

}
















STAT get_rmsd_struct_w_cells( MOL_LIST *a, MOL_LIST *b, XYZ_CELL *a_cell,
  XYZ_CELL *b_cell )
{

  STAT get_rmsd_xyz_cell_w_params( ROT_PARAMS *my_params, XYZ_CELL *major_cell,
    XYZ_CELL *minor_cell );
    
  ROT_PARAMS my_params;

  /*  In comp.c:  */
  set_xyz_cell_carbon_alpha_atoms_index( a_cell, a, 0 );
  set_xyz_cell_carbon_alpha_atoms_index( b_cell, b, 0 );
  
  /*  In ../peptide/rmsd.c:  */
  return get_rmsd_xyz_cell_w_params( &my_params, a_cell, b_cell );

}










STAT get_rmsd_mol_substr( MOL_LIST *ma, MOL_LIST *mb, char substr_a[],
  char substr_b[] )
{

  int a_index = get_seq_index_substr( ma, substr_a );
  int b_index = get_seq_index_substr( mb, substr_b );
  
  printf( "%s\n", ma->my_mol->seq + a_index );
  printf( "%s\n", mb->my_mol->seq + b_index );

  return get_rsmd_mol_indices( ma, mb, a_index, a_index + strlen( substr_a ),
    b_index, b_index + strlen( substr_b ) );

}














int get_seq_index_substr( MOL_LIST *m, char substr[] )
{

  int i;
  
  for ( i = 0; i < strlen( m->my_mol->seq ) - strlen( substr ) + 1; i++ ) {
  
    if ( ! strncmp( m->my_mol->seq + i, substr, strlen( substr )))
      return i;
  
  }
  
  return -1;

}















STAT get_rsmd_mol_indices( MOL_LIST *ma, MOL_LIST *mb, int a_start, int a_end,
  int b_start, int b_end )
{

  XYZ_CELL *create_xyz_cell( XYZ_CELL *xx, int nn );
  
  XYZ_CELL *xyz_a = NULL, *xyz_b = NULL;
  xyz_a = (XYZ_CELL *)create_xyz_cell( xyz_a, a_end - a_start + 1 );
  xyz_b = (XYZ_CELL *)create_xyz_cell( xyz_b, b_end - b_start + 1 );

  get_xyz_cell_indices( xyz_a, ma, a_start, a_end );
  get_xyz_cell_indices( xyz_b, mb, b_start, b_end );

  return 0.0;

}













void get_xyz_cell_indices( XYZ_CELL *my_cell, MOL_LIST *mol, int start, int end )
{

  ;

}















/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/













STAT get_rmsd_xyz_cell( XYZ_CELL *major_cell, XYZ_CELL *minor_cell )
{

  ROT_PARAMS my_params;
  
  return get_rmsd_xyz_cell_w_params( &my_params, major_cell, minor_cell );

}













STAT get_rmsd_xyz_cell_w_params( ROT_PARAMS *my_params, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell )
{

  MATRIX *create_matrix( MATRIX *xx, int ii, int jj );
  
  MX_STRUCT my_mx = { .p = NULL, .f = NULL, .j = NULL, .b = NULL, .d = NULL,
    .z = NULL };
  
  int nn = COORD_BUFFER * PARAM_TYPE_BUFFER;
  
  my_mx.p = (MATRIX *)create_matrix( my_mx.p, nn, 1 );
  my_mx.f = (MATRIX *)create_matrix( my_mx.f, nn, 1 );
  my_mx.j = (MATRIX *)create_matrix( my_mx.j, nn, nn );
  my_mx.b = (MATRIX *)create_matrix( my_mx.b, nn, nn );
  my_mx.d = (MATRIX *)create_matrix( my_mx.d, nn, 1 );
  my_mx.z = (MATRIX *)create_matrix( my_mx.z, nn, 1 );
  
  STAT rmsd = get_rmsd_xyz_cell_w_mxs( my_params, &my_mx, major_cell, minor_cell );
  
  kill_matrix( my_mx.p );
  kill_matrix( my_mx.f );
  kill_matrix( my_mx.j );
  kill_matrix( my_mx.b );
  kill_matrix( my_mx.d );
  kill_matrix( my_mx.z );
  
  return rmsd;

}













STAT get_rmsd_xyz_cell_w_mxs( ROT_PARAMS *my_params, MX_STRUCT *my_mx,
  XYZ_CELL *major_cell, XYZ_CELL *minor_cell )
{

  XYZ_CELL *create_xyz_cell( XYZ_CELL *xx, int nn );
  
  XYZ_CELL *my_major = NULL, *my_minor = NULL;
  my_major = (XYZ_CELL *)create_xyz_cell( my_major, major_cell->n );
  my_minor = (XYZ_CELL *)create_xyz_cell( my_minor, minor_cell->n );
  
  /*  In this file, below:  */
  copy_xyz_cell( my_major, major_cell );
  copy_xyz_cell( my_minor, minor_cell );

  /*  In ../mm/rotate.c:  */
  get_and_set_centered_cells( my_params, my_major, my_minor );
  
  /*  In this file, below:  */
  STAT rmsd = get_my_rmsd_xyz_cell_w_mxs( my_params, my_mx, my_major, my_minor );
  
  kill_xyz_cell( my_major );
  kill_xyz_cell( my_minor );
  
  return rmsd;
  
}

  










void copy_xyz_cell( XYZ_CELL *my_cell, XYZ_CELL *cell )
{

  int i;
  
  for ( i = 0; i < cell->n; i++ ) 
    *(my_cell->array + i) = *(cell->array + i);

}












STAT get_my_rmsd_xyz_cell_w_mxs( ROT_PARAMS *my_params, MX_STRUCT *my_mx, 
  XYZ_CELL *major_cell, XYZ_CELL *minor_cell )
{

  STAT rmsd = BIG;
  
  ROT_PARAMS new_params;
  
  int a, b;
  
  for ( a = 0; a < major_cell->n; a++ ) {
  
    for ( b = a + 1; b < major_cell->n; b++ ) {
    
      STAT new_rmsd = get_rmsd_xyz_cell_single_pt_pair( &new_params, my_mx, 
        major_cell, minor_cell, a, b );
        
//      printf( "rmsd :  %.8f\n", new_rmsd );
        
      update_rot_params_rotated_cells( my_params, &new_params, my_mx, new_rmsd, 
        &rmsd );
    
    }
  
  }
  
  return rmsd;

}















void update_rot_params_rotated_cells( ROT_PARAMS *my_params, 
  ROT_PARAMS *new_params, MX_STRUCT *my_mx, STAT new_rmsd, STAT *rmsd )
{

  if ( *rmsd < new_rmsd )
    return;
    
  *rmsd = new_rmsd;
  
  int i;
  
  for ( i = 0; i < COORD_BUFFER; i++ ) {
    my_params->bef_angle[ i ] = new_params->bef_angle[ i ];
    my_params->aft_angle[ i ] = new_params->aft_angle[ i ];
    my_params->angle[ i ] = *(my_mx->z->x + i);
    my_params->c[ i ] = *(my_mx->z->x + i + COORD_BUFFER);
  }
  
}



















STAT get_rmsd_xyz_cell_single_pt_pair( ROT_PARAMS *my_params, MX_STRUCT *my_mx,
  XYZ_CELL *major_cell, XYZ_CELL *minor_cell, int a, int b )
{

  XYZ_CELL *create_xyz_cell( XYZ_CELL *xx, int nn );
  
  XYZ_CELL *my_major = NULL, *my_minor = NULL;
  my_major = (XYZ_CELL *)create_xyz_cell( my_major, major_cell->n );
  my_minor = (XYZ_CELL *)create_xyz_cell( my_minor, minor_cell->n );

  set_rmsd_rot_params_and_cells_pt_pair( my_params, my_major, my_minor,
    major_cell, minor_cell, a, b );
    
  STAT rmsd = get_rmsd_xyz_cell_rotated_cells( my_params, my_mx, my_major,
    my_minor );
    
  kill_xyz_cell( my_major );
  kill_xyz_cell( my_minor );
  
  return rmsd;

}













void set_rmsd_rot_params_and_cells_pt_pair( ROT_PARAMS *my_params,
  XYZ_CELL *my_major, XYZ_CELL *my_minor, XYZ_CELL *major_cell, XYZ_CELL *minor_cell,
  int a, int b )
{

  set_rot_params_after_array( my_params, major_cell, a, b );
  set_rot_params_before_array( my_params, major_cell, minor_cell, a, b );
  
  set_cells_rot_array( my_major, major_cell, my_params->aft_angle );
  set_cells_rot_array( my_minor, minor_cell, my_params->bef_angle );
  
}















void set_rot_params_after_array( ROT_PARAMS *my_params, XYZ_CELL *major_cell,
  int a, int b )
{

  XYZ *w = major_cell->array + a;
  
  int i;
  
  for ( i = 0; i < COORD_BUFFER; i++ )
    my_params->aft_angle[ i ] = 0.0;
    
  my_params->aft_angle[ X_INDEX ] = get_rot_angle_x( w );

  XYZ u = rotate_w_array( w, my_params->aft_angle );
  
  my_params->aft_angle[ Y_INDEX ] = get_rot_angle_y( &u );
  
  XYZ v = rotate_w_array( w, my_params->aft_angle );
  
  if (( v.x > SMALL ) || ( v.y > SMALL )) {
    printf( "x = %7.3f  y = %7.3f\n", v.x, v.y );
    error( 3630, "Rotated point not on z-axis.\n" );
  }
  
}











STAT get_rot_angle_x( XYZ *w )
{

  STAT x = acos( w->z / sqrt((w->z * w->z) + (w->y * w->y)) );
  
  return x * ((w->y > 0.0) ? (1.0) : (-1.0));
  
}










STAT get_rot_angle_y( XYZ *u )
{

  STAT y = acos( u->z / sqrt((u->z * u->z) + (u->x * u->x)) );
  
  return y * ((u->x < 0.0) ? (1.0) : (-1.0));

}










void set_rot_params_before_array( ROT_PARAMS *my_params, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell, int a, int b )
{

  XYZ *t = minor_cell->array + a;
  XYZ *u = major_cell->array + a;
  XYZ *v = minor_cell->array + b;
  XYZ *w = major_cell->array + b;
  
  int i;
  
  for ( i = 0; i < COORD_BUFFER; i++ )
    my_params->bef_angle[ i ] = 0.0;

  my_params->bef_angle[ X_INDEX ] = get_rot_angle_x( t );
  
  XYZ c = rotate_w_array( t, my_params->bef_angle );
  
  my_params->bef_angle[ Y_INDEX ] = get_rot_angle_y( &c );
  
  XYZ d = rotate_w_array( v, my_params->bef_angle );
  XYZ e = rotate_w_array( w, my_params->aft_angle );
  
  my_params->bef_angle[ Z_INDEX ] = get_rot_bef_angle_z( &d, &e );
  
  XYZ r = rotate_w_array( v, my_params->bef_angle );
  XYZ s = rotate_w_array( w, my_params->aft_angle );
  
}










STAT get_rot_bef_angle_z( XYZ *minor_pt, XYZ *major_pt )
{

  XYZ *a = minor_pt, *b = major_pt;

  STAT minor_length = sqrt( (a->x * a->x) + (a->y * a->y) );
  STAT major_length = sqrt( (b->x * b->x) + (b->y * b->y) );

  STAT z = acos( ((a->x * b->x) + (a->y * b->y)) / (minor_length * major_length) );
  
  STAT c = (( a->x * b->y ) > ( a->y * b->x )) ? (1.0) : (-1.0);
  
  return z * c;

}











void set_cells_rot_array( XYZ_CELL *my_cell, XYZ_CELL *cell, STAT array[] )
{

  XYZ *my_ptr = my_cell->array;
  XYZ *ptr = cell->array;

  int i;
  
  for ( i = 0; i < my_cell->n; i++ ) 
    *my_ptr++ = rotate_w_array( ptr++, array );

}












XYZ rotate_w_array( XYZ *a, STAT array[] )
{

  XYZ b;
  
  b.x = rotate_array_x( a, array );
  b.y = rotate_array_y( a, array );
  b.z = rotate_array_z( a, array );
  
  return b;

}












STAT rotate_array_x( XYZ *a, STAT array[] )
{

  STAT t_x = array[ X_INDEX ];
  STAT t_y = array[ Y_INDEX ];
  STAT t_z = array[ Z_INDEX ];

  STAT xx = a->x * cos( t_y ) * cos( t_z );
  
  STAT yy = a->y * 
    (( sin( t_x ) * sin( t_y ) * cos( t_z )) - ( cos( t_x ) * sin( t_z )));
  
  STAT zz = a->z * 
    (( cos( t_x ) * sin( t_y ) * cos( t_z )) + ( sin( t_x ) * sin( t_z )));
    
  return xx + yy + zz;

}












STAT rotate_array_y( XYZ *a, STAT array[] )
{

  STAT t_x = array[ X_INDEX ];
  STAT t_y = array[ Y_INDEX ];
  STAT t_z = array[ Z_INDEX ];

  STAT xx = a->x * cos( t_y ) * sin( t_z );

  STAT yy = a->y *
    (( sin( t_x ) * sin( t_y ) * sin( t_z )) + ( cos( t_x ) * cos( t_z )));

  STAT zz = a->z *
    (( cos( t_x ) * sin( t_y ) * sin( t_z )) - ( sin( t_x ) * cos( t_z )));
    
  return xx + yy + zz;
  
}














STAT rotate_array_z( XYZ *a, STAT array[] )
{

  STAT t_x = array[ X_INDEX ];
  STAT t_y = array[ Y_INDEX ];
  STAT t_z = array[ Z_INDEX ];

  STAT xx = a->x * -1.0 * sin( t_y );
  STAT yy = a->y * sin( t_x ) * cos( t_y );
  STAT zz = a->z * cos( t_x ) * cos( t_y );
  
  return xx + yy + zz;
  
}












/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/














STAT get_rmsd_xyz_cell_rotated_cells( ROT_PARAMS *my_params, MX_STRUCT *my_mx, 
  XYZ_CELL *my_major, XYZ_CELL *my_minor )
{

  initialize_matrix_to_zero( my_mx->p );
  
  STAT rmsd = get_rmsd_rotation_matrix( my_major, my_minor, my_mx->p );
  
  return get_my_rmsd_xyz_cell_rotated_cells( my_params, my_mx, my_major,
    my_minor, rmsd );

}













void initialize_matrix_to_zero( MATRIX *x )
{

  int i = 0, end = x->i * x->j;
  STAT *ptr = x->x;
  
  while ( i++ < end )
    *ptr++ = 0.0;

}













STAT get_my_rmsd_xyz_cell_rotated_cells( ROT_PARAMS *my_params, MX_STRUCT *my_mx, 
  XYZ_CELL *my_major, XYZ_CELL *my_minor, STAT rmsd )
{

  STAT best_rmsd = BIG;

  int iter = 0, converged = 0;
  
  while (( ! converged ) && ( iter++ < MAX_ITERATIONS ))
    converged = get_rot_params_single_iter( my_mx, my_major, my_minor, &best_rmsd,
      iter );

  return best_rmsd;

}















/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/



















int get_rot_params_single_iter( MX_STRUCT *my_mx, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell, STAT *rmsd, int iter )
{

  get_step_matrix_rot_params( my_mx, major_cell, minor_cell );

  return update_rot_params_single_iter( my_mx, major_cell, minor_cell, rmsd );

}












void get_step_matrix_rot_params( MX_STRUCT *my_mx, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell )
{

  void multiply_scalar_matrix( MATRIX *a, STAT x );

  /*  In this file, below:  */
  get_f_matrix_rot_params( my_mx->f, my_mx->p, major_cell, minor_cell );
  
  /*  In this file, below:  */
  get_j_matrix_rot_params( my_mx->j, my_mx->p, major_cell, minor_cell );

  /*  In matrix.c:  */
  get_matrix_inv( my_mx->b, my_mx->j );
  
  /*  In matrix.c:  */
  multiply_scalar_matrix( my_mx->f, -1.0 );
  
  /*  In matrix.c:  */
  multiply_matrices( my_mx->d, my_mx->b, my_mx->f );
  
}














int update_rot_params_single_iter( MX_STRUCT *my_mx, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell, STAT *rmsd )
{

  MATRIX *create_matrix( MATRIX *mm, int ii, int jj );
  void add_matrices( MATRIX *c, MATRIX *a, MATRIX *b );
  void copy_matrix( MATRIX *b, MATRIX *a );
  STAT dot_prod_vector_matrices( MATRIX *a, MATRIX *b );

  MATRIX *p_copy = NULL;
  p_copy = (MATRIX *)create_matrix( p_copy, my_mx->p->i, my_mx->p->j );
  
  /*  In ../mm/matrix.c:  */
  copy_matrix( p_copy, my_mx->p );
  add_matrices( my_mx->p, p_copy, my_mx->d );

  /*  In this file, below:  */
  STAT new_rmsd = get_rmsd_rotation_matrix( major_cell, minor_cell, my_mx->p );
  
  /*  In ../mm/matrix.c:  */
  STAT dot = dot_prod_vector_matrices( my_mx->d, my_mx->d );
  
  kill_matrix( p_copy );
  
  STAT diff = ABS( new_rmsd - *rmsd );
  
  if ( isnan( new_rmsd ) || new_rmsd == 0.0 ) 
    return 1;
  
  if ( new_rmsd < *rmsd ) {
    copy_matrix( my_mx->z, my_mx->p );
    *rmsd = new_rmsd;
  }
  
  if ( dot < DOT_PROD_THRESH )
    return 1;
    
  return 0;

}











void get_f_matrix_rot_params( MATRIX *f, MATRIX *p, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell )
{

  int i;
  
  for ( i = 0; i < f->i; i++ )
    *(f->x + i) = get_f_matrix_elem_rot_params( p, major_cell, minor_cell, i );

}













STAT get_f_matrix_elem_rot_params( MATRIX *p, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell, int i )
{

  STAT get_deriv_rot_params( MATRIX *pp, XYZ_CELL *mm, XYZ_CELL *mi, int ii );
  
  /*  In der.c:  */
  return get_deriv_rot_params( p, major_cell, minor_cell, i );

}













void get_j_matrix_rot_params( MATRIX *j, MATRIX *p, XYZ_CELL *major_cell, 
  XYZ_CELL *minor_cell )
{

  int i, k;
  
  for ( i = 0; i < j->i; i++ ) {
  
    for ( k = 0; k < j->j; k++ ) {
    
      *(j->x + (i * j->j) + k) = get_j_matrix_elem_rot_params( p, major_cell,
        minor_cell, i, k );
    
    }
  
  }

}













STAT get_j_matrix_elem_rot_params( MATRIX *p, XYZ_CELL *major_cell,
  XYZ_CELL *minor_cell, int a, int b )
{

  STAT get_double_deriv_rot_params( MATRIX *pp, XYZ_CELL *mm, XYZ_CELL *mi, int ii,
    int jj );
  
  /*  In der.c:  */
  return get_double_deriv_rot_params( p, major_cell, minor_cell, a, b );

}










/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/














STAT get_rmsd_rotation_matrix( XYZ_CELL *major_cell, XYZ_CELL *minor_cell, 
  MATRIX *p )
{

  XYZ *major_pt = major_cell->array;
  XYZ *minor_pt = minor_cell->array;
  
  STAT rmsd = 0.0;
  
  int i = 0;
  
  while ( i++ < major_cell->n )
    rmsd += get_rmsd_single_term_rotation( major_pt++, minor_pt++, p );
    
  return sqrt( rmsd / (STAT)major_cell->n );
  
}











STAT get_rmsd_single_term_rotation( XYZ *major_pt, XYZ *minor_pt, MATRIX *p )
{

  STAT rotate( XYZ *point, MATRIX *p, char index );
  
  /*  In ../mm/rotate.c:  */
  STAT x = major_pt->x - rotate( minor_pt, p, X_INDEX );
  STAT y = major_pt->y - rotate( minor_pt, p, Y_INDEX );
  STAT z = major_pt->z - rotate( minor_pt, p, Z_INDEX );
  
  return (x * x) + (y * y) + (z * z);

}
















/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/
/*=============================================================================*/














void rotate_mol_lister_w_rot_params( MOL_LIST *mol, ROT_PARAMS *my_params )
{

  RESID_LIST *resid = mol->my_mol->resid_header;
  
  while ( resid != NULL )
    rotate_resid_lister_w_rot_params_and_step( &resid, my_params );

}












void rotate_resid_lister_w_rot_params_and_step( RESID_LIST **resid, 
  ROT_PARAMS *my_params )
{
  rotate_resid_lister_w_rot_params( *resid, my_params );
  *resid = (*resid)->next;
}










void rotate_resid_lister_w_rot_params( RESID_LIST *resid, ROT_PARAMS *my_params )
{

  ATOM_LIST *atom = resid->my_resid->atom_header;
  
  while ( atom != NULL )
    rotate_atom_lister_rot_params_and_step( &atom, my_params );

}














void rotate_atom_lister_rot_params_and_step( ATOM_LIST **atom, 
  ROT_PARAMS *my_params )
{
  rotate_atom_lister_rot_params( *atom, my_params );
  *atom = (*atom)->next;
}















void rotate_atom_lister_rot_params( ATOM_LIST *atom, ROT_PARAMS *my_params )
{

  translate_atom_lister_xyz_coords( atom, my_params->minor_trans, -1.0 );
  
  rotate_my_atom_lister_rot_params( atom, my_params );
  
  translate_atom_lister_xyz_coords( atom, my_params->major_trans, 1.0 );

}















void translate_atom_lister_xyz_coords( ATOM_LIST *atom, STAT t[], STAT x )
{
  atom->my_atom->xyz->x += t[ X_INDEX ] * x;
  atom->my_atom->xyz->y += t[ Y_INDEX ] * x;
  atom->my_atom->xyz->z += t[ Z_INDEX ] * x;
}















void rotate_my_atom_lister_rot_params( ATOM_LIST *atom, ROT_PARAMS *my_params )
{

  /*  In this file, above:  */
  *(atom->my_atom->xyz) = rotate_w_array( atom->my_atom->xyz, my_params->bef_angle );
  
  /*  In this file, below:  */
  *(atom->my_atom->xyz) = rotate_xyz_angle_params( atom->my_atom->xyz, my_params );
  
  /*  In this file, below:  */
  *(atom->my_atom->xyz) = rotate_opposite_array( atom->my_atom->xyz,
    my_params->aft_angle );

}















XYZ rotate_xyz_angle_params( XYZ *a, ROT_PARAMS *my_params )
{

  MATRIX *create_matrix( MATRIX *xx, int ii, int jj );
  STAT rotate( XYZ *point, MATRIX *p, char index );
  
  int nn = COORD_BUFFER * PARAM_TYPE_BUFFER;
  
  XYZ b;
  
  MATRIX *p = NULL;
  p = (MATRIX *)create_matrix( p, nn, 1 );
  
  *(p->x + X_INDEX) = my_params->angle[ X_INDEX ];
  *(p->x + Y_INDEX) = my_params->angle[ Y_INDEX ];
  *(p->x + Z_INDEX) = my_params->angle[ Z_INDEX ];
  
  *(p->x + X_INDEX + COORD_BUFFER) = my_params->c[ X_INDEX ];
  *(p->x + Y_INDEX + COORD_BUFFER) = my_params->c[ Y_INDEX ];
  *(p->x + Z_INDEX + COORD_BUFFER) = my_params->c[ Z_INDEX ];
  
  /*  In ../mm/rotate.c:  */
  b.x = rotate( a, p, X_INDEX );
  b.y = rotate( a, p, Y_INDEX );
  b.z = rotate( a, p, Z_INDEX );
  
  kill_matrix( p );
  
  return b;

}
















XYZ rotate_opposite_array( XYZ *a, STAT angle[] )
{

  STAT angle_y[ COORD_BUFFER ], angle_x[ COORD_BUFFER ], angle_z[ COORD_BUFFER ];
  
  int i;
  
  for ( i = 0; i < COORD_BUFFER; i++ ) {
  
    angle_x[ i ] = 0.0;
    angle_y[ i ] = 0.0;
    angle_z[ i ] = 0.0;
    
    switch ( i ) {
    
    case X_INDEX :
      angle_x[ i ] = -1.0 * angle[ i ];
      break;
      
    case Y_INDEX :
      angle_y[ i ] = -1.0 * angle[ i ];
      break;
      
    case Z_INDEX :
      angle_z[ i ] = -1.0 * angle[ i ];
      break;
      
    default :
      error( 3610, "Unrecognized coordinate type.\n" );
      break;
    
    }
  
  }
  
  /*  In this file, above:  */
  XYZ b = rotate_w_array( a, angle_z );
  XYZ c = rotate_w_array( &b, angle_y );
  XYZ d = rotate_w_array( &c, angle_x );
  
  return d;

}




























