#include "sub.h"

















#define INPUT_FNAME "data/sp1/1VA2.txt"
#define DATA_FILE_FNAME "data/c2h2/data_files.txt"

#define AA_INDEX 4
#define AA_TYPE 'V'

#define NUMB_ATOMS_IN_BACKBONE 4
#define CA_ATOM 0
#define NN_ATOM 1
#define CC_ATOM 2
#define HA_ATOM 3

#define RMSD_MAX 0.013











void do_amino_acid_substitution( void )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  
  MOL_LIST *mol = NULL;  
  /*  In ../parse/mol_list.c:  */
  mol = (MOL_LIST *)get_mol_list_w_fname( mol, INPUT_FNAME ); 
  
  /*  In this file, below:  */
  mol = (MOL_LIST *)do_amino_acid_substitution_mol( mol, AA_INDEX, AA_TYPE );

}













MOL_LIST *do_amino_acid_substitution_mol( MOL_LIST *mol, int aa_index, char aa )
{

  RESID_LIST *resid = NULL;
  resid = (RESID_LIST *)get_resid_lister_mol_lister_index( mol, aa_index );
  
  return do_amino_acid_sub_single_resid_lister( mol, resid, aa );

}












RESID_LIST *get_resid_lister_mol_lister_index( MOL_LIST *mol, int aa_index )
{

  int i = 0;
  
  RESID_LIST *resid = mol->my_mol->resid_header;
  
  while ( resid != NULL ) {
  
    if ( ++i == aa_index )
      return resid;
      
    resid = resid->next;
  
  }
  
  return NULL;

}














MOL_LIST *do_amino_acid_sub_single_resid_lister( MOL_LIST *mol, RESID_LIST *resid,
  char aa )
{

  RESID_LIST *sub_resid = NULL;
  sub_resid = (RESID_LIST *)get_resid_lister_type_data_fname( sub_resid, aa, 
    DATA_FILE_FNAME );
    
  return replace_amino_acid_single_resid_lister( mol, resid, sub_resid );

}














RESID_LIST *get_resid_lister_type_data_fname( RESID_LIST *sub_resid, char aa,
  char data_fname[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *data_in = NULL;
  data_in = (FILE *)open_text_file_reading( data_fname, "" );
  
  sub_resid = (RESID_LIST *)get_resid_lister_type_w_fh( sub_resid, aa, data_in );
  
  close_file( data_in );
  
  return sub_resid;

}













RESID_LIST *get_resid_lister_type_w_fh( RESID_LIST *resid, char aa, FILE *data_in )
{

  char line[ BUFFER ];
  
  while (( resid == NULL ) && ( fgets( line, BUFFER, data_in ) != NULL ))
    resid = (RESID_LIST *)get_resid_lister_type_single_line( resid, line, aa );
    
  return resid;

}













RESID_LIST *get_resid_lister_type_single_line( RESID_LIST *resid, char line[],
  char aa )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  
  MOL_LIST *mol = NULL;  
  char fname[ BUFFER ], key[ BUFFER ];  

  sscanf( line, "%*d%s", key );
  
  sprintf( fname, "data/c2h2/raw_data/%s.pdb", key );
  
  if ( aa == 'F' ) {
    if (( ! strcmp( key, "1SP1" )) || ( ! strcmp( key, "1X6E" ))
      || ( ! strcmp( key, "1SP2" )) || ( ! strcmp( key, "1U86" ))
      || ( ! strcmp( key, "2COT" )))
      return resid;
  }
  
  printf( "key :  %s\n", key );
  
  /*  In ../parse/mol_list.c:  */
  mol = (MOL_LIST *)get_mol_list_w_fname( mol, fname );
  
  /*  In this file, below:  */
  resid = (RESID_LIST *)get_resid_lister_type_w_mol_list( resid, mol, aa );
  
  kill_mol_list( mol );
  
  return resid;

}














RESID_LIST *get_resid_lister_type_w_mol_list( RESID_LIST *resid, 
  MOL_LIST *mol_header, char aa )
{

  MOL_LIST *mol = mol_header;
  
  while (( resid == NULL ) && ( mol != NULL ))
    resid = (RESID_LIST *)get_resid_type_single_mol_lister_and_step( resid,
      &mol, aa );
      
  return resid;

}










RESID_LIST *get_resid_type_single_mol_lister_and_step( RESID_LIST *resid,
  MOL_LIST **mol, char aa )
{
  resid = (RESID_LIST *)get_resid_type_single_mol_lister( resid, *mol, aa );
  *mol = (*mol)->next;
  return resid;
}












RESID_LIST *get_resid_type_single_mol_lister( RESID_LIST *resid, MOL_LIST *mol,
  char aa )
{

  RESID_LIST *my_resid = mol->my_mol->resid_header;
  
  while ( my_resid != NULL )
    resid = (RESID_LIST *)get_resid_type_single_resid_lister_and_step( resid,
      &my_resid, aa );

  return resid;

}











RESID_LIST *get_resid_type_single_resid_lister_and_step( RESID_LIST *resid,
  RESID_LIST **my_resid, char aa )
{
  resid = (RESID_LIST *)get_resid_type_single_resid_lister( resid, *my_resid, aa );
  *my_resid = (*my_resid)->next;
  return resid;
}












RESID_LIST *get_resid_type_single_resid_lister( RESID_LIST *resid,
  RESID_LIST *my_resid, char aa )
{

  RESID_LIST *copy_resid_lister( RESID_LIST *new_resid, RESID_LIST *resid );

  if ( my_resid->my_resid->aa != aa )
    return resid;
    
  /*  In this file, below:  */  
  if ( ! resid_has_hydro( my_resid ))
    return resid;
     
  /*  In ../mm/mol.c:  */  
  return copy_resid_lister( resid, my_resid );

}












int resid_has_hydro( RESID_LIST *resid )
{

  ATOM_LIST *atom = resid->my_resid->atom_header;
  
  while ( atom != NULL ) {
  
    if ( ! strcmp( atom->my_atom->atom, "HA" ))
      return 1;
      
    atom = atom->next;
  
  }
  
  return 0;

}









/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/













MOL_LIST *replace_amino_acid_single_resid_lister( MOL_LIST *mol, 
  RESID_LIST *resid, RESID_LIST *sub_resid )
{

  if ( sub_resid == NULL )
    error( 3600, "Unexpected null value for substitution amino acid.\n" );
    
  return replace_my_amino_acid_single_resid_lister( mol, resid, sub_resid );

}












MOL_LIST *replace_my_amino_acid_single_resid_lister( MOL_LIST *mol,
  RESID_LIST *resid, RESID_LIST *sub_resid )
{

  XYZ_CELL *create_xyz_cell( XYZ_CELL *xx, int nn );
  
  XYZ_CELL *sub_cell = NULL, *my_cell = NULL;
  sub_cell = (XYZ_CELL *)create_xyz_cell( sub_cell, NUMB_ATOMS_IN_BACKBONE );
  my_cell = (XYZ_CELL *)create_xyz_cell( my_cell, NUMB_ATOMS_IN_BACKBONE );
  
  set_xyz_cell_backbone( sub_cell, sub_resid );
  set_xyz_cell_backbone( my_cell, resid );
  
  replace_amino_acid_resid_lister_xyz_cells( resid, sub_resid, my_cell, sub_cell );
  
  kill_xyz_cell( sub_cell );
  kill_xyz_cell( my_cell );
  
  return mol;

}












void set_xyz_cell_backbone( XYZ_CELL *my_cell, RESID_LIST *resid )
{
  *(my_cell->array + CA_ATOM) = get_xyz_atom_type( resid, "CA" );
  *(my_cell->array + NN_ATOM) = get_xyz_atom_type( resid, "N" );
  *(my_cell->array + CC_ATOM) = get_xyz_atom_type( resid, "C" );
  *(my_cell->array + HA_ATOM) = get_xyz_atom_type( resid, "HA" );
}










XYZ get_xyz_atom_type( RESID_LIST *resid, char type[] )
{

  ATOM_LIST *atom = resid->my_resid->atom_header;
  
  while ( atom != NULL ) {
  
    if ( ! strcmp( type, atom->my_atom->atom ))
      return *(atom->my_atom->xyz);
      
    atom = atom->next;
  
  }
  
  error( 3601, "Did not find atom type in residue.\n" );

}













void replace_amino_acid_resid_lister_xyz_cells( RESID_LIST *resid, 
  RESID_LIST *sub_resid, XYZ_CELL *my_cell, XYZ_CELL *sub_cell )
{

  STAT get_rmsd_xyz_cell_w_params( ROT_PARAMS *my_params, XYZ_CELL *major_cell,
    XYZ_CELL *minor_cell );

  ROT_PARAMS my_params;
  /*  In rmsd.c:  */
  STAT rmsd = get_rmsd_xyz_cell_w_params( &my_params, my_cell, sub_cell );
  
  /*  In rmsd.c:  */
  rotate_resid_lister_w_rot_params( sub_resid, &my_params );
  
  /*  In this file, below:  */
  print_two_resid_listers( resid, sub_resid );
  
  printf( "\n\n\n" );
  
  /*  In this file, below:  */
  replace_amino_acid_resid_lister_w_rotated_aa( resid, sub_resid );
  /*  sub_resid is killed....  */
  
}
















void replace_amino_acid_resid_lister_w_rotated_aa( RESID_LIST *resid,
  RESID_LIST *sub_resid )
{

  ATOM_LIST *sub_atom = sub_resid->my_resid->atom_header, *prev = NULL;
  
  while ( sub_atom != NULL )
    replace_single_atom_lister_rotated_aa_and_step( resid, sub_resid,
      &sub_atom, &prev );
      
  replace_my_amino_acid_resid_lister( resid, sub_resid );
  /*  sub_resid is killed...  */

}













void replace_single_atom_lister_rotated_aa_and_step( RESID_LIST *resid,
  RESID_LIST *sub_resid, ATOM_LIST **sub_atom, ATOM_LIST **prev )
{
  replace_single_atom_lister_rotated_aa( resid, sub_resid, sub_atom, prev );
  *prev = *sub_atom;
  *sub_atom = (*sub_atom)->next;
}















void replace_single_atom_lister_rotated_aa( RESID_LIST *resid,
  RESID_LIST *sub_resid, ATOM_LIST **sub_atom, ATOM_LIST **prev )
{

  if ( ! strcmp( (*sub_atom)->my_atom->atom, "OXT" )) 
    replacement_atom_oxt( sub_resid, sub_atom, prev );

  else if ( atom_is_in_main_backbone( *sub_atom )) 
    replacement_atom_backbone( resid, *sub_atom );
    
}













int resid_has_atom( RESID_LIST *resid, char a[] )
{

  ATOM_LIST *atom = resid->my_resid->atom_header;
  
  while ( atom != NULL ) {
  
    if ( ! strcmp( atom->my_atom->atom, a ))
      return 1;
      
    atom = atom->next;
  
  }
  
  return 0;

}















void replacement_atom_oxt( RESID_LIST *sub_resid, ATOM_LIST **atom, 
  ATOM_LIST **prev )
{
  (*prev)->next = (*atom)->next;
  kill_atom_lister( *atom );
  *atom = *prev;
}












int atom_is_in_main_backbone( ATOM_LIST *atom )
{

  char a[ BUFFER ];
  
  strcpy( a, atom->my_atom->atom );
  
  if ( ! strcmp( a, "N" ))    return 1;
  if ( ! strcmp( a, "CA" ))   return 1;
  if ( ! strcmp( a, "C" ))    return 1;
  if ( ! strcmp( a, "O" ))    return 1;
  if ( ! strcmp( a, "H" ))    return 1;
  if ( ! strcmp( a, "HA" ))   return 1;
  
  return 0;

}
















void replacement_atom_backbone( RESID_LIST *resid, ATOM_LIST *atom )
{

  ATOM_LIST *a = NULL;
  a = (ATOM_LIST *)get_atom_from_resid( resid, atom->my_atom->atom );
  
  if ( a == NULL ) 
    error( 3700, "Could not find atom to substitute.\n" );
  
  *(atom->my_atom->xyz) = *(a->my_atom->xyz);

}













ATOM_LIST *get_atom_from_resid( RESID_LIST *resid, char a[] )
{

  ATOM_LIST *atom = resid->my_resid->atom_header;
  
  while ( atom != NULL ) {
  
    if ( ! strcmp( atom->my_atom->atom, a ))
      return atom;
      
    atom = atom->next;
  
  }
  
  return NULL;
  
}

















void replace_my_amino_acid_resid_lister( RESID_LIST *resid, RESID_LIST *sub_resid )
{

  strcpy( resid->my_resid->resid, sub_resid->my_resid->resid );
  
  resid->my_resid->aa = sub_resid->my_resid->aa;
  resid->my_resid->mol_type = sub_resid->my_resid->mol_type;
  resid->my_resid->type = sub_resid->my_resid->type;
  
  resid->my_resid->atom_header = sub_resid->my_resid->atom_header;
  
  sub_resid->my_resid->atom_header = NULL;
  
  kill_resid_lister( sub_resid );

}












/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/
/*============================================================================*/















void print_two_resid_listers( RESID_LIST *a, RESID_LIST *b )
{

  printf( "%s\t%s\n", a->my_resid->resid, b->my_resid->resid );
  
  ATOM_LIST *atom_a = a->my_resid->atom_header;
  ATOM_LIST *atom_b = b->my_resid->atom_header;
  
  while (( atom_a != NULL ) || ( atom_b != NULL ))
    print_two_atom_listers_and_step( &atom_a, &atom_b );

}














void print_two_atom_listers_and_step( ATOM_LIST **atom_a, ATOM_LIST **atom_b )
{

  print_two_atom_listers( *atom_a, *atom_b );
  
  if ( *atom_a != NULL )
    *atom_a = (*atom_a)->next;
  
  if ( *atom_b != NULL )
    *atom_b = (*atom_b)->next;

}












void print_two_atom_listers( ATOM_LIST *a, ATOM_LIST *b )
{

  if ( a != NULL ) {
    XYZ *aa = a->my_atom->xyz;
    printf( "%4s\t%8.3f  %8.3f  %8.3f\t\t",
      a->my_atom->atom,
      aa->x,
      aa->y,
      aa->z );
  }
  
  if ( b != NULL ) {
    XYZ *bb = b->my_atom->xyz;
    printf( "%4s\t%8.3f  %8.3f  %8.3f", 
      b->my_atom->atom,
      bb->x,
      bb->y,
      bb->z );
  }
  
  printf( "\n" );
  
}





















