#include "zinc.h"



















#define DATA_FILE_FNAME "data/c2h2/data_files.txt"
#define DIR_NAME "data/c2h2/raw_data"
#define OUTPUT_FNAME "data/c2h2/params/zinc_bonds.txt"




#define NUMB_OF_ZINC_BONDS 4















void output_zinc_distances( void )
{

  SLIST *dist_header = NULL;
  dist_header = (SLIST *)get_zinc_distances_raw_data_files( dist_header,
    DATA_FILE_FNAME, DIR_NAME );
    
  output_zinc_list_dist( dist_header, OUTPUT_FNAME );

}















SLIST *get_zinc_distances_raw_data_files( SLIST *dist_header, char fname[],
  char dir[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( fname, "" );
  
  dist_header = (SLIST *)get_zinc_distances_raw_data_fh( dist_header, dir, in );
  
  close_file( in );
  
  return dist_header;

}














SLIST *get_zinc_distances_raw_data_fh( SLIST *dist_header, char dir[], FILE *in )
{

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, in ) != NULL )
    dist_header = (SLIST *)get_zinc_distances_single_line( dist_header, line, dir );
    
  return dist_header;

}














SLIST *get_zinc_distances_single_line( SLIST *dist_header, char line[], char dir[] )
{

  MOL_LIST *get_mol_list_w_fname( MOL_LIST *mol_header, char fname[] );
  
  char key[ BUFFER ], fname[ BUFFER ]; 
  sscanf( line, "%*d%s", key );
  sprintf( fname, "%s/%s.pdb", dir, key );
  
  printf( "key :  %s\n", key );
  
  MOL_LIST *mol_header = NULL;

  /*  In ../parse/mol_list.c:  */
  mol_header = (MOL_LIST *)get_mol_list_w_fname( mol_header, fname );
  
  /*  In this file, below:  */
  dist_header = (SLIST *)get_zinc_dist_single_mol_list( dist_header, mol_header );
  
  kill_mol_list( mol_header );
  
  return dist_header;  

}
















SLIST *get_zinc_dist_single_mol_list( SLIST *dist_header, MOL_LIST *mol_header )
{

  MOL_LIST *mol = mol_header;
  
  while ( mol != NULL )
    dist_header = (SLIST *)get_zinc_dist_single_mol_and_step( dist_header, &mol );
    
  return dist_header;

}













SLIST *get_zinc_dist_single_mol_and_step( SLIST *dist_header, MOL_LIST **mol )
{

  dist_header = (SLIST *)get_zinc_dist_single_mol( dist_header, *mol );
  
  *mol = (*mol)->next;
  
  return dist_header;

}













SLIST *get_zinc_dist_single_mol( SLIST *dist_header, MOL_LIST *mol )
{

  if ( numb_of_zn_in_mol( mol ) != 1 )
    return dist_header;

  RESID_LIST *resid = mol->my_mol->resid_header;
  
  while ( resid != NULL )
    dist_header = (SLIST *)get_zinc_dist_single_resid_and_step( dist_header,
      &resid );
      
  return dist_header;

}














int numb_of_zn_in_mol( MOL_LIST *mol )
{

  int numb = 0;
  
  RESID_LIST *resid = mol->my_mol->resid_header;
  
  while ( resid != NULL ) {
  
    if ( ! strcmp( resid->my_resid->resid, "ZN" ))
      ++numb;
      
    resid = resid->next;
  
  }
  
  return numb;

}












SLIST *get_zinc_dist_single_resid_and_step( SLIST *dist_header, RESID_LIST **resid )
{

  dist_header = (SLIST *)get_zinc_dist_single_resid( dist_header, *resid );
  
  *resid = (*resid)->next;
  
  return dist_header;

}












SLIST *get_zinc_dist_single_resid( SLIST *dist_header, RESID_LIST *resid )
{

  if ( resid->my_resid->mol_type != MOL_TYPE_ION )
    return dist_header;
    
  ATOM_LIST *zinc = resid->my_resid->atom_header;
  
  if ( zinc->next != NULL )
    error( 2000, "Expected single zinc ion in atom list.\n" );
    
  return get_my_zinc_dist_single_resid( dist_header, zinc );

}












SLIST *get_my_zinc_dist_single_resid( SLIST *dist_header, ATOM_LIST *zinc )
{

  ATOM_LIST *atom = zinc->my_atom->con_header;
  
  int i = 0;
  
  while ( i++ < NUMB_OF_ZINC_BONDS ) 
    dist_header = (SLIST *)get_zinc_dist_single_bond_and_step( dist_header, zinc,
      &atom );
    
  return dist_header;

}













SLIST *get_zinc_dist_single_bond_and_step( SLIST *dist_header, ATOM_LIST *zinc,
  ATOM_LIST **atom )
{

  dist_header = (SLIST *)get_zinc_dist_single_bond( dist_header, zinc, *atom );
  
  *atom = (*atom)->next;
  
  return dist_header;

}












SLIST *get_zinc_dist_single_bond( SLIST *dist_header, ATOM_LIST *zinc,
  ATOM_LIST *atom )
{

  STAT get_dist_xyz( XYZ *a, XYZ *b );

  if ( atom == NULL )
    error( 2001, "Expected extra zinc bond.\n" );
    
  /*  In ../main/xyz.c:  */  
  STAT dist = get_dist_xyz( zinc->my_atom->xyz, atom->my_atom->xyz );
  
  printf( "dist :  %s  %.3f\n", atom->my_atom->atom, dist );
  
  /*  In this file, below:  */  
  return add_zinc_dist_single_bond( dist_header, atom->my_atom->atom, dist );

}














SLIST *add_zinc_dist_single_bond( SLIST *dist_header, char atom[], STAT dist )
{

  SLIST *create_slister( SLIST *xx );
  
  SLIST *x = NULL;
  x = (SLIST *)create_slister( x );
  
  strcpy( x->x, atom );
  x->f = dist;
  
  if ( dist_header == NULL )
    return x;
    
  SLIST *curr = dist_header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = x;
  
  return dist_header;

}















void output_zinc_list_dist( SLIST *dist_header, char fname[] )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *fh = NULL;
  fh = (FILE *)open_text_file_writing( fname, "" );
  
  output_zinc_list_dist_w_fh( dist_header, fh );
  
  close_file( fh );

}















void output_zinc_list_dist_w_fh( SLIST *dist_header, FILE *fh )
{

  STAT dist_sg = get_avg_zinc_dist_w_str( dist_header, "SG" );
  STAT dist_ne = get_avg_zinc_dist_w_str( dist_header, "NE2" );
  
  STAT sd_sg = get_sd_zinc_dist_w_str( dist_header, dist_sg, "SG" );
  STAT sd_ne = get_sd_zinc_dist_w_str( dist_header, dist_ne, "NE2" );
  
  fprintf( fh, "CYS_SG_avg:    %.3f   SD:   %.3f\n", dist_sg, sd_sg );
  fprintf( fh, "HIS_NE2_avg:   %.3f   SD:   %.3f\n", dist_ne, sd_ne );

}















STAT get_avg_zinc_dist_w_str( SLIST *dist_header, char str[] )
{

  SLIST *x = dist_header;
  
  float sum = 0.0, numb = 0.0;
  
  while ( x != NULL ) {
    
    if ( ! strcmp( str, x->x )) {
      sum += x->f;
      numb += 1.0;
    }
    
    x = x->next;
  
  }
  
  return sum / numb;

}















STAT get_sd_zinc_dist_w_str( SLIST *dist_header, STAT avg, char str[] )
{

  STAT sum = 0.0, total = 0.0;
  
  SLIST *x = dist_header;
  
  while ( x != NULL ) {
  
    if ( ! strcmp( x->x, str )) {
      sum += (x->f - avg) * (x->f - avg);
      total += 1.0;
    }
    
    x = x->next;
  
  }
  
  return sqrt( sum / total );

}





















