#include "gene.h"















#define HG18_FNAME "data/ft_hg18/hg18.refSeq.500.100.Mar2006.txt.final.txt"
#define HG19_ALL_FNAME "data/ft_hg19/hg19.refSeq.500.100.Feb2009.txt"
#define HG19_OUT_FNAME "data/ft_hg19/hg19.refSeq.500.100.Feb2009.txt.final.txt"








#define INPUT_MAF_FNAME "data/ft_hg19/align/hg19.multiz46way.my_fa.txt"
#define ORTHO_FILE_FNAME_BASE "data/ft_hg19/occur"












#define WIND_WIDTH 10
#define NUMB_OF_WIND ((SEQ_LENGTH) / (WIND_WIDTH))











void do_gene_sequence_file_transfer( void )
{

  FILE *open_text_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *in = NULL, *out = NULL;
  in = (FILE *)open_text_file_reading( HG19_ALL_FNAME, "" );
  out = (FILE *)open_text_file_writing( HG19_OUT_FNAME, "" );
  
  do_my_gene_sequence_file_transfer( in, out );
  
  close_file( in );
  close_file( out );

}













void do_my_gene_sequence_file_transfer( FILE *in, FILE *out )
{

  char label[ BUFFER ];

  while ( fgets( label, BUFFER, in ) != NULL )
    do_gene_sequence_file_single_gene( label, in, out );

}












void do_gene_sequence_file_single_gene( char label[], FILE *in, FILE *out )
{

  if ( label[ 0 ] != '>' )
    error( 2800, "Expected angle character at fasta label line.\n" );
    
  char seq[ LINE_BUFFER ];
  
  if ( ! get_sequence_from_fasta_file( seq, in ))
    return;
    
  do_my_gene_sequence_file_single_gene( label, seq, out );

}













int get_sequence_from_fasta_file( char seq[], FILE *in )
{

  int good_seq = 1;

  register int i = 0;
  
  while ( i < SEQ_LENGTH ) {
  
    char c = fgetc( in );
    
    switch ( tolower( c )) {
    
    case 'a' :
    case 'c' :
    case 'g' :
    case 't' :
      seq[ i++ ] = c;
      break;
      
    case 'n' :
      ++i;
      good_seq = 0;
      break;
      
    case '\n' :
      break;
      
    default :
      error( 2801, "Unrecognized nucleotide type.\n" );
      break;
    
    }
  
  }
  
  if ( fgetc( in ) != '\n' )
    error( 2702, "Expected newline at end of sequence.\n" );
    
  return good_seq;

}














void do_my_gene_sequence_file_single_gene( char label[], char seq[], FILE *out )
{

  char gene[ BUFFER ];
  
  get_gene_accession_fa_label( gene, label );
  
  printf( "gene :  %s\n", gene );
  
  do_gene_sequence_file_transfer_gene_acc( label, gene, seq, out );

}










void get_gene_accession_fa_label( char gene[], char label[] )
{

  int underscores = 0;

  register int i = 0, j = 0;
  
  while ( label[ i ] != '\0' ) {
    if ( label[ i ] == '_' ) {
      if ( underscores++ )
        break;
    }
    ++i;
  }
  
  while ( ! isspace( label[ ++i ] )) 
    gene[ j++ ] = label[ i ];

  gene[ j ] = '\0';

}















void do_gene_sequence_file_transfer_gene_acc( char label[], char gene[], 
  char seq[], FILE *out )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *fh = NULL;
  fh = (FILE *)open_text_file_reading( HG18_FNAME, "" );
  
  do_gene_seq_file_transfer_gene_acc_w_fh( label, gene, seq, fh, out );
  
  close_file( fh );

}













void do_gene_seq_file_transfer_gene_acc_w_fh( char label[], char gene[], char seq[],
  FILE *fh, FILE *out )
{

  int found = 0;

  char line[ BUFFER ];
  
  while (( ! found ) && ( fgets( line, BUFFER, fh ) != NULL )) {
  
    if ( line[ 0 ] == '>' )
      found = gene_accession_matches_fasta_label( gene, line );
  
  }
  
  if ( found )
    put_gene_sequence_matching_seqs( label, gene, seq, out );
  
}












int gene_accession_matches_fasta_label( char gene[], char line[] )
{

  char old_gene[ BUFFER ];
  
  get_gene_accession_fa_label( old_gene, line );
  
  return ( ! strcmp( old_gene, gene )) ? (1) : (0);

}














void put_gene_sequence_matching_seqs( char label[], char gene[], char seq[], 
  FILE *out )
{

  static int j = 0;

  fprintf( out, "%s", label );
  
  register int i = 0;
  
  while ( i < strlen( seq )) {
  
    fprintf( out, "%c", seq[ i++ ] );
    
    if ( i % 50 == 0 )
      fprintf( out, "\n" );
  
  }
  
  printf( "\t\t\tFound gene (%d) :  %s\n", ++j, gene );

}














/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/













void do_gene_numb_of_orthologs( void )
{
  
  printf( "Obtaining number of orthologs...\n\n" );
  
  do_my_gene_numb_of_orthologs();
  
  printf( "\n" );
  
}












void do_my_gene_numb_of_orthologs( void )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *maf_fh = NULL;
  maf_fh = (FILE *)open_text_file_reading( INPUT_MAF_FNAME, "" );
  
  do_gene_numb_orthologs_w_maf_fh( maf_fh );
  
  close_file( maf_fh );

}













void do_gene_numb_orthologs_w_maf_fh( FILE *maf_fh )
{

  STR_LIST *get_spec_list_align_fh( STR_LIST *spec_header, FILE *fh );
  
  STR_LIST *spec_header = NULL;
  /*  In occur.c:  */
  spec_header = (STR_LIST *)get_spec_list_align_fh( spec_header, maf_fh );
  
  /*  In this file, below:  */
  do_gene_numb_orthologs_spec_list( spec_header, maf_fh );

}












void do_gene_numb_orthologs_spec_list( STR_LIST *spec_header, FILE *maf_fh )
{

  STR_LIST *spec = spec_header;
  
  while ( spec != NULL )
    do_gene_numb_orthologs_single_spec_and_step( &spec, maf_fh );

}










void do_gene_numb_orthologs_single_spec_and_step( STR_LIST **spec, FILE *maf_fh )
{
  do_gene_numb_orthologs_single_spec( *spec, maf_fh );
  *spec = (*spec)->next;
}












void do_gene_numb_orthologs_single_spec( STR_LIST *spec, FILE *maf_fh )
{

  FILE *open_binary_file_writing( char a[], char b[] );
  
  char fname[ BUFFER ];
  get_gene_numb_orthologs_single_spec( fname, spec );
  
  FILE *out = NULL;
  out = (FILE *)open_binary_file_writing( fname, "gene.txt" );
  
  do_gene_numb_orthologs_single_spec_w_fh( spec->str, maf_fh, out );
  
  close_file( out );
  
}












void do_gene_numb_orthologs_single_spec_w_fh( char spec[], FILE *maf_fh,
  FILE *out )
{

  set_position( 0, maf_fh );
  
  printf( "Species :  %s\n", spec );
  
  do_gene_numb_orthologs_set_maf_fh( spec, maf_fh, out );

}











void do_gene_numb_orthologs_set_maf_fh( char spec[], FILE *maf_fh, FILE *out )
{

  SEQ seq_numb = 0;

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, maf_fh ) != NULL )
    do_gene_single_ortholog_first_line( line, spec, ++seq_numb, maf_fh, out );

}












void do_gene_single_ortholog_first_line( char line[], char spec[], SEQ seq_numb,
  FILE *maf_fh, FILE *out )
{

  if ( line[ 0 ] == '>' )
    do_my_gene_single_ortholog_first_line( line, spec, seq_numb, maf_fh, out );
  
}







void do_my_gene_single_ortholog_first_line( char line[], char spec[], 
  SEQ seq_numb, FILE *maf_fh, FILE *out )
{

  WIND_CT_CELL *create_wind_ct_cell( WIND_CT_CELL *xx, INDEX nn );
  
  WIND_CT_CELL *my_cell = NULL;
  my_cell = (WIND_CT_CELL *)create_wind_ct_cell( my_cell, NUMB_OF_WIND );
  
  set_initial_wind_ct_cell( my_cell );
  
  do_gene_single_ortholog_single_gene( my_cell, line, spec, seq_numb, maf_fh, out );
  
  kill_wind_ct_cell( my_cell );
  
}












void set_initial_wind_ct_cell( WIND_CT_CELL *cell )
{

  COORD pos = 1 - SEQ_LENGTH;
  
  WIND_CT *wind = cell->array;
  
  register int i = 0;
  
  while ( i++ < cell->n ) {
  
    wind->my_wind.min = pos;
    wind->my_wind.max = pos + WIND_WIDTH - 1;
    wind->count = 0;
    
    pos += WIND_WIDTH;
    
    ++wind;
  
  }

}










void do_gene_single_ortholog_single_gene( WIND_CT_CELL *my_cell, char line[],
  char spec[], SEQ seq_numb, FILE *maf_fh, FILE *out )
{

  char maf_line[ LINE_BUFFER ];
  
  while ( fgets( maf_line, LINE_BUFFER, maf_fh ) != NULL ) {
  
    if ( maf_line[ 0 ] == '=' )
      break;
      
  }

}













/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/
/*=======================================================================*/













void get_gene_numb_orthologs_single_spec( char fname[], STR_LIST *spec )
{
  sprintf( fname, "%s/%s/%s.ft_occur", 
    ORTHO_FILE_FNAME_BASE, 
    spec->str, 
    spec->str );
} 



























