#include "go_genes.h"











#define GO_GENE_FNAME "data/ft_hg19/go/go.hg.condensed.txt"
#define GO_LIST_FNAME "data/ft_hg18/go/go_list.txt"
#define GO_FT_FNAME "data/ft_hg18/go/go_genes.txt"

#define LIST_FNAME_BASE "data/ft_hg18/evol/lists/galGal3.Mammalia.converted.sorted"
#define OUTPUT_BASE "data/ft_hg18/go/signif_go_terms.galGal3.genes"

#define MIN_GENE_NUMB 5















void do_go_genes( void )
{

  printf( "Analyzing GO genes...\n\n" );
  
  do_my_go_genes();

}













void do_my_go_genes( void )
{

  get_and_put_go_list();
  
  get_and_put_go_list_genes();

}












void get_and_put_go_list( void )
{

  SLIST *go_header = NULL;
  go_header = (SLIST *)get_go_term_list( go_header );
  
  put_go_list( go_header );
  
  kill_slist( go_header );

}












SLIST *get_go_term_list( SLIST *go_header )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  FILE *in = NULL;
  in = (FILE *)open_text_file_reading( GO_GENE_FNAME, "" );
  
  go_header = (SLIST *)get_go_term_list_w_fh( go_header, in );
  
  close_file( in );
  
  return go_header;

}












SLIST *get_go_term_list_w_fh( SLIST *go_header, FILE *in )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL )
    go_header = (SLIST *)get_go_term_list_single_line( go_header, line );

  return go_header;

}












SLIST *get_go_term_list_single_line( SLIST *go_header, char line[] )
{

  char go_line[ LINE_BUFFER ];
  
  sscanf( line, "%*s%s", go_line );

  int index = 0;
  char term[ BUFFER ];

  while ( get_next_go_term( term, go_line, &index ))
    go_header = (SLIST *)make_and_put_go_term_lister( go_header, term );
    
  return go_header;

}













int get_next_go_term( char term[], char go_line[], int *index )
{

  if ( go_line[ *index ] == '\0' )
    return 0;
  
  register int i = 0;
    
  while ( go_line[ *index ] != ',' )
    term[ i++ ] = go_line[ (*index)++ ];
    
  term[ i ] = '\0';
  ++(*index);
  
  return 1;

}













SLIST *make_and_put_go_term_lister( SLIST *go_header, char term[] )
{

  if ( strncmp( term, "GO:", 3 ))
    return go_header;

  if ( ! term_in_slist( go_header, term ))
    return make_and_put_new_go_term_lister( go_header, term );
    
  return go_header;

}












int term_in_slist( SLIST *go_header, char term[] )
{

  SLIST *go = go_header;
  
  while ( go != NULL ) {

    if ( ! strcmp( go->str, term ))
      return 1;

    go = go->next;

  }
  
  return 0;

}













SLIST *make_and_put_new_go_term_lister( SLIST *go_header, char term[] )
{

  SLIST *create_slister( SLIST *xx );

  static int i = 0;
  
  printf( "GO terms :  %d\n", ++i );

  SLIST *go = NULL;
  go = (SLIST *)create_slister( go );
  
  strcpy( go->str, term );
  
  return put_new_go_term_lister( go_header, go );

}












SLIST *put_new_go_term_lister( SLIST *go_header, SLIST *go )
{

  if ( go_header == NULL )
    return go;

  SLIST *curr = go_header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = go;
  
  return go_header;

}















void put_go_list( SLIST *go_header )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *out = NULL;
  out = (FILE *)open_text_file_writing( GO_LIST_FNAME, "" );
  
  put_my_go_list( go_header, out );
  
  close_file( out );

}













void put_my_go_list( SLIST *go_header, FILE *out )
{

  int i = 0;
  
  SLIST *go = go_header;
  
  while ( go != NULL ) {
  
    fprintf( out, "%6d\t%s\n", ++i, go->str );
    
    go = go->next;
    
  }

}















/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/














void get_and_put_go_list_genes( void )
{

  FILE *open_text_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *in = NULL, *list_fh = NULL, *out = NULL;
  in = (FILE *)open_text_file_reading( GO_GENE_FNAME, "" );
  list_fh = (FILE *)open_text_file_reading( GO_LIST_FNAME, "" );
  out = (FILE *)open_text_file_writing( GO_FT_FNAME, "" );
  
  get_and_put_go_list_genes_w_fhs( in, list_fh, out );
  
  close_file( in );
  close_file( list_fh );
  close_file( out );

}














void get_and_put_go_list_genes_w_fhs( FILE *in, FILE *list_fh, FILE *out )
{

  char list_line[ BUFFER ];
  
  while ( fgets( list_line, BUFFER, list_fh ) != NULL )
    get_and_put_go_list_genes_single_term( list_line, in, out );

}












void get_and_put_go_list_genes_single_term( char list_line[], FILE *in, FILE *out )
{

  static int i = 0;

  char term[ BUFFER ];
  
  sscanf( list_line, "%*d%s", term );
  
  printf( "GO :  %s   (%d)\n", term, ++i );

  SLIST *gene_header = NULL;
  
  gene_header = (SLIST *)get_go_gene_list_single_term( gene_header, term, in );
  
  put_go_list_genes_single_term( term, gene_header, out );
    
  kill_slist( gene_header );

}












SLIST *get_go_gene_list_single_term( SLIST *gene_header, char term[], FILE *in )
{

  int length = strlen( term );
  
  set_position( 0, in );

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, in ) != NULL )
    gene_header = (SLIST *)get_go_gene_list_single_gene( gene_header, line, term,
      length );
    
  return gene_header;

}












SLIST *get_go_gene_list_single_gene( SLIST *gene_header, char line[], char term[],
  int length )
{

  char entry[ LINE_BUFFER ], curr_term[ BUFFER ];
  char gene[ BUFFER ];
  
  sscanf( line, "%s%s", gene, entry );
  
  int index = 0;
  
  /*  In this file, above:  */
  while ( get_next_go_term( curr_term, entry, &index )) {
  
    if ( ! strcmp( curr_term, term ))
      /*  In this file, below:  */
      return make_and_put_new_gene_slister( gene_header, gene );
  
  }
  
  return gene_header;

}














SLIST *make_and_put_new_gene_slister( SLIST *gene_header, char gene[] )
{

  SLIST *create_slister( SLIST *xx );

  /*  In this file, above:  */
  if ( term_in_slist( gene_header, gene ))
    return gene_header;

  SLIST *new = NULL;
  
  new = (SLIST *)create_slister( new );
  
  strcpy( new->str, gene );
  
  if ( gene_header == NULL )
    return new;
    
  SLIST *curr = gene_header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = new;
  
  return gene_header;

}















void put_go_list_genes_single_term( char term[], SLIST *gene_header, FILE *out )
{

  print_sep( out );

  fprintf( out, "%s\n", term );
  
  SLIST *gene = gene_header;
  
  while ( gene != NULL ) {
    fprintf( out, "\t%s\n", gene->str );
    gene = gene->next;
  }
  
  print_sep( out );

}








/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/











void do_go_genes_corr( void )
{

  
  do_go_genes_corr_single_file( LIST_FNAME_BASE, GO_FT_FNAME, GO_GENE_FNAME,
    OUTPUT_BASE, "C.A.txt" );
    
  do_go_genes_corr_single_file( LIST_FNAME_BASE, GO_FT_FNAME, GO_GENE_FNAME,
    OUTPUT_BASE, "C.noA.txt" );
    
}









void do_go_genes_corr_single_file( char input_base[], char go_fname[],
  char gene_fname[], char output_base[], char ext[] )
{

  FILE *open_text_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );
  
  FILE *list_fh = NULL, *genes_fh = NULL, *ft_fh = NULL, *out = NULL;
  FILE *all_fh = NULL;
  
  list_fh = (FILE *)open_text_file_reading( input_base, ext );
  genes_fh = (FILE *)open_text_file_reading( gene_fname, "" );
  ft_fh = (FILE *)open_text_file_reading( go_fname, "" );
  all_fh = (FILE *)open_text_file_reading( input_base, "C.all.txt" );
  out = (FILE *)open_text_file_writing( output_base, ext );
  
  GO_CAT_LIST *go_header = NULL;
  go_header = (GO_CAT_LIST *)get_go_genes_corr_single_file( go_header, list_fh,
    genes_fh, ft_fh, all_fh );
    
  print_go_cat_corr_single_file( go_header, out );
    
  close_file( list_fh );
  close_file( genes_fh );
  close_file( ft_fh );
  close_file( all_fh );
  close_file( out );

}














GO_CAT_LIST *get_go_genes_corr_single_file( GO_CAT_LIST *go_header, FILE *list_fh,
  FILE *genes_fh, FILE *ft_fh, FILE *all_fh )
{

  int total_genes = get_total_genes_go_fh( genes_fh );
  
  set_position( 0, genes_fh );

  go_header = (GO_CAT_LIST *)get_first_go_list( go_header, total_genes, list_fh, 
    genes_fh, ft_fh, all_fh );
  
  return go_header;

}











int get_total_genes_go_fh( FILE *genes_fh )
{

  SLIST *header = NULL;
  
  int numb = 0;

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, genes_fh ) != NULL )
    header = (SLIST *)get_total_gene_list_single_line( header, line, &numb );
    
  kill_slist( header );
    
  return numb;

}












SLIST *get_total_gene_list_single_line( SLIST *header, char line[], int *numb )
{

  SLIST *create_slister( SLIST *xx );

  char gene[ BUFFER ];
  
  sscanf( line, "%s", gene );
  
  /*  In this file, above:  */
  if ( term_in_slist( header, gene ))
    return header;
    
  printf( "%d :  %s\n", *numb, gene );

  ++(*numb);
  
  SLIST *s = NULL;
  s = (SLIST *)create_slister( s );
  
  strcpy( s->str, gene );
  
  s->next = header;
  
  return s;

}










GO_CAT_LIST *get_first_go_list( GO_CAT_LIST *go_header, int total_genes,
  FILE *list_fh, FILE *genes_fh, FILE *ft_fh, FILE *all_fh )
{

  SLIST *gene_header = NULL, *term_header = NULL, *all_g_header = NULL;
  
  gene_header = (SLIST *)get_gene_slist( gene_header, list_fh );
  all_g_header = (SLIST *)get_gene_slist( all_g_header, all_fh );
  
  term_header = (SLIST *)get_first_go_list_w_gene_list( term_header, gene_header,
    genes_fh );
    
  go_header = (GO_CAT_LIST *)get_go_corr_list( go_header, term_header, gene_header,
    all_g_header, total_genes, ft_fh );
  
  kill_slist( gene_header );
  kill_slist( term_header );
  
  return go_header;

}













SLIST *get_gene_slist( SLIST *gene_header, FILE *fh )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, fh ) != NULL )
    gene_header = (SLIST *)get_gene_slist_single_gene( gene_header, line );
    
  return gene_header;

}












SLIST *get_gene_slist_single_gene( SLIST *gene_header, char line[] )
{

  SLIST *create_slister( SLIST *xx );
  
  SLIST *gene = NULL;
  gene = (SLIST *)create_slister( gene );
  
  char g[ BUFFER ];
  sscanf( line, "%*d%*d%*s%s", g );
  
  strcpy( gene->str, g );
  
  if ( gene_header == NULL )
    return gene;
    
  SLIST *curr = gene_header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = gene;
  
  return gene_header;

}












SLIST *get_first_go_list_w_gene_list( SLIST *term_header, SLIST *gene_header, 
  FILE *genes_fh )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, genes_fh ) != NULL )
    term_header = (SLIST *)get_first_go_list_single_line( term_header,
      gene_header, line );
      
  return term_header;

}












SLIST *get_first_go_list_single_line( SLIST *term_header, SLIST *gene_header, 
  char line[] )
{

  char gene[ BUFFER ], entries[ LINE_BUFFER ];
  
  sscanf( line, "%s%s", gene, entries );
  
  /*  In this file, above:  */
  if ( ! term_in_slist( gene_header, gene ))
    return term_header;

  /*  In this file, below:  */
  return get_my_first_go_list_single_line( term_header, line );

}













SLIST *get_my_first_go_list_single_line( SLIST *go_header, char line[] )
{
  /*  In this file, above:  */
  return get_go_term_list_single_line( go_header, line );
}










GO_CAT_LIST *get_go_corr_list( GO_CAT_LIST *go_header, SLIST *term_header, 
  SLIST *gene_header, SLIST *all_g_header, int total_genes, FILE *ft_fh )
{

  char line[ BUFFER ];
  
  fgets( line, BUFFER, ft_fh );
  
  while ( fgets( line, BUFFER, ft_fh ) != NULL )
    go_header = (GO_CAT_LIST *)get_go_corr_list_single_term( go_header, term_header,
      gene_header, all_g_header, line, total_genes, ft_fh );
      
  return go_header;

}












GO_CAT_LIST *get_go_corr_list_single_term( GO_CAT_LIST *go_header, 
  SLIST *term_header, SLIST *gene_header, SLIST *all_g_header, char line[], 
  int total_genes, FILE *ft_fh )
{

  char term[ BUFFER ];
  
  sscanf( line, "%s", term );
  
  /*  In this file, above:  */
  if ( term_in_slist( term_header, term )) 
    /*  In this file, below:  */
    return get_go_corr_list_single_matching_term( go_header, gene_header, 
      all_g_header, term, total_genes, ft_fh );

  /*  In this file, below:  */
  move_fh_to_next_go_term( ft_fh );      
      
  return go_header;

}












void move_fh_to_next_go_term( FILE *fh )
{

  int count = 0;

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, fh ) != NULL ) {
    if ( line[ 0 ] == '=' ) {
      if ( count++ )
        return;
    }
  }

}












GO_CAT_LIST *get_go_corr_list_single_matching_term( GO_CAT_LIST *go_header, 
  SLIST *gene_header, SLIST *all_g_header, char term[], int total_genes, 
  FILE *ft_fh )
{

  GO_CAT_LIST *create_go_cat_lister( GO_CAT_LIST *xx );
  
  GO_CAT_LIST *go = NULL;
  go = (GO_CAT_LIST *)create_go_cat_lister( go );
  
  strcpy( go->my_go->acc, term );
  
  printf( "term :  %s\n", term );
  
  return get_go_corr_single_new_term( go_header, go, gene_header, all_g_header,
    total_genes, ft_fh );

}












GO_CAT_LIST *get_go_corr_single_new_term( GO_CAT_LIST *go_header,
  GO_CAT_LIST *go, SLIST *gene_header, SLIST *all_g_header, int total_genes, 
  FILE *ft_fh )
{

  SLIST *g_header = NULL;
  g_header = (SLIST *)get_gene_list_go_ft_entry( g_header, ft_fh );
  
  go = (GO_CAT_LIST *)get_go_corr_single_term_gene_lists( go, gene_header,
    g_header, all_g_header, total_genes );
  
  kill_slist( g_header );
  
  return put_go_cat_list_into_sorted_list( go_header, go );

}










SLIST *get_gene_list_go_ft_entry( SLIST *g_header, FILE *ft_fh )
{

  int count = 0;

  char line[ BUFFER ];
  
  while ( fgets( line, BUFFER, ft_fh ) != NULL ) {
  
    if ( line[ 0 ] == '=' ) {
      if ( count++ )
        return g_header;
      continue;
    }
      
    g_header = (SLIST *)get_gene_list_go_ft_single_gene( g_header, line );
    
  }
  
}









SLIST *get_gene_list_go_ft_single_gene( SLIST *g_header, char line[] )
{

  SLIST *create_slister( SLIST *xx );
  
  SLIST *g = NULL;
  g = (SLIST *)create_slister( g );
  
  char gene[ BUFFER ];
  sscanf( line, "%s", gene );
  
  strcpy( g->str, gene );
  
  g->next = g_header;
  
  return g;

}







/*

    gene_header = genes with GA->GC box conversions
    g_header = genes with the given GO category

*/








GO_CAT_LIST *get_go_corr_single_term_gene_lists( GO_CAT_LIST *go, 
  SLIST *gene_header, SLIST *g_header, SLIST *all_g_header, int total_genes )
{

  PVAL get_hyper_geo_pval( int nn, int n, int gg, int g );

  /*  In this file, below:  */
  int nn = total_genes;
  int n = get_numb_elem_slist( g_header );
  int g = get_intersection_slist_pair( gene_header, g_header );
  int gg = get_numb_elem_slist( gene_header );
  
  go->my_go->total_genes = n;
  go->my_go->x_genes = g;
  
  /*  In ../gene/sig.c:  */
  go->my_go->pval = get_hyper_geo_pval( nn, n, gg, g );
  
  if ( go->my_go->pval < 1e-3 )
    printf( "\t\t\t\t%.2e\n", (double)go->my_go->pval );

  go->my_go->exp_numb = (float)gg * (float)n / (float)nn;
  go->my_go->ratio = (float)go->my_go->x_genes / go->my_go->exp_numb; 

  return go;

}










int get_numb_elem_slist( SLIST *header )
{

  int total = 0;
  
  SLIST *x = header;
  
  while ( x != NULL ) {
    ++total;
    x = x->next;
  }
  
  return total;

}












int get_intersection_slist_pair( SLIST *a_header, SLIST *b_header )
{

  int x = 0;

  SLIST *a = a_header, *b = NULL;
  
  while ( a != NULL ) {
  
    b = b_header;
    
    while ( b != NULL ) {
    
      if ( ! strcmp( a->str, b->str )) {
        ++x;
        break;
      }
      
      b = b->next;
    
    }
    
    a = a->next;
  
  }
  
  return x;

}
















GO_CAT_LIST *put_go_cat_list_into_sorted_list( GO_CAT_LIST *go_header, 
  GO_CAT_LIST *go )
{

  if ( go_header == NULL )
    return go;
    
  if ( go_header->my_go->pval >= go->my_go->pval ) {
    go->next = go_header;
    return go;
  }
  
  GO_CAT_LIST *curr = go_header, *prev = NULL;
  
  while (( curr != NULL ) && ( curr->my_go->pval <= go->my_go->pval )) {
    prev = curr;
    curr = curr->next;
  }
  
  if ( curr != NULL )
    go->next = curr;

  if ( prev != NULL )
    prev->next = go;
    
  return go_header;

}















void print_go_cat_corr_single_file( GO_CAT_LIST *go_header, FILE *out )
{

  GO_CAT_LIST *go = go_header;
  
  while ( go != NULL ) 
    print_single_go_cat_corr_single_cat_and_step( &go, out );

}












void print_single_go_cat_corr_single_cat_and_step( GO_CAT_LIST **go, FILE *out )
{
  print_single_go_cat_corr_single_cat( *go, out );
  *go = (*go)->next;
}











void print_single_go_cat_corr_single_cat( GO_CAT_LIST *go, FILE *out )
{

  if ( go->my_go->x_genes < MIN_GENE_NUMB )
    return;

  fprintf( out, "%15s\tpval  = %.2e\n", go->my_go->acc, (double)go->my_go->pval );
  fprintf( out, "%15s\tObs   = %d\n", "", go->my_go->x_genes );
  fprintf( out, "%15s\tExp   = %.2f\n", "", go->my_go->exp_numb );
  fprintf( out, "%15s\tratio = %.2f\n", "", go->my_go->ratio );
  
  print_dot_sep( out );

}
























