#include "ft.h"













#define ORIGINAL_GO_FNAME "data/ft_hg19/go/go.hg.txt"
#define CONDENSED_GO_FNAME "data/ft_hg19/go/go.hg.condensed.txt"
#define GENE_INFO_FNAME "data/ft_hg19/go/gene.hg.txt"
#define TOTAL_COLUMN_FNAME "data/ft_hg19/go/ft_column.txt"













void do_gene_analysis_format( void )
{
  
  printf( "Formatting gene analysis files...\n" );
  
  do_my_gene_analysis_format();
  
  printf( "\n" );
  
}













void do_my_gene_analysis_format( void )
{
  
  condense_go_text_file();
  
  do_gene_analysis_multi_spec_column();
  
}










void condense_go_text_file( void )
{

  FILE *open_text_file_reading( char a[], char b[] );
  FILE *open_text_file_writing( char a[], char b[] );

  FH my_fh;
  
  my_fh.go_in = (FILE *)open_text_file_reading( ORIGINAL_GO_FNAME, "" );
  my_fh.go_out = (FILE *)open_text_file_writing( CONDENSED_GO_FNAME, "" );
  
  condense_go_text_file_w_fhs( &my_fh );
  
  close_file( my_fh.go_in );
  close_file( my_fh.go_out );

}












void condense_go_text_file_w_fhs( FH *my_fh )
{

  MSTR_LIST *go_header = NULL;
  
  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, my_fh->go_in ) != NULL )
    go_header = (MSTR_LIST *)condense_go_text_file_single_line( go_header,
      line, my_fh );

}











MSTR_LIST *condense_go_text_file_single_line( MSTR_LIST *go_header, char line[],
  FH *my_fh )
{

  MSTR_LIST *create_mstr_lister( MSTR_LIST *xx );
  
  if ( line[ 0 ] == '!' )
    return go_header;
  
  MSTR_LIST *mstr = NULL;
  mstr = (MSTR_LIST *)create_mstr_lister( mstr );
  
  sscanf( line, "%*s%*s%s%s", mstr->x, mstr->y );

  return condense_go_text_file_new_mstr_lister( go_header, mstr, my_fh );

}












MSTR_LIST *condense_go_text_file_new_mstr_lister( MSTR_LIST *go_header,
  MSTR_LIST *mstr, FH *my_fh )
{

  if ( go_header == NULL )
    return mstr;
    
  return condense_my_go_text_file_new_mstr_lister( go_header, mstr, my_fh );

}











MSTR_LIST *condense_my_go_text_file_new_mstr_lister( MSTR_LIST *go_header,
  MSTR_LIST *mstr, FH *my_fh )
{

  if ( strcmp( mstr->x, go_header->x ))
    return condense_new_go_text_mstr_list( go_header, mstr, my_fh );
    
  return put_mstr_lister_at_end_of_list( go_header, mstr );

}











MSTR_LIST *condense_new_go_text_mstr_list( MSTR_LIST *go_header,
  MSTR_LIST *mstr, FH *my_fh )
{

  put_go_text_mstr_list( go_header, my_fh );
  
  kill_mstr_list( go_header );
  
  return mstr;

}










void put_go_text_mstr_list( MSTR_LIST *go_header, FH *my_fh )
{

  fprintf( my_fh->go_out, "%s\t", go_header->x );
  
  MSTR_LIST *mstr = go_header;
  
  while ( mstr != NULL ) 
    put_go_text_single_mstr_lister_and_step( &mstr, my_fh );

  fprintf( my_fh->go_out, "\n" );
  
}











void put_go_text_single_mstr_lister_and_step( MSTR_LIST **mstr, FH *my_fh )
{
  fprintf( my_fh->go_out, "%s,", (*mstr)->y );
  *mstr = (*mstr)->next;
}














/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/













void do_gene_analysis_multi_spec_column( void )
{
 
  MSTR_LIST *spec_header = NULL;
  spec_header = (MSTR_LIST *)get_species_mstr_list( spec_header );
  
  do_gene_analysis_multi_spec_column_w_spec_list( spec_header );
  
  kill_mstr_list( spec_header );
 
}












MSTR_LIST *get_species_mstr_list( MSTR_LIST *spec_header )
{

  spec_header = (MSTR_LIST *)get_and_put_mstr_lister_string( spec_header, "hg19" );
  
  spec_header = (MSTR_LIST *)get_and_put_mstr_lister_string( spec_header, "mm9" );
  
  spec_header = (MSTR_LIST *)get_and_put_mstr_lister_string( spec_header, 
    "galGal3" );
    
  spec_header = (MSTR_LIST *)get_and_put_mstr_lister_string( spec_header,
    "taeGut1" );
    
  spec_header = (MSTR_LIST *)get_and_put_mstr_lister_string( spec_header,
    "monDom5" );
    
  spec_header = (MSTR_LIST *)get_and_put_mstr_lister_string( spec_header,
    "macEug1" );
    
  spec_header = (MSTR_LIST *)get_and_put_mstr_lister_string( spec_header,
    "ornAna1" );
    
  spec_header = (MSTR_LIST *)get_and_put_mstr_lister_string( spec_header,
    "anoCar1" );
    
  spec_header = (MSTR_LIST *)get_and_put_mstr_lister_string( spec_header,
    "xenTro2" );
    
  return spec_header;

}













void do_gene_analysis_multi_spec_column_w_spec_list( MSTR_LIST *spec_header )
{

  FH my_fh;
  
  set_spec_fh_struct( &my_fh, spec_header );
  
  do_gene_analysis_multi_spec_column_w_fhs( spec_header, &my_fh );
  
  close_spec_fh_struct( &my_fh, spec_header );
  
}











void set_spec_fh_struct( FH *my_fh, MSTR_LIST *spec_header )
{

  FILE *open_text_file_reading( char a[], char b[] );

  register int i;
  
  for ( i = 0; i < MSTR_SPEC_BUFFER; i++ )
    my_fh->spec_in[ i ] = NULL;
    
  i = 0;
    
  MSTR_LIST *mstr = spec_header;
  
  while ( mstr != NULL ) {
  
    char fname[ BUFFER ];
    get_spec_single_mstr_fname( fname, mstr->x );
    
    my_fh->spec_in[ i++ ] = (FILE *)open_text_file_reading( fname, "" );
    
    mstr = mstr->next;
  
  }

}













void close_spec_fh_struct( FH *my_fh, MSTR_LIST *spec_header )
{

  int i = 0;

  MSTR_LIST *mstr = spec_header;
  
  while ( mstr != NULL ) {
  
    close_file( my_fh->spec_in[ i++ ] );
    
    mstr = mstr->next;
  
  }

}













void do_gene_analysis_multi_spec_column_w_fhs( MSTR_LIST *spec_header, FH *my_fh )
{

  FILE *open_text_file_writing( char a[], char b[] );
  FILE *open_text_file_reading( char a[], char b[] );
  
  my_fh->column_out = (FILE *)open_text_file_writing( TOTAL_COLUMN_FNAME, "" );
  my_fh->gene_info = (FILE *)open_text_file_reading( GENE_INFO_FNAME, "" );
  
  do_gene_analysis_multi_spec_column_out_fh( spec_header, my_fh );
  
  close_file( my_fh->column_out );
  close_file( my_fh->gene_info );

}











void do_gene_analysis_multi_spec_column_out_fh( MSTR_LIST *spec_header,
  FH *my_fh )
{

  put_spec_column_header_spec_line( spec_header, my_fh );
  
  do_my_gene_analysis_multi_spec_column_out_fh( spec_header, my_fh );

}












void put_spec_column_header_spec_line( MSTR_LIST *spec_header, FH *my_fh )
{

  fprintf( my_fh->column_out, "#GeneNumb\tRefSeqAcc\tGene\t" );
  
  MSTR_LIST *mstr = spec_header;
  
  while ( mstr != NULL ) {
    fprintf( my_fh->column_out, "%s\t", mstr->x );
    mstr = mstr->next;
  }
  
  fprintf( my_fh->column_out, "\n" );

}













void do_my_gene_analysis_multi_spec_column_out_fh( MSTR_LIST *spec_header, 
  FH *my_fh )
{

  char hg_line[ LINE_BUFFER ];
  
  while ( fgets( hg_line, LINE_BUFFER, my_fh->spec_in[ 0 ] ) != NULL )
    do_gene_analysis_multi_spec_column_single_line( spec_header, hg_line,
      my_fh );

}












void do_gene_analysis_multi_spec_column_single_line( MSTR_LIST *spec_header,
  char hg_line[], FH *my_fh )
{

  char col[ MSTR_SPEC_BUFFER ][ LINE_BUFFER ];
  
  initialize_column_array( col );
  
  int numb_of_cols = get_column_nucl_array( col, spec_header, hg_line, my_fh );
  
  do_gene_analysis_multi_spec_with_spec_lines( spec_header, col, numb_of_cols,
    my_fh );

}











void initialize_column_array( char col[ MSTR_SPEC_BUFFER ][ LINE_BUFFER ] )
{

  int i;
  
  for ( i = 0; i < MSTR_SPEC_BUFFER; i++ )
    col[ i ][ 0 ] = '\0';

}










int get_column_nucl_array( char col[ MSTR_SPEC_BUFFER ][ LINE_BUFFER ], 
  MSTR_LIST *spec_header, char hg_line[], FH *my_fh )
{

  int numb_of_cols = 1;
  
  strcpy( col[ 0 ], hg_line );
  
  MSTR_LIST *mstr = spec_header->next;
  
  while ( mstr != NULL ) {

    fgets( col[ numb_of_cols ], LINE_BUFFER, my_fh->spec_in[ numb_of_cols ] );
    
    ++numb_of_cols;
    
    mstr = mstr->next;

  }
  
  return numb_of_cols;

}












void do_gene_analysis_multi_spec_with_spec_lines( MSTR_LIST *spec_header, 
  char col[ MSTR_SPEC_BUFFER ][ LINE_BUFFER ], int numb_of_cols, FH *my_fh )
{

  int numb;
  char gene_acc[ BUFFER ], gene_name[ BUFFER ];
  
  sscanf( col[ 0 ], "%d%s", &numb, gene_acc );
  
  get_gene_name_from_fh( gene_name, gene_acc, my_fh );
  
  printf( "(%d)  Gene :  %s\n", numb, gene_name );
  
  do_gene_analysis_multi_spec_w_gene_name( spec_header, col, numb, gene_acc,
    gene_name, numb_of_cols, my_fh );

}











void get_gene_name_from_fh( char gene_name[], char gene_acc[], FH *my_fh )
{

  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, my_fh->gene_info ) != NULL ) {
  
    char my_acc[ BUFFER ], my_name[ BUFFER ];
    
    sscanf( line, "%s%*s%*s%*s%*s%s", my_acc, my_name );
    
    if ( ! strcmp( gene_acc, my_acc )) {
      strcpy( gene_name, my_name );
      return;
    }
  
  }

  set_position( 0, my_fh->gene_info );
  
  while ( fgets( line, LINE_BUFFER, my_fh->gene_info ) != NULL ) {
  
    char my_acc[ BUFFER ], my_name[ BUFFER ];
    
    sscanf( line, "%s%*s%*s%*s%*s%s", my_acc, my_name );
    
    if ( ! strcmp( gene_acc, my_acc )) {
      strcpy( gene_name, my_name );
      return;
    }
  
  }

  gene_name[ 0 ] = '\0';  

}













void do_gene_analysis_multi_spec_w_gene_name( MSTR_LIST *spec_header, 
  char col[ MSTR_SPEC_BUFFER ][ LINE_BUFFER ], int numb, char gene_acc[],
  char gene_name[], int numb_of_cols, FH *my_fh )
{

  fprintf( my_fh->column_out, "%7d\t%15s\t", numb, gene_acc );
  
  if ( gene_name[ 0 ] == '\0' )
    fprintf( my_fh->column_out, "%15s\t", "----" );
    
  else
    fprintf( my_fh->column_out, "%15s\t", gene_name );
  
  register int i = 0;
  
  while ( col[ i ][ 0 ] != '\0' ) {
  
    char x[ BUFFER ];
    
    sscanf( col[ i ], "%*s%*s%s", x );
  
    if ( gene_name[ 0 ] == '\0' )
      fprintf( my_fh->column_out, "%15s\t", "----" );
    else
      fprintf( my_fh->column_out, "%15s\t", x );
  
    ++i;
  
  }
  
  fprintf( my_fh->column_out, "\n" );

}









/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/












MSTR_LIST *create_mstr_lister( MSTR_LIST *x )
{

  x = (MSTR_LIST *)malloc( sizeof(MSTR_LIST) );
  
  check_null( 1, x );
  
  x->x[ 0 ] = '\0';
  x->y[ 0 ] = '\0';
  
  x->next = NULL;
  
  return x;

}











void kill_mstr_list( MSTR_LIST *header )
{

  MSTR_LIST *curr = header;
  
  while ( curr != NULL ) {
  
    MSTR_LIST *next = curr->next;
    
    free( curr );
    
    curr = next;
  
  }

}











MSTR_LIST *put_mstr_lister_at_end_of_list( MSTR_LIST *header, MSTR_LIST *mstr )
{

  if ( header == NULL )
    return mstr;

  MSTR_LIST *curr = header;
  
  while ( curr->next != NULL )
    curr = curr->next;
    
  curr->next = mstr;
  
  return header;

}













MSTR_LIST *get_and_put_mstr_lister_string( MSTR_LIST *mstr_header, char str[] )
{

  MSTR_LIST *mstr = NULL;
  mstr = (MSTR_LIST *)create_mstr_lister( mstr );
  
  strcpy( mstr->x, str );
  
  return put_mstr_lister_at_end_of_list( mstr_header, mstr );

}

















/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/
/*==========================================================================*/














void get_spec_single_mstr_fname( char fname[], char spec[] )
{
  sprintf( fname, "data/ft_hg19/occur/%s/%s.ft_occur.column.txt",
    spec, spec );
}




