#include "gene.h"















#define GO_FNAME "data/ft_hg19/go/go.hg.condensed.txt"
#define COLUMN_FNAME "data/ft_hg19/go/ft_column.txt"

#define OUT_SPLIT_BASE "data/ft_hg19/go/gene_split/function_split"




















#define MIN_ANCESTRAL_SPEC_NUMB 2










void do_gene_analysis( void )
{

  extern char minor_mode;

  switch ( minor_mode ) {
  
  case 'f' :
    /*  In ft.c:  */
    do_gene_analysis_format();
    break;
    
  case 's' :
    /*  In this file, below:  */
    do_go_split();
    break;
    
  case 'S' :
    /*  In go.c:  */
    do_go_file_from_split();
    break;
    
  case 'c' :
    /*  In go.c:  */
    do_go_category_count();
    break;
    
  case 'g' :
    /*  In sig.c:  */
    do_go_category_sig();
    break;
    
  default:
    error( 6000, "Unknown mode passed to gene analysis function.\n" );
    break;
  
  }

}















void do_go_split( void )
{

  FH_SPLIT my_fh;
  
  set_fh_split_fhs( &my_fh );
  
  do_go_split_w_fhs( &my_fh );
  
  close_fh_split_fhs( &my_fh );

}












void set_fh_split_fhs( FH_SPLIT *my_fh )
{

  FILE *open_text_file_reading( char a[], char b[] );
  
  my_fh->go_fh = (FILE *)open_text_file_reading( GO_FNAME, "" );
  my_fh->col_fh = (FILE *)open_text_file_reading( COLUMN_FNAME, "" );
  
  int i;
  
  for ( i = 0; i < FH_BUFFER; i++ )
    my_fh->out_fh[ i ] = NULL;
    
  set_fh_split_output_fhs( my_fh );

}














void close_fh_split_fhs( FH_SPLIT *my_fh )
{

  close_file( my_fh->go_fh );
  close_file( my_fh->col_fh );
  
  int i = 0;
  
  while ( my_fh->out_fh[ i ] != NULL )
    close_file( my_fh->out_fh[ i++ ] );

}













void do_go_split_w_fhs( FH_SPLIT *my_fh )
{

  MSTR_LIST *get_species_mstr_list( MSTR_LIST *spec_header );

  MSTR_LIST *spec_header = NULL;
  /*  In ft.c:  */
  spec_header = (MSTR_LIST *)get_species_mstr_list( spec_header );
  
  /*  In this file, below:  */
  do_go_split_w_spec_list( spec_header, my_fh );
  
  kill_mstr_list( spec_header );

}












void do_go_split_w_spec_list( MSTR_LIST *spec_header, FH_SPLIT *my_fh )
{

  set_position( 0, my_fh->col_fh );
  
  char line[ LINE_BUFFER ];
  
  while ( fgets( line, LINE_BUFFER, my_fh->col_fh ) != NULL )
    do_go_split_single_line( line, spec_header, my_fh );

}













void do_go_split_single_line( char line[], MSTR_LIST *spec_header, FH_SPLIT *my_fh )
{

  initialize_mstr_list_y_vals( spec_header );
  
  do_my_go_split_single_line( line, spec_header, my_fh );

}












void initialize_mstr_list_y_vals( MSTR_LIST *header )
{

  MSTR_LIST *mstr = header;
  
  while ( mstr != NULL ) {
    mstr->y[ 0 ] = '\0';
    mstr = mstr->next;
  }

}
















void do_my_go_split_single_line( char line[], MSTR_LIST *spec_header, 
  FH_SPLIT *my_fh )
{

  GENE_COL my_gene;
  
  my_gene.spec_header = spec_header;
  strcpy( my_gene.line, line );
  
  /*  In parse.c:  */
  get_gene_col_struct_single_line( &my_gene, my_gene.line );
  
  /*  In this file, below:  */
  do_go_split_single_gene_col_entry( &my_gene, my_fh );

}














void do_go_split_single_gene_col_entry( GENE_COL *my_gene, FH_SPLIT *my_fh )
{

  if ( my_gene->line[ 0 ] == '#' ) {
    fprintf( my_fh->out_fh[ 0 ], "#COUNT\t%s", my_gene->line );
    fprintf( my_fh->out_fh[ 1 ], "#COUNT\t%s", my_gene->line );
    fprintf( my_fh->out_fh[ 2 ], "#COUNT\t%s", my_gene->line );
    fprintf( my_fh->out_fh[ 3 ], "#COUNT\t%s", my_gene->line );
    fprintf( my_fh->out_fh[ 4 ], "#COUNT\t%s", my_gene->line );
    fprintf( my_fh->out_fh[ 5 ], "#COUNT\t%s", my_gene->line );
    fprintf( my_fh->out_fh[ 6 ], "#COUNT\t%s", my_gene->line );
    return;
  }
  
  /*  See bottom for output fh array index...  */
  put_functional_a_to_c_gene_col( my_gene, my_fh->out_fh[ 0 ] );
  put_non_function_c_not_a_to_c_gene_col( my_gene, my_fh->out_fh[ 1 ] );
  put_non_functional_a_to_not_c_gene_col( my_gene, my_fh->out_fh[ 2 ] );
  put_functional_a_to_c_human_gene_col( my_gene, my_fh->out_fh[ 3 ] );
  put_functional_a_to_c_bird_gene_col( my_gene, my_fh->out_fh[ 4 ] );
  put_non_functional_a_to_not_c_human_gene_col( my_gene, my_fh->out_fh[ 5 ] );
  put_non_functional_a_to_not_c_bird_gene_col( my_gene, my_fh->out_fh[ 6 ] );





//  put_all_gene_col( my_gene, my_fh->out_fh[ 3 ] );

}














void put_functional_a_to_c_gene_col( GENE_COL *my_gene, FILE *fh )
{

  static int numb = 0;

  if ( is_function_a_to_c_gene_col( my_gene )) {
    fprintf( fh, "%7d\t", ++numb );
    fprintf( fh, "%s", my_gene->line );
  }

}













int is_function_a_to_c_gene_col( GENE_COL *my_gene )
{

  if ( get_ancestral_nucl_count( my_gene, A_NUCL ) < MIN_ANCESTRAL_SPEC_NUMB )
    return 0;
  
  if ( ! gene_col_mstr_spec_has_nucl( my_gene, "hg19", C_NUCL ))
    return 0;
    
  if ( gene_col_mstr_spec_has_nucl( my_gene, "galGal3", C_NUCL ))
    return 1;
    
  if ( gene_col_mstr_spec_has_nucl( my_gene, "taeGut1", C_NUCL ))
    return 1;
    
  return 0;

}













void put_non_function_c_not_a_to_c_gene_col( GENE_COL *my_gene, FILE *fh )
{

  static int numb = 0;

  if ( is_non_function_c_not_a_to_c_gene_col( my_gene )) {
    fprintf( fh, "%7d\t", ++numb );
    fprintf( fh, "%s", my_gene->line );
  }

}












int is_non_function_c_not_a_to_c_gene_col( GENE_COL *my_gene )
{

  if ( get_ancestral_nucl_count( my_gene, C_NUCL ) < MIN_ANCESTRAL_SPEC_NUMB )
    return 0;
    
  if ( get_ancestral_nucl_count( my_gene, A_NUCL ) >= MIN_ANCESTRAL_SPEC_NUMB )
    return 0;
  
  if ( gene_col_mstr_spec_has_nucl( my_gene, "hg19", C_NUCL ))
    return 0;
    
  if ( gene_col_mstr_spec_has_nucl( my_gene, "galGal3", C_NUCL ))
    return 1;
    
  if ( gene_col_mstr_spec_has_nucl( my_gene, "taeGut1", C_NUCL ))
    return 1;
    
  return 0;

}
















void put_non_functional_a_to_not_c_gene_col( GENE_COL *my_gene, FILE *fh )
{

  static int numb = 0;

  if ( is_non_functional_a_to_not_c_gene_col( my_gene )) {
    fprintf( fh, "%7d\t", ++numb );
    fprintf( fh, "%s", my_gene->line );
  }

}















int is_non_functional_a_to_not_c_gene_col( GENE_COL *my_gene )
{

  if ( get_ancestral_nucl_count( my_gene, A_NUCL ) < MIN_ANCESTRAL_SPEC_NUMB )
    return 0;
  
  if ( gene_col_mstr_spec_has_nucl( my_gene, "hg19", C_NUCL ))
    return 0;
    
  if ( gene_col_mstr_spec_has_nucl( my_gene, "galGal3", C_NUCL ))
    return 0;
    
  if ( gene_col_mstr_spec_has_nucl( my_gene, "taeGut1", C_NUCL ))
    return 0;
    
  return 1;
    
}














void put_functional_a_to_c_human_gene_col( GENE_COL *my_gene, FILE *fh )
{

  static int numb = 0;

  if ( is_functional_a_to_c_human_gene_col( my_gene )) {
    fprintf( fh, "%7d\t", ++numb );
    fprintf( fh, "%s", my_gene->line );
  }

}









int is_functional_a_to_c_human_gene_col( GENE_COL *my_gene )
{

  if ( get_ancestral_nucl_count( my_gene, A_NUCL ) < MIN_ANCESTRAL_SPEC_NUMB )
    return 0;
  
  if ( gene_col_mstr_spec_has_nucl( my_gene, "hg19", C_NUCL ))
    return 1;
    
  return 0;    

}












void put_functional_a_to_c_bird_gene_col( GENE_COL *my_gene, FILE *fh )
{

  static int numb = 0;

  if ( is_functional_a_to_c_bird_gene_col( my_gene )) {
    fprintf( fh, "%7d\t", ++numb );
    fprintf( fh, "%s", my_gene->line );
  }

}









int is_functional_a_to_c_bird_gene_col( GENE_COL *my_gene )
{

  if ( get_ancestral_nucl_count( my_gene, A_NUCL ) < MIN_ANCESTRAL_SPEC_NUMB )
    return 0;
  
  if ( gene_col_mstr_spec_has_nucl( my_gene, "galGal3", C_NUCL ))
    return 1;
    
  if ( gene_col_mstr_spec_has_nucl( my_gene, "taeGut1", C_NUCL ))
    return 1;
    
  return 0;
    

}












void put_non_functional_a_to_not_c_human_gene_col( GENE_COL *my_gene, FILE *fh )
{

  static int numb = 0;

  if ( is_non_functional_a_to_not_c_human_gene_col( my_gene )) {
    fprintf( fh, "%7d\t", ++numb );
    fprintf( fh, "%s", my_gene->line );
  }

}









int is_non_functional_a_to_not_c_human_gene_col( GENE_COL *my_gene )
{

  if ( get_ancestral_nucl_count( my_gene, A_NUCL ) < MIN_ANCESTRAL_SPEC_NUMB )
    return 0;
  
  if ( gene_col_mstr_spec_has_nucl( my_gene, "hg19", C_NUCL ))
    return 0;
    
  return 1;    

}












void put_non_functional_a_to_not_c_bird_gene_col( GENE_COL *my_gene, FILE *fh )
{

  static int numb = 0;

  if ( is_non_functional_a_to_not_c_bird_gene_col( my_gene )) {
    fprintf( fh, "%7d\t", ++numb );
    fprintf( fh, "%s", my_gene->line );
  }

}









int is_non_functional_a_to_not_c_bird_gene_col( GENE_COL *my_gene )
{

  if ( get_ancestral_nucl_count( my_gene, A_NUCL ) < MIN_ANCESTRAL_SPEC_NUMB )
    return 0;
  
  if ( gene_col_mstr_spec_has_nucl( my_gene, "galGal3", C_NUCL ))
    return 0;
    
  if ( gene_col_mstr_spec_has_nucl( my_gene, "taeGut1", C_NUCL ))
    return 0;
    
  if ( gene_col_is_empty( my_gene, "galGal3" ) && 
    gene_col_is_empty( my_gene, "taeGut1" ))
    return 0;
    
  return 1;
    

}














void put_all_gene_col( GENE_COL *my_gene, FILE *fh )
{

  static int numb = 0;
  
  fprintf( fh, "%7d\t%s", ++numb, my_gene->line );

}
















int get_ancestral_nucl_count( GENE_COL *my_gene, char nucl_numb )
{

  int numb = 0;
  
  if ( gene_col_mstr_spec_has_nucl( my_gene, "monDom5", nucl_numb )
    ||
    gene_col_mstr_spec_has_nucl( my_gene, "macEug1", nucl_numb ))
  {
    ++numb;
  }
  
  numb += gene_col_mstr_spec_has_nucl( my_gene, "ornAna1", nucl_numb );
  
  numb += gene_col_mstr_spec_has_nucl( my_gene, "anoCar1", nucl_numb );
  
  numb += gene_col_mstr_spec_has_nucl( my_gene, "xenTro2", nucl_numb );
    
  return numb;

}















int gene_col_mstr_spec_has_nucl( GENE_COL *my_gene, char spec_name[], 
  char nucl_numb )
{

  MSTR_LIST *spec = my_gene->spec_header;
  
  while ( spec != NULL ) {
  
    if ( ! strcmp( spec->x, spec_name )) 
      return nucl_column_is_nucl( spec->y, nucl_numb );
      
    spec = spec->next;
  
  }
  
  error( 6010, "Did not find species in multi-string list.\n" );

}












int nucl_column_is_nucl( char str[], char nucl_numb )
{

  if (( str[ nucl_numb ] == '-' ) || ( str[ nucl_numb ] == '.' ))
    return 0;
    
  return 1;

}












int gene_col_is_empty( GENE_COL *my_gene, char spec_name[] )
{

  MSTR_LIST *spec = my_gene->spec_header;
  
  while ( spec != NULL ) {
  
    if ( ! strcmp( spec->x, spec_name )) 
      return nucl_column_is_empty( spec->y );
      
    spec = spec->next;
  
  }
  
  error( 6010, "Did not find species in multi-string list.\n" );

}














int nucl_column_is_empty( char str[] )
{

  return ( str[ 0 ] == '-' ) ? (1) : (0);

}












/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/















void set_fh_split_output_fhs( FH_SPLIT *my_fh )
{

  FILE *open_text_file_writing( char a[], char b[] );
  
  my_fh->out_fh[ 0 ] = (FILE *)open_text_file_writing( OUT_SPLIT_BASE, 
    "A_to_C.txt" );
  
  my_fh->out_fh[ 1 ] = (FILE *)open_text_file_writing( OUT_SPLIT_BASE, 
    "C_not_A_to_C.txt" );
    
  my_fh->out_fh[ 2 ] = (FILE *)open_text_file_writing( OUT_SPLIT_BASE,
    "A_to_not_C.txt" );
    
  my_fh->out_fh[ 3 ] = (FILE *)open_text_file_writing( OUT_SPLIT_BASE, 
    "A_to_C_human.txt" );
    
  my_fh->out_fh[ 4 ] = (FILE *)open_text_file_writing( OUT_SPLIT_BASE,
    "A_to_C_birds.txt" );
    
  my_fh->out_fh[ 5 ] = (FILE *)open_text_file_writing( OUT_SPLIT_BASE,
    "A_to_not_C_human.txt" );
    
  my_fh->out_fh[ 6 ] = (FILE *)open_text_file_writing( OUT_SPLIT_BASE,
    "A_to_not_C_birds.txt" );

//  my_fh->out_fh[ 7 ] = (FILE *)open_text_file_writing( OUT_SPLIT_BASE, "all.txt" );
  
}









